#!/bin/sh
#
# OracleASMDBRemount.sh: shell script to automate cloning and bring up an Oracle database using backup image copies
#
# usage: OracleASMDBRemount.sh <New database sid name> <Oracle Home path> <diskgroup name>"

set +x
nsid=$1
oracleHome=$2
dg=$3
osouser=$4

dataDir="+"$dg

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

########### Function to drop restore point ##########
drop_restorepoint()
{
   su -m $osouser -c "sqlplus -s -L / as sysdba " << EOF
   set echo off
   set feed off
   startup mount;
   exit;
EOF
  restorepoint_cnt=`su -m $osouser -c "sqlplus -s -L / as sysdba " << EOF
  set heading off
  set echo off
  set feed off
  set newpage none;
  select count(1) from v\\$restore_point;
  exit;
EOF`

  vnum=`su -m $osouser -c "sqlplus -S / as sysdba " << EOF1
  set head off;
  set feedback off
  select count(*) from v\\$instance where version like '10%' or version like '11%' or version like '12.1%';
  exit;
EOF1`

  if [ "$vnum" -ge "1" ] && [ "$restorepoint_cnt" -ge 1 ]; then
     echo " *********** Found restore point, dropping the same **********"
     su -m $osouser -c "sqlplus -s -L / as sysdba " << EOF
     begin
         for rec in (select name from v\$restore_point where con_id=0) loop
            execute immediate 'drop restore point '||rec.name;
         end loop;
     end;
     /
EOF
  elif [ "$restorepoint_cnt" -ge "1" ]; then
       echo " *********** Found restore point, dropping the same **********"
       su -m $osouser -c "sqlplus -s -L / as sysdba " << EOF
       begin
           for rec in (select name from v\$restore_point where con_id=0) loop
              execute immediate 'drop restore point '||rec.name;
           end loop;
           for rec in (select b.name con_name, a.name rname  from v\$restore_point a, v\$pdbs b where a.con_id=b.con_id ) loop
              execute immediate 'alter session set container='||rec.con_name;
              execute immediate 'drop restore point '||rec.rname;
           end loop;
       end;
       /
EOF
  fi

   su -m $osouser -c "sqlplus -s -L / as sysdba " << EOF
   set echo off
   set feed off
   shutdown abort;
   exit;
EOF
}

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR
# correctly set the ORACLE_BASE AND ORACLE_HOME  for your environment
export ORACLE_HOME=$oracleHome
#export ORACLE_BASE=$dataDir
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
if [ ! -d $ORACLE_HOME ]; then
   echo "could not find ORACLE_HOME "
   exit 1
fi

sqlplus=$ORACLE_HOME/bin/sqlplus

if [ "$noarchivemode" = "true" ]; then
  archcmd="alter database noarchivelog;"
else
  archcmd="alter database archivelog;"
fi

#setup process to get required info from mounted diskgroup
oratab=/etc/oratab
gridhome=`get_grid_home $oratab`
if [ $? -ne 0 ]; then
   echo "failed to get ORACLE_HOME for grid"
   exit 1
fi

if [ ! -d "$gridhome" ]; then
   echo "ORACLE_HOME $gridhome for grid does not exists"
   exit 1
fi

guser=`get_asm_user`
if [ $? -ne 0 ]; then
   echo "failed to get grid user"
   exit 1
fi

asmsid=`ps -ef | grep asm_pmon | grep -i +ASM | awk '{print $NF}' | awk -F'_' '{print $NF}'`
export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
asmcmd=$ORACLE_HOME/bin/asmcmd
get_asmcmd_nocp asmcmd

#get generation
gen=`get_apm_latest_gen $dg $guser`
psid=`get_first_apm_sid $dg $guser $gen`
if [ $? -ne 0 ]; then
   echo "error locating first mount sid: $psid"
   exit 1
fi
#create datafile dir for remount instance
su -m $guser -c "$asmcmd $ASMCMD_NOCP" << EOF
cd $dg
mkdir $nsid
cd $nsid
mkdir datafile
EOF
#

#check psid exists
spfile=`get_spfile $dg $psid $guser`
pspfile="+"$dg"/"$psid"/"$spfile
spname=`echo $spfile | cut -d"." -f1`
tpsid=${spname:6}
nspfile="+"$dg"/"$psid"/spfile"$nsid".ora"
#remove old spfile
#su -m $guser -c "asmcmd $ASMCMD_NOCP rm $pspfile"
pcontrolfile=`get_controlfile $dg $psid $guser`

export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export ORACLE_SID=$tpsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
pfile=$ORACLE_HOME"/actpfile_"$nsid".ora"

#create parameter file under ORACLE_HOME/dbs
odefaultpfile="$ORACLE_HOME/dbs/init"$tpsid"1.ora"
if [ ! -e "$odefaultpfile" ]; then
   echo "SPFILE='$pspfile'" > $odefaultpfile
fi

change_own $osouser $odefaultpfile

su -m $osouser -c "sqlplus -s -L / as sysdba << EOF
set echo off
set feed off
create pfile='$pfile' from spfile='$pspfile';
exit;
EOF
"

lastdg=`get_last_dgname $pfile`

samediskgroup="true"
if [ "$lastdg" = "$dg" ]; then
    echo "using same diskgroup"
else
    echo "using different diskgroup"
    samediskgroup="false"
fi

tpsidc=`echo $tpsid | tr '[A-Z]' '[a-z]'`
tpsidu=`echo $tpsid | tr '[a-z]' '[A-Z]'`
if [ "$tpsidc" = "$nsid" ] || [ "$tpsidu" = "$nsid" ]; then
    echo "remount with same SID"
    if [ "$samediskgroup" = "true" ]; then
       echo "remount with same SID $nsid using same diskgroup"
       # copy pfile to ORACLE_HOME/dbs, start database
       #cp $initfile $ORACLE_HOME/dbs/
ORACLE_SID=$nsid"1"
export ORACLE_SID
thrcnta=`grep -i ".thread" $pfile | wc -l`

drop_restorepoint
su -m $osouser -c "sqlplus / as sysdba << EOF1
shutdown abort;
startup mount;
alter database end backup;
$archcmd
alter database open;
shutdown abort;
exit;
EOF1
"

else
       echo "remount with same SID $nsid using different diskgroup"
       # same SID, different mp, change dataDir in pfile
       #sed -i "s/$osid/$nsid/g" $initfile
       
       sed -i "s=${lastdg}=${dg}=g" $pfile
pfile_tmp=$ORACLE_HOME"/actpfile_tmp_"$nsid".ora"
su -m $osouser -c "cp $pfile $pfile_tmp"
#make required directory structures
if [ -z "$diagnostic_dest" ]; then
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../admin/$nsid/adump"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../admin/$nsid/adump/*"
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../diag"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../diag/*"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
else
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/*"
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid/admin/adump"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/admin/adump/*"
  dbdiagnostic="*.diagnostic_dest='$diagnostic_dest/$nsid'"
  adest="*.audit_file_dest='$diagnostic_dest/$nsid/admin/adump'"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
  echo -e "$dbdiagnostic" >> $pfile_tmp
  echo -e "$adest" >> $pfile_tmp
fi
mv $pfile_tmp $pfile

export ORACLE_SID=$tpsid"1"
su -m $osouser -c "$sqlplus / as sysdba << EOF2
startup nomount force pfile='$pfile';
alter database mount;
create spfile='$pspfile' from pfile='$pfile';
shutdown abort;
exit;
EOF2
"

initfilex="$ORACLE_HOME/dbs/init"$tpsid"1.ora"
grep -w $lastdg $initfilex >/dev/null 2>&1
if [ $? -eq 0 ]; then 
   sed -i "s=${lastdg}=${dg}=g" $initfilex
fi

drop_restorepoint
su -m $osouser -c "$sqlplus / as sysdba << EOF3
startup mount;
@/act/act_scripts/asmclone/renameDatafile.sql $dataDir $psid;
@/act/act_scripts/asmclone/renamedf_generated.sql
alter database disable block change tracking;
$archcmd
alter database open;
exit;
EOF3
"
    fi
else
    # Different SID for remount
    echo "remount with different SID"
    if [ "$samediskgroup" = "true" ]; then
       echo "remount with SID $nsid to replace $psid using same diskgroup"
       #cp $initfile $ORACLE_HOME/dbs/
#ls -ltr $ORACLE_HOME/dbs/init$psid.ora

#       cd /act/act_scripts/oracleclone
#       . ./setdb_$nsid

export ORACLE_SID=$tpsid"1"
drop_restorepoint
su -m $osouser -c "$sqlplus / as sysdba " << EOF4
startup mount;
alter system set cluster_database=FALSE scope=spfile;
$archcmd
alter database open;
exit;
EOF4


roTBSList=`su -m $osouser -c "sqlplus -S / as sysdba " << 'EOF1'
@/act/act_scripts/readonlyDatafileCatalog.sql
exit;
EOF1`

if [ ! -z "$roTBSList" ]; then
   roTBS=''
   for i in ${roTBSList}; do
      echo "$i"
      if [ -z "$roTBS" ]; then
          roTBS="'"$i"'"
      else
          roTBS=$roTBS",'"$i"'"
      fi
   done
else
   roTBS="'NONE'"
fi

echo "$roTBS"
su -m $osouser -c "$sqlplus / as sysdba " << EOF
@nidROTBS.sql "$roTBS" ONLINE
@nidROTBS.sql "$roTBS" "READ WRITE"
shutdown immediate;
startup mount;
exit;
EOF

set +x
username="sys"
pssword="Change_0n_install"
if [ ! -z "$ouser" ]; then
   username=$ouser
fi
if [ ! -z "$opass" ]; then
   pssword=$opass
fi

su -m $osouser -c "$ORACLE_HOME/bin/nid TARGET=$username/$pssword DBNAME=$nsid SETNAME=yes LOGFILE=newsidlog.log"

        if [ $? != 0 ]; then
          echo "failed to rename database to $nsid"
          exit 1
        fi
pfile_tmp=$ORACLE_HOME"/actpfile_tmp_"$nsid".ora"
if [ -z "$diagnostic_dest" ]; then
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../admin/$nsid/adump"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../admin/$nsid/adump/*"
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../diag"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../diag/*"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
else
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/*"
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid/admin/adump"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/admin/adump/*"
  dbdiagnostic="*.diagnostic_dest='$diagnostic_dest/$nsid'"
  adest="*.audit_file_dest='$diagnostic_dest/$nsid/admin/adump'"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
  echo -e "$dbdiagnostic" >> $pfile_tmp
  echo -e "$adest" >> $pfile_tmp
fi
#process pfile with new name before mount
ctrfile_entry=`grep control_files $pfile`
su -m $osouser -c "grep -v control_files $pfile > $pfile_tmp"
sed -i "s/$tpsid/$nsid/Ig" $pfile_tmp
#sed -i "s=${tpsid}=${nsid}=g" $pfile_tmp
#new controlfile is written after dbname change, get this new name and update initfile
export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
ncontrolfile=`get_controlfile $dg $psid $guser`
echo "$ctrfile_entry" >> $pfile_tmp
sed -i "s=${pcontrolfile}=${ncontrolfile}=g" $pfile_tmp
mv $pfile_tmp $pfile

change_own $osouser $pfile

export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
initfilex="$ORACLE_HOME/dbs/init"$tpsid"1.ora"
pspora="spfile"$tpsid".ora"
nspora="spfile"$nsid".ora"
sed -i "s=${pspora}=${nspora}=g" $initfilex
initfiley="$ORACLE_HOME/dbs/init"$nsid"1.ora"
mv $initfilex $initfiley 

change_own $osouser $initfiley

export ORACLE_SID=$nsid"1"
su -m $osouser -c "$sqlplus / as sysdba " << EOF5
startup mount pfile='$pfile';
create spfile='$nspfile' from pfile='$pfile';
$archcmd
alter database open;
@nidROTBS.sql "$roTBS" "READ ONLY"
shutdown abort;
exit;
EOF5

export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
#remove old spfile
su -m $guser -c "asmcmd $ASMCMD_NOCP rm $pspfile"

    else
         echo "remount with SID $nsid to replace $psid using different diskgroup"
         sed -i "s=${lastdg}=${dg}=g" $pfile
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
export ORACLE_SID=$tpsid"1"

su -m $osouser -c "$sqlplus / as sysdba " << EOF6
startup nomount force pfile='$pfile';
create spfile='$pspfile' from pfile='$pfile';
shutdown abort;
exit;
EOF6


initfilex="$ORACLE_HOME/dbs/init"$tpsid"1.ora"
grep -w $lastdg $initfilex >/dev/null 2>&1
if [ $? -eq 0 ]; then 
   sed -i "s=${lastdg}=${dg}=g" $initfilex
fi

drop_restorepoint
su -m $osouser -c "$sqlplus / as sysdba " << EOF7
startup mount;
@/act/act_scripts/asmclone/renameDatafile.sql $dataDir $tpsid;
@/act/act_scripts/asmclone/renamedf_generated.sql
alter database disable block change tracking;
shutdown immediate;
exit;
EOF7


export ORACLE_SID=$tpsid"1"
su -m $osouser -c "$sqlplus / as sysdba " << EOF8
startup mount;
alter system set cluster_database=FALSE scope=spfile;
$archcmd
alter database open;
exit;
EOF8

#reset ORACLE_HOME to Oracle
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

set +x
username="sys"
pssword="Change_0n_install"
if [ ! -z "$ouser" ]; then
   username=$ouser
fi
if [ ! -z "$opass" ]; then
   pssword=$opass
fi

roTBSList=`su -m $osouser -c "sqlplus -S / as sysdba " << 'EOF1'
@/act/act_scripts/readonlyDatafileCatalog.sql
exit;
EOF1`

if [ ! -z "$roTBSList" ]; then
   roTBS=''
   for i in ${roTBSList}; do
      echo "$i"
      if [ -z "$roTBS" ]; then
          roTBS="'"$i"'"
      else
          roTBS=$roTBS",'"$i"'"
      fi
   done
else
   roTBS="'NONE'"
fi

echo "$roTBS"
su -m $osouser -c "$sqlplus / as sysdba " << EOF
@nidROTBS.sql "$roTBS" ONLINE
@nidROTBS.sql "$roTBS" "READ WRITE"
shutdown immediate;
startup mount;
exit;
EOF

su -m $osouser -c  "$ORACLE_HOME/bin/nid TARGET=$username/$pssword DBNAME=$nsid SETNAME=yes LOGFILE=newsidlog.log"
        if [ $? != 0 ]; then
          echo "failed to rename database to $nsid after changing datafile path"
          exit 1
        fi

pfile_tmp=$ORACLE_HOME"/actpfile_tmp.ora"
#process pfile with new name before mount
ctrfile_entry=`grep control_files $pfile`
su -m $osouser -c "grep -v control_files $pfile > $pfile_tmp"
sed -i "s/$tpsid/$nsid/Ig" $pfile_tmp
#sed -i "s=${tpsid}=${nsid}=g" $pfile_tmp
#new controlfile is written after dbname change, get this new name and update initfile
export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
ncontrolfile=`get_controlfile $dg $psid $guser`
echo "$ctrfile_entry" >> $pfile_tmp
sed -i "s=${pcontrolfile}=${ncontrolfile}=g" $pfile_tmp
#mv $pfile_tmp $pfile

export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

initfilex="$ORACLE_HOME/dbs/init"$tpsid"1.ora"
pspora="spfile"$tpsid".ora"
nspora="spfile"$nsid".ora"
sed -i "s=${pspora}=${nspora}=g" $initfilex

####changes from single to RAC
undocnta=`grep -i ".undo_tablespace" $pfile_tmp | wc -l`
thrcnta=`grep -i ".thread" $pfile_tmp | wc -l`

if [ "$thrcnta" -lt "1" ]; then
sed -i '/undo_tablespace/d' $pfile_tmp
ncount=`grep "ASM_" asm_node_config.conf | cut -f2 -d":" | wc -l`

var=`grep "ASM_" $BASEDIR/asm_node_config.conf | cut -f2 -d":"`
m=0
for x in $var; do
  m=$(($m + 1));
  norasid1=$nsid$m
        cthread="$norasid1.thread=$m"
        cinst="$norasid1.instance_number=$m"
        cundo="$norasid1.undo_tablespace='UNDOTBS_CL$m'"
        echo -e "$cthread" >> $pfile_tmp
        echo -e "$cinst" >> $pfile_tmp
        echo -e "$cundo" >> $pfile_tmp
done

fi

# Check if the AUDIT Directory exists or not

#make required directory structures
if [ -z "$diagnostic_dest" ]; then
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../admin/$nsid/adump"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../admin/$nsid/adump/*"
  su -m $osouser -c "mkdir -p $ORACLE_HOME/../../diag"
  su -m $osouser -c "rm -f $ORACLE_HOME/../../diag/*"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
else
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/*"
  su -m $osouser -c "mkdir -p $diagnostic_dest/$nsid/admin/adump"
  su -m $osouser -c "rm -f $diagnostic_dest/$nsid/admin/adump/*"
  dbdiagnostic="*.diagnostic_dest='$diagnostic_dest/$nsid'"
  adest="*.audit_file_dest='$diagnostic_dest/$nsid/admin/adump'"
  sed -i '/diagnostic_dest=/d' $pfile_tmp
  sed -i '/audit_file_dest=/d' $pfile_tmp
  sed -i '/remote_listener/d' $pfile_tmp
  echo -e "$dbdiagnostic" >> $pfile_tmp
  echo -e "$adest" >> $pfile_tmp
fi

mv $pfile_tmp $pfile
######################################################

initfiley="$ORACLE_HOME/dbs/init"$nsid"1.ora"
mv $initfilex $initfiley

export ORACLE_SID=$nsid"1"
su -m $osouser -c "$sqlplus / as sysdba " << EOF9
startup mount pfile='$pfile';
create spfile='$nspfile' from pfile='$pfile';
$archcmd
alter database open;
@nidROTBS.sql "$roTBS" "READ ONLY"
shutdown abort;
exit;
EOF9

export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
#remove old spfile
su -m $guser -c "asmcmd $ASMCMD_NOCP rm $pspfile"

    fi
    
fi

#reset ORACLE_HOME to Oracle
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

nsid1=$nsid"1"
if [ "$useexistingorapw" = "false" ]; then
orapw1=$ORACLE_HOME"/dbs/orapw"$nsid1

if [ -e $orapw1 ]; then
   rm -f $orapw1
fi

set +x
syspwd="Change_0n_install"
if [ ! -z $ouser ] && [ "$ouser" == "sys" ] && [ ! -z $opass ]; then
   syspwd=$opass
fi

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh
sqlver=`get_sql_major_version $ORACLE_HOME`
set +x
if [ "$sqlver" -gt "11" ]; then
su -m $osouser -c "$ORACLE_HOME/bin/orapwd file=$orapw1 password=\"$syspwd\" format=12 entries=5"
else
su -m $osouser -c "$ORACLE_HOME/bin/orapwd file=$orapw1 password=\"$syspwd\" entries=5"
fi

fi

#grant user sysdba privilege
if [ ! -z $ouser ] && [ "$ouser" != "sys" ]; then
export ORACLE_SID=$nsid"1"
export ORACLE_HOME=$oracleHome
export PATH=$PATH:$ORACLE_HOME/bin
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
su -m $osouser -c "$sqlplus / as sysdba " << EOF
startup;
grant sysdba to $ouser;
shutdown abort;
exit;
EOF

fi

############## changes to RAC ####
if [ "$thrcnta" -lt "1" ]; then
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
export ORACLE_SID=$nsid"1"
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

su -m $osouser -c "$sqlplus / as sysdba " << EOF
startup;
exit;
EOF

dataDir1="+"$dg
dthcnt=`su -m $osouser -c "$sqlplus -S / as sysdba " << EOF
set head off;
startup;
select max(thread#) from v\\$log;
exit;
EOF`
echo "th count $dthcnt"
dgpcnt=`su -m $osouser -c "$sqlplus -S / as sysdba " << EOF
set head off;
select max(group#) from v\\$log;
exit;
EOF`
echo "gp count $dgpcnt"

undocnta=`grep -i ".undo_tablespace" $pfile | wc -l`
thrcnta=`grep -i ".thread" $pfile | wc -l`

openDatabase1="/act/act_scripts/asmclone/openRemountDB_"$nsid".sql"
openDBlog="/act/act_scripts/asmclone/RemountDB_"$nsid".txt"
undofilelist="undofileRemount_"$nsid".sql"

if [ -e $openDatabase1 ]; then
   echo "removing $openDatabase1 "
  rm -f $openDatabase1
fi

if [ -e $undofilelist ]; then
  # echo "removing $undofilelist "
   rm -f $undofilelist
fi

if [ -e $openDBlog ]; then
   echo "removing $openDBlog "
  rm -f $openDBlog
fi

su -m $osouser -c "$sqlplus -S / as sysdba " << EOF > $undofilelist
set head off;
set feed off;
set echo off;
select distinct tablespace_name from dba_undo_extents;
exit;
EOF

chmod 775  $undofilelist

echo -e "create undo tablespace UNDOTBS_DEF datafile '$dataDir1' size 500m autoextend on;" > $openDatabase1
echo -e "alter system set undo_tablespace=UNDOTBS_DEF;" >> $openDatabase1

echo -e "shutdown abort;" >> $openDatabase1
echo -e "startup;" >> $openDatabase1

while read line
do
    set -- $line
    undoname=$1
        echo "outside if $undoname"
        if [ "x$undoname" != "x" ]; then
        echo "inside if $undoname"
        echo "drop tablespace $undoname including contents and datafiles;" >> $openDatabase1
        fi
done < $undofilelist

n=1
dthcnt=1
while [ "$n" -lt "$thrcnta" ]; do
        dthcnt=$(($dthcnt + 1))
        dgpcnt=$(($dgpcnt + 1))
        echo -e "alter database add logfile thread $dthcnt group $dgpcnt ('$dataDir1') size 100m reuse;" >> $openDatabase1
        dgpcnt=$(($dgpcnt + 1))
        echo -e "alter database add logfile thread $dthcnt group $dgpcnt ('$dataDir1') size 100m reuse;" >> $openDatabase1
        echo -e "alter database enable thread $dthcnt;" >> $openDatabase1
        n=$(($n + 1));
done


n=1
while [ "$n" -le "$undocnta" ]; do
        echo -e "create undo tablespace UNDOTBS_CL$n datafile '$dataDir1/$tpsid/datafile/undotbs_cl$n.dbf' size 500m autoextend on maxsize unlimited;" >> $openDatabase1
        n=$(($n + 1));
done

echo -e "alter system set cluster_database=TRUE scope=spfile sid='*';" >> $openDatabase1
echo -e "shutdown abort;;" >> $openDatabase1
echo -e "startup;" >> $openDatabase1
echo -e "drop tablespace UNDOTBS_DEF including contents and datafiles;" >> $openDatabase1
echo -e "shutdown abort;" >> $openDatabase1
echo -e "exit" >> $openDatabase1

chmod 775 $openDatabase1
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
export ORACLE_SID=$nsid"1"
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

su -m $osouser -c "$sqlplus / as sysdba @$openDatabase1 " >> $openDBlog
retval=$?

if [ -e $openDatabase1 ]; then
   echo "removing $openDatabase1 "
  #rm -f $openDatabase1
fi

fi
###########################

#adding to srvctl
su -m $osouser -c "srvctl add database -d $nsid -o $ORACLE_HOME -r primary -s OPEN -p $nspfile"
origsid="init"$nsid".ora"
pwfile=$ORACLE_HOME"/dbs/orapw"$nsid"1"
var=`grep "ASM_" /act/act_scripts/asmclone/asm_node_config.conf | cut -f2 -d:`
m=0
for x in $var; do
  m=$(($m + 1));
  orasid1=$nsid$m
  orasidnode="init"$orasid1".ora"
  pwfilenode="orapw"$orasid1
   if [ $m = 1 ]
    then
      su -m $osouser -c "srvctl add instance -d $nsid -i $orasid1 -n $x "
    else
      su -m $osouser -c "ssh -o StrictHostKeyChecking=no -l$osouser $x exit"
      su -m $osouser -c "scp $ORACLE_HOME/dbs/$origsid $osouser@$x:$ORACLE_HOME/dbs/$orasidnode"
      su -m $osouser -c "srvctl add instance -d $nsid -i $orasid1 -n $x "
      su -m $osouser -c "scp $pwfile $osouser@$x:$ORACLE_HOME/dbs/$pwfilenode"
   fi
done

su -m $osouser -c "srvctl start database -d $nsid"
if [ $? -ne 0 ]; then
   echo "remount of clone database to $nsid failed"
   exit 1
fi

#reconfig controlfile snapshot location
export ORACLE_SID=$nsid"1"
export ORACLE_HOME=$oracleHome
export PATH=$PATH:$ORACLE_HOME/bin
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
dbrecover="dbrecover_"$nsid".txt"

vnum=0
vnum=`su -m $osouser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from v\$instance where version not like '10%' and version not like '11%';
exit;
EOF1`
echo "********************* vnum=$vnum ***********"
pdbcnt=0
if [ "$vnum" -gt "0" ]; then
  pdbcnt=`su -m $osouser -c "sqlplus -S / as sysdba " << 'EOF1'
  set head off;
  set feedback off
  select count(*) from v\$pdbs where name like 'PDB%SEED';
  exit;
EOF1`
fi
export vnum
export pdbcnt

if [ "$pdbcnt" -gt "0" ]; then
su -m $osouser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
CONFIGURE SNAPSHOT CONTROLFILE NAME TO '$dataDir/$nsid/snapcf_$nsid.f';
alter session set "_oracle_script"=true;
ALTER pluggable database all open;
alter pluggable database all save state;
}
exit;
EOF
else
su -m $osouser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
CONFIGURE SNAPSHOT CONTROLFILE NAME TO '$dataDir/$nsid/snapcf_$nsid.f';
}
exit;
EOF
fi

if [ "$nooratabupdate" != "true" ]; then
   if [ -f /etc/oratab ]; then
      otab=$nsid":"$ORACLE_HOME":N"
      grep "$otab" /etc/oratab >/dev/null 2>&1 || echo -e "$otab" >> /etc/oratab

      nsid1=$nsid"1"
      otab=$nsid1":"$ORACLE_HOME":Y"
      grep -w ^"$nsid1" /etc/oratab  >/dev/null 2>&1 || echo -e "$otab" >> /etc/oratab
   fi
fi

exit 0

