#!/bin/bash

csid=$1
oraHome=$2
ouser=$3

if [ "$#" -ne 3 ]; then
 echo "This script needs 3 input parameter"
 echo "usage: OracleASMDBRewind.sh <cloned database name> <Oracle Home path> <Oracle User> "
 echo " For example: OracleASMDBRewind.sh clonedb /home/oracle/app/oracle/product/11.2.0/dbhome_1 oracle "
fi

remove_dir()
{
  dirpath=$1
  if [ -d "$dirpath" ]; then
     if [ -d "$dirpath/cdump" ]; then
        rm -rf $dirpath/cdump
     fi
     if [ -d "$dirpath/alert" ]; then
        rm -rf $dirpath/alert
     fi
     if [ -d "$dirpath/incident" ]; then
        rm -rf $dirpath/incident
     fi
     if [ -d "$dirpath/sweep" ]; then
        rm -rf $dirpath/sweep
     fi
     if [ -d "$dirpath/stage" ]; then
        rm -rf $dirpath/stage
     fi
     if [ -d "$dirpath/lck" ]; then
        rm -rf $dirpath/lck
     fi
     if [ -d "$dirpath/trace" ]; then
        rm -rf $dirpath/trace
     fi
     if [ -d "$dirpath/adump" ]; then
        rm -rf $dirpath/adump
     fi
  fi

}

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR

export ORACLE_HOME=$oraHome
export ORACLE_SID=$csid
export PATH=$ORACLE_HOME/bin:$PATH
orabase=`sed -n '/<PROPERTY NAME="ORACLE_BASE"/s/.*VAL="\(.*\)"[^\n]*/\1/p' $ORACLE_HOME/inventory/ContentsXML/oraclehomeproperties.xml`
export ORACLE_BASE=$orabase

if [ -z $csid ]; then
   echo "need SID to proceed"
   exit 1
fi

if [ ! -d $ORACLE_HOME ]; then
   echo "could not find ORACLE_HOME "
   exit 1
fi

if [ ! -d $ORACLE_BASE ]; then
   echo "could not find ORACLE_BASE "
   exit 1
fi

CSID=`echo $csid | tr [a-z] [A-Z]`
echo $CSID

su -m $ouser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba << EOF
set heading off
set feedback off
shutdown immediate;
exit;
EOF
"

su -m $ouser -c "srvctl stop database -d $csid"

if [ -z "$isrestore" ] || [ "$isrestore" != "true" ]; then
su -m $ouser -c "srvctl remove database -d $csid -f"

dgid=`echo $CSID | tr [A-Z] [a-z]`
if [ -d /act/touch/$dgid ]; then
    rm -rf /act/touch/$dgid
fi
if [ -d /act/touch/$CSID ]; then
    rm -rf /act/touch/$CSID
fi

if [ "$notnsupdate" != "true" ]; then
# cleanup tns entry
csid1=$csid"1"
if [ "$addlistenerentry" = "true" ]; then
/act/act_scripts/oracleclone/cleanupTNSNames.sh $csid $ORACLE_HOME 8 $tnsadmindir
/act/act_scripts/oracleclone/cleanupTNSNames.sh $csid1 $ORACLE_HOME 9 $tnsadmindir
else
/act/act_scripts/oracleclone/cleanupTNSNames.sh $csid $ORACLE_HOME 6 $tnsadmindir
/act/act_scripts/oracleclone/cleanupTNSNames.sh $csid1 $ORACLE_HOME 7 $tnsadmindir
fi
fi

# cleanup oratab
if [ "$nooratabupdate" != "true" ]; then
/act/act_scripts/oracleclone/cleanupOratab.sh $csid
/act/act_scripts/oracleclone/cleanupOratab.sh $csid1
fi

echo "cleanup init$csid.ora"
rm -f $ORACLE_HOME/dbs/init$csid.ora
rm -f $ORACLE_HOME/dbs/init$CSID.ora
rm -f $ORACLE_HOME/dbs/snapcf_$csid.f
rm -f $ORACLE_HOME/dbs/snapcf_$CSID.f

if [ "$useexistingorapw" = "false" ] && [ "$isrestore" = "false" ]; then
   rm -f $ORACLE_HOME/dbs/orapw$csid
   rm -f $ORACLE_HOME/dbs/orapw$CSID
fi
rm -f $ORACLE_HOME/../../init$csid.ora
rm -f $ORACLE_HOME/../../init$CSID.ora

if [ -d "$ORACLE_HOME/../../admin/$CSID" ]; then
    rm -rf $ORACLE_HOME/../../admin/$CSID/adump
fi
if [ -d "$ORACLE_HOME/../../admin/$csid" ]; then
    rm -rf $ORACLE_HOME/../../admin/$csid/adump
fi
if [ -d "$ORACLE_HOME/../../diag/rdbms/$csid" ]; then
   remove_dir $ORACLE_HOME/../../diag/rdbms/$csid/$csid1
   remove_dir $ORACLE_HOME/../../diag/rdbms/$csid/$csid
fi
if [ -d "$ORACLE_HOME/../../diag/rdbms/$CSID" ]; then
    remove_dir $ORACLE_HOME/../../diag/rdbms/$CSID
    remove_dir $ORACLE_HOME/../../diag/rdbms/$CSID/$csid1
    CSID1=$CSID'1'
    remove_dir $ORACLE_HOME/../../diag/rdbms/$CSID/$CSID1
fi
if [ -d "$ORACLE_HOME/log/diag/rdbms/$csid" ]; then
    remove_dir $ORACLE_HOME/log/diag/rdbms/$csid
    remove_dir $ORACLE_HOME/log/diag/rdbms/$csid/$csid1
fi
if [ -d "$ORACLE_HOME/log/diag/rdbms/$CSID" ]; then
    remove_dir $ORACLE_HOME/log/diag/rdbms/$CSID
    remove_dir $ORACLE_HOME/log/diag/rdbms/$CSID/$csid1
    CSID1=$CSID'1'
    remove_dir $ORACLE_HOME/log/diag/rdbms/$CSID/$CSID1

fi
if [ -d "$ORACLE_HOME/../../../diag/rdbms/$csid" ]; then
    remove_dir $ORACLE_HOME/../../../diag/rdbms/$csid
    remove_dir $ORACLE_HOME/../../../diag/rdbms/$csid/$csid1

fi
if [ -d "$ORACLE_HOME/../../../diag/rdbms/$CSID" ]; then
    remove_dir $ORACLE_HOME/../../../diag/rdbms/$CSID
    remove_dir $ORACLE_HOME/../../../diag/rdbms/$CSID/csid1
    CSID1=$CSID'1'
    remove_dir $ORACLE_HOME/../../../diag/rdbms/$CSID/$CSID1
fi

rm -f setdb_$csid*
#rm -f dbrecover_$csid*
#rm -f openDBlog_$csid*
rm -f nid_$csid*

rm -f /act/act_scripts/oracleclone/setdb_$csid*
fi

echo "done rewind"

