#!/bin/bash

set -x
oraHome=$1
oraSid=$2
oraInst=$3
actdgname=$4
Dg_name=$5
archiveDest=$6
parallel=$7
failed_rerun=$8
is_PCRonLocal=$9
redoDg=${10}
ctlDg=${11}

#Temp files
tmp_loc=/act/touch
Op_file=$tmp_loc/act_move_dbf.sql
Log_file=$tmp_loc/act_dbfile_move.log
Offline_DBFile_List=$tmp_loc/OFFlineDBFile.txt
JobProgress=/act/act_scripts/asmrestore/JobProgressLog_$oraInst.txt
TempTSList=$tmp_loc/TempTSList.txt
CrtlFileList=$tmp_loc/CrtlFileList.txt
actdgname1=+$actdgname

remove_file()
{
  fremove=$1
  if [ -f "$fremove" ]; then
     rm -f $fremove
  fi
}

generate_cmd()
{
   filename=$1
   getline=`cat $Op_file | head -1`
   if [ ! -z "$getline" ]; then
      tmpgetline=`echo $getline | sed 's|/|\\\/|g'`
      sed -i "/$tmpgetline/d" $Op_file
      IFPDBDFILE=`echo $getline | awk -F":" '{print $1}'`
      if [ "$IFPDBDFILE" = "PDBDFILE" ]; then
         PDBNAME=`echo $getline | awk -F":" '{print $2}'`
         FNAME=`echo $getline | awk -F":" '{print $3}'`
         echo "WHENEVER SQLERROR EXIT SQL.SQLCODE"> $filename
         echo "alter session set container=$PDBNAME;">> $filename
         echo "ALTER DATABASE MOVE DATAFILE '$FNAME' TO '+$Dg_name';" >> $filename
         echo "exit;" >> $filename
         chmod 777 $filename
      else
         echo "WHENEVER SQLERROR EXIT SQL.SQLCODE"> $filename
         echo "ALTER DATABASE MOVE DATAFILE '$getline' TO '+$Dg_name';" >> $filename
         chmod 777 $filename
      fi
   fi
}

createRedoLocal() {
  local primDG=$1
  local optDG=$2
  local redoInfo=

  # Find redo size
  redoInfo=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
  set newpage none
  set head off;
  set echo off;
  set termout off;
  set feedback off;
  select group#||','||member from v\\$logfile;
EOF`

  # Create Redo group number and member
  for redoGM in $redoInfo; do
    redoG=`echo $redoGM | cut -d, -f1`
    redoM=`echo $redoGM | cut -d, -f2`

    #Prepare SQL statements
    queryPrim="ALTER DATABASE ADD LOGFILE MEMBER '+$primDG' TO GROUP $redoG;"
    queryOpt="ALTER DATABASE ADD LOGFILE MEMBER '+$optDG' TO GROUP $redoG;"
    #queryRedoClear="ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $redoG;"
    #querySwitchLog="ALTER SYSTEM SWITCH LOGFILE;"
    querySwitchLog="ALTER SYSTEM ARCHIVE LOG CURRENT;"
    queryGlbChk="ALTER SYSTEM CHECKPOINT GLOBAL;"
    queryDrop="ALTER DATABASE DROP LOGFILE MEMBER '$redoM';"

    if [ -z "$optDG" ]; then
      queryOpt=
    fi

    # Add new logfile
    su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
    set head on
    set newpage none
    WHENEVER SQLERROR EXIT FAILURE
    $queryPrim
    $queryOpt
EOF
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to add new logfile to group $redoG"
      exit 1
    fi

    # Drop old logfile
    FLAG_CL=1
    while [ $FLAG_CL -ne 0 ]; do
      su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
      WHENEVER SQLERROR EXIT FAILURE
      $querySwitchLog
      $queryGlbChk
      $queryDrop
EOF
      FLAG_CL=$?
    done

  done

  return 0
}

createActTempTS() {
  local TempTSListFile=$1

  while read line
  do
    dbType=`echo $line | awk '{print $1}'`
    TsName=`echo $line | awk '{print $2}'`
    DFTemp=`echo $line | awk '{print $3}'`

    if [ "$dbType" = 'PDB$SEED' ]; then
      pdbSeedClose="alter pluggable database pdb\$seed close;"
      pdbSeedOnline="alter pluggable database pdb\$seed open;"
      pdbSeedReadOnly="alter pluggable database pdb\$seed open read only;"
      pdbSeedSessPar="alter session set \"_oracle_script\"=true;"
    else
      pdbSeedClose=
      pdbSeedOnline=
      pdbSeedReadOnly=
      pdbSeedSessPar=
    fi

    querySession="alter session set container=$dbType;"
    queryCreate="create temporary tablespace act_$TsName tempfile '+$Dg_name' size 5M autoextend on;"
    queryMkDef="ALTER DATABASE DEFAULT TEMPORARY TABLESPACE act_$TsName;"
    queryDropOrig="drop tablespace $TsName including contents and datafiles;"
    queryRenameTS="alter tablespace act_$TsName rename $TsName;"
    queryMakeDefault="ALTER DATABASE DEFAULT TEMPORARY TABLESPACE $DFTemp"
    echo "-------Creating act_temp_tablespaces on $dbType  ------------"

    su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
    set newpage none
    WHENEVER SQLERROR EXIT FAILURE
    $pdbSeedSessPar
    $pdbSeedClose
    $pdbSeedOnline
    $querySession
    $queryCreate
    $queryMkDef
    $pdbSeedClose
    $pdbSeedReadOnly
EOF

    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to create TEMP tablespaces for database $dbType"
      exit 1
    fi
  done <$TempTSListFile

  return 0
}

renameActTmpTS() {
  local TempTSListFile=$1

  while read line
  do
    dbType=`echo $line | awk '{print $1}'`
    TsName=`echo $line | awk '{print $2}'`
    DFTemp=`echo $line | awk '{print $3}'`

    if [ "$dbType" = 'PDB$SEED' ]; then
      pdbSeedClose="alter pluggable database pdb\$seed close;"
      pdbSeedOnline="alter pluggable database pdb\$seed open;"
      pdbSeedReadOnly="alter pluggable database pdb\$seed open read only;"
      pdbSeedSessPar="alter session set \"_oracle_script\"=true;"
    else
      pdbSeedClose=
      pdbSeedOnline=
      pdbSeedReadOnly=
      pdbSeedSessPar=
    fi

    querySession="alter session set container=$dbType;"
    queryCreate="create temporary tablespace act_$TsName tempfile '+$Dg_name' size 5M autoextend on maxsize unlimited;"
    queryMkDef="ALTER DATABASE DEFAULT TEMPORARY TABLESPACE act_$TsName;"
    queryDropOrig="drop tablespace $TsName including contents and datafiles;"
    queryRenameTS="alter tablespace act_$TsName rename to $TsName;"
    queryMakeDefault="ALTER DATABASE DEFAULT TEMPORARY TABLESPACE $DFTemp"
    echo "-------Renaming act_temp_tablespace to original on $dbType  ------------"

    su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
    set newpage none
    WHENEVER SQLERROR EXIT FAILURE
    $pdbSeedSessPar
    $pdbSeedClose
    $pdbSeedOnline
    $querySession
    $queryDropOrig
    $queryRenameTS
    -- $queryMakeDefault
    $pdbSeedClose
    $pdbSeedReadOnly
EOF

    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to rename TEMP tablespaces act_$TsName to $TsName"
      exit 1
    fi
  done <$TempTSListFile

  return 0
}
#------------------------------------------------------------------------------------------
# Pre-checks
#------------------------------------------------------------------------------------------

remove_file $JobProgress
remove_file $Log_file
remove_file $Offline_DBFile_List

touch $JobProgress
chmod 777 $JobProgress
if [ $# -lt 5 ]; then
  echo "ERRORMSG: Correct Usage of script is:"
  echo "$0 ORACLE_HOME ORACLE_SID DISKGROUP_NAME ARCH_LOCATION"
  exit 1;
fi

if [ -z "$oraHome" ]; then
  echo "ERRORMSG: Oracle_Home not provided"
  exit 1
fi

if [ -z "$oraSid" ];then
  echo "ERRORMSG: Oracle_Sid not provided"
  exit 1
fi

if [ -z "$oraInst" ];then
  echo "ERRORMSG: Oracle_Instance not provided"
  exit 1
fi

if [ -z $Dg_name ]; then
  echo "ERRORMSG: Destination Diskgroup not provided"
  exit 1
fi

if [ "$is_PCRonLocal" = "FALSE" ]; then
  if [ -z "$redoDg" ]; then
    echo "WARNINGMSG: Redo multiplexing diskgroup not provided"
    #exit 1
  fi

  if [ -z "$ctlDg" ]; then
    echo "WARNINGMSG: Controlfile multiplexing diskgroup not provided"
    #exit 1
  fi
fi

if [ -z "$parallel" ]; then
   parallel=4
fi

OSUSER=`ps -eo user:20,pid,stime,time,args | grep -w "ora_pmon_$oraInst" | grep -vw grep | awk '{print $1}'`
if [ -z "$OSUSER" ]; then
  echo "ERRORMSG: Database ($oraInst) not running."
  exit 1
fi
OSGROUP=`id -gn $OSUSER`

echo "OSUSER: $OSUSER, OSGROUP: $OSGROUP"

guser=`ps -eo user:20,pid,stime,time,args | grep asm_pmon_+ASM | grep -vw grep | head -1 | awk '{print $1}'`
if [ -z "$guser" ]; then
  echo "ERRORMSG: Fail to get grid osuser or ASM not running."
  exit 1
fi
#ggroup=`id -gn $ggroup`
ggroup=`id -gn $guser`
echo "guser: $guser, ggroup: $ggroup"

export ORACLE_SID=$oraInst
export ORACLE_HOME=$oraHome
export PATH=$ORACLE_HOME/bin:$PATH

#Version check
VersionString=`su $OSUSER -c "sqlplus -S / as sysdba" <<'EOF'
set feedback off;
set head off;
set newpage none
WHENEVER SQLERROR EXIT FAILURE
select banner from v\$version;
EOF`

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Unable to query the DB version"
  exit 1;
fi

version=`echo $VersionString | awk -F "Release" '{print $2}' | cut -d' ' -f2 | cut -d '.' -f1`
if ! [ $version -ge 12 ]; then
  echo "ERRORMSG: Database version should be 12c and higher"
  exit 1;
fi

#Check if the DATA DiskGroup exist
IsDataDGExist=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
set feedback off;
set head off;
set newpage none
WHENEVER SQLERROR EXIT FAILURE
select count(*) from v\\$asm_diskgroup where NAME='$Dg_name';
EOF`

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to check the data diskgroup presence "
  exit 1;
fi

if [ $IsDataDGExist -eq 0 ]; then
  echo "ERRORMSG: DATA DISKGROUP provided does not exist"
  exit 1;
fi

#Check if the REDO DiskGroup exist
IsRdeoDGExist=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
set feedback off;
set head off;
set newpage none
WHENEVER SQLERROR EXIT FAILURE
select count(*) from v\\$asm_diskgroup where NAME='$redoDg';
EOF`

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to check the redo diskgroup presence "
  exit 1;
fi

if [ $IsDataDGExist -eq 0 ]; then
  echo "ERRORMSG: REDO DISKGROUP provided does not exist"
  exit 1;
fi

#Check if CDB or not
isCDB=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
set feedback off;
set head off;
set newpage none
WHENEVER SQLERROR EXIT FAILURE
select cdb from v\\$database;
EOF`

echo "isCDB: $isCDB"
if [ "$isCDB" = "YES" ]; then
  notOpenedPDBs=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
  set feedback off;
  set head off;
  set newpage none;
  WHENEVER SQLERROR EXIT FAILURE
  select name from v\\$pdbs where open_mode not like '%READ%';
EOF`
  echo "notOpenedPDBs: $notOpenedPDBs"
  if [ ! -z "$notOpenedPDBs" ]; then
    echo "ERRORMSG: PDBs $notOpenedPDBs are not in open, cannot proceed to datafile movement"
    exit 1
  fi
fi

# End of Pre-checks
#------------------------------------------------------------------------------------------


#------------------------------------------------------------------------------------------
if [ "$failed_rerun" = "false" ] || [ -z "$failed_rerun" ]; then
   #Delete the O/P file if already exist 
   if [ -e $Op_file ]; then
      rm -f $Op_file
   fi

   #Delete the Logfile if already exist
   if [ -e $Log_file.Root ]; then
      rm -f $Log_file.Root
   fi
   
   #Delete the Tempfile List 
   if [ -e $TempTSList ]; then
      rm -f $TempTSList
   fi


#Get Offline datafile list
   offlineDbFiles=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
   set feedback off;
   set head off;
   set newpage none;
   WHENEVER SQLERROR EXIT FAILURE
   select name from v\\$datafile where status='OFFLINE';
EOF`
   echo -e "offlineDbFiles:\n $offlineDbFiles"
   if [ -e $Offline_DBFile_List ]; then
      rm -f $Offline_DBFile_List
   fi
   echo $offlineDbFiles >> $Offline_DBFile_List

#Tablespace list for CDB$ROOT
   Tablespace_List=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
   set feedback off;
   set head off;
   set newpage none
   WHENEVER SQLERROR EXIT FAILURE
   select Tablespace_name from dba_tablespaces order by 1;
EOF`

  if [ $? -ne 0 ]; then
     echo "ERRORMSG: Fail to find tablespace names of CDB\$ROOT"
     exit 1
  fi

#Datafiles of CDB
   Dfile=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF1
   WHENEVER SQLERROR EXIT FAILURE
   set feedback off;
   set head off;
   set newpage none;
   set linesi 2000
   select file_name from dba_data_files where file_name like '\$actdgname1%';
EOF1`

   if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to find datafile of tablespace $x"
      exit 1
   fi

   for y in $Dfile
   do
     echo "$y" >>$Op_file
   done



#Get the PDB list
   PDBLIST=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF1
   set feedback off;
   set head off;
   WHENEVER SQLERROR EXIT FAILURE
   select NAME from v\\$pdbs;
EOF1`

   if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to find pdb names"
      exit 1
   fi

   for z in $PDBLIST
   do
     Dfile=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF1
     set feedback off;
     set head off;
     set newpage none;
     WHENEVER SQLERROR EXIT FAILURE
     set linesi 2000
     alter session set container = $z;
     select file_name from dba_data_files where file_name like '\$actdgname1%';
EOF1`

     if [ $? -ne 0 ]; then
        echo "ERRORMSG: Fail to get datafiles name from pdb $z"
        exit 1
     fi

     pdbName=`echo $z | sed 's/\\\$//g'`
     echo "pdbName: $pdbName........"

     for y in $Dfile
     do
       echo "PDBDFILE:$z:$y" >>$Op_file
     done
   done 
fi ### End of failed run check
TOTAL_DF_COUNT=`cat $Op_file | wc -l`

#archiveDest1=+$archiveDest
########## Set archiveDest
#su $OSUSER -c "sqlplus -S / as sysdba" <<EOF1
#     set feedback off;
#     set head off;
#     set newpage none;
#     WHENEVER SQLERROR EXIT FAILURE
#     set linesi 2000
#     alter system set db_recovery_file_dest='$archiveDest1';
#     exit;
#EOF1

#if [ "$?" -gt 0 ]; then
#   echo "ERRORMSG: Failed to set archive destination, Please check the error for details!"
#   exit 1
#fi
#------------------------------------------------------------------------------------------



#------------------------------------------------------------------------------------------
#Start moving the datafile

if [ "$TOTAL_DF_COUNT" -le "$parallel" ]; then
   parallel=$TOTAL_DF_COUNT
fi

i=1
while [ $i -le $parallel ]
do
   sqlfile="ACTDFILE_$i".sql
   generate_cmd $sqlfile
   su $OSUSER -c "sqlplus -S / as sysdba @$sqlfile" >> $Log_file &
   if [ -z "$processidList" ]; then
     processidList="$!:$sqlfile"
   else
     processidList=$processidList' '"$!:$sqlfile"
   fi
   i=$(($i+1))
done  
#------------------------------------------------------------------------------------------



#------------------------------------------------------------------------------------------
#Wait for the background process
linenumber=1
while [ $linenumber -ne 0 ]
do
 set -x
  echo "Datafile move is in progress. Please check the log for details $JobProgress"
  set +x
  pid=0
  linenumber=0

  for pidSet in $processidList
  do
    pid=`echo $pidSet | awk -F":" '{print $1}'`
    file=`echo $pidSet | awk -F":" '{print $2}'`
    pidexists=`ps -ef | grep -w $pid | grep -v grep | wc -l`

    if [ "$pidexists" -eq 0 ]; then
      wait $pid
      if [ $? -ne 0 ]; then
        echo "ERRORMSG: Fail to move datafiles. Check $Log_file"
        exit 1
      else
        processidList=`echo $processidList | sed "s/$pidSet//"`
        # echo "*************First move done, running another one***********"
        filecount=`cat $Op_file | wc -l`
        if [ "$filecount" -ne 0 ]; then
          generate_cmd $file
          su $OSUSER -c "sqlplus -S / as sysdba @$file" >> $Log_file &
          if [ -z "$processidList" ]; then
            processidList="$!":"$file"
          else
            processidList=$processidList' '"$!:$file"
          fi
        fi
      fi
    else
      continue;
    fi
  done

  linenumber=$pid
  if [ "$linenumber" -ne 0 ]; then
    sleep 5
    su $OSUSER -c "sqlplus -S / as sysdba"<<EOF >> $JobProgress
    set heading off
    set feedback off
    set linesi 2000
    select message from v\$session_longops where time_remaining!=0 and opname='Online data file move';
    exit;
EOF
    CURRENT_DF_COUNT=`cat $Op_file |wc -l`
    TOTALMOVED=$(($TOTAL_DF_COUNT-$CURRENT_DF_COUNT))
    echo "============= TOTAL DATAFILES MOVED OR BEING MOVED ARE:$TOTALMOVED OUT OF $TOTAL_DF_COUNT ===========" >> $JobProgress
    echo "===============================================================================" >> $JobProgress
    echo "   " >> $JobProgress
    echo "   " >> $JobProgress
  fi

done
#------------------------------------------------------------------------------------------
set -x
tmpFilesList=`ls $Op_file*`
echo "Removing files: $tmpFilesList"
for tmpFile in $tmpFilesList
do
  rm -f $tmpFile
done
rm -f ACTDFILE_*.sql
cat $Log_file*

echo "INFOMSG: Online Datafile movement completed successfully."

if [ "$is_PCRonLocal" != "FALSE" ]; then
  exit 0
fi

#------------------------------------------------------------------------------------------
# Move control file, REDO and TEMP to localDG if required
#------------------------------------------------------------------------------------------

# Create redo files in local DG
createRedoLocal $Dg_name $redoDg
echo "INFOMSG: Redologs Relocated successfully."

#Find Temp Tablespaces and tempfiles
if [ -e "$TempTSList" ]; then
  rm -f $TempTSList
fi

if [ "$isCDB" = "YES" ]; then
  DBLIST=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF1
  set feedback off;
  set head off;
  WHENEVER SQLERROR EXIT FAILURE
  select NAME from v\\$pdbs;
EOF1`

  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to find pdb names"
    exit 1
  fi
fi
DBLIST="$DBLIST CDB\$ROOT"

for pdbb in $DBLIST
do
  cdbTemp=`su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
  set feedback off;
  set head off;
  set newpage none
  col property_value for a100
  set lines 1000
  WHENEVER SQLERROR EXIT FAILURE
  --select 'CDBROOT', tablespace_name from dba_tablespaces where contents='TEMPORARY';
  alter session set container=$pdbb;
  select '$pdbb', t.tablespace_name,p.property_value from dba_tablespaces t, (SELECT property_value FROM database_properties WHERE property_name='DEFAULT_TEMP_TABLESPACE') p where contents='TEMPORARY';
EOF`

  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to find TEMP tablespace names of CDB\$ROOT"
    exit 1
  fi
  echo "$cdbTemp" >>$TempTSList
done
cat $TempTSList

#Find controlfile to restore later
controlfilename=`su $OSUSER -c "sqlplus -S / as sysdba" << EOF
set head off;
set feed off;
set echo off;
set newpage none;
set linesi 20000
select value from v\\$parameter where name='control_files' and rownum<2;
exit;
EOF`

controlfilename=`echo $controlfilename | sed 's/ //g'`
controlfilename=`echo $controlfilename |awk -F"," -v quote="'" -v OFS="','" '$1=$1 {print quote $0 quote}'`

#Update spfil, controlfile controlfile parameter
su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
WHENEVER SQLERROR EXIT SQL.SQLCODE
alter system set control_files='+$Dg_name' scope=spfile;
create pfile='/tmp/pfile_ACT__.ora' from spfile;
create spfile='+$Dg_name' from pfile='/tmp/pfile_ACT__.ora';
EOF

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to update controlfile parameter, spfile"
  exit 1
fi

# Create Temp tablespaces in local DG
createActTempTS $TempTSList

# Shutdown
su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
WHENEVER SQLERROR EXIT FAILURE
shutdown immediate;
exit;
EOF
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to shutdown after TEMP and REDO creation on local DG"
  exit 1
fi

# Create updated spfile location

# Startup
su $OSUSER -c "sqlplus -S / as sysdba" <<EOF
WHENEVER SQLERROR EXIT FAILURE
startup nomount;
exit;
EOF

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to start database after spfile creation on local DG"
  exit 1
fi

# Restore controlfile
su $OSUSER -c "rman target / " <<EOF
restore controlfile from $controlfilename ;
alter database mount;
alter database open;
alter pluggable database all open;
exit;
EOF

if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to restore controlfile to local DG after startup"
  exit 1
fi
echo "INFOMSG: Successfully started database with now controlfile, spfile and redo files"

# Rename actifio temp tablespaces to originel
renameActTmpTS $TempTSList

echo "INFOMSG: Completer TEMP tablespace movement successfully"

# End of control file, REDO and TEMP move
#------------------------------------------------------------------------------------------


exit 0
