#!/bin/sh
set -x
mntpt=$1
orasid=$2
dbswfile=$3
ouser=$4
pdbcnt=$5
archmnt=$6
tstamp=$7

if [ "$#" -lt 4 ]; then
 echo "This script needs 4 input parameter"
 echo "usage: recoverRman_tstamp.sh <Disk Group Name> <database sid name> <switch file> <Oracle User> <Log Backup location mount path> <timestamp yyyymmddhh24miss>"
exit 1
fi

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR

if [ ! -f "$dbfsPath" ]; then
  roTBS=''
else
roTBS=`cat $dbfsPath`
fi

arch_untiltime=`date -d "${tstamp:0:8} ${tstamp:8:2}:${tstamp:10:2}:${tstamp:12:2}" +"%Y-%m-%d %H:%M:%S"`
vdate=`echo $arch_untiltime | awk '{print $1}'`
vtime=`echo $arch_untiltime | awk '{print $2}'`
arch_untiltime=`date -d "$vtime $vdate +30 mins" +"%Y-%m-%d %H:%M:%S"`


if [ -z "$rmanchannels" ]; then
rmanchannels=4;
fi

allocatech=
releasech=
i=0
while [ $i -lt "$rmanchannels" ]; do
if [ -z "$allocatech" ]; then
allocatech="allocate channel d$i device type disk;"
releasech="release channel d$i;"
else
allocatech=$allocatech"allocate channel d$i device type disk;"
releasech=$releasech"release channel d$i;"
fi
i=`expr $i + 1`
done

if [ "$pdbcnt" -gt "0" ]; then
  if [ ! -z "$roTBS" ]; then
    recoverCMD="skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2,"pdb$seed":UNDOTBS1_TEMP,$roTBS"
  else
    recoverCMD='skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2,"pdb$seed":UNDOTBS1_TEMP'
  fi
else
  if [ ! -z "$roTBS" ]; then
     recoverCMD="skip tablespace $roTBS"
  else
    recoverCMD=''
  fi
fi

cuser=`whoami`

if [ "$cuser" != "$ouser" ]; then
suser="su -m $ouser -c "
else
suser=""
fi

export ORACLE_SID=$orasid
dbrecover="dbrecover_"$orasid".txt"
cstrfile="catalogArchivelog_"$orasid".txt"
chmod 777 $cstrfile 2>/dev/null

#chmod 777 $dbrecover
 rmancmd2="run
        {
        $allocatech
        recover database $recoverCMD;
        $releasech
        }"
 rmancmd3="run
        {
        $allocatech
        recover database until time \"to_date('$tstamp','yyyymmddhh24miss')\" $recoverCMD;
        $releasech
        }"

echo "******* recovering database *********** "
export NLS_DATE_FORMAT="yyyymmddhh24miss"

if [ ! -z "$imageLogOffset" ]; then
   imageLogOffset=$((imageLogOffset + 1))
fi

if [ ! -z $archmnt ]; then
     #12c catalog start with need full path
     dirs=`ls -d $archmnt*`
     cstr=""
     for d in ${dirs}; do
         archmntpt=$d"/archivelog/"
         find $archmntpt -mtime -"$imageLogOffset" ! -newermt "$arch_untiltime" -type f >> $cstrfile
     done

chmod 777 $cstrfile
     sed -i -e "s/^/catalog BACKUPPIECE '/" $cstrfile
     sed -i "s/$/' ;/" $cstrfile
#     cstr=`cat $cstrfile`

#creating an RMAN command file for cataloging
     catrcmdfile="catalogrcmdf_"$orasid".rcmd"
     if [ -f "$catrcmdfile" ]; then
        rm "$catrcmdfile"
     fi
     touch $catrcmdfile
     chmod 766 $catrcmdfile
     #build the command file
     echo "run" >> $catrcmdfile
     echo "{" >> $catrcmdfile
     echo "$allocatech" >> $catrcmdfile
     echo "catalog start with '$mntpt/archivelog' noprompt;" >> $catrcmdfile
     echo "catalog start with '$mntpt/datafile' noprompt;" >> $catrcmdfile
     cat $cstrfile >> $catrcmdfile
     echo "$releasech" >> $catrcmdfile
     echo "}" >> $catrcmdfile

   if [ ! -z $tstamp ]; then
	if [ "x$suser" = "x" ]; then
	     retv=`$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
             @$catrcmdfile
	     switch database to copy;
	     $rmancmd3
	     exit
EOF`
        else
             retv=`$suser "$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
             @$catrcmdfile
             switch database to copy;
             $rmancmd3
             exit
EOF"`
       fi
   else
        if [ "x$suser" = "x" ]; then
            retv1=`$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
            @$catrcmdfile
	    switch database to copy;
	    $rmancmd2
            exit
EOF`
        else
            retv1=`$suser "$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
            @$catrcmdfile
            switch database to copy;
            $rmancmd2
            exit
EOF"`
        fi
   fi
else
	rmancmd="run
	{
        $allocatech
	catalog start with '$mntpt/archivelog' noprompt;
	catalog start with '$mntpt/datafile' noprompt;
        $releasech
	}"

	if [ "x$suser" = "x" ]; then
	    retv2=`$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
	    $rmancmd
 	    switch database to copy;
 	    $rmancmd2
 	    exit
EOF`
	else	
            retv2=`$suser "$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
            $rmancmd
            switch database to copy;
            $rmancmd2
            exit
EOF"`
        fi
fi

if [ -e $dbswfile ]; then
   rm -f $dbswfile
fi


echo -e "DB_SWITCH=YES" > $dbswfile
echo -e "DB_REBALANCE=NO" >> $dbswfile
echo -e "UNMAP=NO" >> $dbswfile
if [ ! -z "$ACT_MAN_SWITCH" ]; then
   echo -e "ACT_MANAGED_SWITCH=YES" >> $dbswfile
fi

chmod 775 $dbswfile
	
if [ -f $cstrfile ]; then
 rm -f $cstrfile
fi
exit 0

