#!/bin/bash

set -x

die()
{
        local _ret=$1
        test -n "$_ret" || _ret=1
        test "$_PRINT_HELP" = yes && print_help >&2
        echo "$1" >&2
        exit ${_ret}
}

begins_with_short_option()
{
        local first_option all_short_options
        all_short_options='h'
        first_option="${1:0:1}"
        test "$all_short_options" = "${all_short_options/$first_option/}" && return 1 || return 0
}

conffile=/act/scripts/act_Cassandra_src.conf

source /act/scripts/act_Cassandra_src.conf

export DATABASE_DIR
export RECOVERY_FILE_DIR
export CASSANDRA_HOME
export KEYSPACE_NAME
export CLUSTER_NODE_LIST
export TABLE_NAME

_arg_dir=$DATABASE_DIR
_arg_rf=$RECOVERY_FILE_DIR
_arg_kn=$KEYSPACE_NAME

host_name=`hostname`

#parse_commandline "$@"

osname=$(uname -o)

echo "DB=$DATABASE_DIR"
echo "RF=$RECOVERY_FILE_DIR"
echo "KN=$KEYSPACE_NAME"

###  If using non-default credentials change user_name and password to reflect your installation
#user_name=cassandra
#password=cassandra

#FILE_NAME="cassandra_backup_$(date "+%Y.%m.%d-%H.%M.%S").tar"
FILE_NAME="$(hostname)_$(date "+%Y.%m.%d-%H.%M.%S").tar"


init() {
        init_database_dir
        init_recovery_file_dir
        init_keyspace_name 
}

echo "$(date) - Starting backup" > $RECOVERY_FILE_DIR/backup.txt

flush_data_to_disk() {
        echo "$(date) - Stopping cassandra compactions if they were running"  >> $RECOVERY_FILE_DIR/backup.txt
                $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 stop
        echo "$(date) - Flushing data to disk" >> $RECOVERY_FILE_DIR/backup.txt
                $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 flush
                check_for_errors
}

await_flush() {
        sleep 10
}

check_for_seed_node() {
        cseednode=`grep "seeds:" $CASSANDRA_HOME/conf/cassandra.yaml | grep ","`
        seednode=`grep "seeds:" $CASSANDRA_HOME/conf/cassandra.yaml | cut -d":" -f2 | tail -c +3 | head -c -2 | cut -d"," -f1`
        if [ -z "$cseednode" ]; then
          echo "the current node is seed node"
          seednode=`grep "seeds:" $CASSANDRA_HOME/conf/cassandra.yaml | cut -d":" -f2 | tail -c +3 | head -c -2 | cut -d"," -f1`
        else
          echo "The current node is not a seed node. protect the seed node to start the backup"
          exit 1
        fi  
}

snapshot_cassandra_database() {
        check_for_seed_node
        nodelist=`$CASSANDRA_HOME/bin/nodetool status | awk '/^(U|D)(N|L|J|M)/{print $2}'`
        for i in $nodelist
         do
          echo $i
         done
        if [ ! -z "$CLUSTER_NODE_LIST" ] && [ "$KEYSPACE_NAME" = "ALL" ]; then
           if [ -f "/act/scripts/pssh-hosts" ]; then
             rm -f /act/scripts/pssh-hosts
           fi
        pss=""
        export IFS=":"
        for i in $CLUSTER_NODE_LIST
        do
          if [ "x$pss" = "x" ]; then
           pss=$i
           echo $i > /act/scripts/pssh-hosts
          else
            echo $i >> /act/scripts/pssh-hosts
           fi
          done
          chmod 777 /act/scripts/pssh-hosts
          pssh -h /act/scripts/pssh-hosts -P "$CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 snapshot"
          check_for_errors
       fi
        if [ ! -z "$CLUSTER_NODE_LIST" ] && [ "$KEYSPACE_NAME" != "ALL" ]; then
            if [ -f "/act/scripts/pssh-hosts" ]; then
             rm -f /act/scripts/pssh-hosts
           fi
        pss=""
        export IFS=":"
        for i in $CLUSTER_NODE_LIST
        do
          if [ "x$pss" = "x" ]; then
           pss=$i
           echo $i > /act/scripts/pssh-hosts
          else
            echo $i >> /act/scripts/pssh-hosts
           fi
          done
          chmod 777 /act/scripts/pssh-hosts
          export IFS=","
          for i in $KEYSPACE_NAME
          do
          ksname=$i
         sh /act/scripts/test_eof.sh $ksname
         # create_schema $ksname
          pssh -h /act/scripts/pssh-hosts -P "$CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 snapshot $ksname"
          check_for_errors
         done
        fi
        if [  -z "$CLUSTER_NODE_LIST" ] && [ "$KEYSPACE_NAME" != "ALL" ]; then
        for i in $(echo $KEYSPACE_NAME | sed "s/,/ /g")
         do
          ksname=$i
          sh /act/scripts/test_eof.sh $ksname
         # create_schema $ksname
          echo "$(date) - Taking a snapshot of key space $KEYSPACE_NAME" >> $RECOVERY_FILE_DIR/backup.txt
                $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 snapshot $ksname
                check_for_errors 
       done
       fi
       if [ ! -z "$TABLE_NAME" ]; then
          export IFS=","
          for i in $TABLE_NAME
          do
          tabname=$i
          $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 snapshot $tabname
          check_for_errors
          done
       fi
       if [  -z "$CLUSTER_NODE_LIST" ] && [ "$KEYSPACE_NAME" = "ALL" ]; then
          echo "$(date) - Taking a snapshot of key space $KEYSPACE_NAME" >> $RECOVERY_FILE_DIR/backup.txt
                $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 snapshot 
                check_for_errors
       fi
}

init_database_dir() {
        if [ ! -d "$DATABASE_DIR/" ]; then
                request_db_dir
        fi
}

init_recovery_file_dir() {
        if [ ! -d "$RECOVERY_FILE_DIR" ]; then
                request_recovery_file_dir
        fi
}

init_keyspace_name() {
        if [ ! -z "$KEYSPACE_NAME" ]; then
                request_keyspace_name
        else
          KEYSPACE_NAME="ALL"
        fi
}

request_recovery_file_dir() {
        need_dir=true
        while $need_dir; do
       # read -p "Please enter backup file location. I.e. /home/user/backups/: " RECOVERY_FILE_DIR
                if [ ! -d "$RECOVERY_FILE_DIR" ]; then
                        echo "No backup directory found $RECOVERY_FILE_DIR"
                        echo "Wrong dir entered."
                        exit 1
                else
                        need_dir=false;
                fi
        done
}

request_keyspace_name() {
  KEYSPACE_NAME=$KEYSPACE_NAME
}

request_db_dir() {
        need_dir=true
        while $need_dir; do
        #read -p "Please enter Cassandra database directory. I.e. /var/lib/cassandra: " DATABASE_DIR
                if [ ! -d "$DATABASE_DIR/" ]; then
                        echo "No directory $DATABASE_DIR"
                        echo "Wrong directory entered."
                       exit 1
                else
                        need_dir=false;
                fi
        done
}

archive_snapshots_for_backup() {
        echo "$(date) - Archiving snapshots for backup" >> $RECOVERY_FILE_DIR/backup.txt
        (cd "$DATABASE_DIR/data"
                find . -type f -path "*/snapshots/*/*" | tar -cpf $RECOVERY_FILE_DIR/$FILE_NAME -T - --hard-dereference
                #mv $FILE_NAME $RECOVERY_FILE_DIR
        )
}

create_schema() {
sh /act/scripts/test_eof.sh $key_space_name
key_space_name=$1
sh /act/scripts/test_eof.sh $key_space_name
echo $CASSANDRA_HOME
echo $RECOVERY_FILE_DIR
echo $key_space_name
echo "describe keyspace  $key_space_name;"

scl enable python27 - << 'EOF'
$CASSANDRA_HOME/bin/cqlsh -e "describe keyspace $key_space_name;" >"$RECOVERY_FILE_DIR"/"$key_space_name"_schema.cql >/dev/null 2>&1
EOF

check_for_errors
}

create_schema_all() {
echo $CASSANDRA_HOME
echo $RECOVERY_FILE_DIR
echo $key_space_name
echo "fetch definition for all keyspaces"
scl enable python27 - << 'EOF'
$CASSANDRA_HOME/bin/cqlsh -e "describe keyspace $key_space_name;" >"$RECOVERY_FILE_DIR"/"$key_space_name"_schema.cql
EOF
check_for_errors
}

remove_snapshot() {
        echo "$(date) - Removing old snapshots" >> $RECOVERY_FILE_DIR/backup.txt
                $CASSANDRA_HOME/bin/nodetool -h localhost -p 7199 clearsnapshot
                check_for_errors
}

check_for_errors() {
        rc=$?
        if [[ $rc != 0 ]]; then
                exit $rc
        fi
}

finish() {
        echo "$(date) - Backup file created successfully. Location: $RECOVERY_FILE_DIR/$FILE_NAME" >> $RECOVERY_FILE_DIR/backup.txt
}

fail() {
        echo "$(date)Failed to backup cassandra, check console output for more details" >> $RECOVERY_FILE_DIR/backup.txt
}

#{
#init && flush_data_to_disk && await_flush && snapshot_cassandra_database && finish

#} || {
#        fail
#}

