#!/bin/bash

set -x

die()
{
	local _ret=$1
	test -n "$_ret" || _ret=1
	test "$_PRINT_HELP" = yes && print_help >&2
	echo "$1" >&2
	exit ${_ret}
}

begins_with_short_option()
{
	local first_option all_short_options
	all_short_options='h'
	first_option="${1:0:1}"
	test "$all_short_options" = "${all_short_options/$first_option/}" && return 1 || return 0
}

source /act/scripts/act_restore_Cassandra.conf

export CASSANDRA_PATH
export DATABASE_DIR
export RECOVERY_FILE_DIR
export CASSANDRA_AS_SERVICE
export COMMITLOG_DIR
export KEYSPACE_NAME
#export TABLE_NAME
export CLUSTER_NODE_LIST

osname=$(uname -o)
osuser=hadoop
osgroup=hadoop

wait_s=30

user_name=cassandra
password=cassandra

init() {
	init_database_dir
	init_commitlog_dir
	init_cassandra_home
	init_recovery_file
        init_keyspace_name
        init_table_name
}

init_database_dir() {
	if [ ! -d "$DATABASE_DIR" ]; then
		request_db_dir
	fi
}

init_commitlog_dir() {
	if [ ! -d "$COMMITLOG_DIR" ]; then
		request_commitlog_dir
	fi
}

init_recovery_file() {
	if [ ! -f "$RECOVERY_FILE_DIR" ]; then
		request_recovery_file
	fi
}

init_cassandra_home() {
	if [ "$CASSANDRA_AS_SERVICE" = false ] ; then 
		if [ ! -d "$CASSANDRA_PATH" ]; then
			request_cassandra_home
		fi
	fi
}

init_keyspace_name() {
        if [ ! -z "$KEYSPACE_NAME" ]; then
                request_keyspace_name
        else
           KEYSPACE_NAME="ALL"
           echo "Key space name should be provided"
          # exit 1
        fi
}

init_table_name() {
        if [ ! -z "$TABLE_NAME" ]; then
                request_table_name
          else
            TABLE_NAME=""
        fi
}

remove_old_data() {
	echo "Removing current data *** DESTRUCTIVE ***" 
        if [ ! -z $KEYSPACE_NAME ] && [ $KEYSPACE_NAME != "ALL" ]; then
        for i in $(echo $KEYSPACE_NAME | sed "s/,/ /g")
         do
          ksname=$i
	(cd "$DATABASE_DIR/data/$ksname/"
		#remove the db files from keyspaces
		find . -name "*.db" -delete
		#remove subfolders under snapshots
		find . -type d  -path "*/snapshots/*" | xargs rm -fr
		#find . -type f ! -path "*/snapshots/*" -exec rm -f {} \;
		check_for_errors
		#find . -type d ! -path "*/snapshots/*" -iname "*_Idx" -exec rm -rf {} \;
		#check_for_errors
	)
       done
      else
         (cd "$DATABASE_DIR/data/"
                #remove the db files from keyspaces
                find . -name "*.db" -delete
                #remove subfolders under snapshots
                find . -type d  -path "*/snapshots/*" | xargs rm -fr
                #find . -type f ! -path "*/snapshots/*" -exec rm -f {} \;
                check_for_errors
                #find . -type d ! -path "*/snapshots/*" -iname "*_Idx" -exec rm -rf {} \;
                #check_for_errors
        )  
      fi 
}

stop_cassandra() {
       for i in $(echo $KEYSPACE_NAME | sed "s/,/ /g")
         do
          ksname=$i
          import_schema $ksname
         done 
	if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then 
		echo "Stoping Cassandra service"
			systemctl stop cassandra
			echo "Waiting $wait_s seconds for Cassandra to stop"
			sleep $wait_s
                        nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
			check_for_errors
	else
		echo "Stoping Cassandra process"
                        nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
			check_for_errors
	fi
	sleep 5
}

start_cassandra() {
	if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then 
		echo "Starting Cassandra service"
			systemctl start cassandra
	else
		if [ -d $CASSANDRA_PATH ]; then
			echo "Starting Cassandra process"
				(cd "$CASSANDRA_PATH/bin/"
					su - $osuser -c "$CASSANDRA_PATH/bin/cassandra -f &"
					check_for_errors
				)
		else
			echo "Cassandra home set to: $CASSANDRA_PATH"
			echo "Cassandra home is not set correctly. Example usage: $ cassandra=/opt/cassandra-2.2.5 ./restore_single_node.sh"
			echo "Or you can start Cassandra manually"
		fi
	fi
	echo "Waiting $wait_s seconds for Cassandra"
	sleep $wait_s
}

restore_data() {
sh /act/scripts/test_restore_key.sh 
}

clear_commitlog() {
	(cd "$COMMITLOG_DIR"
	find . -name "*.log"  -delete
	)
}

commitlog_not_found() {
	echo "Commitlog dir not cleared, please clear it manually"
}

commitlog_found() {
	echo "Removing old commit logs"
}

repair_keyspaces_linux() {
export IFS=":"
for i in $CLUSTER_NODE_LIST
do
ssh -t -t root@$i $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 repair >/dev/null 2>&1 
       # $NODETOOL -h localhost -p 7199 repair auth
	#$NODETOOL -h localhost -p 7199 repair esi
	#$NODETOOL -h localhost -p 7199 repair system
	#$NODETOOL -h localhost -p 7199 repair system_auth
	#$NODETOOL -h localhost -p 7199 repair system_distributed
	#$NODETOOL -h localhost -p 7199 repair system_traces
	check_for_errors
done
}

keyspace_refresh() {
export IFS=":"
for i in $CLUSTER_NODE_LIST
do
ssh -t -t root@$i $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 refresh $KEYSPACE_NAME $TABLE_NAME
check_for_errors
done
}

remove_recovery() {
	echo "Removing old snapshots"
	(cd "$DATABASE_DIR/data"
		nohup find . -path "*/snapshots/*" -exec rm -rf {} \; >/dev/null 2>&1 &
		check_for_errors
	)
}

remove_recovery_cluster_nodes() {
export IFS=":"
for i in $CLUSTER_NODE_LIST
do
        echo "Removing old snapshot on $i node"
              ssh -t -t root@$i nohup find $DATABASE_DIR/data/ -path "*/snapshots/*" -exec rm -rf {} \; >/dev/null 2>&1 &
                check_for_errors
done
}

request_recovery_file() {
	need_file=true
	while $need_file; do
    	#read -p "Please enter full backup file path. I.e. /home/user/backups/cassandra_backup_2016.07.29.12.23.24.tar: " RECOVERY_FILE_DIR
		RECOVERY_FILE=`find $RECOVERY_FILE_DIR/ -name *.tar | head -n1`
                if [ ! -f "$RECOVERY_FILE" ]; then
			echo "No backup file found $RECOVERY_FILE"
			echo "Wrong file entered."
		else
			need_file=false;
		fi
	done
}

request_cassandra_home() {
	need_dir=true
	while $need_dir; do
    	#read -p "Please enter Cassandra home directory. I.e. /opt/cassandra-2.2.5: " CASSANDRA_PATH
		if [ ! -d "$CASSANDRA_PATH/bin/" ]; then
			echo "No directory $CASSANDRA_PATH/bin/"
			echo "Wrong directory entered."
		else
			need_dir=false;
		fi
	done
}

request_commitlog_dir() {
	need_dir=true
	while $need_dir; do
    	#read -p "Please enter Cassandra commitlog directory. I.e. /var/lib/cassandra/commitlog: " COMMITLOG_DIR
		if [ ! -d "$COMMITLOG_DIR" ]; then
			echo "No directory $COMMITLOG_DIR"
			echo "Wrong directory entered."
		else
			need_dir=false;
		fi
	done
}

request_db_dir() {
	need_dir=true	
	while $need_dir; do
    	#read -p "Please enter Cassandra database directory. I.e. /var/lib/cassandra: " DATABASE_DIR
		if [ ! -d "$DATABASE_DIR/data/" ]; then
			echo "No directory $DATABASE_DIR/data/"
			echo "Wrong directory entered."
		else
			need_dir=false;
		fi
	done
}

load_sstable() {
for tabstring in `ls $DATABASE_DIR"/data/"$KEYSPACE_NAME"/" | grep -w $TABLE_NAME`; do
for tab in `find $DATABASE_DIR"/data/"$KEYSPACE_NAME"/"$tabstring  -mindepth 2 -maxdepth 2 -type d | sort`; do
$CASSANDRA_PATH/bin/sstableloader -d localhost $tab
check_for_errors
done
done
}

recover_snapshot_from_backup() {
	echo "Restoring snapshot from backup"
	(cd "$DATABASE_DIR/data"
		remove_old_data
                if [ -z "$CLUSTER_NODE_LIST" ]; then
		clear_commitlog
                else
                 clear_commit_cluster_nodes
                fi
		tar --same-owner -C "$DATABASE_DIR/data/" -xvf "$RECOVERY_FILE"
                chmod -R 777 $DATABASE_DIR/data/
		echo "Finished Restoring snapshot from backup"
		check_for_errors
	)
}

request_keyspace_name() {
  KEYSPACE_NAME=$KEYSPACE_NAME
}

request_table_name() {
  TABLE_NAME=$TABLE_NAME
}

import_schema() {
ksname=$1
echo $CASSANDRA_PATH
echo $RECOVERY_FILE_DIR
echo $ksname
ksfile=`find $RECOVERY_FILE_DIR/ -name $ksname"_schema.cql"`
scl enable python27 - << EOF
$CASSANDRA_HOME/bin/cqlsh <$ksfile
EOF

rf=$?
if [[ $rc != 0 ]]; then
tabcid=false
else
 tabcid=true
fi

}

clear_commit_cluster_nodes() {
export IFS=":"
for i in $CLUSTER_NODE_LIST 
do
ssh -t -t root@$i sudo rm -f $COMMITLOG_DIR/*.log
ssh -t -t root@$i sudo rm -f "$DATABASE_DIR"/data/"$ksname"/*.db
done
}

reinit_all_nodes () {
j=0
export IFS=":"
for i in $CLUSTER_NODE_LIST
do
if [ $j -gt 0 ]; then
if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then
                echo "Stoping Cassandra service"
                       ssh -t -t root@$i systemctl stop cassandra
                        echo "Waiting $wait_s seconds for Cassandra to stop"
                        sleep $wait_s
                        ssh -t -t root@$i nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        ssh -t -t root@$i systemctl start cassandra
                        sleep $wait_s
                        ssh -t -t root@$i $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 repair >/dev/null 2>&1
                        check_for_errors
        else
                echo "Stoping Cassandra process"
                        ssh -t -t root@$i nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        sleep 5
                        echo "Starting Cassandra process"
                        ssh -t -t root@$i su - $osuser -c "$CASSANDRA_PATH/bin/cassandra -f" >/dev/null 2>&1 &
                        echo "Waiting $wait_s seconds for Cassandra"
                        sleep $wait_s
                        ssh -t -t root@$i $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 repair >/dev/null 2>&1
                        check_for_errors
        fi
        sleep 5
   fi
j=$(( j + 1 ))
done
}

reinit_local_node () {
if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then
                echo "Stoping Cassandra service"
                        systemctl stop cassandra
                        echo "Waiting $wait_s seconds for Cassandra to stop"
                        sleep $wait_s
                        nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        systemctl start cassandra
                        sleep $wait_s
                        $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 repair >/dev/null 2>&1
                        check_for_errors
        else
                echo "Stoping Cassandra process"
                        nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        sleep 5
                        echo "Starting Cassandra process"
                        su - $osuser -c "$CASSANDRA_PATH/bin/cassandra -f" >/dev/null 2>&1 &
                        echo "Waiting $wait_s seconds for Cassandra"
                        sleep $wait_s
                        $CASSANDRA_PATH/bin/nodetool -h localhost -p 7199 repair >/dev/null 2>&1
                        check_for_errors
        fi
        sleep 5
}

stop_cluster_nodes() {

for i in $(echo $KEYSPACE_NAME | sed "s/,/ /g")
do
  ksname=$i
  import_schema $ksname
done

export IFS=":"
for i in $CLUSTER_NODE_LIST
do
if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then
                echo "Stoping Cassandra service"
                       ssh -t -t root@$i systemctl stop cassandra
                        echo "Waiting $wait_s seconds for Cassandra to stop"
                        sleep $wait_s
                        ssh -t -t root@$i nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        check_for_errors
        else
                echo "Stoping Cassandra process"
                        ssh -t -t root@$i nohup ps aux | grep -i Cassandra | grep -v grep | awk {'print $2'} | xargs kill >/dev/null 2>&1 &
                        check_for_errors
        fi
        sleep 5
done
}

start_cluster_nodes() {
export IFS=":"
for i in $CLUSTER_NODE_LIST
do
if [ "$CASSANDRA_AS_SERVICE" = '' ] || [ "$CASSANDRA_AS_SERVICE" = true ] ; then
                echo "Starting Cassandra service"
                       ssh -t -t root@$i systemctl start cassandra
        else
                if [ -d $CASSANDRA_PATH ]; then
                        echo "Starting Cassandra process"
                                        ssh -t -t root@$i su - $osuser -c "$CASSANDRA_PATH/bin/cassandra -f" >/dev/null 2>&1 &
                                        check_for_errors
                else
                        echo "Cassandra home set to: $CASSANDRA_PATH"
                        echo "Cassandra home is not set correctly. Example usage: $ cassandra=/opt/cassandra-2.2.5 ./restore_single_node.sh"
                        echo "Or you can start Cassandra manually"
                fi
        fi
        echo "Waiting $wait_s seconds for Cassandra"
        sleep $wait_s
done
}

check_for_errors() {
	rc=$?
	if [[ $rc != 0 ]]; then
		exit $rc
	fi
}

repair_keyspaces() {
		repair_keyspaces_linux
}

fail() {
	echo "Database recovery failed, please check console output for more details"
}

finish() {
	echo "Finished snapshot recovery"
}

{
      if [ -z "$CLUSTER_NODE_LIST" ] && [ "x$TABLE_NAME" = "x" ]; then
	init && stop_cassandra && recover_snapshot_from_backup && restore_data && remove_recovery && start_cassandra && repair_keyspaces && finish
       fi
       if [ ! -z "$CLUSTER_NODE_LIST" ] && [ "x$TABLE_NAME" = "x" ]; then
        init && stop_cluster_nodes && recover_snapshot_from_backup && restore_data && remove_recovery_cluster_nodes && start_cluster_nodes && repair_keyspaces && reinit_all_nodes && reinit_local_node && finish
       fi 
       if [ ! -z "TABLE_NAME" ] && [ "x$TABLE_NAME" != "x" ]; then
        init && recover_snapshot_from_backup && remove_recovery && load_sstable && keyspace_refresh && repair_keyspaces && finish 
       fi
} || {
	fail
}

