#!/bin/sh

if [ "$#" -ne 1 ]; then
 echo "This script needs 1 input parameter"
 echo "usage: asmEnvDetail.sh <0/1>, where 1 => Do not run any blocking commands and 0 => Run all diagnostic commands"
exit 1
fi

nonblock=$1

os=`uname -s`
otab=/etc/oratab

orauser=`ps -ef | grep ora_pmon_"$dbsid" | grep -vw grep | head -1 | sed -e 's/^[ \t]*//' | cut -d" " -f1`

awkc=awk
if [ $os = "SunOS" ]; then
   otab=/var/opt/oracle/oratab
   awkc=nawk
fi

echo "********** ORACLE running process result (ps -ef) ************"

ps -ef | grep pmon

if [ "$os" = "Linux" ]; then
   echo ""
   echo "********** Additional details about ASM timezone TZ ************"
   asmpmonid=`ps -aef | grep asm_pmon | grep -v grep | awk '{print $2}'`
   cat /proc/$asmpmonid/environ
   echo ""
   echo "********** Additional details about Oracle timezone TZ ************"
   orapmonid=`ps -aef | grep ora_pmon | grep -v grep | head -1 | awk '{print $2}'`
   cat /proc/$orapmonid/environ
fi

echo "********** ORACLE oratab entry (cat /etc/oratab) ************"

cat $otab

echo "********** Protected ORACLE database ASM Details ************"

isGridExists=`ps -ef | grep "asm_pmon_+ASM" |grep -vw grep| wc -l`
if [ $isGridExists -eq 0 ]; then
   echo "********** ASM process not running ************"
   exit
else
    ASMS=`ps -ef | grep asm_pmon_+ASM | grep -vw grep | sort | head -1 | sed 's/.*asm_pmon_//g'`
    inst_pid=`ps -ef | grep asm_pmon_+ASM | grep -vw grep | sort | head -1 | $awkc '{print $2}'`
    if [ $os = "AIX" ]; then
        full_path=`ls -l /proc/$inst_pid/cwd | $awkc '{print $NF}'`
        ASMHOME=`dirname $full_path`
    else
        if [ $os = "HP-UX" ]; then
            ASMHOME=`pfiles $inst_pid | grep '/bin/' |grep $inst_pid | $awkc '{print $NF}' | $awkc -F'/bin/' '{print $1}'`
        else
            full_path=`pwdx $inst_pid | $awkc '{print $NF}'`
            ASMHOME=`dirname $full_path`
        fi
    fi
fi

if [ -z "$ASMS" ]; then
   echo "********** ASM Details not available ************"
   ASMHOME=''
fi

if [ -z "$ASMHOME" ]; then
   echo "********** ASM Details not available ************"
   exit
else
    echo "asm home:$ASMHOME"
fi

ORACLE_HOME=$ASMHOME
LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
PATH=$ASMHOME/bin:$PATH
ORACLE_SID="$ASMS"
asmcmd=$ASMHOME/bin/asmcmd
if [ $os = "SunOS" ]; then
    LD_LIBRARY_PATH_64=$ASMHOME/lib:/usr/lib:/lib
    export LD_LIBRARY_PATH_64
fi
export ORACLE_HOME
export LD_LIBRARY_PATH
export PATH
export ORACLE_SID

guser=`ps -ef | grep _pmon_ | grep $ORACLE_SID | grep -vw grep | head -1 | sed -e 's/^[ \t]*//' | cut -d" " -f1`
echo "********** Protected ORACLE database ASM lsdg  ************"

echo $ASMHOME
echo $LD_LIBRARY_PATH
echo $ORACLE_SID
echo $guser

if [ "$os" != "Linux" ]; then
su $guser -c "$asmcmd lsdg"
else
su -m $guser -c "$asmcmd lsdg"
fi
echo "********** Protected ORACLE database ASM diskgroup status  ************"

if [ "$nonblock" = "0" ]; then
  if [ "$os" != "Linux" ]; then
    su $guser -c "$ORACLE_HOME/bin/sqlplus -s / as sysasm @/act/act_scripts/asmEnvDetail.sql"
  else
    su -m $guser -c "$ORACLE_HOME/bin/sqlplus -s / as sysasm @/act/act_scripts/asmEnvDetail.sql"
  fi
fi

echo "********** Protected ORACLE database ASM disk link (/dev/actifio/asm  ************"
ls -ltr /dev/actifio/asm
echo "********** Protected ORACLE database RAC CRS details  ************"
crsctl stat res -t

exit;
