#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

DBTYPE=$1
dbsid=$2
LOGBASEPATH=$3
LIMIT=$4
FROM_TIME=$5
END_TIME=$6
HANALOGPATH=${LOGBASEPATH}/HANA

mkdir -p ${HANALOGPATH}
echo "hanaEnvDetail.sh DBTYPE: $DBTYPE DBSID: $dbsid FROM_TIME: $FROM_TIME END_TIME: $END_TIME LIMIT: $LIMIT" >> ${HANALOGPATH}/diag.out

if [ $# -lt 3 ]; then
    echo "Usage ./hanaEnvDetail.sh <DBTYPE> < DBSID> <LOGBASEPATH> <LIMIT>" >> ${HANALOGPATH}/diag.out
    exit 1
fi

if [[ $DBTYPE != HANA ]]
    then echo "ERRORMSG: The DB Type is not recognized" >> ${HANALOGPATH}/diag.out
    exit 1
fi

if [[ $DBTYPE == HANA ]]
    then
    DBSIDlow=`echo "$dbsid" | awk '{print tolower($0)}'`
    DBSIDupper=`echo "$dbsid" | awk '{print toupper($0)}'`
    HANAuser=`echo "$DBSIDlow""adm"`
    SYSNUMBER=`su - "$HANAuser" bash -c 'echo $TINSTANCE' | tail -1`
    globalpath=`su - $HANAuser -c 'echo $DIR_INSTANCE' | tail -1`
    globalpath=`dirname $globalpath`
    globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini
    exepath=$globalpath/SYS/exe/hdb

    if [[ X$dbsid == X ]]
        then echo "ERRORMSG: Missing argument Database SID" >> ${HANALOGPATH}/diag.out
        exit 1
    else
        length=`expr length "$dbsid"`
        if [[ "$length" != 3 ]]
            then echo "ERRORMSG: Invalid argument for Database SID $dbsid - length is different than 3" >> ${HANALOGPATH}/diag.out
            exit 1
        else
            echo "INFO : Database SID is $DBSIDupper" >> ${HANALOGPATH}/diag.out
        fi
    fi

    if id "$HANAuser" >/dev/null 2>&1
        then echo "INFO: user $HANAuser exists" >> ${HANALOGPATH}/diag.out
    else
        echo "ERRORMSG: user $HANAuser does not exist" >> ${HANALOGPATH}/diag.out
        exit 1
    fi
fi

if [[ $DBTYPE == HANA ]]
    then
    tar -czvf ${HANALOGPATH}/ACT_HANA_ini_files.tar.gz  $globalpath/SYS/global/hdb/custom/config/*.ini
    tar -czvf ${HANALOGPATH}/ACT_HANA_ini_TENANT_files.tar.gz $globalpath/SYS/global/hdb/custom/config/DB_*/*.ini

    HANAISUP_CMD=`echo "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function GetSystemInstanceList | grep -Ei \\\\\"GREEN|YELLOW\\\\\" | wc -l\" "`
    echo "$HANAISUP_CMD" >> ${HANALOGPATH}/diag.out
    HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
    if [[ X$HANAISUP == X0 ]]
        then
        touch ${HANALOGPATH}/ACT_HANA_topology.txt
        chown $HANAuser ${HANALOGPATH}/ACT_HANA_topology.txt
        su - "$HANAuser" bash -c "hdbnsutil -exportTopology ${HANALOGPATH}/ACT_HANA_topology.txt"
    fi

    servername=`cat ${globalpath}/SYS/global/hdb/custom/config/nameserver.ini | grep ^roles_ |cut -d'_' -f2- |cut -d'=' -f-1`
    for i in ${servername}
        do
        NODE=`echo $i`
        NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
        TRACEPATH=${globalpath}/HDB${SYSNUMBER}/${NODE}/trace
        if [ ! -z "$FROM_TIME" ] && [ ! -z "$END_TIME" ]; then
            find $TRACEPATH -maxdepth 1 -type f -newermt "$FROM_TIME" ! -newermt "$END_TIME" |xargs tar -czvf ${HANALOGPATH}/ACT_HANA_trace_${NODE}_files.tar.gz;
        else
            find $TRACEPATH -maxdepth 1 -type f -newer $TRACEPATH/`ls -t $TRACEPATH | head -$LIMIT | tail -1`| xargs tar -czvf ${HANALOGPATH}/ACT_HANA_trace_${NODE}_files.tar.gz;
            #find $TRACEPATH -type f -newer $TRACEPATH/`ls -t $TRACEPATH | head -$LIMIT | tail -1` -exec tar -czvf ${HANALOGPATH}/ACT_HANA_trace_${NODE}_files.tar.gz "{}" \;
        fi
        if [ "$BACKINT" = "TRUE" ]; then
            if [ -d /act/custom_apps/saphana/backintlog ]
                then mkdir ${HANALOGPATH}/ACT_HANA_backintlog_${NODE}_files
                scp -pr ${NODE}:/act/custom_apps/saphana/backintlog ${HANALOGPATH}/ACT_HANA_backintlog_${NODE}_files
                tar -czvf ${HANALOGPATH}/ACT_HANA_backintlog_${NODE}_files.tar.gz ${HANALOGPATH}/ACT_HANA_backintlog_${NODE}_files
                rm -rf ${HANALOGPATH}/ACT_HANA_backintlog_${NODE}_files
            fi
        fi
        tenantname=`ls -d ${globalpath}/HDB${SYSNUMBER}/${NODE}/trace/DB_*`
        for Y in ${tenantname}
            do
            TENANT=`echo $Y`
            TENANT=`basename $TENANT`
            TENANTPATH=${globalpath}/HDB${SYSNUMBER}/${NODE}/trace/${TENANT}
            if [ ! -z "$FROM_TIME" ] && [ ! -z "$END_TIME" ]; then
                find ${TENANTPATH} -maxdepth 1 -type f -newermt "$FROM_TIME" ! -newermt "$END_TIME" |xargs tar -czvf ${HANALOGPATH}/ACT_HANA_trace_${NODE}_files.tar.gz;
            else
                find ${TENANTPATH} -maxdepth 1 -type f -newer ${TENANTPATH}/`ls -t ${TENANTPATH} | head -${LIMIT} | tail -1` |xargs tar -czvf ${HANALOGPATH}/ACT_HANA_trace_${TENANT}_${NODE}_files.tar.gz;
            fi
        done
    done
fi

exit 0
