#!/bin/sh
set +x

# This script will take care of the following tasks:
# 1) Delete sky cert
# 2) Remove All Vdiks from sky.
# 3) UnProtect All the Host Apps from Sky.
# 4) Unmount all the image from Sky for ths Host.
# 5) Remove iscsi sessions and remove all devices from dmsetup.
#########################################################################

for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   export $ARGUMENT
done

set -x
if [ "$#" -lt 1 ]; then
  echo " This script takes 1 mandatory input key/value(Hostname) and 2 optional key/value pairs"
  echo " For example: Hostname=maxdb-server-1 SkyIP=172.23.16.126 SkyPassword=xxxxxx"
  exit 1
fi

if [ -z $Hostname ]; then
  echo "Error: Hostname is mandatory"
  exit 1
fi

rm -f /act/certs/trusted/*

hostname=$Hostname

vgnames=`vgs --noheadings -o vg_name 2> /dev/null | grep "dumpvg"`
vgnames=$(echo $vgnames | tr " " "\n")
if [ ! -z "$vgnames" ]; then
	for vgname in $vgnames
	do
		vgname=`echo $vgname | grep "dumpvg$"`
		if [ ! -z "$vgname" ]; then
			lvname=`lvs -o lv_name,vg_name 2> /dev/null | grep $vgname | awk '{print $1}'`
			pvnames=`pvs -o pv_name,vg_name 2> /dev/null | grep $vgname | awk '{print $1}'`
			umount /dev/$vgname/$lvname
      mount | grep -w /dev/$vgname/$lvname
      if [ "$?" -eq 0 ]; then
          echo "ERROR:Exiting /dev/$vgname/$lvname is in use. Probably DB Dump Backup Job is using this mountpoint!!"
          echo "Cancel the DB Job using this mountpoint and after completion try cleanup again!!!"
          exit 1
      fi
			sed -i '/^\/dev\/'$vgname'\/'$lvname'.*/d' /etc/fstab
			vgremove -f $vgname
			pvnames=$(echo $pvnames | tr " " "\n")
			if [ ! -z "$pvnames" ]; then
				for pvname in $pvnames
				do
				    pvremove $pvname
				done
			fi
		fi
	done
fi

if [ ! -z $SkyIP ]; then
	SkyUserName="admin"
	VendorKey="act"

  set +x
	loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
	connectivityCheck=$(curl -w "\n" --max-time 10 -k -XPOST $loginURL)
	if [ "$?" -gt 0 ]; then
		echo "WARNING:  Connection timed out after 10001 milliseconds!"
		echo "Ignoring Sky Operations since its unreachable[could be blocked HTTPS port or deleted]"
    exit 1
	else
		sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
		if [ -z "$sessionid" ] || [ "$sessionid" == "null" ]; then
			echo "Invalid password..PLease check if Sky Password is correct'"
			exit 1
		fi
    set +x
		reportAppsUrl="https://$SkyIP/actifio/api/report/reportapps?sessionid=$sessionid"
		reportAppsResponse=$(curl -w "\n" -s -k -G $reportAppsUrl)
		appIDs=$(echo $reportAppsResponse | jq -cr '.result[] |
		   select(.HostName=="'"$hostname"'") | .AppID')
		echo $appIDs
		appIDs=$(echo $appIDs | tr " " "\n" | sort | uniq)
		if [ ! -z "$appIDs" ]; then
		  for appID in $appIDs
		  do
		  echo "Go For Unprotection of APP: $appID"
		  slaURL="https://$SkyIP/actifio/api/info/lssla?sessionid=$sessionid&filtervalue=appid%3D$appID"
		  slaID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $slaURL | jq -cr '(.result[] | .id)')
		  echo "SLA ID from Name: " $slaID
		  unprotectURL="https://$SkyIP/actifio/api/task/rmsla?sessionid=$sessionid&argument=$slaID"
		  unprotectResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $unprotectURL)
		  done
		fi

		reportMountimagesUrl="https://$SkyIP/actifio/api/report/reportmountedimages?sessionid=$sessionid"
		reportMountedimagesResponse=$(curl -w "\n" -s -k -G $reportMountimagesUrl)
		imagenames=$(echo $reportMountedimagesResponse | jq -cr '.result[] |
		   select(.MountedHost=="'"$hostname"'") | .MountImageName')
		imagenames=$(echo $imagenames | tr " " "\n" | sort | uniq)
		 if [ ! -z "$imagenames" ]; then
		   for imagename in $imagenames
		   do
		     echo "mounted image name: $imagename"
		     unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

		     inputParamsForUnmount='{
		     "sessionid":"'"$sessionid"'",
		     "image":"'"$imagename"'",
		     "delete":"true"
		    }'

		    unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
		    echo "unmountResponse :" $unmountResponse
		   done
		 fi

     set -x
     reportMappedDiskUrl="https://$SkyIP/actifio/api/report/reportmappeddisks?sessionid=$sessionid"
     vdisknames=$(curl -w "\n" -s -k -G $reportMappedDiskUrl | jq -cr '.result[] |
      select(.MappedHost=="'"$hostname"'") | .VDiskName')
     echo $vdisknames
     vdisknames=$(echo $vdisknames | tr " " "\n")
     if [ ! -z "$vdisknames" ]; then
       for vdiskname in $vdisknames
       do
        set +x
        source `dirname $0`/removeVdisk.sh $vdiskname
        set -x
       done
     fi

     hostId=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$hostname" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
     set -x

		 #Remove Host from Sky
		 rmHostURL="https://$SkyIP/actifio/api/task/rmhost?sessionid=$sessionid&argument=$hostId"
		 rmHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $rmHostURL)
		 echo "rmHostResponse :" $rmHostResponse
     if [[ "$rmHostResponse" == *errormessage* ]]; then
      echo "Failed to remove Host. Reason :" $rmHostResponse
      exit 1
     fi
	 fi
fi

if [ ! -z $SkyIP ]; then
	targetIQN=`iscsiadm -m node | grep $SkyIP':' | awk '{print $2}'`
	if [ ! -z $targetIQN ]; then
		iscsiadm -m node -u -T $targetIQN -p $SkyIP:3260
		iscsiadm -m node -o delete -T $targetIQN
	fi
	iscsiadm -m discoverydb -t sendtargets -p $SkyIP:3260 -o delete
	iscsiSessionId=`iscsiadm -m session | grep $SkyIP':' | awk -F" " '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1`
	if [ ! -z $iscsiSessionId ]; then
		iscsiadm -m session -r $iscsiSessionId -u
	fi
fi

dmsetup remove_all
vgimport -a 2>/dev/null
/opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
