#!/bin/sh
# Script to configure iscsi, configure CHAP and start the service.

osRelease=$(cat /etc/*-release | grep -i SLES)
if [ ! -z "$osRelease" ]; then
  isIscsiRunning=$(systemctl status iscsid | grep Active)
  if [[ $isIscsiRunning == *"Active: active (running)"* ]]; then
    echo "ISCSI service is running" > /dev/null
  else
    echo "Trying to Start Iscsi Service"
    checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
    if [ -z $checkopeniscsi ]; then
      if ! type "zypper" 2> /dev/null; then
        echo "Error : Zypper is not installed. Unable to proceed with ISCSI installation."
        exit 1
      fi
    # open-iscsi-2.0.874-53.10.1.x86_64 (ensure that right iscsi package version is present)
      zypper -n install open-iscsi
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: ******* Could not install open-iscsi ************"
        exit $retval
      fi
      checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
      if [ -z $checkopeniscsi ]; then
        echo "Error: Cannot find open-iscsi package in the system"
        exit 1
      fi
    fi
  fi

  if [ -z "$IQN" ]; then
    sed -i 's/:[^:]*$/:'`hostname`'/' /etc/iscsi/initiatorname.iscsi
  else
    sed -i '/InitiatorName=/c\InitiatorName='"$IQN"'' /etc/iscsi/initiatorname.iscsi
  fi
  systemctl enable iscsi
  systemctl enable iscsid
  systemctl stop iscsid 2>/dev/null
  systemctl start iscsid
else
  isIscsiRunning=$(service iscsid status)
  if [[ $isIscsiRunning == *"is running"* ]]; then
     :
  else
    echo "Trying to Start Iscsi Service"
    checkopeniscsi=$(yum list installed | grep iscsi-initiator-utils)
    if [ -z "$checkopeniscsi" ]; then
      if ! type "yum" > /dev/null 2>&1; then
        echo "Error : yum is not installed. Unable to proceed with ISCSI installation."
        exit 1;
      fi
      # open-iscsi-2.0.874-53.10.1.x86_64 (ensure that right iscsi package version is present)
      yum -y install iscsi-initiator-utils
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: ******* Could not install open-iscsi ************"
        exit $retval
      fi
      checkopeniscsi=$(yum list installed | grep iscsi-initiator-utils)
      if [ -z "$checkopeniscsi" ]; then
        echo "Error: Cannot find open-iscsi package in the system"
        exit 1
      fi
    fi
  fi

  if [ -z "$IQN" ]; then
    sed -i 's/:[^:]*$/:'`hostname`'/' /etc/iscsi/initiatorname.iscsi
  else
    sed -i '/InitiatorName=/c\InitiatorName='"$IQN"'' /etc/iscsi/initiatorname.iscsi
  fi
  chkconfig iscsid on
  service iscsid stop > /dev/null 2>&1
  service iscsid start
fi

isIscsiRunning=$(systemctl status iscsid | grep Active)
if [[ $isIscsiRunning == *"Active: active (running)"* ]]; then
    echo "ISCSI service is running"
else
  isIscsiRunning=$(service iscsid status)
  if [[ $isIscsiRunning == *"is running"* ]] || [[ $isIscsiRunning == *"dead but pid file exists"* ]]; then
     echo "ISCSI service is running"
  else
     echo "ISCSI service is stopped"
     exit 1
  fi
fi
