#!/bin/sh
set -x

# Script to check/configure Pre-Requisites.

if [ "$#" -lt 2 ]; then
  echo " This script needs 2 mandatory input parameters..Password is env supplied and IQN/LVMSnapChunkSize(DB2) is optional"
  echo " usage: preRequisites.sh key-value pairs <SkyIP=> <CHAPUsername=> <CHAPPassword=> <IQN=> <LVMSnapChunkSize=>"
  echo " For example: configurePrerequisites.sh SkyIP=172.23.16.230 CHAPUsername=xxxx CHAPPassword=xxx IQN=yyy"
  exit 1
fi

set +x
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   export $ARGUMENT
done

if [ -z $SkyIP ]; then
  echo "Error: Sky Appliance IP is mandatory"
  exit 1
fi

if [ -z $CHAPUsername ]; then
  echo "Error: CHAP Username is mandatory"
  exit 1
fi

if [ -z $CHAPPassword ]; then
  echo "Error: CHAP Password is mandatory"
  exit 1
fi

if [ ! -z "$IQN" ]; then
  echo "IQN Received is -> $IQN"
fi

if [ ! -z "$LVMSnapChunkSize" ]; then
  echo "LVMSnapChunkSize Received is -> $LVMSnapChunkSize"
fi

# Check if ISCSI service is configured and enable CHAP.
ubuntuRelease=$(cat /etc/*-release | grep -i Ubuntu)

set -x
if [ ! -z "$ubuntuRelease" ]; then
    if [ ! -z "$IQN" ]; then
      sed -i '/InitiatorName=/c\InitiatorName='"$IQN"'' /etc/iscsi/initiatorname.iscsi
    fi
    . `dirname $0`/ubuntuchapconfigIscsi.sh
    . `dirname $0`/validateCHAP.sh
  else
    source `dirname $0`/configureIscsi.sh
    #Validate ISCSI Chap Authentication with SKY
    source `dirname $0`/setupCHAP.sh
    cp -r `dirname $0`/restore.sh /act/scripts/filesystem_restore.sh
fi

DisableMultipathCheck=true
grep -q '^DisableMultipathCheck.*' /act/config/connector.conf &&  sed -i 's/^DisableMultipathCheck.*/DisableMultipathCheck='"$DisableMultipathCheck"'/g' /act/config/connector.conf || echo -e 'DisableMultipathCheck='"$DisableMultipathCheck"'' >> /act/config/connector.conf

if [ ! -z "$LVMSnapChunkSize" ]; then
  grep -q '^LVMSnapChunkSize.*' /act/config/connector.conf &&  sed -i 's/^LVMSnapChunkSize.*/LVMSnapChunkSize='"$LVMSnapChunkSize"'/g' /act/config/connector.conf || echo -e 'LVMSnapChunkSize='"$LVMSnapChunkSize"'' >> /act/config/connector.conf
fi

exit
