#!/bin/sh
#
# OracleInBandASMDBClone.sh: shell script to automate cloning and bring up an InBand Oracle database using backup image copies
#
# usage: OracleInBandASMDBClone.sh 
set -x
nsid=$new_sid
oracleHome=$oracle_home
osuser=$oracle_user
pfileloc=$pfile_location
spfileloc=$spfile_location
fileloc=$file_location
nodelist=$nodelist
tstamp=$tstamp
BASEDIR=$(dirname $0)


#process new_sid, if longer than 8, trim it down to 8
nsidlen=`echo -n $nsid | wc -m`
if [ $nsidlen -gt 8 ]; then
newdb_name=`echo $nsid | cut -c1-8`
db_unique_name=$nsid
else
newdb_name=$nsid
fi

ACT_JOBNAME=`echo $ACT_JOBNAME | tr '[a-z]' '[A-Z]'`
grp=`id -gn $osuser`
pdbreadonlyconf=/act/touch/$ACT_JOBNAME/"$osid"_pdb_readonly_datafiles.conf

#--------------------------------------------------------
# Function to open database in read write mode
#---------------------------------------------------------
readwrite() {
  su -m $osuser -c "sqlplus -S / as sysdba " << EOF >> $openDBlog
  set head off;
  set feed off;
  set echo off;
  set newpage none;
  set trimspool on
  alter database open;
  exit;
EOF
}
#---------------------------------------------------------
# Function to Check DB Status 
#--------------------------------------------------------

db_status() {
mode="$1"
flag="$2"

cd $clonelogs
touch status.sql
chmod 777 status.sql
query="select count(*) from v\$database where open_mode = '$mode';"
echo "$query" > status.sql
opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
@status.sql
exit;
EOF1`

retval=$?
opencnt=`echo $opencnt | xargs`
if [ "$opencnt" -eq "0" ] && [ "$flag" = "reset" ]; then
   readwrite
   opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
   WHENEVER SQLERROR EXIT SQL.SQLCODE
   set head off;
   set feedback off
   set newpage none
   @status.sql
   exit;
EOF1`
retval=$?
fi
cleanup_file status.sql
opencnt=`echo $opencnt | xargs`

if [ "$opencnt" = "0" ]; then
 echo "Error: Database is failed to open in $mode"
 exit 1;
elif [ $retval -gt "0" ]; then
  echo "Error: Database is failed to open in $mode"
  exit 1;
fi
cd $BASEDIR
}

#--------------------------------------------------------
# Function to cleanup files
#--------------------------------------------------------

cleanup_file() {
  file_name=$1
  if [ -f $file_name ]; then
    rm -f $file_name
  fi
}

#---------------------------------------------------------
# Generate Node List 
#---------------------------------------------------------

function generate_asm_node_config {
   nodelist=$1
   user=$2
   rachostnames=$3
   #generate an asm_node_config file
   asmnodeconf="/act/act_scripts/inbandscripts/asmclone/asm_node_config.conf"
   rm $asmnodeconf  > /dev/null 2>&1
   touch $asmnodeconf
   OLDIFS=$IFS
   IFS=':'
   i=1
   if [ -z "$rachostnames" ]; then
      localip=`ifconfig -a| grep 'inet' | awk '{print $2}' | sed 's/addr://' | head -n1`
      echo "local ip is $localip"
      for n in ${nodelist}; do
         echo $n
         if [ "$n" = "$localip" ]; then
            fn=`su -m $user -c "hostname --fqdn"`
            echo "local fn is $fn"
         else
            su -m $user -c "ssh -o StrictHostKeyChecking=no -l$user $n exit"
            fn=`su -m $user -c "ssh $user@$n hostname --fqdn"`
            echo "remote fn is $fn"
         fi
         echo "ASM_NODE$i:$fn" >> $asmnodeconf
         i=$(($i + 1));
      done
   else
      for n in ${rachostnames}; do
         echo "ASM_NODE$i:$n" >> $asmnodeconf
         i=$(($i + 1));
      done
   fi
   IFS=$OLDIFS
   return 0
}

#---------------------------------------------------------
# Function to Create password file
#---------------------------------------------------------

create_password_file() {
  orapw1=$1
  set +x
  syspwd="Change_0n_install"
  if [ ! -z $ouser ] && [ "$ouser" == "sys" ] && [ ! -z $opass ]; then
    syspwd=$opass
  fi

  #source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh
  sqlver=`get_sql_major_version $ORACLE_HOME`
  set +x
  if [ "$sqlver" -gt "11" ]; then
     su -m $osuser -c "$ORACLE_HOME/bin/orapwd file=$orapw1 password=\"$syspwd\" format=12 entries=5"
  else
     su -m $osuser -c "$ORACLE_HOME/bin/orapwd file=$orapw1 password=\"$syspwd\" format=12"
  fi
  set -x
} 

#---------------------------------------------------------
# Function to drop UNDO tablespace
#---------------------------------------------------------

drop_undo_tablespace() {
touch $undofilelist
chmod 777 $undofilelist
su -m $osuser -c "sqlplus -S / as sysdba " << EOF >> $undofilelist
set head off;
set feed off;
set echo off;
set newpage none;
set trimspool on
select distinct 'drop tablespace '||tablespace_name ||' including contents and datafiles;' from dba_undo_extents where tablespace_name not like 'UNDOTBS_CL%';
exit;
EOF
echo -e "exit;" >> $undofilelist
su -m $osuser -c "sqlplus -S / as sysdba @$undofilelist" >> $openDBlog
cleanup_file $undofilelist
}

#---------------------------------------------------------
# Function to create setdb files
#---------------------------------------------------------

create_setdb() {
  nsid=$1
  nsid1=$2
  if [ -z "$nsid1" ]; then
    nsid1=$nsid
  fi
  setdb="setdb_"$nsid
  echo -e "# Modify the following variables based on the environment" > $setdb
  echo -e "export ORACLE_SID=$nsid1" >> $setdb
  echo -e "export ORACLE_HOME=$ORACLE_HOME" >> $setdb
  echo -e "export PATH='$ORACLE_HOME'/bin:'$PATH'" >> $setdb
  echo -e "#END OF FILE" >> $setdb
  grp=`id -gn $osuser`
  chown $osuser:$grp $setdb
  chmod 755 $setdb
}

#---------------------------------------------------------
# Function to create oratab entries
#---------------------------------------------------------

create_oratab() {
  nsid=$1
  nsid1=$2
  if [ -f /etc/oratab ]; then
     if [ -z "$nsid1" ]; then
        otab=$nsid":"$ORACLE_HOME":Y"
        grep -w ^"$nsid" /etc/oratab >/dev/null 2>&1 || echo -e "$otab" >> /etc/oratab
      else
        otab=$nsid":"$ORACLE_HOME":N"
        otab1=$nsid1":"$ORACLE_HOME":Y"
        grep -w ^"$nsid" /etc/oratab >/dev/null 2>&1 || echo -e "$otab" >> /etc/oratab
        grep -w ^"$nsid1" /etc/oratab >/dev/null 2>&1 || echo -e "$otab1" >> /etc/oratab
     fi
  fi
}


#---------------------------------------------------------
# Function to create TEMP tablespaces
#---------------------------------------------------------

create_temp_tbs() {

ACT_JOBNAME=`echo $ACT_JOBNAME | tr '[a-z]' '[A-Z]'`
if [ -f /act/touch/$ACT_JOBNAME/"$osid"_temp_tablespace_catalog.conf ]; then
   tempconfig=`cat /act/touch/$ACT_JOBNAME/"$osid"_temp_tablespace_catalog.conf`
   matchtempconfig=matchtempconfig_"$nsid".sql
   cleanup_file $matchtempconfig

   for x in $(echo $tempconfig | tr ',' ' ')
   do
      container_name=`echo $x | awk -F":" '{print $1}'`
      file_count=`echo $x | awk -F":" '{print $3}'`
      temptbs=`echo $x | awk -F":" '{print $2}'`

      if [ "$container_name" = 'CDB$ROOT' ]; then
         currentcount=`su -m $osuser -c "sqlplus -S / as sysdba " << EOF1
         set head off;
         set feed off;
         set echo off;
         set linesize 4000;
         set newpage none;
         select count(1) from dba_temp_files where tablespace_name='\$temptbs';
         exit;
EOF1`
         echo "alter session set container=CDB\$ROOT;" >> $matchtempconfig
         fcnt=$(($currentcount+1))
         while [ "$fcnt" -le "$file_count" ]
         do
           echo "alter tablespace $temptbs add tempfile '$new_db_create_file_dest' size 5g autoextend on;" >> $matchtempconfig
           fcnt=$(($fcnt+1))
         done
      else
          echo "alter session set container=$container_name;" >> $matchtempconfig
          fcnt=1
          while [ "$fcnt" -le "$file_count" ]
          do
            echo "alter tablespace $temptbs add tempfile '$new_db_create_file_dest' size 5g autoextend on;" >> $matchtempconfig
            fcnt=$(($fcnt+1))
          done
      fi
   done
   echo "exit;" >> $matchtempconfig
   chmod 755 $matchtempconfig

   su -m $osuser -c "$sqlplus / as sysdba @$matchtempconfig"
   cleanup_file $matchtempconfig
else
   tmptbs=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
   WHENEVER SQLERROR EXIT SQL.SQLCODE
   set head off;
   set feedback off
   set newpage none
   select count(1) from dba_tablespaces where tablespace_name='TEMP';
   exit;
EOF1`

   if [ "$tmptbs" -gt 0 ]; then
      su -m $osuser -c "sqlplus -S / as sysdba " << EOF
      set head off;
      set feed off;
      set echo off;
      alter tablespace TEMP add tempfile '$new_db_create_file_dest' size 5g autoextend on;
      exit;
EOF
   else
      su -m $osuser -c "sqlplus -S / as sysdba " << EOF
      set head off;
      set feed off;
      set echo off;
      create temporary tablespace TEMP add tempfile '$new_db_create_file_dest' size 5g autoextend on;
      alter database default temporary tablespace TEMP;
      exit;
EOF
   fi
fi
}
#---------------------------------------------------------
# Function to change DBID
#---------------------------------------------------------

change_dbid() {
 if [ ! -f "$dbfsPath" ]; then
     roTBS='NONE'
  else
     roTBS=`cat $dbfsPath`
  fi

  roCTBS=''
  for i in $(echo $roTBS | tr "," "\n")
  do
      echo "$i"
      if [ -z "$roCTBS" ]; then
         roCTBS="'''"${i}"'''"
      else
         roCTBS=${roCTBS}"\n'''"${i}"'''"
      fi
  done

  echo "$roCTBS"

  ctbs_input=''
  echo -e "$roCTBS" > $clonelogs/roCTBS_output.txt
  ctbs_input=`cat $clonelogs/roCTBS_output.txt`
  echo "$ctbs_input"

  if [ -e "$runnidsql" ]; then
    rm -f $runnidsql
  fi

  while read line
  do
      set -- $line
      readtabname=$1
          if [ "x$readtabname" != "x" ]; then
            echo -e "@nidROTBS.sql $readtabname ONLINE" >> $runnidsql
            echo -e "@nidROTBS.sql $readtabname 'READ WRITE'" >> $runnidsql
          fi
  done < $clonelogs/roCTBS_output.txt
   if [ -f $pdbreadonlyconf ]; then
     cat $pdbreadonlyconf | awk -F":" '{print $2,$3}'|sort |uniq | while read line
     do
       if [ ! -z "$line" ]; then
          tbsname=`echo $line | awk '{print $2}'`
          containername=`echo $line | awk '{print $1}'`
          echo -e "alter session set container=$containername;" >> $runnidsql
          echo -e "@nidROTBS.sql '''"$tbsname"''' ONLINE" >> $runnidsql
          echo -e "@nidROTBS.sql '''"$tbsname"''' 'READ WRITE'" >> $runnidsql
          echo -e "alter session set container=CDB\$ROOT;" >> $runnidsql
       fi
     done
  fi
  chmod 755 $runnidsql

su -m $osuser -c "$sqlplus / as sysdba " << EOF
@$runnidsql
shutdown immediate;
startup mount;
exit;
EOF

  su -m $osuser -c "$ORACLE_HOME/bin/nid target=/ LOGFILE=$nidlog "

  retval=$?

  if [ $retval -gt "0" ]; then
  nidfail=yes
  su -m $osuser -c "$ORACLE_HOME/bin/nid target=/ LOGFILE=$nidlog revert=yes"
  fi

  if [ ! -z "$DB_TZ" ]; then
  echo " ********** setting TZ:$DB_TZ  ***************"
  TZ=$DB_TZ
  export TZ
  fi

  if [ ! -f "$runnidsql" ]; then
   rm -f $runnidsql
  fi

  while read line
  do
      set -- $line
      readtabname=$1
          if [ "x$readtabname" != "x" ]; then
          echo -e "@nidROTBS.sql $readtabname 'READ ONLY'" >> $runnidsql
          fi
  done < $clonelogs/roCTBS_output.txt
  if [ -f $pdbreadonlyconf ]; then
     cat $pdbreadonlyconf | awk -F":" '{print $2,$3}'|sort |uniq | while read line
     do
       if [ ! -z "$line" ]; then
          tbsname=`echo $line | awk '{print $2}'`
          containername=`echo $line | awk '{print $1}'`
          echo -e "alter session set container=$containername;" >> $runnidsql
          echo -e "@nidROTBS.sql '''"$tbsname"''' 'READ ONLY'" >> $runnidsql
       fi
     done
  fi
  chmod 755 $runnidsql

  if [ "$nidfail" = "yes" ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
startup mount;
alter database open;
@$runnidsql
exit;
EOF
else
su -m $osuser -c "$sqlplus / as sysdba " << EOF
startup mount;
alter database open resetlogs;
@$runnidsql
exit;
EOF
  fi

cleanup_file $clonelogs/roCTBS_output.txt
}

#---------------------------------------------------------
# Function to generate new controlfile to trace
#---------------------------------------------------------
create_controlfile_trace() {
filelocation=$1
newsid=$2
newcfile=$filelocation/$newsid"_controlfile.trace"
echo "Generating new control file to trace as $newcfile"
if [ -e $newcfile ]; then
  rm -f $newcfile
fi

su -m $osuser -c "sqlplus -S / as sysdba " << EOF
set head off;
set feed off;
set echo off;
alter database backup controlfile to trace as '$newcfile';
exit;
EOF
}

#---------------------------------------------------------
# Function to get SQL Version
#---------------------------------------------------------
get_sql_major_version(){
ORACLE_HOME=$1
export ORACLE_HOME
LD_LIBRARY_PATH=$ORACLE_HOME/lib;
export LD_LIBRARY_PATH

version=`$ORACLE_HOME/bin/sqlplus -version | grep Release  | awk '{print $3}'| awk -F'.' '{print $1}'`
echo $version
exit 0

}


#---------------------------------------------------------
# Function to get grid home
#---------------------------------------------------------
function get_grid_home {
    oratab=$1
    if [ -f "$oratab" ]; then
       gridhome=`grep -v "^[#]" $oratab|grep -i "+ASM" | head -n1 | cut -f2 -d":"`
    fi

    if [ ! -z "$ASM_HOME" ]; then
        gridhome="$ASM_HOME"
    fi

    if [ -z "$gridhome" ]; then
       lsnr=`ps -ef | grep -i tnslsnr | grep -v grep | head -n1 | awk '{print $8}'`
       pname=$(basename $lsnr)
       if [ "$pname" != "tnslsnr" ]; then
          lsnr=`ps -ef | grep -i tnslsnr | grep -v grep | head -n1 | awk '{print $9}'`
       fi
       gridhome=`echo $lsnr | rev | cut -d'/' -f 3-10 | rev`
    fi

    if [ -z "$gridhome" ]; then
       exit 1
    fi
    echo $gridhome
    exit 0
}

#---------------------------------------------------------
# Function to append nocp with asmcmd
#---------------------------------------------------------
get_asmcmd_nocp() {
  asmcmd=$1
  asmcmdVer=`$asmcmd -V | awk -F"asmcmd version " '{print $2}' | cut -d. -f1 `
  if [ $asmcmdVer -ge 12 ]; then
    #asmcmd="$asmcmd --nocp"
    ASMCMD_NOCP="--nocp"
  fi
}

#---------------------------------------------------------
# Function to cleanup archive logs
#---------------------------------------------------------
cleanup_archivelog() {
  osid=$1
  FRADG=$2
  oratab=/etc/oratab
  guser=`ps -eo user:20,pid,stime,time,args | grep -i asm_pmon_+asm | grep -vw grep | awk '{print $1}'`
  var="+ASM"
  for ASMSID in $var; do
    ASMS=`ps -ef | grep asm_pmon | grep -i +ASM | awk '{print $NF}' | awk -F'_' '{print $NF}'`
    echo "asmsid is: $ASMS"
    ASMHOME=`grep -v "^[#]" /etc/oratab | grep -i "$ASMS:"  | head -n1 | cut -f2 -d":"`
    echo "asmhome is: $ASMHOME"
  done

  if [ -z "$ASMHOME" ]; then
     ASMHOME=`get_grid_home $oratab`
  fi

  export ORACLE_BASE=$ASM_BASE
  export ORACLE_HOME=$ASMHOME
  export LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
  export PATH=$ASMHOME/bin:$PATH
  export ORACLE_SID="$ASMS"
  asmcmd=$ASMHOME/bin/asmcmd
  get_asmcmd_nocp $asmcmd

  check_file_exists=`su -m $guser -c "$asmcmd $ASMCMD_NOCP ls -l $FRADG/$osid/archivelog | wc -l"`
  
  if [ "$check_file_exists" -gt 0 ]; then
     su -m $guser -c "$asmcmd $ASMCMD_NOCP rm -rf $FRADG/$osid/archivelog/*"
     retval=$?
     if [ "$retval" -gt 0 ]; then
        echo "WARNINGMSG: Failed to remove archivelog folder"
     fi
  fi
}

#---------------------------------------------------------
# Function to handle PDB Read Only datafiles
#---------------------------------------------------------

pdb_readonly_datafiles()
{
  pdbreadonlyrename=$clonelogs/pdbdatafilerename.sql
  pdbmissingfiles=$clonelogs/pdbmissingfiles.txt
  cleanup_file $pdbreadonlyrename
  cleanup_file $pdbmissingfiles

  su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $pdbmissingfiles
  set head off;
  set feed off;
  set echo off;
  select 'MISSING-'||FILE#||':'||name from v\$datafile where name like '%MISSING%';
  exit;
EOF
  if [ -f $pdbreadonlyconf ]; then
   cat $pdbreadonlyconf | while read line
      do
        dfname=`echo $line | rev | awk -F":" '{print $1}'| rev`
        dgname=`echo $line | awk -F":" '{print $5}' | awk -F"/" '{print $1}'`
        dfnumber=`echo $line | awk -F":" '{print $4}'`
        dfname=`echo $dfname | sed "s/$dgname/$new_db_create_file_dest/g"`
        missingdfname=`cat $pdbmissingfiles | grep "MISSING-$dfnumber" | awk -F":" '{print $2}' |rev |awk -F"/" '{print $1}' | rev`
        echo -e "alter database rename file '$missingdfname' to '$dfname';" >> $pdbreadonlyrename
      done
  fi

  if [ -f $pdbreadonlyrename ]; then
     echo "exit;" >>$pdbreadonlyrename
     chmod 755 $pdbreadonlyrename
     su -m $osuser -c "sqlplus / as sysdba @$pdbreadonlyrename " >> $openDBlog
  fi
  cleanup_file $pdbreadonlyrename
  cleanup_file $pdbmissingfiles
}



#----------------------------------------------------
# Main Processing 
#--------------------------------------------------

if [ ! -z "$israc" ] && [ "$israc" = "true" ]; then
 standalone=false
fi

if [ "$standalone" = "false" ]; then
   generate_asm_node_config $nodelist $osuser $rachostnames
fi

clonelogs=/act/act_scripts/inbandscripts/asmclonelog

if [ ! -d "$clonelogs" ]; then
   mkdir -p $clonelogs
   chmod 777 $clonelogs
fi

cd $BASEDIR
# get the backup parameter file location
# get the production sid name from mount
pname=`basename $pfileloc`
echo $pname
osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
echo "***************"
echo $osid
dbrecover="$clonelogs/dbrecover_"$nsid".txt"
openDatabase="$clonelogs/openDatabase_"$nsid".sql"
openDBlog="$clonelogs/openDatabase_"$nsid".txt"
cstrfile="$clonelogs/catalogarchlogs_"$nsid".txt"
undofilelist="$clonelogs/undofilelist_"$nsid".sql"
runnidsql="$clonelogs/runnidrotbs"_$nsid".sql"
nidlog="$clonelogs/nid_"$nsid".log"

cleanup_file $dbrecover
cleanup_file $openDatabase
cleanup_file $openDBlog
cleanup_file $cstrfile
cleanup_file $runnidsql
cleanup_file $nidlog

touch $dbrecover $openDatabase $openDBlog $cstrfile $nidlog
chmod 777 $dbrecover $openDatabase $openDBlog $cstrfile $nidlog

if [ -z "$osid" ]; then
   echo " ******* failed to get the old sid ********" 
   exit 1
fi
 

origfile=$pfileloc
cp $origfile $fileloc/$osid"__cpfile.ora"
chmod 775 $fileloc/$osid"__cpfile.ora"
orafile=$fileloc/$osid"__cpfile.ora"
echo $orafile

# replace SID with new SID
sed -i "s/'$osid'/'$nsid'/g" $orafile

export ORACLE_SID=$nsid
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export ORA_NLS10=$ORACLE_HOME/nls/data
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
sqlplus=$ORACLE_HOME/bin/sqlplus

#make required directory structures
if [ -z "$diagnostic_dest" ]; then
  su -m $osuser -c "mkdir -p $ORACLE_HOME/../../admin/$nsid/adump"
  su -m $osuser -c "mkdir -p $ORACLE_HOME/../../diag"
else
  su -m $osuser -c "mkdir -p $diagnostic_dest/$nsid"
  su -m $osuser -c "mkdir -p $diagnostic_dest/$nsid/admin/adump"
fi


controlfile="*.control_files='$new_control_file_dest'"
dbcreatefile="*.db_create_file_dest='$new_db_create_file_dest'"
dbrecoveryfile="*.db_recovery_file_dest='$new_db_recovery_file_dest'"
dbname="*.db_name='$newdb_name'"
dbuniquename="*.db_unique_name='$nsid'"
jqprocess="*.job_queue_processes=0"
clustdb="*.cluster_database=false"

if [ -z "$totalmemory" ]; then
    #sgatarget="*.sga_target=800m"
    #pgatarget="*.pga_aggregate_target=400m"
    echo " memory size is not defined "
    echo "source memory setup will be used"
else
    if [ ! -z "$totalmemory" ] && [ -z "$sgapct" ]; then
        echo " SGA is not defined "
        echo "memory_target setup will be used"
        sed -i '/sga_max_size/d' $orafile
        sed -i '/sga_target/d' $orafile
        sed -i '/pga_aggregate_target/d' $orafile
        sed -i '/memory_target/d' $orafile
        sed -i '/shared_pool_size/d' $orafile
        sed -i '/streams_pool_size/d' $orafile
        sed -i '/memory_max_target/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        sed -i '/k_cache_size/d' $orafile
        totalmemoryM=$totalmemory"m"
        mtarget="*.memory_target=$totalmemoryM"
        echo -e "$mtarget" >> $orafile
    else
        echo " total memory and SGA is defined "
        echo "sga and pga setup will be used"
        sed -i '/sga_max_size/d' $orafile
        sed -i '/sga_target/d' $orafile
        sed -i '/pga_aggregate_target/d' $orafile
        sed -i '/memory_target/d' $orafile
        sed -i '/memory_max_target/d' $orafile
        sed -i '/shared_pool_size/d' $orafile
        sed -i '/streams_pool_size/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        sgc=$(($totalmemory*$sgapct/100))
        sgam=$sgc"m"
        sgatarget="*.sga_target=$sgam"
        pgc=$(($totalmemory - $sgc ))
        pgam=$pgc"m"
        pgatarget="*.pga_aggregate_target=$pgam"
        echo -e "$sgatarget" >> $orafile
        echo -e "$pgatarget" >> $orafile
        lps=$((($sgc*10)/100))
        if [ "$lps" -gt "200" ]; then
           lapsm=$lps"m"
        else
           lapsm=200m
        fi

        largepool="*.large_pool_size=$lapsm"
        echo -e "$largepool" >> $orafile

        if [ ! -z "$shared_pool_size" ]; then
            sps=$shared_pool_size"m"
            sharedpoolsize="*.shared_pool_size=$sps"
            echo -e "$sharedpoolsize" >> $orafile
        fi

        db_nk_count=`grep k_cache_size $orafile | wc -l`
        if [ "$db_nk_count" -gt 0 ]; then
           if [ ! -z "$db_cache_size" ]; then
              nk_memory=$(($db_cache_size*60/100))
              db_cache_memory=$(($db_cache_size-nk_memory))
              db_cache_size="$db_cache_memory"m
              nk_memory=$(($nk_memory/$db_nk_count))
              nk_memory="$nk_memory"m
           else
              echo  "ERRORMSG: db_cache_size is not set!"
              exit 1
           fi
           for line in $(grep k_cache_size $orafile | awk '{print $1}' |sed 's/*.//g')
           do
             nk_parameter=`echo $line | awk -F"=" '{print $1}'`
             nk_parameter_size="$nk_parameter"="$nk_memory"
             nk_remove_param=$nk_parameter
             sed -i "/$nk_remove_param/d" $orafile
             echo -e "$nk_parameter_size" >> $orafile
           done
           dbcachesize="*.db_cache_size=$db_cache_size"
           echo -e "$dbcachesize" >> $orafile
        else
           if [ ! -z "$db_cache_size" ]; then
              dcs=$db_cache_size"m"
              dbcachesize="*.db_cache_size=$dcs"
              echo -e "$dbcachesize" >> $orafile
           fi
        fi
    fi
 fi

sed -i '/oracle_base/d' $orafile
sed -i '/inmemory_/d' $orafile
sed -i '/use_large_pages/d' $orafile
sed -i '/control_files/d' $orafile
sed -i '/audit_file_dest/d' $orafile
sed -i '/db_create_file_dest/d' $orafile
sed -i '/db_recovery_file_dest/d' $orafile
sed -i '/diagnostic_dest/d' $orafile
sed -i '/log_archive_dest_1/d' $orafile
sed -i '/log_archive_dest/d' $orafile
sed -i '/cluster_database/d' $orafile
sed -i '/job_queue_processes/d' $orafile
#sed -i '/remote_listener/d' $orafile
sed -i '/local_listener/d' $orafile
sed -i '/user_dump_dest/d' $orafile
sed -i '/core_dump_dest/d' $orafile
sed -i '/db_name/d' $orafile
sed -i '/dispatchers/d' $orafile
sed -i '/SPFILE/d' $orafile

sed -i '/thread/d' $orafile
sed -i '/THREAD/d' $orafile
sed -i '/instance_number/d' $orafile
sed -i '/undo_tablespace/d' $orafile
sed -i '/__/d' $orafile
sed -i '/service_names/d' $orafile
sed -i '/db_create_online_log_dest/d' $orafile
sed -i '/SPFILE/d' $orafile

sed -i '/dg_broker_config_file/d' $orafile
sed -i '/dg_broker_start/d' $orafile
sed -i '/fal_client/d' $orafile
sed -i '/fal_server/d' $orafile
sed -i '/log_archive_config/d' $orafile
sed -i '/log_archive_trace/d' $orafile
sed -i '/log_archive_min_succeed_dest/d' $orafile
sed -i '/log_file_name_convert/d' $orafile
sed -i '/standby_file_management/d' $orafile
sed -i '/db_file_name_convert/d' $orafile
sed -i '/archive_lag_target/d' $orafile
sed -i '/log_archive_trace/d' $orafile

sed -i '/cluster_interconnects/d' $orafile
sed -i '/_diag_adr_trace_dest/d' $orafile

sed -i '/cluster_database_instances/d' $orafile
sed -i '/instance_name/d' $orafile
sed -i '/db_unique_name/d' $orafile
sed -i '/standby_archive_dest/d' $orafile
sed -i '/background_dump_dest/d' $orafile
sed -i '/_fix_control/d' $orafile
sed -i '/event/d' $orafile

sed -i '/disable_streams_pool_auto_tuning/d' $orafile
sed -i '/enable_goldengate_replication/d' $orafile
sed -i '/_adg_parselock_timeout/d' $orafile
sed -i '/parallel_min_servers/d' $orafile
sed -i '/parallel_max_servers/d' $orafile
sed -i '/parallel_servers_target/d' $orafile
sed -i '/use_large_pages/d' $orafile
sed -i '/=/!d' $orafile

chmod 755 $orafile

echo -e "$controlfile" >> $orafile
echo -e "$dbname" >> $orafile
echo -e "$dbuniquename" >> $orafile
#echo -e "$auditfile" >> $orafile
echo -e "$dbcreatefile" >> $orafile
if [ ! -z "$dbrecoveryfile" ]; then
 echo -e "$dbrecoveryfile" >> $orafile
fi

sed -i '/db_recovery_file_dest_size/d' $orafile
if [ ! -z "$db_recovery_file_dest_size" ]; then
   drfds=$db_recovery_file_dest_size"m"
   dbrecoveryfilesize="*.db_recovery_file_dest_size=$drfds"
else
   dbrecoveryfilesize="*.db_recovery_file_dest_size=50g"
fi


echo -e "$dbrecoveryfilesize" >> $orafile
echo -e "$clustdb" >> $orafile
echo -e "$jqprocess" >> $orafile

##### For RAC - Modify this


if [ "$standalone" = "false" ]; then
   ncount=`grep "ASM_" asm_node_config.conf | cut -f2 -d":" | wc -l`
   var=`grep "ASM_" $BASEDIR/asm_node_config.conf | cut -f2 -d":"`
   m=0


   if [ -f /act/touch/$ACT_JOBNAME/"$osid"_undo_catalog.conf ]; then
       undoconfig=`cat /act/touch/$ACT_JOBNAME/"$osid"_undo_catalog.conf`
       undotbscount=`echo $undoconfig|tr ',' '\n' | grep  '^CDB$ROOT' |wc -l`
       m=0
       for x in $(echo $undoconfig | tr ',' ' ' ); do
         container_name=`echo $x | awk -F":" '{print $1}'`
         undotbsname=`echo $x | awk -F":" '{print $2}'`
         if [ "$container_name" = 'CDB$ROOT' ]; then
            m=$(($m + 1));
            norasid1=$nsid$m
            cthread="$norasid1.thread=$m"
            cinst="$norasid1.instance_number=$m"
            cundo="$norasid1.undo_tablespace='$undotbsname'"
            echo -e "$cthread" >> $orafile
            echo -e "$cinst" >> $orafile
            echo -e "$cundo" >> $orafile
         fi
         if [ "$m" -eq "$ncount" ]; then
            break;
         fi
       done
       if [ "$m" -lt "$ncount" ]; then
         while [ "$m" -lt "$ncount" ]
         do
           m=$(($m + 1));
           norasid1=$nsid$m
           cthread="$norasid1.thread=$m"
           cinst="$norasid1.instance_number=$m"
           cundo="$norasid1.undo_tablespace='UNDOTBS$m'"
           echo -e "$cthread" >> $orafile
           echo -e "$cinst" >> $orafile
           echo -e "$cundo" >> $orafile
           if [ -z "$CREATE_UNDOTBSLIST" ]; then
              CREATE_UNDOTBSLIST=UNDOTBS"$m"
           else
               CREATE_UNDOTBSLIST=$CREATE_UNDOTBSLIST,UNDOTBS"$m"
           fi
         done
       fi
        UNDOCONFIG_EXISTS=YES
   else
      for x in $var; do
        m=$(($m + 1));
        norasid1=$nsid$m
        cthread="$norasid1.thread=$m"
        cinst="$norasid1.instance_number=$m"
        cundo="$norasid1.undo_tablespace='UNDOTBS_CL$m'"
        echo -e "$cthread" >> $orafile
        echo -e "$cinst" >> $orafile
        echo -e "$cundo" >> $orafile
      done
   fi
fi

chmod 755 $orafile

if [ "$standalone" = "false" ]; then
 initf1=$ORACLE_HOME/dbs/"init"$nsid"1.ora"
fi

initf=$ORACLE_HOME/dbs/"init"$nsid".ora"

spf=$spfile_location
su -m $osuser -c "sqlplus / as sysdba @/act/act_scripts/inbandscripts/asmclone/createPfile.sql $orafile $spf "

if [ "$standalone" = "false" ]; then
su -m $osuser -c "srvctl add database -d $nsid -o $ORACLE_HOME -p '$spf'"
fi

if [ "$standalone" = "false" ]; then
 su -m $osuser -c "echo 'SPFILE=$spf' > $initf1"
fi

#process database name in the controlfile generated by connector
if [ $nsidlen -gt 8 ]; then
sed -i '/CREATE CONTROLFILE SET DATABASE/Id' $controlfile_location
sed -i "1s/^/CREATE CONTROLFILE SET DATABASE '$newdb_name' RESETLOGS ARCHIVELOG\n/" $controlfile_location
fi

######### Added PDB READ ONLY TBS recovery ################

if [ -f $pdbreadonlyconf ]; then
   cat $pdbreadonlyconf | while read line
   do
     dfname=`echo $line | rev | awk -F"/" '{print $1}'| rev`
     sed -i "/$dfname/d" $controlfile_location
   done
fi

su -m $osuser -c "echo 'SPFILE=$spf' > $initf"
su -m $osuser -c "sqlplus / as sysdba @/act/act_scripts/inbandscripts/asmclone/mountDatabase.sql $controlfile_location"

#Check Database Status
db_status "MOUNTED"


########convert tstamp using db timezone #########

echo "********** convert the tstamp to db timezone DB_TZ:$DB_TZ if set, the tstamp is in UTC time  ***************"
if [ ! -z "$DB_TZ" ]; then
     if [ ! -z "$tstamp" ]; then
        tstamp=`su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -S / as sysdba @/act/act_scripts/TZconvertApp.sql $tstamp $DB_TZ"`
        retval=$?
     
        if [ $retval -gt "0" ]; then
           echo "********** error converting the db timezone DB_TZ:$DB_TZ result: $tstamp_tmp  ***************"
           exit 1
        fi
fi
echo "tstamp after converting to DB timezone $tstamp"
fi

########Allocate RMAN Channesl ####################

if [ -z "$rmanchannels" ]; then
    rmanchannels=4;
fi

allocatech=
releasech=
i=0
set +x
while [ $i -lt "$rmanchannels" ]; do
      if [ -z "$allocatech" ]; then
         allocatech="allocate channel d$i device type disk;"
         releasech="release channel d$i;"
      else
         allocatech=$allocatech"allocate channel d$i device type disk;"
         releasech=$releasech"release channel d$i;"
      fi
      i=`expr $i + 1`
      set -x
done
###############  Check for 12c PDB ##########################

vnum=0
vnum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from v\$instance where version not like '10%' and version not like '11%';
exit;
EOF1`
echo "********************* vnum=$vnum ***********"
pdbcnt=0
if [ "$vnum" -gt "0" ]; then
  pdbcnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
  set head off;
  set feedback off
  select count(*) from v\$pdbs where name like 'PDB%SEED';
  exit;
EOF1`
fi
export vnum
export pdbcnt

if [ "$pdbcnt" -gt "0" ]; then
  if [ ! -z "$roTBS" ]; then
    recoverCMD="skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2,$roTBS"
  else
    recoverCMD='skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2'
  fi
else
  if [ ! -z "$roTBS" ]; then
     recoverCMD="skip tablespace $roTBS"
  else
    recoverCMD=''
  fi
fi


############### Prepare Rman Command with Catalog commands ###############
if [ ! -z "$imageLogOffset" ]; then
   imageLogOffset=$((imageLogOffset + 1))
fi

if [ ! -z "$tstamp" ]; then
   from_tstampsql=$clonelogs/from_tstamp_$nsid.sql
   touch $from_tstampsql
   echo "set head off;" >$from_tstampsql
   echo "set feedback off" >> $from_tstampsql
   echo "set newpage none" >> $from_tstampsql
   echo "select to_char(fromtime,'yyyymmddhh24mi') from (select to_date('$dbImageBeginTime','yyyy-mm-dd hh24:mi:ss')-interval '5' minute as fromtime  from dual);" >>$from_tstampsql
   echo "exit;" >>$from_tstampsql
   chown $osuser:$grp $from_tstampsql
   from_tstamp=`su -m $osuser -c "sqlplus -S / as sysdba @$from_tstampsql"`
   rm -f $from_tstampsql
   touch -t $from_tstamp /act/act_scripts/inbandscripts/asmclonelog/fromtime_$nsid

   until_tstampsql=$clonelogs/until_tstamp_$nsid.sql
   touch /act/act_scripts/inbandscripts/asmclonelog/until_tstamp_$nsid.sql
   echo "set head off;" > $until_tstampsql
   echo "set feedback off" >> $until_tstampsql
   echo "select to_char(to_date('$tstamp','yyyymmddhh24miss')+1,'yyyymmddhh24mi') from dual;" >> $until_tstampsql
   echo "exit;" >> $until_tstampsql
   chown $osuser:$grp $until_tstampsql
   until_tstamp=`su -m $osuser -c "sqlplus -S / as sysdba @$until_tstampsql"`
   rm -f $until_tstampsql
   touch -t $until_tstamp /act/act_scripts/inbandscripts/asmclonelog/untiltime_$nsid
fi

if [ ! -z "$logmount" ]; then
dirs=`ls -d $logmount*`
cstr=""
     for d in ${dirs}; do
         archmntpt=$d"/archivelog/"
         str="-iname $osid"_\*
         pstr="-iname $old_psid"_\*
         chmod 777 $archmntpt
         if [ -f /act/act_scripts/inbandscripts/asmclonelog/untiltime_$nsid ] && [ ! -z "$until_tstamp" ] && [ /act/act_scripts/inbandscripts/asmclonelog/fromtime_$nsid ]; then
            find $archmntpt $str -newer /act/act_scripts/inbandscripts/asmclonelog/fromtime_$nsid ! -newer /act/act_scripts/inbandscripts/asmclonelog/untiltime_$nsid -type f >> $cstrfile
            find $archmntpt $pstr -newer /act/act_scripts/inbandscripts/asmclonelog/fromtime_$nsid ! -newer /act/act_scripts/inbandscripts/asmclonelog/untiltime_$nsid -type f >> $cstrfile
         else
             find $archmntpt $str -mtime -"$imageLogOffset" -type f >> $cstrfile
             find $archmntpt $pstr -mtime -"$imageLogOffset" -type f >> $cstrfile
         fi
     done
cleanup_file /act/act_scripts/inbandscripts/asmclonelog/fromtime_$nsid
cleanup_file /act/act_scripts/inbandscripts/asmclonelog/untiltime_$nsid

sed -i -e "s/^/catalog BACKUPPIECE '/" $cstrfile
sed -i "s/$/' ;/" $cstrfile
cstr=`cat $cstrfile`
fi

if [ ! -z $tstamp ]; then
rmancmd="run
        {
        $allocatech
        $cstr
        recover database until time \"to_date('$tstamp','yyyymmddhh24miss')\" $recoverCMD;
        $releasech
        }"
else
rmancmd="run
        {
        $allocatech
        $cstr
        recover database $recoverCMD;
        $releasech
        }"
fi

##############  Recover Database  ##################
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log $dbrecover append " << EOF
alter database end backup;
$rmancmd
exit;
EOF

if [ "$pdbcnt" -gt "0" ]; then
   su -m $osuser -c "$sqlplus / as sysdba @recoverPDBSeed.sql" >> $openDBlog
fi
############## Open database with Resetlogs ##################

su -m $osuser -c "$ORACLE_HOME/bin/rman target / log $dbrecover append " << EOF
run {
shutdown immediate;
startup mount;
alter database open resetlogs;
}
exit;
EOF

db_status "READ WRITE" reset

############## Open PDBs to create TEMP tablespace ##############

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log $dbrecover append " << 'EOF'
run {
ALTER pluggable database all open;
alter pluggable database all save state;
}
exit;
EOF
fi

db_status "READ WRITE"

#### Process PDB read only datafiles #####
pdb_readonly_datafiles

######################Standalone UNDO and Password file #############

if [ "$standalone" = "true" ]; then
   if [ -f /act/touch/$ACT_JOBNAME/"$osid"_undo_catalog.conf ]; then
      undoconfig=/act/touch/$ACT_JOBNAME/"$osid"_undo_catalog.conf
      undotbsname=`cat $undoconfig | awk -F":" '{print $2}'`
      echo -e "alter system set undo_tablespace=$undotbsname;" >> $openDatabase
   else
       echo -e "create undo tablespace UNDOTBS_CL datafile '$new_db_create_file_dest/$osid/datafile/undotbs_cl.dbf' size 500m autoextend on;" >> $openDatabase
       echo -e "alter system set undo_tablespace=UNDOTBS_CL scope=both;" >> $openDatabase
   fi
   echo -e "shutdown immediate;" >> $openDatabase
   echo -e "startup;" >> $openDatabase
   echo -e "exit" >> $openDatabase
   chmod 755 $openDatabase
  
   su -m $osuser -c "sqlplus / as sysdba @$openDatabase " >> $openDBlog
   db_status "READ WRITE" 
   #drop_undo_tablespace  ### Drop Undo Tablespaces
   nsid1=$nsid
   orapw=$ORACLE_HOME"/dbs/orapw"$nsid1

   if [ -e $orapw ]; then
     rm -f $orapw
   fi
   create_password_file $orapw
fi


############### Check the DB Status #################

db_status "READ WRITE"

############### Create temporary tablespace ##########

create_temp_tbs

################ Change DBID #####################

if [ "$nonid" != "true" ]; then
  change_dbid
fi

db_status "READ WRITE"

#set nsid again, it got lost after change_dbid
nsid=$ORACLE_SID

if [ "$standalone" = "true" ]; then
  create_setdb $nsid
  if [ "$nooratabupdate" != "true" ]; then
     create_oratab $nsid
  fi
fi

################ Handle RAC redo and UNDO ##############
if [ "$standalone" = "false" ]; then
dthcnt=`su -m $osuser -c "sqlplus -S / as sysdba " << EOF
set head off;
select max(thread#) from v\\$log;
exit;
EOF`
#echo "th count $dthcnt"
dgpcnt=`su -m $osuser -c "sqlplus -S / as sysdba " << EOF
set head off;
select max(group#) from v\\$log;
exit;
EOF`

undocnta=`grep -i ".undo_tablespace" $orafile | wc -l`
thrcnta=`grep -i ".thread" $orafile | wc -l`

if [ ! -z "$redosize" ]; then
rsize=$redosize"M"
else
rsize=1000"M"
fi

n=1
dthcnt=1
while [ "$n" -lt "$thrcnta" ]; do
        dthcnt=$(($dthcnt + 1))
        dgpcnt=$(($dgpcnt + 1))
	echo -e "alter database add logfile thread $dthcnt group $dgpcnt ('$new_db_create_file_dest') size $rsize reuse;" >> $openDatabase
        dgpcnt=$(($dgpcnt + 1))
        echo -e "alter database add logfile thread $dthcnt group $dgpcnt ('$new_db_recovery_file_dest') size $rsize reuse;" >> $openDatabase
        echo -e "alter database enable thread $dthcnt;" >> $openDatabase
        n=$(($n + 1));
done

if [ "$UNDOCONFIG_EXISTS" = "YES" ]; then
   for tname in $(echo $CREATE_UNDOTBSLIST | tr ',' ' ')
   do
     echo "create undo tablespace $tname datafile '$new_db_create_file_dest' size 500m autoextend on maxsize unlimited;" >> $openDatabase
   done
else
    n=1
    while [ "$n" -le "$undocnta" ]; do
          echo -e "create undo tablespace UNDOTBS_CL$n datafile '$new_db_create_file_dest/$osid/datafile/undotbs_cl$n.dbf' size 500m autoextend on maxsize unlimited;" >> $openDatabase
          n=$(($n + 1));
    done
fi

echo -e "alter system set cluster_database=TRUE scope=spfile sid='*';" >> $openDatabase
echo -e "shutdown immediate;" >> $openDatabase
echo -e "exit" >> $openDatabase

nsid1=$nsid"1"
orapw1=$ORACLE_HOME"/dbs/orapw"$nsid1

if [ -e $orapw1 ]; then
   rm -f $orapw1
fi

create_password_file $orapw1
./copyPfileToNodes.sh $oracleHome $nsid $BASEDIR $osuser $orapw1

chmod 755 $openDatabase
su -m $osuser -c "sqlplus / as sysdba @$openDatabase " >> $openDBlog

nsid1=$nsid"1"
export ORACLE_SID=$nsid1
su -m $osuser -c "srvctl start database -d $nsid"
#drop_undo_tablespace
retval=$?

db_status "READ WRITE"

create_setdb $nsid $ORACLE_SID
if [ "$nooratabupdate" != "true" ]; then
   create_oratab $nsid $ORACLE_SID
fi

fi  ### End of standalone=false

############ open pdbs and save state ###############
db_status "READ WRITE"

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log $dbrecover append " << 'EOF'
run {
ALTER pluggable database all open;
alter pluggable database all save state;
}
exit;
EOF
fi


create_controlfile_trace $fileloc $nsid
cleanup_archivelog $osid $new_db_recovery_file_dest
cleanup_file $openDatabase

exit 0

