#!/bin/sh

#set -x
# Script to create generic app.
source /act/scripts/act_MaxDB_src.conf

if [ "$#" -lt 1 ]; then
  echo " This script need 1 input parameter"
  echo " usage: createGenericApp.sh <Added Host ID>"
  echo " For example: createGenericApp.sh 2468"
  exit 1
fi

echo "--------------------------------------------"
echo " Creating the Generic app with LVM Volumes"
echo "--------------------------------------------"

appID=""

hostid=$1

set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x
#Check if App already exists

appID=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lsapplication" --data-urlencode "filtervalue=appname=$GenericAppName&apptype=LVM Volume"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

if [ ! -z "$appID" ]; then
    echo "App already exists...App ID  ::" $appID
else
    createGenericAppURL="https://$SkyIP/actifio/application?sessionid=$sessionid"
    appName=$GenericAppName
    protectedLvmList=$PROTECTED_LVM_LIST
    protectedLvmListModified=$(echo $protectedLvmList | cut -d[ -f2 | cut -d] -f1)
    #Fetch volume names using lsappvols API
    #curl -k -X POST -H 'Content-Type: application/json' 'https://172.23.16.6/actifio/host/26018/lsappvols?sessionid=f39156f5-b9a0-4997-87a0-86e7310c66c3'
    getVolumesURL="https://$SkyIP/actifio/host/$hostid/lsappvols?sessionid=$sessionid"
    getVolumesResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $getVolumesURL)

    vol=$(echo "$getVolumesResponse" | cut -d\( -f2 | cut -d\) -f1)

    mountList=($(echo "$vol" | jq -cr '.result [] | .mountpoint[].name'))
    mountListLength=${#mountList[@]}
    echo "Length Check" $mountListLength

    finalLvm=""
    protectedLvmListModified=$(echo $protectedLvmListModified | tr "," "\n")
    for lvm in $protectedLvmListModified
    do
      lvmlist=""
      lvm=$(echo $lvm | sed -e 's/\"//g')
      highestLen=0
      if [ "$mountListLength" -gt "0" ]; then
           for (( i=0; i<${mountListLength}; i++ ))
           do
             mountpath=${mountList[i]}
             echo "$lvm" | grep -w "${mountList[i]}"
             retval=$?
             if [ "$retval" -eq "0" ]; then
              echo "*** MATCH: $mountpath ************"
              currLen="${#mountpath}"
              if [ "$currLen" -gt "$highestLen" ]; then
                lvmlist="$(echo "$vol" | jq -cr '.result [] | select((.mountpoint[].name == "'"$mountpath"'")) | .volname')"
                highestLen=$currLen
              fi
             fi
           done
      fi

      if [ -z "$lvmlist" ]; then
        echo "***** Error: No Database is not present in any of the LVM Volumes...Exiting!!! "
        exit 1
      fi

      if [ ! -z "$lvmlist" ]; then
          if [ -z "$finalLvm" ]; then
            finalLvm+="["\"$lvmlist\"
          else
            finalLvm+=","\"$lvmlist\"
          fi
      fi
    done

    finalLvm+="]"

    inputParamsForGenericApp='{
     "appname":'"\"$appName\""',
     "hostid":'"$hostid"',
     "apptype":"LVM Volume",
     "volumes":'"${finalLvm}"',
     "vgsnapreservespace":"'"$VgSnapReserveSpace"'",
     "genericlvmscriptname":"genlvmscript.sh"
    }'

    createAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $createGenericAppURL -d "$inputParamsForGenericApp")
    if [[ $createAppResponse == *\"status\":\"failure\"* ]]; then
       echo "Failed to create App. Reason: " $createAppResponse
       exit 1
    elif [[ $createAppResponse == *\"status\":\"ok\"* ]]; then
       #appID=$(echo $createAppResponse | cut -d, -f1 | cut -d: -f2)
       appID=$(echo "$createAppResponse" | cut -d\( -f2 | cut -d\) -f1 | jq -cr '.result')
       echo "App created successfully. App ID is :" $appID
    fi
fi
echo "------------------------------------"
echo "Completed creating the Generic app"
echo "------------------------------------"


#Advanced Settings
set +x
updateAdvancedSettingsURL="https://$SkyIP/actifio/api/task/mkpolicyoption"
paramsForAdvancedSettings='{
"sessionid":'"\"$sessionid\""',
"appid":'"\"$appID\""',
"name":'"\"skypassword\""',
"value":'"\"$SkyPassword\""'
}'
updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")
if [[ $updateAdvanceSettingResponse == *errormessage* ]]; then
    echo "$updateAdvanceSettingResponse" | jq -cr '.errormessage'
fi
set -x
chApplicationURL="https://$SkyIP/actifio/application/$appID?sessionid=$sessionid"
inputParamsForApplication='{
    "description":{"type":"db","method":"snapshot","level":"incr","system-id":"'"$GenericAppName"'",
    "dbtype":"maxdb","database-name":"'"$SRC_DBNAME"'"}
}'
chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
echo "DB Application update Response: $chAppResponse"

# Copy scripts from /maxdb/outofband to /act/scripts

echo "------------------------------------------------------------"
echo "Configuring generic application for freeze and thaw phases"
echo "------------------------------------------------------------"


sudo cp -r /act/act_scripts/maxdb/outofband/freeze.xxx /act/scripts/
sudo cp -r /act/act_scripts/maxdb/outofband/thaw.xxx /act/scripts/
sudo cp -r /act/act_scripts/maxdb/outofband/abort.xxx /act/scripts/
sudo cp -r /act/act_scripts/maxdb/outofband/genlvmscript.sh /act/scripts/
sudo cp -r /act/act_scripts/maxdb/outofband/validateVdisk.sh /act/scripts/
chmod 755 /act/scripts/*
# Rename the script files freeze, thaw,abort.xxx to freeze,thaw,abort.appID

sudo mv /act/scripts/freeze.xxx /act/scripts/freeze.$appID
sudo mv /act/scripts/thaw.xxx /act/scripts/thaw.$appID
sudo mv /act/scripts/abort.xxx /act/scripts/abort.$appID

echo "--------------------------------------------"
echo "Completed generic application configuration"
echo "--------------------------------------------"

# Check if App is already protected

echo "-----------------------------------------"
echo "Start phase: Protecting the generic application"
echo "-----------------------------------------"

#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionid&filtervalue=name%3D$MaxdbTemplateName"
templateID=$(curl -s -k -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

if [ -z $templateID ]; then
  source /act/act_scripts/maxdb/automation/Backup/createTemplate.sh $MaxdbTemplateName $dbbackup_rpo "Snap Policy" "MAX DB Policy"
  templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')
fi

profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
profileID=$(curl -s -k -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')

isAppProtected=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lssla" --data-urlencode "filtervalue=appid=$appID"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

if [ ! -z "$isAppProtected" ]; then
    echo "App already protected"
else
   echo "Start App Protection..."
  #Protect the app
  protectionURL="https://$SkyIP/actifio/api/task/mksla"
  inputParamsForProtection='{
   "sessionid":'"$sessionid"',
   "appid":'"$appID"',
   "slpid":'"$profileID"',
   "sltid": '"$templateID"'
  }'

  protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")

  if [[ $protectionResponse == *errormessage* ]]; then
     echo "App Protection Failed. Reason :" $protectionResponse
  elif [[ $protectionResponse == *result* ]]; then
     echo "App Protected successfully."
  fi
fi

echo "------------------------------------------------"
echo "End phase: Completed protecting the application"
echo "------------------------------------------------"
