#!/bin/sh
set -x

function forEachDb()
{
  loopdbname=$i
  dbname=`echo $loopdbname | cut -d ':' -f 1`
  srcdbname=`echo $loopdbname | cut -d ':' -f 2`

  if [ -z $SRC_MAXDB_DBMCLI_PATH ]; then
    MAXDB_DBMCLI_PATH=$TARGET_MAXDB_DBMCLI_PATH
  else
    MAXDB_DBMCLI_PATH=$SRC_MAXDB_DBMCLI_PATH
  fi
  if [ -z $SRC_DBNAME_KEY_PREFIX ]; then
    DBNAME_KEY_PREFIX=$TARGET_DBNAME_KEY_PREFIX
  else
    DBNAME_KEY_PREFIX=$SRC_DBNAME_KEY_PREFIX
  fi
  dbmcli=`echo $MAXDB_DBMCLI_PATH | sed "s/,/\n/g" | grep -E ^"$dbname": | head -n 1 | cut -d ':' -f 2`/dbmcli
  if [ "$dbmcli" == "/dbmcli" ]; then
   echo "*** Error: dbname: $dbname Not Found in dbmcli path: $MAXDB_DBMCLI_PATH"
   exit 1
  fi
  xuserkey=`echo $DBNAME_KEY_PREFIX | sed "s/,/\n/g" | grep -E ^"$dbname": | head -n 1 | cut -d ':' -f 2`
  if [ -z $xuserkey ]; then
   echo "*** Error: dbname: $dbname Not Found in Userstore Key: $DBNAME_KEY_PREFIX"
   exit 1
  fi
  if [ ! -f $dbmcli ]; then
    echo "*** Error: dbmcli_path $dbmcli does not exist for db:$dbname ***"
    exit 1
  fi
  xuserpath=`echo $MAXDB_DBMCLI_PATH | sed "s/,/\n/g" | grep -E ^"$dbname": | head -n 1 | cut -d ':' -f 2`/xuser
  if [ ! -f $xuserpath ]; then
    echo "Error: $xuserpath does not exist"
    exit 1
  fi
  touch /act/touch/XUSER_LIST_MESSAGE
  chmod 777 /act/touch/XUSER_LIST_MESSAGE
  su - $OSUSER -c "$xuserpath list > /act/touch/XUSER_LIST_MESSAGE" > /dev/null 2>&1
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** ERRMSG: $xuserpath list has Failed due to: `cat /act/touch/XUSER_LIST_MESSAGE`"
    exit 1
  fi
  cat /act/touch/XUSER_LIST_MESSAGE | grep -w $xuserkey
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** Error No xuser entry for key(user store key): $xuserkey for db: $dbname ***"
    echo "*** ERRMSG: `cat /act/touch/XUSER_LIST_MESSAGE`"
    exit 1
  fi

  #srcdbname=$dbname
  #if [ ! -z $DB_BACKUP_LOCATION ]; then
    #CHECK_FULL_BKP=`ls $DB_BACKUP_LOCATION | grep -Eiw ^"$dbname"_"fullDataBackup" | wc -l`
    #if [ "$CHECK_FULL_BKP" -gt "0" ]; then
      #srcdbname=$dbname
    #else
      #srcdbname=`ls $DB_BACKUP_LOCATION | grep -i "_fullDataBackup" | tail -n 1 | cut -d '_' -f 1`
      #if [ -z $srcdbname ]; then
        #echo "*** Error: source dbname cannot be null ***"
        #exit 1
      #fi
    #fi
  #fi
}

function getdbnamelist()
{
  if [ -z $SRC_DBNAME ]; then
    dbnamelist=$TARGET_DBNAME
  else
    dbnamelist=$SRC_DBNAME
  fi
}

function validateDumpThreadCount()
{
  if [ -z $dumpthreadcount ]; then
    dumpthreadcount=2
  fi
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/MAX_MEDIA_MESSAGE
    chmod 777 /act/touch/MAX_MEDIA_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey param_directget MaxBackupMedia > /act/touch/MAX_MEDIA_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error param_directget MaxBackupMedia has Failed: `cat /act/touch/MAX_MEDIA_MESSAGE`"
      exit 1
    fi
    MaxBackupMedia=`cat /act/touch/MAX_MEDIA_MESSAGE |  tail -n 1 | awk -F' ' '{print $NF}'`
    if [ "$dumpthreadcount" -gt "$MaxBackupMedia" ]; then
      echo "*** ERROR: DumpThreadCount cannot be greater than MaxBackupMedia for $dbname"
      exit 1
    fi
  done
}

function getDbVolumes()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb

    touch /act/touch/GETVOLSALL_MESSAGE
    chmod 777 /act/touch/GETVOLSALL_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey param_getvolsall > /act/touch/GETVOLSALL_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error param_getvolsall has Failed `cat /act/touch/GETVOLSALL_MESSAGE` ***"
      exit 1
    fi
    dbVolumes=`cat /act/touch/GETVOLSALL_MESSAGE | grep -i VolumeName | awk -F' ' '{print $4}'`
    lvmlist="${lvmlist}${lvmlist:+" "}$dbVolumes"
  done
}

function getDbSize()
{
  dbSize=0;
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb

    touch /act/touch/DBSIZE_MESSAGE
    chmod 777 /act/touch/DBSIZE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey sql_execute select USABLESIZE from DATASTATISTICS > /act/touch/DBSIZE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error sql_execute getDbSize has Failed. `cat /act/touch/DBSIZE_MESSAGE`"
      exit 1
    fi
    dbServerSize=`cat /act/touch/DBSIZE_MESSAGE | tail -n 1`
    if [ -z "$dbServerSize" ]; then
       echo " Error : Query Failure to get DB size for DB:$dbname"
       exit 1
    fi

    dbSize=`expr $dbSize + $dbServerSize`
  done
}


function createCompleteDataTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    db_backup_template=$dbname"_"$templateSuffix
    fileLocation=$backupDirLocation/$srcdbname"_"fullDataBackup
    echo "creating MaxDB data backup template for $dbname *********"
    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_template_create $db_backup_template to FILE $fileLocation OVERWRITE COMPRESSED CONTENT DATA > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "created MaxDB data backup template for $dbname *********"
    else
       echo "*** Error: Data backup template creation failed for $dbname. `cat /act/touch/TEMPLATE_MESSAGE` ******"
       exit 1
    fi
  done
}

function createIncrementalTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    db_backup_template=$dbname"_"$templateSuffix
    fileLocation=$backupDirLocation/$srcdbname"_"incrementalData
    echo "creating MaxDB incremental data backup template for $dbname *********"
    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_template_create $db_backup_template to FILE $fileLocation OVERWRITE COMPRESSED CONTENT INCREMENTAL > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "created MaxDB incremental data backup template for $dbname *********"
    else
       echo "INCREMENTAL Data backup template creation failed. `cat /act/touch/TEMPLATE_MESSAGE` ******"
       exit 1
    fi
  done
}

function executeDbTemplate()
{
  templateSuffix=$1
  isbackuptypeFull=`echo $templateSuffix | grep -i actfullDbTemplate`
  if [ ! -z "$isbackuptypeFull" ]; then
    backup_type="FULL"
  else
    backup_type="INCREMENTAL"
  fi
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    echo "********** Take a DB backup $backup_type for $dbname ************"
    db_backup_template=$dbname"_"$templateSuffix
    touch /act/touch/BACKUP_MESSAGE
    chmod 777 /act/touch/BACKUP_MESSAGE
    OSGROUP=`id -ngr $OSUSER | cut -d" " -f1`
    chown -R $OSUSER:$OSGROUP /act/touch/BACKUP_MESSAGE
su - $OSUSER -c "$dbmcli -o /act/touch/BACKUP_MESSAGE -d $dbname -U $xuserkey" << EOF
db_connect
backup_start $db_backup_template
EOF

    touch /act/touch/HIST_MESSAGE
    chmod 777 /act/touch/HIST_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_history_list -c KEY,LABEL,ACTION,RC,ERROR -Inverted > /act/touch/HIST_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error backup_history_list command has Failed. `cat /act/touch/HIST_MESSAGE`"
      exit 1
    fi
    historyoutput=`cat /act/touch/HIST_MESSAGE | sed -n '3p'`
    echo "*** Last Backup History Record: $historyoutput"
    KEY=`echo $historyoutput | cut -d'|' -f1`
    echo "*** DB Backup Key: $KEY"
    LABEL=`echo $historyoutput | cut -d'|' -f2`
    echo "*** DB Backup LABEL: $LABEL"
    ACTION=`echo $historyoutput | cut -d'|' -f3`
    echo "*** DB Backup ACTION: $ACTION"
    RC=`echo $historyoutput | cut -d'|' -f4 | xargs`
    echo "*** DB Backup Return Code: $RC"
    errMsg=`cat /act/touch/BACKUP_MESSAGE | grep -w "ERR"`
    if [ ! -z "$errMsg" ]; then
      if [ "$RC" == "0" ]; then
        ERROR=`cat /act/touch/BACKUP_MESSAGE`
      else
        ERROR=`echo "$historyoutput" | cut -d'|' -f5`
      fi
      echo "******* ERROR: Failed to take MaxDB data backup for $dbname !! $ERROR ************"
      echo "*********** BACKUP_TYPE = $backup_type and BACKUP_STATUS = $dbname:FAIL ***************"
      exit 1
    fi
    echo "*********** BACKUP_TYPE = $backup_type and BACKUP_STATUS = $dbname:SUCCESS ***************"
  done
}

function createExecuteLogTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  logBackupFile=$3
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    log_backup_template=$dbname"_"$templateSuffix
    fileLocation=$backupDirLocation/$srcdbname"_"$logBackupFile

    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    echo "Check if MaxDB log backup template already exists *********"
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey medium_get $log_backup_template > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "MaxDB log backup template for $dbname  already exists!! Skip Creation *********"
    else
      echo "***** creating MaxDB log backup template *********"
      su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_template_create $log_backup_template to FILE $fileLocation CONTENT LOG > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
      retval=$?
      if [ $retval -eq 0 ]; then
          echo "created MaxDB log backup template for $dbname  *********"
      else
         echo "log backup template creation failed for $dbname ... `cat /act/touch/TEMPLATE_MESSAGE` ******"
      exit $retval
      fi
    fi

    echo "********** PRE: starting MaxDB Log Backup for $dbname   ************"
    touch /act/touch/BACKUP_MESSAGE
    chmod 777 /act/touch/BACKUP_MESSAGE
    OSGROUP=`id -ngr $OSUSER | cut -d" " -f1`
    chown -R $OSUSER:$OSGROUP /act/touch/BACKUP_MESSAGE
su - $OSUSER -c "$dbmcli -o /act/touch/BACKUP_MESSAGE -d $dbname -U $xuserkey" << EOF
db_connect
backup_start $log_backup_template
EOF
    touch /act/touch/HIST_MESSAGE
    chmod 777 /act/touch/HIST_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_history_list -c KEY,LABEL,ACTION,RC,ERROR -Inverted > /act/touch/HIST_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error backup_history_list command has Failed. `cat /act/touch/HIST_MESSAGE`"
      exit 1
    fi
    historyoutput=`cat /act/touch/HIST_MESSAGE | sed -n '3p'`
    echo "*** Last Log Backup History Record: $historyoutput"
    KEY=`echo $historyoutput | cut -d'|' -f1`
    echo "*** Log Backup Key: $KEY"
    LABEL=`echo $historyoutput | cut -d'|' -f2`
    echo "*** Log Backup LABEL: $LABEL"
    ACTION=`echo $historyoutput | cut -d'|' -f3`
    echo "*** Log Backup ACTION: $ACTION"
    RC=`echo $historyoutput | cut -d'|' -f4 | xargs`
    echo "*** Log Backup Return Code: $RC"
    errMsg=`cat /act/touch/BACKUP_MESSAGE | grep -w "ERR"`
    if [ ! -z "$errMsg" ]; then
       logef=`cat /act/touch/BACKUP_MESSAGE | grep -i '\-123,No more log to save'`
       if [ -z "$logef" ]; then
         if [ "$RC" == "0" ]; then
           ERROR=`cat /act/touch/BACKUP_MESSAGE`
         else
           ERROR=`echo "$historyoutput" | cut -d'|' -f5`
         fi
         echo "******* PRE ERROR: Failed MaxDB LogBackup due to:  $ERROR************"
         echo "*********** Log BACKUP_STATUS = $dbname:FAIL ***************"
         exit 1
       else
          echo "********* MaxDB log backup. No more log to save ********"
       fi
    fi
    echo "*********** Log BACKUP_STATUS = $dbname:SUCCESS ***************"
    sleep 10
    sync
    echo "********** end PRE: MaxDB log backup for $dbname ************"

  done
}

function isDatabaseOnline()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/DBSTATE_MESSAGE
    chmod 777 /act/touch/DBSTATE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_state > /act/touch/DBSTATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error db_state command has Failed: `cat /act/touch/DBSTATE_MESSAGE`"
      exit 1
    fi
    LC_DB_STATE=`cat /act/touch/DBSTATE_MESSAGE | grep -w -E 'ONLINE|OFFLINE' | sed -e s/\'//g`
    status=`echo $LC_DB_STATE | grep -i ONLINE | wc -l`

    if [ "$status" -eq "0" ]; then
     echo "############### ERROR ################"
     echo "Database is not online $dbname . Please Verify!"
     exit 1
    fi
  done
}

function suspendLogWriter()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/LOGWRITER_MESSAGE
    chmod 777 /act/touch/LOGWRITER_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey util_execute suspend logwriter > /act/touch/LOGWRITER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error util_execute suspend logwriter has Failed: `cat /act/touch/LOGWRITER_MESSAGE`"
      exit 1
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey sql_execute 'select LOGWRITERSUSPENDED from loginformation' > /act/touch/LOGWRITER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error sql_execute LOGWRITERSUSPENDED Query has Failed: `cat /act/touch/LOGWRITER_MESSAGE`"
      exit 1
    fi
    LC_STATUS=`cat /act/touch/LOGWRITER_MESSAGE | grep -w -E 'NO|YES' | sed -e s/\'//g`
    retval=`echo $LC_STATUS | grep -w "YES"`
    if [ ! -z "$retval" ]; then
      echo "Database $dbname is in Backup Mode now!"
    else
      echo "############### ERROR ################"
      echo "Failed to put Database $dbname in Backup Mode!"
      exit 1
    fi
  done
}

function resumeLogWriter()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/LOGWRITER_MESSAGE
    chmod 777 /act/touch/LOGWRITER_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey util_execute resume logwriter > /act/touch/LOGWRITER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error util_execute resume logwriter has Failed: `cat /act/touch/LOGWRITER_MESSAGE`"
      exit 1
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey util_release > /act/touch/LOGWRITER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error util_release has Failed: `cat /act/touch/LOGWRITER_MESSAGE`"
      exit 1
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey sql_execute 'select LOGWRITERSUSPENDED from loginformation' > /act/touch/LOGWRITER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error Select LOGWRITERSUSPENDED Query has Failed: `cat /act/touch/LOGWRITER_MESSAGE`"
      exit 1
    fi
    LC_STATUS=`cat /act/touch/LOGWRITER_MESSAGE | grep -w -E 'NO|YES' | sed -e s/\'//g`
    retval=`echo $LC_STATUS | grep -w "NO"`
    if [ ! -z "$retval" ]; then
      echo "Successfully Removed Database $dbname from Backup Mode!"
    else
      echo "#################### ERROR ##################"
      echo "Failed to Remove Database $dbname from Backup Mode!"
      exit 1
    fi
  done
}

function backupConfigFiles()
{
  backupDirLocation=$1
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    echo "Copying dbm.mdf and dbm.knl files for db $dbname into $LOG_BKP_LOC"
    touch /act/touch/DATAPATH_MESSAGE
    chmod 777 /act/touch/DATAPATH_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey dbm_getpath > /act/touch/DATAPATH_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error dbm_getpath Query has Failed: `cat /act/touch/DATAPATH_MESSAGE`"
      exit 1
    fi
    dataConfPath=`cat /act/touch/DATAPATH_MESSAGE | grep -w DataPath | cut -d '=' -f 2`
    findCfg=`find "$dataConfPath/wrk/$dbname" -name dbm.mdf | head -n 1`
    su - $OSUSER -c "cp -r "$findCfg" $backupDirLocation/$dbname"_"dbm.mdf"
    findCfg=`find "$dataConfPath/wrk/$dbname" -name dbm.knl | head -n 1`
    su - $OSUSER -c "cp -r "$findCfg" $backupDirLocation/$dbname"_"dbm.knl"
  done
}

function backupConfigParam()
{
  backupDirLocation=$1
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    echo "Copying DB file for db $dbname into $LOG_BKP_LOC"
    touch /act/touch/DATAPATH_MESSAGE
    chmod 777 /act/touch/DATAPATH_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey dbm_getpath > /act/touch/DATAPATH_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error dbm_getpath has Failed: `cat /act/touch/DATAPATH_MESSAGE`"
      exit 1
    fi
    dataConfPath=`cat /act/touch/DATAPATH_MESSAGE | grep -w DataPath | cut -d '=' -f 2`
    su - $OSUSER -c "cp -r $dataConfPath/config/$dbname $backupDirLocation"
    su - $OSUSER -c "cp -r $dataConfPath/config/$dbname.upc $backupDirLocation"
  done
}

function makeDbOffline()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/DB_MESSAGE
    chmod 777 /act/touch/DB_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_offline > /act/touch/DB_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
     echo "*** ERROR: While making db $dbname offline!! : `cat /act/touch/DB_MESSAGE`"
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_state > /act/touch/DB_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error db_state has Failed: `cat /act/touch/DB_MESSAGE`"
      exit 1
    fi
    maxdbStatus=`cat /act/touch/DB_MESSAGE | grep -i offline`
    if [ -z "$maxdbStatus" ]; then
      echo "*** ERROR: Running database $dbname .. Exiting!"
      exit 1
    fi
  done
}

function makeDbOnline()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/DB_MESSAGE
    chmod 777 /act/touch/DB_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_admin > /act/touch/DB_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "Db $dbname is now in Admin Mode *********"
    else
       echo "Error: Admin mode Failed for $dbname .. `cat /act/touch/DB_MESSAGE` ******"
       exit $retval
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_execute clear log > /act/touch/DB_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "Clear Log Is completed for $dbname *********"
    else
       echo "**** Error: clear log has failed. `cat /act/touch/DB_MESSAGE` ******"
       exit $retval
    fi
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_online > /act/touch/DB_MESSAGE" > /dev/null
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "Db $dbname is now online *********"
    else
       echo "**** Error: Could not make $dbname online... `cat /act/touch/DB_MESSAGE` ******"
       exit $retval
    fi
  done
}

function createLogTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  logBackupFile=$3
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    log_backup_template=$dbname"_"$templateSuffix
    fileLocation=$backupDirLocation/$srcdbname"_"$logBackupFile
    echo "creating MaxDB log template for $dbname *********"
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey backup_template_create $log_backup_template to FILE $fileLocation CONTENT LOG > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "created MaxDB log template for $dbname *********"
    else
       echo "*** Error: log template creation failed for $dbname ... `cat /act/touch/TEMPLATE_MESSAGE` ******"
       exit $retval
    fi
  done
}

function createCompleteMultiDataTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  dumpthreads=$3
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    dbfulltemp=""
    db_backup_template=$dbname"_"$templateSuffix
    echo "creating MaxDB data template for $dbname *********"
    if [ -z $dumpthreads ]; then
      dumpthreads=2
    fi
    astring="backup_template_create $db_backup_template"
    for (( j=1; j<= $dumpthreads; j++ ))
    do
      fileLocation=$backupDirLocation/$srcdbname"_"fullDataBackup$j
      if [ -z "$dbfulltemp" ]; then
          dbfulltemp=`echo $astring " to FILE $fileLocation OVERWRITE COMPRESSED NAMED b$j "`
      else
          dbfulltemp=`echo $dbfulltemp " FILE $fileLocation OVERWRITE COMPRESSED NAMED b$j "`
      fi
    done
    dbfulltemp=`echo $dbfulltemp " CONTENT DATA"`
    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey $dbfulltemp > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "****Error: Data template creation failed for $dbname. `cat /act/touch/TEMPLATE_MESSAGE` ******"
      exit 1
    fi
    echo "created MaxDB data template for $dbname *********"
  done
}

function createIncrementMultiDataTemplate()
{
  backupDirLocation=$1
  templateSuffix=$2
  dumpthreads=$3
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    dbincrtemp=""
    db_incr_backup_template=$dbname"_"$templateSuffix
    echo "creating MaxDB data template for $dbname *********"
    if [ -z $dumpthreads ]; then
      dumpthreads=2
    fi
    astring="backup_template_create $db_incr_backup_template"
    for (( j=1; j<= $dumpthreads; j++ ))
    do
      fileLocation=$backupDirLocation/$srcdbname"_"incrDataBackup$j
      if [ -z "$dbincrtemp" ]; then
          dbincrtemp=`echo $astring " to FILE $fileLocation OVERWRITE COMPRESSED NAMED b$j "`
      else
          dbincrtemp=`echo $dbincrtemp " FILE $fileLocation OVERWRITE COMPRESSED NAMED b$j "`
      fi
    done
    dbincrtemp=`echo $dbincrtemp " CONTENT INCREMENTAL"`
    touch /act/touch/TEMPLATE_MESSAGE
    chmod 777 /act/touch/TEMPLATE_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey $dbincrtemp > /act/touch/TEMPLATE_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "****Error: INCREMENTAL Data template creation failed for $dbname `cat /act/touch/TEMPLATE_MESSAGE` ******"
      exit 1
    fi
    echo "created MaxDB INCREMENTAL data template for $dbname *********"
  done
}

function validateDBConnection()
{

  id $OSUSER
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** Error: OSUSER:$OSUSER does not exist "
    exit 1
  fi
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/VALIDATION_MESSAGE
    chmod 777 /act/touch/VALIDATION_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_state > /act/touch/VALIDATION_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "DB Validation Success for $dbname *********"
    else
       echo "*** Error: DB Validation failed for $dbname!!! `cat /act/touch/VALIDATION_MESSAGE` ******"
       exit $retval
    fi
  done
}

function validatePrivileges()
{
  getdbnamelist
  for i in $(echo $dbnamelist | sed "s/,/ /g")
  do
    forEachDb
    touch /act/touch/XUSER_MESSAGE
    chmod 777 /act/touch/XUSER_MESSAGE
    su - $OSUSER -c "$xuserpath -d $dbname -U $xuserkey list > /act/touch/XUSER_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error $xuserkey list has Failed: `cat /act/touch/XUSER_MESSAGE`"
      exit 1
    fi
    dbusername=`cat /act/touch/XUSER_MESSAGE | grep -iw Username | cut -d ':' -f2`
    if [ -z $dbusername ]; then
      echo " Error : dbusername cannot be null"
      exit 1
    fi
    touch /act/touch/RIGHTS_MESSAGE
    chmod 777 /act/touch/RIGHTS_MESSAGE
    su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey user_get $dbusername > /act/touch/RIGHTS_MESSAGE" > /dev/null 2>&1
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "*** Error user_get $dbusername has Failed: `cat /act/touch/RIGHTS_MESSAGE`"
      exit 1
    fi
    dbserverrights=`cat /act/touch/RIGHTS_MESSAGE | grep -iw SERVERRIGHTS`
    if [ -z $dbserverrights ]; then
      echo " Error : dbserverrights cannot be null"
      exit 1
    fi
    echo $dbserverrights | grep -iw Backup | grep -iw Recovery | grep -iw DBStop | grep -iw DBStart | grep -iw ParamRead | grep -iw ParamFull
    retval=$?
    if [ $retval -eq 0 ]; then
        echo "DB Privileges: Good for $dbname *********"
    else
       echo "*** Error: DB Privileges Missing  for $dbname!!! ******"
       exit 1
    fi
  done
}
