#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

function validateDbParams() {
  source /act/act_scripts/maxdb/automation/Backup/dbHelper.sh

  if [ ! -z "$OSUSER" ] && [ ! -z "$SRC_MAXDB_DBMCLI_PATH" ] && [ ! -z "$SRC_DBNAME" ] && [ ! -z "$SRC_DBNAME_KEY_PREFIX" ]; then
    if [[ $LogBackupEnable == "true" ]]; then
      if [ -z $LogBackupFS ]; then
        echo " Error : LogBackupFS cannot be empty."
        exit 1
      fi
      validateDirPath $LogBackupFS
      validateDirPath $LOG_BKP_LOC
      if [ ! -z $LOG_BKP_LOC ]; then
        echo $LOG_BKP_LOC | grep -w $LogBackupFS
        retval=$?
        if [ $retval -ne 0 ]; then
          cat /proc/mounts | grep -w $LogBackupFS
          retval=$?
          if [ $retval -ne 0 ]; then
            echo "*** Error LogLocation: $LOG_BKP_LOC is not contained in Log Mountpoint: $LogBackupFS"
            exit 1
          else
            nfsmountlogloc=$(cat /proc/mounts | grep -w $LogBackupFS | awk -F' ' '{print $2}')
            echo $LOG_BKP_LOC | grep -w $nfsmountlogloc
            retval=$?
            if [ $retval -ne 0 ]; then
              echo "*** Error LogLocation: $LOG_BKP_LOC is not contained in NFS Mountpoint: $nfsmountlogloc"
              exit 1
            fi
          fi
        fi
      fi

      if [ ! -z $DUMPSCHEDULE ]; then
        totalCharCount=$(echo ${#DUMPSCHEDULE})
        charMatchCount=$(echo "$DUMPSCHEDULE" | grep -o '[F|I]' | wc -l)
        if [ "$totalCharCount" -ne "7" ] || [ "$charMatchCount" -ne "7" ]; then
          echo "Error: Invalid Dump Schedule $DUMPSCHEDULE"
          exit 1
        fi
      fi
    fi
  elif [ ! -z "$OSUSER" ] && [ ! -z "$TARGET_MAXDB_DBMCLI_PATH" ] && [ ! -z "$TARGET_DBNAME" ] && [ ! -z "$TARGET_DBNAME_KEY_PREFIX" ]; then
    :
  else
    echo "Error: validateDbParams has Failed.."
    exit 1
  fi
  validateDBConnection
  validatePrivileges
}

function validateDirPath() {
  dirPath=$1
  if [ ! -z $dirPath ]; then
    if [ ! -d $dirPath ]; then
      cat /proc/mounts | grep -w $dirPath
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: $dirPath does not exist"
        exit 1
      fi
    fi
  fi
}

function validateFileExists() {
  fileloc=$1
  if [ ! -z $fileloc ]; then
    if [ ! -f $fileloc ]; then
      echo "Error: $fileloc does not exist"
      exit 1
    fi
  fi
}
