#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

conffile="/act/scripts/act_maxdb_lvm_singledb_restore_config.conf"

echo -e '#Max DB/Logbackup Specific Params' > $conffile
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   passwdstrCheck=`echo $ARGUMENT | grep -i "password\|passwd"`
   if [ -z "$passwdstrCheck" ]; then
     echo -e $ARGUMENT >> $conffile
   else
     export $ARGUMENT
   fi
done

source $conffile
ACTIFIO_UNIQUE="ACTIFIO_UNIQUE"

OSUSER=$OSUSER
SOURCESID=$SOURCESID
TARGET_SERVER_NAME=$DBSID
TARGET_MNT_PNT=$TARGET_MNT_PNT
TARGET_DB_USER=superdba
TARGET_DBUSER_PASSWD=sapuser123

MANIFEST_FILE_LOC=$(find $TARGET_MNT_PNT/ -iname config_backup | head -n 1 )
if [ -z "$MANIFEST_FILE_LOC" ]; then
  echo "ERROR:CONFIG FILE NOT FOUND"
  exit 1
fi

dbmcli=$TARGET_MAXDB_DBMCLI_PATH
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

DB_FREEZEDATE_FILE_LOCATION=$(find $LOG_BKP_MNTPT/ -iname dbfreezedate.dat | rev | cut -d"/" -f2-  | rev)
if [ -z $DB_FREEZEDATE_FILE_LOCATION ]; then
  echo "Error: Cannot Find the File dbfreezedate.dat which contains DB Freeze Date"
  exit 1
fi

UNTIL_TIME=$UNTIL_TIME
ACT_JOBNAME=$JOBID

#Convert TARGET_SERVER_NAME  to uppercase

TARGET_SERVER_NAME=`echo "$TARGET_SERVER_NAME" | tr "[:lower:]" "[:upper:]"`
echo "$TARGET_SERVER_NAME"

#---------------------------------------------------------------------------
# Update the Parameter file with the new Log and DATA Volume
#---------------------------------------------------------------------------
Update_data_log_loc() {
  vol_name=$1
  file_name=$2


  su - $OSUSER -c "$dbmcli -d $TEMP_INSTANCE -u $TARGET_DB_USER,$TARGET_DBUSER_PASSWD  param_put $vol_name  $file_name"
  retval=$?


  if [ $retval -ne 0 ]; then
    echo " Parameter file update failed for Data and Log Location"
    exit 1
  fi
}
#---------------------------------------------------------------------------


#---------------------------------------------------------------------------
# Clear the  info of  Log and DATA Volume from Prameter  file
#---------------------------------------------------------------------------
del_data_log_vol() {
  vol_no=$1
  vol_type=$2


 su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY param_delvolume $vol_no $vol_type"
  retval=$?


  if [ $retval -ne 0 ]; then
    echo " Clearing of Data and Log Location in to the parameter file failed"

  fi
}
#---------------------------------------------------------------------------


OSGRPUSR=`id -ngr $OSUSER | cut -d " " -f2`

tmp_loc=/act/touch/$ACT_JOBNAME
mkdir -p $tmp_loc
chmod -R 777 /act/touch
chown -R $OSUSER:$OSGRPUSR $tmp_loc
find /act -type d -exec chmod 755 {} +


export PATH=$LOCAL_PRG_PATH/bin:$PATH

INDEP_DATA_PATH=`su - $OSUSER -c "$dbmcli dbm_getpath IndepDataPath | sed 's/$/$ACTIFIO_UNIQUE/'" | grep $ACTIFIO_UNIQUE | sed -e "s/$ACTIFIO_UNIQUE//g" | grep -v -w "OK"`


  #Check if the DB Instance is already exist :
  isDB_exists=`su - $OSUSER -c "$dbmcli db_enum | sed 's/$/$ACTIFIO_UNIQUE/'" | grep $ACTIFIO_UNIQUE | sed -e "s/$ACTIFIO_UNIQUE//g" | grep -w $TARGET_SERVER_NAME | wc -l`
  if [ $isDB_exists -eq 0 ]; then
    echo "ERRORMSG: Target server Does not exits"
    exit 1
  fi

#Check if Config Directory already exist
if [ ! -d $INDEP_DATA_PATH/config ]; then
  mkdir -p $INDEP_DATA_PATH/config
fi

#Create Temp Instnace :
TEMP_INSTANCE=${TARGET_SERVER_NAME}_TEMP

su - $OSUSER -c "$dbmcli db_create $TEMP_INSTANCE $TARGET_DB_USER,$TARGET_DBUSER_PASSWD"


#Copy the Config file
cp $MANIFEST_FILE_LOC/$SOURCESID $INDEP_DATA_PATH/config/$TEMP_INSTANCE
retval=$?
if [ $retval -ne 0 ]; then
echo "ERRORMSG: Unable to find the Manifest File"
exit 1
fi

chown -R $OSUSER:$OSGRPUSR $INDEP_DATA_PATH/config/$TEMP_INSTANCE
chmod -R 775  $INDEP_DATA_PATH/config/$TEMP_INSTANCE
##Chekc if RunDirectry already Exist
TRG_RUNDIRECTORY=$INDEP_DATA_PATH/wrk/$TARGET_SERVER_NAME
if [ ! -d $TRG_RUNDIRECTORY ]; then
  mkdir -p $TRG_RUNDIRECTORY
fi
chown -R $OSUSER:$OSGRPUSR $TRG_RUNDIRECTORY
chmod -R 775  $TRG_RUNDIRECTORY
#Login the Temp instance to get the source file details :

SRC_VOLINFO=$tmp_loc/SRC_VOLINFO

su - $OSUSER -c "$dbmcli -d $TEMP_INSTANCE -u $TARGET_DB_USER,$TARGET_DBUSER_PASSWD -o $SRC_VOLINFO param_getvolsall"
retval=$?


#Login the Target Instance to get Target file details :
TRG_VOLINFO=$tmp_loc/TRG_VOLINFO

su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY -o $TRG_VOLINFO param_getvolsall"
retval=$?



#Get the Target location of datafile and logfile

SRC_VOLINFO1=$tmp_loc/SRC_VOLINFO.tmp
TRG_VOLINFO1=$tmp_loc/TRG_VOLINFO.tmp
SRC_VOLINFO2=$tmp_loc/SRC_VOLINFO2.tmp

cat $SRC_VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" >$SRC_VOLINFO1
cat $TRG_VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" >$TRG_VOLINFO1
TRG_DATA_DIR=`dirname $(cat $TRG_VOLINFO1 | grep "DataVolumeName" | awk '{print $4}' | head -1)`
TRG_LOG_DIR=`dirname $(cat $TRG_VOLINFO1 | grep "LogVolumeName" | awk '{print $4}' | head -1)`
#SRC_DATA_DIR=`dirname $(cat $SRC_VOLINFO.tmp | grep "DataVolumeName" | awk '{print $4}' | head -1)`
#SRC_LOG_DIR=` dirname $(cat $SRC_VOLINFO.tmp | grep "LogVolumeName" | awk '{print $4}' | head -1)`

#unalias cp command
#unalias cp

cp $SRC_VOLINFO1 $SRC_VOLINFO2

#Clean the Traget Data and Log files

cat $TRG_VOLINFO1 |  awk '{ print "rm -f " $4}'> $tmp_loc/del_trg_file.sh
sh $tmp_loc/del_trg_file.sh
retval=$?

if [ $retval -ne 0 ]; then
echo "ERROSMSG: Failed to clear the target datafile and log files"
exit 1
fi


#Copy the Mounted datafiles to Target Directory

SRC_DATA_INFO=$tmp_loc/SRC_DATA_INFO
SRC_LOG_INFO=$tmp_loc/SRC_LOG_INFO

cat $SRC_VOLINFO1 | grep "DataVolumeName" | awk '{print $4}'>$SRC_DATA_INFO
while read line
do
src_db_file=`echo $line`
mount_db_file=$TARGET_MNT_PNT$src_db_file
cp $mount_db_file $TRG_DATA_DIR
done < $SRC_DATA_INFO

cat $SRC_VOLINFO1 | grep "LogVolumeName" | awk '{print $4}'>$SRC_LOG_INFO
while read line
do
src_log_file=`echo $line`
mount_log_file=$TARGET_MNT_PNT$src_log_file
cp $mount_log_file $TRG_LOG_DIR
done < $SRC_LOG_INFO

#set the ownership to the data and log files

chown -R $OSUSER:$OSGRPUSR  $TRG_LOG_DIR
chmod -R 777 $TRG_LOG_DIR
chown -R $OSUSER:$OSGRPUSR  $TRG_DATA_DIR
chmod -R 777 $TRG_DATA_DIR

#update the volume name and target file in temp file

while read line
do
src_data_dir=`dirname $(echo $line)`
src_data_dir_sed=`echo $src_data_dir | sed 's./.\\\/.g'`
trg_data_dir_sed=`echo $TRG_DATA_DIR | sed 's./.\\\/.g'`
sed -i "s.$src_data_dir_sed.$trg_data_dir_sed.g" $SRC_VOLINFO2
done <$SRC_DATA_INFO

while read line
do
src_log_dir=`dirname $(echo $line)`
src_log_dir_sed=`echo $src_log_dir | sed 's./.\\\/.g'`
trg_log_dir_sed=`echo $TRG_LOG_DIR | sed 's./.\\\/.g'`
sed -i "s.$src_log_dir_sed.$trg_log_dir_sed.g" $SRC_VOLINFO2
done < $SRC_LOG_INFO

#Clean the Target datafile and logfile info from parameter file

#clean(){
#while read line
#do
#  vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
#  vol_no=`echo ${vol_name: -1}`
# First_char_vol_name=`echo $vol_name | cut -c1`
#  if [ "$First_char_vol_name" = "D" ]; then
#     vol_type=DATA
#  else
#     vol_type=LOG
#  fi
#  del_data_log_vol $vol_no $vol_type
#done < $TRG_VOLINFO1
#}

# Update DATA and LOG volume path in parameter file Temp DB instance

while read line
do
  vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
  trg_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
  su - $OSUSER -c "$dbmcli -d $TEMP_INSTANCE -u $TARGET_DB_USER,$TARGET_DBUSER_PASSWD  param_put $vol_name $trg_file_name"
  #Update_data_log_loc $vol_name $trg_file_name
  retval=$?

  if [ $retval -ne 0 ]; then
    echo " Parameter file update failed for Data and Log Location in Temp DB instance"
    exit 1
  fi
done < $SRC_VOLINFO2

#Update the TragetRunDirectory in Temp db Instance


 su - $OSUSER -c "$dbmcli -d $TEMP_INSTANCE -u $TARGET_DB_USER,$TARGET_DBUSER_PASSWD param_put RUNDIRECTORY $TRG_RUNDIRECTORY"
 retval=$?


if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to update the rundirectory in temp instance"
  exit 1
fi

#Repalce the Target config file from Temp Config file

mv $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME $INDEP_DATA_PATH/config/${TARGET_SERVER_NAME}_old
mv $INDEP_DATA_PATH/config/$TEMP_INSTANCE $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME

chown -R $OSUSER:$OSGRPUSR $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME

# Start Instance in admin mode

su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY db_admin"
retval=$?

if [ $retval -ne 0 ]; then
  echo "Chnaging ownership of target config directory as  Database Instance Failed to admin Mode"
  chown -R sdb:sdba $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME
  su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY db_admin"
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Database Instance Failed to admin Mode"
    exit 1
  fi
fi

su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY db_offline"
retval=$?



#Drop the Temp Instance :


su - $OSUSER -c "$dbmcli -d $TEMP_INSTANCE -u $TARGET_DB_USER,$TARGET_DBUSER_PASSWD -c db_drop"
retval=$?


if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Temp DB  Instance Failed to drop"
  exit 1
fi

MAXDBRESTORECONFIG=/act/scripts/act_maxdb_lvm_restore.conf
cat /act/scripts/act_maxdb_lvm_singledb_restore_config.conf > $MAXDBRESTORECONFIG
LOG_BKP_LOC=$DB_FREEZEDATE_FILE_LOCATION

echo -e "LOG_BKP_LOC='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG
#echo -e "MANIFEST_FILE_LOC='/$MANIFEST_FILE_LOC'" >> $SYBASERESTORECONFIG
echo -e "LOG_RECOVERY='$LOG_RECOVERY'" >> $MAXDBRESTORECONFIG
echo -e "TARGET_DBNAME_KEY_PREFIX='$TARGET_DBNAME_KEY'" >> $MAXDBRESTORECONFIG
echo -e "DB_FREEZEDATE_FILE_LOCATION='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG
echo -e "UNTIL_TIME='$UNTIL_TIME'" >> $MAXDBRESTORECONFIG
echo -e "TARGET_MNT_PNT='$TARGET_MNT_PNT'" >> $MAXDBRESTORECONFIG
echo -e "TARGET_DBNAME='$DBSID'" >> $MAXDBRESTORECONFIG
echo -e "SOURCESID='$SOURCESID'" >> $MAXDBRESTORECONFIG

chmod -R 755 $MAXDBRESTORECONFIG

#Recovery database if needed
if [ "$LOG_RECOVERY" = "YES" ]; then

  echo "creating MaxDB log backup Recovery template *********"
  RECOVERY_TEMPLATE_SUFFIX="actrecoveryTemplate"
  echo -e "RECOVERY_TEMPLATE='$RECOVERY_TEMPLATE_SUFFIX'" >> $MAXDBRESTORECONFIG
  LOG_BACKUP_FILE="LOGBACKUP"

  echo "Creating MaxDB log recovery template *********"

  log_backup_template=$DBSID"_"$RECOVERY_TEMPLATE_SUFFIX
  fileLocation=$LOG_BKP_LOC/$SOURCESID"_"$LOG_BACKUP_FILE
  echo "creating MaxDB log template for $dbname *********"
  bkpt=`su - $OSUSER -c "$dbmcli -d $DBSID -U $TARGET_DBNAME_KEY backup_template_create $log_backup_template to FILE $fileLocation CONTENT LOG"`
  retval=$?
  if [ $retval -eq 0 ]; then
      echo "created MaxDB log template for $DBSID *********"
  else
     echo "log template creation failed for $DBSID ... $bkpt ******"
  exit $retval
  fi


  echo "Successfully created MaxDB log recovery template *********"


  su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY db_admin"
  retval=$?


  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to set database in admin mode"
    exit 1
  fi

  cp -r /act/act_scripts/maxdb/automation/Restore/ACT_MAXDB_Diff_Host_RESTORE_recovery.sh /act/scripts/
  chmod -R 755 /act/scripts/ACT_MAXDB_Diff_Host_RESTORE_recovery.sh
  sh /act/scripts/ACT_MAXDB_Diff_Host_RESTORE_recovery.sh
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to recovery databae "
    exit 1
  fi
fi


# Bring the DB online

su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY db_online"
retval=$?


if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Database Instance Failed to start"

#Updating the Target server with old configuration

#while read line
#do
#  vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
#  trg_file_name=`echo $line | grep "VolumeName" | awk '{ print $2}'`
# su - $OSUSER -c "$dbmcli -d $TARGET_SERVER_NAME -U $TARGET_DBNAME_KEY param_put $vol_name $trg_file_name"
  #Update_data_log_loc $vol_name $trg_file_name
#retval=$?

#  if [ $retval -ne 0 ]; then
#    echo " Parameter file update failed for Data and Log Location for Target server with old configuration"
#    exit 1
#  fi
#  done < $TRG_VOLINFO1
  exit 1
fi

exit 0

