#!/bin/sh
set -x
conffile=/act/scripts/act_maxdb_lvm_restore.conf

if [ -f $conffile ]; then
 source $conffile
else
 echo "ERROR: Configuration file for Log Recovery does not exist!."
fi

LOG_BKP_MNTPT="$LOG_BKP_LOC"
dbmcli=$TARGET_MAXDB_DBMCLI_PATH
SRC_DBNAME_KEY_PREFIX=$TARGET_DBNAME_KEY_PREFIX
SRC_DBNAME=$TARGET_DBNAME

tmp_loc=/act/touch
chmod 775 $tmp_loc
OSGROUP=`id -ngr $OSUSER | cut -d " " -f2`
chown -R $OSUSER:$OSGROUP $tmp_loc

#Get the recovery time from Manifest file location
if [ ! -z "$LOG_BKP_MNTPT" ]
then
    if [ ! -z "$UNTIL_TIME" ]
    then
        recovery_from=$(cat "$DB_FREEZEDATE_FILE_LOCATION"/dbfreezedate.dat)
        recoverytime=$(date -d "$UNTIL_TIME" +'%b %d %Y %H:%M:%S')
        maxdb_until_time=$(echo "$UNTIL_TIME" | sed -e 's/-//g' -e 's/://g')
        UNTIL_TIME=$(date -d "$UNTIL_TIME" +'%Y%m%d%H%M%S')
    fi
fi

echo "$UNTIL_TIME"
################Identify the log backup files needed for recovery#######################
if [ ! -z "$UNTIL_TIME" ]
then
    if [ "$UNTIL_TIME" -gt "$recovery_from" ]
    then
        rollforwardtime=$(date -d "${UNTIL_TIME:0:8} ${UNTIL_TIME:8:2}:${UNTIL_TIME:10:2}:${UNTIL_TIME:12:2}" +"%Y-%m-%d %H:%M:%S")
        echo "rollforward:$rollforwardtime"
    		for i in $(echo $TARGET_DBNAME | sed "s/,/ /g")
        do
    	    dbname=$i

    			recovery_file="$tmp_loc/$dbname"_"maxdbrestore.txt"
    			LOG_RECOVERY_TEMPLATE=$dbname"_"$RECOVERY_TEMPLATE

    			# Remove the recovery file if it exists
    			if [ -f $recovery_file ]
    			then
    			  rm -f $recovery_file
    			fi
          output=`su - $OSUSER -c "$dbmcli -d $dbname -U $SRC_DBNAME_KEY_PREFIX db_admin"`
          retval=$?
          if [ $retval -eq 0 ]; then
              echo "Db $dbname is now in Admin Mode *********"
          else
             echo "Error: Admin mode Failed for $dbname .. $output ******"
             exit $retval
          fi

    			LOGFILENAME=$SOURCESID"_"LOGBACKUP
    			logbackupfiles=($(find "$LOG_BKP_MNTPT"/"$LOGFILENAME"* -type f -newer "$DB_FREEZEDATE_FILE_LOCATION"/dbfreezedate.dat ! -newermt "$rollforwardtime"))
    			len=$(echo "${#logbackupfiles[@]}")

    			# Timestamp of last file present in the directory
    			lastlogtstamp=$(ls -lrt "$LOG_BKP_MNTPT"/"$LOGFILENAME"* | tail -1 | awk '{print $6,$7,$8}')

    			lastlog=$(find "$LOG_BKP_MNTPT"/"$LOGFILENAME"* -type f -newermt "$rollforwardtime" | sort -n | head -1)

    			# There are no log files + no lastlog, just bring the DB online.

    			if [ "$len" == 0 ] && [ -z "$lastlog" ]
    			then
    				echo 'db_connect' >> $recovery_file
    				echo 'db_execute clear log' >> $recovery_file
    				echo 'db_online' >> $recovery_file
    			else
    				# Add the dbmcli commands required for recovery to the recovery file
    				# db_connect - open a session with the database
    				# db_execute clear log - reset of the log volume concatenation

    				echo 'db_connect' >> $recovery_file
    				echo 'db_execute clear log' >> $recovery_file

    				for (( i=0; i<$len; i++ ));
    				do
              recoverCheckLabel=`echo ${logbackupfiles[$i]} |  awk -F"." '{print $NF}'`
    					echo ${logbackupfiles[$i]} | tr '.' ' ' | awk -v out_file=$recovery_file -v logtemplate="$LOG_RECOVERY_TEMPLATE" -v i=$i -v until_time="$maxdb_until_time"        '{
    					if (i == 0)
    						print "recover_start "logtemplate" log "$2" until "until_time" " >> out_file
    					else
    						print "recover_replace "logtemplate" "$1" "$2" " >> out_file
    					close(out_file)
    				}'
    				done

    				# lastlog is available, but there are no other log files present.
    				# Run a recover_start on lastlog, then recover_ignore which should also bring the DB online

    				if [ ! -z "$lastlog" ] && [ "$len" == 0 ];
    				then
    					$(echo "$lastlog" | tr '.' ' ' | awk -v logtemplate="$LOG_RECOVERY_TEMPLATE" -v until_time="$maxdb_until_time" '{print "recover_start "logtemplate" log "$2" until "until_time" "}' >> $recovery_file)
              recoverCheckLabel=`echo $lastlog |  awk -F"." '{print $NF}'`
    				# logs + lastlog present - Add the recover_replace command to process the lastlog file

    				elif [ ! -z "$lastlog" ]; then
    					$(echo "$lastlog" | tr '.' ' ' | awk -v logtemplate="$LOG_RECOVERY_TEMPLATE" '{print "recover_replace "logtemplate" "$1" "$2" "}' >> $recovery_file)
              recoverCheckLabel=`echo $lastlog |  awk -F"." '{print $NF}'`
    				fi

            echo 'recover_ignore' >> $recovery_file
            echo "db_online" >> $recovery_file
    			fi
    			# Change permissions for restore.txt file so that it can be used with dbmcli
          chown -R $OSUSER:$OSGROUP $recovery_file
    			# Restore DB using dbmcli + recovery_file
    			recovery_cmd=$(su - "$OSUSER" -c "$dbmcli -d $dbname -U $SRC_DBNAME_KEY_PREFIX -ic $recovery_file")
          output=`su - $OSUSER -c "$dbmcli -d $dbname -U $SRC_DBNAME_KEY_PREFIX db_online"`
          retval=$?
    			if [ $retval -eq 0 ]
    			then
    				echo "MaxDB log recovery completed for $dbname"
    			else
    				echo "MaxDB log recovery failed : $recovery_cmd for $dbname"
    				exit $retval
    			fi
    		done
    else
        echo "ERROR: UNTIL TIME: $UNTIL_TIME is less than Recovery Time: $recovery_from"
        exit 1
    fi
fi

exit 0

