#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

conffile="/act/scripts/act_maxdb_lvm_singledb_restore_config.conf"

echo -e '#Max DB/Logbackup Specific Params' > $conffile
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   passwdstrCheck=`echo $ARGUMENT | grep -i "password\|passwd"`
   if [ -z "$passwdstrCheck" ]; then
     echo -e $ARGUMENT >> $conffile
   else
     export $ARGUMENT
   fi
done

source $conffile
ACTIFIO_UNIQUE="ACTIFIO_UNIQUE"
DBSID=$DBSID
OSUSER=$OSUSER

dbmcli=$TARGET_MAXDB_DBMCLI_PATH

#export PATH=$LOCAL_PRG_PATH:$PATH

if [ X"$DBSID" = "X" ]; then
  echo "ERRORMSG: Please provide DBSID"
  exit 1
fi

if [ X"$TARGET_DBNAME_KEY" = "X" ]; then
  echo "ERRORMSG: Please provide Source DB User Name"
  exit 1
fi



#Convert the DBSID in Upper case

DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`


#check the password authentication
  set +x
  con=`su - $OSUSER -c "$dbmcli -d $DBSID -U $TARGET_DBNAME_KEY bye | sed 's/$/$ACTIFIO_UNIQUE/'" | grep $ACTIFIO_UNIQUE | sed -e "s/$ACTIFIO_UNIQUE//g" | grep -w "Error! Connection failed" | wc -l`
  set -x

  if [ $con -gt 0 ]; then
      echo "ERRORMSG: User Authentication Failed for $DBSID Database. Please check username and password"
      exit 1
  fi



#Stop Database
db_stop()
{
  set +x
  su - $OSUSER -c "$dbmcli -d $DBSID -U $TARGET_DBNAME_KEY db_offline"
  retval=$?
  set -x

  if [ $retval -eq 0 ]; then
    echo " Database $DBSID stopped successfully"
  else
    echo "ERRORMSG: Database $DBSID stop Failed "
    exit 1
  fi

}

#Check The DB State
check_db_state()
{
  set +x
  DB_STATUS=`su - $OSUSER -c "$dbmcli -d $DBSID -U $TARGET_DBNAME_KEY db_state | sed 's/$/$ACTIFIO_UNIQUE/'" | grep $ACTIFIO_UNIQUE | sed -e "s/$ACTIFIO_UNIQUE//g" | grep -v "OK" | grep -v "State"`
  retval=$?
  set -x

  if [ $retval -eq 0 ]; then
    echo " Database $DBSID state check succeeded"
  else
    echo "ERRORMSG: Database $DBSID state check  Failed "
    exit 1
  fi

}


check_db_state

if [ "$DB_STATUS"  = "ONLINE" ] || [ "$DB_STATUS"  = "ADMIN" ]; then
  echo "Stopping Database ..."
  db_stop
  echo "Database stopped....."
elif  [ "$DB_STATUS"  = "OFFLINE" ]; then
  echo "DB is already shutdown...."
else
  echo "ERRORMSG: DB is unconsistant state.....Check the DB Log"
  exit 1
fi



exit 0

