#!/bin/sh
set +x
conffile=/act/scripts/act_maxdb_dump_restore.conf

if [ -f $conffile ]; then
 source $conffile
else
 echo "ERROR: Configuration file for Log Recovery does not exist!."
 exit 1
fi
set -x

LOG_BKP_MNTPT="$LOG_BKP_LOC"

source /act/act_scripts/maxdb/automation/Backup/dbHelper.sh

tmp_loc=/act/touch
if [ ! -d $tmp_loc ]; then
 mkdir -p $tmp_loc
fi
chmod 775 $tmp_loc
OSGROUP=`id -ngr $OSUSER | cut -d" " -f1`
chown -R $OSUSER:$OSGROUP $tmp_loc

#Get the recovery time from Manifest file location
if [ ! -z "$LOG_BKP_MNTPT" ]
then
    if [ ! -z "$UNTIL_TIME" ]
    then
        recovery_from=$(cat "$DB_FREEZEDATE_FILE_LOCATION"/dbfreezedate.dat)
        recoverytime=$(date -d "$UNTIL_TIME" +'%b %d %Y %H:%M:%S')
        maxdb_until_time=$(echo "$UNTIL_TIME" | sed -e 's/-//g' -e 's/://g')
        UNTIL_TIME=$(date -d "$UNTIL_TIME" +'%Y%m%d%H%M%S')
        if [ "$UNTIL_TIME" -gt "$recovery_from" ]
        then
            rollforwardtime=$(date -d "${UNTIL_TIME:0:8} ${UNTIL_TIME:8:2}:${UNTIL_TIME:10:2}:${UNTIL_TIME:12:2}" +"%Y-%m-%d %H:%M:%S")
            echo "rollforward:$rollforwardtime"
        else
            echo "ERROR: UNTIL TIME: $UNTIL_TIME is less than Recovery Time: $recovery_from"
            echo "******** RESTORE_STATUS=$dbname:fail  *********"
            exit 1
        fi
    fi
fi

dumpthreadcount=`ls $DB_BACKUP_LOCATION | grep -i "fullDataBackup" | awk -F"fullDataBackup" '{print $NF}' | sort | uniq | wc -l`

RECOVERY_FULL_TEMPLATE_SUFFIX=actFullRecoveryDbTemplate
echo "creating MaxDB full data recovery template *********"
createCompleteMultiDataTemplate $DB_BACKUP_LOCATION $RECOVERY_FULL_TEMPLATE_SUFFIX $dumpthreadcount
echo "Done Creating MaxDB full data recovery template *********"

RECOVERY_INCR_TEMPLATE_SUFFIX=actIncrRecoveryDbTemplate
echo "creating MaxDB Incremental data recovery template *********"
createIncrementMultiDataTemplate $DB_BACKUP_LOCATION $RECOVERY_INCR_TEMPLATE_SUFFIX $dumpthreadcount
echo "Done Creating MaxDB Incremental data recovery template *********"

if [ ! -z "$LOG_BKP_MNTPT" ] && [ "$LOG_RECOVERY" = "YES" ]; then
  RECOVERY_TEMPLATE_SUFFIX="actrecoveryTemplate"
  echo "creating MaxDB log backup Recovery template *********"
  LOG_BACKUP_FILE="LOGBACKUP"

  echo "Creating MaxDB log recovery template *********"
  createLogTemplate $LOG_BKP_MNTPT $RECOVERY_TEMPLATE_SUFFIX $LOG_BACKUP_FILE
  echo "Successfully created MaxDB log recovery template *********"
fi

for i in $(echo $TARGET_DBNAME | sed "s/,/ /g")
do
  loopdbname=$i
  dbname=`echo $loopdbname | cut -d ':' -f 1`
  srcdbname=`echo $loopdbname | cut -d ':' -f 2`
  CHECK_SRC_BKP_EXISTS=`ls $DB_BACKUP_LOCATION | grep -i "$srcdbname"_"fullDataBackup" | wc -l`
  if [ "$CHECK_SRC_BKP_EXISTS" -gt "0" ]; then
    :
  else
    echo "Error: Backup does not exist for source database:$srcdbname"
    echo "******** RESTORE_STATUS=$dbname:fail  *********"
    exit 1
  fi
	dbmcli=`echo $TARGET_MAXDB_DBMCLI_PATH | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`/dbmcli
	if [ -z "$dbmcli" ]; then
		 dbmcli=$TARGET_MAXDB_DBMCLI_PATH/dbmcli
	fi
  if [ ! -f $dbmcli ]; then
    echo "Error: $dbmcli does not exist"
    echo "******** RESTORE_STATUS=$dbname:fail  *********"
    exit 1
  fi
  xuserkey=`echo $TARGET_DBNAME_KEY_PREFIX | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
  if [ -z $xuserkey ]; then
   xuserkey=$TARGET_DBNAME_KEY_PREFIX
  fi
	recovery_file="$tmp_loc/$dbname"_"maxdbrestore.txt"

	# Remove the recovery file if it exists
	if [ -f $recovery_file ]
	then
	  rm -f $recovery_file
	fi

  #Do Database Full Recovery
  touch /act/touch/DB_MESSAGE
  chmod 777 /act/touch/DB_MESSAGE
  su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_admin > /act/touch/DB_MESSAGE" > /dev/null 2>&1
  retval=$?
  if [ $retval -eq 0 ]; then
      echo "Db $dbname is now in Admin Mode *********"
  else
     echo "Error: Admin mode Failed for $dbname .. `cat /act/touch/DB_MESSAGE` ******"
     echo "******** RESTORE_STATUS=$dbname:fail  *********"
     exit $retval
  fi

  DB_FULL_RECOVERY_TEMPLATE=$dbname"_"$RECOVERY_FULL_TEMPLATE_SUFFIX
  echo 'db_connect' >> $recovery_file
  echo "db_activate recover $DB_FULL_RECOVERY_TEMPLATE" >> $recovery_file
  # Test with recover_check for any errors
  touch /act/touch/RECOVERY_MESSAGE
  chmod 777 /act/touch/RECOVERY_MESSAGE
  chown -R $OSUSER:$OSGROUP /act/touch/RECOVERY_MESSAGE

su - $OSUSER -c "$dbmcli -o /act/touch/RECOVERY_MESSAGE -d $dbname -U $xuserkey" << EOF
service_connect
recover_check $DB_FULL_RECOVERY_TEMPLATE
service_release
EOF
  retval=`cat /act/touch/RECOVERY_MESSAGE | grep -w "ERR"`
	if [ ! -z "$retval" ]; then
    echo "******* ERROR: Failed recovery check for db full template for:$dbname. `cat /act/touch/RECOVERY_MESSAGE` ************"
    echo "******** RESTORE_STATUS=$dbname:fail  *********"
    exit 1
	fi
	#returncode=`cat /act/touch/RECOVERY_MESSAGE | awk '{sub(/^.*Returncode/,""); print $1}'`
  echo 'recover_ignore' >> $recovery_file
  #Do Database Incremental Recovery if available
  CHECK_INCR_BKP=`ls $DB_BACKUP_LOCATION | grep -i "$srcdbname"_"incrDataBackup" | wc -l`
  if [ "$CHECK_INCR_BKP" -gt "0" ]; then
    DB_INCR_RECOVERY_TEMPLATE=$dbname"_"$RECOVERY_INCR_TEMPLATE_SUFFIX
    echo "recover_start $DB_INCR_RECOVERY_TEMPLATE" >> $recovery_file
    # Test with recover_check for any errors
su - $OSUSER -c "$dbmcli -o /act/touch/RECOVERY_MESSAGE -d $dbname -U $xuserkey" << EOF
service_connect
recover_check $DB_INCR_RECOVERY_TEMPLATE
service_release
EOF
    retval=`cat /act/touch/RECOVERY_MESSAGE | grep -w "ERR"`
  	if [ ! -z "$retval" ]; then
      echo "******* ERROR: Failed recovery check for db incr template for:$dbname. `cat /act/touch/RECOVERY_MESSAGE` ************"
      echo "******** RESTORE_STATUS=$dbname:fail  *********"
      exit 1
  	fi
    #returncode=`cat /act/touch/RECOVERY_MESSAGE | awk '{sub(/^.*Returncode/,""); print $1}'`
  	echo 'recover_ignore' >> $recovery_file
  fi
	# Check for Log Recovery
	if [ ! -z "$LOG_BKP_MNTPT" ] && [ "$LOG_RECOVERY" = "YES" ]; then
		LOG_RECOVERY_TEMPLATE=$dbname"_"$RECOVERY_TEMPLATE_SUFFIX
		LOGFILENAME=$srcdbname"_"LOGBACKUP

		logbackupfiles=($(find "$LOG_BKP_MNTPT"/"$LOGFILENAME"* -type f -newer "$DB_FREEZEDATE_FILE_LOCATION"/dbfreezedate.dat ! -newermt "$rollforwardtime"))
		len=$(echo "${#logbackupfiles[@]}")

		# Timestamp of last file present in the directory
		lastlogtstamp=$(ls -lrt "$LOG_BKP_MNTPT"/"$LOGFILENAME"* | tail -1 | awk '{print $6,$7,$8}')

		lastlog=$(find "$LOG_BKP_MNTPT"/"$LOGFILENAME"* -type f -newermt "$rollforwardtime" | sort -n | head -1)

		# There are no log files + no lastlog, just bring the DB online.

		if [ "$len" == 0 ] && [ -z "$lastlog" ]
		then
			echo 'db_online' >> $recovery_file
		else
			# Add the dbmcli commands required for recovery to the recovery file

			for (( i=0; i<$len; i++ ));
			do
        recoverCheckLabel=`echo ${logbackupfiles[$i]} |  awk -F"." '{print $NF}'`
				echo ${logbackupfiles[$i]} | tr '.' ' ' | awk -v out_file=$recovery_file -v logtemplate="$LOG_RECOVERY_TEMPLATE" -v i=$i -v until_time="$maxdb_until_time"        '{
				if (i == 0)
					print "recover_start "logtemplate" log "$2" until "until_time" " >> out_file
				else
					print "recover_replace "logtemplate" "$1" "$2" " >> out_file
				close(out_file)
			}'
			done

			# lastlog is available, but there are no other log files present.
			# Run a recover_start on lastlog, then recover_ignore which should also bring the DB online

			if [ ! -z "$lastlog" ] && [ "$len" == 0 ];
			then
				$(echo "$lastlog" | tr '.' ' ' | awk -v logtemplate="$LOG_RECOVERY_TEMPLATE" -v until_time="$maxdb_until_time" '{print "recover_start "logtemplate" log "$2" until "until_time" "}' >> $recovery_file)
        recoverCheckLabel=`echo $lastlog |  awk -F"." '{print $NF}'`
			# logs + lastlog present - Add the recover_replace command to process the lastlog file

			elif [ ! -z "$lastlog" ]; then
				$(echo "$lastlog" | tr '.' ' ' | awk -v logtemplate="$LOG_RECOVERY_TEMPLATE" '{print "recover_replace "logtemplate" "$1" "$2" "}' >> $recovery_file)
        recoverCheckLabel=`echo $lastlog | awk -F"." '{print $NF}'`
			fi

# Decide to do recover_ignore if Return code of recover_check is not 0
su - $OSUSER -c "$dbmcli -o /act/touch/RECOVERY_MESSAGE -d $dbname -U $xuserkey" << EOF
service_connect
recover_check $LOG_RECOVERY_TEMPLATE $recoverCheckLabel
service_release
EOF
      retval=`cat /act/touch/RECOVERY_MESSAGE | grep -w "ERR"`
			if [ ! -z "$retval" ]; then
			  echo "******* ERROR: Failed recovery check for log template for :$dbname. `cat /act/touch/RECOVERY_MESSAGE` ************"
        echo "******** RESTORE_STATUS=$dbname:fail  *********"
        exit 1
			fi
			#returncode=`cat /act/touch/RECOVERY_MESSAGE | awk '{sub(/^.*Returncode/,""); print $1}'`
			echo 'recover_ignore' >> $recovery_file
			echo "db_online" >> $recovery_file
		fi
	else
		echo "No Logs to Apply!!!"
		echo "db_online" >> $recovery_file
	fi


	# Change permissions for restore.txt file so that it can be used with dbmcli
	chown -R $OSUSER:$OSGROUP $recovery_file
	# Restore DB using dbmcli + recovery_file
	su - "$OSUSER" -c "$dbmcli -o /act/touch/RECOVERY_MESSAGE -d $dbname -U $xuserkey -ic $recovery_file"
  su - $OSUSER -c "$dbmcli -d $dbname -U $xuserkey db_online > /act/touch/DB_MESSAGE" > /dev/null 2>&1
	retval=$?
  tail -20 /act/touch/RECOVERY_MESSAGE
  cat /act/touch/RECOVERY_MESSAGE >> /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
  cp -f $recovery_file /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
	if [ $retval -eq 0 ]
	then
		echo "MaxDB FileSystem recovery completed for $dbname"
    echo "******** RESTORE_STATUS=$dbname:success  *********"
	else
		echo "MaxDB FileSystem recovery failed : `cat /act/touch/RECOVERY_MESSAGE` for $dbname"
    echo "******** RESTORE_STATUS=$dbname:fail  *********"
		exit $retval
	fi
done

exit 0
