#!/bin/sh
source /act/act_scripts/maxdb/automation/Backup/dbHelper.sh

echo "Image: $dbimagename Maxdb DB MountPath: $TARGET_MNT_PNT"
mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

if [ -z $TARGET_MNT_PNT ]; then
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true"
     }'
else
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true",
      "restoreoption":"'"mountpointperimage=$TARGET_MNT_PNT"'"
     }'
fi

mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
  echo "**** ERRMSG: DB jobID cannot be null: $mountResponse ****"
  exit 1
fi

jobinprogress=true
errorMessage=""
while $jobinprogress
    do
        sleep 30
        getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
        status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
        if [ ! -z "$status" ]; then
                if [ $status == "succeeded" -o $status == "failed" ]
                then
                        jobinprogress=false
                        if [ $status == "failed" ]
                        then
                                getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                        fi
                fi
              else
                jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                echo "Job is still running..Completed $status%"
        fi
done
printf "\n The Maxdb DB Mount job $jobID has $status.\n"

if [ ! -z "$errorMessage" ]; then
  printf "\n Reason: $errorMessage.\n"
  exit 1
fi

if [ -z $TARGET_MNT_PNT ]; then
  TARGET_MNT_PNT=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
fi

if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then

  if [ -z "$UNTIL_TIME" ]; then
    printf "\n Reason: Recovery Time has not been set for logBackup Image\n"
    exit 1
  fi
  LOG_RECOVERY="YES"
  lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"

  filterValue="filtervalue=backupname=$logimagename"
  backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
  appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
  policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
  LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')

  printf "Mounting Data from Log Backup App Image"

  echo "Image: $logimagename MOUNTPATH: $LOG_BKP_LOC"
  mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

  if [ -z $LOG_BKP_LOC ]; then
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true"
       }'
  else
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true",
        "restoreoption":"'"mountpointperimage=$LOG_BKP_LOC"'"
       }'
  fi

  mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
  jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    echo "**** ERRMSG: Log jobID cannot be null: $mountResponse ****"
    exit 1
  fi
  jobinprogress=true
  errorMessage=""
  while $jobinprogress
      do
          sleep 30
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
  printf "\n The Log Backup Mount job $jobID has $status.\n"

  if [ ! -z "$errorMessage" ]; then
    printf "\n Reason: $errorMessage.\n"
    exit 1
  fi
  if [ -z $LOG_BKP_LOC ]; then
    LOG_BKP_LOC=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
  fi

  if [ ! -z $LOGBACKUPPATH ]; then
    logbackupSuffix="${LOGBACKUPPATH#$LogBackupFS}"
    echo "LogbackupSuffix: $logbackupSuffix"
    LOG_BKP_LOC=$LOG_BKP_LOC/$logbackupSuffix
    echo "Log Backup Updated Path: $LOG_BKP_LOC"
  fi

  DB_FREEZEDATE_FILE_LOCATION=$(find $LOG_BKP_LOC/ -iname dbfreezedate.dat | rev | cut -d"/" -f2-  | rev)
  if [ -z $DB_FREEZEDATE_FILE_LOCATION ]; then
    echo "Error: Cannot Find the File dbfreezedate.dat which contains DB Freeze Date"
    exit 1
  fi

fi
set -x
MAXDBRESTORECONFIG=/act/scripts/act_maxdb_dump_restore.conf
cat /act/act_scripts/maxdb/automation/Configuration/maxdbrestore.conf > $MAXDBRESTORECONFIG
echo -e "LOG_BKP_LOC='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG
echo -e "LOG_RECOVERY='$LOG_RECOVERY'" >> $MAXDBRESTORECONFIG
echo -e "DB_BACKUP_LOCATION='$TARGET_MNT_PNT'" >> $MAXDBRESTORECONFIG

if [ "$LOG_RECOVERY" = "YES" ]; then
  echo -e "DB_FREEZEDATE_FILE_LOCATION='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG
fi

OSGROUP=`id -ngr $OSUSER | cut -d" " -f1`
chmod -R 777 $TARGET_MNT_PNT
chown -R $OSUSER:$OSGROUP $TARGET_MNT_PNT
if [ ! -z $LOG_BKP_LOC ]; then
  chmod -R 777 $LOG_BKP_LOC
  chown -R $OSUSER:$OSGROUP $LOG_BKP_LOC
fi

touch /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
chmod 777 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
touch /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
chmod 777 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log

cp -r /act/act_scripts/maxdb/automation/Restore/act_maxdb_dump_recovery.sh /act/scripts/
chmod -R 755 /act/scripts/act_maxdb_dump_recovery.sh

# Execute the Dump Recovery Script.
sh /act/scripts/act_maxdb_dump_recovery.sh
retval=$?
chmod 644 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
chmod 644 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
if [ $retval -gt 0 ]; then
  echo "ERROR: Dump Recovery Failed!!! Exiting"
  exit 1
fi

conffile=/act/scripts/act_MaxDB_src.conf
if [ -f $conffile ]; then
  skyIPmatch=`grep -i $SkyIP $conffile`
  retval=$?
  if [ $retval -eq 0 ]
  then
   source $conffile
  fi
 if [ ! -z "$ACTIFIO_DISK_LOC" ]; then
   #echo "Check if Actifio Disk has enough space for new db backup"
   #source /act/act_scripts/maxdb/automation/Backup/addVdisk.sh $ACTIFIO_DISK_LOC $hostID
   #dbTemplateSuffix=actfullDBTemplate
   #createCompleteMultiDataTemplate $ACTIFIO_DISK_LOC $dbTemplateSuffix $dumpthreadcount
   echo "**** Delete old databackup Files in Staging Disk ****"
   FILES=`ls -lr $ACTIFIO_DISK_LOC | grep -i DataBackup | awk -F" " '{print $NF}'`
   # Delete the files
   for i in `echo $FILES`; do
    rm -f $ACTIFIO_DISK_LOC/$i
   done
   #echo "********** Take a fresh maxdb full backup after restore ************"
   #executeDbTemplate $dbTemplateSuffix
   #echo "**********Done with maxdb full backup after restore ************"
 fi
else
 echo "Backup Configuration File does not Exist..Hence not taking a fresh maxdb full backup post Restore!"
fi

set +x
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x

#Unmount the DB Image
filterValue="filtervalue=jobclass=mount&appname=$GenericAppName&apptype=FileSystem&mountedhost=$hostID"
mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

mountImageIDListLength=${#mountImageIDList[@]}
echo "Existing $GenericAppName mounts for MaxDB are:"  $mountImageIDListLength

if [ $mountImageIDListLength -gt 0 ]; then
   for (( i=0; i<${mountImageIDListLength}; i++ ))
   do
   # Unmount and Delete Image
   unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

    inputParamsForUnmount='{
    "sessionid":"'"$sessionid"'",
    "image":"'"${mountImageIDList[i]}"'",
    "delete":"true"
   }'

   unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
   if [[ $unmountResponse == *errormessage* ]]; then
      echo "Unmount Failed. Reason :" $unmountResponse
   elif [[ $unmountResponse == *result* ]]; then
      echo "DB App Unmounted successfully."
   fi
  done
fi

# Unmount the Log Images as Restoration of Log Transactions are complete.
if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
	  filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
    mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

    mountImageIDListLength=${#mountImageIDList[@]}
    echo "Existing mounts for logBackup are:"  $mountImageIDListLength

    if [ $mountImageIDListLength -gt 0 ]; then
       for (( i=0; i<${mountImageIDListLength}; i++ ))
       do
       # Unmount and Delete Image
       unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

        inputParamsForUnmount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"${mountImageIDList[i]}"'",
        "delete":"true"
       }'

       unmountResponse=$(curl -s -w "\n" -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
       if [[ $unmountResponse == *errormessage* ]]; then
          echo "Unmount Failed. Reason :" $unmountResponse
       elif [[ $unmountResponse == *result* ]]; then
          echo "Logbackup App Unmounted successfully."
       fi
      done
    fi
fi
