#!/bin/sh
########################################################################################################################################################
# Script Version# - Version 1.0
# --------- Below are the high level steps performed:-----
# Disable Schedule For DB Application.
# Disable Schedule for Log Application.
# Bring the MaxDB Offline.
# Mount the required log image to apply Log Transactions.
# Restore the Data and Log Volumes from the Generic App Snapshot Image.
# Create a Recovery Log Template to recover from Iscsi Mounted LogBackups.
# Get the Location of File containing last DB incremental Timestamp to pick Log Files after that.
# Execute the Log Recovery Script.
# Bring the DB online after DB Restore as there are no logs to recover From.
# Create a DB backup Template to run a complete data backup post Restore.
# Unmount the Log Images as Restoration of Log Transactions are complete.
# Enable the DB schedule back.
# Enable the Log Schedule back.
########################################################################################################################################################

restoreConfFile="/act/act_scripts/maxdb/automation/Configuration/maxdbrestore.conf"
MAXDBRESTORECONFIG=/act/scripts/act_maxdb_lvm_restore.conf

cat /act/act_scripts/maxdb/automation/Configuration/maxdbrestore.conf > /act/scripts/act_maxdb_lvm_restore.conf

# Bring the MaxDB Offline.
source /act/act_scripts/maxdb/automation/Backup/dbHelper.sh
makeDbOffline
sleep 10

# Mount the required log image to apply Log Transactions.
  if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
    if [ -z "$UNTIL_TIME" ]; then
      printf "\n Reason: Recovery Time has not been set for logBackup Image\n"
      exit 1
    fi

    LOG_RECOVERY="YES"
    BACKUPTYPE="INCREMENTAL"
    lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
    filterValue="filtervalue=backupname=$logimagename"
    backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
    appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
    policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
    LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')

    printf "Mounting Log Backup Image"

    echo "Image: $logimagename RESTOREPATH: $LOG_BKP_LOC"
    mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

    if [ -z $LOGRESTOREPATH ]; then
      inputParamsForMount='{
          "sessionid":"'"$sessionid"'",
          "image":"'"$logimagename"'",
          "host":"'"$hostID"'",
          "nowait":"true"
         }'
     else
       inputParamsForMount='{
           "sessionid":"'"$sessionid"'",
           "image":"'"$logimagename"'",
           "host":"'"$hostID"'",
           "nowait":"true",
           "restoreoption":"'"mountpointperimage=$LOG_BKP_LOC"'"
          }'
     fi

     mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
     jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
     if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
       echo "**** ERRMSG: Log jobID cannot be null: $mountResponse ****"
       exit 1
     fi
     echo "Log Job id is: $jobID "

    jobinprogress=true
    errorMessage=""
    set +x
     while $jobinprogress
        do
            sleep 30
            getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
            status=$(curl -Ss -w "\n" -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
            if [ ! -z "$status" ]; then
                    if [ $status == "succeeded" -o $status == "failed" ]
                    then
                            jobinprogress=false
                            if [ $status == "failed" ]
                            then
                                    getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                    errorMessage=$(curl -Ss -w "\n"  -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                            fi
                    fi
                  else
                    jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                    status=$(curl -Ss -w "\n" -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                    echo "Job is still running..Completed $status%"
            fi
    done
    set -x
    printf "\n The Log Backup Mount job $jobID has $status.\n"

    if [ ! -z "$errorMessage" ]; then
      printf "\n Reason: $errorMessage.\n"
      exit 1
    fi

    if [ -z $LOG_BKP_LOC ]; then
      LOG_BKP_LOC=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
    fi

    OSGROUP=`id -ngr $OSUSER | cut -d" " -f1`
    chmod -R 777 $LOG_BKP_LOC
    chown -R $OSUSER:$OSGROUP $LOG_BKP_LOC

    if [ "$BACKUPTYPE" = "INCREMENTAL" ]; then
      if [ ! -z $LOGBACKUPPATH ]; then
        logbackupSuffix="${LOGBACKUPPATH#$LogBackupFS}"
        echo "LogbackupSuffix: $logbackupSuffix"
        LOG_BKP_LOC=$LOG_BKP_LOC/$logbackupSuffix
        echo "Log Backup Updated Path: $LOG_BKP_LOC"
      fi
    fi

    sleep 10


    DB_FREEZEDATE_FILE_LOCATION=$(find $LOG_BKP_LOC/ -iname dbfreezedate.dat | rev | cut -d"/" -f2-  | rev)
    if [ -z $DB_FREEZEDATE_FILE_LOCATION ]; then
      echo "Error: Cannot Find the File dbfreezedate.dat which contains DB Freeze Date"
      exit 1
    fi

    cp -r /act/act_scripts/maxdb/automation/Restore/act_maxdb_logrecovery.sh /act/scripts/
    chmod -R 755 /act/scripts/act_maxdb_logrecovery.sh

  fi

  # Restore the Data and Log Volumes from the Generic App Snapshot Image.
  restoreURL="https://$SkyIP/actifio/api/task/restoreimage"
  inputParamsForRestore='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "nowait":"true"
    }'

  restoreResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $restoreURL -d "$inputParamsForRestore")
  jobID=`echo "$restoreResponse" | jq -cr '(.result)' | cut -d ' ' -f1`

  sleep 10
  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    jobFilter="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=appname%3D$GenericAppName%26jobclass=restore"
    jobID=$(curl -Ss -w "\n" -k -X GET $jobFilter | jq -cr '(.result []? | .jobname)')
  fi

  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    echo "**** ERRMSG: DB Restore jobID cannot be null: $restoreResponse ****"
    exit 1
  fi
  echo "DB Restore Job id is: $jobID "
  set +x
  jobinprogress=true
  errorMessage=""
  while $jobinprogress
      do
          sleep 60
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
  set -x
  printf "\n The maxdb DB Restore job $jobID has $status.\n"

  if [ ! -z "$errorMessage" ]; then
    printf "\n Reason: $errorMessage.\n"
    exit 1
  fi

  sleep 10

  set -x
  echo -e "LOG_BKP_LOC='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG
  echo -e "LOG_RECOVERY='$LOG_RECOVERY'" >> $MAXDBRESTORECONFIG

# Apply Log Transactions on the DB if a log Transaction is Applicable.
# Create a Recovery Log Template to recover from Iscsi Mounted LogBackups.
if [ "$LOG_RECOVERY" = "YES" ]; then
  echo "creating MaxDB log backup Recovery template *********"
  RECOVERY_TEMPLATE_SUFFIX="actrecoveryTemplate"
  echo -e "RECOVERY_TEMPLATE='$RECOVERY_TEMPLATE_SUFFIX'" >> $MAXDBRESTORECONFIG
  LOG_BACKUP_FILE="LOGBACKUP"

  source /act/act_scripts/maxdb/automation/Backup/dbHelper.sh
  echo "Creating MaxDB log recovery template *********"
  createLogTemplate $LOG_BKP_LOC $RECOVERY_TEMPLATE_SUFFIX $LOG_BACKUP_FILE
  echo "Successfully created MaxDB log recovery template *********"

  echo -e "DB_FREEZEDATE_FILE_LOCATION='$LOG_BKP_LOC'" >> $MAXDBRESTORECONFIG

  touch /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
  chmod 777 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
  touch /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
  chmod 777 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
  # Execute the Log Recovery Script.
  sh /act/scripts/act_maxdb_logrecovery.sh
  retval=$?
  chmod 644 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery.log
  chmod 644 /var/act/log/UDSAgent-"$TARGET_DBNAME"_recovery_sql.log
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: Log Recovery Failed!! Exiting"
    exit 1
  fi
else
# Bring the DB online after DB Restore as there are no logs to recover From.
  makeDbOnline
fi

sleep 10

# Create disk if VDisk does not exist for complete data backup post restore.

vdiskmountpoint="/justForLog"
source /act/act_scripts/maxdb/automation/Backup/addVdisk.sh $vdiskmountpoint $hostID

# Create a DB backup Template to run a complete data backup post Restore.

dbTemplateSuffix=actFullDbTemplate
echo "creating MaxDB data backup template *********"
set +x
createCompleteDataTemplate $vdiskmountpoint $dbTemplateSuffix
set -x
echo "********** Post Recovery Take a complete data backup for Log Backups to happen************"
executeDbTemplate $dbTemplateSuffix

# Get Appliance Token just to avoid session expiry issue
set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x

# Unmount the Log Images as Restoration of Log Transactions are complete.
if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
	  filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
    mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

    mountImageIDListLength=${#mountImageIDList[@]}
    echo "Existing mounts for logBackup are:"  $mountImageIDListLength

    if [ $mountImageIDListLength -gt 0 ]; then
       for (( i=0; i<${mountImageIDListLength}; i++ ))
       do
       # Unmount and Delete Image
       unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

        inputParamsForUnmount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"${mountImageIDList[i]}"'",
        "delete":"true"
       }'

       unmountResponse=$(curl -s -w "\n" -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
       if [[ $unmountResponse == *errormessage* ]]; then
          echo "Unmount Failed. Reason :" $unmountResponse
       elif [[ $unmountResponse == *result* ]]; then
          echo "Logbackup App Unmounted successfully."
       fi
      done
    fi
fi
