#!/bin/sh

set
set -x

echo "****************** PRESK_JOBTYPE $ACT_JOBTYPE ***************"

if [ $ACT_JOBTYPE == "unmount" ] || [ $ACT_MULTI_OPNAME == "unmount" ] ; then
   if [ $ACT_PHASE == "post" ] ; then
        echo "NO-OP for post-script when unmount"
        echo "** no op exit **"
      exit
   fi
   if [ $ACT_LOGSMART_TYPE == "log" ] ; then
	echo "** no op exit **"
     exit
   fi
   if [ $ACT_PHASE == "pre" ] ; then
	echo "************************* start pre during unmount ************************"

conffile=$2

. /act/scripts/$conffile

osuser=$username
ORACLE_SID=$TARGET_SID
target_dbuser=$TARGET_SCHEMA_NAME
export ORACLE_SID
ORACLE_HOME=$orahome
export ORACLE_HOME
PATH=$ORACLE_HOME/bin:$PATH
if [ ! -z "$TARGET_ORACLE_HOME" ]; then
   ORACLE_HOME=$TARGET_ORACLE_HOME
   export ORACLE_HOME
   PATH=$ORACLE_HOME/bin:$PATH
fi

if [ ! -z "$TARGET_PDB_NAME" ]; then
   setcontainer="alter session set container=$TARGET_PDB_NAME;"
fi

multipleuserscount=`echo $target_dbuser | grep "," | wc -l`

for i in $(echo $target_dbuser | tr ',' ' ')
do
  if [ "$multipleuserscount" -gt 0 ]; then
     username=`echo $i | awk -F":" '{print $2}'`
  else
     username=$target_dbuser
  fi

su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
$setcontainer
begin
  for rec in (select distinct owner,table_name,constraint_name from dba_constraints where constraint_type ='R' and owner = upper('$username'))
   loop
        execute immediate 'alter table '||rec.owner||'.'||rec.table_name||' disable constraint '||rec.constraint_name;
   end loop;
end;
/
begin
  for rec in (select distinct owner,table_name,constraint_name from dba_constraints where owner = upper('$username'))
   loop
        execute immediate 'alter table '||rec.owner||'.'||rec.table_name||' disable constraint '||rec.constraint_name;
   end loop;
end;
/
begin
  for rec in (select distinct TABLESPACE_NAME from dba_segments where owner = upper('$username'))
   loop
        execute immediate 'alter tablespace '||rec.tablespace_name||' offline immediate ';
        execute immediate 'drop tablespace '||rec.tablespace_name||' INCLUDING CONTENTS and datafiles CASCADE CONSTRAINTS ';
   end loop;
end;
/
alter session set "_oracle_script"=true;
drop user $username cascade;
exit;
EOF
done
if [ $? -ne 0 ]; then
   echo "failed to drop tablespace"
   exit 1
fi

if [ ! -z "$TARGET_PDB_NAME" ]; then
   if [ -z "$tnsadmindir" ]; then
      tnsadmindir=$ORACLE_HOME/network/admin
   fi
      /act/act_scripts/oracleclone/cleanupTNSNames.sh $TARGET_PDB_NAME $ORACLE_HOME 6 $tnsadmindir
fi

fi
fi

exit $?

