#!/bin/sh
set -x

function get_asm_user {
   gridu=`ps -eo user:20,pid,stime,time,args | grep -i asm_pmon_+asm | grep -vw grep | awk '{print $1}'`
   if [ -z "$gridu" ]; then
      gridu=grid
   fi
   echo $gridu
}

function get_asm_sid {
     dgname=$1
     guser=$2
     get_asmcmd_nocp asmcmd

     dbsids=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"`
     if [ -z "$dbsids" ]; then
        echo "no subdirectory found under $dgname"
        exit 1
     fi

     for x in ${dbsids}; do
         xdgname=$dgname"/"$x
         onlineredo=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $xdgname | grep -i onlinelog"`
         if [ $? -eq 0 ]; then
            #find onlineredolog directory, this is not backup, but appaware folder from previous mount
            asid=`echo $x | cut -d"/" -f1`
            echo $asid
            exit 0
         fi
     done
     #did not find previously appaware mount
     exit 1
}

function get_o_asm_sid {
     dgname=$1
     guser=$2
     get_asmcmd_nocp asmcmd

     sidcnt=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname|wc -l"`
     dbsids=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"`
     if [ -z "$dbsids" ]; then
        echo "no subdirectory found under $dgname"
        exit 1
     fi
    
   if [ $sidcnt -eq 1 ]; then
    asid=`echo $dbsids | cut -d"/" -f1`
    echo $asid
   else
     for x in ${dbsids}; do
         xdgname=$dgname"/"$x
         onlineredo=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $xdgname | grep -i onlinelog"`
         if [ $? -ne 0 ]; then
            #find onlineredolog directory, this is not backup, but appaware folder from previous mount
            asid=`echo $x | cut -d"/" -f1`
            if [ "$asid" != "DB_UNKNOWN" ]; then
               echo $asid
               exit 0
            fi
         fi
     done
     #did not find previously appaware mount
    fi
     exit 1
}

function get_spfile {
    dgname=$1
    dbsid=$2
    guser=$3
    get_asmcmd_nocp asmcmd

    spfile=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"/"$dbsid | grep spfile"`
    if [ -z "$spfile" ]; then
       echo "spfile for $dbsid is not found under $dgname/$dbsid"
       exit 1
    fi
    echo $spfile
    exit 0
}

function get_controlfile {
    dgname=$1
    dbsid=$2
    guser=$3
    get_asmcmd_nocp asmcmd

    ctrlfile=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"/"$dbsid"/CONTROLFILE""`
    if [ -z "$ctrlfile" ]; then
       echo "failed to get controlfile from diskgroup $dgname"
       exit 1
    fi
    echo $ctrlfile
    exit 0
}

function get_grid_home {
    oratab=$1
    if [ -f "$oratab" ]; then
       gridhome=`grep -v "^[#]" $oratab|grep -i "+ASM" | head -n1 | cut -f2 -d":"`
    fi

    if [ ! -z "$ASM_HOME" ]; then
        gridhome="$ASM_HOME"
    fi

    if [ -z "$gridhome" ]; then
       lsnr=`ps -ef | grep -i tnslsnr | grep -v grep | head -n1 | awk '{print $8}'`
       pname=$(basename $lsnr)
       if [ "$pname" != "tnslsnr" ]; then
          lsnr=`ps -ef | grep -i tnslsnr | grep -v grep | head -n1 | awk '{print $9}'`
       fi
       gridhome=`echo $lsnr | rev | cut -d'/' -f 3-10 | rev`
    fi

    if [ -z "$gridhome" ]; then
       exit 1
    fi
    echo $gridhome
    exit 0
}

function get_last_dgname {
    localspfile=$1
    lastdg=`grep "db_create_file_dest" $localspfile | cut -d"'" -f2 | cut -d"+" -f2`
    if [ -z "$lastdg" ]; then
       exit 1
    fi
    echo $lastdg
    exit 0
}

function get_teardown_sid {
    datadir=$1
    if [ -z $datadir ]; then
       exit 1
    fi
    if [ ! -d $datadir ]; then
       exit 1
    fi

    ls -ltr $datadir/init*.ora >/dev/null 2>&1
    if [ $? = 0 ]; then
       getpname=`ls -ltr $datadir/init*.ora | tail -1 | awk '{print $NF}'`
       pname=$(basename $getpname)
       pname=`echo $pname | cut -d"." -f1`
       osid=`echo $pname | cut -c5-`
       echo $osid
       return 0
    fi

    return 1
}

function get_prev_mountpoint {
   datadir=$1

    if [ -z $datadir ]; then
       exit 1
    fi

    if [ ! -d $datadir ]; then
       exit 1
    fi 

    ls -ltr $datadir/pmpkeep >/dev/null 2>&1
    if [ $? = 0 ]; then
       pmp=`cat $datadir/pmpkeep`
       echo "$pmp"
       exit 0
    fi

    exit 1
}

function is_appmount_instance {
    sid=$1
    
    if [ -z $sid ]; then
       exit 1
    fi

    setdbfile="/act/act_scripts/oracleclone/setdb_"$sid

    if [ ! -e "$setdbfile" ]; then
       setdbfile="/act/act_scripts/asmclone/setdb_"$sid
       if [ ! -e "$setdbfile" ]; then
          echo "$setdbfile does not exist, not app mounted by DK"
          exit 1
       fi
    fi
    
    return 0
}

function generate_asm_node_config {
   nodelist=$1
   user=$2
   rachostnames=$3
   #generate an asm_node_config file
   asmnodeconf="/act/act_scripts/asmclone/asm_node_config.conf"
   rm $asmnodeconf  > /dev/null 2>&1
   touch $asmnodeconf
   OLDIFS=$IFS
   IFS=':'
   i=1
   if [ -z "$rachostnames" ]; then
      localips=`ifconfig -a| grep 'inet' | awk '{print $2}' | sed 's/addr://'`
      echo "local ip is $localips"
      for n in ${nodelist}; do
         echo $n
         localipfound=0
         for localip in ${localips}; do
            if [[ "$localip" == *"$n"* ]]; then
               fn=`su -m $user -c "hostname --fqdn"`
               echo "local fn is $fn"
               localipfound=1
               break
            fi
         done
         if [ $localipfound -ne 1 ]; then
            su -m $user -c "ssh -o StrictHostKeyChecking=no -l$user $n exit"
            fn=`su -m $user -c "ssh $user@$n hostname --fqdn"`
            echo "remote fn is $fn"
         fi
         echo "ASM_NODE$i:$fn" >> $asmnodeconf
         i=$(($i + 1));
      done
   else
      for n in ${rachostnames}; do
         echo "ASM_NODE$i:$n" >> $asmnodeconf
         i=$(($i + 1));
      done
   fi
   IFS=$OLDIFS
   return 0
}

function change_own {
    user=$1
    file=$2
    #get group for the user
    grp=`id -gn $user`
    if [ -z "$grp" ]; then
       echo "failed to get group for $user"
       return 1
    fi
    chown $user:$grp $file
    chmod 755 $file
    return 0 
}

function generate_catalog_str {
    archdirstr=$1
    dirs=`ls -d $archdirstr*`

    catstr="catalog start with "
    catalogstr=""
    for x in ${dirs}; do
        catalogstr=$catalogstr$catstr"'"$x"/archivelog' noprompt;"    
    done
    echo "$catalogstr"
    return 0
}

function change_dbid {
    nsid=$1
    orahome=$2
    if [ -z $nsid ]; then
       echo "new SID is not provided"
       return 1
    fi
    if [ -z $orahome ]; then
       echo "ORACLE_HOME is not provided"
       return 1
    fi
    if [ ! -d $orahome ]; then
       echo "$orahome does not exists"
       return 1
    fi
    ORACLE_SID=$nsid
    ORACLE_HOME=$orahome
    PATH=$ORACLE_HOME/bin:$PATH
    export ORACLE_SID
    export ORACLE_HOME
    export PATH

    sqlplus=$ORACLE_HOME/bin/sqlplus

$sqlplus / as sysdba << EOF
shutdown immediate;
startup mount;
exit;
EOF

    $ORACLE_HOME/bin/nid target=/ LOGFILE=nid_$nsid.log 
    retval=$?

if [ "$retval" -gt "0" ]; then
nidfail=yes
$ORACLE_HOME/bin/nid target=/ LOGFILE=$nid_$nsid.log revert=yes
fi

if [ "$nidfail" = "yes" ]; then
$sqlplus / as sysdba  << EOF
startup mount;
alter database open;
exit;
EOF
else
$sqlplus / as sysdba << EOF
startup mount;
alter database open resetlogs;
exit;
EOF
fi
    return 0
}

function verify_database {
    sid=$1
    oraclehome=$2
    checktag=$3
    oosuser=$4

    ORACLE_SID=$sid
    ORACLE_HOME=$oraclehome
    PATH=$ORACLE_HOME/bin:$PATH
    export ORACLE_SID
    export ORACLE_HOME
    export PATH

    checksql="/act/act_scripts/oracleclone/checkstatus.sql"
    echo "set head off" > $checksql
    echo "set verify off" >> $checksql
    echo "set feed off" >> $checksql
    echo "set timing off" >> $checksql
    echo "set pages 0" >> $checksql
    echo "set echo off" >> $checksql
    echo "select open_mode from v\$database;" >> $checksql
    echo "exit;" >> $checksql

    if [ -z $oosuser ]; then
       omode=`$ORACLE_HOME/bin/sqlplus -s / as sysdba @$checksql`
    else
       omode=`su -m $oosuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba @$checksql"`
    fi
    rm $checksql
    if [ "$omode" = "$checktag" ]; then
       return 0
    fi
    return 1
}

function get_first_apm_sid {
     dgname=$1
     guser=$2
     lgen=$3
     get_asmcmd_nocp asmcmd

     dbsids=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"`
     if [ -z "$dbsids" ]; then
        echo "no subdirectory found under $dgname"
        exit 1
     fi

     for x in ${dbsids}; do
         xdgname=$dgname"/"$x
         apmtag=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $xdgname | grep apmgen"`
         if [ $? -eq 0 ]; then
            #find appare mount generation tag
            #find generation first
            gen=`echo $apmtag | awk -F"_" '{print $(NF-1)}'`
            if [ $gen -eq $lgen ]; then
               ftag=`echo $apmtag | awk -F"_" '{print $NF}' | awk -F"/" '{print $1}'`
               if [ "$ftag" = "f" ]; then
                  asid=`echo $x | cut -d"/" -f1`
                  echo $asid
                  exit 0
               fi
            fi
         fi
     done
     #did not find previously appaware mount
     exit 1
}

function get_apm_latest_gen {
     dgname=$1
     guser=$2
     get_asmcmd_nocp asmcmd
     dbsids=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $dgname"`
     if [ -z "$dbsids" ]; then
        echo "no subdirectory found under $dgname"
        exit 1
     fi
     gen=0
     for x in ${dbsids}; do
         xdgname=$dgname"/"$x
         apmtag=`su -m $guser -c "asmcmd $ASMCMD_NOCP ls $xdgname | grep apmgen"`
         if [ $? -eq 0 ]; then
            #find appare mount generation tag
            tgen=`echo $apmtag | awk -F"_" '{print $(NF-1)}'`
            if [ $tgen -gt $gen ]; then
               gen=$tgen
            fi
         fi
     done
     echo $gen
     exit 0
}

#------------------------------------------------------------
# function get_asmcmd_nocp() <asmcmd>
#------------------------------------------------------------
get_asmcmd_nocp() {
  asmcmd=$1
  asmcmdVer=`$asmcmd -V | awk -F"asmcmd version " '{print $2}' | cut -d. -f1 `
  if [ $asmcmdVer -ge 12 ]; then
    #asmcmd="$asmcmd --nocp"
    ASMCMD_NOCP="--nocp"
  fi
}

#------------------------------------------------------------
# function : clear_archive_log <dbsid oracle_home osuser> 
#------------------------------------------------------------

clear_archive_log() {
ORACLE_SID=$1
ORACLE_HOME=$2
OSUSER=$3
export ORACLE_SID
export ORACLE_HOME
export OSUSER

if [ -f clearArchiveLog.txt ]; then
 rm -f clearArchiveLog.txt
fi

if [ -z $OSUSER ]; then
 $ORACLE_HOME/bin/rman target / log=clearArchiveLog.txt <<EOF1
run{
crosscheck archivelog all;
delete noprompt archivelog all;
}
exit;
EOF1
else
 su -m $OSUSER -c '$ORACLE_HOME/bin/rman target / log=clearArchiveLog.txt' <<EOF1
run{
crosscheck archivelog all;
delete noprompt archivelog all;
}
exit;
EOF1
fi

retval=$?
 if [ $retval -ne 0 ]; then
 echo "ERROMSG: Failed to clear the the archivelog after masking"
 exit 0
 fi

#clear the temp file
#rm -f clearArchiveLog.txt

}


#------------------------------------------------------------
# function : clear_redo_log <dbsid oracle_home osuser>
#------------------------------------------------------------

clear_redo_log(){
ORACLE_SID=$1
ORACLE_HOME=$2
OSUSER=$3
export ORACLE_SID
export ORACLE_HOME
export OSUSER

if [ -f redoGroupInfo.txt ]; then
 rm -f redoGroupInfo.txt
fi

if [ -z $OSUSER ]; then
$ORACLE_HOME/bin/sqlplus -s / as sysdba <<EOF
set heading off
set feedback off
set serveroutput on
spool redoGroupInfo.txt
SELECT lf.group#, l.status FROM v\$log l, v\$logfile lf  where lf.type='ONLINE' and l.group# = lf.group#;
spool off;
EOF
else
su -m $OSUSER -c '$ORACLE_HOME/bin/sqlplus -s / as sysdba' <<EOF
set heading off
set feedback off
set serveroutput on
spool redoGroupInfo.txt
SELECT lf.group#, l.status FROM v\$log l, v\$logfile lf  where lf.type='ONLINE' and l.group# = lf.group#;
spool off;
EOF
fi

retval=$?
if [ $retval -ne 0 ]; then
 echo "ERRORMSG: Failed to get  the redolog file details"
 exit 0
fi

chmod 755 redoGroupInfo.txt

if [ -z $OSUSER ]; then
 cat redoGroupInfo.txt | awk '{print $1}' | sed '/^$/d'  | while read line
  do
$ORACLE_HOME/bin/sqlplus -s / as sysdba <<EOF
set serveroutput on
declare
s varchar2(10);
begin
select status into s FROM v\$log where group#=$line;
if s = 'INACTIVE' OR s = 'UNUSED' THEN
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
elsif
s = 'ACTIVE' THEN
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
ELSE
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
END IF;
END;
/
EOF
retval=$?
   if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to clear the redolog file"
     exit 0
   fi
  done
else
  cat redoGroupInfo.txt | awk '{print $1}' | sed '/^$/d'  | while read line
   do
su -m $OSUSER -c '$ORACLE_HOME/bin/sqlplus -s / as sysdba' <<EOF
set serveroutput on
declare
s varchar2(10);
begin
select status into s FROM v\$log where group#=$line;
if s = 'INACTIVE' OR s = 'UNUSED' THEN
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
elsif
s = 'ACTIVE' THEN
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
ELSE
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER SYSTEM SWITCH LOGFILE';
EXECUTE IMMEDIATE 'ALTER DATABASE CLEAR UNARCHIVED LOGFILE GROUP $line';
END IF;
END;
/
EOF
retval=$?
   if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to clear the redolog file"
    exit 0
   fi
  done
fi

#clear the temporary file
#rm -f redoGroupInfo.txt
}


#------------------------------------------------------------
# function : get_sql_major_version  <oracle_home osuer>
#------------------------------------------------------------

get_sql_major_version(){
ORACLE_HOME=$1
export ORACLE_HOME
LD_LIBRARY_PATH=$ORACLE_HOME/lib;
export LD_LIBRARY_PATH

version=`$ORACLE_HOME/bin/sqlplus -version | grep Release  | awk '{print $3}'| awk -F'.' '{print $1}'`
echo $version
exit 0

}
