#!/bin/sh
set +x
mntpt=$1
orasid=$2
archmnt=$3
tstamp=$4
dbfsPath=$5
flag=$6

if [ "$#" -lt 4 ]; then
 echo "This script needs 4 input parameter"
 echo "usage: recoverRman.sh <Backup location mount path> <New database sid name> <Log Backup location mount path> <timestamp yyyymmddhh24mi>"
exit 1
fi

if [ ! -z "$DB_TZ" ]; then
echo " ********** setting TZ:$DB_TZ  ***************"
TZ=$DB_TZ
export TZ
fi

chmod -R 777 $archmnt*

if [ -z "$rmanchannels" ]; then
rmanchannels=4;
fi

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh
from_tstampsql=from_tstamp_$orasid.sql
echo "set head off;" >$from_tstampsql
echo "set feedback off" >> $from_tstampsql
echo "set newpage none" >> $from_tstampsql
echo "select to_char(fromtime,'yyyymmddhh24mi') from (select to_date('$dbImageBeginTime','yyyy-mm-dd hh24:mi:ss')-interval '5' minute as fromtime  from dual);" >>$from_tstampsql
echo "exit;" >>$from_tstampsql
change_own $username $from_tstampsql
from_tstamp=`sqlplus -S / as sysdba @$from_tstampsql`
rm -f $from_tstampsql
touch -t $from_tstamp /act/act_scripts/oracleclone/fromtime_$orasid

until_tstampsql=until_tstamp_$orasid.sql
echo "set head off;" > $until_tstampsql
echo "set feedback off" >> $until_tstampsql
echo "select to_char(to_date('$tstamp','yyyymmddhh24miss')+1,'yyyymmddhh24mi') from dual;" >> $until_tstampsql
echo "exit;" >> $until_tstampsql
change_own $username $until_tstampsql
until_tstamp=`sqlplus -S / as sysdba @$until_tstampsql`
rm -f $until_tstampsql

#find the timestamp of the first archivelog backup after until_tstamp, and use it as the new until_time
touch -t $until_tstamp /act/act_scripts/oracleclone/untiltime_orig_$orasid
if [ ! -z $archmnt ]; then
   arcdirs=`ls -d $archmnt*`
   keep_date=""
   for a in ${arcdirs}; do
       arcbackupdir=$a"/archivelog/"
       if [ -d "$arcbackupdir" ]; then
          firstbackupname=`find $arcbackupdir -type f -newer /act/act_scripts/oracleclone/untiltime_orig_$orasid  -ls | head -1 | awk '{print $NF}'`
          if [ -f "$firstbackupname" ]; then
             temp_tstamp=`ls -l --time-style=long-iso $firstbackupname | awk '{print $(NF-2)" "$(NF-1)}'`
             temp_date=`date -d "$temp_tstamp" +'%Y-%m-%d %H:%M'`
             if [ -z "$keep_date" ]; then
                keep_date="$temp_date"
             else
                if [[ "$temp_date" < "$keep_date" ]]; then
                   keep_date="$temp_date"
                fi
             fi
          fi
       fi
   done
   if [ ! -z "$keep_date" ]; then
      adj_until_tstamp=`echo "$keep_date" | awk '{ gsub(/-|:| /, ""); print}'`
      #add one day on adjusted time
      until_tstampsql_post=until_tstamp_post_$orasid.sql
      echo "set head off;" > $until_tstampsql_post
      echo "set feedback off" >> $until_tstampsql_post
      echo "select to_char(to_date('$adj_until_tstamp','yyyymmddhh24mi')+1,'yyyymmddhh24mi') from dual;" >> $until_tstampsql_post
      echo "exit;" >> $until_tstampsql_post
      change_own $ouser $until_tstampsql_post
      until_tstamp_post=`sqlplus -S / as sysdba @$until_tstampsql_post`
      rm -f $until_tstampsql_post
      #end ajustment
      touch -t $until_tstamp_post /act/act_scripts/oracleclone/untiltime_$orasid
   else
      touch -t $until_tstamp /act/act_scripts/oracleclone/untiltime_$orasid
   fi
fi
#end forward search

allocatech=
releasech=
i=0
while [ $i -lt "$rmanchannels" ]; do
if [ -z "$allocatech" ]; then
allocatech="allocate channel d$i device type disk;"
releasech="release channel d$i;"
else
allocatech=$allocatech"allocate channel d$i device type disk;"
releasech=$releasech"release channel d$i;"
fi
i=`expr $i + 1`
done

if [ ! -f "$dbfsPath" ]; then
  roTBS=''
else
roTBS=`cat $dbfsPath`
fi

if [ "$pdbcnt" -gt "0" ]; then
  if [ ! -z "$roTBS" ]; then
    recoverCMD='skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2,"pdb$seed":UNDOTBS1_TEMP',"$roTBS"
  else
    recoverCMD='skip tablespace "pdb$seed":SYSTEM, "pdb$seed":SYSAUX, "pdb$seed":UNDOTBS1, "pdb$seed":UNDOTBS2,"pdb$seed":UNDOTBS1_TEMP'
  fi
else
  if [ ! -z "$roTBS" ]; then
     recoverCMD="skip tablespace $roTBS"
  else
    recoverCMD=''
  fi
fi


source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

export ORACLE_SID=$orasid
dbrecover="dbrecover_"$orasid".txt"
cstrfile="catalogArchivelog_"$orasid".txt"
echo "******* recovering database *********** "

if [ -f $cstrfile ]; then
 rm -f $cstrfile
fi

if [ ! -z "$imageLogOffset" ]; then
   imageLogOffset=$((imageLogOffset + 1))
fi

if [ ! -z $archmnt ]; then
     dirs=`ls -d $archmnt*`
     catalogstr=""
     for d in ${dirs}; do
         archmntpt=$d"/archivelog/"
         if [ -d "$archmntpt" ]; then
            if [ -f /act/act_scripts/oracleclone/untiltime_$orasid ] && [ ! -z "$until_tstamp" ] && [ -f /act/act_scripts/oracleclone/fromtime_$orasid ]; then
               find $archmntpt -newer /act/act_scripts/oracleclone/fromtime_$orasid ! -newer /act/act_scripts/oracleclone/untiltime_$orasid -type f >> $cstrfile
             else
               find $archmntpt -mtime -"$imageLogOffset" -type f >> $cstrfile
             fi
         fi
     done
     rm -f /act/act_scripts/oracleclone/untiltime_orig_$orasid
     rm -f /act/act_scripts/oracleclone/fromtime_$orasid
     rm -f /act/act_scripts/oracleclone/untiltime_$orasid

     sed -i -e "s/^/catalog BACKUPPIECE '/" $cstrfile
     sed -i "s/$/' ;/" $cstrfile
     if [ "$flag" != "no" ]; then
        catalogstr=`cat $cstrfile`
     else
        catalogstr=''
     fi

   if [ ! -z $tstamp ]; then
      rmancmd="run
      {
       $allocatech
       catalog start with '$mntpt/archivelog' noprompt;
       $catalogstr 
       recover database until time \"to_date('$tstamp','yyyymmddhh24miss')\" $recoverCMD;
       $releasech
       }"
   else
      rmancmd="run
      {
       $allocatech
       catalog start with '$mntpt/archivelog' noprompt;
       recover database $recoverCMD;
       $releasech
       }"
      
   fi
else
rmancmd="run
{
$allocatech
catalog start with '$mntpt/archivelog' noprompt;
recover database $recoverCMD;
$releasech
}"

fi

echo $rmancmd
export NLS_DATE_FORMAT="yyyymmddhh24mi"
export NLS_DATE_LANGUAGE=ENGLISH
export NLS_LANG=''
$ORACLE_HOME/bin/rman target / log=$dbrecover << EOF
sql "alter session set NLS_DATE_FORMAT=''yyyymmddhh24mi''";
sql "alter session set NLS_DATE_LANGUAGE=''ENGLISH''";
$rmancmd
exit
EOF

if [ -f $cstrfile ]; then
 rm -f $cstrfile
fi
