#!/bin/bash

set

set -x
INPUT_VAR=$2
cd /act/scripts
tmp_loc=/act/touch/pdbclone
#. /act/scripts/$conffile


osuser=$username
ORACLE_HOME=$orahome
export ORACLE_HOME
PATH=$ORACLE_HOME/bin:$PATH
export PATH
TNS_ADMIN=$ORACLE_HOME/network/admin
export TNS_ADMIN
echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE == "unmount" ] || [ $ACT_MULTI_OPNAME == "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE == "mount" ] || [ $ACT_MULTI_OPNAME == "mount" ] ; then
   if [ $ACT_PHASE == "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ $ACT_LOGSMART_TYPE == "log" ] ; then
        echo "** No op for log during unmount **"
     exit
   fi
   if [ $ACT_PHASE == "post" ] ; then
        echo "************************* start post during mount ************************"
dbname=$databasesid
HOSTNAME=`hostname | awk -F'.' '{print $1}'`

POST_SCRIPT_CONFIG=/act/scripts/pdb_post_scripts.conf

TMPSID=`srvctl status database -d $databasesid | grep -wi $HOSTNAME | awk '{print $2}'`
if [ -z "$TMPSID" ]; then
   TMPSID=$databasesid
fi
databasesid=$TMPSID
export ORACLE_SID=$TMPSID

TARGET_SID=`echo $INPUT_VAR | awk -F"_" '{print $1}'`
echo "TARGET SID : $TARGET_SID"

for pdbnames in $(echo $INPUT_VAR | sed "s/_/ /g")
do
  if [ "$pdbnames" = "$TARGET_SID" ]; then
      :
  else
      PDB_TMP=`echo $pdbnames | awk -F "-" '{print $1}'`
        if [ -z "$PDB_NAME" ]; then
           PDB_NAME=$PDB_TMP
        else
           PDB_NAME=$PDB_NAME,$PDB_TMP
        fi
  fi
done

PDB_NAME_TMP=$PDB_NAME
PDB_NAME=`echo $PDB_NAME |awk -F"," -v quote="'" -v OFS="','" '$1=$1 {print quote $0 quote}'|tr [a-z] [A-Z]`
echo $PDB_NAME

echo "************starting *************"
date
grp=`id -gn $osuser`
if [ -d $tmp_loc ]; then
   rm -rf $tmp_loc/*
   chown -R $osuser:$grp $tmp_loc
else
   mkdir -p $tmp_loc
   chown -R $osuser:$grp $tmp_loc
   chmod 755 /act/touch
   chmod 755 -R $tmp_loc
fi

export ORACLE_SID=$databasesid
pdbcount=`su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -S / as sysdba" <<EOF
set head off
select count(1) from dba_pdbs where pdb_name in ($PDB_NAME) ;
EOF`

if [ $pdbcount -lt '1' ]; then
  echo "PDB does not exist!"
  exit 1
fi

undofilelist=/act/act_scripts/asmclone/undofile_"$ORACLE_SID".sql
pdbundocreate=/act/act_scripts/asmclone/pdbundo_"$ORACLE_SID".sql

if [ -f $undofilelist ]; then
   rm -f $undofilelist
fi

if [ -f $pdbundocreate ]; then
   rm -f $pdbundocreate
fi
for pdbname_temp in $(echo $PDB_NAME_TMP | tr ',' ' ')
do
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $undofilelist
set head off;
set feed off;
set echo off;
set newpage none;
alter session set container=$pdbname_temp;
select distinct tablespace_name from  dba_tablespaces where tablespace_name like 'UNDO%';
exit;
EOF

DGGROUP=`su -m $osuser -c "sqlplus -S / as sysdba " << EOF
set head off;
set feed off;
set echo off;
set newpage none;
alter session set container=$pdbname_temp;
select distinct substr(name,1,instr(name, '/',1)-1) from v\\$datafile;
exit;
EOF`

echo "$DGGROUP"

undoexits=`cat $undofilelist |wc -l`
if [ "$undoexits" -gt 0 ]; then
   echo "alter session set container=$pdbname_temp;">>$pdbundocreate
fi
while read line
do
    set -- $line
    undoname=$1
    newundoname="$undoname"_CL
        echo "outside if $undoname"
        if [ "x$undoname" != "x" ]; then
        echo "inside if $undoname"

        echo "create undo tablespace $newundoname datafile '$DGGROUP' size 500M autoextend on;" >>$pdbundocreate
        echo "alter tablespace $undoname offline;" >>$pdbundocreate
        echo "drop tablespace $undoname including contents and datafiles;" >> $pdbundocreate
        fi
done < $undofilelist
echo "exit" >>$pdbundocreate
chmod 777 $pdbundocreate
done

if [ -f $pdbundocreate ]; then
su -m $osuser -c "sqlplus / as sysdba @$pdbundocreate"
fi


echo "****************Creating pluggable database XML file*******************"
su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" <<EOF
set head off;
set serveroutput on
WHENEVER SQLERROR EXIT SQL.SQLCODE
begin

 for rec in (select name from v\$pdbs where upper(name) in ($PDB_NAME)) loop
   for i in (select name from v\$pdbs where upper(name)=rec.name and open_mode='READ WRITE') loop
     execute immediate 'alter pluggable database '|| i.name||' close immediate instances=all';
   end loop;
  execute immediate 'alter pluggable database '||rec.name||' unplug into ''$tmp_loc/'||rec.name||'.xml''';
  execute immediate 'drop pluggable database '||rec.name||' keep datafiles';
 end loop;
end;
/
EOF

if [ $? -ne 0 ]; then
   echo "Failed to create XML file."
   exit 1
fi

su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" <<EOF
shutdown immediate;
exit;
EOF
su $osuser -c "$ORACLE_HOME/bin/srvctl stop database -d $dbname"
su $osuser -c "$ORACLE_HOME/bin/srvctl remove database -d $dbname -f"

echo "**************** Attach the pluggable database ***************"

echo " PATH is : $PATH"
echo "ORACLE_HOME IS: $ORACLE_HOME"

HOSTNAME=`hostname | awk -F'.' '{print $1}'`

TSID=`srvctl status database -d $TARGET_SID | grep -wi $HOSTNAME | awk '{print $2}'`

if [ -z "$TSID" ]; then
   TSID=$TARGET_SID
fi

echo " ORACLE_SID IS: $TSID"

export ORACLE_SID=$TSID

if [ -f /act/scripts/pdb_post_scripts.conf ]; then
   TARGET_ORACLE_HOME=`grep TARGET_ORACLE_HOME /act/scripts/pdb_post_scripts.conf | awk -F":" '{print $2}'`
   if [ ! -z "$TARGET_ORACLE_HOME" ]; then
      ORACLE_HOME=$TARGET_ORACLE_HOME
      export ORACLE_HOME
      PATH=$ORACLE_HOME/bin:$PATH
   fi
fi
#export ORACLE_SID=$TARGET_SID

for pdbname in $(echo $INPUT_VAR | sed "s/_/ /g")
do
  if [ "$pdbname" = "$TARGET_SID" ]; then
      :
  else
    PDB_NAME=`echo $pdbname | awk -F "-" '{print $1}'`
    PDB_NEWNAME=`echo $pdbname | awk -F "-" '{print $2}'`
    xmlfiles=`ls $tmp_loc/*.xml | grep -wi "$PDB_NAME"`
    f=$xmlfiles
    f=`basename $f`
su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
set head off;
WHENEVER SQLERROR EXIT SQL.SQLCODE
create pluggable database $PDB_NEWNAME AS CLONE using  '$tmp_loc/$f' nocopy tempfile reuse;
alter pluggable database $PDB_NEWNAME open instances=all;
exit;
EOF
fi

if [ $? -ne 0 ]; then
   echo "Failed to attach PDB."
su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
alter pluggable database $PDB_NEWNAME close instances=all;
drop pluggable database $PDB_NEWNAME including datafiles;
exit;
EOF
   exit 1
fi
done



if [ -f "$POST_SCRIPT_CONFIG" ]; then
   export DB_LOG_TIME=`date +%b_%d_%Y_%H%M%S`
   echo $ORACLE_SID
   echo $ORACLE_HOME
   echo " Start : Apply datapatch "
   su $osuser -c "$ORACLE_HOME/OPatch/datapatch"
   echo " Datapatch apply completed "

   for pdbname in $(echo $INPUT_VAR | sed "s/_/ /g")
   do
     if [ "$pdbname" = "$TARGET_SID" ]; then
        :
     else
        PDB_NAME=`echo $pdbname | awk -F "-" '{print $1}'`
        PDB_NEWNAME=`echo $pdbname | awk -F "-" '{print $2}'`
        scriptname=`cat $POST_SCRIPT_CONFIG | grep -i $PDB_NEWNAME`
        scriptname=`echo $scriptname | awk -F":" '{print $2}'`
        echo "*********Executing Post script $scriptname for the PDB $PDB_NEWNAME***********"
        su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
        spool $tmp_loc/ide_post_refresh_$PDB_NEWNAME.out 
        set timing on time on 
        set echo on
        alter session set container=$PDB_NEWNAME;
        show con_name
        select name from v$database;
        set echo on
        @$scriptname
        alter pluggable database $PDB_NEWNAME close instances=all;
        spool off
        exit;
EOF


echo "${TARGET_SID}s$PDB_NEWNAME"
echo $TARGET_SID
srvctl start service -d $TARGET_SID -s "${TARGET_SID}s$PDB_NEWNAME"

        echo "********* Completed ***********"
     fi
   done
fi

echo "PDB Clonning process is completed..!"
fi
fi
