#!/bin/bash


set -x
#conffile=$2;
conffile=act_pdb_config.conf;

cd /act/scripts
tmp_loc=/act/touch/pdbclone
. /act/scripts/$conffile

osuser=$OS_USER
export ORACLE_HOME
PATH=$ORACLE_HOME/bin:$PATH
export PATH
TNS_ADMIN=$ORACLE_HOME/network/admin
export TNS_ADMIN
PDB_NAME=`echo $PDB_NAME |awk -F"," -v quote="'" -v OFS="','" '$1=$1 {print quote $0 quote}'|tr [a-z] [A-Z]`
echo $PDB_NAME
echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE == "unmount" ] || [ $ACT_MULTI_OPNAME == "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE == "mount" ] || [ $ACT_MULTI_OPNAME == "mount" ] ; then
   if [ $ACT_PHASE == "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ $ACT_LOGSMART_TYPE == "log" ] ; then
        echo "** No op for log during unmount **"
     exit
   fi
   if [ $ACT_PHASE == "post" ] ; then
        echo "************************* start post during mount ************************"

export ORACLE_SID=$TARGET_TMP_SID

echo "************starting *************"
date

if [ -d $tmp_loc ]; then
   rm -rf $tmp_loc/*
else
   mkdir -p $tmp_loc
   chown -R oracle:dba $tmp_loc
   chmod 755 /act/touch
   chmod 755 -R $tmp_loc
fi

export ORACLE_SID=$TARGET_TMP_SID
pdbcount=`su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -S / as sysdba" <<EOF
set head off
select count(1) from dba_pdbs where pdb_name in ($PDB_NAME) ;
EOF`

if [ $pdbcount -lt '1' ]; then
  echo "PDB does not exist!"
  exit 1
fi

echo "****************Creating pluggable database XML file*******************"
su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" <<EOF
set head off;
set serveroutput on
begin

 for rec in (select name from v\$pdbs where upper(name) in ($PDB_NAME)) loop
   for i in (select name from v\$pdbs where upper(name)=rec.name and open_mode='READ WRITE') loop
     execute immediate 'alter pluggable database '|| i.name||' close immediate';
   end loop;
  execute immediate 'alter pluggable database '||rec.name||' unplug into ''$tmp_loc/'||rec.name||'.xml''';
  execute immediate 'drop pluggable database '||rec.name||' keep datafiles';
 end loop;
end;
/
EOF

if [ $? -ne 0 ]; then
   echo "Failed to create XML file."
   exit 1
fi

su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" <<EOF
shutdown immediate;
exit;
EOF

echo "**************** Attach the pluggable database ***************"

export ORACLE_SID=$TARGET_SID

xmlfiles=$tmp_loc/*.xml

for f in $xmlfiles
do
f=`basename $f`
pdbname=`echo $f | awk -F "." '{print $1}'`
pdbname=$pdbname"_NEW"
su $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
set head off;
create pluggable database $pdbname using  '$tmp_loc/$f' nocopy tempfile reuse;
alter pluggable database $pdbname open;
EOF
done

if [ $? -ne 0 ]; then
   echo "Failed to attach PDB."
   exit 1
fi

echo "PDB Clonning process is completed..!"
fi
fi
