#!/bin/sh
# This script will take care of the following tasks:
# 1) Login to appliance to get Session ID.
# 2) Create Vdisk from sky to host .
# 3) Rescan iscsi services.
#########################################################################

VDISK_CONFIG_FILE="/act/act_scripts/saphana/automation/Configuration/vdiskmap.conf"

if [ "$#" -lt 4 ]; then
echo "This script need 4 input parameter"
echo "usage: addVdisk.sh /<vdiskmountpoint> <hostid/unique hostname> <OSHANAUSER> <KEYNAME>"
echo " For example: addVdisk.sh  /actifiobackup 3456 gc1adm BKPMON"
exit 1
fi

echo "-----------Starting addVdisk script-------"
if [ ! -f "$VDISK_CONFIG_FILE" ]; then
  #Create VDISK_CONFIG_FILE
  echo -e "# vdisk and mountpoint mapping config " > $VDISK_CONFIG_FILE
fi

vdiskmountpoint=$1
host=$2
DBUSER=$3
KEYNAME=$4
vdiskdevicename=""
mountpoint=`echo "$vdiskmountpoint" | awk -F/ '{print $NF}'`
vgName=$mountpoint'dumpvg'
lvName=$mountpoint'dumplv'

HVERSION=`su - $DBUSER -c "HDB version | grep -i version: | cut -d':' -f2 | sed -e 's/^[ \t]*//'"`

hanaVersionCheck=`echo $HVERSION | cut -c1-6 | sed -e 's/\.//g'`

VERSION=`echo $HVERSION | cut -c1-3`

if [ $VERSION == "2.0" ]; then
dataSize=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"SELECT SUM(USED_SIZE) AS DBSIZE_bytes FROM \"SYS_DATABASES\".\"M_VOLUME_FILES\" WHERE FILE_TYPE = 'DATA'\""`
retval=$?
else
dataSize=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"SELECT SUM(USED_SIZE) AS DBSIZE_bytes FROM \"M_VOLUME_FILES\" WHERE FILE_TYPE = 'DATA'\""`
retval=$?
fi

retval=$?
if [ $retval -ne 0 ]; then
  echo "*** ERROR: Query Failure to get DB size using key $KEYNAME ************"
  exit $retval
fi

for i in ${dataSize}; do
 if [ $i = "DBSIZE_bytes" ]; then
   :
else
  dbSizeBytes=$i
fi
done

if [ -z "$dbSizeBytes" ]; then
  echo " Error : Hana Total DB Size cannot be null"
  exit 1
fi
# Create a Vdisk of 3 times the DB size on Sky.
vdisksize=`expr 3 \* $dbSizeBytes`

targetIQN=`iscsiadm -m discovery -t st -p $SkyIP:3260 2> /dev/null | awk '{print $NF}'`
sessionExists=`iscsiadm -m session | grep -w $SkyIP:3260`
if [ -z "$sessionExists" ]; then
  iscsiadm -m node -T $targetIQN -p $SkyIP:3260 -l
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** ERROR: Iscsi Login to $SkyIP has Failed************"
    exit $retval
  fi
fi
iscsiadm -m node -T $targetIQN -o update -n node.startup -v automatic
iscsiadm -m node -T $targetIQN -o update -n node.conn[0].startup -v automatic
iscsiSessionId=`iscsiadm -m session | grep $SkyIP':' | awk -F" " '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1`
iscsiadm -m session -r $iscsiSessionId --rescan
sleep 30

createStagingDisk()
{
  vdisksize=$1
  by=10485760; ceilFunc=$(( ($vdisksize + $by -1 )/$by )); vdisksize=`expr $by \* $ceilFunc`;
  loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
  sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
  addVdiskURL="https://$SkyIP/actifio/api/task/mkvdiskmap"
  inputParamsvDisk='{
  "sessionid":'"$sessionid"',
  "host":'"$host"',
  "size":'"$vdisksize"',
  "diskpool":'"act_per_pool000"'
  }'
  vdiskResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $addVdiskURL -d "$inputParamsvDisk")
  echo "vdiskResponse :" $vdiskResponse
  if [[ $vdiskResponse == *errormessage* ]]; then
    error=$(echo $vdiskResponse | jq -cr '.errormessage')
    echo "Add Vdisk has failed": $error
    exit 1
  else
    if [[ $vdiskResponse == *result* ]]; then
      vdiskname=$(echo $vdiskResponse | jq -cr '.result')
      echo "vdiskname :" $vdiskname
      echo -e "$vdiskname=$vdiskmountpoint" >> $VDISK_CONFIG_FILE
      #Step 4: Login into iscsi service
      iscsiadm -m session -r $iscsiSessionId --rescan
      sleep 30
      reportMappedDiskUrl="https://$SkyIP/actifio/api/report/reportmappeddisks?sessionid=$sessionid"
      LUNID=$(curl -w "\n" -s -k -G $reportMappedDiskUrl | jq -cr '.result[] | select(.VDiskName=="'"$vdiskname"'") | .LUNID')
      if [ -z "$LUNID" ]; then
        echo "Error : No LUNID found for the staging vdisk"
        exit 1
      fi
      vdiskdevicename=`ls -lrt /dev/disk/by-path/ | grep $SkyIP | grep -w lun-$LUNID | tac | awk -F/ '{print $NF}' | head -n 1`
      if [ -z "$vdiskdevicename" ]; then
        echo "Error : No vdisk mapping device found."
        exit 1
      fi
      echo "Staging Disk Device Name :" $vdiskdevicename
    fi
  fi
}

addDiskInNewVG()
{
    echo "vdiskdevicename :" $vdiskdevicename
    pvNames=`pvs -o pv_uuid,pv_name,vg_name 2> /dev/null | grep $vgName | awk '{print $2}'`
    vgremove -f $vgName 2> /dev/null
    if [ ! -z "$pvNames" ]; then
      pvNames=$(echo $pvNames | tr " " "\n")
      for pvName in $pvNames
      do
          pvremove $pvName
      done
    fi
    mkfs -t xfs -f /dev/$vdiskdevicename
    pvcreate -f /dev/$vdiskdevicename
    vgcreate $vgName /dev/$vdiskdevicename
    lvcreate -l 100%VG -n $lvName $vgName
    mkfs -t xfs -f /dev/$vgName/$lvName
    mkdir -p $vdiskmountpoint
    mount /dev/$vgName/$lvName $vdiskmountpoint
    mountpointresult=$(lsblk -b -io MOUNTPOINT | awk '$1=="'"$vdiskmountpoint"'"' | head -n 1)
    echo "mountpointresult :" $mountpointresult
    if [ -z "$mountpointresult" ]; then
      echo "Error : $HANABACKUPPATH mount point could not be created."
      exit 1
    fi
    userGroup=`ls -lrt $LOGBACKUPFS | grep $HanaDBOSUser | head -n 1 | awk -F" " '{print $4}'`
    chown -R $DBUSER:$userGroup $vdiskmountpoint
}

addDiskInExtendedVG()
{
    echo "vdiskdevicename :" $vdiskdevicename
    mkfs -t xfs -f /dev/$vdiskdevicename
    pvcreate -f /dev/$vdiskdevicename
    vgextend $vgName /dev/$vdiskdevicename
    freePE=`vgdisplay $vgName 2> /dev/null | grep "Free  PE / Size" |  awk '{print $5}'`
    lvextend -l +$freePE /dev/$vgName/$lvName
    xfs_growfs /dev/$vgName/$lvName
}

echo "-------Check if there is any staging disk and also if the disk size is sufficient for Hana DB-------"
mountpointresult=$(lsblk -b -io MOUNTPOINT | awk '$1=="'"$vdiskmountpoint"'"' | head -n 1)
echo "Mount point result if exists:: " $mountpointresult

vgExists=`lsblk | grep $vgName'-'$lvName`
if [ ! -z "$vgExists" ]; then
  if [ -z "$mountpointresult" ]; then
    mount /dev/$vgName/$lvName $vdiskmountpoint
  fi
  lvmSize=`vgs --noheadings --nosuffix --units b $vgName`
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** ERROR: Existing Staging Disk Mountpoint is Faulty with no VG $vgName present ************"
    exit $retval
  fi
  lvmSize=`echo $lvmSize | awk '{print $6}'`
  if [[ `echo "$dbSizeBytes $lvmSize" | awk '{print ($1 >= $2)}'` == 1 ]]; then
    requiredVdiskSize=`expr $vdisksize - $lvmSize`
    echo "----Creating another vdisk of size $requiredVdiskSize----"
    createStagingDisk $requiredVdiskSize
    echo "----Extending VolumeGroup-----"
    addDiskInExtendedVG
    echo "-----Disk Created in VG successfully-----"
  else
    echo "-----Found/Verified that existing Staging Disk has enough space for Hana DB backup-----"
  fi
else
  umount $vdiskmountpoint 2> /dev/null
  echo "----Create a staging disk-------"
  createStagingDisk $vdisksize
  echo "----Creating VolumeGroup-----"
  addDiskInNewVG
  echo "-----Disk Created in VG successfully-----"
fi
mountDir=$(echo "$vdiskmountpoint" | sed 's/\//\\\//g')
grep -q '^/dev/'$vgName'/'$lvName'.*' /etc/fstab &&  sed -i 's/^\/dev\/'$vgName'\/'$lvName'.*/\/dev\/'$vgName'\/'$lvName' '$mountDir' xfs _netdev,nofail 0 0/g' /etc/fstab || echo -e '/dev/'$vgName'/'$lvName $vdiskmountpoint' xfs _netdev,nofail 0 0' >> /etc/fstab
echo "----------end vdisk script----------"
