#!/bin/sh

# This script will take care of the following tasks:
# 1) Login to appliance to get Session ID.
# 2) Add host to appliance.
# 3) Discover apps.
#########################################################################

if [ "$#" -lt 1 ]; then
  echo "This script need 1 input parameter"
  echo "usage: applianceOperations.sh <IQN Name>"
  echo " For example: applianceOperations.sh iqn.1996-04.de.suse:01:sap-server1"
  exit 1
fi

iscsiname=$1
# Load the properties from Config file.

echo "--------------------------------------------------"
echo "Start Phase: Adding the host and discovering apps"
echo "--------------------------------------------------"

# Step 1: Login to appliance

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionID=$(curl -s -k -XPOST $loginURL | jq -r '.sessionid')
if [ -z $sessionID ]; then
   echo "Error : Session ID cannot be empty."
   exit 1
fi
# Step 2: Add host to appliance

hostIP=`hostname -i`
hostName=`hostname`
addHostURL="https://$SkyIP/actifio/api/task/mkhost -H 'Content-Type:application/json'"


inputParamsForAddHost='{
 "sessionid":"'"$sessionID"'",
 "type":"generic",
 "hostname":"'"$hostName"'",
 "ipaddress":"'"$hostIP"'",
 "iscsiname":"'"$iscsiname"'"
}'

addHostResponse=$(curl -s -k -XPOST $addHostURL -d "$inputParamsForAddHost")
if [[ $addHostResponse == *"already exists"* ]]; then
   addedHostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$hostName" -d "sessionid=$sessionID" | jq -cr '(.result[] | .id)')
   echo "Host already exists. HOST ID ::  $addedHostID"
   getHostURL="https://$SkyIP/actifio/api/info/lshost?sessionid=$sessionID&argument=$addedHostID"
   oldIscsiName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getHostURL | jq -cr '(.result | .iscsi_name)')
   if [ -z $oldIscsiName ]; then
     chHostURL="https://$SkyIP/actifio/svchost/$addedHostID/port?sessionid=$sessionID"
     inputParamsForChangeHost='{
       "iscsiport": [ "'"$iscsiname"'" ]
     }'
     chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
     echo "Host Update Response with IQN: $chHostResponse"
   else
     if [ $oldIscsiName != $iscsiname ]; then
      deleteIscsiHostURL="https://$SkyIP/actifio/svchost/$addedHostID/port?sessionid=$sessionID&iscsiname=%5B%22$oldIscsiName%22%5D&force=true"
      delHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X DELETE $deleteIscsiHostURL)
      echo "Host Delete IQN Response: $delHostResponse"

      chHostURL="https://$SkyIP/actifio/svchost/$addedHostID/port?sessionid=$sessionID"
      inputParamsForChangeHost='{
        "iscsiport": [ "'"$iscsiname"'" ]
      }'
      chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
      echo "Host Update Response with IQN: $chHostResponse"
     fi
   fi
else
  if [[ $addHostResponse == *errormessage* ]]; then
   echo "Failed to Add Host. Reason :" $addHostResponse
  else
  if [[ $addHostResponse == *result* ]]; then
   addedHostID=$(echo $addHostResponse | jq -cr '.result')
   echo "Host added successfully. Host ID is:" $addedHostID
  fi
 fi
fi

if [ -z $addedHostID ]; then
   echo " Error : Host ID cannot be empty."
   exit 1
fi
# Step: Discover apps on added host to check for security or connectivity issues.
discoverAppsURL="https://$SkyIP/actifio/api/task/appdiscovery -H 'Content-Type: application/json'"
inputParamsForAppDiscovery='{
 "sessionid":'"$sessionID"',
 "host": '"$addedHostID"'
}'

appDiscoveryResponse=$(curl -s -k -X POST $discoverAppsURL -d "$inputParamsForAppDiscovery")
if [[ $appDiscoveryResponse == *errormessage* ]]; then
 echo "App Discovery has Failed. Reason :" $appDiscoveryResponse
 exit 1
fi

source /act/act_scripts/saphana/automation/Backup/addVdisk.sh $HANABACKUPPATH $addedHostID $DBUSER $KEYNAME

userGroup=`ls -lrt $LOGBACKUPFS | grep $HanaDBOSUser | head -n 1 | awk -F" " '{print $4}'`
chown -R $DBUSER:$userGroup $HANABACKUPPATH

# Step 3: Discover apps on added host
appDiscoveryResponse=$(curl -s -k -X POST $discoverAppsURL -d "$inputParamsForAppDiscovery")
if [[ $appDiscoveryResponse == *errormessage* ]]; then
 echo "App Discovery has Failed. Reason :" $appDiscoveryResponse
 exit 1
fi
echo "Discovered Apps: " $appDiscoveryResponse

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "End Phase: Completed Adding the host and App discovery"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"


# Invoke the script to Create generic app, protect and run on-demand job / Create backup using Filesystem way and run on-demand job.

source /act/act_scripts/saphana/automation/Backup/createHanaDump.sh $addedHostID

# Invoke the script to find app for logvolume, protect it.

logMode=`grep log_mode /hana/shared/$DBSID/global/hdb/custom/config/global.ini`
logModeValue=$(echo "$logMode" | cut -d= -f2)
logModeValueTrimmed=$(echo "$logModeValue" | tr -d ' ')

if [[ $logModeValueTrimmed == *normal* && $LogBackupEnable == "true" ]]; then
  logbackupAppExists=`echo $appDiscoveryResponse | jq -cr '(.result[] | select(.appname=="'"$LOGBACKUPFS"'") | .appname)'`
  if [ ! -z $logbackupAppExists ]; then
    source /act/act_scripts/saphana/automation/Backup/logProtection.sh $addedHostID
  else
    echo "----Error: Cannot Find logbackup FS: $LOGBACKUPFS in discovered applications.---"
    exit 1
  fi
fi

echo "-----------------------------------------------------"
echo "-----Start Phase : Protect Any Other File Systems----"
echo "-----------------------------------------------------"

fsAppNames=$(echo $FSAppNames | tr "," "\n")

for appName in $fsAppNames
do
    # Fetching app id for FS App Name
    lsAppUrl="https://$SkyIP/actifio/api/info/lsapplication"
    appID=$(curl -s -k  -G $lsAppUrl --data-urlencode "filtervalue=appname=$appName&hostid=$addedHostID" -d "sessionid=$sessionid"   -H 'Cache-Control: no-cache' | jq -cr '(.result [] | .id)')

    chApplicationURL="https://$SkyIP/actifio/application/$appID?sessionid=$sessionid"
    inputParamsForApplication='{
      "description":{"type":"fs","method":"file","level":"incr"}
    }'
    chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
    echo "Other FS Application update Response: $chAppResponse"
    #Fetch template and profile id from the names.

    templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionID&filtervalue=name%3D$HanaDbTemplateName"
    templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

    profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionID&filtervalue=name%3D$ProfileName"
    profileID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')

    #Protect the app

    protectionURL="https://$SkyIP/actifio/api/task/mksla"

    inputParamsForProtection='{
    "sessionid":'"$sessionID"',
    "appid":'"$appID"',
    "slpid":'"$profileID"',
    "sltid":'"$templateID"'
    }'

    protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")
    if [[ $protectionResponse == *errormessage* ]]; then
      echo $appName `echo  $protectionResponse | jq -cr '.errormessage'`
    else
      if [[ $protectionResponse == *result* ]]; then
       echo "FS App $appName Protected successfully."
      fi
    fi
done

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "End Phase: Completed FS Protections"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
