#!/bin/sh

########################################################################################################################################################
# Script Version# - Version 1.0
# Parent script to invoke the below listed scripts/execute commands remotely via ssh. Scripts / commands are listed below:
# 1) PreRequisites.sh
#    Script functionality below:
#     - Check Network Connectivity and other validations for required inputs.(Fail if anything is missing).
#     - Copy connector into Host System.
#     - Install Connector.
#     - Install/Start ISCSI service if Needed.
# 2) Execute the sapconfig.sh script remotely from Orchestrator VM using ssh.
#     - This will give data to be used while creating the generic app and log volumes.
# 3) ApplianceOperations.sh
#    Script functionality below:
#     - Add Host.
#     - Discover Apps on Actifio Appliance.

if [ "$#" -lt 1 ]; then
  echo " This script needs key-value input parameters"
  echo " usage: backupAutomation.sh <name=value>"
  echo " For example: backupAutomation.sh HanaDBOSUser=gc1adm"
  exit 1
fi


# Check if JQ and CURL is isn
if ! type "jq" > /dev/null 2>&1; then
  echo "Error : JQ is not installed"
  exit 1
fi

if ! type "curl" > /dev/null 2>&1; then
  echo "Error : curl is not installed"
  exit 1
fi

backupConfFile="/act/act_scripts/saphana/automation/Configuration/hanabackup.conf"

echo -e '#Hana DB/Logbackup Specific Params' > $backupConfFile
for ARGUMENT in "$@"
do
   echo -e $ARGUMENT >> $backupConfFile
done

echo -e '#Custom Policy Settings if template is not pre-created for DB and Log Templates' >> $backupConfFile
echo -e 'dbbackup_rpo="24"' >> $backupConfFile
echo -e 'logbackup_rpo="1"' >> $backupConfFile

echo -e '#Key used for Licensing purpose' >> $backupConfFile
echo -e 'VendorKey="act"' >> $backupConfFile
#echo -e '#Profile name used during backup' >> $backupConfFile

if ! grep -q "ProfileName"  "$backupConfFile"; then
 echo -e 'ProfileName="LocalProfile"' >> $backupConfFile
fi

source $backupConfFile

#Checking if all mandatory parameters are present.

if [ -z $HANABACKUPPATH ]; then
   echo " Error : HANABACKUPPATH cannot be empty."
   exit 1
fi

if [ -z $SCHEDULE ]; then
   echo " Error : SCHEDULE cannot be empty."
   exit 1
fi

if [ -z $DELLOG ]; then
   echo " Error : DELLOG cannot be empty."
   exit 1
fi

if [ -z $LogBackupEnable ]; then
   echo " Error : LogBackupEnable cannot be empty."
   exit 1
fi

if [ -z $SkyIP ]; then
   echo " Error : Sky IP cannot be empty."
   exit 1
fi

if [ -z $HanaDbTemplateName ]; then
   echo " Error : Template name cannot be empty."
   exit 1
fi

if [ -z $KEYNAME ]; then
   echo " Error : KEYNAME cannot be empty."
   exit 1
fi

if [ -z $HanaDBOSUser ]; then
   echo " Error : Hana OS User cannot be empty."
   exit 1
fi

if [ -z $SkyPassword ]; then
  echo " Error : Sky Password cannot be empty."
  exit 1
fi

SKYCONFIG=/act/scripts/skyconf/$SkyIP'_details'
if [ ! -d "/act/scripts/skyconf" ]; then
  #Create Sky Conf Folder
  mkdir -p /act/scripts/skyconf
  chmod 755 /act/scripts/skyconf
fi

if [ -f "$SKYCONFIG" ]; then
  source $SKYCONFIG
fi
enSkyPassword=`echo $SkyPassword | openssl base64`
if [ -z "$enCHAPUsername" -o -z "$enCHAPPassword" ]; then
  enCHAPUsername=`date +%s | sha1sum | base64 | head -c 14 | openssl base64`
  enCHAPPassword=`date +%s | sha256sum | base64 | head -c 14 | openssl base64`
fi

echo -e "# sky config " > $SKYCONFIG
echo -e "enSkyPassword='$enSkyPassword'" >> $SKYCONFIG
echo -e "enCHAPUsername='$enCHAPUsername'" >> $SKYCONFIG
echo -e "enCHAPPassword='$enCHAPPassword'" >> $SKYCONFIG
chmod 755 $SKYCONFIG
source $SKYCONFIG

SkyUserName="admin"

echo "-------------------------------------------------------------------------------------"
echo "-----Starting SAP HANA DB Backup Configuration for HANA OS User: $HanaDBOSUser-------"
echo "-------------------------------------------------------------------------------------"


# Step 1 : Invoking PreRequisites.sh script to check pre-requisites (UDSAgent and ISCSI service)
echo "----------------------------------------------------------"
echo "---Start Phase: Checking Pre-Requisites on Host machine---"
echo "----------------------------------------------------------"
source /act/act_scripts/saphana/automation/Backup/preRequisites.sh $SkyIP

# Step 2 : Execute sapconfig.sh remotely to get data for creating generic app and the log volumes.

source /act/act_scripts/saphana/automation/Backup/hanaConfig.sh $DELLOG $LogBackupEnable $KEYNAME $HanaDBOSUser

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "-------End Phase: Completed Checking Pre-Requisites-------"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"

iqnName=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`

# Step 3 : Invoking ApplianceOperations.sh script to perform Add host and Discover apps. It would internally invoke CreateGenericApp.sh and LogProtection.sh

if [ -z $iqnName ]; then
   echo " Error : IQN Name is blank."
   exit 1
fi
source /act/act_scripts/saphana/automation/Backup/applianceOperations.sh $iqnName


echo "-------------------------------------------------------------------------------------"
echo "----------Completed the Backup Configuration for HANA OS User: $HanaDBOSUser---------"
echo "-------------------------------------------------------------------------------------"
