#!/bin/sh

# Script to configure iscsi and start the service.

configureCHAP()
{
  iscsiadm -m discoverydb -t sendtargets -p $SkyIP:3260 -o delete 2> /dev/null

  CHAPUsername=`echo $enCHAPUsername | openssl base64 -d`
  CHAPPassword=`echo $enCHAPPassword | openssl base64 -d`

  sed -i '/node.session.auth.authmethod =/c\node.session.auth.authmethod = CHAP' /etc/iscsi/iscsid.conf
  sed -i '/node.session.auth.username =/c\node.session.auth.username = '"$CHAPUsername"'' /etc/iscsi/iscsid.conf
  sed -i '/node.session.auth.password =/c\node.session.auth.password = '"$CHAPPassword"'' /etc/iscsi/iscsid.conf

  sed -i '/discovery.sendtargets.auth.authmethod =/c\discovery.sendtargets.auth.authmethod = CHAP' /etc/iscsi/iscsid.conf
  sed -i '/discovery.sendtargets.auth.username =/c\discovery.sendtargets.auth.username = '"$CHAPUsername"'' /etc/iscsi/iscsid.conf
  sed -i '/discovery.sendtargets.auth.password =/c\discovery.sendtargets.auth.password = '"$CHAPPassword"'' /etc/iscsi/iscsid.conf

  loginURL="https://$SkyIP/actifio/api/login?name=admin&password=$SkyPassword&vendorkey=$VendorKey"
  sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
  if [ -z $sessionid ]; then
     echo "Error : Session ID cannot be empty..Login has Failed"
     exit 1
  fi
  hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=`hostname`" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')


  configChapURL="https://$SkyIP/actifio/api/task/configchap?sessionid=$sessionid"
  inputParamsCHAP='{
  "sessionid":'"$sessionid"',
  "host":'"$hostID"',
  "username":'"$CHAPUsername"',
  "password":'"$CHAPPassword"'
  }'
  chapConfigResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $configChapURL -d "$inputParamsCHAP")

  if [[ $chapConfigResponse == *errormessage* ]]; then
   echo "CHAP Configuration has Failed. Reason :" $chapConfigResponse
   exit 1
  fi
}

isIscsiRunning=$(systemctl status iscsid | grep Active)
if [[ $isIscsiRunning == *"Active: active (running)"* ]]; then
  echo "ISCSI service is running" > /dev/null
else
  echo "Trying to Start Iscsi Service"
  checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
  if [ -z $checkopeniscsi ]; then
    if ! type "zypper" 2> /dev/null; then
      echo "Error : Zypper is not installed. Unable to proceed with ISCSI installation."
      exit 1
    fi
  # open-iscsi-2.0.874-53.10.1.x86_64 (ensure that right iscsi package version is present)
    zypper -n install open-iscsi
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "Error: ******* Could not install open-iscsi ************"
      exit $retval
    fi
    checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
    if [ -z $checkopeniscsi ]; then
      echo "Error: Cannot find open-iscsi package in the system"
      exit 1
    fi
  fi
fi

sed -i 's/:[^:]*$/:'`hostname`'/' /etc/iscsi/initiatorname.iscsi
sed -i '/^node.startup =/c\node.startup = automatic' /etc/iscsi/iscsid.conf
systemctl enable iscsi
systemctl enable iscsid
systemctl stop iscsid 2>/dev/null
configureCHAP
systemctl start iscsid
