#!/bin/sh

if [ "$#" -lt 1 ]; then
echo " This script need 1 input parameter"
echo " usage: createHanaDump.sh <Added Host ID>"
echo " For example: createHanaDump.sh 2468"
exit 1
fi

hostid=$1

echo "---------------------------------------------------"
echo "-----Start Phase: Protecting the Hana DB App-------"
echo "---------------------------------------------------"

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')


# Fetching app id for /actbackup
lsAppUrl="https://$SkyIP/actifio/api/info/lsapplication"
actBackupAppID=$(curl -s -k  -G $lsAppUrl --data-urlencode "filtervalue=appname=$HANABACKUPPATH&hostid=$hostid" -d "sessionid=$sessionid" | jq -cr '(.result [] | .id)')

if [ -z $actBackupAppID ]; then
   echo " Error : No App found in $SkyIP with name $appName"
   exit 1
fi

chApplicationURL="https://$SkyIP/actifio/application/$actBackupAppID?sessionid=$sessionid"
inputParamsForApplication='{
	"description":{"type":"db","method":"file","level":"'"$SCHEDULE"'","system-id":"'"$DBSID"'",
    "dbtype":"hana","database-name":"'"$DBNAMES"'"}
}'
chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
echo "DB Application update Response: $chAppResponse"
#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionid&filtervalue=name%3D$HanaDbTemplateName"
templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

if [ -z $templateID ]; then
  source /act/act_scripts/saphana/automation/Backup/createTemplate.sh $HanaDbTemplateName $dbbackup_rpo "Snap Policy" "Hana DB Policy"
  templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')
fi

profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
profileID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')


#Protect the app

protectionURL="https://$SkyIP/actifio/api/task/mksla"

inputParamsForProtection='{
"sessionid":'"$sessionid"',
"appid":'"$actBackupAppID"',
"slpid":'"$profileID"',
"sltid":'"$templateID"'
}'

protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")
if [[ $protectionResponse == *errormessage* ]]; then
  echo $HANABACKUPPATH `echo  $protectionResponse | jq -cr '.errormessage'`
else
  if [[ $protectionResponse == *result* ]]; then
   echo "App Protected successfully."
  fi
fi
echo "---------------------------------------------------"
echo "Configuring the Hana DB App for freeze and thaw phases"
echo "---------------------------------------------------"

# Copy scripts from /saphana/outofband to /act/scripts and give permission
sudo cp -r /act/act_scripts/saphana/dump/* /act/scripts/
chmod 755 /act/scripts/*
# Rename the script files
sudo mv /act/scripts/freeze.xxx /act/scripts/freeze.$actBackupAppID
sudo mv /act/scripts/appid.xxx /act/scripts/appid.$actBackupAppID

echo "---------------------------------------------------"
echo "Completed Hana DB App Configuration"
echo "---------------------------------------------------"

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "----End Phase: Completed the Hana DB App Protection---"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
