#!/bin/sh

if [ "$#" -lt 4 ]; then
echo " This script need 4 input parameter"
echo " usage: createTemplate.sh <HanaDbTemplateName> <dbbackup_rpo> <Snap Policy> <Hana DB Policy>"
echo " For example: createTemplate.sh HANADBTemplate 24 SnapPolicy HanaDBPolicy"
exit 1
fi

templateName=$1
backupRpo=$2
snapPolicy=$3
dbPolicy=$4

echo "-------------------------------"
echo "Creating Template on Appliance"
echo "-------------------------------"

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')

mkSLTURL="https://$SkyIP/actifio/api/task/mkslt"
override="true"
description="Template for SAP"

inputParams='{
"sessionid":'"$sessionid"',
"name":'"$templateName"',
"override":'"$override"',
"description":'"$description"'
}'
mkSLTResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkSLTURL -d "$inputParams")

if [[ $mkSLTResponse == *errormessage* ]]; then
  echo $mkSLTResponse | jq -cr '.errormessage'
else
  if [[ $mkSLTResponse == *result* ]]; then
    echo "Template created successfully. Going Ahead for Policy Creation"
    templateID=$(echo $mkSLTResponse | jq -cr .result)
    mkPolicyURL="https://$SkyIP/actifio/api/task/mkpolicy"
    inputParamsForPolicy='{
    "sessionid":'"$sessionid"',
    "rpo":'"$backupRpo"',
    "name":'"$snapPolicy"',
    "description":'"$dbPolicy"',
    "starttime":"19:00",
    "endtime":"7:00",
    "retention":"1",
    "scheduletype":"daily",
    "retentionm":"days",
    "rpom":"hours",
    "exclusiontype":"none",
    "op":"snap",
    "sltid":'"$templateID"',
    "selection":""
    }'

    mkPolicyResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkPolicyURL -d "$inputParamsForPolicy")
    policyID=$(echo $mkPolicyResponse | jq -cr .result)
    echo " Generated Policy ID: "$policyID

    if [ -z "$policyID" ]; then
      echo " Error : Policy Creation has Failed"
      exit 1
    fi

    mkPolicyOptionURL="https://$SkyIP/actifio/api/task/mkpolicyoption"
    inputParams='{
      "sessionid":'"$sessionid"',"name":"scriptinittimeout","value":"3600","sltid":'"$templateID"'
    }'
    mkPolicyOptionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkPolicyOptionURL -d "$inputParams")
    inputParams='{
      "sessionid":'"$sessionid"', "name":"scriptfinishtimeout","value":"3600","sltid":'"$templateID"'
    }'
    mkPolicyOptionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkPolicyOptionURL -d "$inputParams")
    inputParams='{
      "sessionid":'"$sessionid"', "name":"scriptfreezetimeout","value":"72000","sltid":'"$templateID"'
    }'
    mkPolicyOptionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkPolicyOptionURL -d "$inputParams")
    inputParams='{
      "sessionid":'"$sessionid"', "name":"scriptunfreezetimeout","value":"72000","sltid":'"$templateID"'
    }'
    mkPolicyOptionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $mkPolicyOptionURL -d "$inputParams")
  fi
fi
