#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#set -x

DELLOG=$1
LogBackupEnable=$2
KEYNAME=$3
HanaDBOSUser=$4

if [ "$#" -lt 4 ]; then
  echo "This script needs 4 input parameters"
  echo "usage: hanaConfig.sh <DEL LOG> <LOG BACKUP ENABLED> <KEYNAME> <HanaDBOSUser"
  echo " For example: hanaConfig.sh 3 false ACTBACKUP gc1adm"
  exit 1
fi

HANACONFIG="/act/scripts/hana_config.conf"
HANADUMPCONFIG="/act/scripts/act_hana_dump_config.conf"

DBUSER=""
DBSID=""
INSTANCENUM=""
HVERSION=""
VERSION=""
DATAVOL=""
DATAPATH=""
LOGVOL=""
LOGPATH=""
DATAVOLUMEOWNER=""
PYTHONPATH=""
DBNAMES=""

#checking SAPHANA dbuser based on conf user...
DBUSER=`ps -ef | grep -i hdbindexserver | grep -v grep | grep -i $HanaDBOSUser | awk '{print $1}' | head -n 1`
if [ -z $DBUSER ]; then
   echo "Error : Unable to fetch DBUSER."
   exit 1
fi

HVERSION=`su - $DBUSER -c "HDB version | grep -i version: | cut -d':' -f2 | sed -e 's/^[ \t]*//'"`

hanaVersionCheck=`echo $HVERSION | cut -c1-6 | sed -e 's/\.//g'`
#Supported DB version :- 1.00.5* or above
if [[ `echo "$hanaVersionCheck 1005" | awk '{print ($1 < $2)}'` == 1 ]]; then
  echo "Error: Supported Hana DB Version is 1.00.5* or higher"
  exit 1
fi

VERSION=`echo $HVERSION | cut -c1-3`
if [ -z "$VERSION" ]; then
  echo "Error : HANA VERSION cannot be null"
  exit 1
fi

databaseSID=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"select VALUE from \"SYS\".\"M_SYSTEM_OVERVIEW\" where NAME = 'Instance ID'\""`
retval=$?
if [ $retval -ne 0 ]; then
  echo "*** ERROR: Query Failure to get Instance Number using key $KEYNAME ************"
  exit $retval
fi

for i in ${databaseSID}; do
 if [ $i = "VALUE" ]; then
   :
else
  DBSID=$i
fi
done

if [ -z "$DBSID" ]; then
  echo " Error : Unable to fetch DBSID"
  exit 1
fi

#Remove start and end quotes
DBSID="${DBSID%\"}"
DBSID="${DBSID#\"}"

logMode=`grep log_mode /hana/shared/$DBSID/global/hdb/custom/config/global.ini`
logModeValue=$(echo "$logMode" | cut -d= -f2)
logModeValueTrimmed=$(echo "$logModeValue" | tr -d ' ')

if [[ $logModeValueTrimmed != *normal* ]]; then
  echo "Error : Log Mode should be normal for enabling log backups"
  exit 1
fi

if [ $LogBackupEnable == "true" ] && [ -z "$LOGBACKUPFS" ]; then
    echo " Error : LOGBACKUPFS should be configured to take log backups"
    exit 1
fi

keynameExists=`su - $DBUSER -c "hdbuserstore list" | grep -w $KEYNAME`
if [ -z "$keynameExists" ]; then
  echo " Error : $KEYNAME User Store Key is not defined in SAP HANA hdbuserstore"
  exit 1
fi


INSTANCE_NUM=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"select VALUE from \"SYS\".\"M_SYSTEM_OVERVIEW\" where NAME = 'Instance Number'\""`
retval=$?
if [ $retval -ne 0 ]; then
  echo "*** ERROR: Query Failure to get Instance Number using key $KEYNAME ************"
  exit $retval
fi

for i in ${INSTANCE_NUM}; do
 if [ $i = "VALUE" ]; then
   :
else
  INSTANCENUM=$i
fi
done

if [ -z "$INSTANCENUM" ]; then
  echo " Error : INSTANCE NUMBER cannot be null"
  exit 1
fi

#Remove start and end quotes
INSTANCENUM="${INSTANCENUM%\"}"
INSTANCENUM="${INSTANCENUM#\"}"

dbnames=`su - $DBUSER -c "hdbsql -i $INSTANCENUM -U $KEYNAME -a -j -x \"select database_name, ACTIVE_STATUS from m_databases\""`
if [ $? -ne 0 ]; then
	echo "failed to fetch database using $KEYNAME"
	exit 1
fi
for dbname in ${dbnames}; do
	if [ $dbname = "DATABASE_NAME,ACTIVE_STATUS" ]; then
		:
	else
    TDBS=`echo $dbname |awk -F ',' '{print $2}'|awk -F '"' '{print $2}'`
    TSID=`echo $dbname |awk -F ',' '{print $1}'|awk -F '"' '{print $2}'`
    DBNAMES="${DBNAMES}${DBNAMES:+,}$TSID"
	fi
  if [ $TDBS = "YES" ]; then
		keynameUser=`su - $DBUSER -c "hdbuserstore list $KEYNAME" | grep USER | awk 'NF>1{print $NF}'`
    if [ $VERSION == "2.0" ]; then
  	  if [ $TSID = "SYSTEMDB" ]; then
        echo "Checking Privileges for single sign-on key $KEYNAME"
  			USERNAME=`su - $DBUSER -c "hdbsql -U $KEYNAME -a -j -x \"SELECT USER_NAME FROM \"PUBLIC\".\"EFFECTIVE_PRIVILEGES\" where USER_NAME = '$keynameUser' and PRIVILEGE in ('BACKUP ADMIN','CATALOG READ','DATABASE ADMIN') GROUP BY USER_NAME HAVING COUNT(*)>=3\""`
        retval=$?
        if [ $retval -ne 0 ]; then
          echo "*** ERROR: Query Failure to check privileges using key $KEYNAME ************"
          exit $retval
        fi
        for i in ${USERNAME}; do
           if [ $i = "USER_NAME" ]; then
           :
           else
            echo "$KEYNAME has the required Privileges"
            privileges=true
           fi
        done
        if [ -z $privileges ]; then
          echo "ERROR: $KEYNAME key does not have required privileges"
          exit 1
        fi
      else #else Check for Tenant DB for 2.0
        su - $DBUSER -c "hdbsql -i $INSTANCENUM -U $KEYNAME$TSID -a -j -x \"select database_name from m_databases\""
        if [ $? -ne 0 ]; then
          echo "failed to fetch database using $KEYNAME$TSID"
          exit 1
        fi
        tenantDbUsername=`su - $DBUSER -c "hdbuserstore list $KEYNAME$TSID" | grep USER | awk 'NF>1{print $NF}'`
        echo "Checking Privileges for single sign-on key $KEYNAME$TSID"
        USERNAME=`su - $DBUSER -c "hdbsql -U $KEYNAME$TSID -a -j -x \"SELECT USER_NAME FROM \"PUBLIC\".\"EFFECTIVE_PRIVILEGES\" where USER_NAME = '$tenantDbUsername' and PRIVILEGE in ('BACKUP ADMIN','CATALOG READ') GROUP BY USER_NAME HAVING COUNT(*)>=2\""`
        retval=$?
        if [ $retval -ne 0 ]; then
          echo "*** ERROR: Query Failure to check privileges using key $KEYNAME$TSID ************"
          exit $retval
        fi
        for i in ${USERNAME}; do
           if [ $i = "USER_NAME" ]; then
           :
           else
          echo "$KEYNAME$TSID has the required Privileges"
          privileges=true
           fi
        done
        if [ -z $privileges ]; then
          echo "ERROR: $KEYNAME$TSID key does not have required privileges"
          exit 1
        fi
      fi #else ends for 2.0 check
  else #For 1.0 Hana
    echo "Checking Privileges for single sign-on key $KEYNAME"
    USERNAME=`su - $DBUSER -c "hdbsql -U $KEYNAME -a -j -x \"SELECT USER_NAME FROM \"PUBLIC\".\"EFFECTIVE_PRIVILEGES\" where USER_NAME = '$keynameUser' and PRIVILEGE in ('BACKUP ADMIN','CATALOG READ') GROUP BY USER_NAME HAVING COUNT(*)>=2\""`
		retval=$?
		if [ $retval -ne 0 ]; then
			echo "*** ERROR: Query Failure to check privileges using key $KEYNAME ************"
			exit $retval
		fi
		for i in ${USERNAME}; do
		   if [ $i = "USER_NAME" ]; then
			 :
		   else
  			echo "$KEYNAME has the required Privileges"
  			privileges=true
		   fi
		done
		if [ -z $privileges ]; then
			echo "ERROR: $KEYNAME key does not have required privileges"
			exit 1
		fi
  fi
else
  echo "$TSID database privileges check skipped as database is inactive state"
fi
done

#globalinipath=`locate global.ini | grep -i "custom/config/global.ini" | grep -i $DBSID`
globalpath=`su - $DBUSER -c 'echo $DIR_INSTANCE'`
globalpath=`dirname $globalpath`

globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini

DATAVOL=`grep -i basepath_datavolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
DATAPATH=${DATAVOL::-4}
if [ -z $DATAPATH ]; then
  echo "ERROR: DATA PATH cannot be null"
  exit 1
fi
DATAVOLUMEOWNER=`ls -ld $DATAPATH | awk '{print $3":"$4}'`

LOGVOL=`grep -i basepath_logvolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
LOGPATH=${LOGVOL::-4}
if [ -z $LOGPATH ]; then
  echo "ERROR: LOG PATH cannot be null"
  exit 1
fi

#currently only support single SAPHANA instance per host
echo -e "# hana config " > $HANACONFIG
echo -e "DBSID='$DBSID'" >> $HANACONFIG
echo -e "INSTANCENUM='$INSTANCENUM'" >> $HANACONFIG
echo -e "HANAVERSION='$VERSION'" >> $HANACONFIG
echo -e "KEYNAME='$KEYNAME'" >> $HANACONFIG
echo -e "DATAPATH='$DATAPATH'" >> $HANACONFIG
echo -e "LOGPATH='$LOGPATH'" >> $HANACONFIG
echo -e "DELLOG='$DELLOG'" >> $HANACONFIG
echo -e "DBNAMES='$DBNAMES'" >> $HANACONFIG

chmod 755 $HANACONFIG

echo -e "# hana dump config " > $HANADUMPCONFIG
echo -e "DBSID='$DBSID'" >> $HANADUMPCONFIG
echo -e "INSTANCENUM='$INSTANCENUM'" >> $HANADUMPCONFIG
echo -e "HANAVERSION='$VERSION'" >> $HANADUMPCONFIG
echo -e "KEYNAME='$KEYNAME'" >> $HANADUMPCONFIG
echo -e "HANABACKUPPATH='$HANABACKUPPATH'" >> $HANADUMPCONFIG
echo -e "SCHEDULE='$SCHEDULE'" >> $HANADUMPCONFIG
echo -e "SkyIP='$SkyIP'" >> $HANADUMPCONFIG
echo -e "DBUSER='$DBUSER'" >> $HANADUMPCONFIG
chmod 755 $HANADUMPCONFIG
