#!/bin/sh

if [ "$#" -lt 1 ]; then
echo " This script need 1 input parameter"
echo " usage: logProtection.sh <Added Host ID>"
echo " For example: logProtection.sh 2468"
exit 1
fi

hostid=$1

# Script to protect and run the on-demand job for Log volume.

echo "---------------------------------------------------"
echo "-------Start Phase: Protecting the Log App---------"
echo "---------------------------------------------------"

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')

#Advanced Settings
updateAdvancedSettingsURL="https://$SkyIP/actifio/api/task/mkpolicyoption"

# Fetching app id for $LOGBACKUPFS
appUrl="https://$SkyIP/actifio/api/info/lsapplication"
usrSapAppID=$(curl -s -k  -G $appUrl --data-urlencode "filtervalue=appname=$LOGBACKUPFS&hostid=$hostid" -d "sessionid=$sessionid"   -H 'Cache-Control: no-cache' | jq -cr '(.result [] | .id)')

if [ -z $usrSapAppID ]; then
   echo " Error : No App found in $SkyIP with name $LOGBACKUPFS"
   exit 1
fi

chApplicationURL="https://$SkyIP/actifio/application/$usrSapAppID?sessionid=$sessionid"
inputParamsForApplication='{
	"description":{"type":"log","method":"file","level":"incr","system-id":"'"$DBSID"'",
    "dbtype":"hana","database-name":"'"$DBNAMES"'"}
}'
chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
echo "Log Application update Response: $chAppResponse"

if [ ! -z $LOGBACKUPPATH ]; then

  paramsForAdvancedSettings='{
  "sessionid":'"\"$sessionid\""',
  "appid":'"\"$usrSapAppID\""',
  "name":'"\"startpaths\""',
  "value":'"\"$LOGBACKUPPATH\""'
  }'

  updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")

  echo "Advanced Settings Response : " $updateAdvanceSettingResponse
fi

if [ -z $LogTemplateName ]; then
   echo " Error : Log Template name cannot be empty."
   exit 1
fi

if [ -z $logbackup_rpo ]; then
   echo " Error : Log Backup RPO cannot be empty."
   exit 1
fi

#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionid&filtervalue=name%3D$LogTemplateName"
templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

if [ -z $templateID ]; then
  source /act/act_scripts/saphana/automation/Backup/createTemplate.sh $LogTemplateName $logbackup_rpo "Snap Policy" "Log Backup Policy"
  templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')
fi

profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
profileID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')

#Protect the app

protectionURL="https://$SkyIP/actifio/api/task/mksla"

inputParamsForProtection='{
"sessionid":'"$sessionid"',
"appid":'"$usrSapAppID"',
"slpid":'"$profileID"',
"sltid":'"$templateID"'
}'

protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")
if [[ $protectionResponse == *errormessage* ]]; then
   echo $LOGBACKUPFS `echo  $protectionResponse | jq -cr '.errormessage'`
else
  if [[ $protectionResponse == *result* ]]; then
     echo "Log App Protected successfully."
 fi
fi

echo "---------------------------------------------------------------"
echo "Start Phase: Configuring the Log App for freeze and thaw phases"
echo "---------------------------------------------------------------"

# Copy scripts from /saphana/outofband to /act/scripts and give permission
sudo cp -r /act/act_scripts/saphana/log/* /act/scripts/
chmod 755 /act/scripts/*

# Rename the script files
sudo mv /act/scripts/fini.xxx /act/scripts/fini.$usrSapAppID
sudo mv /act/scripts/appid.xxx /act/scripts/appid.$usrSapAppID

echo "------------------------------------------------------"
echo "----End Phase: Completed the Log App Configuration----"
echo "------------------------------------------------------"

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "------End Phase: Completed the Log App Protection-----"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
