#!/bin/sh
set -x

if [ "$#" -lt 1 ]; then
  echo " This script needs key-value input parameters"
  echo " usage: restore.sh <name=value>"
  echo " For example: restore.sh targetOSUser=gc1adm"
  exit 1
fi

restoreConfFile="/act/act_scripts/saphana/automation/Configuration/hanarestore.conf"

echo -e '#Hana Restore Params' > $restoreConfFile
for ARGUMENT in "$@"
do
  ARGUMENT="${ARGUMENT%\"}"
  ARGUMENT="${ARGUMENT#\"}"
  echo -e $ARGUMENT >> $restoreConfFile
done

source $restoreConfFile

if [ -z $HANABACKUPPATH ]; then
  echo "Error: DB App Name is Mandatory"
  exit 1
fi
if [ -z $targetOSUser ]; then
  echo "Error: Target DB SID OS-Username is mandatory"
  exit 1
fi
if [ -z $SkyIP ]; then
  echo "Error: Sky Appliance IP is mandatory"
  exit 1
fi
if [ -z "$RECOVERYTIME" ]; then
  printf "\n Reason: Recovery Time has to set to Most Recent Time or Point in Time\n"
  exit 1
fi

if [ -z $RECDATABASE ]; then
  RECDATABASE='ALL'
fi
if [ -z $KEYNAME ]; then
  KEYNAME='BKPMON'
fi

DBSID=`echo $targetOSUser | cut -c 1-3`
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
if [ -z "$DBSID" ]; then
  echo " Error : Unable to fetch DBSID"
  exit 1
fi

HVERSION=`su - $targetOSUser -c "HDB version | grep -i version: | cut -d':' -f2 | sed -e 's/^[ \t]*//'"`
VERSION=`echo $HVERSION | cut -c1-3`
if [ -z "$VERSION" ]; then
  echo "Error : HANA VERSION cannot be null"
  exit 1
fi

INSTANCENUM=`ls -lrt /usr/sap/$DBSID | grep HDB | awk -FHDB '{print $NF}'`

PYTHON_PATH=/usr/sap/$DBSID/HDB$INSTANCENUM/exe/python_support

SkyUserName="admin"
if [ -z $SkyPassword ]; then
   SkyPassword='password'
fi
enSkyPassword=`echo $SkyPassword | openssl base64`

SKYCONFIG=/act/scripts/skyconf/$SkyIP'_details'
if [ -f "$SKYCONFIG" ]; then
  source $SKYCONFIG
fi
if [ -z "$enCHAPUsername" -o -z "$enCHAPPassword" ]; then
  enCHAPUsername=`date +%s | sha256sum | base64 | head -c 14 | openssl base64`
  enCHAPPassword=`openssl rand -base64 14 | head -c 14 | openssl base64`
fi

echo -e "# sky config " > $SKYCONFIG
echo -e "enSkyPassword='$enSkyPassword'" >> $SKYCONFIG
echo -e "enCHAPUsername='$enCHAPUsername'" >> $SKYCONFIG
echo -e "enCHAPPassword='$enCHAPPassword'" >> $SKYCONFIG
chmod 755 $SKYCONFIG
source $SKYCONFIG

VendorKey="act"
BACKUPTYPE="INCREMENTAL"

echo "----Checking Pre-Requisites(Agent,Iscsi) on Host machine----"
source /act/act_scripts/saphana/automation/Backup/preRequisites.sh $SkyIP
echo "--------Done Checking Pre-Requisites on Host machine--------"

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=`hostname`" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

#Ensure IQN of host is latest in sky
echo "HOST ID ::  $hostID"
iscsiname=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`
getHostURL="https://$SkyIP/actifio/api/info/lshost?sessionid=$sessionid&argument=$hostID"
oldIscsiName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getHostURL | jq -cr '(.result | .iscsi_name)')
if [ -z $oldIscsiName ]; then
  chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
  inputParamsForChangeHost='{
    "iscsiport": [ "'"$iscsiname"'" ]
  }'
  chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
  echo "Host Update Response with IQN: $chHostResponse"
else
  if [ $oldIscsiName != $iscsiname ]; then
   deleteIscsiHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid&iscsiname=%5B%22$oldIscsiName%22%5D&force=true"
   delHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X DELETE $deleteIscsiHostURL)
   echo "Host Delete IQN Response: $delHostResponse"

   chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
   inputParamsForChangeHost='{
     "iscsiport": [ "'"$iscsiname"'" ]
   }'
   chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
   echo "Host Update Response with IQN: $chHostResponse"
  fi
fi

# Check if mounts exist for the given app.

lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"

filterValue="filtervalue=jobclass=mount&appname=$HANABACKUPPATH&apptype=FileSystem&mountedhost=$hostID"
mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

mountImageIDListLength=${#mountImageIDList[@]}
echo "Existing $HANABACKUPPATH mounts for Hana DB are:"  $mountImageIDListLength

if [ $mountImageIDListLength -gt 0 ]; then
   for (( i=0; i<${mountImageIDListLength}; i++ ))
   do
   # Unmount and Delete Image
   unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

    inputParamsForUnmount='{
    "sessionid":"'"$sessionid"'",
    "image":"'"${mountImageIDList[i]}"'",
    "delete":"true"
   }'

   echo "Need to unmount and re-mount for data refresh"
   unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
   if [[ $unmountResponse == *errormessage* ]]; then
      echo "Unmount Failed. Reason :" $unmountResponse
   elif [[ $unmountResponse == *result* ]]; then
      echo "Existing App Unmounted successfully."
   fi
  done
fi

#hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=`hostname`" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
echo "Image: $dbimagename Hana DB RESTOREPATH: $DBRESTOREPATH"
mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

if [ -z $DBRESTOREPATH ]; then
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true"
     }'
else
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true",
      "restoreoption":"'"mountpointperimage=$DBRESTOREPATH"'"
     }'
fi

jobID=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount" |  jq -cr '(.result)' | cut -d ' ' -f1)
printf "\n The Job id is: $jobID \n"

if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
  printf "\n Reason: jobID cannot be null.\n"
  exit 1
fi

jobinprogress=true
errorMessage=""
while $jobinprogress
    do
        sleep 30
        getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
        status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
        if [ ! -z "$status" ]; then
                if [ $status == "succeeded" -o $status == "failed" ]
                then
                        jobinprogress=false
                        if [ $status == "failed" ]
                        then
                                getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                        fi
                fi
              else
                jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                echo "Job is still running..Completed $status%"
        fi
done
printf "\n The Hana DB Mount job $jobID has $status.\n"

if [ ! -z "$errorMessage" ]; then
  printf "\n Reason: $errorMessage.\n"
  exit 1
fi

if [ -z $DBRESTOREPATH ]; then
  DBRESTOREPATH=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
fi

if [ ! -z "$logimagename" ] && [ ! -z "$LOGBACKUPFS" ]; then
  lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
  filterValue="filtervalue=backupname=$logimagename"
  backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
  appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
  policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
  LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')


  filterValue="filtervalue=jobclass=mount&appname=$LOGBACKUPFS&apptype=FileSystem&mountedhost=$hostID"
  mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))
  mountImageIDListLength=${#mountImageIDList[@]}
  echo "Existing mounts for logBackup are:"  $mountImageIDListLength

  if [ $mountImageIDListLength -gt 0 ]; then
     for (( i=0; i<${mountImageIDListLength}; i++ ))
     do

     # Unmount and Delete Image
     unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

      inputParamsForUnmount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"${mountImageIDList[i]}"'",
      "delete":"true"
     }'

     unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
     if [[ $unmountResponse == *errormessage* ]]; then
        echo "Unmount Failed. Reason :" $unmountResponse
     elif [[ $unmountResponse == *result* ]]; then
        echo "Logbackup App Unmounted successfully."
     fi
    done
  fi

  printf "Restoring from Log Backup App Image"

  echo "Image: $logimagename RESTOREPATH: $LOGRESTOREPATH"
  mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

  if [ -z $LOGRESTOREPATH ]; then
    inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$logimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true"
     }'
   else
     inputParamsForMount='{
       "sessionid":"'"$sessionid"'",
       "image":"'"$logimagename"'",
       "host":"'"$hostID"'",
       "nowait":"true",
       "restoreoption":"'"mountpointperimage=$LOGRESTOREPATH"'"
     }'
   fi

  jobID=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount" |  jq -cr '(.result)' | cut -d ' ' -f1)
  printf "\n The Job id is: $jobID \n"
  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    printf "\n Reason: jobID cannot be null.\n"
    exit 1
  fi
  jobinprogress=true
  errorMessage=""
  while $jobinprogress
      do
          sleep 30
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
  printf "\n The Log Backup Mount job $jobID has $status.\n"

  if [ ! -z "$errorMessage" ]; then
    printf "\n Reason: $errorMessage.\n"
    exit 1
  fi

  if [ -z $LOGRESTOREPATH ]; then
    LOGRESTOREPATH=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
  fi
fi

if [ ! -z "$logimagename" ]; then
  if [ ! -z $LOGBACKUPPATH ]; then
    logbackupSuffix="${LOGBACKUPPATH#$LOGBACKUPFS}"
    echo "LogbackupSuffix: $logbackupSuffix"
    LOGRESTOREPATH=$LOGRESTOREPATH/$logbackupSuffix
    echo "Log Backup Updated Path: $LOGRESTOREPATH"
  fi
fi

HANARESTORECONFIG=/act/scripts/act_hana_dump_restore_config.conf
echo -e "# hana Restore config" > $HANARESTORECONFIG
echo -e "DBSID='$DBSID'" >> $HANARESTORECONFIG
echo -e "KEYNAME='$KEYNAME'" >> $HANARESTORECONFIG
echo -e "INSTANCENUM='$INSTANCENUM'" >> $HANARESTORECONFIG
echo -e "HANAVERSION='$VERSION'" >> $HANARESTORECONFIG
echo -e "PYTHON_PATH='$PYTHON_PATH'" >> $HANARESTORECONFIG
echo -e "RECDATABASE='$RECDATABASE'" >> $HANARESTORECONFIG
echo -e "BACKUPTYPE='$BACKUPTYPE'" >> $HANARESTORECONFIG
echo -e "RECOVERYTIME='$RECOVERYTIME'" >> $HANARESTORECONFIG
echo -e "DBRESTOREPATH='$DBRESTOREPATH'" >> $HANARESTORECONFIG
echo -e "LOGBACKUPRESTOREPATH='$LOGRESTOREPATH'" >> $HANARESTORECONFIG


cp -r /act/act_scripts/saphana/dump/act_saphana_dump_restore.sh /act/scripts/
chmod -R 755 /act/scripts/act_saphana_dump_restore.sh
chmod -R 755 $HANARESTORECONFIG
chmod -R 755 $PYTHON_PATH
#Python path give permissions
su - $targetOSUser -c "sh /act/scripts/act_saphana_dump_restore.sh"
