#!/bin/bash
set -x

DBSID=$1
KEYNAME=$2
DBPORT=$3
PORT=$4
HANABACKUPPATH=$5
HANAVERSION=$6

DATE=`date`
echo "************************** DATE:$DATE: start $DBSID database backup ******************"

if [ -z "$HANAVERSION" ]; then
   echo "************* HANAVERSION is missing *************"
   echo "check appid.xxx for setting the right HANAVERSION"
   exit 1
fi

WPWD=/act/scripts
source $WPWD/act_hana_dump_config.conf

TYPE=${SCHEDULE:`date +%w`:1}

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -x "select database_name, ACTIVE_STATUS from m_databases"`

for i in ${dbnames}; do
 if [ $i = "DATABASE_NAME,ACTIVE_STATUS" ]; then
   :
  else
    TSID=`echo $i |awk -F ',' '{print $1}'|awk -F '"' '{print $2}'`
    TDBS=`echo $i |awk -F ',' '{print $2}'|awk -F '"' '{print $2}'`
     if [ $TDBS = "YES" ]; then
     case $TYPE in
    I)
      HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/INCR_DATA_BACKUP
      if [[ $HANAVERSION == "2.0" ]]; then
        BACKUPSQL="BACKUP DATA INCREMENTAL FOR $TSID USING FILE ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_BACKUP'"
      else
        BACKUPSQL="BACKUP DATA INCREMENTAL USING FILE ('$HANABACKUPPATHTDB')"
      fi
      ;;
    F)
      mkdir -p $HANABACKUPPATH/$TSID
      userGroup=`ls -lrt $HANABACKUPPATH | grep $DBUSER | head -n 1 | awk -F" " '{print $4}'`
      chown -R $DBUSER:$userGroup  $HANABACKUPPATH/$TSID
      chmod 777 $HANABACKUPPATH/$TSID
      HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/COMPLETE_DATA_BACKUP
      if [[ $HANAVERSION == "2.0" ]]; then
        BACKUPSQL="BACKUP DATA FOR $TSID USING FILE ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_BACKUP'"
      else
        BACKUPSQL="BACKUP DATA USING FILE ('$HANABACKUPPATHTDB')"
      fi

      # Delete the last full from the staging disk to give us space to run a new one

      # First Get backup ID
      SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where ENTRY_TYPE_NAME = 'complete data backup' and COMMENT = 'ACTIFIO_BACKUP' and STATE_NAME = 'successful' "
      if [ $TSID = "SYSTEMDB" ]; then
        ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j $SQL | head -n1`
      else
        ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME$TSID -d $TSID -a -j $SQL | head -n1`
      fi
        ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

      # Get list of files from M_CATALOG_FILES
      SQL="SELECT DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE BACKUP_ID = '$ID'"
      if [ $TSID = "SYSTEMDB" ]; then
        FILES=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j $SQL | head -n -2`
      else
        FILES=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME$TSID -d $TSID -a -j $SQL | head -n -2`
      fi

      # Delete the files
      for i in `echo $FILES`; do
        rm -f `echo $i | sed s/\"//g`
      done
      ;;
    D)
      HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/DIFF_DATA_BACKUP
      if [[ $HANAVERSION == "2.0" ]]; then
        BACKUPSQL="BACKUP DATA DIFFERENTIAL FOR $TSID USING FILE ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_BACKUP'"
      else
        BACKUPSQL="BACKUP DATA DIFFERENTIAL USING FILE ('$HANABACKUPPATHTDB')"
      fi
      ;;
    *)
      echo No idea what Im going to do
      exit 1
      ;;
    esac

    echo "************************** backingup $TSID database backup ******************"

    # Perform the backup
    /usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j "$BACKUPSQL"
    if [ $? -ne 0 ]; then
      echo Aborting script
      exit 1
    fi

    if [ $TYPE = "F" ]; then

      # Now we need to uncatalog and delete everything older than the backup we just did
      SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where ENTRY_TYPE_NAME = 'complete data backup' and COMMENT = 'ACTIFIO_BACKUP' and STATE_NAME = 'successful' "
      if [ $TSID = "SYSTEMDB" ]; then
        ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j $SQL | head -n1`
      else
        ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME$TSID -d $TSID -a -j $SQL | head -n1`
      fi
      ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

      SQL="BACKUP CATALOG DELETE FOR $TSID ALL BEFORE BACKUP_ID $ID COMPLETE "
      /usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j $SQL

      # Saving the BACKUP_ID of the Full to assist with recovery
      echo $ID > $HANABACKUPPATH/$TSID/ACT_FULL_BACKUPID.txt
    fi

    # Now to grab the latest catalog backup and copy it
    SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
    if [ $TSID = "SYSTEMDB" ]; then
      BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME -a -j $SQL | head -n1 | sed s/\"//g`
    else
      BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $KEYNAME$TSID -d $TSID -a -j $SQL | head -n1 | sed s/\"//g`
    fi
    rm -f $HANABACKUPPATH/$TSID/log_backup_0_0_0_0*
    cp $BCATALOG $HANABACKUPPATH/$TSID
    echo "************************** end $TSID database backup ******************"
   else
     echo "$TSID database backup skipped as database is inactive state"
  fi
  fi
  #echo "************************** end $TSID database backup ******************"
done

echo "************************** end database backup ******************"
exit
