#!/bin/sh
set -x

source /act/scripts/act_hana_dump_restore_config.conf

DBUSER=$KEYNAME
PORT=$INSTANCENUM
DBPORT='HDB'$PORT

DBLIST=$RECDATABASE
DATE=`date`
echo "************************** DATE:$DATE: start $DBSID database restore ******************"

logMode=`grep log_mode /hana/shared/$DBSID/global/hdb/custom/config/global.ini`
logModeValue=$(echo "$logMode" | cut -d= -f2)
logModeValueTrimmed=$(echo "$logModeValue" | tr -d ' ')

if [[ $logModeValueTrimmed == *normal* ]]; then
	logVolume=`grep basepath_logbackup /hana/shared/$DBSID/global/hdb/custom/config/global.ini`
	if [[ ! -z "$logVolume" ]]; then
		logVolumeValue=$(echo "$logVolume" | cut -d= -f2)
		logBackupPath=$(echo "$logVolumeValue" | tr -d ' ')
	else
		logBackupPath="/usr/sap/$DBSID/$DBPORT/backup/log"
	fi
fi

waitforhdbnameserver() {
  #Set a timeout of 8 minutes (8*6*10)
  waitFlag=true
  timeout=0
  while $waitFlag; do
    timeout=$((timeout+1))
    if [ $timeout -gt 40 ]; then
      echo "Timeout after 20 minutes"
      exit 1
    fi
    nameserverProcess=`ps -ef | grep -i hdbnameserver | grep -v grep | grep -i $DBSID'adm'`
    if [ -z "$nameserverProcess" ]; then
       echo "Waiting for hdbnameserver process to be up..."
       sleep 30
       continue
    else
      sleep 30
      dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -x "select database_name from m_databases"`
      if [ $? -ne 0 ]; then
         echo "Not yet able to execute db select query on m_databases..Retry"
         waitFlag=true
      else
         waitFlag=false
      fi
    fi
  done
  return 0
}

getreccmd() {

fbtype=$1
ftsid=$2
fbkppath=$3
frtime=$4
flogpath=$5

if [ ! -z $flogpath ]; then
  flogpath=$flogpath/$ftsid
fi

if [ "$fbtype" = "FULL" ] && [ "$ftsid" = "SYSTEMDB" ]; then
  SQL=HDBSettings.sh $PYTHON_PATH/recoverSys.py --command="RECOVER DATA USING FILE ('$fbkppath/$ftsid/COMPLETE_DATA_BACKUP') CLEAR LOG"
else if [ "$fbtype" = "FULL" ] && [ "$ftsid" != "SYSTEMDB" ]; then
  SQL="RECOVER DATA FOR $ftsid USING FILE ('$fbkppath/$ftsid/COMPLETE_DATA_BACKUP') CLEAR LOG"
fi
fi
if [ "$fbtype" = "INCREMENTAL" ] && [ "$ftsid" = "SYSTEMDB" ]; then
  if [ -z "$frtime" ]; then
    echo "Recovery time should be provided if backup type is incremental"
    exit 1
  else
   if [ -z "$flogpath" ]; then
    if [ -z "$frtime" ]; then
      SQL=HDBSettings.sh $PYTHON_PATH/recoverSys.py --command="RECOVER DATA USING FILE ('$fbkppath/$ftsid/COMPLETE_DATA_BACKUP') CLEAR LOG"
    else
      SQL=HDBSettings.sh $PYTHON_PATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$frtime' CLEAR LOG USING CATALOG PATH ('$fbkppath/$ftsid/') USING DATA PATH ('$fbkppath/$ftsid/');"
    fi
   else
      SQL=HDBSettings.sh $PYTHON_PATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$frtime' CLEAR LOG USING CATALOG PATH ('$flogpath/$ftsid/') USING DATA PATH ('$fbkppath/$ftsid/') USING LOG PATH ('$flogpath/','/"$logBackupPath/"');"
    fi
  fi
else if [ "$fbtype" = "INCREMENTAL" ] && [ "$ftsid" != "SYSTEMDB" ]; then
  if [ -z "$frtime" ]; then
    echo "Recovery time should be provided if backup type is incremental"
    exit 1
  else
    if [ ! -z $flogpath ]; then
      SQL="RECOVER DATABASE FOR $ftsid UNTIL TIMESTAMP '$frtime' CLEAR LOG USING DATA PATH ('$fbkppath/$ftsid/') USING LOG PATH ('$flogpath/')"
    else
      SQL="RECOVER DATABASE FOR $ftsid UNTIL TIMESTAMP '$frtime' CLEAR LOG USING DATA PATH ('$fbkppath/$ftsid/')"
    fi
  fi
  fi
fi
return 0

}

if [ ! -z "$BACKUPTYPE" ] && [ "$BACKUPTYPE" = "FULL" ]; then
   if [ ! -z "$DBLIST" ] && [ "$DBLIST" = "ALL" ]; then
RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH`
#$RECCMD
if [ $? -ne 0 ]; then
   echo "failed to execute SYSTEMDB recovery"
   exit 1
fi
sleep 60
#Call generic function to check for hdbnameserver Process
waitforhdbnameserver
 echo "************************** HANA database restore for tenant databases ******************"
 dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -x "select database_name from m_databases"`
 for i in ${dbnames}; do
 if [ $i = "DATABASE_NAME" ]; then
   :
 else
   TSID=`echo $i |awk -F '"' '{print $2}'`

  if [ $TSID = "SYSTEMDB" ]; then
     echo "system DB is recovered successfully"
    :
  else
    echo "************************** restoring $TSID tenant database ******************"

#sqlcmd=`getreccmd $BACKUPTYPE $TSID $DBRESTOREPATH`
sqlcmd="RECOVER DATA FOR $TSID USING FILE ('$DBRESTOREPATH/$TSID/COMPLETE_DATA_BACKUP') CLEAR LOG"
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $sqlcmd
if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
 fi
fi
echo "************************** end $TSID tenant database restore ******************"
done
 else if [ ! -z "$DBLIST" ] && [ "$DBLIST" = "SYSTEMDB" ]; then
RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH "$RECOVERYTIME" $LOGBACKUPRESTOREPATH`
# $RECCMD
 if [ $? -ne 0 ]; then
   echo "failed to execute SYSTEMDB recovery"
   exit 1
fi
sleep 60
#Call generic function to check for hdbnameserver Process
waitforhdbnameserver
 else if [ ! -z "$DBLIST" ] && [ "$DBLIST" != "ALL" ]; then
    dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -x "select database_name from m_databases"`
    if [ $? -ne 0 ]; then
			echo "DB is down...hence depending on the Database Names Supplied"
      dbnames=`echo '"'$DBLIST'"' | sed "s/,/\" \"/g"`
   fi
 for i in ${dbnames}; do
   TSID=`echo $i |awk -F '"' '{print $2}'`
   TDBList=`echo $DBLIST | sed "s/,/ /g"`
   for j in ${TDBList}; do
  if [ "$TSID" = "SYSTEMDB" ] && [ "$TSID" = "$j" ]; then
RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH "$RECOVERYTIME" $LOGBACKUPRESTOREPATH`
 #$RECCMD
 if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
 sleep 60
 #Call generic function to check for hdbnameserver Process
 waitforhdbnameserver
  else if [ "$TSID" != "SYSTEMDB" ] && [ "$TSID" = "$j" ]; then
    echo "************************** restoring $TSID tenant database ******************"
#sqlcmd=`getreccmd $BACKUPTYPE $TSID $DBRESTOREPATH $RECOVERYTIME`
if [[ $HANAVERSION == "2.0" ]]; then
	stoptenantdbcmd="alter system stop Database $TSID"
	/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $stoptenantdbcmd
	if [ $? -ne 0 ]; then
		echo "failed to stop $TSID"
		exit 1
	fi
else
	#To update this command later for 1.0
	echo "Stop Tenant DB before performing Recovery."
fi
if [ -z "$LOGBACKUPRESTOREPATH" ]; then
   if [ -z "$RECOVERYTIME" ]; then
      sqlcmd="RECOVER DATA FOR $TSID USING FILE ('$DBRESTOREPATH/$TSID/COMPLETE_DATA_BACKUP') CLEAR LOG"
    else
      sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$DBRESTOREPATH/$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/')"
    fi
  else
    sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/') USING LOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/','/"$logBackupPath/"')"
fi

/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $sqlcmd
if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
  else
   echo "All options are completed"
 fi
fi
 done
 done
else
   echo "Invalid option for database list entered"
 fi
fi
fi
else if [ ! -z "$BACKUPTYPE" ] && [ "$BACKUPTYPE" = "INCREMENTAL" ]; then
     if [ ! -z "$DBLIST" ] && [ "$DBLIST" = "ALL" ]; then
  RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH "$RECOVERYTIME" $LOGBACKUPRESTOREPATH`
 #$RECCMD
 if [ $? -ne 0 ]; then
   echo "failed to execute SYSTEMDB recovery"
   exit 1
fi
sleep 60
#Call generic function to check for hdbnameserver Process
waitforhdbnameserver
    echo "************************** HANA database restore for tenant databases ******************"
  dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -x "select database_name from m_databases"`
  if [ $? -ne 0 ]; then
    echo "failed to fetch database from $DBUSER"
    exit 1
 fi
 for i in ${dbnames}; do
 if [ $i = "DATABASE_NAME" ]; then
   :
 else
   TSID=`echo $i |awk -F '"' '{print $2}'`

  if [ $TSID = "SYSTEMDB" ]; then
     echo "system DB is recovered successfully"
    :
  else
    echo "************************** restoring $TSID tenant database ******************"
#sqlcmd=`getreccmd $BACKUPTYPE $TSID $DBRESTOREPATH $RECOVERYTIME`
if [ -z "$LOGBACKUPRESTOREPATH" ]; then
   if [ -z "$RECOVERYTIME" ]; then
       sqlcmd="RECOVER DATA FOR $TSID USING FILE ('$DBRESTOREPATH/$TSID/COMPLETE_DATA_BACKUP') CLEAR LOG"
   else
      sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$DBRESTOREPATH/$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/')"
   fi
 else
sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/') USING LOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/','/"$logBackupPath/"')"
fi
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $sqlcmd
if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
 fi
fi
echo "************************** end $TSID tenant database restore ******************"
done
 else if [ ! -z "$DBLIST" ] && [ "$DBLIST" = "SYSTEMDB" ]; then
RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH "$RECOVERYTIME" $LOGBACKUPRESTOREPATH`
 #$RECCMD
 if [ $? -ne 0 ]; then
   echo "failed to execute SYSTEMDB recovery"
   exit 1
fi
 sleep 60
 #Call generic function to check for hdbnameserver Process
 waitforhdbnameserver
 else if [ ! -z "$DBLIST" ] && [ "$DBLIST" != "ALL" ]; then
    dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -x "select database_name from m_databases"`
    if [ $? -ne 0 ]; then
			echo "DB is down...hence depending on the Database Names Supplied"
      dbnames=`echo '"'$DBLIST'"' | sed "s/,/\" \"/g"`
   fi
 for i in ${dbnames}; do
   TSID=`echo $i |awk -F '"' '{print $2}'`
   TDBList=`echo $DBLIST | sed "s/,/ /g"`
   for j in ${TDBList}; do
  if [ "$TSID" = "SYSTEMDB" ] && [ "$TSID" = "$j" ]; then
RECCMD=`getreccmd $BACKUPTYPE SYSTEMDB $DBRESTOREPATH "$RECOVERYTIME" $LOGBACKUPRESTOREPATH`
 #$RECCMD
 if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
 sleep 60
 #Call generic function to check for hdbnameserver Process
 waitforhdbnameserver
 else if [ "$TSID" != "SYSTEMDB" ] && [ "$TSID" = "$j" ]; then
    echo "************************** restoring $TSID tenant database ******************"
#sqlcmd=`getreccmd $BACKUPTYPE $TSID $DBRESTOREPATH $RECOVERYTIME`
if [[ $HANAVERSION == "2.0" ]]; then
	stoptenantdbcmd="alter system stop Database $TSID"
	/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $stoptenantdbcmd
	if [ $? -ne 0 ]; then
		echo "failed to stop $TSID"
		exit 1
	fi
else
	#To update this command later for 1.0
	echo "Stop Tenant DB before performing Recovery."
fi
if [ -z "$LOGBACKUPRESTOREPATH" ]; then
   if [ -z "$RECOVERYTIME" ]; then
      sqlcmd="RECOVER DATA FOR $TSID USING FILE ('$DBRESTOREPATH/$TSID/COMPLETE_DATA_BACKUP') CLEAR LOG"
   else
     sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$DBRESTOREPATH/$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/')"
   fi
#if [ ! -z "$LOGBACKUPRESTOREPATH" ]; then
 else
sqlcmd="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/') USING DATA PATH ('$DBRESTOREPATH/$TSID/') USING LOG PATH ('$LOGBACKUPRESTOREPATH/DB_$TSID/','/"$logBackupPath/"')"
fi
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $sqlcmd
if [ $? -ne 0 ]; then
   echo "failed to execute $TSID recovery"
   exit 1
fi
  else
   echo "All options are completed"
 fi
fi
 done
 done
else
   echo "Invalid option for database list entered"
 fi
fi
fi
else
   echo "Invalid backup type is specified"
   exit 1
fi
fi

exit
