#!/bin/bash
set -x

WPWD=/act/scripts
source $WPWD/act_hana_dump_config.conf

echo "-----------Starting addVdisk script-------"
VDISK_CONFIG_FILE="/act/act_scripts/saphana/automation/Configuration/vdiskmap.conf"
if [ ! -f "$VDISK_CONFIG_FILE" ]; then
  #Create VDISK_CONFIG_FILE
  echo -e "# vdisk name and mountpoint mapping config " > $VDISK_CONFIG_FILE
fi
vdiskmountpoint=$HANABACKUPPATH
vdiskdevicename=""
mountpoint=`echo "$vdiskmountpoint" | awk -F/ '{print $NF}'`
vgName=$mountpoint'dumpvg'
lvName=$mountpoint'dumplv'
SkyUserName='admin'
SKYCONFIG=/act/scripts/skyconf/$SkyIP'_details'
if [ ! -f "$SKYCONFIG" ]; then
	SkyPassword='password'
else
	source $SKYCONFIG
	SkyPassword=`echo $enSkyPassword | openssl base64 -d`
fi
VendorKey='act'

HVERSION=`su - $DBUSER -c "HDB version | grep -i version: | cut -d':' -f2 | sed -e 's/^[ \t]*//'"`

hanaVersionCheck=`echo $HVERSION | cut -c1-6 | sed -e 's/\.//g'`

VERSION=`echo $HVERSION | cut -c1-3`

if [ $VERSION == "2.0" ]; then
dataSize=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"SELECT SUM(USED_SIZE) AS DBSIZE_bytes FROM \"SYS_DATABASES\".\"M_VOLUME_FILES\" WHERE FILE_TYPE = 'DATA'\""`
retval=$?
else
dataSize=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"SELECT SUM(USED_SIZE) AS DBSIZE_bytes FROM \"M_VOLUME_FILES\" WHERE FILE_TYPE = 'DATA'\""`
retval=$?
fi

retval=$?
if [ $retval -ne 0 ]; then
  echo "*** ERROR: Query Failure to get DB size using key $KEYNAME ************"
  exit $retval
fi

for i in ${dataSize}; do
 if [ $i = "DBSIZE_bytes" ]; then
   :
else
  dbSizeBytes=$i
fi
done

if [ -z "$dbSizeBytes" ]; then
  echo " Error : Hana Total DB Size cannot be null"
  exit 1
fi
# Create a Vdisk of 3 times the DB size on Sky.
vdisksize=`expr 3 \* $dbSizeBytes`

createStagingDisk()
{
  vdisksize=$1
  by=10485760; ceilFunc=$(( ($vdisksize + $by -1 )/$by )); vdisksize=`expr $by \* $ceilFunc`;
  loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
  sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
  host=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=`hostname`" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

  addVdiskURL="https://$SkyIP/actifio/api/task/mkvdiskmap"
  inputParamsvDisk='{
  "sessionid":'"$sessionid"',
  "host":'"$host"',
  "size":'"$vdisksize"',
  "diskpool":'"act_per_pool000"'
  }'
  vdiskResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $addVdiskURL -d "$inputParamsvDisk")
  echo "vdiskResponse :" $vdiskResponse
  if [[ $vdiskResponse == *errormessage* ]]; then
    error=$(echo $vdiskResponse | jq -cr '.errormessage')
    echo "Add Vdisk has failed": $error
    exit 1
  else
    if [[ $vdiskResponse == *result* ]]; then
      vdiskname=$(echo $vdiskResponse | jq -cr '.result')
      echo "vdiskname :" $vdiskname
      echo -e "$vdiskname=$vdiskmountpoint" >> $VDISK_CONFIG_FILE
      #Step 4: Login into iscsi service
      iscsiSessionId=`iscsiadm -m session | grep $SkyIP':' | awk -F" " '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1`
      iscsiadm -m session -r $iscsiSessionId --rescan
      sleep 30
      reportMappedDiskUrl="https://$SkyIP/actifio/api/report/reportmappeddisks?sessionid=$sessionid"
      LUNID=$(curl -w "\n" -s -k -G $reportMappedDiskUrl | jq -cr '.result[] | select(.VDiskName=="'"$vdiskname"'") | .LUNID')
      if [ -z "$LUNID" ]; then
        echo "Error : No LUNID found for the staging vdisk"
        exit 1
      fi
      vdiskdevicename=`ls -lrt /dev/disk/by-path/ | grep $SkyIP | grep -w lun-$LUNID | tac | awk -F/ '{print $NF}' | head -n 1`
      if [ -z "$vdiskdevicename" ]; then
        echo "Error : No vdisk mapping device found."
        exit 1
      fi
      echo "Staging Disk Device Name :" $vdiskdevicename
    fi
  fi
}

addDiskInExtendedVG()
{
    echo "vdiskdevicename :" $vdiskdevicename
    mkfs -t xfs -f /dev/$vdiskdevicename
    pvcreate -f /dev/$vdiskdevicename
    vgextend $vgName /dev/$vdiskdevicename
    freePE=`vgdisplay $vgName 2> /dev/null | grep "Free  PE / Size" |  awk '{print $5}'`
    lvextend -l +$freePE /dev/$vgName/$lvName
    xfs_growfs /dev/$vgName/$lvName
}

echo "-------Check if there is any staging disk and also if the disk size is sufficient for Hana DB-------"
mountpointresult=$(lsblk -b -io MOUNTPOINT | awk '$1=="'"$vdiskmountpoint"'"' | head -n 1)
echo "Mount point result if exists:: " $mountpointresult

vgExists=`lsblk | grep $vgName'-'$lvName`
if [ ! -z "$vgExists" ]; then
  if [ -z "$mountpointresult" ]; then
    mount /dev/$vgName/$lvName $vdiskmountpoint
  fi
  lvmSize=`vgs --noheadings --nosuffix --units b $vgName 2>/dev/null`
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** ERROR: Existing Staging Disk Mountpoint is Faulty with no VG $vgName present ************"
    exit $retval
  fi
  lvmSize=`echo $lvmSize | awk '{print $6}'`
  if [[ `echo "$dbSizeBytes $lvmSize" | awk '{print ($1 >= $2)}'` == 1 ]]; then
    requiredVdiskSize=`expr $vdisksize - $lvmSize`
    echo "----Creating another vdisk of size $requiredVdiskSize----"
    createStagingDisk $requiredVdiskSize
    echo "----Extending VolumeGroup-----"
    addDiskInExtendedVG
    echo "-----Disk Created in VG successfully-----"
    echo "--Fix: Return Non Zero Exit Status for Disk Expansion during backup job--"
    exit 123
  else
    echo "-----Found/Verified that existing Staging Disk has enough space for Hana DB backup-----"
  fi
else
  echo "Error: Atleast 1 disk has to exist-----"
  exit 1
fi
echo "----------end validate vdisk script----------"
exit
