#!/bin/sh
set -x

DBSID=$1
DBUSER=$2
DBPORT=$3
PORT=$4
DELLOG=$5
HANAVERSION=$6


DATA=`date`
echo "************************** DATE:$DATE: start log delete post script for $DBSID database ******************"

if [ -z "$HANAVERSION" ]; then
   echo "************* HANAVERSION is missing *************"
   echo "check appid.xxx for setting the right HANAVERSION"
   exit
fi


if [ "$HANAVERSION" = "1.0" ]
then
SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME IN ('complete data backup','data snapshot') and STATE_NAME = 'successful' "
ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL | head -n1`
ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

if [ "$ID" = "?" ]
then
 echo "********** !!! WARNING !!!: no backup ID found to delete the log for database older then $DELLOG *********"
 exit
fi

SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL

else # For 2.0

SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME IN ('complete data backup','data snapshot') and STATE_NAME = 'successful' "
ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL | head -n1`
ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

if [ "$ID" = "?" ]
then
 echo "********** !!! WARNING !!!: no backup ID found to delete the log for systemdb database for older then $DELLOG *********"
 exit
fi

echo "************************** delete systemdb log ******************"
SQL="BACKUP CATALOG DELETE FOR SYSTEMDB ALL BEFORE BACKUP_ID $ID COMPLETE "
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL

echo "************************** delete tenant log ******************"

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j -x "select database_name from m_databases"`
if [ $? -ne 0 ]; then
	echo "failed to fetch database using $DBUSER"
	exit
fi

for dbname in ${dbnames}; do
	if [ $dbname = "DATABASE_NAME" ]; then
		:
	else
    TSID=`echo $dbname | awk -F '"' '{print $2}'`
    if [ $TSID = "SYSTEMDB" ]; then
      :
    else
      SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME IN ('complete data backup','data snapshot') and STATE_NAME = 'successful' "
      ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER$TSID -a -j $SQL | head -n1`
      ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

      SQLT="SELECT TOP 1 DATABASE_NAME from M_DATABASES where DATABASE_NAME <> 'SYSTEMDB'"
      TSID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER$TSID -a -j $SQLT | head -n1`
      TSID=`echo "$TSID" | sed -e 's/^"//' -e 's/"$//'`

      if [ "$ID" = "?" ]
      then
       echo "********** !!! WARNING !!!: no backup ID found to delete the log for $TSID database for older then $DELLOG *********"
      fi

      SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
      /usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER$TSID -a -j $SQL
    fi
  fi
done


fi #Ends for 2.0
echo "************************** end log delete post script ******************"
exit
