#!/bin/sh
# This script will take care of the following tasks:
# 1) Login to appliance to get Session ID.
# 2) Create Vdisk from sky to host .
# 3) Rescan iscsi services.
#########################################################################

set -x

VDISK_CONFIG_FILE="/act/act_scripts/sybase/automation/Configuration/vdiskmap.conf"

if [ "$#" -lt 3 ]; then
echo "This script need 3 input parameter"
echo "usage: addVdisk.sh /<vdiskmountpoint> <hostid/unique hostname> <SRC_BACKUP_USER>"
echo " For example: addVdisk.sh  /actifiobackup 3456 actuser"
exit 1
fi

echo "-----------Starting addVdisk script-------"
if [ ! -f "$VDISK_CONFIG_FILE" ]; then
  #Create VDISK_CONFIG_FILE
  echo -e "# vdisk and mountpoint mapping config " > $VDISK_CONFIG_FILE
fi

vdiskmountpoint=$1
host=$2
DBUSER=$3
#KEYNAME=$4
vdiskdevicename=""
mountpoint=`echo "$vdiskmountpoint" | awk -F/ '{print $NF}'`
vgName=$mountpoint'dumpvg'
lvName=$mountpoint'dumplv'

#dataSize=`su - $DBUSER -c "hdbsql -U $KEYNAME -x \"SELECT SUM(USED_SIZE) AS DBSIZE_bytes FROM \"SYS_DATABASES\".\"M_VOLUME_FILES\" WHERE FILE_TYPE = 'DATA'\""`
#retval=$?
#if [ $retval -ne 0 ]; then
  #echo "*** ERROR: Query Failure to get DB size using key $KEYNAME ************"
  #exit $retval
#fi

set +x
dbSize=$(echo $SRC_USER_PASSWD |  su - $OSUSER -c "sh /act/act_scripts/sybase/automation/Backup/dbHelper.sh dbsize")
retval=$?
if [ $retval -gt 0 ]; then
  echo "****** ERRORMSG: failed to execute dbsize function ****"
  exit 1
fi
set -x
dbSize=`echo $dbSize | awk '{sub(/^.*dbSizeQuery/,""); print $1}'`
if [ -z "$dbSize" ]; then
   echo " Error : Query Failure to get DB size "
   exit 1
fi

echo "$dbSize"

dbSizeBytes=`expr 1000000 \* $dbSize`
echo "$dbSizeBytes"

if [ -z "$dbSizeBytes" ]; then
  echo " Error : Sybase Total DB Size cannot be null"
  exit 1
fi

# Create a Vdisk of 3 times the DB size on Sky.
vdisksize=`expr 3 \* $dbSizeBytes`

sessionExists=`iscsiadm -m session | grep -w $SkyIP:3260`
if [ -z "$sessionExists" ]; then
  echo "*** ERROR: Iscsi Session to $SkyIP does not exist ************"
  exit 1
fi
iscsiSessionId=`iscsiadm -m session | grep $SkyIP':' | awk -F" " '{print $2}' | cut -d "[" -f2 | cut -d "]" -f1`
iscsiadm -m session -r $iscsiSessionId --rescan
sleep 30

createStagingDisk()
{
  vdisksize=$1
  by=10485760; ceilFunc=$(( ($vdisksize + $by -1 )/$by )); vdisksize=`expr $by \* $ceilFunc`;
  set +x
  loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
  sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
  set -x

  profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
  poolName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .performancepool)')

  if [ -z "$poolName" ]; then
    echo "Error: PoolName cannot be empty"
    exit 1
  fi

  addVdiskURL="https://$SkyIP/actifio/api/task/mkvdiskmap"
  inputParamsvDisk='{
  "sessionid":'"$sessionid"',
  "host":'"$host"',
  "size":'"$vdisksize"',
  "diskpool":'"$poolName"'
  }'
  vdiskResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $addVdiskURL -d "$inputParamsvDisk")
  echo "vdiskResponse :" $vdiskResponse
  if [[ $vdiskResponse == *errormessage* ]]; then
    error=$(echo $vdiskResponse | jq -cr '.errormessage')
    echo "Add Vdisk has failed": $error
    exit 1
  else
    if [[ $vdiskResponse == *result* ]]; then
      vdiskname=$(echo $vdiskResponse | jq -cr '.result')
      echo "vdiskname :" $vdiskname
      echo -e "$vdiskname=$vdiskmountpoint" >> $VDISK_CONFIG_FILE
      #Step 4: Login into iscsi service
      iscsiadm -m session -r $iscsiSessionId --rescan
      sleep 30
      reportMappedDiskUrl="https://$SkyIP/actifio/api/report/reportmappeddisks?sessionid=$sessionid"
      LUNID=$(curl -w "\n" -s -k -G $reportMappedDiskUrl | jq -cr '.result[] | select(.VDiskName=="'"$vdiskname"'") | .LUNID')
      if [ -z "$LUNID" ]; then
        echo "Error : No LUNID found for the staging vdisk"
        exit 1
      fi
      vdiskdevicename=`ls -lrt /dev/disk/by-path/ | grep $SkyIP | grep -w lun-$LUNID | tac | awk -F/ '{print $NF}' | head -n 1`
      if [ -z "$vdiskdevicename" ]; then
        echo "Error : No vdisk mapping device found."
        exit 1
      fi
      echo "Staging Disk Device Name :" $vdiskdevicename
    fi
  fi
}

addDiskInNewVG()
{
    echo "vdiskdevicename :" $vdiskdevicename
    pvNames=`pvs -o pv_uuid,pv_name,vg_name 2> /dev/null | grep $vgName | awk '{print $2}'`
    vgremove -f $vgName 2> /dev/null
    if [ ! -z "$pvNames" ]; then
      pvNames=$(echo $pvNames | tr " " "\n")
      for pvName in $pvNames
      do
          pvremove $pvName
      done
    fi
    mkfs -t xfs -f /dev/$vdiskdevicename
    pvcreate -f /dev/$vdiskdevicename
    vgcreate $vgName /dev/$vdiskdevicename
    lvcreate -l 100%VG -n $lvName $vgName
    mkfs -t xfs -f /dev/$vgName/$lvName
    mkdir -p $vdiskmountpoint
    mount /dev/$vgName/$lvName $vdiskmountpoint
    mountpointresult=$(lsblk -b -io MOUNTPOINT | awk '$1=="'"$vdiskmountpoint"'"' | head -n 1)
    echo "mountpointresult :" $mountpointresult
    if [ -z "$mountpointresult" ]; then
      echo "Error : $SYBASEBACKUPPATH mount point could not be created."
      exit 1
    fi
    userGroup=`ls -lrt $SRC_SYBASE_SQLD | grep $OSUSER | tail -n 1 | awk -F" " '{print $4}'`
    BASE_VDISKMOUNTPOINT=$(echo "$vdiskmountpoint" |cut -d "/" -f2)
    chmod -R 777 /$BASE_VDISKMOUNTPOINT
    chown -R $OSUSER:$userGroup /$BASE_VDISKMOUNTPOINT
}

addDiskInExtendedVG()
{
    echo "vdiskdevicename :" $vdiskdevicename
    mkfs -t xfs -f /dev/$vdiskdevicename
    pvcreate -f /dev/$vdiskdevicename
    vgextend $vgName /dev/$vdiskdevicename
    freePE=`vgdisplay $vgName 2> /dev/null | grep "Free  PE / Size" |  awk '{print $5}'`
    lvextend -l +$freePE /dev/$vgName/$lvName
    xfs_growfs /dev/$vgName/$lvName
}

echo "-------Check if there is any staging disk and also if the disk size is sufficient for Sybase DB-------"
mountpointresult=$(lsblk -b -io MOUNTPOINT | awk '$1=="'"$vdiskmountpoint"'"' | head -n 1)
echo "Mount point result if exists:: " $mountpointresult

vgExists=`lsblk | grep $vgName'-'$lvName`
if [ ! -z "$vgExists" ]; then
  if [ -z "$mountpointresult" ]; then
    mount /dev/$vgName/$lvName $vdiskmountpoint
  fi
  lvmSize=`vgs --noheadings --nosuffix --units b $vgName`
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** ERROR: Existing Staging Disk Mountpoint is Faulty with no VG $vgName present ************"
    exit $retval
  fi
  lvmSize=`echo $lvmSize | awk '{print $6}'`
  if [[ `echo "$dbSizeBytes $lvmSize" | awk '{print ($1 >= $2)}'` == 1 ]]; then
    requiredVdiskSize=`expr $vdisksize - $lvmSize`
    echo "----Creating another vdisk of size $requiredVdiskSize----"
    createStagingDisk $requiredVdiskSize
    echo "----Extending VolumeGroup-----"
    addDiskInExtendedVG
    echo "-----Disk Created in VG successfully-----"
  else
    echo "-----Found/Verified that existing Staging Disk has enough space for Sybase DB backup-----"
  fi
else
  umount $vdiskmountpoint 2> /dev/null
  echo "----Create a staging disk-------"
  createStagingDisk $vdisksize
  echo "----Creating VolumeGroup-----"
  addDiskInNewVG
  echo "-----Disk Created in VG successfully-----"
fi
mountDir=$(echo "$vdiskmountpoint" | sed 's/\//\\\//g')
grep -q '^/dev/'$vgName'/'$lvName'.*' /etc/fstab &&  sed -i 's/^\/dev\/'$vgName'\/'$lvName'.*/\/dev\/'$vgName'\/'$lvName' '$mountDir' xfs _netdev,nofail 0 0/g' /etc/fstab || echo -e '/dev/'$vgName'/'$lvName $vdiskmountpoint' xfs _netdev,nofail 0 0' >> /etc/fstab
echo "----------end vdisk script----------"
