#!/bin/sh
set +x
########################################################################################################################################################
# Script Version# - Version 1.0
# Parent script to invoke the below listed scripts. Scripts / commands are listed below:
# 1) preRequisites.sh
#    Script functionality below:
#     - Check Network Connectivity and other validations for required inputs.(Fail if anything is missing).
#     - Copy connector into Host System.
#     - Install Connector.
#     - Install/Start ISCSI service if Needed.
# 2) Execute the sybaseConfig.sh script.
#     - This will give data to be used while creating the generic app and log volumes.
# 3) ApplianceOperations.sh
#    Script functionality below:
#     - Add Host.
#     - Enable CBT.
#     - Discover Apps on Actifio Appliance.
########################################################################################################################################################

#set -x
if [ "$#" -lt 1 ]; then
  echo " This script needs key-value input parameters"
  echo " usage: backupAutomation.sh <name=value>"
  echo " For example: backupAutomation.sh OSUSER=sybase"
  exit 1
fi

# Check if JQ and CURL is installed
if ! type "jq" > /dev/null 2>&1; then
  echo "Error : JQ is not installed"
  exit 1
fi

if ! type "curl" > /dev/null 2>&1; then
  echo "Error : curl is not installed"
  exit 1
fi

backupConfFile="/act/act_scripts/sybase/automation/Configuration/sybasebackup.conf"

echo -e '#Sybase DB/Logbackup Specific Params' > $backupConfFile
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   passwdstrCheck=`echo $ARGUMENT | grep -i "password\|passwd"`
   if [ -z "$passwdstrCheck" ]; then
     echo -e $ARGUMENT >> $backupConfFile
   else
     export $ARGUMENT
   fi
done

echo -e '#Custom Policy Settings if template is not pre-created for DB and Log Templates' >> $backupConfFile
echo -e 'dbbackup_rpo="24"' >> $backupConfFile
echo -e 'logbackup_rpo="1"' >> $backupConfFile

echo -e '#Key used for Licensing purpose' >> $backupConfFile
echo -e 'VendorKey="act"' >> $backupConfFile
#echo -e '#Profile name used during backup' >> $backupConfFile

if ! grep -q "ProfileName"  "$backupConfFile"; then
 echo -e 'ProfileName="LocalProfile"' >> $backupConfFile
fi

#Fetch Sybase HOME
#HOME=`su - $OSUSER -c "env | grep ^HOME= | cut -d'=' -f2"`
#Set Sybase home value into Config File
#echo -e "HOME='$HOME'" >> $backupConfFile

source $backupConfFile

if [ -z "$SRC_USER_PASSWD" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "/sybase/"$SRC_SERVER_NAME"/dba/bin/dbbp MaGiCpW"`
  if [ -z "$SRC_USER_PASSWD" ]; then
    echo " Error : ISQL Password cannot be empty."
    exit 1
  fi
fi

orig_pass_temp=$SRC_USER_PASSWD
SRC_USER_PASSWD=`printf '%q' "$SRC_USER_PASSWD"`

#Checking if all mandatory parameters are present.
if [ -z $LogBackupEnable ]; then
   echo " Error : LogBackupEnable cannot be empty."
   exit 1
fi

if [[ $LogBackupEnable == "true" ]]; then
  if [ -z $DELLOG ]; then
     echo " Error : DELLOG cannot be empty."
     exit 1
  fi
fi

if [ -z $SkyIP ]; then
   echo " Error : Sky IP cannot be empty."
   exit 1
fi

if [ -z $ASETemplateName ]; then
   echo " Error : Template name cannot be empty."
   exit 1
fi

if [ -z $dbbackup_rpo ]; then
   echo " Error : DB Backup RPO cannot be empty."
   exit 1
fi

if [ -z $OSUSER ]; then
   echo " Error : OSUSER cannot be empty."
   exit 1
fi

if [ -z $SRC_SYBASE_SQLD ]; then
   echo " Error : Sybase home location cannot be empty."
   exit 1
fi

if [ -z $SRC_BACKUP_USER ]; then
   echo " Error : Backup user cannot be empty."
   exit 1
fi

if [ -z $SRC_SERVER_NAME ]; then
   echo " Error : Server name cannot be empty."
   exit 1
fi

if [ -z $BACKUP_TYPE ]; then
   BACKUP_TYPE="Filesystemdump"
fi

if [ "$BACKUP_TYPE" == "Filesystemdump" ]; then
  if [ -z $SYBASEBACKUPPATH ]; then
   echo " Error : SYBASEBACKUPPATH cannot be empty."
   exit 1
  fi
fi

if [ -z $SYBASEBACKUPPATH ] && [ -z $GenericAppName ]; then
  echo " Error : DB Application Name cannot be empty."
  exit 1
fi

if [ -z $SRC_HOSTNAME ]; then
  SRC_HOSTNAME=`hostname`
fi

if [ "$BACKUP_TYPE" != "LVMSnapshot" ] && [ "$BACKUP_TYPE" != "Filesystemdump" ] ; then
    echo "Exiting!!!...Incorrect Backup Type $BACKUP_TYPE Expected Values are LVMSnapshot/Filesystemdump"
    exit 1
fi

if [ "$BACKUP_TYPE" == "LVMSnapshot" ]; then
  if [ -z $VgSnapReserveSpace ]; then
    echo " Error : Percentage of reserved space in Volume Group[ 1-100 desired 20] cannot be empty."
    exit 1
  fi
  if [ -z $GenericAppName ]; then
    echo " Error : DB Application Name cannot be empty."
    exit 1
  fi
fi

source /act/act_scripts/sybase/automation/Backup/paramValueValidator.sh
validateDbParams

if [ -z $STRIPEON ]; then
stripeonQuery="select
case
when convert(int, sum(size)*(@@maxpagesize/(1024*1024.0))) between 0 and 100000
then 1
when convert(int, sum(size)*(@@maxpagesize/(1024*1024.0))) between 100001 and 500000
then 2
when convert(int, sum(size)*(@@maxpagesize/(1024*1024.0))) between 500001 and 1500000
then 4
when convert(int, sum(size)*(@@maxpagesize/(1024*1024.0))) > 1500001
then 8
else
1
end as stripes
from master..sysusages where dbid = db_id(@@servername) and segmap <> 4"

STRIPEON=$(su - $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror" <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
$stripeonQuery
go
quit
EOF
)

  retval=$?
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: stripeonQuery has Failed!! $STRIPEON"
    echo "Exiting...."
    exit 1
  fi

  if [ -z $STRIPEON ]; then
     echo "Assigning default value of stripeon to 4"
     STRIPEON="4"
  fi
  STRIPEON=$(echo "$STRIPEON" | tr -d '[:space:]')
fi
echo "STRIPEON value: $STRIPEON"

SRC_SERVER_NAME=`su - $OSUSER -c "showserver" | grep -i dataserver | grep -i "\-s$SRC_SERVER_NAME" | awk '{sub(/^.*-s/,""); print $1}'`
if [ -z $SRC_SERVER_NAME ]; then
  echo "Error: Invalid ASE Server Name"
  exit 1
fi

#Checking Sybase version
versionQuery="select @@version"
version=$(su - $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror" <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
$versionQuery
go
quit
EOF
)

retval=$?
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: Select Version Query has Failed!!  $version"
fi

SRC_USER_PASSWD=$orig_pass_temp
#versionNumber=`echo "$version" | sed -n '1!p' | cut -d / -f2 | cut -d " " -f1 | sed -n '2!p' | sed -e 's/\.//g'`
versionNumber=`echo $version |  cut -d">" -f2 | cut -d / -f2 | cut -d " " -f1 | sed -e 's/\.//g'`
if [[ `echo "$versionNumber 157" | awk '{print ($1 < $2)}'` == 1 ]]; then
  echo "Error: Supported Sybase DB Version is 15.7 or above"
  exit 1
fi

if [ -z $SkyPassword ]; then
  echo " Error : Sky Password cannot be empty."
  exit 1
fi

SkyUserName="admin"

echo "-----------------------------------------------------"
echo "-----Starting ASE DB Backup Configuration-------"
echo "-----------------------------------------------------"


# Step 1 : Invoking PreRequisites.sh script to check pre-requisites (UDSAgent and ISCSI service)
echo "----------------------------------------------------------"
echo "---Start Phase: Checking Pre-Requisites on Host machine---"
echo "----------------------------------------------------------"
source /act/act_scripts/sybase/automation/Backup/preRequisites.sh $SkyIP

# Step 2 : Execute sybaseconfig.sh remotely to get data for creating generic app and the log volumes.
source /act/act_scripts/sybase/automation/Backup/sybaseConfig.sh

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "-----End Phase: Completed Checking Pre-Requisites-----"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"

iqnName=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`

# Step 3 : Invoking ApplianceOperations.sh script to perform Add host, Enable CBT and Discover apps. It would internally invoke CreateGenericApp.sh and LogProtection.sh

if [ -z $iqnName ]; then
   echo " Error : IQN Name is blank."
   exit 1
fi
source /act/act_scripts/sybase/automation/Backup/applianceOperations.sh $iqnName

DisableMultipathCheck=true
grep -q '^DisableMultipathCheck.*' /act/config/connector.conf &&  sed -i 's/^DisableMultipathCheck.*/DisableMultipathCheck='"$DisableMultipathCheck"'/g' /act/config/connector.conf || echo -e 'DisableMultipathCheck='"$DisableMultipathCheck"'' >> /act/config/connector.conf

echo "------------------------------------------------------"
echo "----------Completed the Backup Configuration----------"
echo "------------------------------------------------------"
