#!/bin/sh
method=$1
source /act/scripts/act_Sybase_Src.conf
set +x
read -r SRC_USER_PASSWD
if [ -z "$SRC_USER_PASSWD" ]; then
  SRC_USER_PASSWD=`/sybase/"$SRC_SERVER_NAME"/dba/bin/dbbp MaGiCpW`
  if [ -z "$SRC_USER_PASSWD" ]; then
    echo " Error : ISQL Password cannot be empty."
    exit 1
  fi
fi

usage()
{
  echo "Error: Missing DB Function: Exiting"
  exit 1
}

getDbSize()
{
dbSizeQuery="select sum(ceiling (sum(case when u.segmap != 4 then u.size/1048576.*@@maxpagesize end ))) from master..sysdatabases d, master..sysusages u where u.dbid = d.dbid  and d.status not in (256,4096)"
dbSize=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
set nocount on
go
$dbSizeQuery
go
EOF
)
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute getDbSize Query due to: $dbSize"
  exit 1
fi
set +x
echo "dbSizeQuery$dbSize"
}

getNotificationDBList()
{
dbListQuery="select distinct sda.name from master..sysdatabases sda"
dblist=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
set nocount on
go
$dbListQuery
go
EOF
)
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute getNotificationDBList"
  exit 1
fi
  notificationDbList=`echo $dblist | sed s/' '/,/g`
  set +x
  echo "notificationDbListQuery$notificationDbList"
}

getNotificationLogdbnamesList()
{
dbQuery="select distinct sda.name from master..sysdatabases sda where sda.name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','saptempdb','sapsatempdb','sybsecurity','dbccdb')"
if [ "$SRC_DBNAME" != "" ]; then
  for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
  do
    dbname=$i
    DBNAMES="${DBNAMES}${DBNAMES:+,}'$dbname'"
  done
   dbQuery+=" and sda.name in ($DBNAMES)"
fi
set +x
dbnameslist=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
set nocount on
go
$dbQuery
go
EOF
)
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute dbnameslistForLogQuery due to: $dbnameslist"
  exit 1
fi
  dbnameslist=`echo $dbnameslist | sed s/' '/,/g`
set +x
  echo "dbnameslistForLogQuery$dbnameslist"
}

getLvmList()
{
lvmQuery="select distinct sde.phyname from master..sysdatabases sda, master..sysdevices sde,master..sysusages su where su.dbid=sda.dbid and su.vdevno=sde.vdevno"
if [ "$SRC_DBNAME" != "" ]; then
  for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
  do
    dbname=$i
    DBNAMES="${DBNAMES}${DBNAMES:+,}'$dbname'"
  done
   lvmQuery+=" and sda.name in ($DBNAMES)"
fi
set +x
lvmlist=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
set nocount on
go
$lvmQuery
go
EOF
)
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute getLvmList Query due to: $lvmlist"
  exit 1
fi
set +x
  echo "lvmListQuery$lvmlist"
}

generateDbnamesList()
{
tmp_loc=/act/touch
query="select name from master..sysdatabases where name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','saptempdb','sapsatempdb','sybsecurity','dbccdb')"
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME  -X -n -b -o $tmp_loc/db_names.txt --retserverror <<EOF
set nocount on
go
$query
go
quit
EOF
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute generateDbnamesList: `cat $tmp_loc/db_names.txt`"
  exit 1
fi
set +x
}

generateLvmnamesList()
{
inlist=`echo $SRC_DBNAME  | sed -e "s/^/'/" -e "s/\$/'/" -e "s/,/',\'/g"`
tmp_loc=/act/touch
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 -o $tmp_loc/lvmlist.txt --retserverror <<EOF
set nocount on
select distinct sda.name, sde.phyname  from master..sysdatabases sda, master..sysdevices sde, master..sysusages su where su.dbid=sda.dbid and su.vdevno=sde.vdevno and sda.name in ($inlist)
go
EOF
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute generateLvmnamesList: `cat $tmp_loc/lvmlist.txt`"
  exit 1
fi
set +x
}

quiesceDb()
{
  SRC_DBNAME=$1
  datadev_dir=$2
  for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
  do
    dbname=$i
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror <<EOF
use master
go
quiesce database $dbname hold $dbname for external dump to '$datadev_dir/Manifest_File_$dbname.dat' with override
go
exit
EOF

retval=$?
if [ "$retval" -gt 0 ]; then
 echo "ERROR: Unable to quiesce the database!."
 echo "*********** BACKUP_TYPE = LVM SNAPSHOT and BACKUP_STATUS = $dbname:fail ***************"
 exit 1
fi
done

echo "**** quiesced all user databases $SRC_DBNAME successfully ****"

DEVICE_MANIFEST_DETAILS=$datadev_dir/device_manifest_systemdb
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 2000  -o $DEVICE_MANIFEST_DETAILS --retserverror <<EOF
use master
go
set nocount on
go
select distinct db.name,dv.phyname from sysdatabases db, sysusages u, sysdevices dv where db.dbid=u.dbid and u.vdevno=dv.vdevno and phyname like '%.dat%' and db.name in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','saptempdb','sapsatempdb','sybsecurity','dbccdb')
go
EOF

retval=$?
if [ "$retval" -gt 0 ]; then
 echo "ERROR: Unable to get the systemdb device information during freeze!."
 echo "*********** BACKUP_TYPE = LVM SNAPSHOT and BACKUP_STATUS = fail ***************"
 exit 1
fi
backupVersionDetails $datadev_dir
}

releaseDb()
{
  SRC_DBNAME=$1
  for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
  do
    dbname=$i
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror <<EOF
use master
go
quiesce database $dbname release
go
exit
EOF
done

retval=$?
if [ $retval -gt 0 ]; then
 echo "ERROR: Failed to Release database!"
 echo "*********** BACKUP_TYPE = LVM SNAPSHOT and BACKUP_STATUS = $dbname:fail ***************"
 exit 1
fi

echo "**** Released all user databases $SRC_DBNAME successfully ****"
}

configureLogOptions()
{
grep SRC_BACKUP_LOCATION /act/scripts/act_Sybase_Src.conf | cut -d"=" -f2 | xargs | grep -w $SRC_BACKUP_LOCATION
retval=$?
invokedFromDbBackup=true
if [ $retval -gt 0 ]; then
  invokedFromDbBackup=false
fi

confPath=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror <<EOF
use master
go
set nocount on
go
sp_configure "configuration file"
go
sp_configure 'enable dump history', 1
go
EOF
)
retval=$?
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute confPath Query: $confPath"
fi
set -x
confPath=`echo $confPath | awk -F" " '{print $6}'`
findCfg=`find "$confPath" -name $SRC_SERVER_NAME'.cfg' | head -n 1`
cp -r "$findCfg" $SRC_BACKUP_LOCATION/
set +x
if [ "$invokedFromDbBackup" == "true" ]; then
  query="select distinct sda.name from master..sysdatabases sda"
else
  query="select distinct sda.name from master..sysdatabases sda where sda.name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','saptempdb','sapsatempdb','sybsecurity','dbccdb')"
fi
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 -o $SRC_BACKUP_LOCATION/source_dbname.txt --retserverror <<EOF
set nocount on
go
$query
go
EOF
retval=$?
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute sourcedbnameslist query: `cat $SRC_BACKUP_LOCATION/source_dbname.txt`"
  exit 1
fi
set -x
while read line
do
set -- $line
dbname=$1
set +x
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror <<EOF
use master
go
sp_dboption $dbname, 'allow incremental dumps', true
go
EOF
set -x
done < $SRC_BACKUP_LOCATION/source_dbname.txt
}

purgebackuphistory()
{
echo "**** Purge Backup History ***"
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
use master
go
sp_dump_history purge
go
EOF
retval=$?
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to purgebackuphistory"
fi
echo "**** Successfully purged backup history"
}

listbackuphistory()
{
echo "**** List Backup History for $dumptype ***"
dumptype=$1
status=$2
location=$3

if [ "$dumptype" == "TRAN" ]; then
  transtype="transactionalLog"
else
  transtype="databaseDump"
fi
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 2000 -o $location/dumphistory.txt --retserverror <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
sp_dump_history list, @dump_type='$dumptype', @status = '$status'
go
EOF

retval=$?
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to listbackuphistory .."
  while read line
  do
    echo "*********** BACKUP_TYPE = $transtype and BACKUP_STATUS = $line:$status ***************"
  done < $location/source_dbname.txt
else
  var=`cat $location/dumphistory.txt | sed -e '1,2d' | sed '/^[[:space:]]*$/d'`
  if [ ! -z "$var" ]; then
    while read line
    do
      backedupdbname=`echo $line | awk -F' ' '{print $3}'`
      backeduplocation=`echo $line | awk -F' ' '{print $9}'`
      echo "*********** BACKUP_TYPE = $transtype and BACKUP_STATUS = $backedupdbname:$status, location:$backeduplocation ***************"
    done <<< "$var"
  else
    while read line
    do
      echo "*********** BACKUP_TYPE = $transtype and BACKUP_STATUS = $line:$status ***************"
    done < $location/source_dbname.txt
  fi
  echo "**** Finished Listing Backup History for $transtype ***"
fi
}

executeLogDumpTransaction()
{
purgebackuphistory
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -i $SRC_BACKUP_LOCATION/syb_trans_dump --retserverror
retval=$?
if [ $retval -gt 0 ]; then
  listbackuphistory "TRAN" "fail" $SRC_BACKUP_LOCATION
  echo "**** ERRORMSG: Transactional Log Backup has Failed, status: fail *****"
  exit 1
fi
listbackuphistory "TRAN" "success" $SRC_BACKUP_LOCATION
}

executeDbDumpTransaction()
{
purgebackuphistory
$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -i $SRC_BACKUP_LOCATION/syb_backup_dump --retserverror
retval=$?
if [ $retval -gt 0 ]; then
  listbackuphistory "DATABASE" "fail" $SRC_BACKUP_LOCATION
  echo "***** ERRORMSG: Database Dump Backup has Failed, status: fail *****"
  exit 1
fi
listbackuphistory "DATABASE" "success" $SRC_BACKUP_LOCATION
backupVersionDetails $SRC_BACKUP_LOCATION
}

backupVersionDetails()
{
location=$1
versionQuery="select @@version"
version=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 200 -o $location/sybaseversion.txt --retserverror <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
$versionQuery
go
quit
EOF
)
retval=$?
if [ $retval -gt 0 ]; then
  echo "*** ERRORMSG: Could not get Version Details"
fi
}

handleDbNameCaseSensitive()
{
dbListQuery="select distinct name from master..sysdatabases"
dblist=$($SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
use master
go
set nocount on
go
$dbListQuery
go
EOF
)
retval=$?
set -x
if [ $retval -gt 0 ]; then
  echo "ERRORMSG: failed to execute dbListQuery"
  exit 1
fi
if [ "$SRC_DBNAME" != "" ]; then
  for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
  do
    dbname=$i
    matchdbname=`echo $dblist | grep -iow $dbname`
    if [ -z "$matchdbname" ]; then
     echo " Error : Cannot Find the Database name: $dbname "
     exit 1
    fi
    DBNAMES="${DBNAMES}${DBNAMES:+,}$matchdbname"
  done
  SRC_DBNAME=$DBNAMES
  echo "handleDbNameCaseSensitive$SRC_DBNAME"
  #sed -i 's/^SRC_DBNAME.*/SRC_DBNAME='$SRC_DBNAME'/' /act/scripts/act_Sybase_Src.conf
fi
set +x
}

if [ -n $method ]; then
case $method in
  dbsize)
    echo "execute dbsize Query"
    getDbSize
    ;;
  notificationDbList)
    echo "execute notificationDbList Query"
    getNotificationDBList
    ;;
  notificationlogDbList)
    echo "execute notificationlogDbList Query"
    getNotificationLogdbnamesList
    ;;
  lvmList)
    echo "execute getLvmList Query"
    getLvmList
    ;;
  dbnameslist)
    echo "execute generateDbnamesList Query"
    generateDbnamesList
    ;;
  lvmnames)
    echo "execute generateLvmnamesList Query"
    SRC_DBNAME=$2
    generateLvmnamesList
    ;;
  quiesceDb)
    echo "execute quiesceDb Query"
    SRC_DBNAME=$2
    datadev_dir=$3
    quiesceDb $SRC_DBNAME $datadev_dir
    ;;
  releaseDb)
    echo "execute releaseDb Query"
    SRC_DBNAME=$2
    releaseDb $SRC_DBNAME
    ;;
  configureLogOptions)
    echo "execute configureLogOptions Query"
    SRC_BACKUP_LOCATION=$2
    configureLogOptions $SRC_BACKUP_LOCATION
    ;;
  executeLogDumpTransaction)
    echo "execute executeLogDumpTransaction Query"
    SRC_BACKUP_LOCATION=$2
    executeLogDumpTransaction $SRC_BACKUP_LOCATION
    ;;
  executeDbDumpTransaction)
    echo "execute executeDbDumpTransaction Query"
    SRC_BACKUP_LOCATION=$2
    executeDbDumpTransaction $SRC_BACKUP_LOCATION
    ;;
  handleDbNameCaseSensitive)
    echo "execute handleDbNameCaseSensitive"
    handleDbNameCaseSensitive
    ;;
  *)
    usage
 esac
else
 usage
fi

exit
