#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

function validateDbParams() {

  if [ ! -z "$OSUSER" ] && [ ! -z "$SRC_SYBASE_SQLD" ] && [ ! -z "$SRC_BACKUP_USER" ] && [ ! -z "$SRC_SERVER_NAME" ]; then
    if [[ $LogBackupEnable == "true" ]]; then
      if [ -z $LogBackupFS ]; then
        echo " Error : LogBackupFS cannot be empty."
        exit 1
      fi
      validateDirPath $LogBackupFS
      validateDirPath $LOG_BKP_LOC
      if [ ! -z $LOG_BKP_LOC ]; then
        echo $LOG_BKP_LOC | grep -w $LogBackupFS
        retval=$?
        if [ $retval -ne 0 ]; then
          cat /proc/mounts | grep -w $LogBackupFS
          retval=$?
          if [ $retval -ne 0 ]; then
            echo "*** Error LogLocation: $LOG_BKP_LOC is not contained in Log Mountpoint: $LogBackupFS"
            exit 1
          else
            nfsmountlogloc=$(cat /proc/mounts | grep -w $LogBackupFS | awk -F' ' '{print $2}')
            echo $LOG_BKP_LOC | grep -w $nfsmountlogloc
            retval=$?
            if [ $retval -ne 0 ]; then
              echo "*** Error LogLocation: $LOG_BKP_LOC is not contained in NFS Mountpoint: $nfsmountlogloc"
              exit 1
            fi
          fi
        fi
      fi
    fi
  elif [ ! -z "$OSUSER" ] && [ ! -z "$TARGET_SYBASE_SQLD" ] && [ ! -z "$TARGET_DB_USER" ] && [ ! -z "$TARGET_SERVER_NAME" ]; then
    validateDirPath $SYBASE_ASE_PATH
  else
    echo "Error: validateDbParams has Failed.."
    exit 1
  fi
  validateDBConnection
  if [ "$BACKUP_TYPE" != "Filesystemdump" ]; then
    isdump=true
  else
    isdump=false
  fi
  #validatePrivileges $isdump
}

function validateDBConnection() {
  if [ -z $SRC_SYBASE_SQLD ]; then
    SYBASE_SQLD=$TARGET_SYBASE_SQLD
  else
    SYBASE_SQLD=$SRC_SYBASE_SQLD
  fi
  if [ -z $SRC_BACKUP_USER ]; then
    DB_USER=$TARGET_DB_USER
  else
    DB_USER=$SRC_BACKUP_USER
  fi
  if [ -z $SRC_USER_PASSWD ]; then
    DB_PASSWORD=$TARGET_DBUSER_PASSWD
  else
    DB_PASSWORD=$SRC_USER_PASSWD
  fi
  if [ -z $SRC_SERVER_NAME ]; then
    SERVER_NAME=$TARGET_SERVER_NAME
  else
    SERVER_NAME=$SRC_SERVER_NAME
  fi
  id $OSUSER
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "*** Error: OSUSER:$OSUSER does not exist "
    exit 1
  fi
  if [ ! -f "$SYBASE_SQLD/bin/isql" ]; then
    echo "*** Error: SYBASE_SQLD $SYBASE_SQLD/bin/isql does not exist on host ***"
    exit 1
  fi
  dbConnectionTest=$(
    su - $OSUSER -c "$SYBASE_SQLD/bin/isql -U$DB_USER -P$DB_PASSWORD -S$SERVER_NAME -X -b -n -w 100 --retserverror" <<EOF
quit
EOF
  )
  retval=$?
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: DB Connection Query has Failed!!  $dbConnectionTest for serverName:$SERVER_NAME, for username: $DB_USER"
    exit 1
  fi
}

function validatePrivileges() {
  isdump=$1

  rolelist=$(
    su - $OSUSER -c "$SYBASE_SQLD/bin/isql -U$DB_USER -P$DB_PASSWORD -S$SERVER_NAME -X -b -n -w 100 --retserverror" <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
sp_displayroles $DB_USER
go
EOF
  )

  roleName=$(echo "$rolelist" | grep sa_role | tr -d '[:space:]')

  if [ ! -z "$roleName" ]; then
    echo "User Login has sa_role. Privileges Check Done"
  else
    if [ -z "$rolelist" ]; then
      echo "**** Error No Roles Assigned to userlogin :$DB_USER"
      echo "**** Privileges Check has Failed!!! Exiting **** "
      exit 1
    fi

    echo "$DB_USER does not have sa_role assigned. Check for Privileges in other roles"
    for role in $(echo "$rolelist"); do
      ROLES_STR="${ROLES_STR}${ROLES_STR:+,}'$role'"
    done

    if [ "$isdump" == "true" ]; then
      permissiondict="'Dump Any Database','Load Any Database','Mount Any Database','Unmount Any Database','Online Any Database'"
      query="select distinct v.name from sysprotects p, master.dbo.spt_values v, sysusers u where p.uid=u.uid and p.action=v.number and p.protecttype=1 and v.type = 'GP' and v.name in ($permissiondict) and u.name in ($ROLES_STR)"
    else
      permissiondict="'Quiesce Any Database','Mount Any Database','Unmount Any Database','Online Any Database','Shutdown'"
      query="select distinct v.name from sysprotects p, master.dbo.spt_values v, sysusers u where p.uid=u.uid and p.action=v.number and p.protecttype=1 and v.type = 'GP' and v.name in ($permissiondict) and u.name in ($ROLES_STR)"
    fi

    permissionlist=$(
      su - $OSUSER -c "$SYBASE_SQLD/bin/isql -U$DB_USER -P$DB_PASSWORD -S$SERVER_NAME -X -b -n -w 100 --retserverror" <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
$query
go
EOF
    )

    while read -r line; do
      echo $line
      PERMISSION_STR="${PERMISSION_STR}${PERMISSION_STR:+,}'$line'"
    done <<<"$permissionlist"

    echo "Defined Db Permissions: $PERMISSION_STR"
    echo "Expected permissions $permissiondict"

    missingPermissions=$(echo ${permissiondict[@]},${PERMISSION_STR[@]} | tr ',' '\n' | sort | uniq -u)
    if [ ! -z "$missingPermissions" ]; then
      echo "**** Error: Missing Permissions: $missingPermissions ****"
      echo "**** Privileges Check has Failed!!! Exiting **** "
      exit 1
    fi
  fi
}

function validateDirPath() {
  dirPath=$1
  if [ ! -z $dirPath ]; then
    if [ ! -d $dirPath ]; then
      cat /proc/mounts | grep -w $dirPath
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: $dirPath does not exist"
        exit 1
      fi
    fi
  fi
}

function validateFileExists() {
  fileloc=$1
  if [ ! -z $fileloc ]; then
    if [ ! -f $fileloc ]; then
      echo "Error: $fileloc does not exist"
      exit 1
    fi
  fi
}
