#!/bin/sh
set +x

read -r TARGET_DBUSER_PASSWD
conffile=/act/scripts/act_sybase_lvm_restore_config.conf
if [ -f $conffile ]; then
 source $conffile
else
 echo "ERROR: Configuration file for Log Recovery does not exist!."
fi

if [ -z "$TARGET_DBUSER_PASSWD" ]; then
  TARGET_DBUSER_PASSWD=`/sybase/"$TARGET_SERVER_NAME"/dba/bin/dbbp MaGiCpW`
  if [ -z "$TARGET_DBUSER_PASSWD" ]; then
    echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
    echo " Error : ISQL Password cannot be empty."
    exit 1
  fi
fi

$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X  <<EOF
use master
go
online database scbtools
go
online database saptools
go
EOF

handleDbNameCaseSensitive()
{
dbListQuery="select distinct name from master..sysdatabases"
dblist=$($TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
set nocount on
go
$dbListQuery
go
EOF
)
retval=$?
if [ $retval -gt 0 ]; then
  echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
  echo "ERRORMSG: failed to execute dbListQuery: $dblist"
  exit 1
fi
if [ "$DATABASE_NAME" != "" ]; then
  for i in $(echo $DATABASE_NAME | sed "s/,/ /g")
  do
    dbname=$i
    matchdbname=`echo $dblist | grep -iow $dbname`
    if [ -z "$matchdbname" ]; then
     echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
     echo " Error : Cannot Find the Database name: $dbname "
     exit 1
    fi
    DBNAMES="${DBNAMES}${DBNAMES:+,}$matchdbname"
  done
  DATABASE_NAME=$DBNAMES
  #sed -i 's/^DATABASE_NAME.*/DATABASE_NAME='$DATABASE_NAME'/' /act/scripts/act_sybase_lvm_restore_config.conf
fi
}

if [ "$DATABASE_NAME" != "" ]; then
  handleDbNameCaseSensitive
fi

tmp_loc=/act/touch

$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 1000 -o $tmp_loc/db_names.txt --retserverror <<EOF
set nocount on
go
select distinct sda.name from master..sysdatabases sda where sda.name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','saptempdb','sapsatempdb','sybsecurity')
go
EOF

retval=$?
if [ $retval -gt 0 ]; then
  echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
  echo "ERRORMSG: failed to execute dbnameslist `cat $tmp_loc/db_names.txt`"
  exit 1
fi

if [ "x$DATABASE_NAME" = "x" ]; then
  while read line
  do
    set -- $line
    if [ -z $db_list ]; then
    db_list=$line
    else
    db_list=$db_list,$line
    fi
  done < $tmp_loc/db_names.txt
  DATABASE_NAME=$db_list
fi

echo $DATABASE_NAME

#Get the recovery time from Manifest file location


logtransdump=$LOG_BKP_MNTPT/syb_restore_trans_dump

if [ -e "$logtransdump" ]; then
  echo "removing log trans dump file"
  rm -f $logtransdump
  rm -f $LOG_BACKUP_LOCATION/tran_dump_*.txt
fi

LOGBACKUP_DAT_LOC=$(find $TARGET_MNT_PNT -iname logbackup.dat -printf "%T+\t%p\n" | sort | tail -n 1 | awk -F" " '{print $NF}' | rev | cut -d"/" -f2-  | rev)

if [ ! -z "$LOG_BKP_MNTPT" ]; then
  rm -f $LOG_BKP_MNTPT/tran_dump_*.txt
  if [ ! -z "$UNTIL_TIME" ]; then
    recovery_from=`cat $LOGBACKUP_DAT_LOC/logbackup.dat`
    recoverytime=`date -d "$UNTIL_TIME" +'%b %d %Y %H:%M:%S'`
    UNTIL_TIME=`date -d "$UNTIL_TIME" +'%Y%m%d%H%M%S'`
  fi
fi

################Identify the transaction dumps needed for recovery#######################

for i in $(echo $DATABASE_NAME | sed "s/,/ /g")
do
dbname=$i
i=$dbname
c="STRIPE ON"
TRANS="load transaction $dbname from "
transFiles=""

cdmpFiles=(`find $LOG_BKP_MNTPT/ -newer $LOGBACKUP_DAT_LOC/logbackup.dat | sort -n | grep -Ew "$dbname""_trans_[0-9]{8}_[0-9]{6}_.*.cdmp"`)
dump_file=$LOG_BKP_MNTPT/tran_dump_$i.txt
if [ ! -z "$UNTIL_TIME" ]; then
 if [ "$UNTIL_TIME" -gt "$recovery_from" ]; then
   rollforwardtime=`date -d "${UNTIL_TIME:0:8} ${UNTIL_TIME:8:2}:${UNTIL_TIME:10:2}:${UNTIL_TIME:12:2}" +"%Y-%m-%d %H:%M:%S"`
   echo "rollforward:$rollforwardtime"
   len=`echo "${#cdmpFiles[@]}"`
   if [ $len -gt 0 ]; then
    for (( j=0; j<$len; j++ ));
   do
     echo "${cdmpFiles[$j]}" >> $LOG_BKP_MNTPT/tran_dump_$i.txt
   done
  fi

   dump_file=$LOG_BKP_MNTPT/tran_dump_$i.txt
   else
    echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
    echo "ERROR: UNTIL TIME: $UNTIL_TIME is less than Recovery Time: $recovery_from"
    exit 1
   fi

STRIPEON=`ls $LOG_BKP_MNTPT | grep -E "*_trans_[0-9]{8}_[0-9]{6}_.*.cdmp" | awk -F"_" '{print $5}' | awk -F"-" '{print $1}' | sort | uniq | wc -l`

lastlogfile=`ls -lrt $LOG_BKP_MNTPT | grep -Ew "$dbname""_trans_[0-9]{8}_[0-9]{6}_.*.cdmp" | tail -n 1`
if [ ! -z "$lastlogfile" ]; then
  lastlogfiletstamp=`echo "$lastlogfile" | rev | awk -F "_" '{print $2$3}' | rev`
fi

transFiles=""
counter="1"
c="STRIPE ON"
finallogtstamp=""
echo use master >> $LOG_BKP_MNTPT/syb_restore_trans_dump
echo go >> $LOG_BKP_MNTPT/syb_restore_trans_dump
while read line
do
    logfiletstamp=`echo "$line" | rev | awk -F "_" '{print $2$3}' | rev`

    if [ "$logfiletstamp" -gt "$UNTIL_TIME" ]; then
        finallogtstamp="$logfiletstamp"
    fi

    if [ "$logfiletstamp" -gt "$recovery_from" -a "$logfiletstamp" -le "$lastlogfiletstamp" ]; then
      if [ ! -z $line ]; then
          if [ "$counter" -eq "$STRIPEON" ]; then
                transFiles=`echo "$transFiles \"$line"\"`
                if [ ! -z "$transFiles" ];then
                if [ ! -z "$finallogtstamp" ];then
                  if [ "$finallogtstamp" -gt "$UNTIL_TIME" ];then
                    transFiles=`echo "$transFiles with until_time=\"$recoverytime\""`
                  fi
                fi
                echo "$TRANS""$transFiles" >> $LOG_BKP_MNTPT/syb_restore_trans_dump
                counter=0
                transFiles=""
                echo go >> $LOG_BKP_MNTPT/syb_restore_trans_dump
                if [ ! -z "$finallogtstamp" ];then
                  if [ "$finallogtstamp" -gt "$UNTIL_TIME" ]; then
                    break;
                  fi
                fi
              fi
            else
                transFiles=`echo "$transFiles "\"$line"\" $c"`
        fi
        counter=$((counter+1))
      fi
    fi

  done < $dump_file

fi
done

set +x
$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -o $LOG_BKP_MNTPT/log_recoveryoutput.txt -i $LOG_BKP_MNTPT/syb_restore_trans_dump --retserverror
retval=$?
cp -f $LOG_BKP_MNTPT/syb_restore_trans_dump /var/act/log/UDSAgent-trans_log_sql.log
echo "***************** Start of transaction logs *****************" > /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
cat $LOG_BKP_MNTPT/log_recoveryoutput.txt >> /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
echo "***************** End of transaction logs *****************" >> /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
tail -20 $LOG_BKP_MNTPT/log_recoveryoutput.txt
if [ $retval -gt 0 ]; then
  echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *******"
  echo "ERRORMSG: There was an error while applying Log transactions... .Check for complete execution details in /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log"
  exit 1
fi
echo "--Finished Applying Log Transactions...Check for complete execution details in /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log"
for i in $(echo $DATABASE_NAME | sed "s/,/ /g")
do
i=$i
echo "Going to bring database:$i online"
$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror <<EOF
use master
go
online database $i
go
EOF

retval=$?
if [ $retval -gt 0 ]; then
  echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:fail  *********"
  echo "ERRORMSG: failed to make database:$i online post recovery"
  exit 1
fi

if [ -f "$LOG_BKP_MNTPT/tran_dump_$dbname.txt" ]; then
 rm -f $LOG_BKP_MNTPT/tran_dump_$dbname.txt
fi
done

echo "******** RESTORE_STATUS=$TARGET_SERVER_NAME:success  *********"

exit 0
