#!/bin/sh
set +x
method=$1
source /act/act_scripts/sybase/automation/Configuration/sybaserestore.conf
read -r TARGET_DBUSER_PASSWD

if [ -z "$TARGET_DBUSER_PASSWD" ]; then
  TARGET_DBUSER_PASSWD=$(/sybase/"$TARGET_SERVER_NAME"/dba/bin/dbbp MaGiCpW)
  if [ -z "$TARGET_DBUSER_PASSWD" ]; then
    echo " Error : ISQL Password cannot be empty."
    exit 1
  fi
fi

usage() {
  echo "Error: Missing DB Function: Exiting"
  exit 1
}

function validate_sybasedumpversion() {
  sourceVersion=$1
  versionQuery="select @@version"
  version=$($TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -b -n -w 200 --retserverror <<EOF
use master
go
set proc_return_status off
go
set nocount on
go
$versionQuery
go
quit
EOF
  )
  retval=$?
  set -x
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: failed to fetch sybase version: $version"
    exit 1
  fi
  targetversionNumber=$(echo $version | cut -d / -f2 | cut -d " " -f1 | sed -e 's/\.//g')
  if [[ $(echo "$targetversionNumber $sourceVersion" | awk '{print ($1 < $2)}') == 1 ]]; then
    echo "***** Error: Cannot load higher major version $sourceVersion source dump to lower version $versionNumber sybase ****"
    exit 1
  fi
  set +x
}

function validatebkp_srv_params() {
  bkpsrvnameQuery="select srvname from sysservers where srvnetname='$BACKUP_RUN_PROCESS_NAME' and srvname='$BACKUP_SERVER_NAME'"
  bkpsrvname=$($TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
use master
go
set nocount on
go
$bkpsrvnameQuery
go
EOF
  )
  retval=$?
  set -x
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: failed to fetch srvname from sysservers :$bkpsrvname"
    exit 1
  fi

  if [ -z "$bkpsrvname" ]; then
    echo "***ERRORMSG: Backup Server:$BACKUP_SERVER_NAME Not Defined for the BACKUP_RUN_PROCESS_NAME:$BACKUP_RUN_PROCESS_NAME"
    exit 1
  fi
  set +x
}

function handleDbNameCaseSensitive() {
  dbListQuery="select distinct name from master..sysdatabases"
  dblist=$($TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -b -n -w 100 --retserverror <<EOF
use master
go
set nocount on
go
$dbListQuery
go
EOF
  )
  retval=$?
  set -x
  if [ $retval -gt 0 ]; then
    echo "ERRORMSG: failed to fetch names from master..sysdatabases :$dblist"
    exit 1
  fi
  if [ "$DATABASE_NAME" != "" ]; then
    for i in $(echo $DATABASE_NAME | sed "s/,/ /g"); do
      dbname=$i
      matchdbname=$(echo $dblist | grep -iow $dbname)
      if [ -z "$matchdbname" ]; then
        echo " Error : Cannot Find the Database name: $dbname "
        exit 1
      fi
      DBNAMES="${DBNAMES}${DBNAMES:+,}$matchdbname"
    done
    DATABASE_NAME=$DBNAMES
    echo "handleDbNameCaseSensitiveQuery$DATABASE_NAME"
  fi
  set +x
}

if [ -n $method ]; then
  case $method in
  handleDbNameCaseSensitive)
    echo "execute handleDbNameCaseSensitive"
    handleDbNameCaseSensitive
    ;;
  validatebkp_srv_params)
    echo "execute validatebkp_srv_params"
    validatebkp_srv_params
    ;;
  validate_sybasedumpversion)
    databasedumpversion=$2
    echo "execute validatebkp_srv_params"
    validate_sybasedumpversion $databasedumpversion
    ;;
  *)
    usage
    ;;
  esac
else
  usage
fi

exit
