#!/bin/sh
set +x
if [ "$#" -lt 1 ]; then
  echo " This script needs key-value input parameters"
  echo " usage: restore.sh <name=value>"
  echo " For example: restore.sh OSUSER=sybase"
  exit 1
fi

restoreConfFile="/act/act_scripts/sybase/automation/Configuration/sybaserestore.conf"

echo -e '#Sybase Restore Params' > $restoreConfFile
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   passwdstrCheck=`echo $ARGUMENT | grep -i "password\|passwd"`
   if [ -z "$passwdstrCheck" ]; then
     echo -e $ARGUMENT >> $restoreConfFile
   else
     export $ARGUMENT
   fi
done

chmod 755 $restoreConfFile
source $restoreConfFile

#HOME=`su - $OSUSER -c "env | grep ^HOME= | cut -d'=' -f2"`
#Set Sybase home value into Config File
#echo -e "HOME='$HOME'" >> $restoreConfFile

if [ -z "$TARGET_DBUSER_PASSWD" ]; then
  TARGET_DBUSER_PASSWD=`su - $OSUSER -c "/sybase/"$TARGET_SERVER_NAME"/dba/bin/dbbp MaGiCpW"`
  if [ -z "$TARGET_DBUSER_PASSWD" ]; then
    echo " Error : ISQL Password cannot be empty."
    exit 1
  fi
fi

if [ ! -z "$SINGLE_DB_RECOVERY" ] && [ "$SINGLE_DB_RECOVERY" == "YES" ] ; then
  sh /act/act_scripts/sybase/automation/Restore/act_sybase_lvm_customdb_recovery.sh "$@"
  if [ "$?" -gt 0 ]; then
    echo "ERROR: Sybase Single Database Recovery Failed!"
    exit 1
  fi
  exit 0
fi

TARGET_ESC_PASSWD=`printf '%q' "$TARGET_DBUSER_PASSWD"`

if [ -z $GenericAppName ]; then
  echo "Error: DB App Name is Mandatory"
  exit 1
fi

if [ -z $dbimagename ]; then
  echo "Error: Image Id is Mandatory"
  exit 1
fi

if [ -z $OSUSER ]; then
  echo "Error: OS-Username is mandatory"
  exit 1
fi

if [ -z $SkyIP ]; then
  echo "Error: Sky IP is mandatory"
  exit 1
fi

if [ -z $TARGET_DB_USER ]; then
  echo "Error: Target DB user is mandatory"
  exit 1
fi

if [ -z $TARGET_SERVER_NAME ]; then
  echo "Error: Target server name is mandatory"
  exit 1
fi

if [ -z $TARGET_SYBASE_SQLD ]; then
  echo "Error: Target Sybase SQLD is mandatory"
  exit 1
fi

if [ -z $TARGET_HOSTNAME ]; then
  TARGET_HOSTNAME=`hostname`
fi
set +x
SkyUserName="admin"
VendorKey="act"
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$TARGET_HOSTNAME" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
echo "HOST ID ::  $hostID"
lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
dbappFilter="filtervalue=appname=$GenericAppName&backupname=$dbimagename"
lsbackupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$dbappFilter"  -d "sessionid=$sessionid")
dbappID=`echo "$lsbackupResponse" | jq -cr '(.result[] | .appid)'`
sourcehostname=`echo "$lsbackupResponse" | jq -cr '(.result[] | .hostname)'`
currenthostname=$TARGET_HOSTNAME
if [ "$sourcehostname" == "$currenthostname" ]; then
  samehostrecovery=true
else
  samehostrecovery=false
fi

if [ -z "$dbappID" ]; then
  echo "*** ERRORMSG: DB Image:$dbimagename not found!!!...May be DB job is still in progress"
  exit 1
fi

if [ "$samehostrecovery" == "true" ]; then
  # Disable Schedule For DB Application
  dbslaid=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lssla" --data-urlencode "filtervalue=appid=$dbappID"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
  chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$dbslaid&scheduleoff=true"
  chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)
fi

if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
  logappFilter="filtervalue=appname=$LogBackupFS&backupname=$logimagename"
  logappID=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$logappFilter"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .appid)'))
  if [ -z "$logappID" ]; then
    echo "*** ERRORMSG: Log Image:$logimagename not found!!!...May be Log job is in progress"
    exit 1
  fi
  # Disable Schedule for Log Application
  if [ "$samehostrecovery" == "true" ]; then
    logslaid=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lssla" --data-urlencode "filtervalue=appid=$logappID"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
    chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$logslaid&scheduleoff=true"
    chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)
  fi
fi



#Check if there are running jobs on this target host
reportdbjobsurl="https://$SkyIP/actifio/api/report/reportrunningjobs?sessionid=$sessionid&a=$dbappID"
reportdbjobsresponse=$(curl -s -k  -H 'Content-Type: application/json' -X GET $reportdbjobsurl)
reportdbjobhostname=`echo "$reportdbjobsresponse" | jq -cr '(.result[] | .HostName)'`
reportdbjobclass=`echo "$reportdbjobsresponse" | jq -cr '(.result[] | .JobClass)'`
if [ ! -z "$reportdbjobhostname" ]; then
  for reporthostname in $reportdbjobhostname
  do
    if [ "$reporthostname" == "$currenthostname" ]; then
      echo "$reportdbjobclass" | grep -i  "snapshot\|restore\|mount"
      retval=$?
      if [ $retval -eq 0 ]; then
        echo "**** Error: There is a Running DB Job for Host:$reporthostname dbappname:$GenericAppName ****"
        echo "*** Cancel/Wait for Any Running Jobs to finish Before Proceeding with Restore"
        exit 1
      fi
    fi
  done
fi

if [ ! -z "$logappID" ]; then
  reportlogjobsurl="https://$SkyIP/actifio/api/report/reportrunningjobs?sessionid=$sessionid&a=$logappID"
  reportlogjobsresponse=$(curl -s -k  -H 'Content-Type: application/json' -X GET $reportdbjobsurl)
  reportlogjobhostname=`echo "$reportlogjobsresponse" | jq -cr '(.result[] | .HostName)'`
  reportlogjobClass=`echo "$reportlogjobsresponse" | jq -cr '(.result[] | .JobClass)'`
  if [ ! -z "$reportlogjobhostname" ]; then
    for reporthostname in $reportlogjobhostname
    do
      if [ "$reporthostname" == "$currenthostname" ]; then
        echo "$reportlogjobClass" | grep -i  "snapshot\|restore\|mount"
        retval=$?
        if [ $retval -eq 0 ]; then
          echo "**** Error: There is a Running Log Job for Host:$reporthostname logappname:$LogBackupFS ****"
          echo "*** Cancel/Wait for Any Running Jobs to finish Before Proceeding with Restore"
          exit 1
        fi
      fi
    done
  fi
fi


echo "----Checking Pre-Requisites(Agent,Iscsi) on Host machine----"
source /act/act_scripts/sybase/automation/Backup/preRequisites.sh $SkyIP
echo "--------Done Checking Pre-Requisites on Host machine--------"
set +x
#Ensure IQN of host is latest in sky
iscsiname=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`
getHostURL="https://$SkyIP/actifio/api/info/lshost?sessionid=$sessionid&argument=$hostID"
oldIscsiName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getHostURL | jq -cr '(.result | .iscsi_name)')
if [ -z $oldIscsiName ]; then
  chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
  inputParamsForChangeHost='{
    "iscsiport": [ "'"$iscsiname"'" ]
  }'
  chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
  echo "Host Update Response with IQN: $chHostResponse"
else
  if [ $oldIscsiName != $iscsiname ]; then
   deleteIscsiHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid&iscsiname=%5B%22$oldIscsiName%22%5D&force=true"
   delHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X DELETE $deleteIscsiHostURL)
   echo "Host Delete IQN Response: $delHostResponse"

   chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
   inputParamsForChangeHost='{
     "iscsiport": [ "'"$iscsiname"'" ]
   }'
   chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
   echo "Host Update Response with IQN: $chHostResponse"
  fi
fi

appFilter="filtervalue=appname=$GenericAppName&backupname=$dbimagename"
appType=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$appFilter"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .apptype)'))

if [ -z $appType ]; then
  echo "**** Error: Apptype cannot be null for appname=$GenericAppName ******"
  exit 1
fi
if [ "$appType" == "FileSystem" ]; then
  source /act/act_scripts/sybase/automation/Restore/restoreFS.sh
else
  source /act/act_scripts/sybase/automation/Restore/restoreLVM.sh
fi

if [ "$samehostrecovery" == "true" ]; then
  # Enable the DB schedule back.
  chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$dbslaid&scheduleoff=false"
  chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)

  # Enable the Log Schedule back.
  if [ ! -z "$logslaid" ]; then
    chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$logslaid&scheduleoff=false"
    chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)
  fi
fi
