#!/bin/sh

orig_pass_temp=$TARGET_DBUSER_PASSWD
TARGET_DBUSER_PASSWD=$TARGET_ESC_PASSWD
source /act/act_scripts/sybase/automation/Backup/paramValueValidator.sh
validateDbParams
TARGET_DBUSER_PASSWD=$orig_pass_temp

if [ "$DATABASE_NAME" != "" ]; then
  dbnamelist=$(echo $TARGET_DBUSER_PASSWD |  su - $OSUSER -c "sh /act/act_scripts/sybase/automation/Restore/dbHelper.sh handleDbNameCaseSensitive")
  retval=$?
  if [ $retval -gt 0 ]; then
    echo "***** ERRORMSG: failed to execute handleDbNameCaseSensitive function ******"
    exit 1
  fi
  DATABASE_NAME=`echo $dbnamelist | awk '{sub(/^.*handleDbNameCaseSensitiveQuery/,""); print $1}'`
fi

restoremaster=`echo $DATABASE_NAME | grep -iw master`
if [ ! -z "$restoremaster" ]; then
  if [ -z $BACKUP_SERVER_NAME ]; then
    echo "Error: Backup server name is mandatory to restore master"
    exit 1
  fi

  if [ -z $SYBASE_ASE_PATH ]; then
    echo "Error: Sybase ASE Path is mandatory to restore master"
    exit 1
  else
    if [ ! -d $SYBASE_ASE_PATH ]; then
      echo "Error: SYBASE_ASE_PATH:$SYBASE_ASE_PATH does not exist"
      exit 1
    fi
  fi

  if [ -z $BACKUP_RUN_PROCESS_NAME ]; then
    echo "Error: Backup run process name is mandatory to restore master"
    exit 1
  fi

  BACKUP_RUN_PROCESS_NAME=`ls $SYBASE_ASE_PATH/install/ | grep -iw RUN_$BACKUP_RUN_PROCESS_NAME | awk -F' ' '{print $NF}' | cut -d'_' -f2-`
  if [ -z $BACKUP_RUN_PROCESS_NAME ]; then
    echo "Error: Invalid BACKUP_RUN_PROCESS_NAME!!!..Exiting"
    exit 1
  fi

  echo $TARGET_DBUSER_PASSWD |  su - $OSUSER -c "sh /act/act_scripts/sybase/automation/Restore/dbHelper.sh validatebkp_srv_params"
  retval=$?
  if [ $retval -gt 0 ]; then
    echo "***** ERRORMSG: failed during execution of validatebkp_srv_params function ******"
    exit 1
  fi
fi

tgt_casesensitve_SERVER_NAME=`su - $OSUSER -c "showserver" | grep -i dataserver | grep -i "\-s$TARGET_SERVER_NAME" | awk '{sub(/^.*-s/,""); print $1}'`
if [ -z $tgt_casesensitve_SERVER_NAME ]; then
  echo "Error: Invalid ASE Target Server Name: $TARGET_SERVER_NAME Exiting!!!"
  exit 1
fi
TARGET_SERVER_NAME=$tgt_casesensitve_SERVER_NAME

# Check if mounts exist for the given app.

filterValue="filtervalue=jobclass=mount&appname=$GenericAppName&apptype=FileSystem&mountedhost=$hostID"
mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

mountImageIDListLength=${#mountImageIDList[@]}
echo "Existing $GenericAppName mounts for Sybase DB are:"  $mountImageIDListLength

if [ $mountImageIDListLength -gt 0 ]; then
   for (( i=0; i<${mountImageIDListLength}; i++ ))
   do
   # Unmount and Delete Image
   unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

    inputParamsForUnmount='{
    "sessionid":"'"$sessionid"'",
    "image":"'"${mountImageIDList[i]}"'",
    "delete":"true"
   }'

   echo "Need to unmount and re-mount for data refresh"
   unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
   if [[ $unmountResponse == *errormessage* ]]; then
      echo "Unmount Failed. Reason :" $unmountResponse
   elif [[ $unmountResponse == *result* ]]; then
      echo "Existing App Unmounted successfully."
   fi
  done
fi

echo "Mount Image: $dbimagename for restoring Sybase dumpDatabase"
mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

if [ -z $TARGET_MNT_PNT ]; then
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true"
     }'
else
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true",
      "restoreoption":"'"mountpointperimage=$TARGET_MNT_PNT"'"
     }'
fi
mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
  echo "**** ERRMSG: DB jobID cannot be null: $mountResponse ****"
  exit 1
fi
echo "DB Job id is: $jobID "

jobinprogress=true
errorMessage=""
while $jobinprogress
    do
        sleep 30
        getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
        status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
        if [ ! -z "$status" ]; then
                if [ $status == "succeeded" -o $status == "failed" ]
                then
                        jobinprogress=false
                        if [ $status == "failed" ]
                        then
                                getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                        fi
                fi
              else
                jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                echo "Job is still running..Completed $status%"
        fi
done
printf "\n The Sybase DB Mount job $jobID has $status.\n"

if [ ! -z "$errorMessage" ]; then
  printf "\n Reason: $errorMessage.\n"
  exit 1
fi

if [ -z $TARGET_MNT_PNT ]; then
  TARGET_MNT_PNT=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
fi

if [ -f $TARGET_MNT_PNT/sybaseversion.txt ]; then
  sourceDumpVersion=`cat $TARGET_MNT_PNT/sybaseversion.txt | cut -d / -f2 | cut -d " " -f1 | sed -e 's/\.//g'`
  if [ ! -z "$sourceDumpVersion" ]; then
    echo $TARGET_DBUSER_PASSWD |  su - $OSUSER -c "sh /act/act_scripts/sybase/automation/Restore/dbHelper.sh validate_sybasedumpversion $sourceDumpVersion"
    retval=$?
    if [ $retval -gt 0 ]; then
      echo "***** ERRORMSG: Validation of Source Database Dump Major Version has Failed with target Sybase Major Version-> Post Mount Check ******"
      exit 1
    fi
  fi
fi


if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then

  if [ -z "$UNTIL_TIME" ]; then
    printf "\n Reason: Recovery Time has not been set for logBackup Image\n"
    exit 1
  fi
  lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"

  filterValue="filtervalue=backupname=$logimagename"
  backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
  appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
  policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
  LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')


  filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
  mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))
  mountImageIDListLength=${#mountImageIDList[@]}
  echo "Existing mounts for logBackup are:"  $mountImageIDListLength

  if [ $mountImageIDListLength -gt 0 ]; then
     for (( i=0; i<${mountImageIDListLength}; i++ ))
     do

     # Unmount and Delete Image
     unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

      inputParamsForUnmount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"${mountImageIDList[i]}"'",
      "delete":"true"
     }'

     unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
     if [[ $unmountResponse == *errormessage* ]]; then
        echo "Unmount Failed. Reason :" $unmountResponse
     elif [[ $unmountResponse == *result* ]]; then
        echo "Logbackup App Unmounted successfully."
     fi
    done
  fi

  printf "Restoring from Log Backup App Image"

  echo "Mounting Log Image: $logimagename"
  mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

  if [ -z $LOG_BKP_LOC ]; then
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true"
       }'
  else
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true",
        "restoreoption":"'"mountpointperimage=$LOG_BKP_LOC"'"
       }'
  fi

  mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
  jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    echo "**** ERRMSG: Log jobID cannot be null: $mountResponse ****"
    exit 1
  fi
  echo "Log Job id is: $jobID "

  jobinprogress=true
  errorMessage=""
  while $jobinprogress
      do
          sleep 30
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
  printf "\n The Log Backup Mount job $jobID has $status.\n"

  if [ ! -z "$errorMessage" ]; then
    printf "\n Reason: $errorMessage.\n"
    exit 1
  fi
  if [ -z $LOG_BKP_LOC ]; then
    LOG_BKP_LOC=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
  fi
fi

if [ ! -z "$logimagename" ]; then
  if [ ! -z $LOGBACKUPPATH ]; then
    logbackupSuffix="${LOGBACKUPPATH#$LogBackupFS}"
    echo "LogbackupSuffix: $logbackupSuffix"
    LOG_BKP_LOC=$LOG_BKP_LOC/$logbackupSuffix
    echo "Log Backup Updated Path: $LOG_BKP_LOC"
  fi
fi

SYBASERESTORECONFIG=/act/scripts/act_sybase_dump_restore_config.conf
echo -e "# Sybase Restore config" > $SYBASERESTORECONFIG
echo -e "OSUSER='$OSUSER'" >> $SYBASERESTORECONFIG
echo -e "TARGET_DB_USER='$TARGET_DB_USER'" >> $SYBASERESTORECONFIG
echo -e "TARGET_SYBASE_SQLD='$TARGET_SYBASE_SQLD'" >> $SYBASERESTORECONFIG
echo -e "TARGET_SERVER_NAME='$TARGET_SERVER_NAME'" >> $SYBASERESTORECONFIG
echo -e "DATABASE_NAME='$DATABASE_NAME'" >> $SYBASERESTORECONFIG
echo -e "DB_BACKUP_LOCATION='$TARGET_MNT_PNT'" >> $SYBASERESTORECONFIG
echo -e "LOG_BACKUP_LOCATION='$LOG_BKP_LOC'" >> $SYBASERESTORECONFIG
echo -e "UNTIL_TIME='$UNTIL_TIME'" >> $SYBASERESTORECONFIG
echo -e "SYBASE_ASE_PATH='$SYBASE_ASE_PATH'" >> $SYBASERESTORECONFIG
echo -e "BACKUP_RUN_PROCESS_NAME='$BACKUP_RUN_PROCESS_NAME'" >> $SYBASERESTORECONFIG
echo -e "BACKUP_SERVER_NAME='$BACKUP_SERVER_NAME'" >> $SYBASERESTORECONFIG
#echo -e "STRIPEON='$STRIPEON'" >> $SYBASERESTORECONFIG

OSGROUP=`id -ngr $OSUSER`
chmod -R 777 $TARGET_MNT_PNT
chown -R $OSUSER:$OSGROUP $TARGET_MNT_PNT
if [ ! -z $LOG_BKP_LOC ]; then
  chmod -R 777 $LOG_BKP_LOC
  chown -R $OSUSER:$OSGROUP $LOG_BKP_LOC
fi

cp -r /act/act_scripts/sybase/automation/Restore/sybase_restore_dump.sh /act/scripts/
chmod 755 /act/scripts/sybase_restore_dump.sh
chmod -R 755 $SYBASERESTORECONFIG
touch /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
chmod 777 /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
touch /var/act/log/UDSAgent-trans_log_sql.log
chmod 777 /var/act/log/UDSAgent-trans_log_sql.log
touch /var/act/log/UDSAgent-db_dump_sql.log
chmod 777 /var/act/log/UDSAgent-db_dump_sql.log
touch /var/act/log/UDSAgent-db_master_dump_sql.log
chmod 777 /var/act/log/UDSAgent-db_master_dump_sql.log
set +x
echo $TARGET_DBUSER_PASSWD | su - $OSUSER -c "sh /act/scripts/sybase_restore_dump.sh"
retval=$?
chmod 644 /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
chmod 644 /var/act/log/UDSAgent-trans_log_sql.log
chmod 644 /var/act/log/UDSAgent-db_dump_sql.log
chmod 644 /var/act/log/UDSAgent-db_master_dump_sql.log
if [ $retval -gt 0 ]; then
  echo "***** ERRORMSG: Error during execution of sybase_restore_dump.sh *****"
  exit 1
fi

set -x
