#!/bin/sh
restoreConfFile="/act/act_scripts/sybase/automation/Configuration/sybaserestore.conf"
SYBASERESTORECONFIG=/act/scripts/act_sybase_lvm_restore_config.conf

cat /act/act_scripts/sybase/automation/Configuration/sybaserestore.conf > /act/scripts/act_sybase_lvm_restore_config.conf

#Checking mandatory parameters for this approach.
if [ -z $BACKUP_SERVER_NAME ]; then
  echo "Error: Backup server name is mandatory"
  exit 1
fi

if [ -z $SYBASE_ASE_PATH ]; then
  echo "Error: Sybase ASE Path is mandatory"
  exit 1
fi

if [ -z $BACKUP_RUN_PROCESS_NAME ]; then
  echo "Error: Backup run process name is mandatory"
  exit 1
fi

tgt_casesensitve_SERVER_NAME=`ls $SYBASE_ASE_PATH/install/ | grep -iw RUN_$TARGET_SERVER_NAME | cut -d'.' -f1 | head -n 1 | awk -F' ' '{print $NF}' | cut -d'_' -f2-`
if [ -z $tgt_casesensitve_SERVER_NAME ]; then
  echo "Error: Invalid Target ASE Server Name: $TARGET_SERVER_NAME !!!..Exiting"
  exit 1
fi
TARGET_SERVER_NAME=$tgt_casesensitve_SERVER_NAME
sed -i 's/^TARGET_SERVER_NAME.*/TARGET_SERVER_NAME='$TARGET_SERVER_NAME'/' /act/scripts/act_sybase_lvm_restore_config.conf

BACKUP_RUN_PROCESS_NAME=`ls $SYBASE_ASE_PATH/install/ | grep -iw RUN_$BACKUP_RUN_PROCESS_NAME | cut -d'.' -f1 | head -n 1 | awk -F' ' '{print $NF}' | cut -d'_' -f2-`
if [ -z $BACKUP_RUN_PROCESS_NAME ]; then
  echo "Error: Invalid BACKUP_RUN_PROCESS_NAME!!!..Exiting"
  exit 1
fi
sed -i 's/^BACKUP_RUN_PROCESS_NAME.*/BACKUP_RUN_PROCESS_NAME='$BACKUP_RUN_PROCESS_NAME'/' /act/scripts/act_sybase_lvm_restore_config.conf

SYBASERESTORECONFIG=/act/scripts/act_sybase_lvm_restore_config.conf

#shutdown Backup and Data server.
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_ESC_PASSWD -S$TARGET_SERVER_NAME -X" <<EOF
use master
go
shutdown $BACKUP_SERVER_NAME
go
shutdown
go
EOF

if [ $? -gt "0" ]; then
 echo "ERROR: While Shutting down servers.. May be there are already down!"
 backupserverPID=$(su - $OSUSER -c "showserver" | grep -i backupserver | grep -iw "\-s$BACKUP_RUN_PROCESS_NAME" | awk -F" " '{print $4}')
 if [ ! -z "$backupserverPID" ]; then
   echo "WARNING: Running ASE backupserver. Kill it!"
   kill $backupserverPID
 fi
fi

sleep 60

sybaseStatus=$(su - $OSUSER -c "showserver" | grep -i dataserver | grep -iw "\-s$TARGET_SERVER_NAME")
if [ ! -z "$sybaseStatus" ]; then
  echo "ERROR: Running ASE dataserver. Exiting!"
  exit 1
fi

sybasebackupStatus=$(su - $OSUSER -c "showserver" | grep -i backupserver | grep -iw "\-s$BACKUP_RUN_PROCESS_NAME")
if [ ! -z "$sybasebackupStatus" ]; then
  echo "ERROR: Running ASE backupserver. Exiting!"
  exit 1
fi

# Unmount the existing log images.
  if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
    if [ -z "$UNTIL_TIME" ]; then
      printf "\n Reason: Recovery Time has not been set for logBackup Image\n"
      exit 1
    fi

    LOG_RECOVERY="YES"
    BACKUPTYPE="INCREMENTAL"
    lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
    filterValue="filtervalue=backupname=$logimagename"
    backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
    appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
    policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
    LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')

    filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
    mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

    mountImageIDListLength=${#mountImageIDList[@]}
    echo "Existing mounts for logBackup are:"  $mountImageIDListLength

    if [ $mountImageIDListLength -gt 0 ]; then

       for (( i=0; i<${mountImageIDListLength}; i++ ))
       do

       # Unmount and Delete Image
       unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

        inputParamsForUnmount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"${mountImageIDList[i]}"'",
        "delete":"true"
       }'


       unmountResponse=$(curl -s -w "\n" -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
       if [[ $unmountResponse == *errormessage* ]]; then
          echo "Unmount Failed. Reason :" $unmountResponse
       elif [[ $unmountResponse == *result* ]]; then
          echo "Logbackup App Unmounted successfully."
       fi
      done
    fi

    echo "Mounting Log Image: $logimagename"
    mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

    if [ -z $LOGRESTOREPATH ]; then
      inputParamsForMount='{
          "sessionid":"'"$sessionid"'",
          "image":"'"$logimagename"'",
          "host":"'"$hostID"'",
          "nowait":"true"
         }'
     else
       inputParamsForMount='{
           "sessionid":"'"$sessionid"'",
           "image":"'"$logimagename"'",
           "host":"'"$hostID"'",
           "nowait":"true",
           "restoreoption":"'"mountpointperimage=$LOG_BKP_LOC"'"
          }'
     fi

     mountResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount")
     jobID=`echo "$mountResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
     if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
       echo "**** ERRMSG: Log jobID cannot be null: $mountResponse ****"
       exit 1
     fi
     echo "Log Job id is: $jobID "

    jobinprogress=true
    errorMessage=""

     while $jobinprogress
        do
            sleep 30
            getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
            status=$(curl -Ss -w "\n" -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
            if [ ! -z "$status" ]; then
                    if [ $status == "succeeded" -o $status == "failed" ]
                    then
                            jobinprogress=false
                            if [ $status == "failed" ]
                            then
                                    getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                    errorMessage=$(curl -Ss -w "\n"  -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                            fi
                    fi
                  else
                    jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                    status=$(curl -Ss -w "\n" -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                    echo "Job is still running..Completed $status%"
            fi
    done
    printf "\n The Log Backup Mount job $jobID has $status.\n"

    if [ ! -z "$errorMessage" ]; then
      printf "\n Reason: $errorMessage.\n"
      exit 1
    fi

    if [ -z $LOG_BKP_LOC ]; then
      LOG_BKP_LOC=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
    fi

    OSGROUP=`id -ngr $OSUSER`
    chmod -R 777 $LOG_BKP_LOC
    chown -R $OSUSER:$OSGROUP $LOG_BKP_LOC

    if [ "$BACKUPTYPE" = "INCREMENTAL" ]; then
      if [ ! -z $LOGBACKUPPATH ]; then
        logbackupSuffix="${LOGBACKUPPATH#$LogBackupFS}"
        echo "LogbackupSuffix: $logbackupSuffix"
        LOG_BKP_LOC=$LOG_BKP_LOC/$logbackupSuffix
        echo "Log Backup Updated Path: $LOG_BKP_LOC"
      fi
    fi

    sleep 10

    cp -r /act/act_scripts/sybase/automation/Restore/act_sybase_logrecovery.sh /act/scripts/
    chmod -R 755 /act/scripts/act_sybase_logrecovery.sh

  fi

  # Invoke restore API to do a full Restore.
  restoreURL="https://$SkyIP/actifio/api/task/restoreimage"
  inputParamsForRestore='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "nowait":"true"
    }'

    restoreResponse=$(curl -sS -k -H 'Content-Type:application/json' -X POST $restoreURL -d "$inputParamsForRestore")
    jobID=`echo "$restoreResponse" | jq -cr '(.result)' | cut -d ' ' -f1`
    if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
      echo "**** ERRMSG: DB Restore jobID cannot be null: $restoreResponse ****"
      exit 1
    fi
    echo "DB Restore Job id is: $jobID "

    jobinprogress=true
    errorMessage=""
    while $jobinprogress
        do
            sleep 30
            getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
            status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
            if [ ! -z "$status" ]; then
                    if [ $status == "succeeded" -o $status == "failed" ]
                    then
                            jobinprogress=false
                            if [ $status == "failed" ]
                            then
                                    getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                    errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                            fi
                    fi
                  else
                    jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                    status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                    echo "Job is still running..Completed $status%"
            fi
    done
    printf "\n The Sybase DB Restore job $jobID has $status.\n"

    if [ ! -z "$errorMessage" ]; then
      printf "\n Reason: $errorMessage.\n"
      exit 1
    fi

    sleep 5

    # Find the Target Mount Point which contains the db manifest files
    protectedLvmList=$(curl -sS -w '\n' -k -G https://$SkyIP/actifio/api/info/lsbackup?sessionid=$sessionid\&argument=$dbimagename | jq -cr '(.result | .sourcemountpoint)' | cut -d[ -f2 | cut -d] -f1)

    protectedLvmList=$(echo $protectedLvmList | tr -d \" | tr "," "\n" )
    for lvm in $protectedLvmList
    do
            MANIFEST_FILE_LOC=$(find $lvm/ -iname manifest_file*.dat -printf "%T+\t%p\n" | sort | tail -n 1 | awk -F" " '{print $NF}' | rev | cut -d"/" -f2-  | rev)
            if [ ! -z "$MANIFEST_FILE_LOC" ]; then
                TARGET_MNT_PNT=$MANIFEST_FILE_LOC
                echo "Target Mount point: $TARGET_MNT_PNT"
                break
            fi
    done

    if [  -z "$TARGET_MNT_PNT" ]; then
        echo "Error: No Manifest file found at location $TARGET_MNT_PNT"
        exit 1
    fi

    if [ -e "$SYBASE_ASE_PATH/install/logfiledataserver.out" ]; then
      rm -f $SYBASE_ASE_PATH/install/logfiledataserver.out
    fi

    if [ -e "$SYBASE_ASE_PATH/install/logfilebackupserver.out" ]; then
      rm -f $SYBASE_ASE_PATH/install/logfilebackupserver.out
    fi


  #MANIFEST_FILE_LOC=$(find $TARGET_MNT_PNT/ -iname manifest_file | cut -d / -f3- | rev | cut -d/ -f2- | rev)

  #if [ -z "$MANIFEST_FILE_LOC" ]; then
    #echo "Error: No Manifest file found at location $TARGET_MNT_PT"
    #exit 1
  #fi
set -x
  #echo "MFILE : $MANIFEST_FILE_LOC"
  echo -e "LOG_BKP_MNTPT='$LOG_BKP_LOC'" >> $SYBASERESTORECONFIG
  #echo -e "MANIFEST_FILE_LOC='/$MANIFEST_FILE_LOC'" >> $SYBASERESTORECONFIG
  echo -e "LOG_RECOVERY='$LOG_RECOVERY'" >> $SYBASERESTORECONFIG
  echo -e "UNTIL_TIME='$UNTIL_TIME'" >> $SYBASERESTORECONFIG
  echo -e "TARGET_MNT_PNT='$TARGET_MNT_PNT'" >> $SYBASERESTORECONFIG
  echo -e "DATABASE_NAME='$DATABASE_NAME'" >> $SYBASERESTORECONFIG

  chmod -R 755 $SYBASERESTORECONFIG

  if [ "$LOG_RECOVERY" = "YES" ]; then
    rundata=$SYBASE_ASE_PATH"/install/RUNQ_"$TARGET_SERVER_NAME
    rundatatmp=$SYBASE_ASE_PATH"/install/RUNQ_"$TARGET_SERVER_NAME"_temp"
    echo "OS user is $OSUSER"
   userGroup=`ls -lrt $TARGET_SYBASE_SQLD | grep $OSUSER | tail -n 1 | awk -F" " '{print $4}'`
    rm -f $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME

    cp $SYBASE_ASE_PATH/install/RUN_$TARGET_SERVER_NAME $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME

    rm -f $rundatatmp

    sed "$(( $( wc -l < $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME) -2 ))s/$/\n-q \\\/" $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME > $rundatatmp
    mv $rundatatmp $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME
    chmod 755 $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME
    chown $OSUSER:$userGroup $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME

     cmd="$SYBASE_ASE_PATH/bin/startserver -f $SYBASE_ASE_PATH/install/RUNQ_$TARGET_SERVER_NAME"
     su - $OSUSER -c "nohup $cmd >& $SYBASE_ASE_PATH/install/logfiledataserver.out &"
     sleep 60
     backupServercmd="$SYBASE_ASE_PATH/bin/startserver -f $SYBASE_ASE_PATH/install/RUN_$BACKUP_RUN_PROCESS_NAME"
     su - $OSUSER -c "nohup $backupServercmd >& $SYBASE_ASE_PATH/install/logfilebackupserver.out &"
     sleep 60
 set +x
     sybaseStatus=$(su - $OSUSER -c "showserver" | grep -i dataserver | grep -iw "\-s$TARGET_SERVER_NAME")
     if [ -z "$sybaseStatus" ]; then
       echo "ERROR: ASE dataserver is not up.!"
       #exit 1
     fi

     sybasebackupStatus=$(su - $OSUSER -c "showserver" | grep -i backupserver | grep -iw "\-s$BACKUP_RUN_PROCESS_NAME")
     if [ -z "$sybasebackupStatus" ]; then
       echo "ERROR: ASE backupserver is not up.!"
       #exit 1
     fi


   tmp_loc=/act/touch
   chmod 775 $tmp_loc
   userGroup=`ls -lrt $TARGET_SYBASE_SQLD | grep $OSUSER | tail -n 1 | awk -F" " '{print $4}'`
   chown -R $OSUSER:$userGroup $tmp_loc
   touch /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
   chmod 777 /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
   touch /var/act/log/UDSAgent-trans_log_sql.log
   chmod 777 /var/act/log/UDSAgent-trans_log_sql.log
   set +x
   echo $TARGET_DBUSER_PASSWD | su - $OSUSER -c "/act/scripts/act_sybase_logrecovery.sh"
   retval=$?
   chmod 644 /var/act/log/UDSAgent-"$TARGET_SERVER_NAME"_recovery.log
   chmod 644 /var/act/log/UDSAgent-trans_log_sql.log
   if [ $retval -gt 0 ]; then
     echo "******** ERRORMSG: Log Recovery Failed! *******"
     exit 1
   fi

#shutdown Backup and Data server.
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_ESC_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
shutdown $BACKUP_SERVER_NAME
go
shutdown
go
EOF

if [ $? -gt "0" ]; then
 echo "ERROR: Shutting down Backup Server $BACKUP_SERVER_NAME Failed!"
fi

set -x
sybaseStatus=$(su - $OSUSER -c "showserver" | grep -i dataserver | grep -iw "\-s$TARGET_SERVER_NAME")
if [ ! -z "$sybaseStatus" ]; then
  echo "ERROR: Running ASE dataserver. Exiting!"
  exit 1
fi

sybasebackupStatus=$(su - $OSUSER -c "showserver" | grep -i backupserver | grep -iw "\-s$BACKUP_RUN_PROCESS_NAME")
if [ ! -z "$sybasebackupStatus" ]; then
  echo "ERROR: Running ASE backupserver. Exiting!"
  exit 1
fi

sleep 30
fi

cmd="$SYBASE_ASE_PATH/bin/startserver -f $SYBASE_ASE_PATH/install/RUN_$TARGET_SERVER_NAME"
su - $OSUSER -c "nohup $cmd >& $SYBASE_ASE_PATH/install/logfiledataserver.out &"
sleep 60
backupServercmd="$SYBASE_ASE_PATH/bin/startserver -f $SYBASE_ASE_PATH/install/RUN_$BACKUP_RUN_PROCESS_NAME"
su - $OSUSER -c "nohup $backupServercmd >& $SYBASE_ASE_PATH/install/logfilebackupserver.out &"
sleep 60

sybaseStatus=$(su - $OSUSER -c "showserver" | grep -i dataserver | grep -iw "\-s$TARGET_SERVER_NAME")
if [ -z "$sybaseStatus" ]; then
 echo "ERROR: ASE dataserver is not up.!"
 #exit 1
fi

sybasebackupStatus=$(su - $OSUSER -c "showserver" | grep -i backupserver | grep -iw "\-s$BACKUP_RUN_PROCESS_NAME")
if [ -z "$sybasebackupStatus" ]; then
 echo "ERROR: ASE backupserver is not up!"
 #exit 1
fi

set +x
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x

# Unmount the Log Images as Restoration of Log Transactions are complete.
if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
	  filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
    mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

    mountImageIDListLength=${#mountImageIDList[@]}
    echo "Existing mounts for logBackup are:"  $mountImageIDListLength

    if [ $mountImageIDListLength -gt 0 ]; then
       for (( i=0; i<${mountImageIDListLength}; i++ ))
       do
       # Unmount and Delete Image
       unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

        inputParamsForUnmount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"${mountImageIDList[i]}"'",
        "delete":"true"
       }'

       unmountResponse=$(curl -s -w "\n" -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
       if [[ $unmountResponse == *errormessage* ]]; then
          echo "Unmount Failed. Reason :" $unmountResponse
       elif [[ $unmountResponse == *result* ]]; then
          echo "Logbackup App Unmounted successfully."
       fi
      done
    fi
fi
