#!/bin/sh
# Script to configure iscsi and start the service.

setupCHAP()
{
  set +x
  sessionExists=$(iscsiadm -m session | grep -w $SkyIP:3260)
  if [ -z "$sessionExists" ]; then
    #targetIQN=`iscsiadm -m discovery -t st -p $SkyIP:3260 2> /dev/null | tail -n 1 | awk '{print $NF}'`
    #iscsiadm -m node -T $targetIQN -p $SkyIP:3260 -l
    iscsiadm -m iface -I actifiosap -o new

    iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 -o new

    iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 -o update -n discovery.sendtargets.auth.authmethod -v CHAP

    iqnName=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`

    iscsiadm -m iface -I actifiosap --op=update -n iface.initiatorname -v $iqnName

    iscsiadm -m iface -I actifiosap --op=update -n iface.discovery_auth -v none

    iscsiadm -m iface -I actifiosap --op=update -n iface.bidi_chap -v none
    #iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 -o update -n discovery.sendtargets.auth.username -v $CHAPUsername

    #iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 -o update -n discovery.sendtargets.auth.password -v $CHAPPassword

    targetIQN=`iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 --discover 2> /dev/null | tail -n 1 | awk '{print $NF}'`
    if [ -z "$targetIQN" ]; then
      echo "********** Error discovering Sky Portal $SkyIP: $targetIQN *************"
      exit 1
    fi

    CHAPUsername=`iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -S | grep -w "node.session.auth.username" | cut -d'=' -f2 | awk 'NF' | grep -v "<empty>" | xargs`

    if [ -z "$CHAPUsername" ]; then
        CHAPUsername=`date +%s | sha1sum | base64 | head -c 14`
    fi

    CHAPPassword=`iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -S | grep -w "node.session.auth.password" | cut -d'=' -f2 | awk 'NF'| grep -v "<empty>" | xargs`

    if [ -z "$CHAPPassword" ]; then
        CHAPPassword=`date +%s | sha256sum | base64 | head -c 14`
    fi

    configureCHAP

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.session.auth.authmethod -v CHAP

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.session.auth.username -v $CHAPUsername

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.session.auth.password -v $CHAPPassword

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.session.auth.username_in -v ''

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.session.auth.password_in -v ''

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.startup -v automatic

    iscsiadm -m node -T $targetIQN --portal=$SkyIP:3260 -o update -n node.conn[0].startup -v automatic

    iscsiadm -m node -T $targetIQN --login --portal=$SkyIP:3260 -I actifiosap
    retval=$?
    if [ $retval -ne 0 ]; then
      if [ $retval -eq 15 ]; then
        echo "*** Session Already Exists****"
      else
        echo "*** ERROR: Iscsi Login to $SkyIP has Failed************"
        exit $retval
      fi
    fi
  fi

  set -x
}

configureCHAP()
{
  #iscsiadm -m discoverydb -t sendtargets -p $SkyIP:3260 -o delete 2> /dev/null

  #sed -i '/node.session.auth.authmethod =/c\node.session.auth.authmethod = CHAP' /etc/iscsi/iscsid.conf
  #sed -i '/node.session.auth.username =/c\node.session.auth.username = '"$CHAPUsername"'' /etc/iscsi/iscsid.conf
  #sed -i '/node.session.auth.password =/c\node.session.auth.password = '"$CHAPPassword"'' /etc/iscsi/iscsid.conf

  #sed -i '/discovery.sendtargets.auth.authmethod =/c\discovery.sendtargets.auth.authmethod = CHAP' /etc/iscsi/iscsid.conf
  #sed -i '/discovery.sendtargets.auth.username =/c\discovery.sendtargets.auth.username = '"$CHAPUsername"'' /etc/iscsi/iscsid.conf
  #sed -i '/discovery.sendtargets.auth.password =/c\discovery.sendtargets.auth.password = '"$CHAPPassword"'' /etc/iscsi/iscsid.conf

  loginURL="https://$SkyIP/actifio/api/login?name=admin&password=$SkyPassword&vendorkey=$VendorKey"
  sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
  if [ -z $sessionid ]; then
     echo "Error : Session ID cannot be empty...Login has Failed"
     exit 1
  fi
  hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$SRC_HOSTNAME" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

  configChapURL="https://$SkyIP/actifio/api/task/configchap?sessionid=$sessionid"
  inputParamsCHAP='{
  "sessionid":'"$sessionid"',
  "host":'"$hostID"',
  "username":'"$CHAPUsername"',
  "password":'"$CHAPPassword"'
  }'
  chapConfigResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $configChapURL -d "$inputParamsCHAP")

  if [[ $chapConfigResponse == *errormessage* ]]; then
   echo "CHAP Configuration has Failed. Reason :" $chapConfigResponse
   exit 1
  fi
}

if [ -z $SRC_HOSTNAME ]; then
  if [ ! -z $TARGET_HOSTNAME ]; then
    SRC_HOSTNAME=$TARGET_HOSTNAME
  else
    SRC_HOSTNAME=`hostname`
  fi
fi

set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionID=$(curl -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x
if [ -z $sessionID ]; then
   echo "Error : Session ID cannot be empty."
   exit 1
fi

addedHostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$SRC_HOSTNAME" -d "sessionid=$sessionID" | jq -cr '(.result[] | .id)')
if [ -z $addedHostID ]; then
   echo " Error : Host ID cannot be empty."
   exit 1
fi
echo "Host already exists. HOST ID ::  $addedHostID"
getHostURL="https://$SkyIP/actifio/api/info/lshost?sessionid=$sessionID&argument=$addedHostID"
skyIscsiName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getHostURL | jq -cr '(.result | .iscsi_name)')
if [ -z $skyIscsiName ]; then
  sed -i 's/:[^:]*$/:'`hostname`'/' /etc/iscsi/initiatorname.iscsi
fi

osRelease=$(cat /etc/*-release | grep -i SLES)
if [ ! -z "$osRelease" ]; then
  isIscsiRunning=$(systemctl status iscsid | grep Active)
  if [[ $isIscsiRunning == *"Active: active (running)"* ]]; then
    echo "ISCSI service is running" > /dev/null
  else
    echo "Trying to Start Iscsi Service"
    checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
    if [ -z $checkopeniscsi ]; then
      if ! type "zypper" 2> /dev/null; then
        echo "Error : Zypper is not installed. Unable to proceed with ISCSI installation."
        exit 1
      fi
    # open-iscsi-2.0.874-53.10.1.x86_64 (ensure that right iscsi package version is present)
      zypper -n install open-iscsi
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: ******* Could not install open-iscsi ************"
        exit $retval
      fi
      checkopeniscsi=$(rpm -qa | grep -w open-iscsi*)
      if [ -z $checkopeniscsi ]; then
        echo "Error: Cannot find open-iscsi package in the system"
        exit 1
      fi
    fi
  fi

  #sed -i '/^node.startup =/c\node.startup = automatic' /etc/iscsi/iscsid.conf
  systemctl enable iscsi
  systemctl enable iscsid
  systemctl stop iscsid 2>/dev/null
  systemctl start iscsid
else
  isIscsiRunning=$(service iscsid status)
  if [[ $isIscsiRunning == *"is running"* ]]; then
     :
  else
    echo "Trying to Start Iscsi Service"
    checkopeniscsi=$(yum list installed | grep iscsi-initiator-utils)
    if [ -z "$checkopeniscsi" ]; then
      if ! type "yum" > /dev/null 2>&1; then
        echo "Error : yum is not installed. Unable to proceed with ISCSI installation."
        exit 1;
      fi
      # open-iscsi-2.0.874-53.10.1.x86_64 (ensure that right iscsi package version is present)
      yum -y install iscsi-initiator-utils
      retval=$?
      if [ $retval -ne 0 ]; then
        echo "Error: ******* Could not install open-iscsi ************"
        exit $retval
      fi
      checkopeniscsi=$(yum list installed | grep iscsi-initiator-utils)
      if [ -z "$checkopeniscsi" ]; then
        echo "Error: Cannot find open-iscsi package in the system"
        exit 1
      fi
    fi
  fi

  #sed -i '/^node.startup =/c\node.startup = automatic' /etc/iscsi/iscsid.conf
  chkconfig iscsid on
  service iscsid stop > /dev/null 2>&1
  service iscsid start
fi

setupCHAP
