#!/bin/sh

if [ "$#" -lt 1 ]; then
echo " This script need 1 input parameter"
echo " usage: createSybaseIQDump.sh <Added Host ID>"
echo " For example: createSybaseIQDump.sh 2468"
exit 1
fi

hostid=$1

echo "---------------------------------------------------"
echo "-----Start Phase: Protecting the Sybase IQ DB App-------"
echo "---------------------------------------------------"

set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x

# Fetching app id for /actbackup
lsAppUrl="https://$SkyIP/actifio/api/info/lsapplication"
actBackupAppID=$(curl -s -k  -G $lsAppUrl --data-urlencode "filtervalue=appname=$SYBASEIQBACKUPPATH&hostid=$hostid" -d "sessionid=$sessionid" | jq -cr '(.result [] | .id)')

if [ -z $actBackupAppID ]; then
   echo " Error : No App found in $SkyIP with name $appName"
   exit 1
fi

if [ "$SRC_DBNAME" == "" ]; then
  DBNAMES="ALL"
else
  DBNAMES=$SRC_DBNAME
fi

chApplicationURL="https://$SkyIP/actifio/application/$actBackupAppID?sessionid=$sessionid"
inputParamsForApplication='{
	"description":{"type":"db","method":"file","level":"full","system-id":"'"$SRC_SERVER_NAME"'",
    "dbtype":"ase","database-name":"'"$DBNAMES"'"}
}'
chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
echo "DB Application update Response: $chAppResponse"
#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionid&filtervalue=name%3D$IQTemplateName"
templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

if [ -z $templateID ]; then
  source /act/act_scripts/sybaseiq/automation/Backup/createTemplate.sh $IQTemplateName $dbbackup_rpo "Snap Policy" "ASE DB Policy"
  templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')
fi

profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
profileID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')

#Advanced Settings
set +x
updateAdvancedSettingsURL="https://$SkyIP/actifio/api/task/mkpolicyoption"
paramsForAdvancedSettings='{
"sessionid":'"\"$sessionid\""',
"appid":'"\"$actBackupAppID\""',
"name":'"\"skypassword\""',
"value":'"\"$SkyPassword\""'
}'
updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")
if [[ $updateAdvanceSettingResponse == *errormessage* ]]; then
    echo "$updateAdvanceSettingResponse" | jq -cr '.errormessage'
fi
if [ ! -z $SRC_USER_PASSWD ]; then
  paramsForAdvancedSettings='{
  "sessionid":'"\"$sessionid\""',
  "appid":'"\"$actBackupAppID\""',
  "name":'"\"sapdbpassword\""',
  "value":'"\"$SRC_USER_PASSWD\""'
  }'
  updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")
  if [[ $updateAdvanceSettingResponse == *errormessage* ]]; then
      echo "$updateAdvanceSettingResponse" | jq -cr '.errormessage'
  fi
fi
set -x

#Protect the app

protectionURL="https://$SkyIP/actifio/api/task/mksla"

inputParamsForProtection='{
"sessionid":'"$sessionid"',
"appid":'"$actBackupAppID"',
"slpid":'"$profileID"',
"sltid":'"$templateID"'
}'

protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")
if [[ $protectionResponse == *errormessage* ]]; then
  echo $SYBASEIQBACKUPPATH `echo  $protectionResponse | jq -cr '.errormessage'`
else
  if [[ $protectionResponse == *result* ]]; then
   echo "App Protected successfully."
  fi
fi
   echo "---------------------------------------------------"
   echo "Configuring the Sybase IQ DB App for freeze and thaw phases"
   echo "---------------------------------------------------"

   #Copy scripts from /sybase/outofband to /act/scripts and give permission

   sudo cp /act/act_scripts/sybaseiq/dump/* /act/scripts/ 2>/dev/null
   chmod 755 /act/scripts/*
   # Rename the script files
   sudo mv /act/scripts/freeze.xxx /act/scripts/freeze.$actBackupAppID
   sudo mv /act/scripts/appid.xxx /act/scripts/appid.$actBackupAppID

   echo "---------------------------------------------------"
   echo "Completed Sybase IQ DB App Configuration"
   echo "---------------------------------------------------"

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "----End Phase: Completed the Sybase IQ DB App Protection---"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
