#!/bin/sh

if [ "$#" -lt 1 ]; then
echo " This script need 1 input parameter"
echo " usage: logProtection.sh <Added Host ID>"
echo " For example: logProtection.sh 2468"
exit 1
fi

hostid=$1
if [ -z $LOG_BKP_LOC ]; then
  LOGBKPLOC=$LogBackupFS
else
  LOGBKPLOC=$LOG_BKP_LOC
fi
#Create Log backup configuration file.
SYBASEIQLOGBACKUPCONFIG="/act/scripts/act_SybaseIQ_logbackup.conf"

#Creating config file with values.
echo -e "# sybase log backup config " > $SYBASEIQLOGBACKUPCONFIG
echo -e "OSUSER='$OSUSER'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "SRC_SYBASEIQ_SQLD='$SRC_SYBASEIQ_SQLD'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "SRC_BACKUP_USER='$SRC_BACKUP_USER'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "SRC_DBNAME='$SRC_DBNAME'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "SRC_SERVER_NAME='$SRC_SERVER_NAME'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "LogBackupFS='$LogBackupFS'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "LOG_BKP_LOC='$LOGBKPLOC'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "BKP_LOG_RETENTION='$DELLOG'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "LOGBKPLOC='$LOGBKPLOC'" >> $SYBASEIQLOGBACKUPCONFIG
echo -e "STRIPEON='$STRIPEON'" >> $SYBASEIQLOGBACKUPCONFIG
# Script to protect and run the on-demand job for Log volume.

echo "---------------------------------------------------"
echo "-------Start Phase: Protecting the Log App---------"
echo "---------------------------------------------------"

set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x
#Advanced Settings
updateAdvancedSettingsURL="https://$SkyIP/actifio/api/task/mkpolicyoption"

# Fetching app id for $LogBackupFS
appUrl="https://$SkyIP/actifio/api/info/lsapplication"
logBackupAppID=$(curl -s -k  -G $appUrl --data-urlencode "filtervalue=appname=$LogBackupFS&hostid=$hostid" -d "sessionid=$sessionid"   -H 'Cache-Control: no-cache' | jq -cr '(.result [] | .id)')


if [ -z $logBackupAppID ]; then
   echo " Error : No App found in $SkyIP with name $LogBackupFS"
   exit 1
fi


if [ "$SRC_DBNAME" == "" ]; then
  DBNAMES="ALL"
else
  DBNAMES=$SRC_DBNAME
fi
chApplicationURL="https://$SkyIP/actifio/application/$logBackupAppID?sessionid=$sessionid"
inputParamsForApplication='{
	"description":{"type":"log","method":"file","level":"incr","system-id":"'"$SRC_SERVER_NAME"'",
    "dbtype":"ase","database-name":"'"$DBNAMES"'"}
}'
chAppResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chApplicationURL -d "$inputParamsForApplication")
echo "Log Application update Response: $chAppResponse"

if [ ! -z $LOG_BKP_LOC ]; then
paramsForAdvancedSettings='{
"sessionid":'"\"$sessionid\""',
"appid":'"\"$logBackupAppID\""',
"name":'"\"startpaths\""',
"value":'"\"$LOG_BKP_LOC\""'
}'

updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")

echo "Advanced Settings Response : " $updateAdvanceSettingResponse
fi

set +x
paramsForAdvancedSettings='{
"sessionid":'"\"$sessionid\""',
"appid":'"\"$logBackupAppID\""',
"name":'"\"skypassword\""',
"value":'"\"$SkyPassword\""'
}'
updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")
if [[ $updateAdvanceSettingResponse == *errormessage* ]]; then
    echo "$updateAdvanceSettingResponse" | jq -cr '.errormessage'
fi
paramsForAdvancedSettings='{
"sessionid":'"\"$sessionid\""',
"appid":'"\"$logBackupAppID\""',
"name":'"\"sapdbpassword\""',
"value":'"\"$SRC_USER_PASSWD\""'
}'
updateAdvanceSettingResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $updateAdvancedSettingsURL -d "$paramsForAdvancedSettings")
if [[ $updateAdvanceSettingResponse == *errormessage* ]]; then
    echo "$updateAdvanceSettingResponse" | jq -cr '.errormessage'
fi
set -x
if [ -z $LogTemplateName ]; then
   echo " Error : Log Template name cannot be empty."
   exit 1
fi

if [ -z $logbackup_rpo ]; then
   echo " Error : Log Backup RPO cannot be empty."
   exit 1
fi

#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lsslt?sessionid=$sessionid&filtervalue=name%3D$LogTemplateName"
templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')

if [ -z $templateID ]; then
  source /act/act_scripts/sybaseiq/automation/Backup/createTemplate.sh $LogTemplateName $logbackup_rpo "Snap Policy" "Log Backup Policy"
  templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .id)')
fi

profileURL="https://$SkyIP/actifio/api/info/lsslp?sessionid=$sessionid&filtervalue=name%3D$ProfileName"
profileID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $profileURL | jq -cr '(.result[] | .id)')

#Protect the app

protectionURL="https://$SkyIP/actifio/api/task/mksla"

inputParamsForProtection='{
"sessionid":'"$sessionid"',
"appid":'"$logBackupAppID"',
"slpid":'"$profileID"',
"sltid":'"$templateID"'
}'

protectionResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $protectionURL -d "$inputParamsForProtection")
if [[ $protectionResponse == *errormessage* ]]; then
   echo $LogBackupFS `echo  $protectionResponse | jq -cr '.errormessage'`
else
  if [[ $protectionResponse == *result* ]]; then
    echo "Log App Protected successfully."
fi
fi
   echo "---------------------------------------------------------------"
   echo "Start Phase: Configuring the Log App for freeze and thaw phases"
   echo "---------------------------------------------------------------"

   # Copy scripts from /sybase/outofband to /act/scripts and give permission
   #if [ "$BACKUP_TYPE" == "LVMSnapshot" ]; then
     #sudo cp -r /act/act_scripts/sybase/outofband/log/* /act/scripts/
   #else
     sudo cp -r /act/act_scripts/sybaseiq/dump/log/* /act/scripts/
   #fi
   chmod -R 755 /act/scripts/*

   # Rename the script files
   sudo mv /act/scripts/appid.xxx /act/scripts/appid.$logBackupAppID
   sudo mv /act/scripts/freeze.xxx /act/scripts/freeze.$logBackupAppID
   sudo mv /act/scripts/thaw.xxx /act/scripts/thaw.$logBackupAppID
   chmod 755 /act/scripts/act_SybaseIQ_logbackup.conf

   echo "------------------------------------------------------"
   echo "----End Phase: Completed the Log App Configuration----"
   echo "------------------------------------------------------"

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "------End Phase: Completed the Log App Protection-----"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
