#!/bin/sh

if [ "$#" -lt 2 ]; then
echo " This script need 2 input parameters"
echo " usage: runOnDemandBackup.sh <SkyIP> /<App Name>"
echo " For example: runOnDemandBackup.sh 172.23.16.6 /testApp"
exit 1
fi

SkyIP=$1
appName=$2
SkyPassword=$3
schedule=$4

if [[ -z "$appName" ]]; then
   echo " Error : AppName cannot be empty"
   exit 1
fi

echo "-------------------------------------------------------------"
echo "--------------Creating MAXDB Snapshot for $appName----------------"
echo "-------------------------------------------------------------"

SkyUserName="admin"
VendorKey="act"

loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
# Fetching Host id
hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=`hostname`" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')

# Fetching app id for /actbackup
lsAppUrl="https://$SkyIP/actifio/api/info/lsapplication"
backupAppID=$(curl -s -k  -G $lsAppUrl --data-urlencode "filtervalue=appname=$appName&hostid=$hostID" -d "sessionid=$sessionid"   -H 'Cache-Control: no-cache' | jq -cr '(.result [] | .id)')
if [[ -z "$backupAppID" ]]; then
   echo " Error : No App found in $SkyIP with name $appName"
   exit 1
fi

if [ ! -z $schedule ]; then
  if [ -f "/act/scripts/act_SybaseIQ_Src.conf" ]; then
    echo "Updating the schedule"
    sed -i 's/^DUMPSCHEDULE.*/DUMPSCHEDULE='$schedule'/' /act/scripts/act_SybaseIQ_Src.conf
  fi
fi
#Fetch template and profile id from the names.

templateURL="https://$SkyIP/actifio/api/info/lssla?sessionid=$sessionid&filtervalue=appid%3D$backupAppID"
templateID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $templateURL | jq -cr '(.result[] | .sltid)')
echo "Template ID from Name: " $templateID


# Get the policy id for running on-demand job

getPolicyDetailsURL="https://$SkyIP/actifio/api/info/lspolicy?sessionid=$sessionid&filtervalue=sltid%3D$templateID%26op%3Dsnap&apistart=0&apilimit=1"
policyID=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getPolicyDetailsURL | jq -cr '(.result[] | .id)')

onDemandJobURL="https://$SkyIP/actifio/api/task/backup"

inputParamsForOnDemandJob='{
"sessionid":'"$sessionid"',
"app":'"$backupAppID"',
"policy":'"$policyID"'
}'

onDemandJobResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $onDemandJobURL -d "$inputParamsForOnDemandJob")
jobID=$(echo $onDemandJobResponse | jq -cr .result)
echo "Job ID: " $jobID

jobTracking()
{
  errorMessage=""
  errorCode=""
  jobinprogress=true
  while $jobinprogress
      do
          sleep 30
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                                  errorCode=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .errorcode)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
}
jobTracking
printf "\n The Backup job $jobID has $status.\n"

if [ ! -z "$errorMessage" ]; then
  printf "\n Reason: $errorMessage.\n"
  printf "\n errorCode: $errorCode.\n"
  retryBackup=`echo $errorMessage | grep -i "scriptexecretcode 123"`
  if [ ! -z "$retryBackup" ]; then
    echo "Retry the backup job"
    onDemandJobResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $onDemandJobURL -d "$inputParamsForOnDemandJob")
    jobID=$(echo $onDemandJobResponse | jq -cr .result)
    echo "Job ID: " $jobID
    jobTracking
    printf "\n The Backup job $jobID has $status.\n"
    if [ ! -z "$errorMessage" ]; then
        printf "\n Reason: $errorMessage.\n"
        printf "\n errorCode: $errorCode.\n"
        exit 1
    fi
  fi
  exit 1
fi

echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
echo "---------Successfully Created MAXDB Snapshot for $appName---------"
echo "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
