#!/bin/sh

source /act/scripts/act_sybaseiq_dump_restore_config.conf
OSGROUP=`id -ngr $OSUSER`
tmp_loc=/act/touch
if [ ! -d $tmp_loc ]; then
  mkdir -p $tmp_loc
fi
chmod 775 $tmp_loc

touch $tmp_loc/restore.sql
touch $tmp_loc/stopdb.sql
chown -R $OSUSER:$OSGROUP $tmp_loc

TARGET_MNT=$DB_BACKUP_LOCATION

for i in $(echo $TARGET_DBNAME | sed "s/,/ /g")
do
    dbname=$i
    dbuser=`echo $TARGET_DB_USER | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    set +x
    dbpasswd=`echo $TARGET_DBUSER_PASSWD | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    utilitydbuser=`echo $UTILITYDB_USER | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    utilitydbpasswd=`echo $UTILITYDB_PASSWD | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    sybaseiqsqlpath=`echo $TARGET_SYBASEIQ_SQLD | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    dbcatalogfile=`echo $DB_CATALOG_PATH | sed "s/,/\n/g" | grep -w $dbname | head -n 1 | cut -d ':' -f 2`
    if [ -z $dbuser ] || [ -z $dbpasswd ] || [ -z $utilitydbuser ] || [ -z $utilitydbpasswd ] || [ -z $sybaseiqsqlpath ]; then
      echo "Error: dbuser/dbpasswd/utilitydbuser/utilitydbpasswd/sybaseiqsqlpath is empty"
      exit 1
    fi
    if [ -z $dbcatalogfile ]; then
      echo "Error dbcatalogfile cannot be empty"
      exit 1
    fi
    set -x
    # Remove Trailing / just in case its present
    sybaseiqsqlpath="${sybaseiqsqlpath%\/}"
    if [ ! -d $sybaseiqsqlpath ]; then
      echo "Error: $sybaseiqsqlpath does not exist"
      exit 1
    fi
    SYBIQ_HOME=$sybaseiqsqlpath
    echo "SYBIQ_HOME = $SYBIQ_HOME"
    version=`echo "$sybaseiqsqlpath" | awk -F"/" '{print $NF}'`
    echo "Version= $version"
    #SET  env
    set +x
    . $SYBIQ_HOME/$version.sh
    set -x

    #checking utility server status
    util_status=`pgrep -a iqsrv | grep -w utility_db | grep -w $sybaseiqsqlpath | wc -l`
    if [ "$util_status" -eq 0 ] ; then
     echo "WARNING : utility server is not running ! Trying to start"
     su -m $OSUSER -c "start_iq -n utility_db -su "$utilitydbuser","$utilitydbpasswd""
     util_status=`pgrep -a iqsrv | grep -w utility_db | grep -w $sybaseiqsqlpath | wc -l`
     if [ "$util_status" -eq 0 ] ; then
       echo "ERRORMSG : utility server is still not running ! Exiting!!!"
       exit 1
     fi
    else
     echo "utility server is running"
    fi

    #creating sql for stopping database engine
    echo "stop engine $dbname unconditionally; " > /act/touch/stopdb.sql
    echo " exit;" >> /act/touch/stopdb.sql
    #stopping database server
    set +x
    su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$dbuser;pwd=$dbpasswd;eng=$dbname;dbn=$dbname;'  /act/touch/stopdb.sql"
    output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$dbuser;pwd=$dbpasswd;eng=$dbname;dbn=$dbname;' sp_iqstatus" 2>&1 | grep -i "could not connect"`
    set -x
    if [ ! -z "$output" ]
    then
      echo "sybaseiq Stopped DB Successfully for $dbname"
    else
      echo "Error: sybaseiq DB Stop failed for $dbname"
      exit 1
    fi

    CHECK_INCR_BKP=`ls $TARGET_MNT/$dbname/INCR | grep -i "dump" | wc -l`
    if [ "$CHECK_INCR_BKP" -gt "0" ]; then
      RESTORE_TYPE=I
    else
      RESTORE_TYPE=F
    fi
    #connecting to utility server and restoring database
    if [ "$RESTORE_TYPE" = "I" ] && [ ! -z "$RECOVERY_TIME" ]; then
       filename_tmp=`ls -ltr $TARGET_MNT/$dbname/FULL | tail -1 | awk '{print $(NF)}'`
       filename=`echo ${filename_tmp:0:-2}`
       echo "filename $filename"

       filename_tmp2=`ls -ltr $TARGET_MNT/$dbname/INCR | tail -1 | awk '{print $(NF)}'`
       filename2=`echo ${filename_tmp2:0:-2}`
       echo "filename2 $filename2"
       echo " RESTORE DATABASE '$dbcatalogfile' FROM '$TARGET_MNT/$dbname/FULL/$filename' overwrite existing ; " > /act/touch/restore.sql
       echo "RESTORE DATABASE '$dbcatalogfile' FROM '$TARGET_MNT/$dbname/INCR/$filename2' RECOVER UNTIL TIMESTAMP '$RECOVERY_TIME' USING LOG PATH  '$LOG_BKP_LOC/PITR/$dbname/'  overwrite existing clear log;" >> /act/touch/restore.sql

       echo "exit;" >> /act/touch/restore.sql
       set +x
       output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$utilitydbuser;pwd=$utilitydbpasswd;eng=utility_db;dbn=utility_db;'  /act/touch/restore.sql"`
       retval=$?
       set -x
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq FileSystem recovery completed for $dbname"
       else
         echo "ERRORMSG: sybaseiq FileSystem recovery failed because of $output for $dbname"
         exit $retval
       fi
       su -m $OSUSER -c "start_iq $dbcatalogfile"
       retval=$?
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq Server Startup completed for $dbname"
       else
         echo "sybaseiq Server Startup failed with exit status $retval for $dbname"
         exit $retval
       fi
    elif [ "$RESTORE_TYPE" = "I" ] && [ -z "$RECOVERY_TIME" ];then
       filename_tmp=`ls -ltr $TARGET_MNT/$dbname/FULL | tail -1 | awk '{print $(NF)}'`
       filename=`echo ${filename_tmp:0:-2}`
       echo "filename $filename"

       filename_tmp2=`ls -ltr $TARGET_MNT/$dbname/INCR | tail -1 | awk '{print $(NF)}'`
       filename2=`echo ${filename_tmp2:0:-2}`
       echo "filename2 $filename2"
       echo " RESTORE DATABASE '$dbcatalogfile' from '$TARGET_MNT/$dbname/FULL/$filename' overwrite existing; " > /act/touch/restore.sql
       echo " RESTORE DATABASE '$dbcatalogfile' from '$TARGET_MNT/$dbname/INCR/$filename2' overwrite existing clear log; " >> /act/touch/restore.sql
       echo "exit;" >> /act/touch/restore.sql
       set +x
       output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$utilitydbuser;pwd=$utilitydbpasswd;eng=utility_db;dbn=utility_db;'  /act/touch/restore.sql"`
       retval=$?
       set -x
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq FileSystem recovery completed for $dbname"
       else
         echo "sybaseiq FileSystem recovery failed because of $output for $dbname"
         exit $retval
       fi
       su -m $OSUSER -c "start_iq $dbcatalogfile"
       retval=$?
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq Server Startup completed for $dbname"
       else
         echo "sybaseiq Server Startup failed with exit status $retval for $dbname"
         exit $retval
       fi
    fi

    if [ "$RESTORE_TYPE" = "F" ] && [ ! -z "$RECOVERY_TIME" ];then
       filename_tmp=`ls -ltr $TARGET_MNT/$dbname/FULL | tail -1 | awk '{print $(NF)}'`
       filename=`echo ${filename_tmp:0:-2}`
       echo "filename $filename"
       echo "RESTORE DATABASE '$dbcatalogfile' FROM '$TARGET_MNT/$dbname/FULL/$filename' RECOVER UNTIL TIMESTAMP '$RECOVERY_TIME' USING LOG PATH  '$LOG_BKP_LOC/PITR/$dbname/' overwrite existing clear log;" > /act/touch/restore.sql

       echo "exit;" >> /act/touch/restore.sql
       set +x
       output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$utilitydbuser;pwd=$utilitydbpasswd;eng=utility_db;dbn=utility_db;'  /act/touch/restore.sql"`
       retval=$?
       set -x
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq FileSystem recovery completed for $dbname"
       else
         echo "ERRORMSG: sybaseiq FileSystem recovery failed because of $output for $dbname"
         exit $retval
       fi
       su -m $OSUSER -c "start_iq $dbcatalogfile"
       retval=$?
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq Server Startup completed for $dbname"
       else
         echo "sybaseiq Server Startup failed with exit status $retval for $dbname"
         exit $retval
       fi
    elif [ "$RESTORE_TYPE" = "F" ] && [  -z "$RECOVERY_TIME" ];then
       filename_tmp=`ls -ltr $TARGET_MNT/$dbname/FULL | tail -1 | awk '{print $(NF)}'`
       filename=`echo ${filename_tmp:0:-2}`
       echo "filename $filename"
       echo " RESTORE DATABASE '$dbcatalogfile' from '$TARGET_MNT/$dbname/FULL/$filename' overwrite existing clear log; " > /act/touch/restore.sql
       echo "exit;" >> /act/touch/restore.sql
       set +x
       output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$utilitydbuser;pwd=$utilitydbpasswd;eng=utility_db;dbn=utility_db;'  /act/touch/restore.sql"`
       retval=$?
       set -x
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq FileSystem recovery completed for $dbname"
       else
         echo "sybaseiq FileSystem recovery failed because of $output for $dbname"
         exit $retval
       fi
       su -m $OSUSER -c "start_iq $dbcatalogfile"
       retval=$?
       if [ $retval -eq 0 ]
       then
         echo "sybaseiq Server Startup completed for $dbname"
       else
         echo "sybaseiq Server Startup failed with exit status $retval for $dbname"
         exit $retval
       fi
     fi
done

su -m $OSUSER -c "dbisql -nogui -onerror exit -c 'uid=$utilitydbuser;pwd=$utilitydbpasswd;eng=utility_db;dbn=utility_db;' stop engine utility_db unconditionally"
