#!/bin/sh
set +x
if [ "$#" -lt 1 ]; then
  echo " This script needs key-value input parameters"
  echo " usage: restore.sh <name=value>"
  echo " For example: restore.sh OSUSER=sybaseiq"
  exit 1
fi

restoreConfFile="/act/act_scripts/sybaseiq/automation/Configuration/sybaseiqrestore.conf"

echo -e '#sybaseiq Restore Params' > $restoreConfFile
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   passwdstrCheck=`echo $ARGUMENT | grep -i "password\|passwd"`
   if [ -z "$passwdstrCheck" ]; then
     echo -e $ARGUMENT >> $restoreConfFile
   else
     export $ARGUMENT
   fi
done

source $restoreConfFile

if [ -z $GenericAppName ]; then
  echo "Error: Actifio Generic App Name is Mandatory"
  exit 1
fi

if [ -z $dbimagename ]; then
  echo "Error: Image Id is Mandatory"
  exit 1
fi

if [ -z $OSUSER ]; then
  echo "Error: OS-Username is mandatory"
  exit 1
fi

if [ -z $SkyIP ]; then
  echo "Error: Sky IP is mandatory"
  exit 1
fi

if [ -z $SkyPassword ]; then
  echo "Error: SkyPassword is mandatory"
  exit 1
fi

if [ -z $TARGET_DBNAME ]; then
  echo "Error: DBNAME is mandatory"
  exit 1
fi

if [ -z $TARGET_SYBASEIQ_SQLD ]; then
  echo "Error: TARGET_SYBASEIQ_SQLD is mandatory"
  exit 1
fi

if [ -z $TARGET_HOSTNAME ]; then
  TARGET_HOSTNAME=`hostname`
fi

SkyUserName="admin"
VendorKey="act"

echo "----Checking Pre-Requisites(Agent,Iscsi) on Host machine----"
source /act/act_scripts/sybaseiq/automation/Backup/preRequisites.sh $SkyIP
echo "--------Done Checking Pre-Requisites on Host machine--------"
set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x
hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$TARGET_HOSTNAME" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
iscsiname=`grep iqn /etc/iscsi/initiatorname.iscsi | cut -d '=' -f 2`
hostIP=`hostname -i`
if [ -z "$hostID" ]; then
  addHostURL="https://$SkyIP/actifio/api/task/mkhost -H 'Content-Type:application/json'"

  inputParamsForAddHost='{
   "sessionid":"'"$sessionid"'",
   "type":"generic",
   "hostname":"'"$TARGET_HOSTNAME"'",
   "ipaddress":"'"$hostIP"'",
   "iscsiname":"'"$iscsiname"'"
  }'
  addHostResponse=$(curl -s -k -XPOST $addHostURL -d "$inputParamsForAddHost")
  hostID=$(echo $addHostResponse | jq -cr '.result')
  echo "Do CHAP configuration"
  source /act/act_scripts/sybaseiq/automation/Backup/preRequisites.sh $SkyIP
fi

#Ensure IQN of host is latest in sky
echo "HOST ID ::  $hostID"
getHostURL="https://$SkyIP/actifio/api/info/lshost?sessionid=$sessionid&argument=$hostID"
oldIscsiName=$(curl -s -k  -H 'Content-Type: application/json' -X GET $getHostURL | jq -cr '(.result | .iscsi_name)')
if [ -z $oldIscsiName ]; then
  chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
  inputParamsForChangeHost='{
    "iscsiport": [ "'"$iscsiname"'" ]
  }'
  chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
  echo "Host Update Response with IQN: $chHostResponse"
else
  if [ $oldIscsiName != $iscsiname ]; then
   deleteIscsiHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid&iscsiname=%5B%22$oldIscsiName%22%5D&force=true"
   delHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X DELETE $deleteIscsiHostURL)
   echo "Host Delete IQN Response: $delHostResponse"

   chHostURL="https://$SkyIP/actifio/svchost/$hostID/port?sessionid=$sessionid"
   inputParamsForChangeHost='{
     "iscsiport": [ "'"$iscsiname"'" ]
   }'
   chHostResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chHostURL -d "$inputParamsForChangeHost")
   echo "Host Update Response with IQN: $chHostResponse"
  fi
fi

lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
appFilter="filtervalue=appname=$GenericAppName&backupname=$dbimagename"
appType=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$appFilter"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .apptype)')

if [ "$appType" == "FileSystem" ]; then
  source /act/act_scripts/sybaseiq/automation/Restore/restoreFS.sh
else
  if [ "$appType" == "LVM Volume" ]; then
    source /act/act_scripts/sybaseiq/automation/Restore/restoreLVM.sh
  else
    echo "ERROR: Invalid AppType: $appType"
    exit 1
  fi
fi
