#!/bin/sh

set +x
loginURL="https://$SkyIP/actifio/api/login?name=$SkyUserName&password=$SkyPassword&vendorkey=$VendorKey"
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')

lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"
dbappFilter="filtervalue=appname=$GenericAppName&backupname=$dbimagename"
dbappID=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$dbappFilter"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .appid)'))

# Disable Schedule For DB Application
dbslaid=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lssla" --data-urlencode "filtervalue=appid=$dbappID"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$dbslaid&scheduleoff=true"
chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)

# Disable Schedule for Log Application
if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
  logappFilter="filtervalue=appname=$LogBackupFS&backupname=$logimagename"
  logappID=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$logappFilter"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .appid)'))

  logslaid=$(curl -sS -w "\n" -k -G  "https://$SkyIP/actifio/api/info/lssla" --data-urlencode "filtervalue=appid=$logappID"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')
  chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$logslaid&scheduleoff=true"
  chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)
fi
hostID=$(curl -sS  -k -G  "https://$SkyIP/actifio/api/info/lshost"  --data-urlencode "filtervalue=hostname=$TARGET_HOSTNAME" -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)')


echo "Image: $dbimagename sybaseiq DB MountPath: $TARGET_MNT_PNT"
mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

if [ -z $TARGET_MNT_PNT ]; then
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true"
     }'
else
  inputParamsForMount='{
      "sessionid":"'"$sessionid"'",
      "image":"'"$dbimagename"'",
      "host":"'"$hostID"'",
      "nowait":"true",
      "restoreoption":"'"mountpointperimage=$TARGET_MNT_PNT"'"
     }'
fi

jobID=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount" |  jq -cr '(.result)' | cut -d ' ' -f1)
printf "\n The Job id is: $jobID \n"

if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
  printf "\n Reason: jobID cannot be null.\n"
  exit 1
fi

jobinprogress=true
errorMessage=""
while $jobinprogress
    do
        sleep 30
        getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
        status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
        if [ ! -z "$status" ]; then
                if [ $status == "succeeded" -o $status == "failed" ]
                then
                        jobinprogress=false
                        if [ $status == "failed" ]
                        then
                                getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                        fi
                fi
              else
                jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                echo "Job is still running..Completed $status%"
        fi
done
printf "\n The sybaseiq DB Mount job $jobID has $status.\n"

if [ ! -z "$errorMessage" ]; then
  printf "\n Reason: $errorMessage.\n"
  exit 1
fi

if [ -z $TARGET_MNT_PNT ]; then
  TARGET_MNT_PNT=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
fi

if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then

  if [ -z "$UNTIL_TIME" ]; then
    printf "\n Reason: Recovery Time has not been set for logBackup Image\n"
    exit 1
  fi
  LOG_RECOVERY="YES"
  lsBackupURL="https://$SkyIP/actifio/api/info/lsbackup"

  filterValue="filtervalue=backupname=$logimagename"
  backupResponse=$(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid")
  appid=$(echo $backupResponse | jq -cr '(.result[] | .appid)')
  policyOptionURL="https://$SkyIP/actifio/api/info/lspolicyoption?sessionid=$sessionid&filtervalue=appid%3D$appid%26name%3Dstartpaths&apistart=0&apilimit=1"
  LOGBACKUPPATH=$(curl -s -k  -H 'Content-Type: application/json' -X GET $policyOptionURL | jq -cr '(.result[] | .value)')

  printf "Mounting Data from Log Backup App Image"

  echo "Image: $logimagename MOUNTPATH: $LOG_BKP_LOC"
  mountImageURL="https://$SkyIP/actifio/api/task/mountimage"

  if [ -z $LOG_BKP_LOC ]; then
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true"
       }'
  else
    inputParamsForMount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"$logimagename"'",
        "host":"'"$hostID"'",
        "nowait":"true",
        "restoreoption":"'"mountpointperimage=$LOG_BKP_LOC"'"
       }'
  fi

  jobID=$(curl -sS -k -H 'Content-Type:application/json' -X POST $mountImageURL -d "$inputParamsForMount" |  jq -cr '(.result)' | cut -d ' ' -f1)
  printf "\n The Job id is: $jobID \n"
  if [ -z "$jobID" ] || [ "$jobID" == "null" ]; then
    printf "\n Reason: jobID cannot be null.\n"
    exit 1
  fi
  jobinprogress=true
  errorMessage=""
  while $jobinprogress
      do
          sleep 30
          getStatusURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&filtervalue=jobname=$jobID"
          status=$(curl -Ss -k -X GET $getStatusURL | jq -cr '(.result []? | .status)')
          if [ ! -z "$status" ]; then
                  if [ $status == "succeeded" -o $status == "failed" ]
                  then
                          jobinprogress=false
                          if [ $status == "failed" ]
                          then
                                  getMessageURL="https://$SkyIP/actifio/api/info/lsjobhistory?sessionid=$sessionid&argument=$jobID"
                                  errorMessage=$(curl -Ss -k -X GET $getMessageURL | jq -cr '(.result | .message)')
                          fi
                  fi
                else
                  jobProgress="https://$SkyIP/actifio/api/info/lsjob?sessionid=$sessionid&filtervalue=jobname=$jobID"
                  status=$(curl -Ss -k -X GET $jobProgress | jq -cr '(.result []? | .progress)')
                  echo "Job is still running..Completed $status%"
          fi
  done
  printf "\n The Log Backup Mount job $jobID has $status.\n"

  if [ ! -z "$errorMessage" ]; then
    printf "\n Reason: $errorMessage.\n"
    exit 1
  fi
  if [ -z $LOG_BKP_LOC ]; then
    LOG_BKP_LOC=`lsblk | grep $jobID | awk -F" " '{print $NF}' | head -n 1`
  fi

  if [ ! -z $LOGBACKUPPATH ]; then
    logbackupSuffix="${LOGBACKUPPATH#$LogBackupFS}"
    echo "LogbackupSuffix: $logbackupSuffix"
    LOG_BKP_LOC=$LOG_BKP_LOC/$logbackupSuffix
    echo "Log Backup Updated Path: $LOG_BKP_LOC"
  fi

fi

SYBASEIQRESTORECONFIG=/act/scripts/act_sybaseiq_dump_restore_config.conf
echo -e "OSUSER='$OSUSER'" > $SYBASEIQRESTORECONFIG
if [ "$LOG_RECOVERY" = "YES" ]; then
  echo -e "LOG_BKP_LOC='$LOG_BKP_LOC'" >> $SYBASEIQRESTORECONFIG
  echo -e "RECOVERY_TIME='$UNTIL_TIME'" >> $SYBASEIQRESTORECONFIG
fi
echo -e "DB_BACKUP_LOCATION='$TARGET_MNT_PNT'" >> $SYBASEIQRESTORECONFIG
echo -e "DB_CATALOG_PATH='$DB_CATALOG_PATH'" >> $SYBASEIQRESTORECONFIG
echo -e "TARGET_DBNAME='$TARGET_DBNAME'" >> $SYBASEIQRESTORECONFIG
echo -e "TARGET_DB_USER='$TARGET_DB_USER'" >> $SYBASEIQRESTORECONFIG
echo -e "UTILITYDB_USER='$UTILITYDB_USER'" >> $SYBASEIQRESTORECONFIG
echo -e "TARGET_SYBASEIQ_SQLD='$TARGET_SYBASEIQ_SQLD'" >> $SYBASEIQRESTORECONFIG

OSGROUP=`id -ngr $OSUSER`
chmod -R 777 $TARGET_MNT_PNT
chown -R $OSUSER:$OSGROUP $TARGET_MNT_PNT
if [ ! -z $LOG_BKP_LOC ]; then
  chmod -R 777 $LOG_BKP_LOC
  chown -R $OSUSER:$OSGROUP $LOG_BKP_LOC
fi

cp -r /act/act_scripts/sybaseiq/automation/Restore/act_sybaseiq_restore.sh /act/scripts/
chmod -R 755 /act/scripts/act_sybaseiq_restore.sh

# Execute the Dump Recovery Script.
set +x
source /act/scripts/act_sybaseiq_restore.sh
if [ "$?" -gt 0 ]; then
  echo "ERROR: sybaseiq Dump Recovery Failed!..Exiting.."
  exit 1
fi
set -x
conffile=/act/scripts/act_SybaseIQ_Src.conf
if [ -f $conffile ]; then
 skyIPmatch=`grep -i $SkyIP $conffile`
 retval=$?
 if [ $retval -eq 0 ]
 then
  source $conffile
 fi
 if [ ! -z "$SRC_BACKUP_LOCATION" ]; then
   echo "**** Delete old dump Files in Staging Disk ****"
   FILES=`ls -lr $SRC_BACKUP_LOCATION/*/FULL/*dump* | awk -F" " '{print $NF}'`
   # Delete the files
   for i in `echo $FILES`; do
    rm -f $i
   done
 fi
else
 echo "act_SybaseIQ_Src.conf File does not Exist..!"
fi

set +x
sessionid=$(curl -w "\n" -s -k -XPOST $loginURL | jq -r '.sessionid')
set -x

#Unmount the DB Image
filterValue="filtervalue=jobclass=mount&appname=$GenericAppName&apptype=FileSystem&mountedhost=$hostID"
mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

mountImageIDListLength=${#mountImageIDList[@]}
echo "Existing $GenericAppName mounts for sybaseiq are:"  $mountImageIDListLength

if [ $mountImageIDListLength -gt 0 ]; then
   for (( i=0; i<${mountImageIDListLength}; i++ ))
   do
   # Unmount and Delete Image
   unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

    inputParamsForUnmount='{
    "sessionid":"'"$sessionid"'",
    "image":"'"${mountImageIDList[i]}"'",
    "delete":"true"
   }'

   unmountResponse=$(curl -s -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
   if [[ $unmountResponse == *errormessage* ]]; then
      echo "Unmount Failed. Reason :" $unmountResponse
   elif [[ $unmountResponse == *result* ]]; then
      echo "DB App Unmounted successfully."
   fi
  done
fi

# Unmount the Log Images as Restoration of Log Transactions are complete.
if [ ! -z "$logimagename" ] && [ ! -z "$LogBackupFS" ]; then
	  filterValue="filtervalue=jobclass=mount&appname=$LogBackupFS&apptype=FileSystem&mountedhost=$hostID"
    mountImageIDList=($(curl -sS -w "\n" -k -G  "$lsBackupURL" --data-urlencode "$filterValue"  -d "sessionid=$sessionid" | jq -cr '(.result[] | .id)'))

    mountImageIDListLength=${#mountImageIDList[@]}
    echo "Existing mounts for logBackup are:"  $mountImageIDListLength

    if [ $mountImageIDListLength -gt 0 ]; then
       for (( i=0; i<${mountImageIDListLength}; i++ ))
       do
       # Unmount and Delete Image
       unmountURL="https://$SkyIP/actifio/api/task/unmountimage"

        inputParamsForUnmount='{
        "sessionid":"'"$sessionid"'",
        "image":"'"${mountImageIDList[i]}"'",
        "delete":"true"
       }'

       unmountResponse=$(curl -s -w "\n" -k  -H 'Cache-Control:no-cache'  -H 'Content-Type:application/json' -X POST $unmountURL -d "$inputParamsForUnmount")
       if [[ $unmountResponse == *errormessage* ]]; then
          echo "Unmount Failed. Reason :" $unmountResponse
       elif [[ $unmountResponse == *result* ]]; then
          echo "Logbackup App Unmounted successfully."
       fi
      done
    fi
fi

# Enable the DB schedule back.
chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$dbslaid&scheduleoff=false"
chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)

# Enable the Log Schedule back.
if [ ! -z "$logslaid" ]; then
  chslaURL="https://$SkyIP/actifio/api/task/chsla?sessionid=$sessionid&argument=$logslaid&scheduleoff=false"
  chslaResponse=$(curl -s -k  -H 'Content-Type: application/json' -X POST $chslaURL)
fi
