#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

PWDS=/act/scripts/
source $PWDS/act_SybaseIQ_Src.conf

OSUSER=$OSUSER
SRC_DB_NAME=$SRC_DBNAME
SRC_DB_DBA_USER=$SRC_BACKUP_USER
set +x
SRC_DB_DBA_PWD=$SRC_USER_PASSWD
set -x
DB_LIST=$SRC_DBNAME
DUMPSCHEDULE=$DUMPSCHEDULE
TARGET_MNT=$SRC_BACKUP_LOCATION

DUMPSCHEDULE=${DUMPSCHEDULE:`date +%w`:1}

touch_dir="/act/touch"

cleanupOldDumpFiles()
{
    FILES=$(find "$TARGET_MNT" -iname "*.dump*")
    # Delete old dump files listed under $TARGET_NT
    for i in $(echo "$FILES")
    do
        rm -f "$i"
    done
}
# Delete old dump files listed under $TARGET_MNT
if [ "$DUMPSCHEDULE" = "F" ]
then
    cleanupOldDumpFiles
else
  for DB_NAME in $(echo "$DB_LIST" | tr "," " ")
  do
    CHECK_FULL_BKP=$(ls $TARGET_MNT/$DB_NAME/FULL |wc -l)
    if [ "$CHECK_FULL_BKP" -eq 0 ]; then
        DUMPSCHEDULE=F
        cleanupOldDumpFiles
        break;
    fi
  done
fi

for DB_NAME in $(echo "$DB_LIST" | tr "," " ")
do
    dbisqlpath=$(echo "$SRC_SYBASEIQ_SQLD" | sed "s/,/\n/g" | grep -w "$DB_NAME" | head -n 1 | cut -d ':' -f 2)
    if [ -z "$dbisqlpath" ]; then
        dbisqlpath=$SRC_SYBASEIQ_SQLD
    fi
    #Finding sybase IQ home

    sybiq_home_tmp=$dbisqlpath
    SYBIQ_HOME=$(dirname "$sybiq_home_tmp")
    echo "SYBIQ_HOME = $SYBIQ_HOME"
    version=$(echo "$sybiq_home_tmp" | awk -F"/" '{print $NF}')
    echo "Version= $version"

    set +x
    . "$SYBIQ_HOME/$version/$version".sh
    set -x

    dbuser=$(echo "$SRC_DB_DBA_USER" | sed "s/,/\n/g" | grep -w "$DB_NAME" | cut -d ':' -f 2)
    if [ -z "$dbuser" ]; then
        dbuser=$SRC_DB_DBA_USER
    fi

    #check DB status
    DB_STATUS=$(pgrep -a iqsrv | grep -c -w "$DB_NAME")
    if [ "$DB_STATUS" -eq 0 ];then
        echo "ERRORMSG : Database $DB_NAME is not running"
        exit 1
    else
        echo "continue"
    fi

    #creating database backup directories on $TARGET_MNT
    if [ ! -d "$TARGET_MNT"/"$DB_NAME" ]; then
        mkdir -p "$TARGET_MNT"/"$DB_NAME"/FULL
        mkdir -p "$TARGET_MNT"/"$DB_NAME"/INCR
    fi

    DT=$(date +%Y%m%d_%H_%M_%S)

    #Change ownership of new loaction
    OSGROUP=$(id -gnr "$OSUSER")
    chown -R "$OSUSER":"$OSGROUP" "$TARGET_MNT"
    chmod -R 777 "$TARGET_MNT"

    if [ ! -d $touch_dir ]
    then
        mkdir /act/touch/
    fi
    chmod -R 777 /act/touch
    touch /act/touch/backup.sql
    #create sql file
    createsqlfile(){
        if [ "$BACKUP_FLAG" = "full" ];then
            echo "backup database $BACKUP_FLAG to '$TARGET_MNT/$DB_NAME/FULL/FULL_$DT.dump'; " > /act/touch/backup.sql
            echo "exit;" >> /act/touch/backup.sql
        else
            echo " backup database $BACKUP_FLAG to '$TARGET_MNT/$DB_NAME/INCR/INCR_$DT.dump'; " > /act/touch/backup.sql
            echo "exit;" >> /act/touch/backup.sql
        fi
    }

    chown -R "$OSUSER":"$OSGROUP" /act/touch/backup.sql
    chmod -R 777 /act/touch/backup.sql

    set +x
    dbpasswd=$(echo "$SRC_DB_DBA_PWD" | sed "s/,/\n/g" | grep -w "$DB_NAME" | cut -d ':' -f 2)
    if [ -z "$dbpasswd" ]
    then
        dbpasswd=$SRC_DB_DBA_PWD
    fi


    #Backup
    if [ "$DUMPSCHEDULE" = "F" ];
    then
        BACKUP_FLAG=full
        createsqlfile
        output=`su - "$OSUSER" -c "dbisql -nogui -onerror exit -c 'UID=$dbuser;PWD=$dbpasswd;eng=$DB_NAME;dbn=$DB_NAME;' /act/touch/backup.sql"`
        retval=$?
        if [ $retval -eq 0 ];then
            echo "Database $DB_NAME full backup completed successfully"
        else
            echo "ERRORMSG: Failed to take Full Backup for $DB_NAME because of $output"
            exit $retval
        fi
    elif [ "$DUMPSCHEDULE" = "I" ]; then
        BACKUP_FLAG="incremental since full"
        createsqlfile
        output=`su - "$OSUSER" -c "dbisql -nogui -onerror exit -c 'UID=$dbuser;PWD=$dbpasswd;eng=$DB_NAME;dbn=$DB_NAME;' /act/touch/backup.sql"`
        retval=$?
        if [ $retval -eq 0 ];
        then
            echo "Database $DB_NAME Incremental backup completed successfully"
        else
            echo "ERRORMSG: Failed to take Incremental Backup for $DB_NAME because of $output"
            exit $retval
        fi
    fi
    set -x
done
exit 0
