#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

PWDS=/act/scripts/
source $PWDS/act_SybaseIQ_logbackup.conf

OSUSER=$OSUSER
DB_LIST=$SRC_DBNAME
DBA_USER=$SRC_BACKUP_USER
set +x
DBA_PWD=$SRC_USER_PASSWD
set -x
DUMPSCHEDULE=$DUMPSCHEDULE

if [ -z $LOG_BKP_LOC ]; then
  TARGET_MNT=$LogBackupFS
  TARGET_MNT=$(cat /proc/mounts | grep -w $TARGET_MNT | awk -F' ' '{print $2}')
else
  TARGET_MNT=$LOG_BKP_LOC
fi

OSGROUP=`id -ngr $OSUSER`

for DB_NAME in $(echo $DB_LIST | tr "," " ")
do
    dbisqlpath=$(echo $SRC_SYBASEIQ_SQLD | sed "s/,/\n/g" | grep -w $DB_NAME | head -n 1 | cut -d ':' -f 2)
    if [ -z $dbisqlpath ]; then
        dbisql=$SRC_SYBASEIQ_SQLD
    fi

    #Finding sybase IQ home

    sybiq_home_tmp=$dbisqlpath
    SYBIQ_HOME=`dirname $sybiq_home_tmp`
    echo "SYBIQ_HOME = $SYBIQ_HOME"
    version=`echo "$sybiq_home_tmp" | awk -F"/" '{print $NF}'`
    echo "Version= $version"
    set +x
    . $SYBIQ_HOME/$version/$version.sh
    set -x

    DB_ENG=$DB_NAME
    dbuser=$(echo "$DBA_USER" | sed "s/,/\n/g" | grep -w "$DB_NAME" | cut -d ':' -f 2)
    if [ -z "$dbuser" ]; then
        dbuser=$DBA_USER
    fi

    #check DB status
    DB_STATUS=$(pgrep -a iqsrv | grep -c -w "$DB_NAME")
    if [ "$DB_STATUS" -eq 0 ];then
        echo "ERRORMSG : Database $DB_NAME is not running"
        exit 1
    else
        echo "continue"
    fi

    #PITR logging status
    chmod -R 777 /act/touch
    touch /act/touch/$DB_NAME.pitrstatus.txt
    touch /act/touch/$DB_NAME.pitrlogfile.txt
    chown -R $OSUSER:$OSGROUP /act/touch
    set +x
    dbpasswd=$(echo "$DBA_PWD" | sed "s/,/\n/g" | grep -w "$DB_NAME" | cut -d ':' -f 2)
    if [ -z "$dbpasswd" ]; then
        dbpasswd=$DBA_PWD
    fi

    PITR_STATUS=$(su - $OSUSER -c  "dbisql -nogui -onerror exit  -c \"uid=$dbuser;pwd=$dbpasswd; eng=$DB_NAME; dbn=$DB_NAME;\" \"select db_property('IQPointInTimeRecoveryLogStatus')\" | sed 's/$/ACTIFIO_UNIQUE/'" | grep ACTIFIO_UNIQUE | sed -e "s/ACTIFIO_UNIQUE//g" | sed -n 3p)
    PITR_STATUS=`echo $PITR_STATUS`

    set -x

    if [ "$PITR_STATUS" = "On" ] ; then
        echo "continue"
        echo "$PITR_LOGFILE_TMP"
    else
        echo " PITR logging is not enabled ! Please check logs for more details"
        exit 1
    fi

    TARGET_MNT_TMP=$TARGET_MNT/$DB_NAME

    DT=`date +%Y%m%d_%H%M%S`
    #Change ownership of new loaction
    OSGROUP=`id -gnr $OSUSER`
    mkdir -p "$TARGET_MNT/PITR/$DB_NAME"
    chown -R $OSUSER:$OSGROUP $TARGET_MNT
    chmod -R 777 $TARGET_MNT
    set +x
    output=`su - $OSUSER -c  "dbisql -nogui -onerror exit  -c \"uid=$dbuser;pwd=$dbpasswd; eng=$DB_NAME; dbn=$DB_NAME;\" \"select db_property('IQPointInTimeRecoveryLogFilename');OUTPUT TO \"/act/touch/$DB_NAME.pitrlogfile.txt\" format text;exit;\""`
    retval=$?
    if [ $retval -eq 0 ] ; then
        echo " DBNAME $DB_NAME IQPointInTimeRecoveryLogFilename Query has been executed"
    else
        echo "ERRORMSG:DBNAME $DB_NAME IQPointInTimeRecoveryLogFilename Query Failed due to $output !"
        exit $retval
    fi
    # Remove UTF-8 byte order mark from the file output and redirect clean output to another file.
    set -x
    awk '{if(NR==1)sub(/^\xef\xbb\xbf/,"");print}' /act/touch/$DB_NAME.pitrlogfile.txt > /act/touch/$DB_NAME.pitrlogfile.txt_tmp
    PITR_LOGFILE_TMP=`cat /act/touch/$DB_NAME.pitrlogfile.txt_tmp | sed "s/'//g"`
    cp -f "$PITR_LOGFILE_TMP" "$TARGET_MNT/PITR/$DB_NAME/"

    #Recovery PITR logs backup
    echo "BACKUP DATABASE POINT IN TIME RECOVERY LOGS ONLY TO '$TARGET_MNT/PITR/$DB_NAME/"$DT"_logbackup_$DB_NAME'; " > /act/touch/backup.sql
    echo "exit;" >> /act/touch/backup.sql
    chown -R $OSUSER:$OSGROUP $TARGET_MNT
    chmod -R 777 $TARGET_MNT
    set +x
    output=`su - $OSUSER -c "dbisql -nogui -onerror exit -c 'UID=$dbuser;PWD=$dbpasswd;eng=$DB_NAME;dbn=$DB_NAME;' /act/touch/backup.sql"`
    retval=$?
    if [ $retval -eq 0 ] ; then
        echo " DBNAME $DB_NAME PITR logs backup is success"
    else
        echo "ERRORMSG:DBNAME $DB_NAME Failed to take PITR  logs backup due to $output !"
        exit $retval
    fi
    output=`su - $OSUSER -c  "dbisql -nogui -onerror exit  -c \"uid=$dbuser;pwd=$dbpasswd; eng=$DB_NAME; dbn=$DB_NAME;\" \"select db_property('IQPointInTimeRecoveryLogFilename');OUTPUT TO \"/act/touch/$DB_NAME.pitrlogfile.txt\" format text;exit;\""`
    retval=$?
    if [ $retval -eq 0 ] ; then
        echo " DBNAME $DB_NAME IQPointInTimeRecoveryLogFilename Query has been executed"
    else
        echo "ERRORMSG:DBNAME $DB_NAME IQPointInTimeRecoveryLogFilename Query Failed due to $output !"
        exit $retval
    fi
    # Remove UTF-8 byte order mark from the file output and redirect clean output to another file.
    set -x
    awk '{if(NR==1)sub(/^\xef\xbb\xbf/,"");print}' /act/touch/$DB_NAME.pitrlogfile.txt > /act/touch/$DB_NAME.pitrlogfile.txt_tmp
    PITR_LOGFILE_TMP=`cat /act/touch/$DB_NAME.pitrlogfile.txt_tmp | sed "s/'//g"`
    cp -f "$PITR_LOGFILE_TMP" "$TARGET_MNT/PITR/$DB_NAME/"
done
exit 0
