#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
export LANG=C
export LC_ALL=C

set -x

function usage
{
    echo "Please set environment vars ACT_NAME ACT_FRIENDLYTYPE ACT_PHASE and re-run the script"
    echo "Supported phases [prepare|freeze|unfreeze|abort|predump|dbdump|cleanup|premount|postmount|postunmount|prerestore|postrestore|predumprestore|postdumprestore|premounttest|prerestoretest|predumprestoretest|logbackup|logpurge]"
    return 0;
}

function cleanup_func
{
    echo "**** Cleanup phase started ****"
    su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_abort.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST"
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POST ERROR: Failed db2 cleanup for $DB2_INSTANCE_NAME ************"
        return $retval
    fi     
    echo "**** Cleanup phase ended ****"
    return 0;
}

function prepare_func
{
    echo "**** Prepare phase started ****";
    /act/custom_apps/db2/act_db2_prepare.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PREPARE ERROR: Failed db2 freeze for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Prepare phase ended ****";
    return 0;
}

function freeze_func
{
    echo "**** Freeze phase started ****"

    echo "********** PRE: starting db2 create snapshot for $DB2_INSTANCE_NAME  ************"
    #Create backupstatus.xml to tracke individual db backup status (success or fail)
    if [ ! -d /act/tmpdata/$JOBID ]; then
      mkdir -p /act/tmpdata/$JOBID
    fi
    chown -R $DB2_INSTANCE_NAME /act/tmpdata/$JOBID
    su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_pre.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST"
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 freeze for $DB2_INSTANCE_NAME ************"
        return $retval
    fi

    echo "********** end PRE: db2 freeze for $DB2_INSTANCE_NAME ************"

    return 0;
}

function premount_func
{
    echo "**** Premount phase started ****"
    /act/custom_apps/db2/clone/ACT_DB2_premount.sh 
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed premount for $DB2_INSTANCE_NAME ************"
        return 0
    fi
    echo "**** Premount phase ended ****"
    return 0;
}

function unfreeze_func
{
    echo "**** UN_Freeze phase started ****"

    echo "********** POST: starting db2 unfreeze for create snapshot for $DB2_INSTANCE_NAME  ************"
    su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_post.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST"
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 unfreeze for create snapshot for $DB2_INSTANCE_NAME ************"
        return $retval
    fi

    echo "********** end POST: db2 unfreeze for $DB2_INSTANCE_NAME ************"
    return 0;

}

function premounttest_func
{
    echo "**** Pre-mount test phase started ****"
    /act/custom_apps/db2/clone/ACT_DB2_premounttest.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 premount test for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Premount test phase ended ****"
    return 0;
}


function abort_func
{
    echo "**** abort phase started ****"

    echo "********** PRE: starting db2 abort for create snapshot for $DB2_INSTANCE_NAME  ************"
    su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_abort.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST"
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POST ERROR: Failed db2 abort for create snapshot for $DB2_INSTANCE_NAME ************"
        return $retval
    fi

    echo "********** end POST: db2 abort for snapshot for $DB2_INSTANCE_NAME ************"
    return 0;
}


function predump_func
{
    echo "**** Pre Dump phase started ****"
    /act/custom_apps/db2/dump/ACT_DB2_predump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 predump for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    return 0;
}

function dbdump_func
{
    echo "**** DB Dump phase started ****"
    /act/custom_apps/db2/dump/act_db2_dump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POST ERROR: Failed db2 dump backup for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    return 0;
}

function postmount_func
{
    echo "**** Postmount phase started ****"
    /act/custom_apps/db2/clone/ACT_DB2_postmount.sh 
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POST ERROR: Failed db2 post mount for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Postmount phase ended ****"
    return 0;
}

function postunmount_func
{
    echo "**** Post unmount phase started ****"
    ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$JOBID
    if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
      rm -f $ACT_PREMOUNT_FAILURE_FILE
    fi
    echo "**** Post unmount phase ended ****";

    return 0;
}

function prerestoretest_func
{
    echo "**** Pre-restore test phase started ****"
    echo "**** Pre-restore test phase ended ****";

    return 0;
}

function predumprestoretest_func
{
    echo "**** Pre-dumprestore test phase started ****"
    echo "**** Pre-dumprestore test phase ended ****";

    return 0;
}

function prerestore_func
{
    echo "**** Prerestore phase started ****"
    /act/custom_apps/db2/restore/ACT_DB2_prerestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 rerestore  for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Prerestore phase ended ****"
    return 0;
}

function postrestore_func
{
    echo "**** Postrestore phase started ****"
    /act/custom_apps/db2/restore/ACT_DB2_postrestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 rerestore  for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Postrestore phase ended ****"
    return 0;
}

function predumprestore_func
{
    echo "**** PreDumpRestore phase started ****"
    /act/custom_apps/db2/dump/ACT_DB2_predump_restore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 dump rerestore  for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** PreDumprestore phase ended ****"
    return 0;
}

function postdumprestore_func
{
    echo "**** PostDumpRestore phase started ****"
    /act/custom_apps/db2/dump/ACT_DB2_dump_restore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed db2 dump rerestore  for $DB2_INSTANCE_NAME ************"
        return $retval
    fi
    echo "**** Postrestore phase ended ****"
    return 0;
}

function logbackup_func
{

    echo "**** log backup started ****"

    su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_logbackup.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST"
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POST ERROR: Failed to backup logs for $DB2_INSTANCE_NAME ************"
        return $retval
    fi

    echo "**** log backup ended ****"
    return 0;
}


function logpurge_func
{
     echo "**** log purge started ****"
     su  $DB2_INSTANCE_NAME -c "/act/custom_apps/db2/act_db2_logdelete.sh $DB2_INSTANCE_NAME $DB_DATABASE_LIST $DELETE_LOGFILES_HOURS"
     retval=$?
     if [ $retval -ne 0 ]; then
        echo "******* LOG PURGE: Failed purge logs for $DB2_INSTANCE_NAME ************"
        return $retval
     fi
     echo "**** log purge ended ****"
    return 0;
}

if [ -n $ACT_PHASE ] 
then
   PARAM=$ACT_PHASE

   case $PARAM in 
     cleanup)
         echo "Executing cleanup phase of the script"
         cleanup_func
         retval=$?
         ;;
     prepare)
         echo "Executing prepare phase of the script"
         prepare_func
         retval=$?
         ;;
     freeze)
         echo "Executing freeze phase of the script"
         freeze_func
         retval=$?
         ;;
     unfreeze)
         echo "Executing unfreeze phase of the script"
         unfreeze_func
         retval=$?
         ;;
     predump)
         echo "Executing predump phase of the script"
         predump_func
         retval=$?
         ;;
     dbdump)
         echo "Executing dbdump phase of the script"
         dbdump_func
         retval=$?
         ;;
     prerestoretest)
         echo "Executing prerestore test phase of the script"
         prerestoretest_func
         retval=$?
         ;;
     predumprestoretest)
         echo "Executing predumprestore test phase of the script"
         predumprestoretest_func
         retval=$?
         ;;
     prerestore)
         echo "Executing prerestore phase of the script"
         prerestore_func
         retval=$?
         ;;
     postrestore)
         echo "Executing postrestore phase of the script"
         postrestore_func
         retval=$?
         ;;
     predumprestore)
         echo "Executing prerestore phase of the script"
         predumprestore_func
         retval=$?
         ;;
     postdumprestore)
         echo "Executing postrestore phase of the script"
         postdumprestore_func
         retval=$?
         ;;
     premounttest)
         echo "Executing premount test phase of the script"
         premounttest_func
         retval=$?
         ;;
     premount)
         echo "Executing premount phase of the script"
         premount_func
         retval=$?
         ;;
     postmount)
         echo "Executing postmount phase of the script"
         postmount_func
         retval=$?
         ;;
     postunmount)
         echo "Executing postunmount phase of the script"
         postunmount_func
         retval=$?
         ;;
      logbackup)
         echo "Executing logbackup phase of the script"
         logbackup_func
         retval=$?
         ;;
      logpurge)
         echo "Executing logpurge phase of the script"
         logpurge_func
         retval=$?
         ;;
      abort)
         echo "Executing abort phase of the script"
         abort_func
         retval=$?
         ;;           
     *)
         usage
   esac 
else 
    usage
fi

exit $retval;
