#!/bin/sh
set -x

PWDS=/act/custom_apps/db2

DB2INSTNAME=$1
DATABASE_LIST=$2
OSUSER=$DB2INSTNAME
##Validated env
if [ "$DB2INSTNAME" = "" ]; then
  echo "ERRORMSG: Db2 Instance name (DB2INSTNAME) is not provided."
  exit 1
fi

if [ "$DATABASE_LIST" = "" ]; then
  echo "ERRORMSG: Database list (DATABASE_LIST) is not provided"
  exit 1
fi

 proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
 if [ -z "$proc_id" ];then
     echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
     exit 1
 fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

   if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi


dblist=$(echo $DATABASE_LIST | tr ',' ' ')

for dbname in $dblist
do
    hadr_db_role=`db2 get db cfg for $dbname | awk '/HADR database role/{print toupper($5)}'`
    if [ "$hadr_db_role" != "STANDBY" ]; then

       # Find current active log which needs to be archived
       db2 connect to $dbname
       if [ $? -ne 0 ]; then
          echo "ERRORMSG: Fail to connect to database $dbname"
          exit 1
       fi
       CURRENT_LOG=`db2 -x "select CURRENT_ACTIVE_LOG from table(mon_get_transaction_log(-1))"`
       CURRENT_LOG=`echo $CURRENT_LOG`
       echo "INFOMSG: Current active log is $CURRENT_LOG"
       db2 terminate

       # Archive the current log
       db2 archive log for database $dbname
       retval=$?
       if [ "$retval" -gt 0 ]; then
          echo "ERRORMSG: Failed to generate logs for  $dbname, Please check database logs for details!"
          exit 1
       fi

       # Wait for 30sec to archive the log
       db2 connect to $dbname
       if [ $? -ne 0 ]; then
          echo "ERRORMSG: Fail to connect to database $dbname"
          exit 1
       fi
       NEXT_LOG_TOBE_ARCHIVE=`db2 -x "select METHOD1_NEXT_LOG_TO_ARCHIVE from table(mon_get_transaction_log(-1))"`
       NEXT_LOG_TOBE_ARCHIVE=`echo $NEXT_LOG_TOBE_ARCHIVE`
       maxLoop=6
       currLoop=0
       while [ $currLoop -lt $maxLoop ] && [ $NEXT_LOG_TOBE_ARCHIVE -le $CURRENT_LOG ]
       do
          sleep 5
          currLoop=$(( currLoop + 1 ))
          NEXT_LOG_TOBE_ARCHIVE=`db2 -x "select METHOD1_NEXT_LOG_TO_ARCHIVE from table(mon_get_transaction_log(-1))"`
          NEXT_LOG_TOBE_ARCHIVE=`echo $NEXT_LOG_TOBE_ARCHIVE`
       done
       db2 terminate

    fi
done

exit 0


