#!/bin/sh
set -x

PWDS=/act/custom_apps/db2

DB2INSTNAME=$1
DATABASE_LIST=$2
LOG_RETENTION=$3
OSUSER=$DB2INSTNAME
#DB2INSTNAME=db2inst1
#DATABASE_LIST=SAK1,VIN1

#Validating env
if [ "$DB2INSTNAME" = "" ]; then
  echo "ERRORMSG: Db2 Instance name (DB2INSTNAME) is not provided."
  exit 1
fi

if [ "$DATABASE_LIST" = "" ]; then
  echo "ERRORMSG: Database list (DATABASE_LIST) is not provided"
  exit 1
fi

if [ "$LOG_RETENTION" = "" ]; then
  echo "ERRORMSG: Log retention time (LOG_RETENTION) is not provided"
  exit 1
fi

 proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
 if [ -z "$proc_id" ];then
     echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
     exit 1
 fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

   if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

echo "DATABASE:$DATABASE_LIST"
OLDIFS=IFS
IFS=','

LOG_RETENTION_MINS=`expr $LOG_RETENTION "*" 60`

for dbname in $DATABASE_LIST
do
  NUM_DB_PART=`awk '{print $1}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | wc -l`
  awk '{print $1" "$2}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | while read DB_PART_NUM_NODE
     do
       db_part_nums=`echo $DB_PART_NUM_NODE | awk '{print $1}'`
       db_part_node=`echo $DB_PART_NUM_NODE | awk '{print $2}'`

       export DB2NODE=$db_part_nums
       db2 terminate
       db2 connect to $dbname
       log_arch_meth1=`db2 get db cfg for $dbname | awk '/LOGARCHMETH1/{print ($7)}'`
       log_arch_meth2=`db2 get db cfg for $dbname | awk '/LOGARCHMETH2/{print ($7)}'`
       db2 connect reset

       [[ "$log_arch_meth1" == "OFF" ]] && [[ "$log_arch_meth2" == "OFF" ]] && break
       [[ "$log_arch_meth1" == "NO" ]] && [[ "$log_arch_meth2" == "NO" ]] && break

       log_arch_opt1=`echo $log_arch_meth1 | cut -f1 -d: | awk '{print ($1)}'`
       if [[ "$log_arch_opt1" == "DISK" ]]; then
          log_arch_path1=`echo $log_arch_meth1 | cut -f2- -d:`
          if [ -z "$log_arch_path1" ]; then
             echo "ERRORMSG: Fail to get LOGARCHMETH1 location for db $dbname (partition number: $db_part_nums)"
             exit 1
          fi

          # Find exact directory of LOGARCHMETH1
          LATEST_ARCHIVELOG=`db2 list history archive log all for db $dbname | grep Location | grep $log_arch_path1 | tail -1 | awk '{print $NF}' `
          if [ -z "$LATEST_ARCHIVELOG" ] || [ ! -f "$LATEST_ARCHIVELOG" ]; then
             echo "ERRORMSG: No archive-logs found in location $log_arch_path1 (LOGARCHMETH1) for db $dbname (partition number: $db_part_nums)"
          fi
          log_arch_path1=`dirname $LATEST_ARCHIVELOG`
          if [ ! -d "$log_arch_path1" ]; then
             echo "ERRORMSG: Fail to find archive log directory of db $dbname (partition number: $db_part_nums)"
             exit 1
          fi

          if [[ "$num_db_part" -gt 1 ]]; then
             rsh $db_part_node "find ${log_arch_path1}/../ -name S\*\.LOG -type f -mmin +$LOG_RETENTION_MINS -print -exec rm -f {} \;"
          else
             find ${log_arch_path1}/../ -name S\*\.LOG -type f -mmin +$LOG_RETENTION_MINS -print -exec rm -f {} \;
          fi
       fi
     done
done

exit 0
