#!/bin/sh
set -x

PWDS=/act/custom_apps/db2

DB2INSTNAME=$1
DATABASE_LIST=$2
OSUSER=$DB2INSTNAME
#DB2INSTNAME=db2inst1
#DATABASE_LIST=SAK1,VIN1

if [ "$DB2INSTNAME" = "" ]; then
  echo "ERRORMSG: Db2 Instance name (DB2INSTNAME) is not provided."
  exit 1
fi

if [ "$DATABASE_LIST" = "" ]; then
  echo "ERRORMSG: Database list (DATABASE_LIST) is not provided"
  exit 1
fi

proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
 if [ -z "$proc_id" ];then
     echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
     exit 1
 fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

   if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

echo "DATABASE:$DATABASE_LIST"
OLDIFS=IFS
IFS=','

for dbname in $DATABASE_LIST
do
   hadr_db_role=`db2 get db cfg for $dbname | awk '/HADR database role/{print toupper($5)}'`
   db_part_nums=`awk '{print $1}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -nr`
   if [[ "$hadr_db_role" == "STANDBY" ]];then
      db2 activate db $dbname
      retval=$?
      if [ "$retval" -gt 0 ]; then
         echo "ERRORMSG: Failed to activate database $dbname, Please check database logs for details!"
         exit 1
      fi
   else
        for DB_PART_NUM in $db_part_nums; do
            export DB2NODE=$DB_PART_NUM
            db2 terminate 
            db2 connect to $dbname
            db2 set write resume for database
            retval=$?
            db2 connect reset
            if [ "$retval" -gt 0 ]; then
               echo "ERRORMSG: Failed to resume write operations for $dbname, Please check database logs for details!"
               exit 1
            fi
        done
   fi
done

#Create backupstatus.xml to tracke individual db backup status (success or fail)
BACKUP_STATUS_XML=/act/tmpdata/$JOBID/backupstatus.xml
echo "<tenants>" > $BACKUP_STATUS_XML
for dbname in $DATABASE_LIST
do
  echo "<tenant name=\"$dbname\" status=\"success\" />" >> $BACKUP_STATUS_XML
done
echo "</tenants>" >> $BACKUP_STATUS_XML


exit 0

