#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
JOBNAME=$JOBID
TARGET_INSTANCE=$TARGET_INSTANCE
DB_LIST=$DB_LIST
TARGET_MNT=$TARGET_MNT
ARCHIVELOG_MNT=$ARCHIVELOG_MNT
TARGET_NODENUM=0
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST


#JOBNAME=Job_2417976
#TARGET_INSTANCE=db2tgt
#TARGET_MNT=/rmount
#TARGET_DBNAME_LIST=IND:RIND,ENG:RENG,AUS:RAUS
#TARGET_NODENUM=0


############ Create Config Files for Remount ##############

createconfig(){
  if [ ! -d /act/touch/$JOBNAME ]; then
     mkdir -p /act/touch/$JOBNAME
  fi
  DB_CONFIG=/act/touch/$JOBNAME/REMOUNT_CONFIG
  $ECHO_E "OLD_MOUNT=$TARGET_MNT" > $DB_CONFIG
  $ECHO_E "REMOUNT_DB_LIST=$REMOUNT_CONFIG_LIST" >> $DB_CONFIG
  $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $DB_CONFIG
}


OS_TYPE=`uname -s`
if [ "$OS_TYPE" == "AIX" ]; then
   ECHO_E="echo "
elif [ "$OS_TYPE" == "Linux" ]; then
     ECHO_E="echo -e "
fi

if [ ! -d /act/touch ]; then
   mkdir -p /act/touch
fi


###### Function to Remove all the configuration file

cleanup_files(){
   RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
   RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
   CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params

   remove_file $CONF_FILE
   remove_file $RENAME_DB_CONFIG
   remove_file $RELOCATE_DB_CONFIG
   rm -f /act/touch/*"$JOBNAME"_params
   rm -f /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"*
   rm -f /act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_*
}


###### Function to remove file

remove_file(){
   fname=$1
   if [ -f "$fname" ]; then
      rm -f $fname
   fi
}

######### Function to Cleanup DB after failure ###

cleanup_db(){
   dbname=$1
   instance_name=$2

   echo "*********** Cleaning up Stale database entries after mount failure ***********"
   su $SU_FLAG  $instance_name -c "db2 deactivate database $dbname"
   su $SU_FLAG  $instance_name -c "db2 uncatalog database $dbname"
   echo "*********** Cleanup done ***********"
}

########## MAIN ########

############### Generate New Remount DB LIST ############

REMOUNT_CONF_FILE=$CONFIG_FILE_LOC
REMOUNT_DB_LIST=`cat $REMOUNT_CONF_FILE | grep REMOUNT_DB_LIST | cut -d= -f2` 
OLD_MOUNT=`cat  $REMOUNT_CONF_FILE | grep OLD_MOUNT | cut -d= -f2`
SOURCE_INSTANCE=`cat  $REMOUNT_CONF_FILE | grep INSTANCE | cut -d= -f2`

for dbname in $(echo $REMOUNT_DB_LIST | tr "," " ")
do
   ORIGINAL_DBNAME=`echo $dbname | awk -F":" '{print $1}'`
   PREV_MOUNTED_DBNAME=`echo $dbname | awk -F":" '{print $2}'`
   for remountdb in $(echo $TARGET_DBNAME_LIST | tr "," " ")
   do
     ORIGINAL_REMOUNT_DBNAME=`echo $remountdb | awk -F":" '{print $1}'`
     if [ "$ORIGINAL_DBNAME" = "$ORIGINAL_REMOUNT_DBNAME" ]; then
        NEW_REMOUNT_DBNAME=`echo $remountdb | awk -F":" '{print $2}'`
        break;
     fi
   done
   if [ -z "$NEW_REMOUNT_DBNAME" ]; then
      continue;
   fi
   if [ -z "$NEW_REMOUNT_DBLIST" ]; then
      NEW_REMOUNT_DBLIST="$dbname":"$NEW_REMOUNT_DBNAME"
   else
      NEW_REMOUNT_DBLIST=$NEW_REMOUNT_DBLIST,"$dbname":"$NEW_REMOUNT_DBNAME"
   fi
   if [ -z "$REMOUNT_CONFIG_LIST" ]; then
      REMOUNT_CONFIG_LIST="$ORIGINAL_DBNAME":"$NEW_REMOUNT_DBNAME"
   else
      REMOUNT_CONFIG_LIST=$REMOUNT_CONFIG_LIST,"$ORIGINAL_DBNAME":"$NEW_REMOUNT_DBNAME"
   fi 
done

echo "New Remount DB List: $NEW_REMOUNT_DBLIST"

# Find one of the target dbname (OFTDN) to make unique key with TARGET_INSTANCE and OFTDN
OFTDN=`cat /act/touch/*_"$JOBNAME"_params | grep TARGET_DB_NAME | head -1 | cut -d= -f2 |tr '[a-z]' '[A-Z]'`
echo "OFTDN=$OFTDN" > /act/touch/OFTDN_"$JOBNAME"


########### REMOUNT Use Cases ######

for name in $(echo $NEW_REMOUNT_DBLIST | tr "," " ")
do
     ORIG_DBNAME=`echo $name | awk -F":" '{print $1}'`
     dbname=`echo $name |awk -F":" '{print $2}'`
  
     CONF_FILE=/act/touch/"$ORIG_DBNAME"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!" 
        exit 1
     fi

     SOURCE_DB_NAME=$dbname
     TARGET_DB_NAME=`cat $CONF_FILE | grep TARGET_DB_NAME | cut -d= -f2`
     TARGET_DB_NAME=`echo $TARGET_DB_NAME | tr '[a-z]' '[A-Z]'`
     SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
     SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
     SOURCE_STORAGE_PATH=`cat $CONF_FILE | grep SOURCE_STORAGE_PATH | cut -d= -f2`
     SOURCE_TBSP_PATH=`cat $CONF_FILE | grep TBSP_PATH | cut -d= -f2`
     SOURCE_LOG_DIR=`cat $CONF_FILE | grep SOURCE_LOG_DIR | cut -d= -f2`
     ORIG_INSTANCE=`cat $CONF_FILE | grep SOURCE_INSTANCE | cut -d= -f2`

     TARGET_LOG_DIR=$TARGET_MNT$SOURCE_LOG_DIR
     TARGET_DB_PATH=$TARGET_MNT$SOURCE_DB_PATH
     
     #Change ownership of new loaction
     OSUSER=$TARGET_INSTANCE
     OSGROUP=`id -ngr $OSUSER`
     chown -R $OSUSER:$OSGROUP $TARGET_MNT

     #identify db2_home
     proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
     if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
     fi
     db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
     INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
     DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`
  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

     #### Case 1: DBNAME, TARGET MOUNT and INSTANCE ARE SAME
     if [ "$TARGET_MNT" = "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" = "$TARGET_INSTANCE" ] && [ "$dbname" = "$TARGET_DB_NAME" ]; then
        echo "Case 1: DBNAME, TARGET MOUNT and INSTANCE ARE SAME"
        if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ;then
            su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
        else
            su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
        fi
        su  $OSUSER -c "db2 activate database $SOURCE_DB_NAME"
        retval=$?
        if [ "$retval" -gt 0 ]; then
           echo "ERRORMSG: Failed to Activate the database $dbname"
           cleanup_db $SOURCE_DB_NAME $OSUSER
           exit 1
        else
            createconfig
        fi
     fi

#########CASE 2 :SAME MOUNT,SAME DB NAME and DIFFERENT INSTANCE
     if [ "$TARGET_MNT" = "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ] && [ "$dbname" = "$TARGET_DB_NAME" ];then
        echo "CASE 2 :SAME MOUNT,SAME DB NAME and DIFFERENT INSTANCE"
        RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg

        $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
        if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
            $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
        else
            $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
        fi
	$ECHO_E "INSTANCE=$SOURCE_INSTANCE,$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
	$ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG
        ### Rename instance
        find_path=`find "\$TARGET_MNT" -name "\$SOURCE_INSTANCE" -type d | xargs | tr " " ","`
        for path in $(echo $find_path | tr "," " ")
          do
          name=`dirname $path`
          echo $name
          mv $path  $name/$TARGET_INSTANCE
          done
           chown -R $OSUSER:$OSGROUP $TARGET_MNT
           chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
          su  $OSUSER -c "db2relocatedb -f $RELOCATE_DB_CONFIG"
          if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Failed to relocate  Instance  $SOURCE_INSTANCE "
             cleanup_db $SOURCE_DB_NAME $OSUSER
             exit 1
          else
              su $OSUSER -c "db2 activate database $TARGET_DB_NAME"
              createconfig
          fi
     fi

##### CASE 3:SAME INSTANCE,SAME TARGET MOUNT and DIFFERENT DB NAME
        if [ "$TARGET_MNT" = "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" = "$TARGET_INSTANCE" ] && [ "$dbname" != "$TARGET_DB_NAME" ];then
           echo "CASE 3:SAME INSTANCE,SAME TARGET MOUNT and DIFFERENT DB NAME"
           RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
           if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
               su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
           else
               su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
           fi
           su  $OSUSER -c "db2 activate database $SOURCE_DB_NAME"
           retval=$?
           if [ "$retval" -gt 0 ]; then
           echo "ERRORMSG: Failed to Activate  database $dbname"
           cleanup_db $SOURCE_DB_NAME $OSUSER
           exit 1
           fi
           $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
           if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
               $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RENAME_DB_CONFIG
           else
               $ECHO_E "DB_PATH=$TARGET_DB_PATH"  >> $RENAME_DB_CONFIG
           fi
           $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
           $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
           chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG
           for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
           do
            NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
            su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
           done
           su  $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
           if [ $? -ne 0 ] ; then
              echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
              for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
              do
               NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
               su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
              done
              cleanup_db $SOURCE_DB_NAME $OSUSER
              cleanup_db $TARGET_DB_NAME $OSUSER
              exit 1
           else
             su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
             createconfig
           fi
        fi

##### CASE 4:DIFFERENT INSTANCE,DIFFERENT DBNAME and SAME TARGET MOUNT
        if [ "$TARGET_MNT" = "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ] && [ "$dbname" != "$TARGET_DB_NAME" ];then
        
        	echo "CASE 4:DIFFERENT INSTANCE,DIFFERENT DBNAME and SAME TARGET MOUNT"
                RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
                RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
                $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
                if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                    $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
                else
                    $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
                fi
                $ECHO_E "INSTANCE=$SOURCE_INSTANCE,$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
                $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG
                ### Rename instance
                find_path=`find "\$TARGET_MNT" -name "\$SOURCE_INSTANCE" -type d | xargs | tr " " ","`
                for path in $(echo $find_path | tr "," " ")
                do
                name=`dirname $path`
                echo $name
                mv $path  $name/$TARGET_INSTANCE
                done
                chown -R $OSUSER:$OSGROUP $TARGET_MNT
                chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
                su  $OSUSER -c "db2relocatedb -f $RELOCATE_DB_CONFIG"
                if [ $? -ne 0 ] ; then
                echo "ERRORMSG: Failed to relocate  Instance  $TARGET_INSTANCE"
                cleanup_db $SOURCE_DB_NAME $OSUSER
                cleanup_db $TARGET_DB_NAME $OSUSER
                exit 1
                fi
                ###Renaming DB NAME
                $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
                if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                    $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RENAME_DB_CONFIG
                else
                    $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RENAME_DB_CONFIG
                fi
                $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
                $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
                chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG
                for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
                do
                   NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
                   su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
                done
                su  $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
                if [ $? -ne 0 ] ; then
                   echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
                   for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
                   do
                     NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
                     su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
                   done
                   cleanup_db $SOURCE_DB_NAME $OSUSER
                   cleanup_db $TARGET_DB_NAME $OSUSER
                   exit 1
                else
                     su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
                     createconfig
                fi
 
        fi        
##### CASE 5:SAME INSTANCE,SAME DBNAME and DIFFERENT TARGET MOUNT
            if [ "$TARGET_MNT" != "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" = "$TARGET_INSTANCE" ] && [ "$dbname" = "$TARGET_DB_NAME" ];then
               echo "CASE 5:SAME INSTANCE,SAME DBNAME and DIFFERENT TARGET MOUNT"
                RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
                $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
                if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                    $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH/"$ORIG_DBNAME"_DBP,$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
                else
                    $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
                fi
                $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
                $ECHO_E "LOG_DIR=$OLD_MOUNT$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
                $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG

                for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
                do
                   target_pathname=$TARGET_MNT$pathname
                   $ECHO_E "STORAGE_PATH=$OLD_MOUNT$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
                done
                if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
                for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
                do
                 target_tbspath=$TARGET_MNT$tbspath
                 $ECHO_E "CONT_PATH=$OLD_MOUNT$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
                done
                fi
                   unlink $TARGET_DB_PATH/$ORIG_INSTANCE
                   ln -s $TARGET_DB_PATH/$TARGET_INSTANCE $TARGET_DB_PATH/$ORIG_INSTANCE
                   chown $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG 
                   
                   if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                       su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
                   else
                       su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
                   fi
                   su  $OSUSER -c "db2relocatedb -f  $RELOCATE_DB_CONFIG"
                   if [ $? -ne 0 ] ; then
                        echo "ERRORMSG: Relocation Failed "
                        cleanup_db $SOURCE_DB_NAME $OSUSER
                        cleanup_db $TARGET_DB_NAME $OSUSER
                         exit 1
                   else
                          su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
                           createconfig
                   fi                   
            fi

### CASE 6:DIFFERENT MOUNT,DIFFERENT DB NAME and SAME INSTANCE
        if [ "$TARGET_MNT" != "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" = "$TARGET_INSTANCE" ] && [ "$dbname" != "$TARGET_DB_NAME" ];then
            echo "CASE 6 :DIFFERENT MOUNT,DIFFERENT DB NAME and SAME INSTANCE"     
            RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
            RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg

            $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
            if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH/"$ORIG_DBNAME"_DBP,$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
            else
                $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
            fi
            $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
	    $ECHO_E "LOG_DIR=$OLD_MOUNT$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
            $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG

            for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
            do
            target_pathname=$TARGET_MNT$pathname
            $ECHO_E "STORAGE_PATH=$OLD_MOUNT$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
            done
             if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
                for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
                do
                 target_tbspath=$TARGET_MNT$tbspath
                 $ECHO_E "CONT_PATH=$OLD_MOUNT$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
                done
              fi
            unlink $TARGET_DB_PATH/$ORIG_INSTANCE
            ln -s $TARGET_DB_PATH/$TARGET_INSTANCE $TARGET_DB_PATH/$ORIG_INSTANCE
            chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
            if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
               su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
            else
               su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
            fi
            cat $RELOCATE_DB_CONFIG
            su  $OSUSER -c "db2relocatedb -f  $RELOCATE_DB_CONFIG"
            if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Relocation is Failed ! check log for more details"
             cleanup_db $SOURCE_DB_NAME $OSUSER
             cleanup_db $TARGET_DB_NAME $OSUSER
             exit 1
            fi
   ###Renaming DB NAME
            $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
            if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
                $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RENAME_DB_CONFIG
            else
                $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RENAME_DB_CONFIG
            fi
            $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
            $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
            chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG
            for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
             do
            NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
            su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
            done
            su  $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
            if [ $? -ne 0 ] ; then
                   echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
                   for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
                   do
                   NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
                   su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
                   done
                   cleanup_db $SOURCE_DB_NAME $OSUSER
                   cleanup_db $TARGET_DB_NAME $OSUSER
                   exit 1
            else
                  su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
                   createconfig
            fi  

        fi
   

#####case 7: DIFFERENT MOUNT,DIFFERENT INSTANCE and SAME DBNAME
     if [ "$TARGET_MNT" != "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ] && [ "$dbname" = "$TARGET_DB_NAME" ];then
        echo "case 7: DIFFERENT MOUNT,DIFFERENT INSTANCE and SAME DBNAME"
        RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
        RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg

        $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
        if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
          $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH/"$ORIG_DBNAME"_DBP,$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
        else
          $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
        fi
        $ECHO_E "INSTANCE=$SOURCE_INSTANCE,$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
        $ECHO_E "LOG_DIR=$OLD_MOUNT$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
        $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG

        for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
        do
          target_pathname=$TARGET_MNT$pathname
          $ECHO_E "STORAGE_PATH=$OLD_MOUNT$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
        done
        if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
                for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
                do
                 target_tbspath=$TARGET_MNT$tbspath
                 $ECHO_E "CONT_PATH=$OLD_MOUNT$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
                done
        fi
        ### Rename instance
        find_path=`find "\$TARGET_MNT" -name "\$SOURCE_INSTANCE" -type d | xargs | tr " " ","`
        for path in $(echo $find_path | tr "," " ")
        do
          name=`dirname $path`
          echo $name
          mv $path  $name/$TARGET_INSTANCE
        done
        chown -R $OSUSER:$OSGROUP $TARGET_MNT
        chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
        unlink $TARGET_DB_PATH/$ORIG_INSTANCE
        ln -s $TARGET_DB_PATH/$TARGET_INSTANCE $TARGET_DB_PATH/$ORIG_INSTANCE
        if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
           su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
        else
           su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
        fi
        su  $OSUSER -c "db2relocatedb -f  $RELOCATE_DB_CONFIG"
        if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Relocation is Failed ! check log for more details"
             cleanup_db $SOURCE_DB_NAME $OSUSER
             exit 1
        else
            su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
            createconfig
        fi
   fi


##### CASE 8: DBNAME, TARGET MOUNT and INSTANCE ARE NOT SAME

     if [ "$TARGET_MNT" != "$OLD_MOUNT" ] && [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ] && [ "$dbname" != "$TARGET_DB_NAME" ];then
        echo "CASE 8: DBNAME, TARGET MOUNT and INSTANCE ARE NOT SAME"
        RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
        RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
       
        $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
        if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
            $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH/"$ORIG_DBNAME"_DBP,$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RELOCATE_DB_CONFIG
        else
            $ECHO_E "DB_PATH=$OLD_MOUNT$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
        fi
        $ECHO_E "INSTANCE=$SOURCE_INSTANCE,$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
        $ECHO_E "LOG_DIR=$OLD_MOUNT$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
        $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG

        for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
        do
          target_pathname=$TARGET_MNT$pathname
          $ECHO_E "STORAGE_PATH=$OLD_MOUNT$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
        done
        if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
                for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
                do
                 target_tbspath=$TARGET_MNT$tbspath
                 $ECHO_E "CONT_PATH=$OLD_MOUNT$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
                done
        fi
         ### Rename instance
         find_path=`find "\$TARGET_MNT" -name "\$SOURCE_INSTANCE" -type d | xargs | tr " " ","`
         for path in $(echo $find_path | tr "," " ")
         do
          name=`dirname $path`
          echo $name
          mv $path  $name/$TARGET_INSTANCE
         done
         chown -R $OSUSER:$OSGROUP $TARGET_MNT
         chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
         unlink $TARGET_DB_PATH/$ORIG_INSTANCE
         ln -s $TARGET_DB_PATH/$TARGET_INSTANCE $TARGET_DB_PATH/$ORIG_INSTANCE
         if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
             su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH/'$ORIG_DBNAME'_DBP"
         else
             su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
         fi
         su  $OSUSER -c "db2relocatedb -f  $RELOCATE_DB_CONFIG"
          if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Instance $TARGET_INSTANCE Relocation is Failed ! check log for more details"
             cleanup_db $SOURCE_DB_NAME $OSUSER
             cleanup_db $TARGET_DB_NAME $OSUSER
             exit 1
          fi
  ###Renaming DB NAME
         $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
         if [[ -d "$TARGET_DB_PATH/$ORIG_DBNAME"_DBP ]] ; then
            $ECHO_E "DB_PATH=$TARGET_DB_PATH/"$ORIG_DBNAME"_DBP" >> $RENAME_DB_CONFIG
         else
             $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RENAME_DB_CONFIG
         fi
         $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
         $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
         chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG
          for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
          do
            NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
            su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
          done
          su  $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
          if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
             for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
             do      
               NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
               su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
             done
             cleanup_db $SOURCE_DB_NAME $OSUSER
             cleanup_db $TARGET_DB_NAME $OSUSER
             exit 1
          else
               su  $OSUSER -c "db2 activate database $TARGET_DB_NAME"
               createconfig
          fi

    fi

done  ### End of First for loop for same instance name
cleanup_files
exit 0
