#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done


usage() {
  echo "Usage: "
  echo "/act/custom_apps/db2/clone/ACT_DB2_mountrecover.sh TARGET_MNT=<TARGET_MNT>  TARGET_INSTANCE=<TARGET_INSTANCE>  TARGET_DBNAME_LIST=<TARGET_DBNAME_LIST> [UNTIL_TIME=<UNTIL_TIME>]  JOBID=<JOBID> "
}

OSUSER=$TARGET_INSTANCE
#identify db2_home
proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
if [ -z "$proc_id" ];then
   echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
   exit 1
fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

 if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

for dbn in $(echo $TARGET_DBNAME_LIST | tr "," " ")
do
    TARGET_DB=`echo $dbn | awk -F":" '{print $1}'`
    TARGET_DB=`echo $TARGET_DB | tr '[a-z]' '[A-Z]'`
    if [ "x$DB_LIST" = "x" ];then
        DB_LIST=$TARGET_DB
    else
        DB_LIST="$DB_LIST,$TARGET_DB"
    fi
done
APPREMOUNT=FALSE
export APPREMOUNT
export DB_LIST


isProvided() {
  local varName=$1
  local varVal=$2
  if [ -z "$varVal" ]; then
    echo "ERRORMSG: $varName is empty."
    usage
    exit 1
  fi
}

isProvided TARGET_MNT $TARGET_MNT
isProvided TARGET_INSTANCE $TARGET_INSTANCE
isProvided TARGET_DBNAME_LIST $TARGET_DBNAME_LIST
isProvided JOBID $JOBID

if [ ! -z "$UNTIL_TIME" ]; then
  isProvided UNTIL_TIME $UNTIL_TIME
fi


JOBNAME=$JOBID
export JOBNAME
export JOBID

for dbs in $(echo $TARGET_DBNAME_LIST| tr "," " ")
   do
     dbs=`echo $dbs | tr '[a-z]' '[A-Z]'`
     srcdb=`echo $dbs | awk -F":" '{print $1}'`
     tgtdb=`echo $dbs | awk -F":" '{print $2}'`
     CONF_FILE=/act/touch/"$srcdb"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
        exit 1
     else
     SOURCE_VERSION=`cat $CONF_FILE |grep SOURCE_VERSION | cut -d= -f2`
     SOURCE_INSTANCE=`cat $CONF_FILE | grep SOURCE_INSTANCE | cut -d= -f2`
     fi
     sed -i "s/TARGET_DB_NAME=.*/TARGET_DB_NAME=$tgtdb/g" $CONF_FILE
    srcdbs=`su  $OSUSER  -c "db2 list db directory" | grep 'Database name'|sort|uniq | sed -e 's/=//g' | awk '{print $3}'`
    echo "$srcdbs" |grep -w $srcdb
    if [ $? -eq 0 ] ; then
        su $OSUSER -c "db2 terminate;db2 force applications all;db2 deactivate database $srcdb;db2 drop database $srcdb"
    fi
   done

export SOURCE_VERSION
export SOURCE_INSTANCE
KEEPCONFIGFILE=TRUE
export KEEPCONFIGFILE

set -x
#Premount-test
/act/custom_apps/db2/clone/ACT_DB2_premounttest.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: premount_test failed"
  exit 1
fi

#Postmount
/act/custom_apps/db2/clone/ACT_DB2_postmount.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Postmount failed"
  exit 1
else
  echo "INFOMSG: Mount recovery is successful"
fi
exit 0
