#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being

# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
#set
set -x
JOBNAME=$JOBID
SOURCE_INSTANCE=$SOURCE_INSTANCE
TARGET_INSTANCE=$TARGET_INSTANCE
DB_LIST=$DB_LIST
TARGET_MNT=$TARGET_MNT
ARCHIVELOG_MNT=$ARCHIVELOG_MNT
RECOVERYTIME="$UNTIL_TIME"
TARGET_NODENUM=0
APPREMOUNT=$APPREMOUNT

rollforwardUsingUTC_Output=/act/touch/rollforwardUsingUTC_$JOBNAME

OS_TYPE=`uname -s`
if [ "$OS_TYPE" == "AIX" ]; then
   ECHO_E="echo "
   SU_FLAG=""
elif [ "$OS_TYPE" == "Linux" ]; then
     ECHO_E="echo -e "
     SU_FLAG=""
fi

if [ ! -d /act/touch ]; then
   mkdir -p /act/touch
   chmod 755 /act/touch
else
  chmod 755 /act/touch
fi

if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
   CHILD_MNT=$TARGET_MNT
   TARGET_MNT=
fi

if [ "$APPREMOUNT" = TRUE ]; then
   /act/custom_apps/db2/clone/ACT_DB2_Remount.sh
   if [ "$?" -gt 0 ]; then
      echo "ERRORMSG:Failed to remount databases, Please check the logs"
      exit 1
   else
      echo "Remount completed successfully!."
      exit 0
   fi
fi

####### Function to create Configuration files for Remount

createconfig(){
  DB_CONFIG=/act/touch/$JOBNAME/REMOUNT_CONFIG
  if [ ! -d /act/touch/$JOBNAME ]; then
     mkdir -p /act/touch/$JOBNAME
  fi
  for dbname in $(echo $DB_LIST| tr "," " ")
  do
    CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
    source_db=$dbname
    target_db=`cat $CONF_FILE | grep TARGET_DB_NAME | cut -d= -f2`    
    if [ -z "$remount_db_list" ]; then
       remount_db_list="$source_db":"$target_db"
    else
       remount_db_list="$remount_db_list","$source_db":"$target_db" 
    fi
  done
  
  $ECHO_E "REMOUNT_DB_LIST=$remount_db_list" > $DB_CONFIG
  $ECHO_E "OLD_MOUNT=$TARGET_MNT" >> $DB_CONFIG
  $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $DB_CONFIG
}

###### Function to Remove all the configuration file

cleanup_files(){
   RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
   RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
   CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
   
   remove_file $CONF_FILE
   remove_file $RENAME_DB_CONFIG
   remove_file $RELOCATE_DB_CONFIG
   rm -f /act/touch/*"$JOBNAME"_params
   rm -f /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_*
   rm -f /act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_*
   rm -f /act/touch/dbpathsortlist_"$JOBNAME"_"$TARGET_INSTANCE"_"$SOURCE_DB_NAME"
   
} 


###### Function to remove file

remove_file(){
   fname=$1
   if [ -f "$fname" ]; then
      rm -f $fname
   fi
}

######### Function to cleanup incase of failure ###########

cleanup_db(){
   instance_name=$1
   for dbname in $(echo $DB_LIST| tr "," " ")
   do
    CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
    source_db=$dbname
    target_db=`cat $CONF_FILE | grep TARGET_DB_NAME | cut -d= -f2`
   echo "*********** Cleaning up Stale database entries after mount failure ***********"
   su $SU_FLAG  $instance_name -c "db2 deactivate database $source_db"
   su $SU_FLAG  $instance_name -c "db2 uncatalog database $source_db"
   su $SU_FLAG  $instance_name -c "db2 deactivate database $target_db"
   su $SU_FLAG  $instance_name -c "db2 uncatalog database $target_db"
   echo "*********** Cleanup done ***********"
   done
}

   OSUSER=$TARGET_INSTANCE
   #identify db2_home
   proc_id=`ps aux | grep -w $TARGET_INSTANCE |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

  db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
  INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
  DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`
  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

# Find one of the target dbname (OFTDN) to make unique key with TARGET_INSTANCE and OFTDN
OFTDN=`cat /act/touch/*_"$JOBNAME"_params | grep TARGET_DB_NAME | head -1 | cut -d= -f2 |tr '[a-z]' '[A-Z]'`
echo "OFTDN=$OFTDN" > /act/touch/OFTDN_"$JOBNAME"

   DBPATHSORT="/act/touch/dbpathsortlist"_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME"
#removing temporary file
   DBPATHSORT="/act/touch/dbpathsortlist"_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME"
   if [ -f "$DBPATHSORT" ] ; then
       rm -f $DBPATHSORT
   fi
   
   DBCOUNT=0
   for dbName in $(echo $DB_LIST| tr "," " ")
   do
     CONF_FILE=/act/touch/"$dbName"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
        exit 1
     fi
     SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
     if [ ! -f "$DBPATHSORT" ] ; then
          touch $DBPATHSORT
     fi  
     echo "$SOURCE_DB_PATH" >> $DBPATHSORT
     DBCOUNT=`expr $DBCOUNT + 1`
   done
   
   DBP_COUNT=`cat $DBPATHSORT |sort | uniq | wc -l`
   echo "***SOURCE DBAPATH INFORMATION***"
   cat $DBPATHSORT
if [[ "$DBCOUNT" -gt 1 ]] && [[ "$DBP_COUNT" -ne "$DBCOUNT" ]] ;then
   for dbName in $(echo $DB_LIST| tr "," " ")
   do
     CONF_FILE=/act/touch/"$dbName"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
        exit 1
     fi

     SOURCE_DB_NAME=$dbName
     TARGET_DB_PATH=$TARGET_MNT/"$SOURCE_DB_NAME"_DBP

     SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
     OSUSER=$TARGET_INSTANCE
     OSGROUP=`id -ngr $OSUSER`
     #creating directories for new dbpath
     mkdir -p $TARGET_MNT$SOURCE_DB_PATH/"$SOURCE_DB_NAME"_DBP
     SOURCE_LOG_DIR=`cat $CONF_FILE | grep SOURCE_LOG_DIR | cut -d= -f2`
     SOURCE_LOG_DIR_ORIG=$SOURCE_LOG_DIR
     echo $SOURCE_LOG_DIR_ORIG | grep -i SQL00*
     if [ $? -ne 0 ];then
         echo "continue"
     else
        LOGDIR=`echo $SOURCE_LOG_DIR_ORIG | awk -F"/" '{print $2}'`
        mkdir -p $TARGET_MNT/$LOGDIR/"$SOURCE_DB_NAME"_ACL/NODE0000/LOGSTREAM0000
        cp  $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG $TARGET_MNT/$LOGDIR/"$SOURCE_DB_NAME"_ACL/NODE0000/LOGSTREAM0000
     fi
     chown -R $OSUSER:$OSGROUP $TARGET_MNT
     PATH_TEMP="$TARGET_MNT$SOURCE_DB_PATH"
     #Renaming Instance name
     if [ -d "$PATH_TEMP/$TARGET_INSTANCE" ] ;then
        echo "continue"
     else
        mv $PATH_TEMP/$SOURCE_INSTANCE $PATH_TEMP/$TARGET_INSTANCE
     fi
     chown -R $OSUSER:$OSGROUP $TARGET_MNT
     dbDir=`su $SU_FLAG $OSUSER -c "db2 list db directory on $PATH_TEMP"| grep -v "Database alias"|grep -v "Comment" | awk "/$SOURCE_DB_NAME/{getline;print}" |xargs | awk '{print $NF}'`
     sqldir=`find $PATH_TEMP -name "$dbDir" -type d |grep $TARGET_INSTANCE |xargs`
     dirtmp=`dirname $sqldir`
     NODENUM=`basename $dirtmp`
     mkdir -p $TARGET_MNT$SOURCE_DB_PATH/"$SOURCE_DB_NAME"_DBP/$TARGET_INSTANCE/$NODENUM
     chown -R $OSUSER:$OSGROUP $TARGET_MNT$SOURCE_DB_PATH/"$SOURCE_DB_NAME"_DBP/$TARGET_INSTANCE/$NODENUM
     cp -r $sqldir $TARGET_MNT$SOURCE_DB_PATH/"$SOURCE_DB_NAME"_DBP/$TARGET_INSTANCE/$NODENUM/
     cp -r $dirtmp/sqldbdir $TARGET_MNT$SOURCE_DB_PATH/"$SOURCE_DB_NAME"_DBP/$TARGET_INSTANCE/$NODENUM/
   done
fi
########### MAIN ######
if [[ "$SOURCE_INSTANCE" == "$TARGET_INSTANCE" ]];then
   for dbname in $(echo $DB_LIST| tr "," " ")
   do
     CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!" 
        exit 1
     fi

     SOURCE_DB_NAME=$dbname
     TARGET_DB_NAME=`cat $CONF_FILE | grep TARGET_DB_NAME | cut -d= -f2`
     TARGET_DB_NAME=`echo $TARGET_DB_NAME | tr '[a-z]' '[A-Z]'`
     SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
     SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
     SOURCE_STORAGE_PATH=`cat $CONF_FILE | grep SOURCE_STORAGE_PATH | cut -d= -f2`
     SOURCE_TBSP_PATH=`cat $CONF_FILE | grep TBSP_PATH | cut -d= -f2`
     MIRRORLOG_PATH=`cat $CONF_FILE | grep MIRRORLOGPATH |cut -d= -f2`
     SOURCE_LOG_DIR=`cat $CONF_FILE | grep SOURCE_LOG_DIR | cut -d= -f2`
     SOURCE_LOG_DIR_ORIG=$SOURCE_LOG_DIR
     SOURCE_LOG_DIR=`echo $SOURCE_LOG_DIR |awk -F"/NODE" '{print $1}'`   
     TARGET_DB_PATH_base=$TARGET_MNT$SOURCE_DB_PATH

     # Prepare TARGET_DB_PATH
     TARGET_DB_PATH=$TARGET_DB_PATH_base
     if [[ "$DBCOUNT" -gt 1 ]] && [[ "$DBP_COUNT" -ne "$DBCOUNT" ]] ; then
        TARGET_DB_PATH=$TARGET_DB_PATH_base/"$SOURCE_DB_NAME"_DBP
     fi

     # Prepare TARGET_LOG_DIR
     TARGET_LOG_DIR=$TARGET_DB_PATH_base
     echo $SOURCE_LOG_DIR_ORIG | grep -i SQL00*
     if [ $? -eq 0 ];then
        TARGET_LOG_DIR=$TARGET_DB_PATH_base/"$SOURCE_DB_NAME"_ACL
        mkdir -p $TARGET_LOG_DIR
     fi
          
     if [ -z "$ARCHIVELOG_MNT" ] ; then
         ARCHIVELOG_MNT=`cat $CONF_FILE | grep ARCHIVELOGMOUNTPATH |cut -d= -f2`
     fi
     RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
     RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg 
     $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
     $ECHO_E "DB_PATH=$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
     if [ ! -z "$MIRRORLOG_PATH" ] ;then
          $ECHO_E "MIRRORLOG_PATH=$MIRRORLOG_PATH,$TARGET_MNT$MIRRORLOG_PATH" >> $RELOCATE_DB_CONFIG
     fi
     $ECHO_E "INSTANCE=$SOURCE_INSTANCE" >> $RELOCATE_DB_CONFIG
     LOG_BASE=`basename $TARGET_LOG_DIR`
     LOG_DIR=`dirname $TARGET_LOG_DIR`
     if [ "$LOG_BASE" = "$SOURCE_INSTANCE" ] ; then
            #mv $TARGET_MNT$SOURCE_LOGDIR $LOG_DIR/$TARGET_INSTANCE
            TARGET_LOG_DIR=$LOG_DIR/$TARGET_INSTANCE
            $ECHO_E "LOG_DIR=$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
     else
     $ECHO_E "LOG_DIR=$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
     fi
     $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG

     
     if [ ! -z "$SOURCE_STORAGE_PATH" ] ; then
          for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
          do
            target_pathname=$TARGET_MNT$pathname
            $ECHO_E "STORAGE_PATH=$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
          done
     fi
       if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
           for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
           do
            target_tbspath=$TARGET_MNT$tbspath
            $ECHO_E "CONT_PATH=$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
            done
       fi

     mv $RELOCATE_DB_CONFIG "$RELOCATE_DB_CONFIG".tmp
     cat "$RELOCATE_DB_CONFIG".tmp |sort -n|uniq > $RELOCATE_DB_CONFIG
     rm -f "$RELOCATE_DB_CONFIG".tmp

     #Change ownership of new loaction
     OSUSER=$SOURCE_INSTANCE
     OSGROUP=`id -ngr $OSUSER`
     chown -R $OSUSER:$OSGROUP $TARGET_MNT
     chown -R $OSUSER:$OSGROUP $ARCHIVELOG_MNT
     chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
     chown -R $OSUSER:$OSGROUP $TARGET_DB_PATH
     chmod 755 $RELOCATE_DB_CONFIG
     chmod -R 755 $TARGET_MNT
     if [ ! -z "$CHILD_MNT" ] ; then 
        chown -R $OSUSER:$OSGROUP $CHILD_MNT
     fi
     #Catalog database
     su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH" 
     if [ $? -ne 0 ]; then
        echo "ERRORMSG: Failed to catalog database $SOURCE_DB_NAME"
        cleanup_db $OSUSER
        exit 1
     fi

     #Reloacte database
     su  $OSUSER -c "db2inidb $SOURCE_DB_NAME as standby relocate using $RELOCATE_DB_CONFIG"
     if [ $? -ne 0 ] ; then
        sleep 5
        su  $OSUSER -c "db2inidb $SOURCE_DB_NAME as standby"
        retval=$?
        if [ "$retval" -gt 0 ]; then
           echo "ERRORMSG: Failed to relocate database $SOURCE_DB_NAME"
           cleanup_db  $OSUSER
           exit 1
        fi
     fi

     #Rollforwad db recovery
     if [ ! -z "$RECOVERYTIME" ]; then
        LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
        su  $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "

        rollfwd_status=`su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
        su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME to $RECOVERYTIME using UTC time" > $rollforwardUsingUTC_Output.$SOURCE_DB_NAME
        rollforwardUsingUTC=`grep 'SQL[0-9][0-9][0-9][0-9][A-Z]' $rollforwardUsingUTC_Output.$SOURCE_DB_NAME | grep -v "SQL4970N  Rollforward recovery stopped on the database" | wc -l`
        if [ $rollforwardUsingUTC -gt 0 ]; then
          echo "ERRORMSG: Fail to rollforward the database $SOURCE_DB_NAME to $RECOVERYTIME"
          cat $rollforwardUsingUTC_Output.$SOURCE_DB_NAME
          cleanup_db  $OSUSER
          exit 1
        fi
        su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME complete"

     else
         LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
         if [ "$LOGPATH" != "$TARGET_MNT$SOURCE_LOG_DIR_ORIG/" ]; then
            echo "$TARGET_MNT$SOURCE_LOG_DIR"
            if [ ! -d "$LOGPATH" ]; then
               mkdir -p $LOGPATH
            fi
         fi
          mkdir -p $TARGET_LOG_DIR/NODE0000/LOGSTREAM0000
          #cp -n -p  $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG $TARGET_LOG_DIR/NODE0000/LOGSTREAM0000
          LOG_TMP=`echo $TARGET_MNT$SOURCE_LOG_DIR_ORIG| awk -F "/NODE0000/" '{print $1}'`
          LOG_TMP2=`echo $TARGET_MNT$SOURCE_LOG_DIR_ORIG| awk -F "/NODE0000/" '{print $2}'`
          LOG_BASE1=`basename $LOG_TMP`
          LOG_DIR1=`dirname $LOG_TMP`
          
          if [ "$LOG_BASE1" = "$SOURCE_INSTANCE" ] ;then
             #cp  $LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2/*.LOG "$LOGPATH"
             if [ ! -d "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" ] && [ ! -d "$LOGPATH" ] ;then
                if [ "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" == "$LOGPATH" ] ;then
                   echo "softlink not required"
                else
                  ln -s "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" "$LOGPATH"
                fi
             fi
          else
              echo "soft link checking"
              if [ "$TARGET_MNT$SOURCE_LOG_DIR_ORIG" == "$LOGPATH" ] ; then
                 ln -s  "$TARGET_MNT$SOURCE_LOG_DIR_ORIG"  "$LOGPATH"
              fi
             #cp -n -p $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG "$LOGPATH"
          fi
            
          if [ ! -d "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" ] ; then
             mv $LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2/*LOG $LOGPATH
          fi

         chown -R $OSUSER:$OSGROUP $LOGPATH
       # NEW_LOGPATH=`echo $TARGET_LOG_DIR | awk -F"NODE" '{print $1}'`
       # su $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using NEWLOGPATH $NEW_LOGPATH"
         LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`

         #su  $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "
         su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME to end of logs"
         su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME complete"
         rollfwd_status=`su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
         if [ "$rollfwd_status" = "DBpending" ]; then
            su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME stop"
         fi
     fi


     #Archivelog Path change as source
     mkdir -p $TARGET_LOG_DIR/db2archivelog
     chown -R $OSUSER:$OSGROUP $TARGET_LOG_DIR/db2archivelog
     su  $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using LOGARCHMETH1 DISK:$TARGET_LOG_DIR/db2archivelog"

     #Rename database if needed
     if [ "$ACT_MOUNT_REPLACERESTORE" != "TRUE" ]; then
        $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
        $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RENAME_DB_CONFIG
        $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
        $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
        chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG
        if [ "$TARGET_DB_NAME" != "$SOURCE_DB_NAME" ] ; then
           for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
           do
             NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
             su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME  $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
           done
           su  $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
           su  $OSUSER -c "db2 activate db $TARGET_DB_NAME"
           if [ $? -ne 0 ] ; then
              echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
              for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
              do
                NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
                su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
              done
              cleanup_db $OSUSER
              exit 1
           fi
        fi
     fi
           if [ "$OS_TYPE" == "Linux" ]; then
              cp $RELOCATE_DB_CONFIG /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$TARGET_DB_NAME".cfg
              sed -i "s/DB_NAME.*/DB_NAME=$TARGET_DB_NAME/g" /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$TARGET_DB_NAME".cfg
           fi
   done  ### End of First for loop for same instance name
elif [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ];then
     dbname=""
     for dbname in $(echo $DB_LIST| tr "," " ")
     do
       OS_TYPE=`uname -s`
       if [ "$OS_TYPE" == "AIX" ]; then
          ECHO_E="echo "
          SU_FLAG=""
       elif [ "$OS_TYPE" == "Linux" ]; then
            ECHO_E="echo -e "
            SU_FLAG=""
       fi

       CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
       if [ ! -f $CONF_FILE ]; then
          echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
          exit 1
       fi
        
       
       SOURCE_DB_NAME=$dbname
       TARGET_DB_NAME=`cat $CONF_FILE | grep TARGET_DB_NAME | cut -d= -f2`
       TARGET_DB_NAME=`echo $TARGET_DB_NAME | tr '[a-z]' '[A-Z]'`
       SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
       SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
       SOURCE_STORAGE_PATH=`cat $CONF_FILE | grep SOURCE_STORAGE_PATH | cut -d= -f2`
       SOURCE_TBSP_PATH=`cat $CONF_FILE | grep TBSP_PATH | cut -d= -f2`
       SOURCE_LOG_DIR=`cat $CONF_FILE | grep SOURCE_LOG_DIR | cut -d= -f2`
       SOURCE_LOG_DIR_ORIG=$SOURCE_LOG_DIR
       SOURCE_LOG_DIR=`echo $SOURCE_LOG_DIR |awk -F"/NODE" '{print $1}'`
       MIRRORLOG_PATH=`cat $CONF_FILE | grep MIRRORLOGPATH |cut -d= -f2`
       TARGET_DB_PATH_base=$TARGET_MNT$SOURCE_DB_PATH

       # Prepare TARGET_DB_PATH
       TARGET_DB_PATH=$TARGET_DB_PATH_base
       if [[ "$DBCOUNT" -gt 1 ]] && [[ "$DBP_COUNT" -ne "$DBCOUNT" ]] ; then
          TARGET_DB_PATH=$TARGET_DB_PATH_base/"$SOURCE_DB_NAME"_DBP
       fi

       # Prepare TARGET_LOG_DIR
       TARGET_LOG_DIR=$TARGET_DB_PATH_base
       echo $SOURCE_LOG_DIR_ORIG | grep -i SQL00*
       if [ $? -eq 0 ];then
          TARGET_LOG_DIR=$TARGET_DB_PATH_base/"$SOURCE_DB_NAME"_ACL
          mkdir -p $TARGET_LOG_DIR
       fi

       RENAME_DB_CONFIG=/act/touch/act_db2_rename_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
       RELOCATE_DB_CONFIG=/act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$SOURCE_DB_NAME".cfg
       if [ -z "$ARCHIVELOG_MNT" ] ; then
         ARCHIVELOG_MNT=`cat $CONF_FILE | grep ARCHIVELOGMOUNTPATH |cut -d= -f2`
       fi 
       $ECHO_E "DB_NAME=$SOURCE_DB_NAME" > $RELOCATE_DB_CONFIG
       $ECHO_E "DB_PATH=$SOURCE_DB_PATH,$TARGET_DB_PATH" >> $RELOCATE_DB_CONFIG
       if [ ! -z "$MIRRORLOG_PATH" ] ;then
          $ECHO_E "MIRRORLOG_PATH=$MIRRORLOG_PATH,$TARGET_MNT$MIRRORLOG_PATH" >> $RELOCATE_DB_CONFIG
       fi
       if [ "$SOURCE_INSTANCE" != "$TARGET_INSTANCE" ];then
          $ECHO_E "INSTANCE=$SOURCE_INSTANCE,$TARGET_INSTANCE" >> $RELOCATE_DB_CONFIG
       else
          $ECHO_E "INSTANCE=$SOURCE_INSTANCE" >> $RELOCATE_DB_CONFIG
       fi
       LOG_BASE=`basename $TARGET_LOG_DIR`
       LOG_DIR=`dirname $TARGET_LOG_DIR`
       if [ "$LOG_BASE" = "$SOURCE_INSTANCE" ] ; then
         #   mv $TARGET_MNT$SOURCE_LOGDIR $LOG_DIR/$TARGET_INSTANCE
            TARGET_LOG_DIR=$LOG_DIR/$TARGET_INSTANCE
            $ECHO_E "LOG_DIR=$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
       else
       $ECHO_E "LOG_DIR=$SOURCE_LOG_DIR,$TARGET_LOG_DIR" >> $RELOCATE_DB_CONFIG
       fi
       $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RELOCATE_DB_CONFIG
        
       if [ ! -z "$SOURCE_STORAGE_PATH" ] ; then
          for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
          do
            target_pathname=$TARGET_MNT$pathname
            $ECHO_E "STORAGE_PATH=$pathname,$target_pathname" >> $RELOCATE_DB_CONFIG
          done
       fi
       if [ ! -z "$SOURCE_TBSP_PATH" ] ; then
           for tbspath in $(echo $SOURCE_TBSP_PATH | tr "," " ")
           do
            target_tbspath=$TARGET_MNT$tbspath
            $ECHO_E "CONT_PATH=$tbspath,$target_tbspath" >> $RELOCATE_DB_CONFIG
            done
       fi
       #Change ownership of new loaction
       OSUSER=$TARGET_INSTANCE
       OSGROUP=`id -ngr  $OSUSER`

       mv $RELOCATE_DB_CONFIG "$RELOCATE_DB_CONFIG".tmp
       cat "$RELOCATE_DB_CONFIG".tmp |sort -n|uniq > $RELOCATE_DB_CONFIG
       rm -f "$RELOCATE_DB_CONFIG".tmp
       #Renaming Instance name
       if [ -d "$TARGET_DB_PATH/$TARGET_INSTANCE" ] ;then
            echo "continue"
       else
            mv $TARGET_DB_PATH/$SOURCE_INSTANCE $TARGET_DB_PATH/$TARGET_INSTANCE
            if [ $? -eq 0 ] ; then
               ln -s $TARGET_DB_PATH/$TARGET_INSTANCE $TARGET_DB_PATH/$SOURCE_INSTANCE
            fi
       fi
       for path in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
       do
         target_path=$TARGET_MNT$path
         mv $target_path/$SOURCE_INSTANCE $target_path/$TARGET_INSTANCE
            if [ $? -eq 0 ] ; then
               ln -s $target_path/$TARGET_INSTANCE $target_path/$SOURCE_INSTANCE
            fi
       done
       mkdir -p $TARGET_MNT$SOURCE_LOG_DIR
       chown -R $OSUSER:$OSGROUP $TARGET_MNT
       chmod -R 755 $TARGET_MNT
       chown -R $OSUSER:$OSGROUP $ARCHIVELOG_MNT
       chown -R $OSUSER:$OSGROUP $RELOCATE_DB_CONFIG
       chown -R $OSUSER:$OSGROUP $TARGET_DB_PATH
       if [ ! -z "$CHILD_MNT" ] ;then
          chown -R $OSUSER:$OSGROUP $CHILD_MNT
       fi
       #Catalog database
       su  $OSUSER -c "db2 catalog database $SOURCE_DB_NAME  on $TARGET_DB_PATH"
       if [ $? -ne 0 ]; then
          echo "ERRORMSG: Failed to catalog database $SOURCE_DB_NAME"
          cleanup_db $OSUSER
          exit 1
       fi

       #Reloacte database
       su  $OSUSER -c "db2inidb $SOURCE_DB_NAME as standby relocate using $RELOCATE_DB_CONFIG"
       if [ $? -ne 0 ] ; then
          sleep 5
          su  $OSUSER -c "db2inidb $SOURCE_DB_NAME as standby"
          retval=$?
         if [ "$retval" -gt 0 ]; then
            echo "ERRORMSG: Failed to relocate database $DB_NAME"
            cleanup_db $OSUSER
            exit 1
         fi
       fi

       #Rollforwad db recovery
       if [ ! -z "$RECOVERYTIME" ]; then
          LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
          mv $ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1/$SOURCE_INSTANCE $ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1/$TARGET_INSTANCE
          chown -R $OSUSER:$OSGROUP $ARCHIVELOG_MNT

          su  $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "
          rollfwd_status=`su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
          su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME to $RECOVERYTIME using UTC time" > $rollforwardUsingUTC_Output.$SOURCE_DB_NAME
          rollforwardUsingUTC=`grep 'SQL[0-9][0-9][0-9][0-9][A-Z]' $rollforwardUsingUTC_Output.$SOURCE_DB_NAME | grep -v "SQL4970N  Rollforward recovery stopped on the database" | wc -l`
	  if [ $rollforwardUsingUTC -gt 0 ]; then
	    echo "ERRORMSG: Fail to rollforward the database $SOURCE_DB_NAME to $RECOVERYTIME"
	    cat $rollforwardUsingUTC_Output.$SOURCE_DB_NAME
	    cleanup_db  $OSUSER
	    exit 1
          fi
          su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME complete"
       else
          LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
          if [ "$LOGPATH" != "$TARGET_MNT$SOURCE_LOG_DIR_ORIG/" ]; then
            echo "$TARGET_MNT$SOURCE_LOG_DIR"
            if [ ! -d "$LOGPATH" ]; then
               mkdir -p $LOGPATH
            fi
          fi
          
          mkdir -p $TARGET_LOG_DIR/NODE0000/LOGSTREAM0000
          #cp  $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG $TARGET_LOG_DIR/NODE0000/LOGSTREAM0000
          LOG_TMP=`echo $TARGET_MNT$SOURCE_LOG_DIR_ORIG| awk -F "/NODE0000/" '{print $1}'` 
          LOG_TMP2=`echo $TARGET_MNT$SOURCE_LOG_DIR_ORIG| awk -F "/NODE0000/" '{print $2}'`
          LOG_BASE1=`basename $LOG_TMP`
          LOG_DIR1=`dirname $LOG_TMP`

          if [ "$LOG_BASE1" = "$SOURCE_INSTANCE" ] ;then
             #cp  $LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2/*.LOG "$LOGPATH"
             if [ ! -d "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" ] && [ ! -d "$LOGPATH" ] ;then
                if [ "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" == "$LOGPATH" ] ;then
                   echo "softlink not required"
                else
                  ln -s "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" "$LOGPATH"
                fi
             fi
          else
              echo "soft link checking"
              if [ "$TARGET_MNT$SOURCE_LOG_DIR_ORIG" == "$LOGPATH" ] ; then
                 ln -s  "$TARGET_MNT$SOURCE_LOG_DIR_ORIG"  "$LOGPATH"
              fi
             #cp -n -p $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG "$LOGPATH"
          fi
           if [ ! -d "$LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2" ] ; then
              mv $LOG_DIR1/$TARGET_INSTANCE/NODE0000/$LOG_TMP2/*.LOG $LOGPATH
           fi
           chown -R $OSUSER:$OSGROUP "$LOGPATH"
         # LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
          su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME to end of logs"
          su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME complete"
          rollfwd_status=`su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
          if [ "$rollfwd_status" = "DBworking" ]; then
              su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME stop"
          elif [ "$rollfwd_status" = "DBpending" ]; then
              su  $OSUSER -c "db2 rollforward db $SOURCE_DB_NAME stop"
          fi
       fi

       #Archivelog Path change as source

       mkdir -p $TARGET_LOG_DIR/db2archivelog
       chown -R $OSUSER:$OSGROUP $TARGET_LOG_DIR/db2archivelog
       su  $OSUSER -c "db2 update db cfg for $SOURCE_DB_NAME  using LOGARCHMETH1 DISK:$TARGET_LOG_DIR/db2archivelog"

       #Rename database if needed

       $ECHO_E "DB_NAME=$SOURCE_DB_NAME,$TARGET_DB_NAME" > $RENAME_DB_CONFIG
       $ECHO_E "DB_PATH=$TARGET_DB_PATH" >> $RENAME_DB_CONFIG
       $ECHO_E "INSTANCE=$TARGET_INSTANCE" >> $RENAME_DB_CONFIG
       $ECHO_E "NODENUM=$TARGET_NODENUM" >> $RENAME_DB_CONFIG
       chown $OSUSER:$OSGROUP $RENAME_DB_CONFIG

       if [ "$TARGET_DB_NAME" != "$SOURCE_DB_NAME" ] ; then
          for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
          do
            NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
            su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME"
          done
          su $OSUSER -c "db2relocatedb -f $RENAME_DB_CONFIG"
          su  $OSUSER -c "db2 activate db $TARGET_DB_NAME"
          if [ $? -ne 0 ] ; then
             echo "ERRORMSG: Failed to rename database $SOURCE_DB_NAME to $TARGET_DB_NAME "
             for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
             do
               NODE=`ls -lrt $TARGET_MNT$pathname/$TARGET_INSTANCE |awk '{print $9}'| grep "NODE*"|sed -n '$p'`
               su  $OSUSER -c "mv $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$TARGET_DB_NAME $TARGET_MNT$pathname/$TARGET_INSTANCE/$NODE/$SOURCE_DB_NAME"
             done
             cleanup_db $OSUSER
             exit 1
          fi
       fi
           if [ "$OS_TYPE" == "Linux" ]; then
              cp $RELOCATE_DB_CONFIG /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$TARGET_DB_NAME".cfg
              sed -i "s/DB_NAME.*/DB_NAME=$TARGET_DB_NAME/g" /act/touch/act_db2_relocate_"$JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$TARGET_DB_NAME".cfg
           fi
     done
fi
if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
   chmod -R 755 $CHILD_MNT
fi

createconfig
#cleanup_files
exit 0
