#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
ACT_JOBNAME=$JOBID
SOURCE_INSTANCE=$SOURCE_INSTANCE
TARGET_INSTANCE=$TARGET_INSTANCE
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST

#OS_TYPE [ linux or AIX ]

OS_TYPE=`uname -s`
if [ "$OS_TYPE" == "AIX" ]; then
      SU_FLAG=""
      ECHO_E="echo"
elif [ "$OS_TYPE" == "Linux" ]; then
     ECHO_E="echo -e "
     SU_FLAG=""
fi

SUCCESS_DBS=""

DB_LIST=$TARGET_DBNAME_LIST
PWDS=/act/custom_apps/db2
#OSUSER Details:
OSUSER=$TARGET_INSTANCE

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

cleanup_files(){
   echo "MOUNT_JOBID=$MOUNT_JOBID"
   rm -f /act/touch/*_"$MOUNT_JOBID"_params
   rm -f /act/touch/act_db2_relocate_"$MOUNT_JOBID"_"$TARGET_INSTANCE"_*
   rm -f /act/touch/act_db2_rename_"$MOUNT_JOBID"_"$TARGET_INSTANCE"_*
   rm -f /act/touch/dbpathsortlist_"$MOUNT_JOBID"_"$TARGET_INSTANCE"_*
   rm -f /act/touch/OFTDN_"$MOUNT_JOBID"
}
find_user=`cat /etc/passwd | grep $OSUSER | cut -d ":" -f1`
if [ $? -ne 0 ]; then
   echo "ERRORMSG: os user : $OSUSER doesn't exists"
   exit 1
fi
  
#### Do not delete databases when mount and migrate is done ###
DB2_MIGRATE_CONF=/act/touch/.ACT_DB2_migrate.conf
if [ -f "$DB2_MIGRATE_CONF" ]; then
   MIGRATED=`cat $DB2_MIGRATE_CONF | awk -F"=" '{print $2}'`
   if [ "$MIGRATED" = "YES" ]; then
      rm -f $DB2_MIGRATE_CONF
      exit 0
   fi
fi

#identify db2_home
 proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
 if [ -z "$proc_id" ];then
     echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
     exit 1
 fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#find instance
find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $TARGET_INSTANCE)
if [ "$find_inst" != "$OSUSER" ]; then
   echo "ERRORMSG: Instance: $TARGET_INSTANCE doesn't exists or not running.Please check log For more details"
   exit 1
fi
echo " *******************Deactivating Database: $dbname ******************************** "

# Need MOUNT_JOBID during cleanup
MOUNT_JOBID=$MOUNT_JOBID

for dbname in $(echo $DB_LIST| tr "," " ")
do
  dbname=`echo $dbname |awk -F":" '{print $2}'`
  dbname=`echo $dbname | tr '[a-z]' '[A-Z]'`
  if [ -z "$dbname" ]; then
     continue;
  else

     #Db2 connect and Deactivate

     su $SU_FLAG $OSUSER -c "db2 connect to $dbname"
     if [ $? -ne 0 ] ; then
        echo " Failed to connect Database $dbname"
        continue 
     fi

     # Get MOUNT_JOBID befor uncatalog
     if [ -z "$MOUNT_JOBID" ] && [ "$OS_TYPE" = "Linux" ]; then
       DB_directory=`su $SU_FLAG  $OSUSER -c "db2 list database directory " | grep -A1 -w "= $dbname" | grep "Local database directory" | cut -d= -f2`
       DB_directory_mntpnt=`df -P $DB_directory | tail -1 | awk '{print $NF}'`
       DB_directory_vg_name=`df | grep -w $DB_directory_mntpnt | awk '{print $1}' | cut -d/ -f4 | cut -d'-' -f1`
       MOUNT_JOBID=`find /act/tmpdata/ -name $DB_directory_vg_name | cut -d/ -f4 |grep -vw $ACT_JOBNAME | head -1`
     fi

     su $SU_FLAG  $OSUSER -c "db2 terminate"
     su $SU_FLAG  $OSUSER -c "db2 force application all"
     su $SU_FLAG  $OSUSER -c "db2 deactivate database $dbname"
     su $SU_FLAG  $OSUSER -c "db2 uncatalog database $dbname"

     if [ $? -ne 0 ] ; then
        echo "ERRORMSG: Failed to deactivate database $dbname" 
        continue
     else 
        echo "$dbname deactivation is successfull"
     fi
  fi
#uncatalog database 

  su $SU_FLAG  $OSUSER -c "db2 uncatalog database $dbname"
  if [ $? -ne 0 ] ; then
     echo "ERRORMSG: Failed to uncatalog database $dbname" 
  else 
     echo " $dbname  uncatalog is completed"
     SUCCESS_DBS="$SUCCESS_DBS  $dbname"
  fi
done

echo "Successfully deactivated  databases: $SUCCESS_DBS"
cleanup_files

#Cleanup if MOUNT_JOBID is null
for dbname in $(echo $DB_LIST| tr "," " ")
do
  dbname=`echo $dbname |awk -F":" '{print $2}'`
  dbname=`echo $dbname | tr '[a-z]' '[A-Z]'`
  if [ -z "$dbname" ]; then
     continue;
  fi
  rm -f /act/touch/act_db2_relocate_*_"$TARGET_INSTANCE"_"$dbname"_*
  rm -f /act/touch/act_db2_rename_*_"$TARGET_INSTANCE"_"$dbname"_*
  rm -f /act/touch/dbpathsortlist_*_"$TARGET_INSTANCE"_"$dbname"_*
done

exit 0


