#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
JOBNAME=$JOBID
SOURCE_INSTANCE=$SOURCE_INSTANCE
TARGET_INSTANCE=$TARGET_INSTANCE
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST
ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE
TARGET_NODENUM=0
SOURCE_VERSION=$SOURCE_VERSION
OVERWRITE_DATABASE=$OVERWRITE_DATABASE

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

OSUSER=$TARGET_INSTANCE
if [ "$ACT_JOBTYPE" = "unmount" ]; then
   echo "No op for unmount!"
   exit 0
fi
#identify db2_home
proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
if [ -z "$proc_id" ];then
   echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
   echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
   exit 1
fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`
  
  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

# Delete DB function
deleteDB() {
  DbName=$1
  echo "INFOMSG: Dropping the database $DbName"
  su  $OSUSER -c "db2 terminate;db2 deactivate db $DbName"
  su  $OSUSER -c "db2 drop  db $DbName"
  su  $OSUSER -c "db2 uncatalog db $DbName"
  return 0
}

# Find DB stale
deleteDBifStale() {
  DbName=$1

  # Check db stale or not. If active fail.
  su $OSUSER -c "db2 terminate;db2 connect to $DbName"
  if [ $? -eq 0 ]; then
    echo "ERRORMSG: Database $DbName exists and is in active state"
    echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

  deleteDB $DbName
  return 0
}


TARGET_VERSION=`su $OSUSER -c "db2licm -v "`
#TARGET_VERSION=`echo $TARGET_VERSION |awk -F":" '{print $2}' | xargs`

if [ "$TARGET_VERSION" != "$SOURCE_VERSION" ]; then
   echo "ERRORMSG: App aware mount from $SOURCE_VERSION to $TARGET_VERSION is not supported!"
   echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
   exit 1
fi

for dbname in $(echo $TARGET_DBNAME_LIST | tr "," " ")
do
  TARGET_DB=`echo $dbname | awk -F":" '{print $2}'`
  TARGET_DB=`echo $TARGET_DB | tr '[a-z]' '[A-Z]'`
  TARGET_DB_LEN=${#TARGET_DB}
  if [ "$TARGET_DB_LEN" -gt 8 ]; then
     echo "ERRORMSG: Target database name $dbname is more than 8 characters.!"
     echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
     exit 1
  fi
done

#instance status

find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $TARGET_INSTANCE)

if [ "$find_inst" != "$OSUSER" ]; then
   echo "ERRORMSG: Instance: $TARGET_INSTANCE is not running on this host.Please check log For more details"
   echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
   exit 1
fi
          
SRC_DBS=`su  $OSUSER -c "db2 list db directory" |  awk -F"=" '/Database name/{print $2}' |xargs`

if [ -z "$SRC_DBS" ]; then
   echo "Target instance do not have any databases registered!"
   exit 0
fi

#DB list checking in Target 

for dbname in $(echo $TARGET_DBNAME_LIST | tr "," " ")
do
  TARGET_DB=`echo $dbname | awk -F":" '{print $2}'`
  TARGET_DB=`echo $TARGET_DB | tr '[a-z]' '[A-Z]'`
  SOURCE_DB=`echo $dbname | awk -F":" '{print $1}'`
  
  CHECK_SOURCE_DB=`su  $OSUSER -c "db2 list db directory" |  awk -F"=" '/Database name/{print $2}' |grep -w $SOURCE_DB|xargs`
  CHECK_TARGET_DB=`su  $OSUSER -c "db2 list db directory" |  awk -F"=" '/Database name/{print $2}' |grep -w $TARGET_DB|xargs`
  if [ ! -z "$CHECK_SOURCE_DB" ]; then
     if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
        su  $OSUSER -c "db2 deactivate db $SOURCE_DB"
        su  $OSUSER -c "db2 uncatalog db $SOURCE_DB"
     else
        echo "ERRORMSG: Source Database $CHECK_SOURCE_DB is already registered with instance!. Please check!"
        echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
        exit 1
     fi
  elif [ ! -z "$CHECK_TARGET_DB" ]; then
    case $OVERWRITE_DATABASE in
      yes)
	deleteDB $CHECK_TARGET_DB
	;;
      stale)
	deleteDBifStale $CHECK_TARGET_DB
        ;;
      *)
        echo "ERRORMSG: Target Database $CHECK_TARGET_DB is already registered with instance!. Please check!"
        echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
        exit 1
        ;;
    esac
  fi
done   

exit 0
