#!/bin/bash
#
# DB2 script for discovery
#

# Globals
AUTHID_STR="SQL authorization ID"
LOGARCHMETH_NAME="LOGARCHMETH"
SCRIPTS="/act/custom_apps/CustomApp_DB2_V1.sh"
XMLFILE="/act/custom_apps/customapps.xml"
TIMEOUT="600"
STORAGE_PATHS=""
APPNAME=""
INSTANCE=""
FTYPE=""

LOGMODE="LOGARCHMETH1"

function xmlOutput() {
   echo -e "\t<application name=\"$APPNAME\" friendlytype=\"$FTYPE\" instance=\"$INSTANCE\"> 
\t\t<files>" >> $XMLFILE

    filepaths=$STORAGE_PATHS
    for element in ${filepaths/' '/ } ; do
        echo -e "\t\t\t<file path=\"$element\" />" >>$XMLFILE
    done
    echo -e "\t\t</files>
\t\t<scripts>" >>$XMLFILE
    scripts=$SCRIPTS
    for scr in ${scripts/' '/ } ; do
        echo -e "\t\t\t<script phase=\"all\" path=\"$scr\" timeout=\"$TIMEOUT\" />" >>$XMLFILE
    done
    echo -e "\t\t</scripts>
\t</application>" >>$XMLFILE
}

function cleanup() {
    dbiname=$1
    echo "DB2 $dbiname Cleanup $XMLFILE"
    rm $XMLFILE
}

function enumerate_storage_paths() {
    local dbinst=$1
    local dbname=$2
    local storagepath=""
    
    echo Enumerating storage paths for db: $dbname using instance: $dbinst

    PENDING_BACKUP=$(su - $dbinst -c "db2 connect to $dbname | grep 'BACKUP PENDING.  SQLSTATE=57019'")
    if [[ $PENDING_BACKUP == "" ]]; then
        echo "DB $dbname is in good state to discover"
        TMP=$(su - $dbinst -c "db2 connect to $dbname>nul")
    else
        echo "$dbname is in pending backup state, please run first backup to enable log backup"
        cleanup $dbinst
        return 1
    fi

    STORAGE_PATHS=$(su - $dbinst -c "db2 connect to $dbname>nul; db2 -x \"select PATH from sysibmadm.DBPATHS\"" | \
			   while read p; do df -h $p| tail -1 | awk '{print $6}'; done | sort | uniq)
    
    if [[ $STORAGE_PATHS == " " ]]; then
	echo Storage enumeration for DB $dbname returned error
	STORAGE_PATHS=""
	return 1
    else
	local LOG_PATHS=$(su - $dbinst -c "db2 get db cfg for $dbname" | grep -i $LOGARCHMETH_NAME | \
				 awk -F "=" '{print $2}' | grep -i "disk" | awk -F ":" '{print $2}' | \
				 while read l; do df -h $l| tail -1 | awk '{print $6}'; done | sort | uniq)
	if [[ $LOG_PATHS == " " ]]; then
	    echo Log path enumeration failed for DB $dbname
	    STORAGE_PATHS=""
	    LOG_PATHS=""
	    return 1
	else
	    STORAGE_PATHS="$STORAGE_PATHS $LOG_PATHS"
            # Make sure you have set 
            xmlOutput 
	    return 0
	 fi
    fi
	
}
    
function enumerate_db2_dbs() {
    local dbinst=$1

    local db=$(su - $dbinst -c "db2 list db directory" | grep 'Database name' | sed -e 's/=//g' | awk '{print $3}')
    echo list of databases $db in Instances $dbinst
    for dbname in $db; do
	#echo Found database: $dbname
        APPNAME=$dbname
        FTYPE=$dbname"_customapp"
        local logenabled=$(su - $dbinst -c "db2 get db cfg for $dbname" | grep $LOGMODE | cut -d '=' -f2| grep OFF)
        if [[ "$logenabled" == " OFF" ]]; then
            echo "Skipping database $dbname as log archive mode is not enabled"
        else
            echo finding database $dbname paths
	    enumerate_storage_paths $dbinst $dbname
        fi
    done
}

function enum_db2_instances(){
    # Print out all the instance users
    >$XMLFILE
    for inst_name in $(ps -ef | grep [d]b2sys | awk '{print $1}'); do
        INSTANCE=$inst_name
        echo instancename is $inst_name
	XMLFILE=${inst_name}.xml
	echo XMLFILE1 is now $XMLFILE
        >$XMLFILE
        echo -e "<applications>" >$XMLFILE
        err=$(enumerate_db2_dbs $inst_name)
        if [[ $err == 0 ]]; then
            echo -e "</applications>" >>$XMLFILE
        fi
    done

}

enum_db2_instances

