#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

PWDS=/act/custom_apps/db2
JOBNAME=$JOBID
DB2INSTNAME=$SOURCE_INSTANCE
DATABASE_LIST=$DB_LIST
OSUSER=$DB2INSTNAME
TARGET_MNT=$TARGET_MNT
DB_LIST=`echo $DATABASE_LIST | tr [:lower:] [:upper:]`
echo "$DB_LIST"
RECOVERYTIME=$UNTIL_TIME   #YYYYMMDDHHMMSS
ARCHIVELOG_MNT=$ARCHIVELOG_MNT
rollforwardUsingUTC_Output=/act/touch/rollforwardUsingUTC_$JOBNAME
#Instance status

#JOBNAME=Job_0449500
#DB2INSTNAME=inst2
#DATABASE_LIST=DP3
#OSUSER=$DB2INSTNAME
#TARGET_MNT=/act/mnt/Job_0449500_mountpoint_1565793415997
#DB_LIST=`echo $DATABASE_LIST | tr [:lower:] [:upper:]`
#echo "$DB_LIST"
#RECOVERYTIME=
#ARCHIVELOG_MNT=/act/mnt/Job_0449500_mountpoint_1565793397338

function remove_file
{
   fname=$1
   if [ -f $fname ]; then
      rm -f $fname
   fi
}

function generate_dump_file
{
   DBNAME=$1
   ls -lt $TARGET_MNT/$DBNAME | rev | awk '{print $1}'|rev > $DUMP_LIST
   while read dumpfile
   do
     DUMP_TYPE=`db2ckbkp -h $TARGET_MNT/$dbname/$dumpfile | grep -i "Backup Granularity"|xargs`
     DUMP_TYPE=`echo $DUMP_TYPE |awk '{print $4}'`
     if [ "$DUMP_TYPE" = "16" ]; then
        echo $dumpfile >> $DUMP_LIST_TMP
     elif [ "$DUMP_TYPE" = "0" ]; then 
        echo $dumpfile >> $DUMP_LIST_TMP
        break; 
     fi
   done < $DUMP_LIST
   cat $DUMP_LIST_TMP | sort -n > $DUMP_LIST
}

function restore_db
{
  DUMP_TSTAMP=$1
  INCR_FLAG=$2
  dbname=$3
     ls -ltr $TARGET_MNT/$dbname | grep $DUMP_TSTAMP
     if [ $? -ne 0 ];then
         echo "ERRORMSG:dump backup is not available in current directory!! timestamp: $DUMP_TSTAMP"
         exit 1
     fi
  su  $OSUSER -c  "db2 deactivate db $dbname"
  if [ "$INCR_FLAG" = "FULL" ]; then
     su  $OSUSER <<EOF
     db2 terminate
     db2 restore db $dbname  from $TARGET_MNT/$dbname taken at $DUMP_TSTAMP logtarget $TARGET_MNT/$dbname/logs without prompting
EOF
  elif [ "$INCR_FLAG" = "INCR" ]; then
     su  $OSUSER <<EOF
     db2 terminate
     db2 restore db $dbname  incremental from $TARGET_MNT/$dbname  taken at $DUMP_TSTAMP logtarget $TARGET_MNT/$dbname/logs without prompting
EOF
  fi
}


############# MAIN #####################

find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2INSTNAME | wc -l)
                  
   if [[ $find_inst -ne 1 ]] ; then
          echo " ERRORMSG: Instance is not running "
          exit 1
   fi

 #db2 Instance home
 #identify db2_home
   proc_id=`ps aux | grep -w $DB2INSTNAME |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#OLDIFS=IFS

#IFS=','

#changing owner ship TARGET_MNT
OSGROUP=`id -gnr $OSUSER`
chown -R $OSUSER:$OSGROUP $TARGET_MNT

for dbname in $(echo $DB_LIST| tr "," " ")
do
  CONF_FILE=/act/touch/"$dbname"_"$JOBNAME"_params
  if [ ! -f $CONF_FILE ]; then
     echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
     exit 1
  fi    
  db_part_nums=`awk '{print $1}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -nr `
  DUMP_LIST=/act/touch/"$dbname"_DUMP_FILE.txt
  DUMP_LIST_TMP=/act/touch/"$dbname"_DUMP_FILE_TMP.txt
  remove_file $DUMP_LIST
  remove_file $DUMP_LIST_TMP

  if [ ! d = "$TARGET_MNT/$dbname/logs" ]; then
     su  $OSUSER -c "mkdir -p $TARGET_MNT/$dbname/logs"
  fi
 
  if [ -d "$TARGET_MNT/$dbname/logs" ]; then
     rm -rf $TARGET_MNT/$dbname/logs/*
  fi
  generate_dump_file $dbname  

  DUMP_TYPE=`cat $DUMP_LIST | wc -l`
  FIRST_INCR_DUMP=`tail -1 $DUMP_LIST`
  FULL_DUMP=`head -1 $DUMP_LIST`
  
  ### Apply latest incremental
  echo "*********** Applying first incremental backup...!!***********"

  if [ "$DUMP_TYPE" -gt 1 ]; then
     DUMP_TSTAMP=`echo $FIRST_INCR_DUMP | awk -F"." '{print $5}'`
#     incr_dep_list=`su  $OSUSER -c "db2ckrst -d $dbname -t $DUMP_TSTAMP" | grep "restore db" | awk '{print $NF}' | sort -n | uniq`
#      for timestamp in $incr_dep_list
#      do
      restore_db $DUMP_TSTAMP INCR $dbname
#      done

  ### Apply First full backup

  echo "*********** Applying first full backup...!!***********"
     DUMP_TSTAMP=`echo $FULL_DUMP | awk -F"." '{print $5}'`
#      incr_dep_list=`su  $OSUSER -c "db2ckrst -d $dbname -t $DUMP_TSTAMP" | grep "restore db" | awk '{print $NF}' | sort -n | uniq`
#      for timestamp in $incr_dep_list
#      do
      restore_db $DUMP_TSTAMP INCR $dbname
#      done
  elif [ "$DUMP_TYPE" -eq 1 ]; then
       DUMP_TSTAMP=`echo $FULL_DUMP | awk -F"." '{print $5}'`
       restore_db $DUMP_TSTAMP FULL $dbname
  fi

  ###Apply other incrementals

  echo "*********** Applying all incremental backups..!!***********"
  while read dumpname
  do
     if [ "$dumpname" = "$FIRST_INCR_DUMP" ] || [ "$dumpname" = "$FULL_DUMP" ]; then
        continue;
     fi

     DUMP_TSTAMP=`echo $dumpname | awk -F"." '{print $5}'`
#     incr_dep_list=`su  $OSUSER -c "db2ckrst -d $dbname -t $DUMP_TSTAMP" | grep "restore db" | awk '{print $NF}' | sort -n | uniq`
#      for timestamp in $incr_dep_list
#      do
      restore_db $DUMP_TSTAMP INCR $dbname
#      done
  done < $DUMP_LIST
  
  ##### Final apply of Incremental backup
  echo " *********** Applying last incremental backups..!***********"
  if [ "$DUMP_TYPE" -gt 1 ]; then
     DUMP_TSTAMP=`echo $FIRST_INCR_DUMP | awk -F"." '{print $5}'`
#      incr_dep_list=`su  $OSUSER -c "db2ckrst -d $dbname -t $DUMP_TSTAMP" | grep "restore db" | awk '{print $NF}' | sort -n | uniq`
#     for timestamp in $incr_dep_list
#     do
      restore_db $DUMP_TSTAMP INCR $dbname
#      done
  fi

  ROLLFORWARD_STATUS=`su  $OSUSER -c "db2 rollforward db $dbname  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
  
  SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
  ARCHIVE_LOG_LOC=$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1

  
  ##### Apply Logs
  if [ "$ROLLFORWARD_STATUS" = "DBpending" ]; then
     if [ -z "$RECOVERYTIME" ] && [ -z "$ARCHIVELOG_MNT" ]; then    
        su  $OSUSER -c "db2 rollforward db $dbname  to end of logs and stop overflow log path \($TARGET_MNT/$dbname/logs\)"
     elif [ ! -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su  $OSUSER -c "db2 update db cfg for $dbname  using LOGARCHMETH1 DISK:$ARCHIVE_LOG_LOC"
        su  $OSUSER -c "db2 rollforward db $dbname  to $RECOVERYTIME using UTC time" > $rollforwardUsingUTC_Output.$dbname
        rollforwardUsingUTC=`grep 'SQL[0-9][0-9][0-9][0-9][A-Z]' $rollforwardUsingUTC_Output.$dbname | grep -v "SQL4970N  Rollforward recovery stopped on the database" | wc -l`
        if [ $rollforwardUsingUTC -gt 0 ]; then
          echo "ERRORMSG: Fail to rollforward the database $dbname to $RECOVERYTIME"
          cat $rollforwardUsingUTC_Output.$dbname
          exit 1
        fi

     elif [ -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su  $OSUSER -c "db2 update db cfg for $dbname  using LOGARCHMETH1 DISK:$ARCHIVE_LOG_LOC"
        su  $OSUSER -c "db2 rollforward db $dbname  to end of logs"
     fi
 
     
     su  $OSUSER -c "db2 update db cfg for $dbname  using LOGARCHMETH1 DISK:$SOURCE_LOGARCHMETH1"
     ##### Check the status after recovery
     ROLLFORWARD_STATUS=`su  $OSUSER -c "db2 rollforward db $dbname  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
     if [ "$ROLLFORWARD_STATUS" != "notpending" ]; then
        su  $OSUSER -c "db2 rollforward db $dbname complete"
        ROLLFORWARD_STATUS_afterComplete=`su  $OSUSER -c "db2 rollforward db $dbname  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
        if [ "$ROLLFORWARD_STATUS_afterComplete" = "DBpending" ]; then
          echo "ERRORMSG: Fail to rollforward the database $dbname. ($ROLLFORWARD_STATUS_afterComplete)"
          exit 1
        fi
     elif [ "$ROLLFORWARD_STATUS" = "notpending" ]; then
          echo " ROLL FORWARD recovery is completed "
          echo "Database $dbname restore succesfull"
     fi

     su  $OSUSER -c "db2 \"UPDATE DATABASE CONFIGURATION FOR $dbname USING TRACKMOD YES\" "
     retval=$?

     if [ "$retval" -gt 0 ]; then
        echo "WARNING MSG: Failed to update the trackmod for $dbname"
     fi

     for DB_PART_NUM in $db_part_nums; do
        export DB2NODE=$DB_PART_NUM
        dbstatus=`su  $OSUSER <<EOF | grep -v "selected"|xargs
db2 connect to $dbname>/dev/null
db2 -x "select db_status FROM SYSIBMADM.SNAPDB"
EOF`
        if [ "$dbstatus" = "ACTIVE" ]; then
           echo "Database $dbname is already activated"
        else
           su  $OSUSER -c "db2 activate database $dbname "
           retval=$?
           if [ "$retval" -eq 0 ]; then
              echo "$dbname database successfully activated"
           fi
        fi
     done
  else
     echo "ERRORMSG: Failed to restore the database, please check the logs!"
     exit 1
  fi
done
exit 0
