#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
JOBNAME=$JOBID
DB2INSTNAME=$DB2_INSTANCE_NAME
DATABASE_LIST=$DB_DATABASE_LIST
DUMPSCHEDULE=$DUMPSCHEDULE
OSUSER=$DB2INSTNAME
IGNOREPARTIALFAILURE=$IGNOREPARTIALFAILURE

if [ "$JOBNAME" = "" ]; then
  echo "ERRORMSG: JOBNAME (JOBID) is not provided."
  exit 1
fi

if [ "$DB2INSTNAME" = "" ]; then
  echo "ERRORMSG: Db2 (DB2INSTNAME) is not provided"
  exit 1
fi

if [ "$DATABASE_LIST" = "" ]; then
  echo "ERRORMSG: Database list (DATABASE_LIST) is not provided."
  exit 1
fi

if [ "$DUMPSCHEDULE" = "" ]; then
  echo "ERRORMSG: Dump Schedule (DUMPSCHEDULE) is not provided"
  exit 1
fi

#identify db2_home
   proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

   if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#instance status

find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2INSTNAME)

if [ "$find_inst" != "$OSUSER" ]; then
   echo "ERRORMSG: Instance: $TARGET_INSTANCE is not running on this host.Please check log For more details"
   exit 1
fi
          
SRC_DBS=`su  $OSUSER -c "db2 list db directory" |  awk -F"=" '/Database name/{print $2}' |xargs`

if [ -z "$SRC_DBS" ]; then
   echo "ERRORMSG:Instance $DB2INSTNAME do not have any databases registered!"
   exit 1
fi

#DUMPSCHEDULE
#Find dump type
if [ "$DUMPSCHEDULE" = "F" ]; then
     DUMPSCHEDULE="F"
elif [ -z "$DUMPSCHEDULE" ]; then
     DUMPSCHEDULE="I"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
     DUMPSCHEDULE="I"
else
     weekday=`date +%w`
     DUMPSCHEDULE=`expr substr  $DUMPSCHEDULE $weekday 1`
fi

#DBSIZE
DBSIZE_Total=0

#Create backupstatus.xml.pre to trace individual db predump status (success or fail)
if [ ! -d /act/tmpdata/$JOBID ] && [ "$IGNOREPARTIALFAILURE" = "TRUE" ]; then
   mkdir -p /act/tmpdata/$JOBID
fi
DUMP_STATUS_XML_pre=/act/tmpdata/$JOBID/backupstatus.xml.pre
SOME_BKP_SUCCESS=FALSE

#DB list checking in Target 
for dbname in $(echo $DATABASE_LIST | tr "," " ")
do
      dbstatus=`su $OSUSER <<EOF | grep -v "selected"|xargs
      db2 connect to $dbname>/dev/null
      db2 -x "select db_status FROM SYSIBMADM.SNAPDB"
EOF`

   if [ "$dbstatus" != "ACTIVE" ]; then
      if [ "$IGNOREPARTIALFAILURE" = "FALSE" ]; then
         echo "ERRORMSG: $dbname is not in active state. Please check!"
         exit 1
      else
         echo "WARNINGMSG: $dbname is not in active state. Skipping backup for this database."
         echo "$dbname _DB not in active state" >> $DUMP_STATUS_XML_pre
         continue
      fi
   fi

  SOURCE_DB=`echo $dbname | awk -F":" '{print $1}'`
  DBSIZE=`su  $OSUSER -c "db2 connect to $dbname > /dev/null;db2 'call get_dbsize_info(?,?,?,-1)';echo" |grep "Parameter Value"|awk 'NR==2{print $NF}'`
  DBSIZE_Total=`expr $DBSIZE_Total "+" $DBSIZE`
  CHECK_SOURCE_DB=`su  $OSUSER -c "db2 list db directory" |  awk -F"=" '/Database name/{print $2}' |grep -w $SOURCE_DB|xargs`
  if [  -z "$CHECK_SOURCE_DB" ]; then
     if [ "$IGNOREPARTIALFAILURE" = "FALSE" ]; then
        echo "ERRORMSG:Source Database $CHECK_SOURCE_DB is not registered with instance!. Please check!"
        exit 1
     else
        echo "WARNINGMSG: $dbname is not registered with instance!. Skipping backup for this database."
        echo "$dbname _DB not registered with instance" >> $DUMP_STATUS_XML_pre
        continue
     fi
  fi
  TRACK_MODE=`su  $OSUSER -c "db2 get db cfg for $dbname | grep -i TRACKMOD"`
  TRACK_MODE=`echo $TRACK_MODE | awk -F"=" '{print $2}'|xargs`

  if [ "$TRACK_MODE" = "NO" ]; then
      echo "WARNINGMSG: Incremental Backups cannot be taken as TRACKMOD is not set for $dbname!"
      DUMPSCHEDULE=F
  fi
  logarch_status=`su  $OSUSER -c "db2 get db cfg for $dbname" | grep LOGARCHMETH1`
  logarch_status=`echo $logarch_status | awk -F"=" '{print $2}' | xargs`
  if [ "$logarch_status" = "" ] || [ "$logarch_status" = "OFF" ]; then
     if [ "$IGNOREPARTIALFAILURE" = "FALSE" ]; then
        echo "ERRORMSG: $dbname LOGARCHMETH1 is not configured "
        exit 1
     else
        echo "WARNINGMSG: $dbname LOGARCHMETH1 is not configured. Skipping backup for this database."
        echo "$dbname _LOGARCHMETH1 not configured" >> $DUMP_STATUS_XML_pre
        continue
     fi
  fi

  SOME_BKP_SUCCESS=TRUE
done   

if [ "$SOME_BKP_SUCCESS" = "FALSE" ]; then
   echo "ERRORMSG: Pre-Dump backup failed for all selected databases."
   if [ -e "$DUMP_STATUS_XML_pre" ]; then
      rm -f $DUMP_STATUS_XML_pre
   fi
   exit 1
fi

echo "DBDUMPSIZE=$DBSIZE_Total"
echo "DBDUMPTYPE=$DUMPSCHEDULE"
exit 0
