#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

PWDS=/act/custom_apps/db2

DB2INSTNAME=$DB2_INSTANCE_NAME
DATABASE_LIST=$DB_DATABASE_LIST
TARGET_MNT=$DUMPBKPLOC
OSUSER=$DB2INSTNAME
DB_LIST=`echo $DATABASE_LIST | tr [:lower:] [:upper:]`
DUMPSCHEDULE=$DUMPSCHEDULE
IGNOREPARTIALFAILURE=$IGNOREPARTIALFAILURE
PREVDBFULLDUMPSAMEDAY=$PREVDBFULLDUMPSAMEDAY

function removeFile {
   if [ -e "$1" ]; then
      rm -f $1
   fi
}

function backup_db
{
  BACKUP_CMD=$1
  su  $OSUSER <<EOF
  db2 terminate
  db2 connect to $dbname
  db2 backup db $dbname online $BACKUP_CMD to $TARGET_MNT_TMP without prompting
EOF
  retval=$?

  if [ "$retval" -gt 0 ]; then
     if [ "$IGNOREPARTIALFAILURE" = "FALSE" ]; then
        echo "ERRORMSG: failed to backup database $dbname, Please check database logs for details!"
        removeFile $DUMP_STATUS_XML_pre
        removeFile $DUMP_STATUS_XML
        exit 1
     fi
     echo "WARNINGMSG: Failed to backup database $dbname, Please check database logs for details!"
     return 1
     BACKUP_STATUS=Failed
  fi

  END_TIME=`date +"%d:%m:%Y %H:%M:%S"`
  BACKUP_STATUS=Successfull
}

#Instance status

find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2INSTNAME | wc -l)
                  
if [[ $find_inst -ne 1 ]] ; then
   echo " ERRORMSG: Instance is not running "
   exit 1
fi

#db2 Instance home

if [ "$DUMPSCHEDULE" = "F" ]; then
  DUMPSCHEDULE="F"
elif [ -z "$DUMPSCHEDULE" ]; then
  DUMPSCHEDULE="I"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
  DUMPSCHEDULE="I"
else
  weekday=`date +%w`
  DUMPSCHEDULE=`expr substr  $DUMPSCHEDULE $weekday 1`
fi

#Instance Home
#identify db2_home
   proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

 if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#OLDIFS=IFS
#IFS=','

#Create backupstatus.xml to tracke individual db backup status (success or fail)
if [ ! -d /act/tmpdata/$JOBID ]; then
  mkdir -p /act/tmpdata/$JOBID
fi
DUMP_STATUS_XML=/act/tmpdata/$JOBID/backupstatus.xml
DUMP_STATUS_XML_pre=/act/tmpdata/$JOBID/backupstatus.xml.pre
echo "<tenants>" > $DUMP_STATUS_XML

#Flags to trace overall backup
ALL_BKP_SUCCESS=TRUE
SOME_BKP_SUCCESS=FALSE

# Call backup for databases one by one
for dbname in $(echo $DB_LIST| tr "," " ")
do
  isPreDumpFailed=`grep ^"$dbname " $DUMP_STATUS_XML_pre`
  if [ ! -z "$isPreDumpFailed" ] ; then
     preError=`echo "$isPreDumpFailed" | cut -d_ -f2`
     echo "<tenant name=\"$dbname\" status=\"failed\" error=\"$preError\" />" >> $DUMP_STATUS_XML
     ALL_BKP_SUCCESS=FALSE
     continue
  fi

  #Check db active or not
  dbstatus=`su $OSUSER <<EOF
  db2 connect to $dbname>/dev/null
  db2 -x "select db_status FROM SYSIBMADM.SNAPDB"
EOF`

  dbstatus=`echo $dbstatus | xargs`
  if [ "$dbstatus" != "ACTIVE" ];then
    echo "WARNINGMSG: Database:$dbname  is not in active mode ,Please check log for more details"
    echo "<tenant name=\"$dbname\" status=\"failed\" error=\"$dbname is not in active mode \" />" >> $DUMP_STATUS_XML
    ALL_BKP_SUCCESS=FALSE
    continue
  fi

  db_part_nums=`awk '{print $1}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -nr`

  #creating database backup directories on $TARGET_MNT
  if [ ! -d $TARGET_MNT/$dbname ]; then
     mkdir -p $TARGET_MNT/$dbname
  fi
  TARGET_MNT_TMP=$TARGET_MNT/$dbname

  TRACK_MODE=`su  $OSUSER -c "db2 get db cfg for $dbname | grep -i TRACKMOD"`
  TRACK_MODE=`echo $TRACK_MODE | awk -F"=" '{print $2}'|xargs`
  if [ "$TRACK_MODE" = "NO" ]; then
      echo "WARNINGMSG: Incremental Backups cannot be taken as TRACKMOD is not set for $dbname!"
      DUMPSCHEDULE=F
  fi
 
  CHECK_FULL_BKP=`ls $TARGET_MNT_TMP/ |wc -l` 
  if [ "$CHECK_FULL_BKP" -eq 0 ]; then
     DUMPSCHEDULE=F
  fi

  #Change ownership of new loaction
  OSGROUP=`id -gnr $OSUSER`
  chown -R $OSUSER:$OSGROUP $TARGET_MNT
  su  $OSUSER <<EOF
  db2 terminate
  db2 connect to $dbname
  db2 archive log for database $dbname
EOF
  backupFlag=TRUE
  if [ "$DUMPSCHEDULE" = "F" ]; then
     for DB_PART_NUM in $db_part_nums
     do
       export DB2NODE=$DB_PART_NUM
       echo "********Starting Full backup...!********"
       backup_db
       if [ $? -ne 0 ]; then
         backupFlag=FALSE
	 break
       fi
     done
  elif [ "$DUMPSCHEDULE" = "I" ]; then
       for DB_PART_NUM in $db_part_nums
       do
         export DB2NODE=$DB_PART_NUM
         echo "********Starting Incremental backup...!********"
         backup_db incremental
         if [ $? -ne 0 ]; then
           backupFlag=FALSE
	   break
	 fi
       done
  fi

  if [ "$backupFlag" = "FALSE" ]; then
    echo "<tenant name=\"$dbname\" status=\"failed\" error=\"db2 backup db command failed\"/>" >> $DUMP_STATUS_XML
    ALL_BKP_SUCCESS=FALSE
    continue
  fi

  if [ -z $BACKUP_CMD ]; then
     BACKUP_TYPE=FULL
  else
     BACKUP_TYPE=INCR
  fi
     echo "$JOBID:$DB2INSTNAME:DBNAME $dbname:BACKUP_TYPE $BACKUP_TYPE:BACKUP_STATUS $BACKUP_STATUS :END_TIME $END_TIME"
#Backup Status
    BACKUPFILE=`ls -ltr $TARGET_MNT_TMP | tail -1|awk '{print $NF}'`
    echo "*****  Database Backup Status****** "
    su $OSUSER -c "db2ckbkp -h $TARGET_MNT_TMP/$BACKUPFILE"

  #As backup success, update DUMP_STATUS_XML and flag SOME_BKP_SUCCESS
  echo "<tenant name=\"$dbname\" status=\"success\" />" >> $DUMP_STATUS_XML
  SOME_BKP_SUCCESS=TRUE

done

echo "</tenants>" >> $DUMP_STATUS_XML
removeFile $DUMP_STATUS_XML_pre

if [ "$SOME_BKP_SUCCESS" = "FALSE" ]; then
  echo "ERRORMSG: Dump backup failed for all selected databases."
  exit 1
fi

if [ "$ALL_BKP_SUCCESS" = "FALSE" ]; then
  echo "WARNINGMSG: Some of the database backups failed."
else
  echo "INFOMSG: Dump backup successful for all the selected databases."
fi

exit 0
