#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

ACT_JOBNAME=$JOBID
TARGET_INSTANCE=$SOURCE_INSTANCE
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST
OSUSER=$SOURCE_INSTANCE
ARCHIVELOG_LOC=$ARCHIVELOG_LOC

#-------------------------------------------------------------------
# function: isVarNull()
#-------------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    echo "usage : ./ACT_DB2_lvm_migrate_newtarget.sh SOURCE_INSTANCE=<SOURCE_INSTANCE> TARGET_DBNAME_LIST=<comma seperated DB LIST> JOBID=Job_4438046 DATAVOL_DISK_MAPPING=<Actifio Mountpoint>:<prod equivalent lvm device name>  ARCHIVELOG_LOC=<PROD Archivelog location>"
    exit 1
  fi
}
#-------------------------------------------------------------------
#Pre-checks
isVarNull OSUSER $OSUSER
isVarNull TARGET_DBNAME_LIST $TARGET_DBNAME_LIST
isVarNull SOURCE_INSTANCE $SOURCE_INSTANCE
isVarNull ARCHIVELOG_LOC  $ARCHIVELOG_LOC

export OSUSER
export SOURCE_INSTANCE
TARGET_DBNAME_LIST=`echo $TARGET_DBNAME_LIST |sed 's/::/,/g'`
TARGET_DBNAME_LIST=`echo $TARGET_DBNAME_LIST|tr '[:lower:]' '[:upper:]'`
set -x
DATAVOL_DISK_MAPPING=$DATAVOL_DISK_MAPPING
ACT_JOBNAME=$JOBID


#Restrict UnMount if pvmove is in progress
DB2_MIGRATE_CONF=/act/touch/.ACT_DB2_migrate.conf

#-------------------------------------------------------------------
#Prepare tmp location for temp files
#-------------------------------------------------------------------
tmp_loc=/act/tmpdata/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
  mkdir -p $tmp_loc
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $tmp_loc


#find instance Home

#identify db2_home
   proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z $proc_id ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi



#-------------------------------------------------------------------
call_commentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -e "/$devicename/ s/^#*/#/" -i /etc/fstab
}

#-------------------------------------------------------------------
call_uncommentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -i "/$devicename/s/^#//g" /etc/fstab
}

call_uncommentfstabs()
{
  for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
  do
    PROD_VOL=`echo $datavolMap|cut -d: -f2`
    devicename=`echo $PROD_VOL |sed 's|\/|\\\/|g'`
    sed -i "/$devicename/s/^#//g" /etc/fstab
  done
}

#-------------------------------------------------------------------
remove_file()
{
  local fname=$1
  if [ -f $fname ]; then
    rm -f $fname
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: startDB2()
#-------------------------------------------------------------------
chown -R $OSUSER:$OSGROUP $ARCHIVELOG_LOC
startDB2() {
su $OSUSER -c "db2start"
for dbName in $(echo $TARGET_DBNAME_LIST | sed "s/,/ /g" )
do
 
 su - $OSUSER -c "db2relocatedb -f /act/touch/act_db2_relocate_"$ACT_JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_$dbName.cfg"
 if [ $? -ne 0 ];then
     echo "ERRORMSG: DB2 Database $dbName Relocation is failed, Please check logs for more details!"
     call_uncommentfstabs
     exit 1;
 else
     echo "continue"
     su  $OSUSER -c "db2 update db cfg for $dbName using LOGARCHMETH1 DISK:$ARCHIVELOG_LOC"
 fi
 done 
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1
  local PROD_VG_NAME=$2
  echo "INFOMSG: Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     call_uncommentfstabs
     exit 1
  fi

  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     call_uncommentfstabs
     exit 1
  fi

  #Update LV_MERGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MERGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "INFOMSG: Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"

  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     call_uncommentfstabs
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1
  isMntd=`grep -w $mntpnt /proc/mounts | uniq | wc -l`
  if [ $isMntd -ne 0 ]; then
    echo "Before unmount: "`date`
   for i in {1..5} 
   do
    umount $mntpnt
    systemctl stop $mntpnt
    systemctl daemon-reload
    grep " $mntpnt " /proc/mounts 
    if [ $? -eq 0 ]; then
      #grep " $mntpnt " /proc/mounts 
      if [ $i -eq 5 ]; then
        #echo "INFOMSG: Mountpoint is Already unmounted"
        echo "****Checking the fuser process****"
        fuser -c -u $mntpnt
        echo "ERRORMSG: Fail to unmount $mntpnt after retries"
        mount -a
        call_uncommentfstabs
        exit 1
      fi
      echo "ERRORMSG: Fail to unmount $mntpnt. Retrying"
      sleep 5
    else
      echo "$mntpt is unmounted"
      break
    fi
    done
  fi
  echo "INFOMSG: Successfully unmounted $mntpnt"
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
#Prepare temp files
#-------------------------------------------------------------------
LV_MERGE_LIST=$tmp_loc/LV_MERGE_LIST_$ACT_JOBNAME.txt
VG_MERGE_LIST=$tmp_loc/VG_MERGE_LIST_$ACT_JOBNAME.txt
PV_MERGE_LIST=$tmp_loc/PV_MERGE_LIST_$ACT_JOBNAME.txt
PV_MERGE_LIST_final=$tmp_loc/PV_MERGE_LIST_final_$ACT_JOBNAME.txt
MT_MERGE_LIST=$tmp_loc/MT_MERGE_LIST_$ACT_JOBNAME.txt
NST_MNTPNTS=$tmp_loc/NST_MNTPNTS_$ACT_JOBNAME.txt
NST_MNTPNTS_toMount=$tmp_loc/NST_MNTPNTS_toMount_$ACT_JOBNAME.txt
UNMNT_SORT_LIST=$tmp_loc/UNMNT_SORT_LIST_$ACT_JOBNAME.txt
MNT_SORT_LIST=$tmp_loc/MNT_SORT_LIST_$ACT_JOBNAME.txt
remove_file $LV_MERGE_LIST
remove_file $VG_MERGE_LIST
remove_file $PV_MERGE_LIST
remove_file $PV_MERGE_LIST_final
remove_file $UNMNT_SORT_LIST
remove_file $MNT_SORT_LIST
remove_file $MT_MERGE_LIST
remove_file $NST_MNTPNTS
remove_file $NST_MNTPNTS_toMount
remove_file $NST_MNTPNTS_toMount.tmp
#-------------------------------------------------------------------


DATAVOL_DISK_MAPPING=`echo $DATAVOL_DISK_MAPPING | sed 's/::/,/g'`
#Prepare files conntents and also check lvm size
#---------------------------------------------------------------------------------------------------------
echo "Creating Merge list......!!"
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  call_commentfstab $PROD_VOL
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    call_uncommentfstabs
    exit 1
  fi

  #Get LV paths
  echo "$ACT_LV_PATH $PROD_LV_PATH"  >> $LV_MERGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MERGE_LIST

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  for ACT_PV_DEV in $(echo $ACT_PV_DEV_LIST| sed 's/,/ /g'); do
    echo "$ACT_PV_DEV $PROD_PV_DEV_LIST" >> $PV_MERGE_LIST
  done

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
     echo "ERRORMSG: unable to find production volume $PROD_VOL !Please check logs for more details"
     call_uncommentfstabs
     exit 1
    PROD_MNT=$ACT_MNT
  fi
  echo "$ACT_MNT $PROD_VOL $PROD_MNT" >> $MT_MERGE_LIST
done
 
#Creating Unmount Sort List####################
cat $MT_MERGE_LIST | awk '{print $1}' >> $UNMNT_SORT_LIST 
cat $MT_MERGE_LIST | awk '{print $3}' >> $UNMNT_SORT_LIST 
cat $UNMNT_SORT_LIST | sort -r > $UNMNT_SORT_LIST.tmp
mv $UNMNT_SORT_LIST.tmp $UNMNT_SORT_LIST

#Create mount sort list
cat $MT_MERGE_LIST | awk '{print $3}' | sort > $MNT_SORT_LIST

NSTMNT_CHECK=`cat /act/touch/*"$ACT_JOBNAME"_params | grep "NESTEDVOLUME_UNMOUNT" |tail -1| cut -d"=" -f2`

#====================
#Find additional mounts
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt1/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt1/"`
  echo "$nstMnt" >> $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt2/"`
 # nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt2/"`
  echo "$nstMnt" >> $NST_MNTPNTS
done < $MT_MERGE_LIST

#Sort, uniq
cat $NST_MNTPNTS | sort -r | uniq > $NST_MNTPNTS.tmp
mv $NST_MNTPNTS.tmp $NST_MNTPNTS

#Remove existing
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  grep -vw "$umnt1\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  grep -vw "$umnt2\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS
done < $MT_MERGE_LIST

echo "INFOMSG: Additional mountpoints to unmount"
cat $NST_MNTPNTS

#Update NST_MNTPNTS_toMount with mount command
cat $NST_MNTPNTS | sort > $NST_MNTPNTS_toMount
while read line
do
  if [ -z $line ]; then
    continue
  fi
  mntDev=`grep " $line " /proc/mounts | awk '{print $1}' | uniq`
  echo "mount $mntDev $line" >> $NST_MNTPNTS_toMount.tmp
done < $NST_MNTPNTS_toMount
mv $NST_MNTPNTS_toMount.tmp $NST_MNTPNTS_toMount
cat $NST_MNTPNTS_toMount

# Find one of the target dbname (OFTDN) to make unique key with TARGET_INSTANCE and OFTDN
OFTDN=`cat /act/touch/OFTDN_"$ACT_JOBNAME" | cut -d= -f2`
rm -f /act/touch/OFTDN_"$JOBNAME"

for dbName in $(echo $TARGET_DBNAME_LIST | sed "s/,/ /g" )
do 
    RELOCATEDB=/act/touch/act_db2_relocate_"$ACT_JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$dbName".cfg
    RELOCATEDBNEW=/act/touch/act_db2_relocate_"$ACT_JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$dbName".mig
    echo "DB_NAME=$dbName" > $RELOCATEDBNEW
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    call_uncommentfstabs
    exit 1
  fi

  #Get LV paths

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
     echo "ERRORMSG: unable to find production volume $PROD_VOL !Please check logs for more details"
     call_uncommentfstabs
     exit 1
    PROD_MNT=$ACT_MNT
  fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Preparing mearge files completed"
#---------------------------------------------------------------------------------------------------------
#Configuration file creation
#----------------------------------------------------------------------------------------------------------
    PRODMNTSED=`echo $PROD_MNT`
    ACTMNTSED=`echo $ACT_MNT`
    DBP=`cat $RELOCATEDB | grep DB_PATH | cut -d "," -f2`
    DBP_ACT_MNT_EXISTS=`echo $DBP | grep -w "$ACTMNTSED" | wc -l`
    if [ "$DBP_ACT_MNT_EXISTS" -gt 0 ]; then
       DBPTGT=`echo $DBP | sed "s:$ACTMNTSED:$PRODMNTSED:g" `
       echo "DB_PATH=$DBP,$DBPTGT" >> $RELOCATEDBNEW
       echo "INSTANCE=$SOURCE_INSTANCE" >> $RELOCATEDBNEW
       echo "NODENUM=0" >> $RELOCATEDBNEW
    fi
    STORAGE_PATHS=`cat $RELOCATEDB | grep STORAGE_PATH | cut -d "," -f2 |xargs`
    for pathname in $(echo $STORAGE_PATHS)
    do
        if [  -z "$STORAGE_PATHS" ] ;then
            break
        fi
        STP=$pathname
        STP_ACT_MNT_EXISTS=`echo $STP | grep -w "$ACTMNTSED" | wc -l`
        if [ "$STP_ACT_MNT_EXISTS" -gt 0 ]; then
           STPTGT=`echo $STP | sed "s:$ACTMNTSED:$PRODMNTSED:g" `
           echo "STORAGE_PATH=$STP,$STPTGT" >> $RELOCATEDBNEW
        fi
    done
    CONTAINER_PATHS=`cat $RELOCATEDB | grep CONT_PATH | cut -d "," -f2 |xargs`
    for cntpaths in $(echo $CONTAINER_PATHS)
    do
        if [  -z "$CONTAINER_PATHS" ] ;then
            break
        fi
      CTP=$cntpaths
      CTP_ACT_MNT_EXISTS=`echo $CTP | grep -w "$ACTMNTSED" | wc -l`
      if [ "$CTP_ACT_MNT_EXISTS" -gt 0 ]; then
         CTPTGT=`echo $CTP | sed "s:$ACTMNTSED:$PRODMNTSED:g" `
         echo "CONT_PATH=$CTP,$CTPTGT" >> $RELOCATEDBNEW
      fi
    done

    MIGLOG=`cat $RELOCATEDB | grep MIRRORLOG_PATH | cut -d "," -f2`
    if [ ! -z "$MIGLOG" ] ; then
    MIGTMP=$MIGLOG
    MIGLOG_ACT_MNT_EXISTS=`echo $MIGLOG | grep -w "$ACTMNTSED" | wc -l `
       if [ "$MIGLOG_ACT_MNT_EXISTS" -gt 0 ]; then
           MIGTGT=`echo $MIGLOG | sed "s:$ACTMNTSED:$PRODMNTSED:g"`
           echo "MIRRORLOG_PATH=$MIGTMP,$MIGTGT" >> $RELOCATEDBNEW
       fi
    fi
    LOG=`cat $RELOCATEDB | grep LOG_DIR | cut -d "," -f2`
    LOGTMP=$LOG
    LOGTMP_ACT_MNT_EXISTS=`echo $LOG | grep -w "$ACTMNTSED" | wc -l `
    if [ "$LOGTMP_ACT_MNT_EXISTS" -gt 0 ]; then
       LOGTGT=`echo $LOG | sed "s:$ACTMNTSED:$PRODMNTSED:g"`
       echo "LOG_DIR=$LOGTMP,$LOGTGT" >> $RELOCATEDBNEW
    fi
done
    cat $RELOCATEDBNEW | sort | uniq > "$RELOCATEDBNEW".tmp
    mv "$RELOCATEDBNEW".tmp /act/touch/act_db2_relocate_"$ACT_JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$dbName".cfg
    chmod 777 /act/touch/act_db2_relocate_"$ACT_JOBNAME"_"$TARGET_INSTANCE"_"$OFTDN"_"$dbName".cfg
done
#Stop application for unmounting actifio and source mount points
#-------------------------------------------------------------------
for dbname in $(echo $TARGET_DBNAME_LIST| tr "," " ")
  do
     su   $OSUSER -c "db2 terminate"
     su   $OSUSER -c "db2 force application all"
     su   $OSUSER -c "db2 deactivate database $dbname"
     su   $OSUSER -c "db2 uncatalog database $dbname"
     if [ $? -ne 0 ]; then
         echo "ERRORMSG: failed at stoping application access to source/actifio mount points"
         call_uncommentfstabs
         exit 1
     fi
   done
   su $OSUSER -c "db2stop force"

#-------------------------------------------------------------------


#Update hidden migrate config file to handel unmount+delete
if [ ! -f $DB2_MIGRATE_CONF ]; then
  echo -e "MIGRATED=NO" > $DB2_MIGRATE_CONF
fi

echo "INFOMSG: Actifio CBT stoping..."
call_stop_tracking
echo "INFOMSG: Actifio CBT has been stopped successfully"

#Unmount additional mounts
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  #unmntpnt=`echo $line | awk '{print $3}'`
  unmntpnt=`echo $line | awk '{print $1}'`
  if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
    unmount_mnt $unmntpnt
  else
    echo "ERRORMSG: Found nested mount points ($unmntpnt). cannot proceed..."
    call_uncommentfstabs
    exit 1
  fi
done < $NST_MNTPNTS

#Unmount all mounts
#---------------------------------------------------------------------------------------------------------
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  #umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  #unmount_mnt $umnt2
done < $UNMNT_SORT_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Unmounted all required mount points"

#LV rename to get diff lv names
#---------------------------------------------------------------------------------------------------------
cnt=100
remove_file $LV_MERGE_LIST.tmp
while read line
do
  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "INFOMSG: Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MERGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MERGE_LIST
mv $LV_MERGE_LIST.tmp $LV_MERGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Renaming actifio lvm names done (temparory)"


#VG mearge
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MERGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: VG mearge completed....."


#LV rename and update
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MERGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: LV rename completed....."


#Mount devices
#---------------------------------------------------------------------------------------------------------
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  #mntP=`echo $line| awk '{print $3}'`
  #mntD=`echo $line | awk '{print $2}'`

  mntP=$line
  mntD=`grep " $line$" $MT_MERGE_LIST | awk '{print $2}'`

  mount -o rw,nouuid $mntD $mntP
  if [ $? -ne 0 ]; then
    mount -o rw $mntD $mntP
    if [ $? -ne 0 ]; then
       echo "INFOMSG: Device Already Mounted to $mntP"
      #echo "ERRORMSG: Fail to mount $mntD to $mntP"
      #exit 1
    fi
  fi
done < $MNT_SORT_LIST

if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
  echo " INFOMSG: Mounting the Additional Mountpoints...."
  #Mount additional mounts
  while read line
  do
    addMntPnt=`echo $line | awk '{print $3}'`
    if [ ! -d "$addMntPnt" ]; then
      mkdir -p $addMntPnt
    fi
    $line
  done < $NST_MNTPNTS_toMount
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Mounting mount points completed....."


#Call AppAware mount script
#---------------------------------------------------------------------------------------------------------
startDB2
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to mount databases with required mount points"
  call_uncommentfstabs
  exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Application started successfully"

for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  call_uncommentfstab $PROD_VOL
done

#Prepare pvmove final list
#---------------------------------------------------------------------------------------------------------
while read line
do
  #Skip empty lines
  if [ -z "$line" ]; then
    continue
  fi

  #Read Actifio PVs
  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  ACT_PV_NAME=`echo $ACT_PV_NAME | sed 's/,/ /g'`

  #Read Production PVs
  PROD_PV_NAME=`echo $line | awk '{print $2}'`
  PROD_PV_NAME=`echo $PROD_PV_NAME | sed 's/,/ /g'`

  #Prepare lines
  for ACT_PVNAME in $ACT_PV_NAME
  do
    echo "$ACT_PVNAME $PROD_PV_NAME" >> $PV_MERGE_LIST_final
  done

done < $PV_MERGE_LIST
#---------------------------------------------------------------------------------------------------------


#PV move
#---------------------------------------------------------------------------------------------------------
# Calculate pending pvmoves
NoOfPvmovesRequired=`grep "^/dev" $PV_MERGE_LIST_final | wc -l`
echo "INFOMSG: NoOfPvmovesRequired=$NoOfPvmovesRequired"

# Perfrom pvmove
while [ $NoOfPvmovesRequired -gt 0 ]
do
  # Get current line (Set of pv devices to be participate in pvmove)
  getline=`grep "^/dev" $PV_MERGE_LIST_final | head -1`

  # Check if any of the pvs are already in use
  PV_BUSY=FALSE
  for pvNameCurr in $getline
  do
    echo $pvNameCurr
    isPVinUse=`lvs -a -o+devices | grep pvmove | awk '{print $5}' | grep "/dev/sd" | cut -d "(" -f1 | sort | uniq | grep -w $pvNameCurr | wc -l`
    if [ $isPVinUse -gt 0 ]; then
      PV_BUSY=TRUE
      break
    fi
  done
  if [ "$PV_BUSY" = "TRUE" ]; then
    echo "INFOMSG: $pvNameCurr is in use ($getline)"

    # Remove this line
    tmpgetline=`echo $getline | sed 's./.\\\/.g'`
    sed -i "/$tmpgetline/d" $PV_MERGE_LIST_final

    # Write this line at the end for next time
    echo $getline >> $PV_MERGE_LIST_final

    # Display pending
    echo "INFOMSG: Pending pvmoves"
    cat $PV_MERGE_LIST_final

    sleep 30
    continue
  fi

  # Perform pvmove
  echo "INFOMSG: Moving Physical Volumes: $getline"
  echo "Command: pvmove --atomic -A y -b -q $getline"
  pvmove --atomic -A y -b -q $getline
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: pvmove command failes (for $getline)"
    pvmove --abort
    call_uncommentfstabs
    exit 1
  fi
  echo "INFOMSG: pvmove initiated successfully (for $getline) .........."

  # remove current line on successful pvmove initiation
  tmpgetline=`echo $getline | sed 's./.\\\/.g'`
  sed -i "/$tmpgetline/d" $PV_MERGE_LIST_final

  # Re-calculate pending pvmoves
  NoOfPvmovesRequired=`grep "^/dev" $PV_MERGE_LIST_final | wc -l`
  echo "INFOMSG: NoOfPvmovesRequired=$NoOfPvmovesRequired"
done
#---------------------------------------------------------------------------------------------------------


echo "INFOMSG: pvmove status ......................"
#---------------------------------------------------------------------------------------------------------
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   pvmove --abort
   call_uncommentfstabs
   exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove completed successfully ....."

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    call_uncommentfstabs
    exit 1
  fi
done < $PV_MERGE_LIST
echo "Actifio pvs removed successfully .................."

#Update hidden migrate config file to handel unmount+delete
echo -e "MIGRATED=YES" > $DB2_MIGRATE_CONF

echo "INFOMSG: Starting Actifio CBT..."
call_start_tracking
echo "INFOMSG: Actifion CBT started successfully"

if [ -d "$tmp_loc" ]; then
     rm -rf $tmp_loc
fi

echo "************ Migration Operation Completed Successfully ***************"

exit 0

