#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set +x 
JOBNAME=$JOBID
DB2_INSTANCE_NAME=$SOURCE_INSTANCE
DB_LIST=$DB_LIST
RECOVERYTIME=$UNTIL_TIME
ARCHIVELOG_MNT=$ARCHIVELOG_MNT


rollforwardUsingUTC_Output=/act/touch/rollforwardUsingUTC_$JOBNAME
OSUSER=$DB2_INSTANCE_NAME

#check OSUSER
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: osuser $OSUSER not exists"
  exit 1
fi


#Start db2 instance
su - $OSUSER -c "db2start"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to start db2 instance $DB2_INSTANCE_NAME"
  exit 1
fi

 #identify db2_home
 proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#su - $OSUSER -c "db2 list db directory"

#Recover database
for dbName in $(echo $DB_LIST | sed "s/,/ /g" )
do
  CONF_FILE=/act/touch/"$dbName"_"$JOBNAME"_params
  if [ ! -f $CONF_FILE ]; then
     echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
     exit 1
  fi
  SOURCE_DB_PATH=`cat $CONF_FILE | grep -w SOURCE_DB_PATH | cut -d= -f2`

  echo "Recovering database $dbName ...."
  su  $OSUSER -c "db2 catalog database $dbName on  '$SOURCE_DB_PATH'"
  su  $OSUSER -c "db2inidb $dbName as standby"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to initialize database $dbName as standby"
    exit 1
  fi
  
  SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
  ARCHIVE_LOG_LOC=$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1
  rollfwd_status=`su  $OSUSER -c "db2 rollforward db $dbName  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`
  if [ "$rollfwd_status" = "DBpending" ]; then
     if [ -z "$RECOVERYTIME" ] && [ -z "$ARCHIVELOG_MNT" ]; then 
        su  $OSUSER -c "db2 rollforward db $dbName stop"
     elif [ ! -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su  $OSUSER -c "db2 update db cfg for $dbName  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "
        su  $OSUSER -c "db2 rollforward db $dbName  to $RECOVERYTIME using UTC time and stop" > $rollforwardUsingUTC_Output.$dbName
        rollforwardUsingUTC=`grep 'SQL[0-9][0-9][0-9][0-9][A-Z]' $rollforwardUsingUTC_Output.$dbName | grep -v "SQL4970N  Rollforward recovery stopped on the database" | wc -l`
        if [ $rollforwardUsingUTC -gt 0 ]; then
	  echo "ERRORMSG: Fail to rollforward the database $dbName to $RECOVERYTIME"
          $rollforwardUsingUTC_Output.$dbName
	  exit 1
        fi
     elif [ -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su  $OSUSER -c "db2 update db cfg for $dbName  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "
        su  $OSUSER -c "db2 rollforward db $dbName  to end of logs and stop" 
     fi
     
    su  $OSUSER -c "db2 rollforward db $dbName complete "
    if [ $? -ne 0 ] ;then
       su  $OSUSER -c "db2 rollforward db $dbName stop"
    fi
    rollfwd_status2=`su  $OSUSER -c "db2 rollforward db $dbName  query status | grep \"Rollforward status\" | cut -d= -f2 | sed 's/ //g' | xargs"`

    if [ "$rollfwd_status2" = "DBworking" ]; then
      su  $OSUSER -c "db2 rollforward db $dbName complete "
    fi  
  fi
  su  $OSUSER -c "db2 update db cfg for $dbName  using LOGARCHMETH1 DISK:$SOURCE_LOGARCHMETH1 "
  su  $OSUSER -c "db2 terminate;db2 deactivate database $dbName"
done
 
#Restarting the db2instance
su $OSUSER -c "db2stop force;db2start"

  for dbName in $(echo $DB_LIST | sed "s/,/ /g" )
  do
      dbstatus=`su  $OSUSER -c "db2 connect to $dbName>/dev/null;db2 -x \"select db_status FROM SYSIBMADM.SNAPDB\"" | grep -v "selected"|xargs`
      if [ "$dbstatus" = "ACTIVE" ]; then
          echo "Database $dbName is already activated"
      else
         su  $OSUSER -c "db2 terminate;db2 activate database $dbName "
         retval=$?
         if [ "$retval" -eq 0 ]; then
            echo "$dbName database successfully activated"
         fi
      fi
  done
  
exit 0
