#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set +x
DB2_INSTANCE_NAME=$SOURCE_INSTANCE
DB_LIST=$DB_LIST

set -x
DB2_INSTANCE_NAME=$DB2_INSTANCE_NAME 
DB_LIST=$DB_LIST
OSUSER=$DB2_INSTANCE_NAME
DB2HOME=$DB2HOME
#check OSUSER
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: osuser $OSUSER not exists"
  exit 1
fi

#Creating the /act/touch directory
if [ ! -d /act/touch ]; then
   mkdir -p /act/touch
   chmod 755 /act/touch
else
  chmod 755 /act/touch
fi


#Instance status
inst_status=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2_INSTANCE_NAME | wc -l)
if [ $inst_status -eq 0 ];then
    echo "Starting DB2 Instance "
    su - $OSUSER -c "db2start" 
    if [ $? -ne 0 ];then 
       echo " ERRORMSG: Failed to start DB2 Instance ! Please check logs for more details"
       exit 1   
    fi 
fi

#find instance Home
#identify db2_home
 proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi
 
 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

  if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi



#find instance running or not
is_DB2_Running=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2_INSTANCE_NAME | wc -l)


#Stop server if running
if [ $is_DB2_Running -eq 1 ]; then
  
  #Deactivating Databases
  for dbname in $(echo $DB_LIST| tr "," " ")
  do
    hadr_db_role=`db2 get db cfg for $dbname | awk '/HADR database role/{print toupper($5)}'`
    #db_part_nums=`awk '{print $1}' $db2home/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -nr`
    if [[ "$hadr_db_role" == "STANDBY" ]];then
        su  $OSUSER -c "db2 deactivate db $dbname"
         retval=$?
         if [ "$retval" -gt 0 ]; then
            echo "ERRORMSG: Failed to deactivate database $dbname, Please check database logs for details!"
            exit 1
         fi
    else
	#su  $OSUSER -c "db2 archive log for database $dbname"
        #for DB_PART_NUM in $db_part_nums; do
            
            #export DB2NODE=$DB_PART_NUM
            su  $OSUSER -c " db2 terminate; db2 force application all ; db2 deactivate  database $dbname "
            retval=$?
            if [ "$retval" -gt 0 ]; then
               echo "INFOMSG: Failed to deactivate database $dbname"
            fi
        #done
     fi
   done
  
  #Stop server
   su  $OSUSER -c "db2 terminate;db2 force applications all;db2stop"
   retval=$?
   if [ $retval -eq 0 ]; then
      echo "DB2 instance $DB2_INSTANCE_NAME is stopped Successfully"
   else
      su $OSUSER -c "db2_kill"
      if [ $? -ne 0]; then
          echo "ERRORMSG: Failed to stop db2 instance $DB2_INSTANCE_NAME"
          exit 1
      fi
   fi
fi

#ipclean 
su  $OSUSER -c "ipclean"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to clean db2 resources"
  exit 1
fi
