#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set +x 
#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

set -x
JOBNAME=$JOBID
DB2_INSTANCE_NAME=$SOURCE_INSTANCE
DB_NAME=$DB_NAME
RECOVERYTIME=$UNTIL_TIME
ARCHIVELOG_MNT=$ARCHIVELOG_MNT
TARGET_MNT=$TARGET_MNT 
SOURCE_INSTANCE=$DB2_INSTANCE_NAME

OSUSER=$DB2_INSTANCE_NAME
#check OSUSER
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: osuser $OSUSER not exists"
  exit 1
fi

#instance status
find_inst=$(ps -ef | grep [d]b2sys | awk '{print $1}' | grep -w $DB2_INSTANCE_NAME|uniq)
if [ "$find_inst" != "$OSUSER" ]; then
   echo "ERRORMSG: Instance: $TARGET_INSTANCE is not running on this host.Please check log For more details"
   exit 1
fi

#identify db2_home
   proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
   if [ -z "$proc_id" ];then
       echo "ERRORMSG: DB2 Instance $OSUSER is not running! Please check logs for more details"
       exit 1
   fi

 db2home=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
 INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
 DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`

   if [ -z "$db2home" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       db2home=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       db2home="$db2home"/sqllib
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  elif [ -f "$INSTHOME/sqllib/db2profile" ] ;then
       db2home="$INSTHOME"/sqllib
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
  elif [ ! -z "$DB2LIB" ]; then
       db2home=`dirname $DB2LIB`
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  else
       if [ -f "$db2home/db2profile" ]; then
       set +x
       . "$db2home"/db2profile
       set -x
       db2bin=$db2home"/bin"
       fi
  fi

#MAIN

     dbName=$DB_NAME
     CONF_FILE=/act/touch/"$dbName"_"$JOBNAME"_params
     if [ ! -f $CONF_FILE ]; then
        echo "ERRORMSG: Configuration  file $CONF_FILE is not created!"
        exit 1
     fi

     SOURCE_DB_NAME=$dbName
     SOURCE_DB_PATH=`cat $CONF_FILE | grep SOURCE_DB_PATH | cut -d= -f2`
     OSUSER=$SOURCE_INSTANCE
     OSGROUP=`id -ngr $OSUSER`
     #creating directories for new dbpath
     PATH_TEMP="$TARGET_MNT$SOURCE_DB_PATH"
     chown -R $OSUSER:$OSGROUP $TARGET_MNT
     dbDir=`su $SU_FLAG $OSUSER -c "db2 list db directory on $PATH_TEMP"| grep -v "Database alias" | awk "/$SOURCE_DB_NAME/{getline;print}" |xargs | awk '{print $NF}'`
     sqldir=`find $PATH_TEMP -name "$dbDir" -type d |grep $SOURCE_INSTANCE |xargs`
     dirtmp=`dirname $sqldir`
     NODENUM=`basename $dirtmp`
     if [ ! -z "$SOURCE_DB_PATH/$SOURCE_INSTANCE/$NODENUM$sqldir" ]; then
         rm -rf  $SOURCE_DB_PATH/$SOURCE_INSTANCE/$NODENUM$sqldir
     fi
     cp -r -p $sqldir $SOURCE_DB_PATH/$SOURCE_INSTANCE/$NODENUM/
    #cp -r $dirtmp/sqldbdir $SOURCE_DB_PATH/$SOURCE_INSTANCE/$NODENUM/

     SOURCE_LOG_DIR=`cat $CONF_FILE | grep SOURCE_LOG_DIR | cut -d= -f2`
     SOURCE_LOG_DIR_ORIG=$SOURCE_LOG_DIR
     SOURCE_LOG_DIR=`echo $SOURCE_LOG_DIR |awk -F"/NODE" '{print $1}'`
     SOURCE_STORAGE_PATH=`cat $CONF_FILE | grep SOURCE_STORAGE_PATH | cut -d= -f2`

     for pathname in $(echo $SOURCE_STORAGE_PATH | tr "," " ")
     do
        if [ ! -z "$pathname/$SOURCE_INSTANCE/$NODENUM/$SOURCE_DB_NAME" ];then
            rm -rf $pathname/$SOURCE_INSTANCE/$NODENUM/$SOURCE_DB_NAME
        fi
        cp -r -p $TARGET_MNT$pathname/$SOURCE_INSTANCE/$NODENUM/$SOURCE_DB_NAME $pathname/$SOURCE_INSTANCE/$NODENUM/
     done
     #rm -rf $SOURCE_LOG_DIR/$NODENUM
     #cp -r -p  $TARGET_MNT$SOURCE_LOG_DIR/$NODENUM $SOURCE_LOG_DIR/
       

#Recover database
  echo "Recovering database $dbName ...."
  su - $OSUSER -c "db2 catalog database $dbName on  '$SOURCE_DB_PATH'"
  su - $OSUSER -c "db2inidb $dbName as standby"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to initialize database $dbName as standby"
    exit 1
  fi
 
 
  SOURCE_LOGARCHMETH1=`cat $CONF_FILE | grep SOURCE_LOGARCHMETH1 | cut -d= -f2`
  ARCHIVE_LOG_LOC=$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1
  LOGPATH=`su  $OSUSER -c "db2 get db cfg for $SOURCE_DB_NAME | grep \"Path to log files\"| cut -d= -f2 | xargs"`
  rollfwd_status=`su - $OSUSER -c "db2 rollforward db $dbName  query status" | grep "Rollforward status" | cut -d "=" -f2 | sed 's/ //g' | xargs`
  chown -R $OSUSER:$OSGROUP $ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1
  su - $OSUSER -c "db2 update db cfg for $dbName  using LOGARCHMETH1 DISK:$ARCHIVELOG_MNT$SOURCE_LOGARCHMETH1 "
  if [ "$rollfwd_status" = "DBpending" ]; then
     if [ -z "$RECOVERYTIME" ] && [ -z "$ARCHIVELOG_MNT" ]; then 
        cp -n -p  $TARGET_MNT$SOURCE_LOG_DIR_ORIG/*.LOG "$LOGPATH"
        chown -R $OSUSER:$OSGROUP "$LOGPATH"
        su - $OSUSER -c "db2 rollforward db $dbName stop"
     elif [ ! -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su - $OSUSER -c "db2 rollforward db $dbName  to $RECOVERYTIME using local time and stop" 
     elif [ -z "$RECOVERYTIME" ] && [ ! -z "$ARCHIVELOG_MNT" ]; then
        su - $OSUSER -c "db2 rollforward db $dbName  to end of logs and stop" 
     fi

    rollfwd_status2=`su - $OSUSER -c "db2 rollforward db $dbName  query status" | grep "Rollforward status" | cut -d "=" -f2 | sed 's/ //g' | xargs`

    if [ "$rollfwd_status2" = "DBpending" ]; then
      su - $OSUSER -c "db2 rollforward db $dbName complete "
    fi  
  fi
  dbstatus=`su - $OSUSER -c "db2 connect to $dbName>/dev/null;db2 -x \"select db_status FROM SYSIBMADM.SNAPDB\"" | grep -v "selected"|xargs`
 su - $OSUSER -c "db2 update db cfg for $dbName  using LOGARCHMETH1 DISK:$SOURCE_LOGARCHMETH1 "
  if [ "$dbstatus" = "ACTIVE" ]; then
     echo "Database $dbName is already activated"
  else
     su - $OSUSER -c "db2 activate database $dbName "
     retval=$?
     if [ "$retval" -eq 0 ]; then
        echo "$dbName database successfully activated"
     fi
  fi

exit 0
