#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
#
# DB2 script for discovery
#


set -x
unset LANG
unset LC_ALL
export LANG=C
export LC_ALL=C


# Globals
AUTHID_STR="SQL authorization ID"
LOGARCHMETH_NAME="LOGARCHMETH1"
LOGARCH_METH2_PATH=""
MIRROR_LOG_PATH=""
STORAGE_PATHS=""
ARCHIVELOG_PATH=""
SHALLOW_DISCOVERY=$SHALLOW_DISCOVERY

OS_TYPE=`uname -s`
if [ "$OS_TYPE" == "AIX" ]; then
  SU_FLAG=""
  ECHO_E="echo "
elif [ "$OS_TYPE" == "Linux" ]; then
  SU_FLAG=""
  ECHO_E="echo -e "
fi

#XML
XMLFILE="/act/custom_apps/DB2.xml"
SCRIPTS="/act/custom_apps/db2/CustomApp_DB2.sh"
SRC_SERVER_NAME=`ps -ef | grep [d]b2sys | awk '{print $1}'`
FTYPE="DB2Instance"
DEV_TYPE="datavol"
LOG_TYPE="logvol"
TBS_TYPE="tbspvol"
VERSION=" "

#. /home/$OSUSER/sqllib/db2profile
DB2_tablespaceDetailsFile=/act/touch/DB2_tablespaceDetailsFile
echo > $DB2_tablespaceDetailsFile

shallow_discovery() {
set -x
    local dbinst=$1
    local db="$(su $SU_FLAG $dbinst -c "db2 list db directory" | grep 'Database name'|sort|uniq | sed -e 's/=//g' | awk '{print $3}')"
    version_chk_db=`echo $db |xargs | awk '{print $1}'`
    VERSION=`su $SU_FLAG $dbinst -c "db2licm -l" | grep -w "Version information" | head -1 | awk '{print $3}'`
    $ECHO_E  "\t<application name=\"$ASE_SERVER\" friendlytype=\"$FTYPE\" db2home=\"$HOME\" version=$VERSION >" >> $XMLFILE
    for dbname in $db; do
	$ECHO_E  "\t\t<database name=\"$dbname\" />" >> $XMLFILE
    done
    $ECHO_E  "\t</application>" >>$XMLFILE
}

#Extracting Data Storage and Log paths for DB's
enumerate_db_path() {
set -x
    local dbinst=$1
    local dbname=$2
    DB_PATH=`su $SU_FLAG $dbinst -c "db2 connect to $dbname>/dev/null;db2 -x  \"select substr(PATH,1,1000) as PATH from sysibmadm.DBPATHS where TYPE='DBPATH'\";echo"` 
    DB_PATH=`echo $DB_PATH | xargs| awk '{print $1}'`
    DB_PATH_tmp=`dirname $DB_PATH`
    DB_PATH_tmp=`dirname $DB_PATH_tmp`
    DB_PATH_tmp=`dirname $DB_PATH_tmp`
    DB_PATH=$DB_PATH_tmp


    if [[ $DB_PATH == " " ]]; then
        echo DbPath enumeration for DB $dbname returned error
        DB_PATH=""
        return 1
    fi
}

enumerate_storage_paths() {
set -x
    local dbinst=$1
    local dbname=$2
    local storagepath=""

    STORAGE_PATHS=`su $SU_FLAG $dbinst -c "db2 connect to $dbname>/dev/null; echo STORAGE_PATH:START; db2 -x \"select substr(PATH,1,1000) as PATH from sysibmadm.DBPATHS where TYPE='DB_STORAGE_PATH'\"; echo STORAGE_PATH:END" `
    STORAGE_PATHS_tmp=`echo $STORAGE_PATHS |sed 's/.*STORAGE_PATH:START//g' |sed 's/STORAGE_PATH:END.*//g'`
    STORAGE_PATHS=$STORAGE_PATHS_tmp
   
    TBSPS_PATHS=`su $SU_FLAG $dbinst -c "db2 connect to $dbname>/dev/null; echo TBSPS_PATH:START;  db2 -x \"select substr(PATH,1,1000) as PATH from sysibmadm.DBPATHS where TYPE like 'TBSP%'\"; echo TBSPS_PATH:END" `
    TBSPS_PATHS_tmp=`echo $TBSPS_PATHS |sed 's/.*TBSPS_PATH:START//g' |sed 's/TBSPS_PATH:END.*//g'`
    TBSPS_PATHS=$TBSPS_PATHS_tmp

    if [[ $STORAGE_PATHS == " " ]] && [[ $TBSPS_PATHS == " " ]]; then
         echo Storage enumeration for DB $dbname returned error
        STORAGE_PATHS=""
        TBSPS_PATHS=""
        return 1
    else
        LOG_PATHS=`su $SU_FLAG $dbinst -c "db2 connect to $dbname>/dev/null; echo LOG_PATH:START; db2 -x \"select substr(PATH,1,1000) as PATH from sysibmadm.DBPATHS where TYPE='LOGPATH'\"; echo LOG_PATH:END"`
        LOG_PATHS_tmp=`echo $LOG_PATHS | sed 's/.*LOG_PATH:START//g'|sed 's/LOG_PATH:END.*//g'`
        LOG_PATHS=$LOG_PATHS_tmp
        if [[ $LOG_PATHS == " " ]]; then
            echo Log path enumeration failed for DB $dbname
            STORAGE_PATHS=""
            LOG_PATHS=""
            return 1
        else
            return 0
         fi
    fi
}

#Extracting Data Storage and Log paths for DB's
enumerate_archivelog_path() {
set -x
    local dbinst=$1
    local dbname=$2


    #ARCHIVELOG_PATH=`su $SU_FLAG $dbinst -c "db2 get db cfg for $dbname | grep $LOGARCHMETH_NAME "`
    ARCHIVELOG_PATH=`su $SU_FLAG $dbinst -c "db2 get db cfg for $dbname | grep $LOGARCHMETH_NAME "`
    ARCHIVELOG_PATH=`echo $ARCHIVELOG_PATH | awk -F'=' '{print $2}' | awk -F ':' '{print $2}'`

    if [[ $ARCHIVELOG_PATH == " " ]]; then
         echo "$ARCHIVELOG_PATH is not set " 
        return 1
    fi
    LOGARCH_METH2=`su $SU_FLAG $dbinst -c "db2 get db cfg for $dbname | grep LOGARCHMETH2 "`
    LOGARCH_METH2_PATH=`echo $LOGARCH_METH2 | awk -F'=' '{print $2}' |xargs`
    if [ "$LOGARCH_METH2_PATH" = "OFF" ]; then
       :
    else
       LOGARCH_METH2_PATH=`echo $LOGARCH_METH2 | awk -F'=' '{print $2}' | awk -F ':' '{print $2}'`
    fi
    MIRROR_LOG=`su $SU_FLAG $dbinst -c "db2 get db cfg for $dbname | grep MIRRORLOGPATH "`
    MIRROR_LOG_PATH=`echo $MIRROR_LOG | awk -F'=' '{print $2}'`
    if [ ! -z "$MIRROR_LOG_PATH" ]; then
      # MIRROR_LOG_PATH=`echo $MIRROR_LOG | awk -F'=' '{print $2}' | awk -F ':' '{print $2}'` 
        MIRROR_LOG_PATH=`dirname $MIRROR_LOG_PATH`
        MIRROR_LOG_PATH=`dirname $MIRROR_LOG_PATH`
    fi
    return 0
}

shallow_discovery() {
set -x
    local dbinst=$1
    local db="$(su $SU_FLAG $dbinst -c "db2 list db directory" | grep 'Database name'|sort|uniq | sed -e 's/=//g' | awk '{print $3}')"
    version_chk_db=`echo $db |xargs | awk '{print $1}'`
    VERSION=`su $SU_FLAG $dbinst -c "db2licm -l" | grep -w "Version information" | head -1 | awk '{print $3}'`
    $ECHO_E  "\t<application name=\"$ASE_SERVER\" friendlytype=\"$FTYPE\" db2home=\"$HOME\" version=$VERSION >" >> $XMLFILE
    DB_LIST=
    for dbname in $db; do
        $ECHO_E  "\t\t<database name=\"$dbname\" />" >> $XMLFILE
        if [ -z "$DB_LIST" ]; then
           DB_LIST=$dbname
        else
          DB_LIST=$DB_LIST","$dbname
        fi
    done
    if [ ! -z "$DB_LIST" ]; then
       $ECHO_E  "\t\t<userdblist >" >> $XMLFILE
       $ECHO_E  "\t\t\t\t<database list=\"$DB_LIST\" />" >> $XMLFILE
       $ECHO_E  "\t\t</userdblist>" >>$XMLFILE
    fi
    $ECHO_E  "\t</application>" >>$XMLFILE
}

#Main logic to derive storage paths and XML file 
enumerate_db2_dbs() {
set -x
    local dbinst=$1
    local db="$(su $SU_FLAG $dbinst -c "db2 list db directory" | grep 'Database name'|sort|uniq | sed -e 's/=//g' | awk '{print $3}')"
    version_chk_db=`echo $db |xargs | awk '{print $1}'`
    VERSION=`su $SU_FLAG $dbinst -c "db2 connect to $version_chk_db>/dev/null; db2licm -l" | grep -w "Version information" | head -1 | awk '{print $3}'`

    #XML generation logic for applications and application part
	$ECHO_E  "\t<application name=\"$ASE_SERVER\" friendlytype=\"$FTYPE\" db2home=\"$HOME\" version=$VERSION datavolowner=\"$OSUSER:$OSGROUP\" >" >> $XMLFILE

    for dbname in $db; do
	#echo Found database: $dbname
        offlinedb=`su $SU_FLAG $dbinst -c "db2 connect to $dbname"`
        retval=$?
        if [ "$retval" -gt 0 ]; then
           continue;
        fi

	enumerate_storage_paths $dbinst $dbname
        if [ $? -ne 0 ]; then echo "Failed to execute function enumerate_storage_paths "; exit 1; fi
	enumerate_archivelog_path $dbinst $dbname
        if [ $? -ne 0 ]; then echo "Failed to execute function enumerate_archivelog_path "; exit 1; fi
	enumerate_db_path $dbinst $dbname
	if [ $? -ne 0 ]; then echo "Failed to execute function enumerate_db_path "; exit 1; fi

    
    if [ ! -z "$STORAGE_PATHS" ];then
        DATA_PATH=`echo  "$STORAGE_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'`
    fi
    LOG_PATH=`echo  "$LOG_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'` 
    DB_PATH=$DB_PATH

    if [ ! -z "$DATA_PATH" ]; then
	     DATA_VOL_NAME=`df -P $DATA_PATH | awk 'NR==2{print $NF}'`     
    fi
    if [ ! -z "$TBSPS_PATHS" ];then
        TBSP_PATH=`echo  "$TBSPS_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'`
    fi

    if [ ! -z "$TBSP_PATH" ]; then
             TBSP_VOL_NAME=`df -P $TBSP_PATH | awk 'NR==2{print $NF}'`
    fi
    
    if [ ! -z "$LOG_PATH" ]; then
	     LOG_LVM_NAME=`df -P $LOG_PATH | awk 'NR==2{print $NF}'`     
    fi

    if [ ! -z "$DB_PATH" ]; then
	     DB_PATH_LVM_NAME=`df -P $DB_PATH | awk 'NR==2{print $NF}'`
    fi
    
    if [ ! -z "$MIRROR_LOG_PATH" ] ;then
             MIRRORLOG_LVM_NAME=`df -P $MIRRORLOG_LVM_NAME | awk 'NR==2{print $NF}'`
    fi
 
#DBSIZE Calculation
 DBSIZE_Total=0
 DBSIZE=`su $SU_FLAG $inst_name -c "db2 connect to $dbname > /dev/null;db2 'call get_dbsize_info(?,?,?,-1)';echo" |grep "Parameter Value"|awk 'NR==2{print $NF}'`
 DBSIZE_Total=`expr $DBSIZE_Total "+" $DBSIZE`

    #XML generation logic for database related

	$ECHO_E  "\t\t<database name=\"$dbname\" logmeth1=\"$ARCHIVELOG_PATH\" logmeth2=\"$LOGARCH_METH2_PATH\" mirrorlogpath=\"$MIRROR_LOG_PATH\" SIZE=\"$DBSIZE_Total\" >" >> $XMLFILE
					$ECHO_E  "\t\t\t<files>" >> $XMLFILE
                                        for k in ${DATA_PATH}
                                        do
                                           if [ ! -z "$k" ]; then
                                              DATA_VOL_NAME=`df -P $k | awk 'NR==2{print $NF}'`
                                           fi
                                            if [ ! -z "$k" ]; then
    						k=`echo $k |sed -e 's|/*$||g'`
                                                $ECHO_E  "\t\t\t\t<file path=\"$DATA_VOL_NAME\" $DEV_TYPE=\"$k\" />" >> $XMLFILE
                                            fi
                                        done


                                        for y in ${LOG_PATH}
                                        do
                                          if [ ! -z "$y" ]; then
                                             LOG_LVM_NAME=`df -P $y | awk 'NR==2{print $NF}'`
                                          fi
                                          if [ ! -z "$y" ]; then
    						y=`echo $y |sed -e 's|/*$||g'`
                                                $ECHO_E  "\t\t\t\t<file path=\"$LOG_LVM_NAME\" $LOG_TYPE=\"$y\" />" >> $XMLFILE
                                          fi
                                       done
                                        
                                        for k in ${MIRROR_LOG_PATH}
                                        do
                                          if [ ! -z "$k" ]; then
                                            MIRRORLOG_LVM_NAME=`df -P $k | awk 'NR==2{print $NF}'`
                                          fi
                                          if [ ! -z "$k" ]; then
                                                k=`echo $k |sed -e 's|/*$||g'`
                                                $ECHO_E  "\t\t\t\t<file path=\"$MIRRORLOG_LVM_NAME\" mirrorlogvol=\"$k\" />" >> $XMLFILE
                                          fi
                                       done

					$ECHO_E  "\t\t\t\t<file path=\"$DB_PATH_LVM_NAME\" dbpathvol=\"$DB_PATH\" />" >> $XMLFILE

                                        for m in ${TBSP_PATH}
                                        do
                                           if [ ! -z "$m" ]; then
                                              TBSP_VOL_NAME=`df -P $m | awk 'NR==2{print $NF}'`
                                           fi
                                            if [ ! -z "$m" ]; then
                                                m=`echo $m |sed -e 's|/*$||g'`
                                                $ECHO_E  "\t\t\t\t<file path=\"$TBSP_VOL_NAME\" tbspvol=\"$m\" />" >> $XMLFILE
                                            fi
                                        done

					$ECHO_E  "\t\t\t</files>" >> $XMLFILE

                                        $ECHO_E  "\t\t\t<logbackuppath>" >> $XMLFILE
                                            if [ ! -z "$ARCHIVELOG_PATH" ]; then
                                                $ECHO_E  "\t\t\t\t<file path=\"$ARCHIVELOG_PATH\" />" >> $XMLFILE
                                        fi
                                        $ECHO_E  "\t\t\t</logbackuppath>" >> $XMLFILE
				$ECHO_E  "\t\t</database>" >>$XMLFILE
    done
   $ECHO_E  "\t\t<files >" >> $XMLFILE 
        for dbname in $db; do
          #echo Found database: $dbname
	  offlinedb=`su $SU_FLAG $dbinst -c "db2 connect to $dbname"`
          retval=$?
          if [ "$retval" -gt 0 ]; then
             continue;
          fi
           enumerate_storage_paths $dbinst $dbname
           enumerate_archivelog_path $dbinst $dbname
	   enumerate_db_path $dbinst $dbname

           DATA_PATH=`echo  "$STORAGE_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'`
           LOG_PATH=`echo  "$LOG_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'`
           if [ ! -z "$TBSPS_PATHS" ];then
                TBSP_PATH=`echo  "$TBSPS_PATHS" | sed 's/^[ \t]*//;s/[ \t]*$//'`
           fi

           DB_PATH_LVM_NAME=`df -P $DB_PATH | awk 'NR==2{print $NF}'`
	   $ECHO_E  "\t\t\t\t<file path=\"$DB_PATH_LVM_NAME\" dbpathvol=\"$DB_PATH\" />" >> $XMLFILE

           for x in ${DATA_PATH}
           do
             if [ ! -z "$DATA_PATH" ]; then
                DATA_VOL_NAME=`df -P $x | awk 'NR==2{print $NF}'`
             fi
             if [ ! -z "$DATA_PATH" ]; then
                x=`echo $x |sed -e 's|/*$||g'`
                $ECHO_E  "\t\t\t\t<file path=\"$DATA_VOL_NAME\" $DEV_TYPE=\"$x\" />" >> $XMLFILE
             fi
           done

           for i in ${LOG_PATH}
           do
             if [ ! -z "$i" ]; then
                LOG_LVM_NAME=`df -P $i | awk 'NR==2{print $NF}'`
             fi
             if [ ! -z "$i" ]; then
		i=`echo $i |sed -e 's|/*$||g'`
                $ECHO_E  "\t\t\t\t<file path=\"$LOG_LVM_NAME\" $LOG_TYPE=\"$i\" />" >> $XMLFILE
             fi
           done
            for n in ${MIRROR_LOG_PATH}
            do
             if [ ! -z "$n" ]; then
                  MIRRORLOG_LVM_NAME=`df -P $n | awk 'NR==2{print $NF}'`
             fi
             if [ ! -z "$n" ]; then
                 n=`echo $n |sed -e 's|/*$||g'`
               $ECHO_E  "\t\t\t\t<file path=\"$MIRRORLOG_LVM_NAME\" mirrorlogvol=\"$n\" />" >> $XMLFILE
             fi
            done
          for t in ${TBSP_PATH}
           do
             if [ ! -z "$TBSP_PATH" ]; then
                DATA_VOL_NAME=`df -P $t | awk 'NR==2{print $NF}'`
             fi

             if [ ! -z "$TBSP_PATH" ]; then
                t=`echo $t |sed -e 's|/*$||g'`
                $ECHO_E  "\t\t\t\t<file path=\"$DATA_VOL_NAME\" $DEV_TYPE=\"$t\" />" >> $XMLFILE
                #$ECHO_E "$TBSP_VOL_NAME" >> $DB2_tablespaceDetailsFile
             fi
           done

        if [ -z "$DBNAME_LIST" ]; then
           DBNAME_LIST=$dbname
        else
          DBNAME_LIST=$DBNAME_LIST","$dbname
        fi
        done   
   $ECHO_E  "\t\t</files>" >>$XMLFILE 

    #DBNAME_LIST=$db
    #DBNAME_LIST=`echo $DBNAME_LIST |sed 's/ /,/g'`

    if [ ! -z "$DBNAME_LIST" ]; then
       $ECHO_E  "\t\t<userdblist >" >> $XMLFILE
       $ECHO_E  "\t\t\t\t<database list=\"$DBNAME_LIST\" />" >> $XMLFILE
       $ECHO_E  "\t\t</userdblist>" >>$XMLFILE
    fi
    DBNAME_LIST=
    #XML generation logic for applications and application part

    $ECHO_E  "\t\t<scripts>" >> $XMLFILE
	$ECHO_E  "\t\t\t<script phase=\"all\" path=\"$SCRIPTS\" />" >> $XMLFILE
	$ECHO_E  "\t\t</scripts>" >> $XMLFILE

 	$ECHO_E  "\t</application>" >>$XMLFILE
}



#Instance detection logic
enum_db2_instances(){
    # Print out all the instance users
    echo  "<applications>" > $XMLFILE
    for OSUSERS in $(ps -ef | grep db2sysc |grep -v grep|awk '{print $1}' |xargs);do
    OSUSER=$OSUSERS
    inst_name=$OSUSER
    if [ ! -z "$DISCOVERY_INSTANCENAME" ]; then
        if [ "$DISCOVERY_INSTANCENAME" != "$inst_name" ]; then
            continue;
        fi
    fi
    OSGROUP=`id -gnr  $OSUSER`
    proc_id=`ps aux | grep -w $OSUSER |grep -w  db2sysc|grep -v grep|awk '{print $2}'`
    DB2_HOME=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2_HOME|cut -d "=" -f2`
    INSTHOME=`ps eww $proc_id |tr ' ' '\n'|grep -w HOME|cut -d "=" -f2`
    DB2LIB=`ps eww $proc_id |tr ' ' '\n'|grep -w DB2LIB|cut -d "=" -f2`
    if [ -z "$DB2_HOME" ] && [ -z "$INSTHOME" ] && [ -z "$DB2LIB" ]; then
       DB2_HOME=`cat /etc/passwd | grep -w $OSUSER | awk -F : '{print $6}'`
       DB2_HOME="$DB2_HOME"/sqllib
       if [ -f "$DB2_HOME/db2profile" ]; then
          set +x
          . "$DB2_HOME"/db2profile
          set -x
          db2bin=$DB2_HOME"/bin"
        fi
    elif [  -f "$INSTHOME/sqllib/db2profile" ] ;then
       DB2_HOME="$INSTHOME"/sqllib
       set +x
       . "$DB2_HOME"/db2profile
       set -x
       db2bin=$DB2_HOME"/bin"
    elif [ ! -z "$DB2LIB" ]; then
       DB2_HOME=`dirname $DB2LIB`
       if [ -f "$DB2_HOME/db2profile" ]; then
          set +x
          . "$DB2_HOME"/db2profile
          set -x
          db2bin=$DB2_HOME"/bin"
        fi
    else
        if [ -f "$DB2_HOME/db2profile" ]; then
           set +x
           . "$DB2_HOME"/db2profile
           set -x
           db2bin=$DB2_HOME"/bin"
        fi
 
    fi
    if [ ! -f "$DB2_HOME/db2profile" ]; then
       echo "WARNINGMSG: Unable to find the DB2 Home for the instance $OSUSER"
       continue;
    fi

    HOME=`dirname $DB2_HOME`
    ASE_SERVER=$inst_name
    if [ "$SHALLOW_DISCOVERY" = "TRUE" ] ; then
        shallow_discovery $inst_name
    else
        enumerate_db2_dbs $inst_name
    fi
    done
    $ECHO_E  "</applications>" >> $XMLFILE

}

enum_db2_instances


