#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
XMLFILE="/act/custom_apps/MAXDB.xml"
SCRIPTS="/act/custom_apps/maxdb/CustomApp_MAXDB.sh"
FTYPE="MAXDB"
DBMUSER="$SRC_BACKUP_USER"
set +x
DBMPWD="$SRC_USER_PASSWD"
set -x

if [ -e $XMLFILE ]; then
   rm -f $XMLFILE
fi

#------------------------------------------------------------------------------------------------------------------------
# Function: dbSizeCal()
#------------------------------------------------------------------------------------------------------------------------
dbSizeCal() {
  DBSID=$1

  set +x
  DBState=`dbmcli -d $DBSID $MAXDB_KEY db_state | grep -vw "OK\|State"`
  set -x

  if [ "$DBState" != "ONLINE" ]; then
    dbSize=0
    return 1
  fi

  set +x
  dbSizeInfo=`dbmcli -d $DBSID $MAXDB_KEY info DATA | grep "Used Space " | grep -v "%"`
  dbSize=`dbmcli -d $DBSID $MAXDB_KEY info DATA | grep "Used Space " | grep -v "%" | awk '{sum+=$NF} END {print sum}'`
  set -x

  sizeType=`echo $dbSizeInfo| cut -d'(' -f2| cut -d ')' -f1`
  echo "DBSize: $dbSize $sizeType"
  if [ "$sizeType" = "GB" ]; then
    dbSize=`expr $dbSize \* 1024 \* 1024 \* 1024`
  elif [ "$sizeType" = "MB" ]; then
    dbSize=`expr $dbSize \* 1024 \* 1024`
  elif [ "$sizeType" = "KB" ]; then
    dbSize=`expr $dbSize \* 1024`
  fi

  return 0
}
#------------------------------------------------------------------------------------------------------------------------


chk_running_inst_count=`ps -eaf | grep "/pgm/kernel" | grep -vw grep | awk -F"/pgm/kernel" '{print $1}' | grep -v awk | awk '{print $NF}' | uniq | wc -l`

if [ $chk_running_inst_count -eq 0 ]; then
  echo "ERRORMSG: None of the  DB Instance is Running."
  exit 1
else
  Local_soft_loc=`ps -eaf | grep "/pgm/kernel" | grep -vw grep | awk -F"/pgm/kernel" '{print $1}' | grep -v awk | awk '{print $NF}' | uniq | head -1`
  LOCAL_PROG_PATH=$Local_soft_loc/bin
fi

GLOBALPROGRAMPATH=`$LOCAL_PROG_PATH/dbmcli dbm_getpath | grep "GlobalProgPath" | cut -d'=' -f2`
echo "$GLOBALPROGRAMPATH"

if [ -z $GLOBALPROGRAMPATH ]; then
 echo "ERRORMSG: Unable to find the Software Location on the server. Discovery Failed"
 exit 1;
fi

#Set the Environment Variable

export PATH=$GLOBALPROGRAMPATH/bin:$PATH

#Check if UserName is empty 
if [ -z $DBMUSER ]; then 
   echo " ERRORMSG: DBM username is missing"
   exit 1
fi

#Check the credentials (user/password or key)
set +x
if [ -z $DBMPWD ]; then 
  MAXDB_KEY=" -U $DBMUSER"
else
  MAXDB_KEY=" -u $DBMUSER,$DBMPWD"
fi
set -x


#------------------------------------------------------------------------------------------------------------------------
#Check if the UserName and Password is correct
#------------------------------------------------------------------------------------------------------------------------
db_connection(){
  DBSID=$1
  set +x
  $GLOBALPROGRAMPATH/bin/dbmcli -d $DBSID $MAXDB_KEY bye
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
      echo "ERRORMSG: User Authentication Failed for $DBSID Database. Please check username and password"
  fi
  return $retval
}
#------------------------------------------------------------------------------------------------------------------------


#------------------------------------------------------------------------------------------------------------------------
get_DBHOME_INFO() {
  DBSID=$1

  OSUSER=`ps -eaf | grep -iw $DBSID | grep -w kernel | awk '{print $1}' | uniq`
  OSGROUP=`id -ngr $OSUSER`
  version=`dbmcli -d $DBSID dbm_version VERSION | grep -vw "OK"`
  GLB_DATA_PATH=`dbmcli -d $DBSID dbm_getpath GlobalDataPath | grep -vw "OK"`
  LOCAL_PRG_PATH=`dbmcli -d $DBSID dbm_version INSTROOT | grep -vw "OK"`
  DB_CONFIG_FILE=$GLB_DATA_PATH/config/$DBSID

  set +x
  RUNDIR=`dbmcli -d $DBSID $MAXDB_KEY param_directget RUNDIRECTORY | grep -v "OK" | awk '{ print $NF }'`
  DB_STATE=`dbmcli -d $DBSID $MAXDB_KEY db_state | grep -v "OK" | grep -v "State"`
  set -x

  if ! [ "$DB_STATE" = "ADMIN" ]; then
    set +x
    AUTOLOGBACKUP_STATUS=`dbmcli -d $DBSID $MAXDB_KEY  autolog_show | grep -v "OK"`
    set -x
  else
    AUTOLOGBACKUP_STATUS=""
  fi

  set +x
  LOG_BKP_TEMPLATE_NAME=`dbmcli -d $DBSID $MAXDB_KEY backup_history_list -I -c label,media,action | grep "LOG_" | grep -w "SAVE" | head -1 | cut -d'|' -f2 | sed -e 's/ *$//'`
  set -x

  if [ -z $LOG_BKP_TEMPLATE_NAME ]; then
    LOG_BKP_DIR=""
  else
    set +x
    LOG_BKP_DIR=`dirname $(dbmcli -d $DBSID $MAXDB_KEY backup_template_show NAME $LOG_BKP_TEMPLATE_NAME | grep LOCATION | awk '{ print $NF }')`
    set -x
    if [ "${LOG_BKP_DIR:0:1}" != "/" ]; then
      LOG_BKP_DIR=$RUNDIR
    fi
  fi

  #Find SizeInBytes
  dbSizeCal $DBSID

  echo -e "\t<application name=\"$DBSID\" friendlytype=\"MAXDB\" OSUSER=\"$OSUSER\" DATAVOLUMEOWNER=\"$OSUSER:$OSGROUP\" VERSION=\"$version\" RUNDIRECTORY=\"$RUNDIR\" LOCAL_PRG_PATH=\"$LOCAL_PRG_PATH\" DB_CONFIG_FILE=\"$DB_CONFIG_FILE\" AUTOLOG_BACKUP_STATUS=\"$AUTOLOGBACKUP_STATUS\" LOG_BACKUP_TEMPLATE_NAME=\"$LOG_BKP_TEMPLATE_NAME\" LOG_BACKUP_DIR=\"$LOG_BKP_DIR\" SIZE=\"$dbSize\" >" >>$XMLFILE

}
#------------------------------------------------------------------------------------------------------------------------


#------------------------------------------------------------------------------------------------------------------------
get_FILE_XML() {
  DBSID=$1

  set +x
  Data_File_List=`dbmcli -d $DBSID $MAXDB_KEY  param_getvolsall |grep -vw "OK\|MaxDataVolumes\|MaxLogVolumes\|UseMirroredLog"| awk -F" F " '{print $2}' | awk '{print $1}'| xargs`
  set -x

  for y in $Data_File_List
  do

    if [ "${y:0:1}" != "/" ]; then
      yFullPath=$RUNDIR/$y
    else
      yFullPath=$y
    fi

    local file_tmp=`dirname $yFullPath`
    DB_MNT=`df -P $file_tmp |tail -1 | awk '{print $NF}'`
    echo -e "\t\t\t<file path=\"$DB_MNT\" datavol=\"$file_tmp\" />" >> $XMLFILE
  done
}
#------------------------------------------------------------------------------------------------------------------------


count=0
db_count=`dbmcli db_enum | grep running | awk '{ print $1 }' | wc -l`
echo "Total db count: $db_count"

#Start xml generation
echo -e "<applications>" >> $XMLFILE
DB_LIST=`dbmcli db_enum | grep running | awk '{ print $1 }'`
for x in $DB_LIST
do
  db_connection $x
  if [ $? -ne 0 ]; then
     count=`expr $count + 1`
     echo $count
     continue
  fi
  get_DBHOME_INFO $x

  echo -e "\t\t<files>" >> $XMLFILE
  get_FILE_XML $x
  echo -e "\t\t</files>" >> $XMLFILE

  echo -e "\t\t<scripts>" >> $XMLFILE
  echo -e "\t\t\t<script phase=\"all\" path=\"$SCRIPTS\" />" >> $XMLFILE
  echo -e "\t\t</scripts>" >> $XMLFILE

  echo -e "\t</application>" >> $XMLFILE
done
echo -e "</applications>" >> $XMLFILE

if [ $count -eq $db_count ]; then
   echo "ERRORMSG: Discovery Failed for all the Databases"
   exit 1
elif [ $count -eq 0 ]; then
   echo "Discovery Succeeded for all the Databases"   
else 
   echo "WARNINGMSG: Discovery failed for some of the databases"
fi

exit 0
