#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
XMLFILE="/act/custom_apps/MYSQL.xml"
SCRIPTS="/act/custom_apps/mysql/CustomApp_MYSQL.sh"
FTYPE="MYSQLInstance"
OSUSER=""
DBUSER=$SRC_BACKUP_USER
set +x
DBPASSWORD="$SRC_USER_PASSWD"
set -x
SHALLOW_DISCOVERY=$SHALLOW_DISCOVERY

#MYSQLCONFIG="/act/custom_apps/mysql/mysql_config.conf"
. /act/custom_apps/mysql/act_mysql_common_functions.sh


#-------------------------------------------------------------------
# function: xml_files()
#-------------------------------------------------------------------
xml_files() {
  DBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' | xargs"`

  echo -e "\t\t<files>" >>$XMLFILE
  for db in $DBLIST
  do
    find_DBDIR $DATADIR $db	#Will get value into DB_DIR
    DB_MNTPNT=`df -P $DB_DIR | tail -1 | awk '{print $NF}'`
    if [ -d "$DB_DIR" ]; then
      echo -e "\t\t\t<file path=\"$DB_MNTPNT\" datavol=\"$DB_DIR\" />" >>$XMLFILE
    fi
  done

  for tbf in $(echo $INNODB_DATA_FILE_PATH|sed 's/;/ /g')
  do
    if [ "${tbf:0:1}" != "/" ]; then
      tbf=$DATADIR/$tbf
    fi
    tbf_file=`echo $tbf| cut -d: -f1`
    tbf_mntpnt=`df -P $tbf_file | tail -1 | awk '{print $NF}'`
    #enable if required in future
    #echo -e "\t\t\t<file path=\"$tbf_mntpnt\" datavol=\"$tbf_file\" />" >>$XMLFILE
  done
  echo -e "\t\t</files>" >>$XMLFILE
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: xml_database()
#-------------------------------------------------------------------
xml_database() {
  DBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' | xargs"`

  for db in $DBLIST 
  do
    #Find SizeInBytes
    dbSize=`su $OSUSER -c "$MYSQL_CMD -e \"SELECT SUM(data_length + index_length) FROM information_schema.TABLES where table_schema='$db';\" "`
    if [ "$dbSize" = "NULL" ]; then
      dbSize=0;
    fi
    echo "$dbName size: $dbSize"

    echo -e "\t\t<database name=\"$db\" SIZE=\"$dbSize\" >" >>$XMLFILE
    echo -e "\t\t\t<files>" >>$XMLFILE

    find_DBDIR $DATADIR $db	#Will get value into DB_DIR
    if [ -d "$DB_DIR" ]; then
      echo -e "\t\t\t\t<file path=\"$DB_DIR\" />" >>$XMLFILE
    fi

    echo -e "\t\t\t</files>" >>$XMLFILE

    echo -e "\t\t</database>" >>$XMLFILE
  done
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: xml_userdbList()
#-------------------------------------------------------------------
xml_userdbList() {
  USER_DBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases'" | grep -v -w "information_schema\|mysql\|performance_schema\|sys" |xargs`
  USER_DBLIST=`echo $USER_DBLIST | sed 's/ /,/g'`
  echo -e "\t\t<userdblist>\n\t\t\t<database list=\"$USER_DBLIST\" />\n\t\t</userdblist>" >>$XMLFILE
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: xml_sysdbList()
#-------------------------------------------------------------------
xml_sysdbList() {
  SYS_DBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' " | grep -w "information_schema\|mysql\|performance_schema\|sys" |xargs`
  SYS_DBLIST=`echo $SYS_DBLIST | sed 's/ /,/g'`
  echo -e "\t\t<sysdblist>\n\t\t\t<database list=\"$SYS_DBLIST\" />\n\t\t</sysdblist>" >>$XMLFILE
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: xml_variable()
#-------------------------------------------------------------------
xml_variable() {
  local varName=$1
  MYSQL_VARVAL=`su -m $OSUSER -c "$MYSQL_CMD -e \"show variables like '$varName'\" " | awk '{print $2}'`
  echo -e "\t\t<$varName>\n\t\t\t<file path=\"$MYSQL_VARVAL\" />\n\t\t</$varName>" >>$XMLFILE
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
# function: findConfigFile()
#-------------------------------------------------------------------
findConfigFile() {
  line=$1

  #Find config file
  CONFIG_FILE=`echo $line | awk -F"--defaults-file=" '{print $2}' | awk '{print $1}'`
  IS_NODEFAULTS_SPECIFIED=`echo $line | grep -w "\-\-no-defaults" | wc -l`
  if [ -z "$CONFIG_FILE" ] && [ $IS_NODEFAULTS_SPECIFIED -eq 0 ]; then
    MYSQLD_SRC_CMD=`echo $line| awk -F"mysqld" '{print $2}'`
    if [ -e /etc/my.cnf ]; then
      CONFIG_FILE=/etc/my.cnf
    elif [ -e /etc/mysql/my.cnf ]; then
      CONFIG_FILE=/etc/mysql/my.cnf
    else
       b_dir=`echo $line | awk -F"--basedir=" '{print $2}' | awk '{print $1}'`
       d_dir=`echo $line | awk -F"--datadir=" '{print $2}' | awk '{print $1}'`
       if [ ! -z "$b_dir" ] && [ -e "$b_dir/my.cnf" ]; then
         CONFIG_FILE="$b_dir/my.cnf"
       elif [ ! -z "$d_dir" ] && [ -e "$d_dir/mysqld-auto.cnf" ]; then
         CONFIG_FILE="$d_dir/mysqld-auto.cnf"
       fi
    fi
  fi

  if [ $IS_NODEFAULTS_SPECIFIED -ne 0 ]; then
    return 1
  fi

  #if config file is soft link, find actual file
  isLink=`ls -l $CONFIG_FILE | awk '{print $(NF-1)}'`
  if [ "$isLink" = "->" ]; then
    CONFIG_FILE=`ls -l $CONFIG_FILE | awk '{print $NF}'`
  fi
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
# Main script starts here 
#-------------------------------------------------------------------
# Capture into XML file
echo -e "<applications>" > $XMLFILE
ps -ef | grep -w "mysqld" | grep -v -w mysqld_safe | grep -v -w grep | while read line
  do
    OSUSER=`echo $line | awk '{print $1}'`
    OSGROUP=`id -gn $OSUSER`

    #Find config file
    findConfigFile "$line"
    if [ -z "$CONFIG_FILE" ]; then
      echo "WARNINGMSG: Fail to find config file"
      continue
    fi

    BASEDIR=`echo $line | awk -F"--basedir=" '{print $2}' | awk '{print $1}'`
    if [ -z "$BASEDIR" ]; then
      BASEDIR=`grep "^basedir" $CONFIG_FILE | cut -d= -f2 | sed 's/ //g'`
      if [ -z "$BASEDIR" ] && [ -e /usr/bin/mysql ]; then
        BASEDIR=/usr
      fi
    fi
    DATADIR=`echo $line | awk -F"--datadir=" '{print $2}' | awk '{print $1}'`
    if [ -z "$DATADIR" ]; then
      DATADIR=`grep "^datadir" $CONFIG_FILE | cut -d= -f2 | sed 's/ //g'`
      if [ -z "$DATADIR" ]; then
	echo "WARNINGMSG: Fail to find DATADIR"
        continue
      fi
    fi
    VERSION=`$BASEDIR/bin/mysql -V | awk -F" for" '{print $1}' | awk '{print $NF}' | cut -d, -f1`
    isMySQL=`$BASEDIR/bin/mysql -V | grep -w "MariaDB" | wc -l`
    if [ $isMySQL -gt 0 ]; then
      continue
    fi

    SOCKETFILE=`echo $line | awk -F"--socket=" '{print $2}' | awk '{print $1}'`
    if [ "$SOCKETFILE" = "" ]; then
      SOCKETFILE=`grep "^socket" $CONFIG_FILE | cut -d= -f2 | head -1 | sed 's/ //g'`
    fi
    if [ "$SOCKETFILE" != "" ] && [ "${SOCKETFILE:0:1}" != "/"  ]; then
      SOCKETFILE="$DATADIR/$SOCKETFILE"
    fi

    #Prepare env and MYSQL_CMD
    set +x
    prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
    set -x

    #Check connection to MySQL
    checkMySQL
    if [ $? -ne 0 ]; then
      continue;
    fi

    #Crosscheck basedir, datadir and socketfile matches are not
    basedirVar=`getMySqlVarValue basedir`
    basedirVar=`echo ${basedirVar%/}`
    BASEDIR=`readlink -f $BASEDIR` # if BASEDIR is a softlink, find the actual one
    BASEDIR=`echo ${BASEDIR%/}`
    if [ "$basedirVar" != "$BASEDIR" ]; then
      echo "WARNINGMSG: Fail to find correct basedir"
      continue
    fi
    datadirVar=`getMySqlVarValue datadir`
    datadirVar=`echo ${datadirVar%/}`
    DATADIR=`echo ${DATADIR%/}`
    if [ "$datadirVar" != "$DATADIR" ]; then
      echo "WARNINGMSG: Fail to find correct datadir"
      continue
    fi
    socketVar=`getMySqlVarValue socket`
    if [ "$SOCKETFILE" != "" ] && [ "$SOCKETFILE" != "$socketVar" ]; then
      echo "WARNINGMSG: Fail to find correct socket file"
      continue
    fi
    SOCKETFILE=$socketVar

    if [ "$SHALLOW_DISCOVERY" = "TRUE" ]; then
      PORTNO=`getMySqlVarValue port`
      INSTANCE_NAME=mysqld_$PORTNO
      DBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' | xargs"`
      echo -e "\t<application name=\"$INSTANCE_NAME\" friendlytype=\"$FTYPE\" OSUSER=\"$OSUSER\" BASEDIR=\"$BASEDIR\" VERSION=\"$VERSION\" >" >> $XMLFILE
      for DB_NAME in $DBLIST; do
        echo -e "\t\t<database name=\"$DB_NAME\" />" >> $XMLFILE
      done
      xml_userdbList
      xml_sysdbList
      echo -e "\t</application>" >>$XMLFILE
      continue
    fi

    #Find required variable values
    TMPDIR=`getMySqlVarValue tmpdir`
    MSG_DIR=`getMySqlVarValue lc_messages_dir`
    MSG_LN=`getMySqlVarValue lc_messages`
    PORTNO=`getMySqlVarValue port`
    LOGBIN_LOC=`getMySqlVarValue log_bin_basename`
    #LOGBIN_LOC=`dirname $LOGBIN_LOC`
    SERVRID=`getMySqlVarValue server_id`
    INNODB_FILE_PER_TABLE=`getMySqlVarValue innodb_file_per_table`
    INNODB_DATA_FILE_PATH=`getMySqlVarValue innodb_data_file_path`
    INNODB_DATA_HOME_DIR=`getMySqlVarValue innodb_data_home_dir`
    if [ "$INNODB_FILE_PER_TABLE" = "ON" ]; then
      INNODB_DATA_HOME_DIR=$DATADIR
    fi

    PIDFILE=`getMySqlVarValue pid_file`
    if [ "${PIDFILE:0:1}" != "/" ]; then
      PIDFILE=$DATADIR/$PIDFILE
    fi

    LOGERROR=`getMySqlVarValue log_error`
    if [ "${LOGERROR:0:1}" != "/" ]; then
      LOGERROR=$DATADIR/$LOGERROR
    fi

    SOCKET_FILE=`getMySqlVarValue socket`
    if [ "$SOCKET_FILE" != "" ] && [ "${SOCKET_FILE:0:1}" != "/"  ]; then
      SOCKET_FILE="$DATADIR/$SOCKET_FILE"
    fi

    SRV_TYPE=`su -m $OSUSER -c "$MYSQL_CMD -e \"show slave status; \" " | wc -l`
    if [ "$SRV_TYPE" -eq 0 ];then
      SRV_TYPE="Master"
    else
      SRV_TYPE="Slave"
    fi

    CLUST_MANG=`getMySqlVarValue ndb_connectstring`
    IS_CLUST=
    if [ -z $CLUST_MANG ]; then
      IS_CLUST='FALSE'
    else
      IS_CLUST='TRUE'
    fi
 
    INSTANCE_NAME=mysqld_$PORTNO

    echo -e "\t<application name=\"$INSTANCE_NAME\" friendlytype=\"$FTYPE\" OSUSER=\"$OSUSER\" DATAVOLUMEOWNER=\"$OSUSER:$OSGROUP\" VERSION=\"$VERSION\" BASEDIR=\"$BASEDIR\" DATADIR=\"$DATADIR\" TMPDIR=\"$TMPDIR\" MESSAGES_DIR=\"$MSG_DIR\" PORTNO=\"$PORTNO\" SOCKET_FILE=\"$SOCKET_FILE\" CONFIG_FILE=\"$CONFIG_FILE\" SRV_TYPE=\"$SRV_TYPE\" LOGBIN_LOC=\"$LOGBIN_LOC\" IS_CLUST=\"$IS_CLUST\" PIDFILE=\"$PIDFILE\" LOGERROR=\"$LOGERROR\" SERVRID=\"$SERVRID\" >" >> $XMLFILE

    #enable fi required in future
    #echo -e "\t\t<mysqld_cmd>\n\t\t\t<command MYSQLD_SRC_CMD=\"$MYSQLD_SRC_CMD\" />\n\t\t</mysqld_cmd>" >> $XMLFILE

    xml_database 
    xml_files
    xml_userdbList
    xml_sysdbList

    echo -e "\t\t<scripts>\n\t\t\t<script phase=\"all\" path=\"$SCRIPTS\" />\n\t\t</scripts>" >> $XMLFILE
    echo -e "\t</application>" >>$XMLFILE
  done
echo -e "</applications>" >>$XMLFILE
#-------------------------------------------------------------------
