#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
XMLFILE="/act/custom_apps/POSTGRESQL.xml"
SCRIPTS="/act/custom_apps/postgresql/CustomApp_POSTGRESQL.sh"
FTYPE="POSTGRESQLInstance"
DBUSER="$SRC_BACKUP_USER"
DBSID=""
KEYNAME=""
INSTANCENUM=""
HVERSION=""
VERSION=""
DATAVOL=""
DATAPATH=""
LOGVOL=""
LOGPATH=""
DATAVOLUMEOWNER=""
PYTHONPATH=""
set +x
DBPASSWD="$SRC_USER_PASSWD"
set -x
SHALLOW_DISCOVERY=$SHALLOW_DISCOVERY


#-------------------------------------------------------------------------------------------------
# function: removeFile()
#-------------------------------------------------------------------------------------------------
removeFile() {
  local fileName=$1
  if [ -e $fileName ]; then
    rm -f $fileName
  fi
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function shallowDiscovery()
#-------------------------------------------------------------------------------------------------
shallowDiscovery() {
  local pgPID=$1

  #Check connection to PostgreSQL with given credentials
  . /act/custom_apps/postgresql/act_postgresql_common_functions.sh
  set +x
  prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
  set -x

  su $OSUSER -c "$PSQL -c \"select 1;\" "
  if [ $? -ne 0 ]; then
    echo "WARNINGMSG: Fail to connect to $PG_DATA with given credentials"
    flagALL=1
    return 1
  fi

  DBLIST=`su $OSUSER -c "$PSQL -c \"select datname from pg_database;\" " | grep -v "Warning: your password will expire in" | xargs`

  echo -e "\t<application name=\"$INSTANCE_NAME\" friendlytype=\"$FTYPE\" OSUSER=\"$OSUSER\" DBUSER=\"$DBUSER\" VERSION=\"$VERSION\" BASEDIR=\"$PG_HOME\" >" >>$XMLFILE
  for db in $DBLIST
  do
    echo -e "\t\t<database name=\"$db\" />" >>$XMLFILE
  done
  xml_userdbList
  xml_sysdbList
  echo -e "\t</application>" >>$XMLFILE

  flagAny=1 # At-least one discovery is success
  return 0
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: xml_database()
#-------------------------------------------------------------------------------------------------
xml_database() {
  DBLIST=`su $OSUSER -c "$PSQL -c \"select datname from pg_database;\" " | grep -v "Warning: your password will expire in" | xargs`
  for db in $DBLIST
  do
    #Find SizeInBytes
    dbSize=`su $OSUSER -c "$PSQL -c \"SELECT pg_database_size('$db');\" " | grep -v "Warning: your password will expire in"`

    echo -e "\t\t<database name=\"$db\" SIZE=\"$dbSize\" >" >>$XMLFILE
    echo -e "\t\t\t<files>" >>$XMLFILE

    DB_DIR_LIST=$PG_DATA
    TS_list=`su $OSUSER -c "$PSQL -d$db -c \"select tablespace from pg_tables where tablespace not in ('pg_global','');\" " | grep -v "Warning: your password will expire in" | xargs`
    for tsName in $TS_list
    do
      tsLoc=`su $OSUSER -c "$PSQL -c \"select pg_tablespace_location(oid) from pg_tablespace where spcname = '$tsName';\" " | grep -v "Warning: your password will expire in"`
      DB_DIR_LIST="$DB_DIR_LIST $tsLoc"
    done

    for DB_DIR in $DB_DIR_LIST
    do
      echo -e "\t\t\t\t<file path=\"$DB_DIR\" />" >>$XMLFILE
      DB_VOL=`df -P $DB_DIR | awk 'NR==2{print $NF}'`
      echo -e "\t\t\t<file path=\"$DB_DIR\" datavol=\"$DB_VOL\" />" >> $FILESS
    done

    echo -e "\t\t\t</files>" >>$XMLFILE

    echo -e "\t\t</database>" >>$XMLFILE
  done
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: xml_files
#-------------------------------------------------------------------------------------------------
xml_files() {
  echo -e "\t\t<files>" >>$XMLFILE
  cat $FILESS | sort | uniq > $FILESS.txt
  cat $FILESS.txt >>$XMLFILE
  echo -e "\t\t</files>" >>$XMLFILE
  removeFile $FILESS
  removeFile $FILESS.txt
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: xml_userdbList
#-------------------------------------------------------------------------------------------------
xml_userdbList() {
  USER_DBLIST=`su $OSUSER -c "$PSQL -c \"select datname from pg_database;\" " | grep -vw "postgres\|template1\|template0" | grep -v "Warning: your password will expire in" | xargs`
  USER_DBLIST=`echo $USER_DBLIST|sed 's/ /,/g'`
  echo -e "\t\t<userdblist>\n\t\t\t<database list=\"$USER_DBLIST\" />\n\t\t</userdblist>" >>$XMLFILE
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: xml_sysdbList
#-------------------------------------------------------------------------------------------------
xml_sysdbList() {
  SYS_DBLIST=`su $OSUSER -c "$PSQL -c \"select datname from pg_database;\" " | grep -w "postgres\|template0\|template1" | grep -v "Warning: your password will expire in"| xargs`
  SYS_DBLIST=`echo $SYS_DBLIST|sed 's/ /,/g'`
  echo -e "\t\t<sysdblist>\n\t\t\t<database list=\"$SYS_DBLIST\" />\n\t\t</sysdblist>" >>$XMLFILE
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# Main script starts here
#-------------------------------------------------------------------------------------------------

#Prepare temp location
TMPDIR=/act/touch
mkdir -p $TMPDIR

#set DiscoveryFlag
flagAny=0
flagALL=0

FILESS=$TMPDIR/FILESS
removeFile $FILESS

#Get list of PostgreSQL server processes
#postgresPID_list=`ps -ef | grep 'bin/postgres' | grep -vw grep | awk '{print $2}' | xargs`
postgresPID_list=`ps -ef | grep " postgres: "| grep -vw grep | awk '{print $3}'| sort | uniq | xargs`

echo -e "<applications>" >$XMLFILE

#For ever PostgreSQL server process discover details
for pstgrsPID in $postgresPID_list
do
  OSUSER=`ps -p $pstgrsPID -o user | awk 'NR==2{print $1}'`
  OSGROUP=`id -gn $OSUSER`

  PG_HOME=`ps -p $pstgrsPID -o cmd | awk -F'/bin/' 'NR==2{print $1}'`
  if [ "${PG_HOME:0:1}" != "/"  ]; then
    PG_HOME_Path=`xargs -0 -L1 -a /proc/$pstgrsPID/environ | grep -w "PATH=" | cut -d= -f2`
    for pgPath in $(echo $PG_HOME_Path | sed "s/:/ /g")
    do
      if [ -e "$pgPath/pg_ctl" ] && [ ${pgPath: -4} = "/bin" ]; then
        PG_HOME=${pgPath:0:-4}
        break
      fi
    done
  fi
  if [ -z "$PG_HOME" ] || [ ! -d "$PG_HOME" ]; then
    echo "WARNINGMSG: Fail to find the PostgresHome from process id $pstgrsPID"
    flagALL=1
    continue
  fi

  #Set env
  export PATH=$PG_HOME/bin:$PATH
  export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

  VERSION=`pg_ctl -V | awk '{print $NF}'`

  PGDATA_onCmdLine=`ps -p $pstgrsPID -o cmd | grep " -D " | wc -l`
  if [ $PGDATA_onCmdLine -eq 0 ]; then
    PG_DATA=`xargs -0 -L1 -a /proc/$pstgrsPID/environ | grep -w "PGDATA=" | cut -d= -f2`
  else
    PG_DATA=`ps -p $pstgrsPID -o cmd | awk -F'-D ' 'NR==2{print $2}' | awk '{print $1}'`
  fi
  if [ -z "$PG_DATA" ] || [ ! -d "$PG_DATA" ]; then
    echo "WARNINGMSG: Fail to find the PGDATA location for process id $pstgrsPID"
    flagALL=1
    continue
  fi

  PG_CNF=$PG_DATA/postgresql.conf
  if [ ! -e "$PG_CNF" ]; then
    echo "WARNINGMSG: postgresql.conf not present in PGDATA ($PG_DATA) for process id $pstgrsPID"
    flagALL=1
    continue
  fi

  PORT_onCmdLine=`ps -p $pstgrsPID -o cmd | grep " -p" | wc -l`
  if [ $PORT_onCmdLine -eq 0 ]; then
    PORT=`grep -w port $PG_CNF |sed 's/ //g'|sed 's/\t//g'| grep ^port=| cut -d= -f2 | cut -d'#' -f1`
  else
    PORT=`ps -p $pstgrsPID -o cmd | awk -F'-p' 'NR==2{print $2}' | awk '{print $1}'`
  fi
  if [ -z "$PORT" ]; then
    PORT=5432
  fi

  INSTANCE_NAME=postgresql_$PORT
  SOCKET_DIR=`grep -w unix_socket_directories $PG_CNF | sed 's/ //g' | sed 's/\t//g' | grep -v "^#" | grep -vw grep| cut -d"=" -f2 | sed "s/'//g" | cut -d, -f1`

  if [ "$SHALLOW_DISCOVERY" = "TRUE" ]; then
    shallowDiscovery $pstgrsPID
    continue
  fi

  ARCHIVE_MODE=`grep -w archive_mode $PG_CNF |sed 's/ //g'|sed 's/\t//g'| grep ^archive_mode=| cut -d= -f2 | cut -d'#' -f1`
  WAL_LEVEL=`grep -w wal_level $PG_CNF | sed 's/ //g'|sed 's/\t//g'| grep ^wal_level=|cut -d= -f2 | cut -d'#' -f1`
  isLsngLocal=`grep -w listen_addresses $PG_CNF | sed 's/ //g'|sed 's/\t//g' | grep ^listen_addresses= | grep -w localhost | wc -l`
  if [ $isLsngLocal -eq 0 ]; then
    HOSTIP=`grep -w listen_addresses $PG_CNF | sed 's/ //g'|sed 's/\t//g' | grep ^listen_addresses= | cut -d= -f2 | cut -d'#' -f1 | cut -d, -f1 | sed "s.'..g"`
  fi

  if [ -z "$DBUSER" ]; then
    DBUSER=$OSUSER
  fi

  #Check connection to PostgreSQL with given credentials
  . /act/custom_apps/postgresql/act_postgresql_common_functions.sh
  set +x
  prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
  set -x

  su $OSUSER -c "$PSQL -c \"select 1;\" "
  if [ $? -ne 0 ]; then
    echo "WARNINGMSG: Fail to connect to $PG_DATA with given credentials"
    flagALL=1
    continue
  fi

  IS_SLAVE=`su $OSUSER -c "$PSQL -c \"select pg_is_in_recovery();\" " | grep -v "Warning: your password will expire in"`
  if [ "$IS_SLAVE" = "f" ]; then
    IS_SLAVE="FALSE"
  else
    IS_SLAVE="TRUE"
  fi

  #ARCHIVE_LOC=`su postgres -c 'psql -AXt -c "show archive_command;" ' | awk -F" cp %p " '{print $2}' | awk -F"%f" '{print $1}'`

  echo -e "\t<application name=\"$INSTANCE_NAME\" friendlytype=\"$FTYPE\" OSUSER=\"$OSUSER\" DBUSER=\"$DBUSER\" DATAVOLUMEOWNER=\"$OSUSER:$OSGROUP\" VERSION=\"$VERSION\" BASEDIR=\"$PG_HOME\" DATADIR=\"$PG_DATA\" PG_CNF=\"$PG_CNF\" PORTNO=\"$PORT\" ARCHIVE_MODE=\"$ARCHIVE_MODE\" WAL_LEVEL=\"$WAL_LEVEL\" ISSLAVE=\"$IS_SLAVE\" SOCKET_DIR=\"$SOCKET_DIR\" >" >>$XMLFILE

  xml_database
  xml_files
  xml_userdbList
  xml_sysdbList

  echo -e "\t\t<scripts>\n\t\t\t<script phase=\"all\" path=\"$SCRIPTS\" />\n\t\t</scripts>" >> $XMLFILE

  echo -e "\t</application>" >>$XMLFILE

  # At-least one discovery is success
  flagAny=1

done

echo -e "</applications>" >>$XMLFILE

if [ $flagAny -eq 0 ]; then
  echo "ERRORMSG: All applications discovery failed."
  exit 1
elif [ $flagALL -ne 0 ]; then
  echo "Warning: Some applications discovery failes."
else
  echo "All applications are discovered successfully"
fi

## Remove temp files and directories
#-------------------------------------------------------------------------------------------------

exit 0
