#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
XMLFILE_ORIG="/act/custom_apps/SYBASE.xml"
XMLFILE="/act/custom_apps/SYBASE_tmp_$RANDOM.xml"
while [ -e "$XMLFILE" ]; do
   XMLFILE="/act/custom_apps/SYBASE_tmp_$RANDOM.xml"
done
SCRIPTS="/act/custom_apps/sybase/CustomApp_SYBASE.sh"
FTYPE="SYBASEInstance"
OSUSER=""
ASE_SERVER=""
SYBASECONFIG="/act/custom_apps/sybase/SYBASE_config.conf"
SHALLOW_DISCOVERY=$SHALLOW_DISCOVERY

SRC_BACKUP_USER=$SRC_BACKUP_USER
SRC_USER_OTHERCRED=$SRC_USER_OTHERCRED
set +x
SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
set -x

unset LANG
unset LC_ALL
export COLUMNS=4000
ERR_CODE=

if [ "$SRC_BACKUP_USER" = "" ]; then
  ERR_CODE=ACT_CAF_SYB_DIS_08
fi

# Function: dbSizeCal()
dbSizeCal() {
  dbSize=0
  local dbName=$1

  set +x
  DBSIZE_dts=`su -m $OSUSER -c "isql -S$ASE_SERVER -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -D$dbName -X -w 2000 --retserverror " <<EOF | grep -v "Your password will expire in" | awk 'NR==6{print $3" "$4" "$5" "$6}'
set nocount on
go
sp_spaceused
go
EOF`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Database $dbName not present !"
    return 1
  fi

  # Calculate data_size
  dataSizeType=`echo $DBSIZE_dts| awk '{print $2}'`
  data_size=`echo $DBSIZE_dts | awk '{print $1}'`
  data_size=`echo $data_size | awk -F"." '{print $1}'`
  if [ "$dataSizeType" = "GB" ]; then
    data_size=`expr $data_size "*" 1024 "*" 1024 "*" 1024`
  elif [ "$dataSizeType" = "MB" ]; then
    data_size=`expr $data_size "*" 1024 "*" 1024`
  elif [ "$dataSizeType" = "KB" ]; then
    data_size=`expr $data_size "*" 1024`
  fi

  # Calculate index_size
  indexSizeType=`echo $DBSIZE_dts| awk '{print $4}'`
  index_size=`echo $DBSIZE_dts | awk '{print $3}'`
  index_size=`echo $index_size | awk -F"." '{print $1}'`
  if [ "$dataSizeType" = "GB" ]; then
    index_size=`expr $index_size "*" 1024 "*" 1024 "*" 1024`
  elif [ "$dataSizeType" = "MB" ]; then
    index_size=`expr $index_size "*" 1024 "*" 1024`
  elif [ "$dataSizeType" = "KB" ]; then
    index_size=`expr $index_size "*" 1024`
  fi

  dbSize=`expr $data_size + $index_size`
  echo "dbSize of $dbName db: $dbSize"
  return 0
}

# Function: shallow_discovery()
shallow_discovery() {
  local ASE_SERVER=$1

  OSUSER=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_SERVER | awk -F"OSUSER=" '{print $2}' | awk '{print $1}'`
  syb_home=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_SERVER | awk -F"INSTANCE_HOME=" '{print $2}' | awk '{print $1}'`
  set +x
  . $syb_home/SYBASE.sh
  set -x
  SRC_SYBASE_SQLD=$SYBASE/$SYBASE_OCS
  SYBASE_VERSION=`dataserver -v |head -1| awk -F"/" '{print $2}' | awk '{print $1}'`

  #Find db_Names
  chown -R $OSUSER:$OSGROUP $tmp_use_loc
  isqlPrepare $ISQL_IFILE $OSUSER ALLDB
  isqlExecute $ISQL_IFILE $DBNAMES_FILE
  if [ $? -ne 0 ]; then
        #echo "ERRORMSG: failed to find database names of sybase server"
        ERR_CODE=ACT_CAF_SYB_DIS_03
        flagALL=1
        return 1
  else
    sed -i "/Your password will expire in/d" $DBNAMES_FILE
  fi
  cat $DBNAMES_FILE

  echo -e "\t<application name=\"$ASE_SERVER\" friendlytype=\"$FTYPE\" SYBOSUSER=\"$OSUSER\" SRC_SYBASE_SQLD=\"$SRC_SYBASE_SQLD\" VERSION=\"$SYBASE_VERSION\" >" >> $XMLFILE
  while read line
  do
    DB_NAME=$line
    echo -e "\t\t<database name=\"$DB_NAME\" />" >> $XMLFILE
  done < $DBNAMES_FILE

  USERDB_LIST=`cat $DBNAMES_FILE | grep -vw "master\|model\|sybmgmtdb\|sybpcidb\|sybsystemdb\|sybsystemprocs\|sybsecurity\|saptools\|saptempdb" | grep -v "^pubs"| xargs |sed 's/ /,/g'`
  echo -e "\t\t<userdblist>" >> $XMLFILE
  echo -e "\t\t\t<database list=\"$USERDB_LIST\" />" >>$XMLFILE
  echo -e "\t\t</userdblist>" >> $XMLFILE

  SYSTEMDB_LIST=`cat $DBNAMES_FILE | grep -w "master\|model\|sybmgmtdb\|sybpcidb\|sybsystemdb\|sybsystemprocs\|sybsecurity\|saptools\|saptempdb" | xargs |sed 's/ /,/g'`
  echo -e "\t\t<sysdblist>" >> $XMLFILE
  echo -e "\t\t\t<database list=\"$SYSTEMDB_LIST\" />" >>$XMLFILE
  echo -e "\t\t</sysdblist>" >> $XMLFILE


  echo -e "\t</application>" >>$XMLFILE

  # Atleast one instance is discovered
  flagAny=1

  return 0
}

######################### function: displayError()
displayError() {
  case $ERR_CODE in
    ACT_CAF_SYB_DIS_01)
      echo "ERRORMSG: The database username or password is incorrect"
    ;;
    ACT_CAF_SYB_DIS_02)
      echo "ERRORMSG: failed to find device names of sybase server"
    ;;
    ACT_CAF_SYB_DIS_03)
      echo "ERRORMSG: failed to find database names of sybase server"
    ;;
    ACT_CAF_SYB_DIS_04)
      echo "ERRORMSG: failed to find user database names of sybase server"
    ;;
    ACT_CAF_SYB_DIS_05)
      echo "ERRORMSG: failed to find system database names of sybase server"
    ;;
    ACT_CAF_SYB_DIS_06)
      echo "ERRORMSG: failed to find default backup server of sybase server"
    ;;
    ACT_CAF_SYB_DIS_07)
      echo "ERRORMSG: failed to find page_size of the server"
    ;;
    ACT_CAF_SYB_DIS_08)
      echo "ERRORMSG: SRC_BACKUP_USER is not provided"
    ;;
    ACT_CAF_SYB_DIS_09)
      echo "ERRORMSG: failed to find character set details"
    ;;
  esac
}

########################################################

######################### function: prepareTempFiles()
prepareTempFiles() {
  echo "prepareTempFiles.........."
  tmp_loc='/act/touch'
  tmp_use_loc=$tmp_loc/sybTmpDir_$RANDOM
  while [ -e $tmp_use_loc ]; do
     tmp_use_loc=$tmp_loc/sybTmpDir_$RANDOM
  done

  ALL_SRV_DTLS_FILE=$tmp_use_loc/SybaseServersDetails.txt
  ISQL_IFILE=$tmp_use_loc/ISQL_IFILE.sql
  ISQL_OFILE=$tmp_use_loc/ISQL_OFILE.txt
  DBNAMES_FILE=$tmp_use_loc/source_dbnames.txt
  LVLIST_FILE=$tmp_use_loc/lvmlist.txt
  LVLIST_FILE_tmp=$tmp_use_loc/lvmlist_tmp.txt

  if [ ! -d $tmp_loc ]; then
    mkdir -p $tmp_loc
  fi
  chmod 755 $tmp_loc

  if [ ! -d $tmp_use_loc ]; then
    mkdir -p $tmp_use_loc
    chmod 755 $tmp_use_loc
  fi

  #tmp_loc_Base=`echo $tmp_loc | cut -d/ -f2`
  #find /$tmp_loc_Base -type d -exec chmod 755 {} +
}
########################################################



######################### function: getAllSrvsDetails() ( find all sybase servers details into file given file )
getAllSrvsDetails() {
  echo "getAllSrvsDetails.........."
  local outFile=$1
  if [ ! -z $outFile ] && [ -e $outFile ]; then
    rm -f $outFile
  fi

  ps -ef | grep "bin/dataserver" | grep -v -w grep | while read line
  do
    local OSUSER=`echo $line|awk '{print $1}'`
    local Server_Name=`echo $line | awk -F"/bin/dataserver" '{print $2}' | awk -F"-s" '{print $2}' | awk '{print $1}'`
    local SybAseHome=`echo $line | awk -F"/bin/dataserver" '{print $1}' | awk '{print $NF}'`
    local MasterDAT=`echo $line | awk -F"/bin/dataserver" '{print $2}' | awk -F"-d" '{print $2}' | awk '{print $1}'`
    local SrvLog=`echo $line | awk -F"/bin/dataserver" '{print $2}' | awk -F"-e" '{print $2}' | awk '{print $1}'`
    local CfgFile=`echo $line | awk -F"/bin/dataserver" '{print $2}' | awk -F"-c" '{print $2}' | awk '{print $1}'`

    echo "OSUSER=$OSUSER INSTANCE_NAME=$Server_Name INSTANCE_HOME=$SybAseHome/..  MASTER_DAT=$MasterDAT ASE_LOG=$SrvLog ASE_CFG=$CfgFile" >> $outFile
  done

  cat $outFile | uniq > $outFile'_tmp'
  mv $outFile'_tmp' $outFile
}
########################################################



######################### function: discoverSrvDetails()
discoverSrvDetails() {
  echo "discoverSrvDetails.........."
  local srvName=$1

  ASE_SERVER=$srvName
  OSUSER=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_SERVER | awk -F"OSUSER=" '{print $2}' | awk '{print $1}'`
  OSGROUP=`id $OSUSER | awk '{print $NF}' |cut -d'(' -f2 | cut -d')' -f1`
  syb_config=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_SERVER | awk -F"ASE_CFG=" '{print $2}' | awk '{print $1}'`
  #syb_home=`su - $OSUSER -c 'echo \$SYBASE'`
  if [ "$syb_home" = "" ]; then
      syb_home=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_SERVER | awk -F"INSTANCE_HOME=" '{print $2}' | awk '{print $1}'`
  fi
  set +x
  . $syb_home/SYBASE.sh
  set -x
  SRC_SYBASE_SQLD=$SYBASE/$SYBASE_OCS
  SYBASE_VERSION=`dataserver -v |head -1| awk -F"/" '{print $2}' | awk '{print $1}'`
  MASTER_DAT=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_NAME | awk -F"MASTER_DAT=" '{print $2}' | awk '{print $1}'`
  ASE_LOG=`cat $ALL_SRV_DTLS_FILE | grep -w $ASE_NAME | awk -F"ASE_LOG=" '{print $2}' | awk '{print $1}'`
}
########################################################



######################### function: isqlExecute()
isqlExecute() {
  local iFile=$1
  local oFile=$2
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$ASE_SERVER -X -n -b -w 1000 -i $iFile -o $oFile --retserverror"
  retval=$?
  set -x
  return $retval
}
########################################################



######################### function: isqlPrepare()
isqlPrepare() {
  local iFile=$1
  local OSUSER=$2
  local PARAM=$3
  local SystemDB_List="'master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','sybsecurity','saptools','saptempdb'"

  echo "set nocount on" > $iFile
  echo "use master" >> $iFile
  echo "go" >> $iFile

  case $PARAM in
    VRFY)
      echo "go" >> $ISQL_IFILE
    ;;
    DEVICE)
      echo "select su.segmap, sda.name, reverse(substring(reverse(sde.phyname),charindex('/',reverse(sde.phyname))+1,100000)) from sysdevices sde, sysdatabases sda, master..sysusages su where su.dbid=sda.dbid and su.vdevno=sde.vdevno and sda.name not in ('tempdb','model') and sda.name not like '%pubs%' and sde.status not in (16)" >> $iFile
    ;;
    ALLDB)
      echo "select distinct name from master..sysdatabases where name not in ('tempdb','model') and name not like '%pubs%'" >> $ISQL_IFILE
    ;;
    SYSDB)
      echo "select distinct name from master..sysdatabases where name in ($SystemDB_List) and name not like '%pubs%'" >> $ISQL_IFILE
    ;;
    USERDB)
      echo "select distinct name from master..sysdatabases where name not in ($SystemDB_List,'tempdb') and name not like '%pubs%'" >> $ISQL_IFILE
    ;;
    LOGMOD)
      echo "sp_helpdb $DB_NAME" >> $ISQL_IFILE
    ;;
    BKPSRV)
      echo "select srvnetname from sysservers where srvname='SYB_BACKUP'" >> $ISQL_IFILE
    ;;
    PAGESIZE)
      echo "select @@maxpagesize" >> $ISQL_IFILE
    ;;
    CHARSORT)
      echo "sp_helpsort" >> $ISQL_IFILE
    ;;
  esac

  echo "go" >> $ISQL_IFILE

  chown $OSUSER $iFile
}
########################################################



######################### function: xml_database()
xml_database() {
  echo "xml_database.........."
  chown -R $OSUSER:$OSGROUP $tmp_use_loc

  ######################### DEVICE NAMES ###################
  isqlPrepare $ISQL_IFILE $OSUSER DEVICE
  isqlExecute $ISQL_IFILE $LVLIST_FILE
  if [ $? -ne 0 ]; then
        #echo "ERRORMSG: failed to find device names of sybase server"
        ERR_CODE=ACT_CAF_SYB_DIS_02
        return 1;
  else
    sed -i "/Your password will expire in/d" $LVLIST_FILE
  fi

  ## remove newlines and duplicate lines from lvmlist.txt
  if [ -f $LVLIST_FILE_tmp ]; then
        rm -f $LVLIST_FILE_tmp
  fi
  cat $LVLIST_FILE | sort | uniq | while read line
    do
       DEV_TYPE=`echo $line | awk '{print $1}'`
       if [ $DEV_TYPE -eq 4 ]; then
          DEV_TYPE="logvol"
       elif [ $DEV_TYPE -eq 3 ]; then
            DEV_TYPE="datavol"
       else
            DEV_TYPE="datavol"
       fi

       LVM_PATH=`echo $line | awk '{print $3}'`
       #if [ ! -z "$LVM_PATH" ]; then
         #LVM_NAME=`df -P $LVM_PATH | awk 'NR==2{print $NF}'`
         LVM_NAME=$LVM_PATH
       #fi
       DBNAME=`echo $line | awk '{print $2}'`

       echo "$DBNAME $LVM_PATH $LVM_NAME $DEV_TYPE" >> $LVLIST_FILE_tmp
    done
  mv $LVLIST_FILE_tmp $LVLIST_FILE
  chown -R $OSUSER:$OSGROUP $tmp_use_loc

  ######################## DB NAMES ###################
  isqlPrepare $ISQL_IFILE $OSUSER ALLDB
  isqlExecute $ISQL_IFILE $DBNAMES_FILE
  if [ $? -ne 0 ]; then
        #echo "ERRORMSG: failed to find database names of sybase server"
        ERR_CODE=ACT_CAF_SYB_DIS_03
        return 1
  else
    sed -i "/Your password will expire in/d" $DBNAMES_FILE
  fi

  ######################### Start writing to XML ####################
  while read line
  do
    DB_NAME=$line
    #Find SizeInBytes
    dbSizeCal "$DB_NAME"

    echo -e "\t\t<database name=\"$DB_NAME\" SIZE=\"$dbSize\" >" >> $XMLFILE

    DATAVOL=`cat $LVLIST_FILE | grep -w "$line" | head -1 | awk '{print $3}'`
    DATAVOLUMEOWNER=`ls -ld $DATAVOL | grep -v "grep" | awk '{print $3":"$4}'`

    echo -e "\t\t\t<files>" >> $XMLFILE
    if [ -f $LVLIST_FILE ]; then
      cat $LVLIST_FILE | while read line
      do
	 if [ `echo $line | awk '{print $1}'` == "$DB_NAME" ]; then
	       LVM_PATH=`echo $line | awk '{print $2}'`
               echo -e "\t\t\t\t<file path=\"$LVM_PATH\" />" >> $XMLFILE
	 fi
      done
    fi
    echo -e "\t\t\t</files>" >> $XMLFILE

    echo -e "\t\t\t<logmode>" >> $XMLFILE
    if [ ! -z $LOGMODE ]; then
      echo -e "\t\t\tmode=\"$LOGMODE\" " >>$XMLFILE
    fi
    isqlPrepare $ISQL_IFILE $OSUSER LOGMOD
    isqlExecute $ISQL_IFILE $ISQL_OFILE
    flag=`cat $ISQL_OFILE | grep -w "trunc log on chkpt" |wc -l`
    if [ "$flag" -gt 0 ]; then
          LOG_MODE="TRUNCATE ON CHKPT TRUE"
    else
          LOG_MODE="TRUNCATE ON CHKPT FALSE"
    fi
    echo -e "\t\t\t\tmode=\"$LOG_MODE\" " >>$XMLFILE
    echo -e "\t\t\t</logmode>" >> $XMLFILE

    echo -e "\t\t</database>" >>$XMLFILE
  done < $DBNAMES_FILE
}
########################################################


######################### function: xml_userdbList()
xml_userdbList() {
  isqlPrepare $ISQL_IFILE $OSUSER USERDB
  isqlExecute $ISQL_IFILE $ISQL_OFILE
  if [ $? -ne 0 ]; then
        #echo "ERRORMSG: failed to find database names of sybase server"
        ERR_CODE=ACT_CAF_SYB_DIS_04
        return 1
  else
    sed -i "/Your password will expire in/d" $ISQL_OFILE
  fi
  USERDB_LIST=`cat  $ISQL_OFILE | xargs | sed 's/ /,/g'`

  echo -e "\t\t<userdblist>" >> $XMLFILE
  echo -e "\t\t\t<database list=\"$USERDB_LIST\" />" >>$XMLFILE
  echo -e "\t\t</userdblist>" >> $XMLFILE
}
########################################################


######################### function: xml_sysdbList()
xml_sysdbList() {
  isqlPrepare $ISQL_IFILE $OSUSER SYSDB
  isqlExecute $ISQL_IFILE $ISQL_OFILE
  if [ $? -ne 0 ]; then
        #echo "ERRORMSG: failed to find database names of sybase server"
        ERR_CODE=ACT_CAF_SYB_DIS_05
        return 1
  else
    sed -i "/Your password will expire in/d" $ISQL_OFILE
  fi
  SYSTEMDB_LIST=`cat $ISQL_OFILE | xargs | sed 's/ /,/g'`

  echo -e "\t\t<sysdblist>" >> $XMLFILE
  echo -e "\t\t\t<database list=\"$SYSTEMDB_LIST\" />" >>$XMLFILE
  echo -e "\t\t</sysdblist>" >> $XMLFILE
}
########################################################



################## Main script starts here #############
prepareTempFiles
getAllSrvsDetails $ALL_SRV_DTLS_FILE

SRC_SERVER_NAME=`ps -ef | grep "bin/dataserver" |grep -v -w grep | awk -F"bin/dataserver" '{print $2}'|awk -F "-s" '{print $2}'| awk '{print $1}' | uniq |xargs`

#set DiscoveryFlag
flagAny=0
flagALL=0

echo -e "<applications>" > $XMLFILE
for ASE_NAME in ${SRC_SERVER_NAME}
do

  if [ "$SHALLOW_DISCOVERY" = "TRUE" ]; then
    shallow_discovery $ASE_NAME
    continue
  fi

  # clear Global variables
  ASE_SERVER=
  OSUSER=
  OSGROUP=
  syb_config=
  syb_home=
  SRC_SYBASE_SQLD=
  SYBASE_VERSION=
  MASTER_DAT=
  ASE_LOG=
  
  # discover details
  discoverSrvDetails $ASE_NAME
  if [ $? -ne 0 ]; then
    flagALL=1; continue
  fi
  chown -R $OSUSER:$OSGROUP $tmp_use_loc

  #Get the password
  set +x
  if [ ! -z "$SRC_USER_OTHERCRED" ]; then
    SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
      exit 1
    else
      SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
    fi
  fi
  set -x

  #Find Backup server
  BKP_SERVER=
  isqlPrepare $ISQL_IFILE $OSUSER BKPSRV
  isqlExecute $ISQL_IFILE $ISQL_OFILE
  if [ $? -ne 0 ]; then
    #echo "ERRORMSG: failed to find default backup server of sybase server"
    BKP_SERVER=
    if [ ! -z "$SRC_BACKUP_USER" ]; then
      ERR_CODE=ACT_CAF_SYB_DIS_06
    fi
  else
    sed -i "/Your password will expire in/d" $ISQL_OFILE
    BKP_SERVER=`cat $ISQL_OFILE | xargs `
  fi

  #Find Backup server user,group
  if [ ! -z "$BKP_SERVER" ]; then
    BKP_SERVER_OSUSER=`showserver | grep "S$BKP_SERVER" | awk '{print $3}'`
    BKP_SERVER_OSGROUP=`id -Gn $BKP_SERVER_OSUSER`
    BKP_SERVER_OWNER="$BKP_SERVER_OSUSER:$BKP_SERVER_OSGROUP"
  fi

  #Find pageSize
  SRC_PAGE_SIZE=
  isqlPrepare $ISQL_IFILE $OSUSER PAGESIZE
  isqlExecute $ISQL_IFILE $ISQL_OFILE
  if [ $? -ne 0 ]; then
    #echo "ERRORMSG: failed to find page_size of the server"
    SRC_PAGE_SIZE=
    if [ ! -z "$SRC_BACKUP_USER" ]; then
      ERR_CODE=ACT_CAF_SYB_DIS_07
    fi
  else
    sed -i "/Your password will expire in/d" $ISQL_OFILE
    SRC_PAGE_SIZE=`cat $ISQL_OFILE | xargs `
  fi

  #Find Character_Set and Sort_order
  CHAR_SET=
  SORT_ORD=
  isqlPrepare $ISQL_IFILE $OSUSER CHARSORT
  isqlExecute $ISQL_IFILE $ISQL_OFILE
  if [ $? -ne 0 ]; then
     CHAR_SET=
     SORT_ORD=
     ERR_CODE=ACT_CAF_SYB_DIS_09
  else
     sed -i "/Your password will expire in/d" $ISQL_OFILE
     CHAR_SET=`grep "Character Set = " $ISQL_OFILE | cut -d= -f2 | cut -d, -f1 | xargs`
     SORT_ORD=`grep "Sort Order = " $ISQL_OFILE | cut -d= -f2 | cut -d, -f1 | xargs`
  fi

  ## Capture into XML file
  echo -e "\t<application name=\"$ASE_SERVER\" friendlytype=\"$FTYPE\" SYBOSUSER=\"$OSUSER\" SRC_SYBASE_SQLD=\"$SRC_SYBASE_SQLD\" BKP_SERVER_NAME=\"$BKP_SERVER\" BKP_SERVER_OWNER=\"$BKP_SERVER_OWNER\" DATAVOLUMEOWNER=\"$OSUSER:$OSGROUP\" VERSION=\"$SYBASE_VERSION\" SYBASE_CONFIG_PATH=\"$syb_config\" SRC_PAGE_SIZE=\"$SRC_PAGE_SIZE\" CHAR_SET=\"$CHAR_SET\" SORT_ORD=\"$SORT_ORD\" >" >> $XMLFILE

  echo -e "\t\t<sybasemasterdat>" >> $XMLFILE
  echo -e "\t\t\t<file path=\"$MASTER_DAT\" />" >> $XMLFILE
  echo -e "\t\t</sybasemasterdat>" >> $XMLFILE

  echo -e "\t\t<sybaselog>" >> $XMLFILE
  echo -e "\t\t\t<file path=\"$ASE_LOG\" />" >> $XMLFILE
  echo -e "\t\t</sybaselog>" >> $XMLFILE

  ## need sybase database username and password
  if [ ! -z "$SRC_BACKUP_USER" ]; then

    #Verify provided username and password
    isqlPrepare $ISQL_IFILE $OSUSER VRFY
    isqlExecute $ISQL_IFILE $LVLIST_FILE
    if [ $? -ne 0 ]; then
      #echo "ERRORMSG: The database username or password is incorrect."
      ERR_CODE=ACT_CAF_SYB_DIS_01
      echo -e "\t</application>" >>$XMLFILE
      flagALL=1; continue
    else
      sed -i "/Your password will expire in/d" $LVLIST_FILE
    fi

    #source $syb_home/SYBASE.sh
    xml_database
    if [ $? -ne 0 ]; then
      echo -e "\t</application>" >>$XMLFILE
      flagALL=1; continue
    fi

    echo -e "\t\t<files>" >> $XMLFILE
    cat $LVLIST_FILE | awk '{print $2" "$3" "$4}' | sort | uniq | while read line
    do
       dirpath=`echo $line | awk '{print $1}'`
       lvmtype=`echo $line | awk '{print $3}'`
       lvmname=`echo $line | awk '{print $2}'`
       echo -e "\t\t\t<file path=\"$dirpath\" $lvmtype=\"$lvmname\" />" >> $XMLFILE
    done
    echo -e "\t\t</files>" >> $XMLFILE

    echo -e "\t\t<scripts>" >> $XMLFILE
    echo -e "\t\t\t<script phase=\"all\" path=\"$SCRIPTS\" />" >> $XMLFILE
    echo -e "\t\t</scripts>" >> $XMLFILE

    xml_userdbList
    if [ $? -ne 0 ]; then
      echo -e "\t</application>" >>$XMLFILE
      flagALL=1; continue
    fi

    xml_sysdbList
    if [ $? -ne 0 ]; then
      echo -e "\t</application>" >>$XMLFILE
      flagALL=1; continue
    fi

    # At-least one discovery is success
    flagAny=1

  fi

  echo -e "\t</application>" >>$XMLFILE
done
echo -e "</applications>" >>$XMLFILE

mv $XMLFILE $XMLFILE_ORIG

if [ $flagAny -eq 0 ]; then
  echo "ERRORMSG: All applications discovery failed."
  displayError $ERR_CODE
  exit 1
elif [ $flagALL -ne 0 ]; then
  echo "Warning: Some applications discovery failes."
else
  echo "All applications are discovered successfully"
fi

## Remove temp files and directories
if [ -e $ALL_SRV_DTLS_FILE ]; then
  rm -f $ALL_SRV_DTLS_FILE
fi

if [ -d $tmp_use_loc ]; then
  rm -rf $tmp_use_loc
fi
