#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

function usage()
{
    echo "Please set environment vars ACT_NAME ACT_FRIENDLYTYPE ACT_PHASE and re-run the script"
    echo "Supported phases [prepare|freeze|unfreeze|abort|predump|dbdump|cleanup|premount|postmount|postunmount|prerestore|postrestore|predumprestore|postdumprestore|premounttest|prerestoretest|predumprestoretest|logpurge|logbackup]"
    return 0;
}

function cleanup_func()
{
    echo "**** Cleanup phase started ****"

    /act/custom_apps/mariadb/act_mariadb_abort.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* CLEANUP ERROR: Failed mariadb snapshot cleanup for $DATADIR ************"
      return $retval
    fi

    echo "**** Cleanup phase ended ****"
    return 0;
}

function prepare_func()
{
    echo "**** Prepare phase started ****";

    /act/custom_apps/mariadb/act_mariadb_prepare.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PREPARE ERROR: Failed mariadb prepare snapshot for $DATADIR ************"
      return $retval
    fi

    echo "**** Prepare phase ended ****";
    return 0;
}

function freeze_func()
{
    echo "**** Freeze phase started ****"

    /act/custom_apps/mariadb/act_mariadb_pre.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE ERROR: Failed mariadb create snapshot for $DATADIR ************"
      return $retval
    fi

    echo "**** Freeze phase ended ****"
    return 0;
}

function unfreeze_func()
{
    echo "**** UN_Freeze phase started ****"

    /act/custom_apps/mariadb/act_mariadb_post.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE ERROR: Failed mariadb unfreeze for create snapshot for $DATADIR ************"
      return $retval
    fi

    echo "**** UN_Freeze phase ended ****"
    return 0;
}

function abort_func()
{
    echo "**** abort phase started ****"

    /act/custom_apps/mariadb/act_mariadb_abort.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE ERROR: Failed mariadb abort for create snapshot for $DATADIR ************"
      return $retval
    fi

    echo "**** abort phase ended ****"
    return 0;
}

function premount_func()
{
    echo "**** Premount phase started ****"

    echo "********** PRE: starting mariadb premount for $TARGET_SERVER_NAME  ************"
    /act/custom_apps/mariadb/clone/ACT_MARIADB_premount.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE MOUNT ERROR: Failed mariadb premount before create Appaware mount for $TARGET_SERVER_NAME ************"
      return $retval
    fi

    echo "**** Premount phase ended ****"
    return 0;
}

function predump_func()
{
    echo "**** Predump phase started ****"

    /act/custom_apps/mariadb/dump/act_mariadb_predump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE ERROR: Some environment variables are missing or invalied for mariadb DUMP backup"
      exit 1
    fi

    echo "**** Predump phase ended ****"
    return 0
}

function dbdump_func()
{
    echo "**** dbdump phase started ****"

    /act/custom_apps/mariadb/dump/act_mariadb_dump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* DBDUMP ERROR: Failed mariadb dbdump for $DATADIR ************"
      return $retval
    fi

    echo "**** dbdump phase ended ****"
    return 0;
}

function postunmount_func
{
    echo "**** Post unmount phase started ****"
    ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$JOBID
    if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
      rm -f $ACT_PREMOUNT_FAILURE_FILE
    fi
    echo "**** Post unmount phase ended ****";

    return 0;
}

function postmount_func()
{
    echo "**** Postmount phase started ****"

    /act/custom_apps/mariadb/clone/ACT_MARIADB_postmount.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* POST MOUNT ERROR: Failed mariadb postmount for $TARGET_SERVER_NAME *************"
      return $retval
    fi

    echo "**** Postmount phase ended ****"
    return 0;
}

function premounttest_func()
{
    echo "**** Premount test phase started ****"

    /act/custom_apps/mariadb/clone/ACT_MARIADB_premount_test.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PRE-MOUNT-TEST ERROR: Failed mariadb premount for $TARGET_SERVER_NAME *************"
      return $retval
    fi

    echo "**** Premount test phase ended ****"
    return 0;
}

function predumprestoretest_func()
{
    echo "**** Pre-dumprestore test phase started ****"
    /act/custom_apps/mariadb/dump/ACT_MARIADB_predumprestore_test.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRE DUMP RESTORE: Failed mariadb dumprestore in predumprestore phase *****"
      return $retval
    fi
    echo "**** Pre-dumprestore test phase ended ****";
    return 0;
}

function prerestoretest_func()
{
    echo "**** Pre-restore test phase started ****"
    /act/custom_apps/mariadb/dump/ACT_MARIADB_predumprestore_test.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRE DUMP RESTORE: Failed mariadb dumprestore in predumprestore phase *****"
      return $retval
    fi
    echo "**** Pre-restore test phase ended ****";
    return 0;
}

function prerestore_func()
{
    echo "**** Prerestore phase started ****"
    /act/custom_apps/mariadb/restore/ACT_MARIADB_prerestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRE RESTORE: Failed mariadb prerestore *****"
      return $retval
    fi
    echo "**** Prerestore phase ended ****"
    return 0;
}

function postrestore_func()
{
    echo "**** Postresotre phase started ****"
    /act/custom_apps/mariadb/restore/ACT_MARIADB_postrestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** POST RESTORE: Failed mariadb restore in postrestore phase *****"
      return $retval
    fi
    echo "**** Postresotre phase ended ****"
    return 0;
}

function predumprestore_func()
{
    echo "**** PreDumpRestore phase started ****"
    /act/custom_apps/mariadb/dump/ACT_MARIADB_predumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRE DUMP RESTORE: Failed mariadb dumprestore in predumprestore phase *****"
      return $retval
    fi
    echo "**** PreDumprestore phase ended ****"
    return 0;
}

function postdumprestore_func()
{
    echo "**** PostDumpResotre phase started ****"
    /act/custom_apps/mariadb/dump/ACT_MARIADB_dumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** POST DUMP RESTORE: Failed mariadb dump restore in postdumprestore phase *****"
      return $retval
    fi
    echo "**** PostDumpResotre phase ended ****"
    return 0;
}

function logbackup_func()
{
    echo "**** log backup phase started ****"

    /act/custom_apps/mariadb/act_mariadb_logbackup.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* LOG ERROR: Failed mariadb log backup for $DATADIR ************"
        return $retval
    fi

    echo "**** log backup phase ended ****"
    return 0;
}

function logpurge_func()
{
    echo "**** log purge phase started ****"
    /act/custom_apps/mariadb/act_mariadb_logpurge.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** LOG PURGE ERROR: failed to purge mariadb log files for $DATADIR *****"
      return $retval
    fi
    echo "**** log purge phase ended ****"
    return 0;
}

if [ -n $ACT_PHASE ] 
then
   PARAM=$ACT_PHASE

   case $PARAM in 
     cleanup)
         echo "Executing cleanup phase of the script"
         cleanup_func
         retval=$?
         ;;
     prepare)
         echo "Executing prepare phase of the script"
         prepare_func
         retval=$?
         ;;
     freeze)
         echo "Executing freeze phase of the script"
         freeze_func
         retval=$?
         ;;
     unfreeze)
         echo "Executing unfreeze phase of the script"
         unfreeze_func
         retval=$?
         ;;
     predump)
         echo "Executing predump phase of the script"
         predump_func
         retval=$?
         ;;
     dbdump)
         echo "Executing dbdump phase of the script"
         dbdump_func
         retval=$?
         ;;
     prerestoretest)
         echo "Executing prerestore test phase of the script"
         prerestoretest_func
         retval=$?
         ;;
     predumprestoretest)
         echo "Executing predumprestore test phase of the script"
         predumprestoretest_func
         retval=$?
         ;;
     prerestore)
         echo "Executing prerestore phase of the script"
         prerestore_func
         retval=$?
         ;;
     postrestore)
         echo "Executing postrestore phase of the script"
         postrestore_func
         retval=$?
         ;;
     predumprestore)
         echo "Executing prerestore phase of the script"
         predumprestore_func
         retval=$?
         ;;
     postdumprestore)
         echo "Executing postrestore phase of the script"
         postdumprestore_func
         retval=$?
         ;;
     premounttest)
         echo "Executing premount test phase of the script"
         premounttest_func
         retval=$?
         ;;
     premount)
	 if [ "$ACT_JOBTYPE" = "unmount" ]; then
           echo "Executing premount phase of the script"
           premount_func
           retval=$?
	 fi
         ;;
     postunmount)
         echo "Executing post unmount phase of the script"
         postunmount_func
         retval=$?
         ;;
     postmount)
         echo "Executing postmount phase of the script"
         postmount_func
         retval=$?
         ;;
      logbackup)
         echo "Executing logbackup phase of the script"
         logbackup_func
         retval=$?
         ;;
      logpurge)
	 echo "Executing logpurge phase of the script"
	 logpurge_func
	 retval=$?
         ;;
      abort)
         echo "Executing abort phase of the script"
         abort_func
         retval=$?
         ;;           
     *)
         usage
   esac 
else 
    usage
fi

exit $retval;
