#!/bin/sh

set -x
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
BASEDIR=$BASEDIR
SOCKETFILE=$SOCKET_FILE
SRV_TYPE=$SRV_TYPE
LOGBKPLOC=$LOGBKPLOC

BEGINPIT=$BEGINPIT     # Previous DB beginpit

# Get Common functions
. /act/custom_apps/mariadb/act_mariadb_common_functions.sh

#Preflight check for log backup
if [ "$LOGBKPLOC" = "" ] || [ ! -d "$LOGBKPLOC" ]; then
  echo "ERRORMSG: LOGBKPLOC is not provided or it is not a directory"
  exit 1
fi

# Prepare MySQL Command
set +x
prepare_MARIADBCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

#Check connection to MySQL
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: failed to connect to MariaDB"
  exit 1
fi

# get binary logfile, location
binaryLogFile=`getMySqlVarValue log_bin_index`
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to find logfiles location"
  exit 1
fi
if [ -z $binaryLogFile ]; then
  echo "ERRORMSG: log_bin is disabled. Please enable log_bin and try"
  exit 1
fi
binaryLogFileLoc=`dirname $binaryLogFile`
echo "binaryLogFileLoc=$binaryLogFileLoc  File=$binaryLogFile"


# Get the log start position
DATE="$(date +'%m%d%Y%H%M%S')"
logPos=`su -m $OSUSER -c "$MARIADB_CMD -e \"show master status; \" " | awk '{print $2}'`
echo "LogPosition: $logPos"


# Flush the log file before backup
execMySQL "flush logs"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to flush binary log"
  exit 1
fi


# backup the logfile
lastlogfile=`tail -1 $binaryLogFile`
for i in `cat $binaryLogFile`
do
  prefix=
  if [ "${i:0:1}" = "." ]; then
    prefix=`getMySqlVarValue datadir`
  fi

  srcLogFile=$prefix$i
  srcLogFileName=`basename $srcLogFile`
  trgLogFile=$LOGBKPLOC/$srcLogFileName

  ##Skip if file is before begin time
  if [ ! -z "$BEGINPIT" ]; then
    isOldFile=`find $srcLogFile -newermt "$BEGINPIT" |wc -l`
    if [ $isOldFile -eq 0 ]; then
      continue
    fi
  fi

  ##Skip if file already copied
  if [ -e "$trgLogFile" ]; then
    srcSize=`ls -l $srcLogFile | awk '{print $5}'`
    trgSize=`ls -l $trgLogFile | awk '{print $5}'`
    if [ $srcSize -eq $trgSize ]; then
      continue
    fi
  fi

  cp -p $prefix$i $LOGBKPLOC/

  if [ "$i" = "$lastlogfile" ]; then
    break;
  fi
done

exit 0
