#!/bin/sh

set -x
SRC_DATADIR=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
SRC_TMPDIR=$SRC_TMPDIR
MESSAGES_DIR=$MESSAGES_DIR
TARGET_MNT_PNT=$TARGET_MNT_PNT
BASEDIR=$BASEDIR
PORT=$PORT
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

# Get Common functions
. /act/custom_apps/mariadb/act_mariadb_common_functions.sh

#hidden file to handel Mount and Migrate
MARIADB_MIGRATE_CONF=/act/touch/.ACT_MARIADB_migrate_"$PORT".conf

#Check unmout database required or not
if [ -f $MARIADB_MIGRATE_CONF ]; then
  PVMOVE_STATUS=`cat $MARIADB_MIGRATE_CONF | awk -F "=" '{print $2}'`
  if [ "$PVMOVE_STATUS" = "YES" ]; then
    echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
    rm -f $MARIADB_MIGRATE_CONF
    exit 0
  else
    echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
    exit 1
  fi
fi

SOCKETFILE=`ps -ef | grep -w "\-\-port=$PORT" | awk -F"--socket=" '{print $2}' | awk '{print $1}'`
if [ ! -e $SOCKETFILE ] || [ -z "$SOCKETFILE" ]; then
  echo "MariaDB Server not running."
  exit 0
fi

# Prepare MariaDB Command
set +x
prepare_MARIADBCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

## Shutdown Server
su -m $OSUSER -c "$MARIADB_ADMIN shutdown"
retval=$?

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to shutdown the mariadb server with socket $SOCKETFILE"
  exit 1
fi

isSrvRunning=`ps -ef | grep "mysqld\|mariadbd" | grep -w $PORT | wc -l `
if [ $isSrvRunning -eq 1 ]; then
  echo "ERRORMSG: Failed to shutdown MariaDB server !"
  exit 1
fi

#Check with netstat for port number as well
isSrvRunning=`netstat -nalp | grep "mysqld\|mariadbd" | grep -w LISTEN | awk '{print $4}' | cut -d: -f4 | grep -w $PORT | wc -l`
if [ $isSrvRunning -eq 1 ]; then
  echo "ERRORMSG: Failed to shutdown MariaDB server !"
  exit 1
fi

echo "MariaDB server shutdown successfull"

exit 0
