#!/bin/sh

set -x
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
BASEDIR=$BASEDIR
SOCKETFILE=$SOCKET_FILE
SRV_TYPE=$SRV_TYPE
USER_DBLIST="$USER_DBLIST"

DUMPSCHEDULE=$DUMPSCHEDULE
DUMPBKPLOC=$DUMPBKPLOC

# Validate env
if [ "x$OSUSER" = "x" ]; then
  echo "ERRORMSG: OSUSER is not provided"
  exit 1
fi

if [ "x$DBUSER" = "x" ]; then
  echo "WARNINGMSG: DBUSER is not provided. Considering default user 'root' "
fi

if [ "x$BASEDIR" = "x" ]; then
  echo "ERRORMSG: BASEDIR is not provided"
  exit 1
fi

if [ "x$SOCKETFILE" = "x" ]; then
  echo "ERRORMSG: SOCKETFILE is not provided"
  exit 1
fi

if [ "x$SRV_TYPE" = "x" ]; then
  echo "ERRORMSG: SRV_TYPE is not provided"
  exit 1
fi

if [ "x$USER_DBLIST" = "x" ]; then
  echo "ERRORMSG: USER_DBLIST is not provided"
  exit 1
fi

# Verify 
if [ ! -e $SOCKETFILE ]; then
  echo "ERRORMSG: SOCKETFILE ($SOCKETFILE) not exists."
  exit 1
fi

if [ ! -d $DUMPBKPLOC ]; then
  echo "ERRORMSG: DUMPBKPLOC ($DUMPBKPLOC) not exists or not a directory."
  exit 1
fi

# Get Common functions
. /act/custom_apps/mariadb/act_mariadb_common_functions.sh

# Prepare MARIADB Command
set +x
prepare_MARIADBCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

DBSIZE_Total=0
# check if databases exists or not
for dbName in $(echo $USER_DBLIST | sed 's/,/ /g')
do
  su -m $OSUSER -c "$MARIADB_CMD -D $dbName -e \"select 'yes' from dual;\" "
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Database $dbName not found."
    exit 1
  fi

  dbSize=`su $OSUSER -c "$MARIADB_CMD -e \"SELECT SUM(data_length + index_length) FROM information_schema.TABLES where table_schema='$dbName';\" "`
  if [ "$dbSize" = "NULL" ]; then
    dbSize=0;
  fi
  echo "$dbName size: $dbSize"
  DBSIZE_Total=`expr $DBSIZE_Total "+" $dbSize`
done

DUMPSCHEDULE_DB=F
echo "DBDUMPSIZE=$DBSIZE_Total"
echo "DBDUMPTYPE=$DUMPSCHEDULE_DB"

exit 0
