#!/bin/sh

set -x
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
BASEDIR=$BASEDIR
SOCKETFILE=$SOCKET_FILE
SRV_TYPE=$SRV_TYPE
DATADIR=$DATADIR
PORT=$PORT
CONFIG_FILE=$CONFIG_FILE
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

LOGBIN_LOC=$LOGBIN_LOC
PIDFILE=$PIDFILE
LOGERROR=$LOGERROR
SERVRID=$SERVRID
SRC_TMPDIR=$TMPDIR


# Get Common functions
. /act/custom_apps/mariadb/act_mariadb_common_functions.sh

OSGROUP=`id -gn $OSUSER`

# exit, if config file not provided
if [ "$CONFIG_FILE" = "" ]; then
  echo "ERRORMSG: CONFIG_FILE not provided"
  exit 1
fi

MARIADB_CNF=$DATADIR/mariadb_$PORT.cnf
if [ -e $MARIADB_CNF ]; then
  mv $MARIADB_CNF $MARIADB_CNF.bkp.$(date +'%Y%m%d_%H%M%S')
fi
cp $CONFIG_FILE $MARIADB_CNF
chown $OSUSER:$OSGROUP $MARIADB_CNF

#Set env for mysqld_safe
export PATH=$BASEDIR/bin:$PATH
export PATH=$PATH:/sbin:$BASEDIR/sbin
MYSQLD_SAFE=mysqld_safe
which mysqld_safe
if [ $? -ne 0 ]; then
  MYSQLD_SAFE=mysqld
  which mysqld
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to find mysqld(_safe) binary"
    exit 1
  fi
fi

getErrorLog "$MARIADB_CNF" LOGERROR
if [ $? -ne 0 ]; then
   echo "WARNINGMSG: Fail to find log_error file"
else
   mv $LOGERROR ${LOGERROR}_bkp$(date +%Y%m%d%H%M%S)
fi

## Start server
su -m $OSUSER -c "$MYSQLD_SAFE --defaults-file=$MARIADB_CNF & "
retval=$?

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to start mariadb server !"
  exit 1
fi

#Check MariaDB started successfully or not
sleep 10
isSrvStarted=`ps -ef | grep "mysqld\|mariadbd" | grep -w "\-\-defaults-file=$MARIADB_CNF" | grep -vw grep |wc -l `
if [ $isSrvStarted -lt 1 ]; then
   echo "ERRORMSG: Failed to bringup the MariaDB server !"
   exit 1
fi

if [ -z "$LOGERROR" ]; then
   sleep 30
else
   waitForMariaDB "$LOGERROR" 10
   if [ $? -ne 0 ]; then
      echo "ERRORMSG: Failed to bringup the MariaDB server !"
      cp $LOGERROR /act/touch/errorLog_$ACT_JOBNAME.log
      exit 1
   fi
fi

# Prepare MariaDB Command
set +x
prepare_MARIADBCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

# Check server is running or not
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to bringup the MariaDB server !"
  exit 1
fi
echo "MariaDB server started successfully"

# Recovery if needed
if [ ! -z "$UNTIL_TIME" ]; then
  /act/custom_apps/mariadb/clone/ACT_MARIADB_recover.sh $SOCKETFILE
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to recover MariaDB server!"
    exit 1
  fi
fi

exit 0
