#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x
export COLUMNS=4000
unset LANG
unset LC_ALL

function usage()
{
    echo "Please set environment vars ACT_NAME ACT_FRIENDLYTYPE ACT_PHASE and re-run the script"
    echo "Supported phases [prepare|freeze|unfreeze|abort|predump|dbdump|cleanup|premount|postmount|postunmount|prerestore|postrestore|predumprestore|postdumprestore|premounttest|prerestoretest|predumprestoretest|logbackup|logpurge]"
    return 0;
}

function cleanup_func()
{
    echo "**** Cleanup phase started ****"
    /act/custom_apps/maxdb/act_maxdb_abort.sh     
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRE ERROR: Failed maxdb abort for $SRC_SERVER_NAME ************"
        return $retval
    fi
    echo "**** Cleanup phase ended ****"
    return 0;
}

function prepare_func()
{
    echo "**** Prepare phase started ****";

    /act/custom_apps/maxdb/act_maxdb_prepare.sh
    retval=$?
    if [ $retval -ne 0 ]; then
       echo "******* PREPARE ERROR: Failed maxdb prepare snapshot for $SRC_SERVER_NAME ************"
       return $retval
    fi

    echo "**** Prepare phase ended ****";
    return 0;
}

function freeze_func()
{
        echo "**** Freeze phase started ****"

        echo "********** PRE: starting maxdb create snapshot for $SRC_SERVER_NAME  ************"
        /act/custom_apps/maxdb/act_maxdb_pre.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* PRE ERROR: Failed maxdb create snapshot for $SRC_SERVER_NAME ************"
                return $retval
        fi

        echo "********** end PRE: maxdb create snapshot for $SRC_SERVER_NAME ************"

    	return 0;
}

function unfreeze_func()
{
        echo "**** UN_Freeze phase started ****"

        echo "********** PRE: starting maxdb unfreeze for create snapshot for $SRC_SERVER_NAME  ************"
        /act/custom_apps/maxdb/act_maxdb_post.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* PRE ERROR: Failed maxdb unfreeze for create snapshot for $SRC_SERVER_NAME ************"
                return $retval
        fi

        echo "********** end POST: maxdb create snapshot for $SRC_SERVER_NAME ************"
        return 0;

}

function abort_func()
{
        echo "**** abort phase started ****"

        echo "********** PRE: starting maxdb abort for create snapshot for $SRC_SERVER_NAME  ************"
        /act/custom_apps/maxdb/act_maxdb_abort.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* PRE ERROR: Failed maxdb abort for create snapshot for $SRC_SERVER_NAME ************"
                return $retval
        fi

        echo "********** end POST: maxdb abort for snapshot for $SRC_SERVER_NAME ************"
        return 0;

}

function premount_func()
{
    echo "**** Premount phase started ****"
        echo "********** PRE: starting maxdb premount for $TARGET_SERVER_NAME  ************"
	/act/custom_apps/maxdb/clone/ACT_MAXDB_premount.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* PRE MOUNT ERROR: Failed maxdb premount before create Appaware mount for $TARGET_SERVER_NAME ************"
                return $retval
	fi
    echo "**** Premount phase ended ****"
    return 0;
}

function predump_func()
{
	echo "**** Predump phase started ****"

	echo "********** PRE: starting maxdb create snapshot for $SRC_SERVER_NAME with dumps ************"
	/act/custom_apps/maxdb/dump/act_maxdb_predump.sh
	retval=$?
	if [ $retval -ne 0 ]; then
		echo "******* PRE ERROR: Some environment variables are missing or invalied for maxdb DUMP backup"
		return $retval
	fi

	echo "**** end PREDUMP: Predump phase ended ****"
        return 0
}

function dbdump_func()
{
	echo "**** dbdump phase started ****"

	echo "********** PRE: starting maxdb dbdump for $SRC_SERVER_NAME  ************"
	/act/custom_apps/maxdb/dump/act_maxdb_dump.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* DBDUMP ERROR: Failed maxdb dbdump for $SRC_SERVER_NAME ************"
                return $retval
        fi

	echo "**** end DBDUMP: dbdump phase ended ****"
	return 0;
}

function postmount_func()
{
    echo "**** Postmount phase started ****"
        echo "********** PRE: starting maxdb post moount for $TARGET_SERVER_NAME *************"
	/act/custom_apps/maxdb/clone/ACT_MAXDB_postmount.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* POST MOUNT ERROR: Failed maxdb postmount for $TARGET_SERVER_NAME *************"
                return $retval
        fi

    echo "**** Postmount phase ended ****"
    return 0;
}

function postunmount_func()
{
    echo "**** Postunmount phase started ****"
        echo "********** PRE: starting maxdb post unmount for $TARGET_SERVER_NAME *************"
        /act/custom_apps/maxdb/clone/ACT_MAXDB_postunmount.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* POST UNMOUNT ERROR: Failed maxdb postunmount for $TARGET_SERVER_NAME *************"
                return $retval
        fi

    echo "**** Postunmount phase ended ****"
    return 0;
}

function premounttest_func()
{
    echo "**** Pre-mount test phase started ****"
        echo "********** PRE: premount-test for $TARGET_SERVER_NAME *************"
        /act/custom_apps/maxdb/clone/ACT_MAXDB_premount_test.sh
        retval=$?
        if [ $retval -ne 0 ]; then
                echo "******* PRE-MOUNT-TEST ERROR: Failed maxdb postmount for $TARGET_SERVER_NAME *************"
                return $retval
        fi


    echo "**** Premount test phase ended ****"
    return 0;
}

function predumprestoretest_func()
{
    echo "**** Pre-dumprestore test phase started ****"
    echo "**** Pre-dumprestore test phase ended ****";

    return 0;
}

function prerestoretest_func()
{
    echo "**** Pre-restore test phase started ****"
    echo "**** Pre-restore test phase ended ****";

    return 0;
}

function prerestore_func()
{

    echo "**** Prerestore phase started ****"
    /act/custom_apps/maxdb/restore/ACT_MAXDB_prerestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PRERESTORE ERROR: Failed maxdb LVM restore ************"
        return $retval
    fi
    echo "**** Prerestore phase ended ****"
    return 0;
}

function postrestore_func()
{

    echo "**** Postrestore phase started ****"
    /act/custom_apps/maxdb/restore/ACT_MAXDB_postrestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POSTRESTORE ERROR: Failed maxdb LVM restore ************"
        return $retval
    fi
    echo "**** Postrestore phase ended ****"
    return 0;

}

function predumprestore_func()
{

    echo "**** PreDumpRestore phase started ****"
    /act/custom_apps/maxdb/dump/ACT_MAXDB_predumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* PREDUMPRESTORE ERROR: Failed maxdb predump restore ************"
        return $retval
    fi    
    echo "**** PreDumprestore phase ended ****"
    return 0;

}

function postdumprestore_func()
{
    echo "**** PostDumpRestore phase started ****"
    /act/custom_apps/maxdb/dump/ACT_MAXDB_dumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* POSTDUMPRESTORE ERROR: Failed maxdb postdump restore ************"
        return $retval
    fi
    echo "**** Postrestore phase ended ****"
    return 0;
}

function logbackup_func()
{

    echo "**** log backup started ****"
    /act/custom_apps/maxdb/act_maxdb_logbackup.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* LOG ERROR: Failed maxdb log backup for $SRC_SERVER_NAME ************"
        return $retval
    fi

    echo "**** log backup ended ****"
    return 0;
}

function logpurge_func()
{

    echo "**** log purge started ****"
    /act/custom_apps/maxdb/act_maxdb_purgelog.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* LOG ERROR: Failed maxdb log purge for $SRC_SERVER_NAME ************"
        return $retval
    fi

    echo "**** log purge ended ****"
    return 0;
}

if [ -n $ACT_PHASE ] 
then
   PARAM=$ACT_PHASE

   case $PARAM in 
     cleanup)
         echo "Executing cleanup phase of the script"
         cleanup_func
         retval=$?
         ;;
     prepare)
         echo "Executing prepare phase of the script"
         prepare_func
         retval=$?
         ;;
     freeze)
         echo "Executing freeze phase of the script"
         freeze_func
         retval=$?
         ;;
     unfreeze)
         echo "Executing unfreeze phase of the script"
         unfreeze_func
         retval=$?
         ;;
     predump)
         echo "Executing predump phase of the script"
         predump_func
         retval=$?
         ;;
     dbdump)
         echo "Executing dbdump phase of the script"
         dbdump_func
         retval=$?
         ;;
     predumprestoretest)
         echo "Executing predumprestore test phase of the script"
         predumprestoretest_func
         retval=$?
         ;;
     prerestoretest)
         echo "Executing prerestore test phase of the script"
         prerestoretest_func
         retval=$?
         ;;
     prerestore)
         echo "Executing prerestore phase of the script"
         prerestore_func
         retval=$?
         ;;
     postrestore)
         echo "Executing postrestore phase of the script"
         postrestore_func
         retval=$?
         ;;
     predumprestore)
         echo "Executing prerestore phase of the script"
         predumprestore_func
         retval=$?
         ;;
     postdumprestore)
         echo "Executing postrestore phase of the script"
         postdumprestore_func
         retval=$?
         ;;
     premounttest)
         echo "Executing premount test phase of the script"
         premounttest_func
         retval=$?
         ;;
     premount)
         if [ "$ACT_JOBTYPE" = "unmount" ]; then
           echo "Executing premount phase of the script"
           premount_func
           retval=$?
         fi
         ;;
     postmount)
         echo "Executing postmount phase of the script"
         postmount_func
         retval=$?
         ;;
     postunmount)
         echo "Executing postunmount phase of the script"
         postunmount_func
         retval=$?
         ;;
      logbackup)
         echo "Executing logbackup phase of the script"
         logbackup_func
         retval=$?
         ;;
      logpurge)
         echo "Executing logpurge phase of the script"
         logpurge_func
         retval=$?
         ;;
      abort)
         echo "Executing abort phase of the script"
         abort_func
         retval=$?
         ;;           
     *)
         usage
   esac 
else 
    usage
fi

exit $retval;
