#!/bin/sh


#-----------------------------------------------------------------
# function: isVarNull()
#-----------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    exit 1
  fi
}
#-----------------------------------------------------------------


#-----------------------------------------------------------------
# function: removeFile()
#-----------------------------------------------------------------
removeFile() {
  local fileName=$1
  if [ -e "$fileName" ]; then
    rm -f $fileName
  fi
}
#-----------------------------------------------------------------


#-----------------------------------------------------------------
# function: prepareMaxdbKey()
#-----------------------------------------------------------------
prepareMaxdbKey() {
  local username=$1
  set +x
  local passwd="'$2'"
  set -x
  MAXDB_KEY=

  set +x
  if [ -z $passwd ]; then
    MAXDB_KEY=" -U $username"
  else
    MAXDB_KEY=" -u $username,$passwd"
  fi
  set -x
}
#-----------------------------------------------------------------


#-----------------------------------------------------------------
# function: checkAutologBkp()
#-----------------------------------------------------------------
checkAutologBkp() {
  local osuser=$1
  local dbName=$2
  local maxKey=$3

  set +x
  AUTOLOG_BKP_STATE=`su -m $osuser -c "dbmcli -d $dbName $maxKey autolog_show" | grep -vw "OK"`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: fail to get autolog state"
    exit 1
  fi

  if ! [ "$AUTOLOG_BKP_STATE" = "AUTOSAVE IS ON" ]; then
    echo "ERRORMSG: Log Backup Failed as AutoLog Backup is OFF"
    exit 1
  fi

  return 0
}
#-----------------------------------------------------------------


#-----------------------------------------------------------------
# function: getAutologbkpLoc()
#-----------------------------------------------------------------
getAutologbkpLoc() {
  local osuser=$1
  local dbName=$2
  local maxKey=$3
  AUTOLOG_TMPLT=
  ArchiveLog_Loc_base=

  #Get backup template
  set +x
  AUTOLOG_TMPLT=`su -m $osuser -c "dbmcli -d $dbName $maxKey backup_history_list -I -c label,media,action" | grep "LOG_" | grep -w "SAVE" | head -1 | cut -d'|' -f2`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get log backup template"
    exit 1
  fi

  #Get archivelog location
  set +x
  ArchiveLog_Loc_base=`su -m $osuser -c "dbmcli -d $dbName $maxKey backup_template_show NAME $AUTOLOG_TMPLT" |grep LOCATION | awk '{print $NF}'`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get log backup location"
    exit 1
  fi

  if [ "${ArchiveLog_Loc_base:0:1}" != "/" ]; then
    runDir=`dbmcli -d $dbName $maxKey param_directget RUNDIRECTORY | grep -v "OK" | awk '{ print $NF }'`
    ArchiveLog_Loc_base=$runDir/$ArchiveLog_Loc_base
  fi

  echo "Auto log backup template: $AUTOLOG_TMPLT"
  echo "Auto log backup base location: $ArchiveLog_Loc_base"
}



#-----------------------------------------------------------------
# function: getDB_State()
#-----------------------------------------------------------------
getDB_State() {
  local osuser=$1
  local dbName=$2
  local maxKey=$3
  DB_STATE=

  #Get db state
  set +x
  DB_STATE=`su -m $osuser -c "dbmcli -d $dbName $maxKey db_state" | grep -vw "OK" | grep -vw "State"`
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get database state ($dbName)"
    exit 1
  fi
}

#-----------------------------------------------------------------
# function: getDBName_charCount()
#-----------------------------------------------------------------
getDBName_charCount() {
  dbsid=$1
  char_count=`echo -n "$dbsid" | wc -c`
  if [ $char_count -gt 8 ]; then
    echo "ERRORMSG: DBName should be less than equal to 8 charachers"
    exit 1
  fi
}
#-----------------------------------------------------------------


#-----------------------------------------------------------------
# function: recov_check()
#-----------------------------------------------------------------
recov_check() {
  local logfile=$1
  local rec_check_input=/act/touch/$ACT_JOBNAME/rec_check_input.sql
  local rec_check_out=/act/touch/$ACT_JOBNAME/rec_check_out.txt

  #Create backup template with logfile
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c backup_template_create recov_test to file $logfile content log"
  set -x
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to create restore template "
    exit 1
  fi

  #Prepare query for recovery check
  echo "service_connect" > $rec_check_input
  echo "recover_check recov_test" >> $rec_check_input
  chown -R $OSUSER:$OSGROUP $rec_check_input

  #Recovery_check
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -i $rec_check_input  -o $rec_check_out"
  retval=$?
  set -x
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to check recovery status of log file $logfile"
    exit 1
  fi

  F_SEQ=`cat $rec_check_out | grep -w "First LOG Page" | awk '{print $NF}'`
  L_SEQ=`cat $rec_check_out | grep -w "Last LOG Page" | awk '{print $NF}'`
  return 0
}
#-----------------------------------------------------------------
