#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD="$SRC_USER_PASSWD"
set -x
MAXDB_BACKUPPATH=$MAXDB_BACKUPPATH
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
SRC_DB_VERSION=$SRC_DB_VERSION
ACT_JOBNAME=$JOBID

BEGINPIT=$BEGINPIT     # Previous DB beginpit

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

isVarNull OSUSER $OSUSER
isVarNull DBSID $DBSID
isVarNull SRC_BACKUP_USER $SRC_BACKUP_USER
#isVarNull SRC_USER_PASSWD $SRC_USER_PASSWD
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH
isVarNull MAXDB_BACKUPPATH $MAXDB_BACKUPPATH

#Check Backup Location.....
if [ ! -d $MAXDB_BACKUPPATH ]; then
  echo "ERRORMSG: Backup Direcroy Does not exist"
  exit 1
fi

#Convert the DBSID in Upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

OSGROUP=`id -gn $OSUSER`
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#check the password authentication
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY bye"
retval=$?
set -x

if [ $retval -gt 0 ]; then
  set +x
  if [ -z "$SRC_USER_PASSWD" ]; then
    echo "ERRORMSG: DB Login Failed. Please enter the valid userkey "
  else
    echo "ERRORMSG: DB Login Failed. Please enter the correct username and password"
  fi
  set -x
  exit 1
fi

#check auto logbackup on or off
set +x
checkAutologBkp "$OSUSER" "$DBSID" "$MAXDB_KEY"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Autologbackup is off"
  exit 1
fi


# Prepare variables
tmp_loc=/act/touch/$ACT_JOBNAME
mkdir -p $tmp_loc
chown -R $OSUSER:$OSGROUP $tmp_loc
OUTPUT_FILE=$tmp_loc/OUTPUT.log

LOGBKP_FIN_LOC=$MAXDB_BACKUPPATH/$DBSID/LOG
mkdir -p $LOGBKP_FIN_LOC
chmod 755 $LOGBKP_FIN_LOC
chmod 755 $MAXDB_BACKUPPATH/$DBSID
chown -R $OSUSER:$OSGROUP $LOGBKP_FIN_LOC

#Get archivelog location
set +x
getAutologbkpLoc "$OSUSER" "$DBSID" "$MAXDB_KEY"
set -x
echo "ArchiveLog_Loc_base is: $ArchiveLog_Loc_base"

#Switch log to get latest logs
set +x
su -m $OSUSER -c "dbmcli -d $DBSID  $MAXDB_KEY -o $OUTPUT_FILE -uUTL -c backup_start $AUTOLOG_TMPLT LOG"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  isNoLogToSave=`grep "No more log to save" $OUTPUT_FILE | wc -l`
  if [ $isNoLogToSave -ne 1 ]; then
    echo "ERRORMSG: fail to trigger autolog backup template"
    exit 1
  fi
fi

#Copy logs
Does_Alias_Exist_for_cp_command=`alias | grep -w cp | wc -l`
if [ $Does_Alias_Exist_for_cp_command -ge 1 ]; then
  unalias cp
fi

logFilesToCopyBeforeBEGINPIT=`find $ArchiveLog_Loc_base.* ! -newermt "$BEGINPIT" | sort | tail -15 | xargs`
if ! [ -z "$logFilesToCopyBeforeBEGINPIT" ]; then 
 for logFile in $logFilesToCopyBeforeBEGINPIT; do
   srcLogFileName=`basename $logFile`
   trgLogFile=$MAXDB_BACKUPPATH/$DBSID/LOG/$srcLogFileName

   #Skip if file already copied
   if [ -e "$trgLogFile" ]; then
     srcTstamp=`stat -c %Y $logFile`
     trgTstamp=`stat -c %Y $trgLogFile`

     if [ $srcTstamp -le $trgTstamp ]; then
        continue
     fi
   fi

   cp -frp $logFile $trgLogFile
 done
fi

logFilesToCopy=`find $ArchiveLog_Loc_base.* -newermt "$BEGINPIT" | xargs`
for logFile in $logFilesToCopy; do
  srcLogFileName=`basename $logFile`
  trgLogFile=$MAXDB_BACKUPPATH/$DBSID/LOG/$srcLogFileName

  #Skip if file already copied
  if [ -e "$trgLogFile" ]; then
    srcTstamp=`stat -c %Y $logFile`
    trgTstamp=`stat -c %Y $trgLogFile`

   if [ $srcTstamp -le $trgTstamp ]; then
       continue
   fi
  fi

  cp -frp $logFile $trgLogFile
done

exit 0
