#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
TARGET_SERVER_NAME=$TARGET_SERVER_NAME
TARGET_DB_USER=$TARGET_DB_USER
set +x
TARGET_DBUSER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
TARGET_MNT_PNT=$TARGET_MNT_PNT
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC
LOCAL_PRG_PATH=$MAXDB_HOME
SRC_DB_VERSION=$SRC_DB_VERSION
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
ACT_JOBNAME=$JOBID
APPREMOUNT=$APPREMOUNT
ACT_MOUNT_REPLACERESTORE=$ACT_MOUNT_REPLACERESTORE
. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#Convert TARGET_SERVER_NAME  to uppercase
  TARGET_SERVER_NAME=`echo "$TARGET_SERVER_NAME" | tr "[:lower:]" "[:upper:]"`

#prepare MAXDB_KEY
  set +x
  prepareMaxdbKey "$TARGET_DB_USER" "$TARGET_DBUSER_PASSWD"
  set -x

  if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
     TARGET_MNT_PNT=
  fi

#---------------------------------------------------------------------------
# function: CreateDB() // Drop the database
#---------------------------------------------------------------------------
CreateDB(){

 TARGET_SERVER_NAME=$1
 TARGET_DB_USER=$2
 set +x
 TARGET_DBUSER_PASSWD="$3"
 set -x

  #Create DB Instance with required name
  set +x
  su -m $OSUSER -c "dbmcli db_create $TARGET_SERVER_NAME $TARGET_DB_USER,'$TARGET_DBUSER_PASSWD'"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: DB Instance Creation Failed "
    exit 1
  fi

}

#---------------------------------------------------------------------------
# function: dropDB() // Drop the database
#---------------------------------------------------------------------------
dropDB() {
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_offline"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG:  Fail to stop database the database before drop"
  fi
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_drop"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG:  Fail to drop the database"
    exit 1
  fi
}
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Update the Parameter file with the new Log and DATA Volume
#---------------------------------------------------------------------------
Update_data_log_loc() {
  vol_name=$1
  file_name=$2
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY param_put $vol_name  $file_name"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Parameter file update failed for Data and Log Location"
    dropDB
    exit 1
  fi
}
#---------------------------------------------------------------------------

  OSGRPUSR=`id -gn $OSUSER`

#set the Path
if [ -z $LOCAL_PRG_PATH ]; then
   Local_soft_loc=`ps -eaf | grep "/pgm/kernel" | grep -vw grep | awk -F"/pgm/kernel" '{print $1}' | grep -vw awk | awk '{print $NF}' | uniq `

   # Fail if more than one MaxDB home found
   NumOfHomes=`echo $Local_soft_loc | wc -w`
   if [ $NumOfHomes -gt 1 ]; then
      echo "ERRORMSG: More than one MaxDB home found. Please specify MaxDB home and try again."
      exit 1
   fi

  if [ -z "$Local_soft_loc" ]; then
     echo "ERRORMSG: Fail to find MaxDB_Home as no instance is running on this host."
     exit 1
  else
     LOCAL_PRG_PATH=$Local_soft_loc
  fi
fi

  export PATH=$LOCAL_PRG_PATH/bin:$PATH

  tmp_loc=/act/touch/$ACT_JOBNAME
  mkdir -p $tmp_loc
  chown -R $OSUSER:$OSGRPUSR $tmp_loc

# Change the ownership of the Target Mount Point
  if [ ! "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
    chown -R $OSUSER:$OSGRPUSR $TARGET_MNT_PNT
    TARGET_MNT_BASE=`echo $TARGET_MNT_PNT | cut -d/ -f2`
     if [ "$TARGET_MNT_BASE" != "" ]; then
       find /$TARGET_MNT_BASE -type d -exec chmod 755 {} +
       chown -R $OSUSER:$OSGRPUSR /$TARGET_MNT_BASE
     fi
  fi

INDEP_DATA_PATH=`dbmcli dbm_getpath IndepDataPath | grep -v -w "OK"`

TGT_DB_VERSION=`dbmcli dbm_version VERSION | grep -v -w "OK"`
if ! [ "$TGT_DB_VERSION" = "$SRC_DB_VERSION" ]; then
  echo "ERRORMSG: Target DB has different version from source"
  exit 1
fi

#Check if the DB already exist
isDB_exists=`dbmcli db_enum | grep -w $TARGET_SERVER_NAME | wc -l`

if [ "$ACT_MOUNT_REPLACERESTORE" = "FALSE" ] && [ $APPREMOUNT = "FALSE" ]; then
  if [ $isDB_exists -gt 0 ]; then
    echo "ERRORMSG: Database $TARGET_SERVER_NAME already exists"
    exit 1
  fi
fi

#Create DB Instance with required name, if ReMount/ChildRestore
if [ ! $isDB_exists -gt 0 ]; then
  set +x
  CreateDB $TARGET_SERVER_NAME $TARGET_DB_USER "$TARGET_DBUSER_PASSWD"
  set -x
fi

#Check if Config Directory already exist
  if [ ! -d $INDEP_DATA_PATH/config ]; then
    mkdir -p $INDEP_DATA_PATH/config
  fi

#Copy the Config file
  cp $MANIFEST_FILE_LOC $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Unable to find the Manifest File"
    dropDB
    exit 1
  fi

  chown -R $OSUSER:$OSGRPUSR $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME

#Chekc if RunDirectry already Exist
  OLD_RUNDIRECTORY=`strings $MANIFEST_FILE_LOC | grep -wA1 RUNDIRECTORY | tail -1 | cut -c2-`
  TGT_RUNDIRECTORY=$INDEP_DATA_PATH/wrk/$TARGET_SERVER_NAME
  if [ ! -d $TGT_RUNDIRECTORY ]; then
    mkdir -p $TGT_RUNDIRECTORY
  fi
  chown -R $OSUSER:$OSGRPUSR $TGT_RUNDIRECTORY


# Start the session to update the paramerer
set +x
su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY" <<EOF
param_startsession
param_put  RUNDIRECTORY $TGT_RUNDIRECTORY
param_checkall
param_commitsession
EOF
retval=$?
set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Rundirectory update Failed in Config file"
    dropDB
    exit 1
  fi

# Update DATA and LOG volume path in parameter file
  VOLINFO=$tmp_loc/VOLINFO
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY -o $VOLINFO param_getvolsall"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get volumes info from configfile"
    dropDB
    exit 1
  fi

  grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" $VOLINFO >$VOLINFO.tmp
  while read line
  do
    vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`

    src_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
    if [ "${src_file_name:0:1}" != "/" ]; then
      src_file_name=$OLD_RUNDIRECTORY/$src_file_name
    fi

    trg_file_name=$TARGET_MNT_PNT$src_file_name
    if [ ! -e "$trg_file_name" ]; then
      echo "ERRORMSG: Fail to find file path ($trg_file_name)"
      dropDB
      exit 1
    fi

    Update_data_log_loc $vol_name $trg_file_name
  done <$VOLINFO.tmp

  if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
    MNT=$tmp_loc/MNT.tmp
    cat $VOLINFO.tmp | grep "VolumeName" | awk '{print $4}' | cut -d'/' -f2 | uniq>$MNT
     while read line
     do
       find /$line -type d -exec chmod 755 {} +
       chown -R $OSUSER:$OSGRPUSR /$line
     done < $MNT
  fi



#Reset Password
TRG_GLB_DATA_PATH=`xinstinfo $TARGET_SERVER_NAME | grep ^GlobalDataPath|awk '{print $3}'`
TRG_UPC_FILE=$TRG_GLB_DATA_PATH/config/$TARGET_SERVER_NAME.upc
MAX_SERVICE_NAME=`dbmcli db_enum -s | grep "^\."| grep -w "$LOCAL_PRG_PATH" | awk '{print $1}'| uniq|cut -d. -f2`
if [ -z "$MAX_SERVICE_NAME" ]; then
   echo "ERRORMSG: Fail to find service database name for MaxDB_Home $LOCAL_PRG_PATH"
   dropDB
   exit 1
fi
MAX_SERVICE_UPC=$TRG_GLB_DATA_PATH/config/.$MAX_SERVICE_NAME.upc
if [ ! -f $MAX_SERVICE_UPC ]; then
   echo "ERRORMSG: MaxDB service database upc file not available ($MAX_SERVICE_UPC) for MaxDB_Home $LOCAL_PRG_PATH"
   dropDB
   exit 1
fi

cp -f $TRG_UPC_FILE $TRG_UPC_FILE.orig
cp -f $MAX_SERVICE_UPC $TRG_UPC_FILE
chown $OSUSER:$OSGRPUSR $TRG_UPC_FILE
chmod 600 $TRG_UPC_FILE

set +x
su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME -u SERVICE,SERVICE" <<EOF
user_rename SERVICE $TARGET_DB_USER
user_put $TARGET_DB_USER password=$TARGET_DBUSER_PASSWD
EOF
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to reset password"
  dropDB
  exit 1
fi

#Recovery database if needed
  if [ ! -z "$UNTIL_TIME" ]; then
    set +x
    su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_admin"
    retval=$?
    set -x

    if [ $retval -ne 0 ]; then
      echo "ERRORMSG: Fail to set database in admin mode before Recovery"
      dropDB
      exit 1
    fi

  /act/custom_apps/maxdb/clone/ACT_MAXDB_RESTORE_recovery.sh
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to recovery database "
      dropDB
      exit 1
    else
      set +x
      getDB_State "$OSUSER" "$TARGET_SERVER_NAME" "$MAXDB_KEY"
      set -x
      if [ "$DB_STATE" = "ONLINE" ];then
        echo "DB is online. Clone successful"
        exit 0
      fi
    fi
  fi

# Bring the DB online
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_online"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
     echo "ERRORMSG: Database Instance Failed to start"
     dropDB
     exit 1
  fi
echo ".............AppAware Mount completed successfully ............."

#Remove temp files
if [ $retval -eq 0 ]; then
  rm -rf $tmp_loc
fi

exit 0


