#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
TARGET_SERVER_NAME=$TARGET_SERVER_NAME
TARGET_DB_USER=$TARGET_DB_USER
set +x
TARGET_DBUSER_PASSWD=$TARGET_DBUSER_PASSWD
set -x
TARGET_MNT_PNT=$TARGET_MNT_PNT
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC
LOCAL_PRG_PATH=$MAXDB_HOME
SRC_DB_VERSION=$SRC_DB_VERSION
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
ACT_JOBNAME=$JOBID
APPREMOUNT=$APPREMOUNT
SOURCE_DBSID=$SOURCE_DBSID
ACT_MOUNT_REPLACERESTORE=$ACT_MOUNT_REPLACERESTORE
OVERWRITE_DATABASE=$OVERWRITE_DATABASE

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

# Delete DB function
deleteDB() {
  local DbName=$1
  echo "INFOMSG: Dropping the database $DbName"

  # Kill database if running
  local dbPIDs=`ps -ef | grep kernel | grep " \-diagpipe " | grep -w $DbName | awk '{print $2}' | xargs`
  if [ ! -z "$dbPIDs" ]; then
    echo "INFOMSG: Killing MaxDB processes $dbPIDs"
    kill -9 $dbPIDs
  fi

  # Clean database entries
  dbRelatedDirs=`dbmcli -d $DbName dbm_getpath | grep -vw OK | cut -d= -f2 | sort | uniq | xargs`
  for dirName in $dbRelatedDirs
  do
    find $dirName -iname $DbName -print -exec rm -rf {} \;
    find $dirName -iname $DbName.* -print -exec rm -rf {} \;
    DB_INI_FILE=`find $dirName -name Databases.ini -print`
    sed -i "/$DbName=/d" /var/opt/sdb/globaldata/config/Databases.ini
  done

  return 0
}

# Find DB stale
deleteDBifStale() {
  local DbName=$1
  local STALE='FALSE'
  local RUNDIR=
  local pahFile=
  local configFile=

  # Check db stale or not. If active fail.
  dbRelatedDirs=`dbmcli -d $DbName dbm_getpath | grep -vw OK | cut -d= -f2 | sort | uniq | xargs`
  for dirName in $dbRelatedDirs
  do
    if [ -z "$pahFile" ]; then
      pahFile=`find $dirName -iname $DbName.pah -print`
    fi
    if [ -z "$configFile" ]; then
      configFile=`find $dirName -iname $DbName -type f -print | grep "/config/"`
    fi
    if [ ! -z "$configFile" ] && [ ! -z "$pahFile" ]; then
      break
    fi
  done

  if [ ! -z "$configFile" ]; then
    RUNDIR=`strings $configFile | grep -wA1 RUNDIRECTORY | tail -1 | cut -c2-`
  fi
  if [ ! -z "$pahFile" ]; then
    dbFilesList=`grep VolumeName $pahFile | awk '{print $4}' | xargs`
  fi

  for dbFile in $dbFilesList
  do
    if [ "${dbFile:0:1}" != "/"  ]; then
      dbFile=$RUNDIR/$dbFile
    fi
    if [ ! -e $dbFile ]; then
      dbFileLength=${#dbFile}
      if [ $dbFileLength -eq 64 ]; then
        dbFileBase=`dirname $dbFile`
        if [ ! -e $dbFileBase ]; then
          STALE=TRUE
          break
        fi
      else
        STALE=TRUE
        break
      fi
    fi
  done

  if [ "$STALE" = "FALSE" ]; then
    echo "ERRORMSG: Database $DbName exists and not stale"
    echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

  deleteDB $DbName
  return 0
}

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#set the Path
if [ -z $LOCAL_PRG_PATH ]; then

   Local_soft_loc=`ps -eaf | grep "/pgm/kernel" | grep -vw grep | awk -F"/pgm/kernel" '{print $1}' | grep -v awk | awk '{print $NF}' | uniq | head -1`
  if [ -z "$Local_soft_loc" ]; then
     echo "ERRORMSG: Fail to find MaxDB_Home as no instance is running on this host."
     exit 1
  else
   LOCAL_PRG_PATH=$Local_soft_loc
  fi
fi

export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Check the No of characters in DB Name
getDBName_charCount $TARGET_SERVER_NAME

#Convert the TARGET_SERVER_NAME in Upper case
TARGET_SERVER_NAME=`echo $TARGET_SERVER_NAME | tr "[:lower:]" "[:upper:]"`

#check all values passed or not
isVarNull OSUSER $OSUSER
isVarNull TARGET_SERVER_NAME $TARGET_SERVER_NAME
isVarNull TARGET_DB_USER $TARGET_DB_USER
#isVarNull TARGET_DBUSER_PASSWD $TARGET_DBUSER_PASSWD
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH

if ! [ -z "$UNTIL_TIME" ]; then
  isVarNull BEGIN_TIME $BEGIN_TIME
  isVarNull LOG_BKP_MNTPT $LOG_BKP_MNTPT
  isVarNull SOURCE_DBSID $SOURCE_DBSID
fi

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$TARGET_DB_USER" "$TARGET_DBUSER_PASSWD"
set -x

#Does OSUSER Exist

id $OSUSER
retval=$?

if [ $retval -ne 0 ]; then
 echo "ERRORMSG: $OSUSER is not a valid OSUSER"
 exit 1
fi

if [ "$OSUSER" = "root" ]; then
 echo "WARNINGMSG: We should not use root as OSUSER"
 exit 1
fi

OSGRPUSR=`id -gn $OSUSER`
export PATH=$LOCAL_PRG_PATH/bin:$PATH

  isDB_exists=`dbmcli db_enum | grep -w $TARGET_SERVER_NAME | wc -l`

if [ "$ACT_JOBTYPE" = "mount" ]; then
  isVarNull MANIFEST_FILE_LOC $MANIFEST_FILE_LOC
  isVarNull SRC_DB_VERSION $SRC_DB_VERSION

  if [ "$ACT_MOUNT_REPLACERESTORE" = "FALSE" ] && [ $APPREMOUNT = "FALSE" ]; then

  #Check if the DB Instance is already exist

  if [ $isDB_exists -gt 0 ]; then
    case $OVERWRITE_DATABASE in
      yes)
        deleteDB $TARGET_SERVER_NAME
        ;;
      stale)
        deleteDBifStale $TARGET_SERVER_NAME
        ;;
      *)
        echo "ERRORMSG: Database $TARGET_SERVER_NAME already exists"
        echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
        exit 1
        ;;
    esac
  fi
 fi
fi

if [ "$ACT_JOBTYPE" = "unmount" ] && [ ! -e "$ACT_PREMOUNT_FAILURE_FILE" ]; then
 if [ "$isDB_exists" -gt 0 ]; then
  #check the password authentication
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY bye"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: User Authentication Failed for $DBSID Database. Please check username and password"
    exit 1
  fi
 fi
fi
