#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
DUMPBKPLOC=$TARGET_MNT_PNT
ACT_JOBNAME=$JOBID

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#Set Path Variable
if [ -z "$LOCAL_PRG_PATH" ]; then
  Local_soft_loc=`ps -eaf | grep "/pgm/kernel" | grep -vw grep | awk -F"/pgm/kernel" '{print $1}' | grep -v awk | awk '{print $NF}' | uniq | head -1`

  if [ -z "$Local_soft_loc" ]; then
    echo "ERRORMSG: Fail to find MaxDB_Home as no instance is running on this host."
    exit 1
  else
    LOCAL_PRG_PATH=$Local_soft_loc
  fi
fi
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Convert DBSID to upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#Create temp dir
tmp_loc=/act/touch/$ACT_JOBNAME
if [ ! -d $tmp_loc ]; then
  mkdir -p $tmp_loc
  chmod 755 $tmp_loc
fi

#Change permissions
OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $LOG_BKP_MNTPT
chown -R $OSUSER:$OSGROUP $tmp_loc

#prepare required flages, files
REC_OUTPUT=$tmp_loc/RECOVERY_OUTPUT.log
REC_INPUT=$tmp_loc/RECOVERY_INPUT.sql
rec_check_out=$tmp_loc/rec_check_out.txt
rec_check_input=$tmp_loc/rec_check_input.txt
logbkpFiles_list_AF=$tmp_loc/logbkpFiles_list_AF.txt
logbkpFiles_list=$tmp_loc/logbkpFiles_list.txt
logbkpFiles_list1=$tmp_loc/logbkpFiles_list1.txt
UN_TM=`echo $UNTIL_TIME | sed 's/-//g' | sed 's/://g'`
flagR=0

#---------------------------------------------
#Function: clear_log
#--------------------------------------------

#Clear log area function
clear_log(){
  set +x
  echo "Clearing the log area"
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_execute clear log"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: fail to clear log area"
    exit 1
  fi
}

#Clear log area
clear_log

##############################
#get log bkp files 
#############################

#Get the last logsequence no Restored from the backup. 
set +x
LastLogSeq=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_restartinfo" | grep -w "Used LOG Page" | awk '{print $NF}'`
set -x 
echo "LastLogSeq: $LastLogSeq"

#Get the list of the files after dump backup 
if [ -e "$logbkpFiles_list_AF" ]; then
  rm -f $logbkpFiles_list_AF
fi

# Get full backup filename
Full_bkp_file=`find $DUMPBKPLOC/$DBSID/COMPLETE -name 'DATA_Bkp_Full*' |sort| tail -1 | awk '{print $NF}'`

# Get list of incremental backupfiles
Last_Inc_bkp_file=`find $DUMPBKPLOC/$DBSID/COMPLETE/ -name 'DATA_Bkp_Incr*' -newer $Full_bkp_file |awk '{print $NF}'| sort | tail -1`

if ! [ -z $Last_Inc_bkp_file ]; then
  find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newer $Last_Inc_bkp_file | sort >>$logbkpFiles_list_AF
else
  find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newer $Full_bkp_file | sort >>$logbkpFiles_list_AF
fi


#Count of the logfiles 
No_of_File=`cat $logbkpFiles_list_AF | wc -l`

#Find the First log file based on BEGIN_TIME 
if [ $No_of_File -ne 0 ]; then
  FFile=`cat $logbkpFiles_list_AF | head -1`
fi

#echo $FFile

if [ -z "$FFile" ]; then
 echo "No log file to recover"
 exit 0
fi


# Check if the Log file is truly recoverable based on begin time
#----------------------------------------------------------------
#Function: recov_check
#--------------------------------------------------------------
recov_check() {
  logfile=$1
  
  set +x 
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c backup_template_create recov_test to file $logfile content log"
  set -x 
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to create restore template "
    exit 1
  fi

  if [ -f $rec_check_input ]; then
    rm -f $rec_check_input
  fi 

  echo "service_connect" >> $rec_check_input
  echo "recover_check recov_test" >> $rec_check_input
  chown -R $OSUSER:$OSGROUP $tmp_loc

  #Recovery_check 
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -i $rec_check_input  -o $rec_check_out"
  retval=$?
  set -x
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to check recovery status of log file $logfile"
    exit 1
  fi

  F_SEQ=`cat $rec_check_out | grep -w "First LOG Page" | awk '{print $NF}'`
  L_SEQ=`cat $rec_check_out | grep -w "Last LOG Page" | awk '{print $NF}'`
  #echo $F_SEQ
  return 0
}

#Recovery check for Log File previous than FFile which we get just after the Begin Time.
#----------------------------------------------------------------------------------------
#Function: recover_check_priv
#----------------------------------------------------------------------------------------
recover_check_priv(){
  N_File=$FFile

  while [ $LastLogSeq -lt $F_SEQ ]
  do
     FFile=$N_File
     currnt_seq=`echo $FFile | rev | cut -d'.' -f1 | rev`
     cdigitcount=`echo -n $currnt_seq |wc -c`
     echo $cdigitcount
     curr_file=`echo $FFile | awk -F"$currnt_seq" '{print $1}'`
     next_seq=`expr $currnt_seq - 1`
     ndigitcount=`echo -n $next_seq | wc -c`
     echo $ndigitcount

     if [ $(($cdigitcount - $ndigitcount)) -eq 0 ]; then
        next_seq="$next_seq"
     elif [ $(($cdigitcount - $ndigitcount)) -eq 1 ]; then 
        next_seq="0$next_seq"
     elif [ $(($cdigitcount - $ndigitcount)) -eq 2 ]; then
        next_seq="00$next_seq"
     else
        echo "Please check the sequnce number"
     fi 

     N_File=$curr_file$next_seq
     recov_check $N_File
  done

}

# Recovery Check for the file newer than Begin Time
#---------------------------------------------------
#Function: recover_check_next 
#---------------------------------------------------
recover_check_next(){
  N_File=$FFile

  while [ $LastLogSeq -gt $L_SEQ ]
  do
     FFile=$N_File
     currnt_seq=`echo $FFile | rev | cut -d'.' -f1 | rev`
     cdigitcount=`echo -n $currnt_seq |wc -c`
     echo $cdigitcount
     curr_file=`echo $FFile | awk -F"$currnt_seq" '{print $1}'`
     next_seq=`expr $currnt_seq + 1`
     ndigitcount=`echo -n $next_seq | wc -c`
     echo $ndigitcount

     if [ $(($cdigitcount - $ndigitcount)) -eq 0 ]; then
        next_seq="$next_seq"
     elif [ $(($cdigitcount - $ndigitcount)) -eq 1 ]; then
        next_seq="0$next_seq"
     elif [ $(($cdigitcount - $ndigitcount)) -eq 2 ]; then
        next_seq="00$next_seq"
     else
        echo "Please check the sequnce number"
     fi

     N_File=$curr_file$next_seq
     recov_check $N_File
  done

}

#Recovery Check start for the list of log backup files
recov_check $FFile

if [ $LastLogSeq -lt $F_SEQ ]; then
  recover_check_priv
  RFile=$N_File
elif [ $LastLogSeq -ge $F_SEQ ] && [ $LastLogSeq -le $L_SEQ ]; then
  RFile=$FFile
else
  recover_check_next
  RFile=$N_File
fi

#Get the logfile list to recover :
if [ -f $logbkpFiles_list1 ]; then
  rm -f $logbkpFiles_list1
fi

if [ -f $logbkpFiles_list ]; then
  rm -f $logbkpFiles_list
fi

echo "$RFile" >> $logbkpFiles_list1
find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newer $RFile ! -newermt "$UNTIL_TIME" | sort >> $logbkpFiles_list1
find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newermt "$UNTIL_TIME" | sort | head -1 >> $logbkpFiles_list1

cat $logbkpFiles_list1 | uniq >> $logbkpFiles_list
cat $logbkpFiles_list
chown -R sdb:sdba $logbkpFiles_list
chmod 755 $logbkpFiles_list 
IsLogEmpty=`cat $logbkpFiles_list | sed '/^$/d' | wc -l`

if [ $IsLogEmpty -eq 0 ]; then
  echo "No log file to recover "
  exit 0
fi

#Recover the database in one session
#---------------------------------------------
#Function: recover_db
#--------------------------------------------
recover_db() {
  if [ -f "$REC_INPUT" ]; then
    rm -f $REC_INPUT
  fi

  if [ -f "$REC_OUTPUT" ]; then
    rm -f $REC_OUTPUT 
  fi

  flagR=0
  echo "db_connect" >> $REC_INPUT
  while read line
  do
     echo "backup_template_create act_maxdb_Log_rst_tmplt to FILE $line CONTENT LOG" >>$REC_INPUT
     if [ $flagR -eq 0 ]; then
       echo "recover_start act_maxdb_Log_rst_tmplt LOG until $UN_TM" >> $REC_INPUT
       flagR=1
     else
       echo "recover_replace act_maxdb_Log_rst_tmplt " >> $REC_INPUT
     fi
  done < $logbkpFiles_list
    
  echo "recover_ignore" >> $REC_INPUT
 
  chown -R $OSUSER:$OSGROUP $tmp_loc
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -i $REC_INPUT -o $REC_OUTPUT"
  set -x
  isErr=`grep ERR $REC_OUTPUT | wc -l`
  LastReruntcode=`cat $REC_OUTPUT | grep "Returncode" | tail -1 | awk '{ print $NF }'`
  cat $REC_OUTPUT
}

#Recover database  
recover_db

#Online database
if [ $isErr -gt 0 ]; then
  if [ ! -z "$LastReruntcode" ] && [ "$LastReruntcode" -eq 0 ]; then
    if ! [ "$dbState" = "ONLINE" ]; then
      set +x
      su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_online"
      retval=$?
      set -x
      if [ $retval -ne 0 ]; then
        echo "ERRORMSG: Fail to recover the database"
        exit 1
      fi
    fi 
    echo "Recovery is successful we can Ignore this error"
  else
    echo "ERRORMSG: Fail to recover the database"
    exit 1 
  fi
fi

exit 0
