#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD=$TARGET_DBUSER_PASSWD
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
DUMPBKPLOC=$TARGET_MNT_PNT
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
ACT_JOBNAME=$JOBID

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#set Environment variable
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Convert DBSID to upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

#-----------------------------------------------------------------
# function: restoreMAXDB()
#-----------------------------------------------------------------
restoreMAXDB() {
  local TMPL_NAME=$1
  local TMPL_TYPE=$2

  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c recover_start $TMPL_NAME $TMPL_TYPE"
  retval=$?
  set -x

  return $retval
}
#-----------------------------------------------------------------

if [ "$DUMPBKPLOC" = "" ]; then
  echo "ERRORMSG: DUMPBKPLOC is not provided"
  exit 1
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $DUMPBKPLOC

# Get full backup filename
Full_bkp_file=`find $DUMPBKPLOC/$DBSID/COMPLETE -name 'DATA_Bkp_Full*' |sort| tail -1 | awk '{print $NF}'`
if [ -z "$Full_bkp_file" ]; then
  echo "ERRORMSG: No full backup found"
  exit 1
fi

# Get list of incremental backupfiles
Inc_bkp_files=`find $DUMPBKPLOC/$DBSID/COMPLETE/ -name 'DATA_Bkp_Incr*' -newer $Full_bkp_file |awk '{print $NF}'| sort`

#Prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#Create the Full backup Restore Template 
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY backup_template_create act_maxdb_F_rst_tmplt to FILE $Full_bkp_file CONTENT DATA"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to create template for full backup restore"
  exit 1
fi

#Restore full backup
restoreMAXDB act_maxdb_F_rst_tmplt DATA
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to restore full backup from file $Full_bkp_file"
  exit 1
fi

#Restore incremental backups
for incFile in $Inc_bkp_files
do
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY backup_template_create act_maxdb_I_rst_tmplt to FILE $incFile CONTENT INCREMENTAL"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to create template for incremental backup restore"
    exit 1
  fi

  #Restore
  restoreMAXDB act_maxdb_I_rst_tmplt PAGES
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to restore incremental backup from $incFile"
    exit 1
  fi
done

#Recovery database if needed
if [ ! -z "$UNTIL_TIME" ]; then
  /act/custom_apps/maxdb/dump/ACT_MAXDB_RESTORE_recovery.sh
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Log recovery failed"
    exit 1
  fi
fi

#Bring database online
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_online"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to bring database online"
  exit 1
fi

exit 0

