#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
TARGET_SERVER_NAME=$TARGET_SERVER_NAME
TARGET_DB_USER=$TARGET_DB_USER
set +x 
TARGET_DBUSER_PASSWD="$TARGET_DBUSER_PASSWD"
DBADMIN_PWD=$DBADMIN_PWD
set -x 
LOG_MNT_PNT=$LOG_MNT_PNT
DATA_MNT_PNT=$DATA_MNT_PNT
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
SRC_DB_VERSION=$SRC_DB_VERSION
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
ACT_JOBNAME=$JOBID

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#---------------------------------------------------------------------------
# function: dropDB() // Drop the database 
#---------------------------------------------------------------------------
dropDB() {
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_offline"
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_drop"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG:  Fail to drop the database"
    exit 1
  fi
}
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Update the Parameter file with the new Log and DATA Volume
#---------------------------------------------------------------------------
Update_data_log_loc() {
 if [ "$Config_File_already_exists" -eq 1 ]; then
   vol_name=$1
   file_name=$2
   set +x
   su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY param_put $vol_name  $file_name"
   set -x
 else
   vol_no=$1
   vol_type=$2
   file_name=$3
   file_type=$4
   file_size=$5
   set +x
   su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY param_addvolume $vol_no $vol_type $file_name $file_type $file_size"
   set -x
 fi
   retval=$?

  if [ $retval -ne 0 ]; then
    echo " Parameter file update failed for Data and Log Location"
    exit 1
  fi
}
#---------------------------------------------------------------------------

#Convert TARGET_SERVER_NAME  to uppercase 
  TARGET_SERVER_NAME=`echo "$TARGET_SERVER_NAME" | tr "[:lower:]" "[:upper:]"`

  OSGRPUSR=`id -gn $OSUSER`
  INDEP_DATA_PATH=`dbmcli dbm_getpath IndepDataPath | grep -v -w "OK"`
  export PATH=$LOCAL_PRG_PATH/bin:$PATH

  tmp_loc=/act/touch/$ACT_JOBNAME
  mkdir -p $tmp_loc
  find /act -type d -exec chmod 755 {} +
  chown -R $OSUSER:$OSGRPUSR $tmp_loc

  TGT_DB_VERSION=`dbmcli dbm_version VERSION | grep -v -w "OK"`
  if ! [ "$TGT_DB_VERSION" = "$SRC_DB_VERSION" ]; then
   echo "ERRORMSG: Target DB has different version from source"
   exit 1
  fi

#prepare MAXDB_KEY
  set +x
  prepareMaxdbKey "$TARGET_DB_USER" "$TARGET_DBUSER_PASSWD"
  set -x

isDB_exists=`dbmcli db_enum | grep -w $TARGET_SERVER_NAME | wc -l`

if [ $isDB_exists -gt 0 ]; then
  echo "Database $TARGET_SERVER_NAME  already exist. Please proceed with Restore and recovery."
else 
  echo "Creating and configuring the database before restore and recovery."

#Create DB Instance with required name
  set +x
  su -m $OSUSER -c "dbmcli db_create $TARGET_SERVER_NAME $TARGET_DB_USER,$TARGET_DBUSER_PASSWD"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: DB Instance Creation Failed "
    exit 1
  fi
fi
#Check if Config Directory already exist
  Config_File_already_exists=0
if [ ! -d $INDEP_DATA_PATH/config ]; then
   mkdir -p $INDEP_DATA_PATH/config
#Copy the Config file
   cp $MANIFEST_FILE_LOC $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME
   retval=$?
     if [ $retval -ne 0 ]; then
       echo "ERRORMSG: Unable to find the Manifest File"
       dropDB
        exit 1
     fi
else
  if [ ! -e $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME ]; then
    cp $MANIFEST_FILE_LOC $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME
    retval=$?
    if [ $retval -ne 0 ]; then
       echo "ERRORMSG: Unable to find the Manifest File"
       dropDB
       exit 1
    fi
  else
    Config_File_already_exists=1
  fi
fi

OLD_RUNDIRECTORY=`strings $MANIFEST_FILE_LOC | grep -wA1 RUNDIRECTORY | tail -1 | cut -c2-`

  chown -R $OSUSER:$OSGRPUSR $INDEP_DATA_PATH/config/$TARGET_SERVER_NAME

#Check if RunDirectry already Exist
  TGT_RUNDIRECTORY=$INDEP_DATA_PATH/wrk/$TARGET_SERVER_NAME

  if [ ! -d $TGT_RUNDIRECTORY ]; then
    mkdir -p $TGT_RUNDIRECTORY
  fi

  chown -R $OSUSER:$OSGRPUSR $TGT_RUNDIRECTORY

# Start the session to update the paramerer
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY" <<EOF
  param_startsession
  param_put  RUNDIRECTORY $TGT_RUNDIRECTORY
  param_checkall
  param_commitsession
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
   echo "ERRORMSG: Rundirectory update Failed in Config file"
   dropDB
   exit 1
  fi
# Update DATA and LOG volume path in parameter file
  VOLINFO=$tmp_loc/VOLINFO
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY -o $VOLINFO param_getvolsall"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
   echo "ERRORMSG: Fail to get volumes info from configfile"
   dropDB
   exit 1
  fi

  if [ "$Config_File_already_exists" -eq 1 ]; then

   cat $VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" >$VOLINFO.tmp
   while read line
   do
     vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
     src_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
     if [ "${src_file_name:0:1}" != "/" ]; then
       src_file_name=$OLD_RUNDIRECTORY/$src_file_name
     fi

     ########trg_file_name=$TARGET_MNT_PNT$src_file_name
     trg_file_name=$src_file_name
     trg_dir=`dirname $trg_file_name`
     Update_data_log_loc $vol_name $trg_file_name
     if [ ! -d $trg_dir ]; then
       mkdir -p $trg_dir
     fi
   done <$VOLINFO.tmp
    chown -R $OSUSER:$OSGRPUSR $trg_dir
  else
   if [ ! -z $LOG_MNT_PNT ] && [ ! -z $DATA_MNT_PNT ]; then
  cat $VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" | grep "DataVolume" >$VOLINFO.tmp
     while read line
     do
       vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
       src_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
       if [ "${src_file_name:0:1}" != "/" ]; then
         src_file_name=$OLD_RUNDIRECTORY/$src_file_name
       fi
       trg_file_name=$DATA_MNT_PNT$src_file_name
       vol_no="${vol_name: -1}"
       vol_type=DATA
       src_file_size=`echo $line | awk '{print $2}'`
       src_file_type=`echo $line | awk '{print $3}'`
       trg_data_dir=`dirname $trg_file_name`
       if [ ! -d $trg_data_dir ]; then
         mkdir -p $trg_data_dir
       fi       
       Update_data_log_loc $vol_no $vol_type $trg_file_name $src_file_type $src_file_size
     done <$VOLINFO.tmp
       cat $VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" | grep "LogVolume" >$VOLINFO.tmp
       while read line
     do
       vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
       src_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
       if [ "${src_file_name:0:1}" != "/" ]; then
         src_file_name=$OLD_RUNDIRECTORY/$src_file_name
       fi
       trg_file_name=$LOG_MNT_PNT$src_file_name
       vol_no="${vol_name: -1}"
       vol_type=LOG
       src_file_size=`echo $line | awk '{print $2}'`
       src_file_type=`echo $line | awk '{print $3}'`
       trg_log_dir=`dirname $trg_file_name`
       if [ ! -d $trg_log_dir ]; then
         mkdir -p $trg_log_dir
       fi
       Update_data_log_loc $vol_no $vol_type $trg_file_name $src_file_type $src_file_size 
     done <$VOLINFO.tmp

      chown -R $OSUSER:$OSGRPUSR $LOG_MNT_PNT
      chown -R $OSUSER:$OSGRPUSR $DATA_MNT_PNT
    else
       echo "ERRORMSG: Please provide the Log and Data Mount Point for the target server"
       dropDB
       exit 1
    fi
  fi

#Bring the database in Admin Mode :
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_admin"
  set -x
  retval=$?
  if [ $retval -ne 0 ]; then
    echo " Failed the Bring the database $TARGET_SERVER_NAME in admin Mode"
    dropDB
    exit 1
  fi

if [ $isDB_exists -eq 0 ]; then

#Activate database 
  set +x
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_activate DBADMIN,$DBADMIN_PWD"
  set -x
  retval=$?
  if [ $retval -ne 0 ]; then 
     echo " Failed to activate the database $TARGET_SERVER_NAME "
     dropDB
     exit 1
  fi   
fi

#Bring the database  Again in Admin Mode :
  set +x 
  su -m $OSUSER -c "dbmcli -d $TARGET_SERVER_NAME $MAXDB_KEY db_admin"
  set -x
  retval=$?
  if [ $retval -ne 0 ]; then
    echo " Failed the Bring the database $TARGET_SERVER_NAME in admin Mode"
    dropDB
    exit 1
  fi

#Restore the database

  /act/custom_apps/maxdb/dump/Act_Maxdb_restoreScript_newTarget.sh
  retval=$?
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Restore and Recovery Failed. Please take look in logs for the details."
    dropDB
    exit 1
  fi

  exit 0
