#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD=$SRC_USER_PASSWD
set -x
ACT_JOBNAME=$JOBID
MAXDB_BACKUPPATH=$MAXDB_BACKUPPATH
SRC_DB_VERSION=$version
DUMPSCHEDULE=$DUMPSCHEDULE
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
INTERVAL=$INTERVAL

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#Set Environment Variable 
OSGROUP=`id -gn $OSUSER`
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Change DBSID in Upper Case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

#Prepare variables
DATE=`date +%Y%m%d_%H%M%S`

if [ "$DUMPSCHEDULE" = "F" ]; then
  DUMPSCHEDULE_DB="F"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
  DUMPSCHEDULE_DB="I"
else
  DUMPSCHEDULE_DB=${DUMPSCHEDULE:`date +%w`:1}
fi

# Create or Modify Existing Backup Template 
create_BKP_Templ() {
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY backup_template_create $BKP_TEMPL_NAME to FILE $BACKUP_FILE_NAME  CONTENT $TMPL_TYPE"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "Backup Tempate Creation Failed...."
    exit 1
  fi
}

# Backup Database
backup_DB() {
  echo " Backup starrted..."
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c backup_start $BKP_TEMPL_NAME $BKP_TYPE AUTOIGNORE"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "DB Backup Failed......" 
    exit 1
  fi
}

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#Prepare backup directory
Relative_bkp_dir="$MAXDB_BACKUPPATH/$DBSID/COMPLETE"
if [ ! -d $Relative_bkp_dir ]; then
  mkdir -p $Relative_bkp_dir
fi

if [ "$DUMPSCHEDULE_DB" = "F" ]; then
  Flag=Full
  BKP_TYPE=DATA
  TMPL_TYPE=DATA
  BKP_TEMPL_NAME=ACT_MAXDB_${DBSID}_FULL
else
  Flag=Incr
  BKP_TYPE=PAGES
  TMPL_TYPE=INCREMENTAL
  BKP_TEMPL_NAME=ACT_MAXDB_${DBSID}_INC
fi

#Create BackupFile based on Backup Location :
BACKUP_FILE_NAME=$Relative_bkp_dir/DATA_Bkp_${Flag}_$DATE.dat
echo "Backup File is :  $BACKUP_FILE_NAME "
chown -R $OSUSER:$OSGROUP $Relative_bkp_dir
Relative_bkp_base=`echo $Relative_bkp_dir | cut -d/ -f2`
if [ ! -z "$Relative_bkp_base" ]; then
  find /$Relative_bkp_base -type d -exec chmod 755 {} +
fi

#create template
create_BKP_Templ

#Take backup
backup_DB
echo "Backup completed successfully..."

#Commenting this code for now----
#Enable autolog backup if it is first time
#Get the Autolog Backup State and Autolog Backup Tempalte Name
set +x
DB_STATE=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_state" | grep -v "OK" | grep -v "State"`
set -x

if [ "$DB_STATE" = "ONLINE" ]; then
  set +x
  AUTOLOG_BKP_STATE=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY autolog_show" | grep -vw "OK"`
  AUTO_LOG_BKP_TEMPL_NAME=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY backup_history_list -I -c label,media,action"  | grep "LOG_" | grep -w "SAVE" | head -1 | cut -d'|' -f2`
  set -x
else
  echo "WARNINGMSG: As the Database is not in online mode, not able to find the Autolog backup state."
  exit 0
fi

if [ "$AUTOLOG_BKP_STATE" = "AUTOSAVE IS ON" ]; then
  echo "Autolog backup is already on"
  exit 0
fi

#set +x
#su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_execute SET LOG AUTO OVERWRITE OFF"
#retval=$?
#set -x 

#if [ $retval -ne 0 ]; then
#  echo "WARNINGMSG: Unable to change Autolg Overwrite to OFF"
#  exit 0
#fi

if [ -z $AUTO_LOG_BKP_TEMPL_NAME ]; then
  echo "LOG Backup Template not found In Backup History. Either Log backup Template is not created or deleted from the backup hostory. Please create log backup template and enable Autolog backup on"

  set +x
  isError=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c autolog_on" | grep "No medium found for automatic log backup" | wc -l`
  set -x

 if [ $isError -ne 0 ]; then
  echo "WARNINGMSG: Autolog Backup failed to bring on as Backup Template not found in Backup History."
  exit 0
 fi
 else

  if [ x"$INTERVAL" = "x" ]; then
     INTERVAL=3600 
  fi
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c autolog_on $AUTO_LOG_BKP_TEMPL_NAME INTERVAL $INTERVAL"
  restval=$?
  set -x

  if [ $restval -ne 0 ]; then
  echo "WARNINGMSG: Autolog Backup failed to bring on"
  exit 0
  fi

fi
exit 0
