#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD="$SRC_USER_PASSWD"
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
MAXDB_BACKUPPATH=$MAXDB_BACKUPPATH
DUMPSCHEDULE=$DUMPSCHEDULE
SRC_DB_VERSION=$version
ACT_JOBNAME=$JOBID
CHILD=$CHILD
. /act/custom_apps/maxdb/act_maxdb_common_func.sh

isVarNull OSUSER $OSUSER
isVarNull DBSID $DBSID
isVarNull SRC_BACKUP_USER $SRC_BACKUP_USER
#isVarNull SRC_USER_PASSWD $SRC_USER_PASSWD
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH
isVarNull DUMPSCHEDULE $DUMPSCHEDULE

#convert DBSID in upper case 
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

OSGROUP=`id -gn $OSUSER`
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#check the password authentication
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY bye"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  set +x
  if [ -z "$SRC_USER_PASSWD" ]; then
    echo "ERRORMSG: User Authentication Failed. Please check userkey"
  else
    echo "ERRORMSG: User Authentication Failed. Please check username and password"
  fi
  set -x
  exit 1
fi

#Check Backup Location.....
if [ ! -d $MAXDB_BACKUPPATH ]; then
  echo " Backup Location is Missing...."
  exit 1
fi


#----------------------------------------------------


#Check if this is child backup

if [ "$CHILD" = "TRUE" ]; then

 #check if this first dump backup of the child database

  AUTOLOG_BKP_TEMP_NAME=`echo ACT_AUTOLOGBKP_${DBSID}_TEMP`
  FULL_BKP_TEMP_NAME=`echo ACT_MAXDB_${DBSID}_FULL`
  set +x
  IsLogBkpTempExist=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c backup_template_show all"  | grep "$AUTOLOG_BKP_TEMP_NAME" | wc -l`
  set -x

  if [ $IsLogBkpTempExist -eq 0 ]; then


#Create log backup Template
  AUTOLOG_BKP_TEMP_NAME=ACT_AUTOLOGBKP_${DBSID}_TEMP
  set +x
  CHILD_BKP_MNT=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c param_getvolsall" | grep -v "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" | grep "DataVolumeName0001" | awk '{print $4}' `
  set -x 
  echo "INFOMSG: DataVolumeName0001 = $CHILD_BKP_MNT"
  CHILD_BKP_MNT=`df -P $CHILD_BKP_MNT | awk 'NR==2{print $NF}'`
  FIN_CHLD_BKP_LOC=/$CHILD_BKP_MNT/$DBSID/LOG
  AUTOLOG_BKP_FILE=$FIN_CHLD_BKP_LOC/AUTOLOG_${DBSID}_BKP.dat

  if [ ! -d $FIN_CHLD_BKP_LOC ]; then
     mkdir -p $FIN_CHLD_BKP_LOC
  fi

  find /$CHILD_BKP_MNT -type d -exec chmod 755 {} +
  CHILD_BKP_MNT_BASE=`dirname /$CHILD_BKP_MNT`
  chown -R $OSUSER:$OSGROUP $CHILD_BKP_MNT_BASE

  set +x
  echo "INFOMSG: Creating autolog backup template ($AUTOLOG_BKP_TEMP_NAME)"
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c backup_template_create $AUTOLOG_BKP_TEMP_NAME to file $AUTOLOG_BKP_FILE content log"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Failed to create Autolog backup template post appaware mount"
    exit 0
  fi
  else 
   echo "Autolog backup template Already exist"
  fi

  #Temp created ----


#Set the Log backup Interval if Not set

  if [ x"$INTERVAL"="x" ]; then
     INTERVAL=3600
  fi

#Disable the autolog overwrite feature
  set +x
  echo "INFOMSG: Changing autolg overwrite to OFF"
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_execute SET LOG AUTO OVERWRITE OFF"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Unable to change Autolg Overwrite to OFF"
    exit 0
  fi


#Check DB_STATE
  set +x
  getDB_State "$OSUSER" "$DBSID" "$MAXDB_KEY"
  set -x
  echo "$DB_STATE"

  if [ !  "$DB_STATE" = "ONLINE" ]; then

  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_online"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Unable to bring the database in Online Mode Before Enabling the autolog"
    exit 0
  fi

  fi

#Check the Autolog Backup state:

 set +x
 AUTOLOLOG_STATUS=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c autolog_show" | grep -v "OK"`
 set -x
 echo "Autolog Backup status is : $AUTOLOLOG_STATUS "
 if [ ! "$AUTOLOLOG_STATUS" = "AUTOSAVE IS ON" ]; then

#Enable the Autolog Backup
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c autolog_on $AUTOLOG_BKP_TEMP_NAME INTERVAL $INTERVAL"
  retval=$?
  set -x

   if [ $retval -ne 0 ]; then
     echo "WARNINGMSG: fail Enable Autolog Backup"
     #exit 0
   fi

  else

  echo "Autolog Backup is already on"

  fi

fi

#Check db backup type
if [ "$DUMPSCHEDULE" = "F" ]; then
  DUMPSCHEDULE_DB="F"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
  DUMPSCHEDULE_DB="I"
else
  DUMPSCHEDULE_DB=${DUMPSCHEDULE:`date +%w`:1}
fi

#Find db size
set +x
dbSizeInfo=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY info DATA" | grep "Used Space " | grep -v "%"`
dbSize=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY info DATA" | grep "Used Space " | grep -v "%" | awk '{sum+=$NF} END {print sum}'`
set -x
echo $dbSize

sizeType=`echo $dbSizeInfo| cut -d'(' -f2| cut -d ')' -f1`
echo "DBSize: $dbSize $sizeType"
if [ "$sizeType" = "GB" ]; then
  dbSize=`expr $dbSize \* 1024 \* 1024 \* 1024`
elif [ "$sizeType" = "MB" ]; then
  dbSize=`expr $dbSize \* 1024 \* 1024`
elif [ "$sizeType" = "KB" ]; then
  dbSize=`expr $dbSize \* 1024`
fi

echo "DBDUMPSIZE=$dbSize"
echo "DBDUMPTYPE=$DUMPSCHEDULE_DB"

exit 0
