#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

usage() {
  echo "Usage: "
  echo "/act/custom_apps/maxdb/lvm_migrate/ACT_MAXDB_lvm_migrate_newTarget.sh OSUSER=<OSUSER> DBSID=<DBSID> DBUSER=<DBUSER> DBPASSWORD=<DBPASSWORD> JOBID=<Mount_JOB_ID> LOCAL_PRG_PATH=<MAXDB_HOME> DATAVOL_DISK_MAPPING=<Actifio_Mount>:<Production_LVM_device>"
  echo ""
  echo "Example:"
  echo "/act/custom_apps/maxdb/lvm_migrate/ACT_MAXDB_lvm_migrate_newTarget.sh OSUSER=sdb DBSID=DEMODB DBUSER=dbm DBPASSWORD= JOBID=Job_0377678 LOCAL_PRG_PATH=/opt/sdb/MaxDB DATAVOL_DISK_MAPPING=/mmTST/maxdbdata:/dev/mapper/vg1-m7"
  echo ""
}
usage

# Get Common functions
. /act/custom_apps/maxdb/act_maxdb_common_func.sh

isVarNull OSUSER $OSUSER
isVarNull DBSID $DBSID
isVarNull DBUSER $DBUSER
isVarNull JOBID $JOBID
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH
isVarNull DATAVOL_DISK_MAPPING $DATAVOL_DISK_MAPPING

set -x
DATAVOL_DISK_MAPPING=$DATAVOL_DISK_MAPPING
ACT_JOBNAME=$JOBID

OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$DBUSER" "$DBPASSWORD"
set -x

#Restrict UnMount if pvmove is in progress
MAXDB_MIGRATE_CONF=/act/touch/.ACT_MAXDB_migrate_$DBSID.conf

#-------------------------------------------------------------------
#Prepare tmp location for temp files
#-------------------------------------------------------------------
tmp_loc=/act/tmpdata/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
  mkdir -p $tmp_loc
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $tmp_loc

export PATH=$LOCAL_PRG_PATH/bin:$PATH
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_commentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -e "/$devicename/ s/^#*/#/" -i /etc/fstab
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
call_uncommentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -i "/$devicename/s/^#//g" /etc/fstab
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
call_uncommentfstabs()
{
  for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
  do
    PROD_VOL=`echo $datavolMap|cut -d: -f2`
    devicename=`echo $devicename |sed 's|\/|\\\/|g'`
    sed -i "/$devicename/s/^#//g" /etc/fstab
  done
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
remove_file()
{
  local fname=$1
  if [ -f $fname ]; then
    rm -f $fname
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1
  local PROD_VG_NAME=$2
  echo "INFOMSG: Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     call_uncommentfstabs
     exit 1
  fi

  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     call_uncommentfstabs
     exit 1
  fi

  #Update LV_MEARGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MEARGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "INFOMSG: Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"

  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     call_uncommentfstabs
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1
  isMntd=`grep -w $mntpnt /proc/mounts | uniq | wc -l`
  if [ $isMntd -eq 1 ]; then
    echo "Before unmount: "`date`
    for i in {1..5}
    do
      umount $mntpnt
      systemctl stop $mntpnt
      systemctl daemon-reload
      grep " $mntpnt " /proc/mounts
      if [ $? -eq 0 ]; then
        if [ $i -eq 5 ]; then
	  echo "****Checking the fuser process****"
	  fuser -c -u $mntpnt
          echo "ERRORMSG: Fail to unmount $mntpnt after retries"
	  mount -a
	  call_uncommentfstabs
          exit 1
        fi
        echo "ERRORMSG: Fail to unmount $mntpnt. Retrying"
        sleep 5
      else
	echo "$mntpt is unmounted"
        break
      fi
    done
  fi
  echo "INFOMSG: Successfully unmounted $mntpnt"
  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
stop_application()
{
  echo "****** Stopping database: $dbsid ******"

  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY  -c db_offline"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    return 1
  fi
}
#-------------------------------------------------------------------

#---------------------------------------------------------------------------
# Update the Parameter file for new location of data and log file. 
#---------------------------------------------------------------------------
Update_data_log_loc() {
  local vol_name=$1
  local file_name=$2

  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY param_put $vol_name  $file_name"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo " Parameter file update failed for Data and Log Location"
    exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
configure_maxdb() {

  #Login the Target Instance to get Target file details :
  TRG_VOLINFO=$tmp_loc/TRG_VOLINFO
  TRG_VOLINFO1=$tmp_loc/TRG_VOLINFO1

  #clear the previous config files
  remove_file $TRG_VOLINFO
  remove_file $TRG_VOLINFO1
  
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -o $TRG_VOLINFO param_getvolsall"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Unable to find the Target Instane Deatils."
    return 1
  fi

  cat $TRG_VOLINFO | grep -vw "OK\|UseMirroredLog\|MaxLogVolumes\|MaxDataVolumes" >$TRG_VOLINFO1
  
  #Edit TRG_VOLINFO1 to have new mount-point locations
  while read line
  do
    mntP=`echo $line| awk '{print $3}'`
    mntA=`echo $line | awk '{print $1}'`
    mntP_sed=`echo $mntP | sed 's./.\\\/.g' `
    mntA_sed=`echo $mntA | sed 's./.\\\/.g' `
    sed -i "s.//./.g" $TRG_VOLINFO1
    sed -i "s/$mntA_sed/$mntP_sed/g" $TRG_VOLINFO1
  done < $MT_MEARGE_LIST

  while read line
  do
    vol_name=`echo $line | grep "VolumeName" | awk '{ print $1 }'`
    trg_file_name=`echo $line | grep "VolumeName" | awk '{ print $4}'`
    Update_data_log_loc $vol_name $trg_file_name
  done < $TRG_VOLINFO1
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
start_application()
{
  echo "****** Starting database: $dbsid ******"

  set -x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -c db_online"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
     exit $retval
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
#Prepare temp files
#-------------------------------------------------------------------
LV_MEARGE_LIST=$tmp_loc/LV_MEARGE_LIST_$ACT_JOBNAME.txt
VG_MEARGE_LIST=$tmp_loc/VG_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST=$tmp_loc/PV_MEARGE_LIST_$ACT_JOBNAME.txt
MT_MEARGE_LIST=$tmp_loc/MT_MEARGE_LIST_$ACT_JOBNAME.txt
NST_MNTPNTS=$tmp_loc/NST_MNTPNTS_$ACT_JOBNAME.txt
NST_MNTPNTS_toMount=$tmp_loc/NST_MNTPNTS_toMount_$ACT_JOBNAME.txt
UNMNT_SORT_LIST=$tmp_loc/UNMNT_SORT_LIST_$ACT_JOBNAME.txt
MNT_SORT_LIST=$tmp_loc/MNT_SORT_LIST_$ACT_JOBNAME.txt
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST
remove_file $MT_MEARGE_LIST
remove_file $UNMNT_SORT_LIST
remove_file $MNT_SORT_LIST
remove_file $NST_MNTPNTS
remove_file $NST_MNTPNTS_toMount
remove_file $NST_MNTPNTS_toMount.tmp
#-------------------------------------------------------------------

#For SAP, DATAVOL_DISK_MAPPING will be :: seperated values
DATAVOL_DISK_MAPPING=`echo $DATAVOL_DISK_MAPPING | sed 's/::/,/g'`

#Prepare files conntents and also check lvm size
#---------------------------------------------------------------------------------------------------------
echo "Creating Merge list......!!"
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  call_commentfstab $PROD_VOL
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    call_uncommentfstabs
    exit 1
  fi

  #Get LV paths
  echo "$ACT_LV_PATH $PROD_LV_PATH"  >> $LV_MEARGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MEARGE_LIST

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  for ACT_PV_DEV in $(echo $ACT_PV_DEV_LIST|sed 's/,/ /g'); do
    echo "$ACT_PV_DEV $PROD_PV_DEV_LIST" >> $PV_MEARGE_LIST
  done

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
    echo "ERRORMSG: unable to find production volume $PROD_VOL !Please check logs for more details"
    call_uncommentfstabs
    exit 1
    PROD_MNT=$ACT_MNT
  fi
  echo "$ACT_MNT $PROD_VOL $PROD_MNT" >> $MT_MEARGE_LIST
done

#Creating Unmount Sort List####################
cat $MT_MEARGE_LIST | awk '{print $1}' >> $UNMNT_SORT_LIST
cat $MT_MEARGE_LIST | awk '{print $3}' >> $UNMNT_SORT_LIST
cat $UNMNT_SORT_LIST | sort -r > $UNMNT_SORT_LIST.tmp
mv $UNMNT_SORT_LIST.tmp $UNMNT_SORT_LIST

#Create mount sort list
cat $MT_MEARGE_LIST | awk '{print $3}' | sort > $MNT_SORT_LIST

NSTMNT_CHECK=`cat /act/touch/*"$ACT_JOBNAME"_params | grep "NESTEDVOLUME_UNMOUNT" |tail -1| cut -d"=" -f2`
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Preparing mearge files completed"

#Find additional mounts
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt1/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt1/"`
  echo "$nstMnt" >> $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt2/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt2/"`
  echo "$nstMnt" >> $NST_MNTPNTS
done < $MT_MEARGE_LIST

#Sort, uniq
cat $NST_MNTPNTS | sort -r | uniq > $NST_MNTPNTS.tmp
mv $NST_MNTPNTS.tmp $NST_MNTPNTS
sed -i '/^$/d' $NST_MNTPNTS

#Remove migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  grep -vw "$umnt1\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  grep -vw "$umnt2\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS
done < $MT_MEARGE_LIST

echo "INFOMSG: Additional mountpoints to unmount"
cat $NST_MNTPNTS

#Update NST_MNTPNTS_toMount with mount command
cat $NST_MNTPNTS | sort > $NST_MNTPNTS_toMount
while read line
do
  if [ -z $line ]; then
    continue
  fi
  mntDev=`grep " $line " /proc/mounts | awk '{print $1}' | uniq`
  echo "mount $mntDev $line" >> $NST_MNTPNTS_toMount.tmp
done < $NST_MNTPNTS
mv $NST_MNTPNTS_toMount.tmp $NST_MNTPNTS_toMount
cat $NST_MNTPNTS_toMount


#stop the application
#-------------------------------------------------------------------
stop_application
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Pre Mount: Failed to stop MAXDB services. Please check!"
  exit 1
fi

echo "INFOMSG: Application has been stopped successfully"
#-------------------------------------------------------------------


#Update hidden migrate config file to handel unmount+delete
if [ ! -f $MAXDB_MIGRATE_CONF ]; then
  echo -e "MIGRATED=NO" > $MAXDB_MIGRATE_CONF
fi

echo "INFOMSG: Actifio CBT stoping..."
call_stop_tracking
echo "INFOMSG: Actifio CBT has been stopped successfully"

#Unmount all mounts
#---------------------------------------------------------------------------------------------------------
#Unmount additional mounts
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  #unmntpnt=`echo $line | awk '{print $3}'`
  unmntpnt=`echo $line | awk '{print $1}'`
  if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
    unmount_mnt $unmntpnt
  else
    echo "ERRORMSG: Found nested mount points ($unmntpnt). cannot proceed..."
    call_uncommentfstabs
    exit 1
  fi
done < $NST_MNTPNTS

#Unmount migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  unmount_mnt $umnt2
done < $MT_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Unmounted all required mount points"

#LV rename to get diff lv names
#---------------------------------------------------------------------------------------------------------
cnt=100
remove_file $LV_MEARGE_LIST.tmp
while read line
do
  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "INFOMSG: Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MEARGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MEARGE_LIST
mv $LV_MEARGE_LIST.tmp $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Renaming actifio lvm names done (temparory)"


#VG mearge
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: VG mearge completed....."


#LV rename and update
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: LV rename completed....."


#Mount devices
#---------------------------------------------------------------------------------------------------------
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  #mntP=`echo $line| awk '{print $3}'`
  #mntD=`echo $line | awk '{print $2}'`

  mntP=$line
  mntD=`grep " $line$" $MT_MEARGE_LIST | awk '{print $2}'`

  mount -o rw,nouuid $mntD $mntP
  if [ $? -ne 0 ]; then
    mount -o rw $mntD $mntP
    if [ $? -ne 0 ]; then
      #echo "INFOMSG: Device Already Mounted to $mntP"
      echo "ERRORMSG: Fail to mount $mntD to $mntP"
      exit 1
    fi
  fi
done < $MNT_SORT_LIST

#Mount additional mounts
if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
  echo "INFOMSG: Mounting the Additional Mountpoints...."
  while read line
  do
    addMntPnt=`echo $line | awk '{print $3}'`
    if [ ! -d "$addMntPnt" ]; then
      mkdir -p $addMntPnt
    fi
    $line
  done < $NST_MNTPNTS_toMount
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Mounting mount points completed....."


#PV move
#---------------------------------------------------------------------------------------------------------
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_PV_NAME=`echo $line | awk '{print $2}'`
  PROD_PV_NAME=`echo $PROD_PV_NAME | sed 's/,/ /g'`
  echo "Moving Physical Volumes: $ACT_PV_NAME $PROD_PV_NAME"

  pvmove --atomic -A y -b -q $ACT_PV_NAME $PROD_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to initiate pvmove $ACT_PV_NAME $PROD_PV_NAME"
    exit 1
  fi
done < $PV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove initiated successfully .........."


# Reconfigure DB data and log path 
#---------------------------------------------------------------------------------------------------------
configure_maxdb
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to configure application after pvmove initilization."
  exit 1
fi

#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: DB reconfigureation completed successfully"


# Start the application 
#---------------------------------------------------------------------------------------------------------
start_application
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Post Mount: Failed to start MAXDB services. Please check!"
  exit 1
fi

#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Application started successfully"


echo "INFOMSG: pvmove starting ......................"
#---------------------------------------------------------------------------------------------------------
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove completed successfully ....."

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    call_uncommentfstabs
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    call_uncommentfstabs
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "Actifio pvs removed successfully .................."

#Update hidden migrate config file to handel unmount+delete
echo -e "MIGRATED=YES" > $MAXDB_MIGRATE_CONF

echo "INFOMSG: Starting Actifio CBT..."
call_start_tracking
echo "INFOMSG: Actifion CBT started successfully"

if [ -d "$tmp_loc" ]; then
  rm -rf $tmp_loc
fi

echo "************ Migration Operation Completed Successfully ***************"

exit 0
