#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x
OSUSER=$OSUSER
DBSID=$DBSID
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
DUMPBKPLOC=$TARGET_MNT_PNT
ACT_JOBNAME=$JOBID

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

#Set Path Variable
export PATH=$LOCAL_PRG_PATH/bin:$PATH

#Convert DBSID to upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SRC_BACKUP_USER" "$SRC_USER_PASSWD"
set -x

#Create temp dir
tmp_loc=/act/touch/$ACT_JOBNAME
if [ ! -d $tmp_loc ]; then
  mkdir -p $tmp_loc
  chmod 755 /act/touch/$ACT_JOBNAME
fi

#Change permissions
OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $LOG_BKP_MNTPT
chown -R $OSUSER:$OSGROUP $tmp_loc

# prepare required flages, files
REC_OUTPUT=$tmp_loc/RECOVERY_OUTPUT.log
REC_INPUT=$tmp_loc/RECOVERY_INPUT.sql
logbkpFiles_list=$tmp_loc/logbkpFiles_list.txt
echo "db_connect" > $REC_INPUT
UN_TM=`echo $UNTIL_TIME | sed 's/-//g' | sed 's/://g'`
flagR=0

#Clear log area
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_execute clear log"
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: fail to clear log area"
  exit 1
fi

#Get the last logsequence number Restored from the backup.
set +x
LastLogSeq=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -uUTL -c db_restartinfo" | grep -w "Used LOG Page" | awk '{print $NF}'`
set -x
echo "LastLogSeq: $LastLogSeq"

#Get the first log file
FFile=`find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newermt "$BEGIN_TIME" | sort| head -1`
FLAG_FF=0
while [ $FLAG_FF -eq 0 ]; do
  recov_check $FFile
  FFile_next=`find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newer $FFile -exec stat  -c "%n %y" {} \; | sort -k2 | head -1 | awk '{print $1}'`
  FFile_Priv=`find $LOG_BKP_MNTPT/$DBSID/LOG -type f ! -newer $FFile -exec stat  -c "%n %y" {} \; | sort -k2 | tail -2 | head -1 | awk '{print $1}'`
  if [ $LastLogSeq -lt $F_SEQ ]; then
    FFile=$FFile_Priv
  elif [ $LastLogSeq -gt $L_SEQ ]; then
    FFile=$FFile_next
  else
    FLAG_FF=1
  fi
done

##############################
#get log bkp files
##############################
echo "$FFile" > $logbkpFiles_list
find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newer $FFile -exec stat  -c "%n %y" {} \; | sort -k2 | awk '{print $1}'  >> $logbkpFiles_list
find $LOG_BKP_MNTPT/$DBSID/LOG -type f -newermt "$UNTIL_TIME" -exec stat  -c "%n %y" {} \; | sort -k2 | head -1 | awk '{print $1}' >> $logbkpFiles_list

logbkpFiles=`cat $logbkpFiles_list | sort| uniq | sed '/^$/d'`
if [ -z "$logbkpFiles" ]; then
  echo "No Log file to recover"
  exit 0
fi

#Prepare recovery script to recover in one session
for logFile in $logbkpFiles
do
  echo "backup_template_create act_maxdb_Log_rst_tmplt to FILE $logFile CONTENT LOG" >> $REC_INPUT
  if [ $flagR -eq 0 ]; then
    echo "recover_start act_maxdb_Log_rst_tmplt LOG until $UN_TM" >> $REC_INPUT
    flagR=1
  else
    echo "recover_replace act_maxdb_Log_rst_tmplt $logFile" >> $REC_INPUT
  fi
done
echo "recover_ignore" >> $REC_INPUT

#Recover the database with on session
chown $OSUSER $REC_INPUT
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY -i $REC_INPUT -o $REC_OUTPUT"
set -x
cat $REC_OUTPUT

#Check for error
isErr=`grep ERR $REC_OUTPUT | wc -l`
if [ $isErr -gt 0 ]; then
  set +x
  dbState=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_state" | grep -vw "OK\|State" | xargs`
  set -x

  if ! [ "$dbState" = "ONLINE" ]; then
    echo "ERRORMSG: Fail to recover the database"
    cat $REC_OUTPUT
    exit 1
  fi
fi


#Remove temp files
rm -rf $tmp_loc
