#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

config_file=/act/custom_apps/maxdb/restore/ACT_MAXDB_lvmRestore_newTarget.conf

usage() {
  echo "Usage: "
  echo "/act/custom_apps/maxdb/restore/ACT_MAXDB_lvmRestore_newTarget.sh OSUSER=<OSUSER> TARGET_SERVER_NAME=<TARGET_SERVER_NAME> TARGET_DB_USER=<TARGET_DB_USER> TARGET_DBUSER_PASSWD=<TARGET_DBUSER_PASSWD/null> SOURCE_DBSID=<SOURCE_DBSID> MANIFEST_FILE_LOC=<MANIFEST_FILE_LOC> SRC_DB_VERSION=<SRC_DB_VERSION> TARGET_MNT_PNT=<TARGET_MNT_PNT> [ UNTIL_TIME=<UNTIL_TIME> BEGIN_TIME=<BEGIN_TIME> LOG_BKP_MNTPT=<LOG_BKP_MNTPT> ]"
  exit 1
}

isProvided() {
  local varName=$1
  local varVal=$2
  if [ -z "$varVal" ]; then
    echo "ERRORMSG: $varName is empty."
    usage
    exit 1
  fi
}

if [ $# -eq 0 ]; then
 
   if [ ! -f $config_file ]; then 
      echo "ERRORMSG: Configfile $config_file is missing"
      echo " Please create the config file $config_file and update the required parameter in the folllowing format. for example :"
      echo "cat $config_file" 
      echo "OSUSER=sdb"
      echo "TARGET_SERVER_NAME=TMS"
      echo "TARGET_DB_USER=dbm"
      echo "TARGET_DBUSER_PASSWD=welcome123"
      echo "MANIFEST_FILE_LOC=/act/tmpdata/Job_0818287/Manifest_File_SLD"
      echo "SRC_DB_VERSION=7.9.09"
      echo "SOURCE_DBSID=SLD"
      echo "LOG_BKP_MNTPT=lvm22_archivelog"
      echo "BEGIN_TIME='2019-12-20 06:45:07'"
      echo "UNTIL_TIME='2019-12-20 09:40:07'"
      echo "TARGET_MNT_PNT=/mntpt"
        exit 1
   else
     source /act/custom_apps/maxdb/restore/ACT_MAXDB_lvmRestore_newTarget.conf
     export OSUSER
     export TARGET_SERVER_NAME
     export TARGET_DB_USER
     export TARGET_DBUSER_PASSWD
     export MANIFEST_FILE_LOC
     export SRC_DB_VERSION
     export SOURCE_DBSID 
     export LOG_BKP_MNTPT
     export BEGIN_TIME
     export UNTIL_TIME
     export TARGET_MNT_PNT
   fi
else
  for ARGUMENT in "$@"
    do
       ARGUMENT="${ARGUMENT%\"}"
       ARGUMENT="${ARGUMENT#\"}"
       argLft=`echo $ARGUMENT|cut -d= -f1`
       argRit=`echo $ARGUMENT|cut -d= -f2`
       export "$argLft=$argRit"
    done
fi

 isProvided OSUSER $OSUSER
 isProvided TARGET_SERVER_NAME $TARGET_SERVER_NAME
 isProvided TARGET_DB_USER $TARGET_DB_USER
 isProvided SOURCE_DBSID $SOURCE_DBSID
 isProvided MANIFEST_FILE_LOC $MANIFEST_FILE_LOC
#isProvided LOCAL_PRG_PATH $LOCAL_PRG_PATH
 isProvided SRC_DB_VERSION $SRC_DB_VERSION
 isProvided TARGET_MNT_PNT $TARGET_MNT_PNT

   if [ ! -z "$UNTIL_TIME" ]; then
     isProvided BEGIN_TIME $BEGIN_TIME
     isProvided LOG_BKP_MNTPT $LOG_BKP_MNTPT
   fi

set -x
if [ ! -z "$ACT_JOBNAME" ]; then
  JOBID=$ACT_JOBNAME
else
  JOBID=JOB_54321
  export JOBID
fi

LOCAL_PRG_PATH=`dbmcli -d $TARGET_SERVER_NAME dbm_version INSTROOT | grep -vw "OK"`
export PATH=$LOCAL_PRG_PATH:$PATH

#Call the Prerestore script to validate the env

 /act/custom_apps/maxdb/restore/ACT_MAXDB_PreRestore_newTarget.sh
 retval=$?

 if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Preresore script failed. Please check the log"
    exit 1
 else
    echo "Proceeding with the lvm restore for single database"

    /act/custom_apps/maxdb/restore/ACT_MAXDB_Postrestore_newTarget.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "ERRORMSG: LVM Restore failed. Please check the error log"
      exit 1
    else
       echo "LVM Restore succeeded"
    fi
 fi

exit 0



