#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
DBSID=$DBSID
SOURCE_SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SOURCE_SRC_USER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH
UNTIL_TIME=$UNTIL_TIME
export PATH=$LOCAL_PRG_PATH/bin:$PATH

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

isVarNull DBSID $DBSID
isVarNull SOURCE_SRC_BACKUP_USER $SOURCE_SRC_BACKUP_USER
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SOURCE_SRC_BACKUP_USER" "$SOURCE_SRC_USER_PASSWD"
set -x

#Convert the DBSID in Upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

###########################
#Function : Start  Database 
###########################
db_start()
{ 
  local mode=$1
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY $mode"
  retval=$?
  set -x

  if [ $retval -eq 0 ]; then
    echo " Database $DBSID started in $mode mode  successfully"
  else
    echo "ERRORMSG: fail to change db mode to $mode"
    exit 1
  fi
}


################################
#Function : Check Database State
################################
check_db_state()
{
  set +x
  DB_STATUS=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_state" | grep -v "OK" | grep -v "State"`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: fail to get db state"
    exit 1
  fi
}

###########################################
# Check the DB state and start the database 
###########################################
check_db_state
if [ "$DB_STATUS"  = "OFFLINE" ]; then
  echo "Starting Database in admin mode"
  db_start db_admin
  echo "Database started in ADMIN mode"
fi

check_db_state
if [ "$DB_STATUS" != "ADMIN" ]; then
  echo "ERRORMSG: Fail to make the database into admin mode"
  exit 1
fi

#Recovery database if needed
if [ ! -z "$UNTIL_TIME" ]; then
  /act/custom_apps/maxdb/restore/ACT_MAXDB_RESTORE_recovery.sh
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Log recovery failed"
    exit 1
  fi
fi

check_db_state
if [ "$DB_STATUS" = "ADMIN" ]; then
  echo "Starting the database"
  db_start db_online
fi

#Please run the dummy Full backup to enable autolog backup

echo "Running the dummy full backup post the LVM Restore to enable log backup"

export DBSID=$DBSID
export SRC_BACKUP_USER=$TARGET_DB_USER
set +x
export SRC_USER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
export JOBID=$JOBID
export DUMPSCHEDULE=F

/act/custom_apps/maxdb/clone/ACT_MAXDB_dump.sh
  if [ $? -ne 0 ]; then
    echo "WARNINGMSG: Failed to trigger the dummy backup post the restore. Please run one dump backup manually and  enable autolog backup to reprotect" 
    exit 0
  fi

#Enable the autolog backup 
#Get the autolog template name

set +x
AUTOLOG_TEMP_NAME=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY  backup_template_show all" | grep -w "AUTO" -B 1 | grep "NAME" | awk '{print $NF}'`
set -x

if [ -z $AUTOLOG_TEMP_NAME ]; then
  set +x 
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY autolog_on"
  retval=$?
  set -x 

  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Failed to bring the autolog on as backup_template name not found. Please enable the autolog backup manually"
    exit 0
  fi

else
  set +x 
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY autolog_on $AUTOLOG_TEMP_NAME"
  retval=$?
  set -x
 
  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Failed to bring the autolog on. Please enable the autolog backup manually"
  fi 
fi
 
exit 0
