#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x 
DBSID=$DBSID
SOURCE_SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SOURCE_SRC_USER_PASSWD="$TARGET_DBUSER_PASSWD"
set -x
LOCAL_PRG_PATH=$LOCAL_PRG_PATH

. /act/custom_apps/maxdb/act_maxdb_common_func.sh

export PATH=$LOCAL_PRG_PATH/bin:$PATH

isVarNull DBSID $DBSID
isVarNull SOURCE_SRC_BACKUP_USER $SOURCE_SRC_BACKUP_USER
isVarNull LOCAL_PRG_PATH $LOCAL_PRG_PATH

#Check the No of characters in DB Name
getDBName_charCount $DBSID

#Convert the DBSID in Upper case
DBSID=`echo $DBSID | tr "[:lower:]" "[:upper:]"`

#prepare MAXDB_KEY
set +x
prepareMaxdbKey "$SOURCE_SRC_BACKUP_USER" "$SOURCE_SRC_USER_PASSWD"
set -x

#check the password authentication
set +x
su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY bye"
retval=$?
set -x

if [ $retval -gt 0 ]; then
  set +x
  if [ -z "$SRC_USER_PASSWD" ]; then
    echo "ERRORMSG: User Authentication Failed. Please check userkey"
  else
    echo "ERRORMSG: User Authentication Failed. Please check username and password"
  fi
  set -x
  exit 1
fi

#Stop Database 
db_stop()
{
  set +x
  su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_offline" 
  retval=$?
  set -x

  if [ $retval -eq 0 ]; then
    echo " Database $DBSID stopped successfully"
  else
    echo "ERRORMSG: Database $DBSID stop Failed " 
    exit 1
  fi
}

set +x
DB_STATUS=`su -m $OSUSER -c "dbmcli -d $DBSID $MAXDB_KEY db_state" | grep -v "OK" | grep -v "State"`
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: fail to get the db state"
  exit 1
fi

if [ "$DB_STATUS"  = "ONLINE" ] || [ "$DB_STATUS"  = "ADMIN" ]; then
  echo "Stopping Database ..."
  db_stop
elif  [ "$DB_STATUS"  = "OFFLINE" ]; then
  echo "DB is already shutdown...."
else
  echo "ERRORMSG: DB is unconsistant state.....Check the DB Log"
  exit 1
fi

exit 0
