#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


#-------------------------------------------------------------------
# function: find_DBDIR()
#-------------------------------------------------------------------
find_DBDIR() {
  local datadir=$1
  local dbname=$2
  DB_DIR=

  local IS_LINK=`ls -l $datadir/$dbname | grep " \-> " | wc -l`
  if [ $IS_LINK -eq 1 ]; then
    DB_DIR=`ls -l $datadir/$dbname | awk '{print $NF}'`
  else
    DB_DIR="$datadir/$dbname"
  fi
}
#-------------------------------------------------------------------



#-------------------------------------------------------------------
# function: removeFile()
#-------------------------------------------------------------------
removeFile() {
   local fileName=$1
   if [ -e "$fileName" ]; then
      rm -f $fileName
   fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: isVarNull()
#-------------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    exit 1
  fi
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
# function: prepare_MYSQLCMD()
#-------------------------------------------------------------------
prepare_MYSQLCMD() {
  set -x
  echo "function call: prepare_MYSQLCMD() ...."
  local user=$1
  set +x
  local passwd=$2
  set -x
  local socketFile=$3
  local mysqlHome=$4

  MYSQL_CMD=
  MYSQL_DMP_CMD=
  MYSQL_ADMIN=

  # set env
  PATH=$mysqlHome/bin:$PATH

  # set password as env variable
  set +x
  export MYSQL_PWD="$DBPASSWORD"
  set -x

  # set default user if user is null
  if [ "$user" = "" ]; then
    user=root
  fi

  # prepare statement
  MYSQL_CMD="mysql --no-defaults -u $user -s -N "
  if [ "$socketFile" != "" ]; then
    MYSQL_CMD="$MYSQL_CMD -S $socketFile "
  fi

  # prepare dump statment
  MYSQL_DMP_CMD="mysqldump --no-defaults -u $user "
  if [ "$socketFile" != "" ]; then
    MYSQL_DMP_CMD="$MYSQL_DMP_CMD -S $socketFile "
  fi

  # prepare mysqladmin statement
  MYSQL_ADMIN="mysqladmin --no-defaults -u $user "
  if [ "$socketFile" != "" ]; then
    MYSQL_ADMIN="$MYSQL_ADMIN -S $socketFile "
  fi

  # prepare mysqlbinlog statement
  MYSQLBINLOG="mysqlbinlog --no-defaults -u $user "
  if [ "$socketFile" != "" ]; then
    MYSQLBINLOG="$MYSQLBINLOG -S $socketFile"
  fi

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: checkMySQL()
#-------------------------------------------------------------------
checkMySQL() {
  echo "function call: checkMySQL() ...."

  local IS_CONN=`su -m $OSUSER -c "$MYSQL_CMD -e \"select 'yes' from dual;\" "`
  if [ "$IS_CONN" != "yes" ]; then
    echo "WARNINGMSG: Failed to connect to mysql @$DATADIR with socket $SOCKETFILE "
    return 1
  fi

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: execMySQL()
#-------------------------------------------------------------------
execMySQL() {
  echo "function call: execMySQL() ...."
  local CMD=$1
  su -m $OSUSER -c "$MYSQL_CMD -e \"$CMD;\" "
  return $?
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: getMySqlVarValue()
#-------------------------------------------------------------------
getMySqlVarValue() {
  local varName=$1
  local varVal=`su -m $OSUSER -c "$MYSQL_CMD -e \"show variables like '$varName'; \" " | awk '{print $2}'`
  retval=$?

  echo $varVal
  return $retval
}
#-------------------------------------------------------------------


#--------------------------------------------------------------------
# Function: recoveryBinLog()
#--------------------------------------------------------------------
recoveryBinLog() {
  local binLog=$1
  local UNTIL_TIME=$2
  local DB_NAME=$3

  if [ ! -z "$DB_NAME" ]; then
    DB_REC=" --database=$DB_NAME"
  fi

  su -m $OSUSER -c "$MYSQLBINLOG $DB_REC --start-datetime=\"$BEGIN_TIME\" --stop-datetime=\"$UNTIL_TIME\" $binLog | $MYSQL_CMD -f"
  retval=$?

  return $retval
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# Function: renameDB()
#--------------------------------------------------------------------
renameDB() {
  local orig_dbname=$1
  local new_dbname=$2

  if [ "$orig_dbname" = "$new_dbname" ]; then
    return 0
  fi

  #Get required details
  #dbtables
  dbtables=`su -m $OSUSER -c "$MYSQL_CMD " <<EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$orig_dbname' and TABLE_TYPE='BASE TABLE';
EOF`
  echo "dbTables: $dbtables"

  #Triggers
  dbtriggers=`su -m $OSUSER -c "$MYSQL_CMD " <<EOF
select GROUP_CONCAT(TRIGGER_NAME SEPARATOR ' ') from information_schema.triggers where trigger_schema='$orig_dbname';
EOF`
  echo "dbtriggers: $dbtriggers"

  #dbviews
  dbviews=`su -m $OSUSER -c "$MYSQL_CMD " <<EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$orig_dbname' and TABLE_TYPE='VIEW';
EOF`
  echo "dbviews: $dbviews"

  #char set
  dbcharset=`su -m $OSUSER -c "$MYSQL_CMD -e 'show create database $orig_dbname' " `
  dbcharset=`echo $dbcharset | awk -F'CHARACTER SET ' '{print $2}' | awk '{print $1}'`
  echo "dbcharset: $dbcharset"

  #get dump of Triggers and Views
  dumploc=/act/touch/$ACT_JOBNAME
  mkdir -p $dumploc
  #find /act -type d -exec chmod 755 {} +
  chown -R $OSUSER $dumploc

  DMP_TRIGG_FILE="$dumploc/$orig_dbname"_triggers.dump
  DMP_VIEWS_FILE="$dumploc/$orig_dbname"_views.dump

  if [ "$dbviews" != "NULL" ]; then
    su -m $OSUSER -c "$MYSQL_DMP_CMD $orig_dbname $dbviews > $DMP_VIEWS_FILE"
  fi

  if [ "$dbtriggers" != "NULL" ]; then
    su -m $OSUSER -c "$MYSQL_DMP_CMD $orig_dbname -d -t -R -E > $DMP_TRIGG_FILE"
  fi

  # Drop triggers if exists
  if [ "$dbtriggers" != "NULL" ]; then
    for trgName in $dbtriggers
    do
      su -m $OSUSER -c "$MYSQL_CMD -e 'DROP TRIGGER IF EXISTS $orig_dbname.$trgName'"
    done
  fi

  echo "Renaming $orig_dbname to $new_dbname ........"
  echo "Creating database $new_dbname......"
  su -m $OSUSER -c "$MYSQL_CMD -e 'create database $new_dbname CHARACTER SET $dbcharset;'"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to create new database with name $new_dbname"
    return 1
  fi

  echo "Renaming tables ......"
  if [ "$dbtables" != "NULL" ]; then
    for tabName in $dbtables; do
      su -m $OSUSER -c "$MYSQL_CMD -e 'SET FOREIGN_KEY_CHECKS=0;RENAME TABLE $orig_dbname.$tabName to $new_dbname.$tabName;'"
    done
  fi

  echo "Load views if exists"
  if [ "$dbviews" != "NULL" ]; then
    su -m $OSUSER -c "$MYSQL_CMD $new_dbname < $DMP_VIEWS_FILE"
  fi

  echo "Load triggers if exists"
  if [ "$dbtriggers" != "NULL" ]; then
    su -m $OSUSER -c "$MYSQL_CMD $new_dbname < $DMP_TRIGG_FILE "
  fi

  #Drop old database
  dbtables=
  dbtables=`su -m $OSUSER -c "$MYSQL_CMD " <<EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$orig_dbname' and TABLE_TYPE='BASE TABLE';
EOF`
  if [ "$dbtables" = "NULL" ]; then
    su -m $OSUSER -c "$MYSQL_CMD -e 'drop database $orig_dbname' "
  fi

  #Update privilizes
  updatePrivs columns_priv
  updatePrivs procs_priv
  updatePrivs tables_priv
  updatePrivs db

  su -m $OSUSER -c "$MYSQL_CMD -e 'flush privileges' "

  #Remove temp files if exits
  if [ -e $DMP_TRIGG_FILE ]; then
    rm -f $DMP_TRIGG_FILE
  fi

  if [ -e $DMP_VIEWS_FILE ]; then
    rm -f $DMP_VIEWS_FILE
  fi

  return 0
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# Function: updatePrivs()
#--------------------------------------------------------------------
updatePrivs() {
  local prvType=$1

  prvSet=`su -m $OSUSER -c "$MYSQL_CMD -e \"select count(*)  from mysql.$prvType  where db='$orig_dbname'; \" "`
  if [ $prvSet -gt 0 ]; then
    su -m $OSUSER -c "$MYSQL_CMD -e \"update mysql.$prvType set db='$new_dbname' WHERE db='$orig_dbname';\" "
  fi

  return 0
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: Create the config file for remount
#--------------------------------------------------------------------
remount_config() {
  config_file=/act/touch/$ACT_JOBNAME/REMOUNT_CONFIG
  echo "OLD_MOUNT=$TARGET_MNT_PNT" > $config_file
  echo "REMOUNT_DB_LIST=$TARGET_DB_NAME" >> $config_file
}
#--------------------------------------------------------------------



#--------------------------------------------------------------------
# functon update_TRG_DBLIST
#--------------------------------------------------------------------
update_TRG_DBLIST() {
  local REMOUNT_CONFIG_file=$1
  local fmName=`grep REMOUNT_DB_LIST $REMOUNT_CONFIG_file | cut -d"=" -f2`
  local flName=$TARGET_DB_NAME

  TARGET_DB_NAME=
  TARGET_DB_NAME_orig=

  for ocf in $(echo $fmName | sed 's/,/ /g')
  do
    fName=`echo $ocf | cut -d: -f1`
    mName=`echo $ocf | cut -d: -f2`
    lName=

    for ncf in $(echo $flName |sed 's/,/ /g')
    do
      p_db=`echo $ncf | cut -d: -f1`
      n_db=`echo $ncf | cut -d: -f2`
      if [ "$p_db" = "$fName" ]; then
        lName=$n_db
        break
      fi
    done

    if [ -z $TARGET_DB_NAME ]; then
      TARGET_DB_NAME=$mName:$lName
      TARGET_DB_NAME_orig=$fName:$lName
    else
      TARGET_DB_NAME="$TARGET_DB_NAME,$mName:$lName"
      TARGET_DB_NAME_orig="$TARGET_DB_NAME_orig,$fName:$lName"
    fi
  done

  echo "TARGET_DB_NAME=$TARGET_DB_NAME"
  echo "TARGET_DB_NAME_orig=$TARGET_DB_NAME_orig"
}
#--------------------------------------------------------------------

#--------------------------------------------------------------------
# function cleanUpMount
#--------------------------------------------------------------------
cleanUpMount() {
  nL=`ps -ef | grep "\-\-port=$PORT" |grep "bin/mysqld" | grep -vw grep | awk '{print $2" "$3}' | wc -l`
  if [ $nL -eq 0 ]; then
    #Process not running
    return 0
  fi

  if [ $nL -gt 1 ]; then
    echo "ERRORMSG: Fail to find the pids to cleanup"
    exit 1
  fi

  killPID=`ps -ef | grep "\-\-port=$PORT" |grep "bin/mysqld" | grep -vw grep | awk '{print $2" "$3}'`
  echo "Killing pids to clean MySQL mount:  $killPID"
  kill -9 $killPID
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: isDB_exists()
#--------------------------------------------------------------------
isDB_exists() {
  local dbName=$1
  local dbexistsCount=`su -m $OSUSER -c "$MYSQL_CMD -e \"show databases;\" "|grep -w $dbName| wc -l`
  if [ $dbexistsCount -eq 0 ]; then
    return 1
  else
    return 0
  fi
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: checkPrivilege()
#--------------------------------------------------------------------
checkPrivilege() {
  local privR="$1"

  isAll=`su $OSUSER -c "$MYSQL_CMD -e 'show GRANTS FOR CURRENT_USER'" | grep 'GRANT ALL PRIVILEGES ON \*.\*' | wc -l`
  if [ $isAll -gt 0 ]; then
    return 0
  fi

  isPrivE=`su $OSUSER -c "$MYSQL_CMD -e 'show GRANTS FOR CURRENT_USER'" | grep ' \*.\* ' | grep " $1" | wc -l`
  if [ $isPrivE -eq 0 ]; then
    return 1
  fi
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: isOsUser()
#--------------------------------------------------------------------
isOsUser() {
   isUserExist=`grep -w ^$1 /etc/passwd | wc -l`
   if [ $isUserExist -eq 0 ]; then
      return 1
   fi
   return 0
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: waitForMySQL() <errorLog> <Mins>
#--------------------------------------------------------------------
waitForMySQL() {
   local errorLog=$1
   local waitMins=$2

   local waitSecs=$(( waitMins * 60 ))
   local sleepSecs=10
   local isSuccess=
   local ErrorMsgInLog=

   while [ -e "$errorLog" ] && [ $waitSecs -gt 0 ]
   do
      sleep $sleepSecs

      isSuccess=`grep "ready for connections" $errorLog`
      if [ ! -z "$isSuccess" ]; then
         return 0
      fi

      ErrorMsgInLog=`grep "\[ERROR\]" $errorLog | grep -v " Slave I/O for channel '': error connecting"`
      if [ ! -z "$ErrorMsgInLog" ]; then
         echo "$ErrorMsgInLog"
         return 2
      fi

      waitSecs=$(( waitSecs - sleepSecs ))
   done

   return 1
}
#--------------------------------------------------------------------


#--------------------------------------------------------------------
# function: getErrorLog() <defaults-file> <errorLog_VarName>
#--------------------------------------------------------------------
getErrorLog() {
   local defaultsFile=$1
   local varName=$2

   # Prepare duplicate defaults file where comment lines, new lines are deleted and reformated the paramter like "param_name=<value>"
   local tempDefaultFile=/act/touch/${JOBID}_defaultsFile.$RANDOM
   grep -v "^#" $defaultsFile | sed '/^$/d' | sed 's/\t/ /g' | sed 's/ \+/ /g' | sed 's/ =/=/g' | sed 's/^ //g' > $tempDefaultFile

   # Get log-error, log-basename and datadir valules
   local logError=`grep "^log_error=\|^log-error=" $tempDefaultFile | tail -1 | cut -d= -f2 | awk '{print $1}'`
   local logBasename=`grep "^log_basename=\|^log-basename=" $tempDefaultFile | tail -1 | cut -d= -f2 | awk '{print $1}'`
   local dataDir=`grep "^datadir=" $tempDefaultFile | tail -1 | cut -d= -f2 | awk '{print $1}'`

   # Find right log-error
   if [ -z "$dataDir" ]; then
      if [ -z "$logError" ] || [ ${logError:0:1} != "/" ]; then
         echo "WARNINGMSG: datadir parameter is empty and log-error value is not full path. Cannot find log-error file"
         removeFile $tempDefaultFile
         return 1
      fi
   fi

   if [ -z "$logError" ]; then
      if [ -z "$logBasename" ]; then
         logError=${dataDir}/$(hostname).err
      else
         logError=${dataDir}/${logBasename}.err
      fi
   elif [ ${logError:0:1} != "/" ]; then
      logError=${dataDir}/${logError}
   fi

   export $varName=$logError
   removeFile $tempDefaultFile
   return 0
}
#--------------------------------------------------------------------
