#!/bin/sh

set -x
OSUSER=$OSUSER
BASEDIR=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
SOCKETFILE=$SOCKET_FILE

if [ "$OSUSER" = "" ]; then
  echo "ERRORMSG: OSUSER is not provided"
  exit 1
fi

if [ "$BASEDIR" = "" ]; then
  echo "ERRORMSG: BASEDIR is not provided"
  exit 1
fi

if [ "$DBUSER" = "" ]; then
  echo "WARNINGMSG: DBUSER is not provided. Assuming default DBUSER is 'root' "
fi

set +x
if [ "$DBPASSWORD" = "" ]; then
  echo "WARNINGMSG: DBPASSWORD is not provided"
fi
set -x

if [ "$SOCKETFILE" = "" ]; then
  echo "WARNINGMSG: SOCKETFILE is not provided"
fi

## Verification
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: No such user $OSUSER"
  exit 1
fi

if [ ! -d $BASEDIR ]; then
  echo "ERRORMSG: Not a directory - $BASEDIR "
  exit 1
fi

if [ ! -d $BASEDIR/bin ]; then
  echo "ERRORMSG: bin directory not found in - $BASEDIR "
  exit 1
fi

if [ ! -f $BASEDIR/bin/mysql ]; then
  echo "ERRORMSG: mysql not found in - $BASEDIR/bin "
  exit 1
fi

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

# Prepare MySQL Command
set +x
prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

#------------------------------------------------------------
# Check DB connection with given credentials
#------------------------------------------------------------
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to connect to mysql with given credentials"
  exit 1
fi
#------------------------------------------------------------


#------------------------------------------------------------
# Check DB user privileges
#------------------------------------------------------------
# Required privileges: RELOAD, SELECT, LOCK TABLES, SUPER (or) REPLICATION CLIENT

checkPrivilege "SELECT"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: SELECT privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "RELOAD"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: RELOAD privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "LOCK TABLES"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: LOCK TABLES privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "SUPER"
isS=$?
checkPrivilege "REPLICATION CLIENT"
isR=$?

if [ $isS -ne 0 ] && [ $isR -ne 0 ]; then
  echo "ERRORMSG: SUPER and REPLICATION CLIENT privilege is missing to database user $DBUSER. Required atleast one."
  exit 1
fi
#------------------------------------------------------------
