#!/bin/sh

set -x
SRC_DATADIR=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
SRC_TMPDIR=$SRC_TMPDIR
MESSAGES_DIR=$MESSAGES_DIR
TARGET_MNT_PNT=$TARGET_MNT_PNT
BASEDIR=$BASEDIR
PORT=$PORT
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
TARGET_DB_NAME=$TARGET_DB_NAME
ACT_JOBNAME=$JOBID
REMOUNT_CONFIG_file=$CONFIG_FILE_LOC #Don't change this order as it may over-write CONFIG_FILE_LOC
CONFIG_FILE_LOC=$CONFIG_FILE

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

#Child restore
if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
   TARGET_MNT_PNT=
fi

OSGROUP=`id -gn $OSUSER`
SOCKFILE=/tmp/.MySQL_sock.$PORT
PIDFILE=$TARGET_MNT_PNT$SRC_DATADIR/act_mysql.pid
ERRORLOG=$TARGET_MNT_PNT$SRC_DATADIR/act_mysql.log
MYSQL_CNF=$TARGET_MNT_PNT$SRC_DATADIR/Act_mysqld-auto.cnf
DATETIME=`date +%Y%m%d`_`date +%H%M%S`

#Create temp location
mkdir -p /act/touch/$ACT_JOBNAME
chown -R $OSUSER:$OSGROUP /act/touch/$ACT_JOBNAME
chmod 755 /act/touch

#Change mount point permissions
if [ "$ACT_MOUNT_REPLACERESTORE" != "TRUE" ]; then
  chown -R $OSUSER:$OSGROUP $TARGET_MNT_PNT
  chmod -R 750 $TARGET_MNT_PNT
  TRG_MNT_BASE=`echo $TARGET_MNT_PNT | cut -d/ -f2`
  find /$TRG_MNT_BASE -type d -exec chmod 755 {} +
fi

if [ "$MESSAGES_DIR" = "" ]; then
  MESSAGES_DIR=$BASEDIR/share
fi

if [ ! -e $CONFIG_FILE_LOC ]; then
  echo "ERRORMSG: Config file not provided"
  exit 1
fi
removeFile $SOCKFILE
removeFile $PIDFILE
removeFile $ERRORLOG

#Make a cope of config file and edit it
cp $CONFIG_FILE_LOC $CONFIG_FILE_LOC.bkp_${ACT_JOBNAME}
CONFIG_FILE_LOC_orig=$CONFIG_FILE_LOC
CONFIG_FILE_LOC=$CONFIG_FILE_LOC.bkp_${ACT_JOBNAME}

#remove blankspaces
tr -d ' ' < $CONFIG_FILE_LOC > $CONFIG_FILE_LOC.tmp
tr -d '[:blank:]' < $CONFIG_FILE_LOC > $CONFIG_FILE_LOC.tmp
mv $CONFIG_FILE_LOC.tmp $CONFIG_FILE_LOC

## Edit config file ##
SOCKFILE_sed=`echo $SOCKFILE | sed 's./.\\\/.g' `
PIDFILE_sed=`echo $PIDFILE | sed 's./.\\\/.g' `
ERRORLOG_sed=`echo $ERRORLOG | sed 's./.\\\/.g' `
BASEDIR_sed=`echo $BASEDIR |sed 's./.\\\/.g' `
sed -i "s/socket=.*/socket=$SOCKFILE_sed/g" $CONFIG_FILE_LOC
sed -i "s/port=.*/port=$PORT/g" $CONFIG_FILE_LOC
sed -i "s/server_id=.*/server_id=5106$PORT/g" $CONFIG_FILE_LOC
sed -i "s/server-id=.*/server-id=5106$PORT/g" $CONFIG_FILE_LOC
sed -i "s/bind-address=.*/bind-address=localhost/g" $CONFIG_FILE_LOC
sed -i "s/bind_address=.*/bind-address=localhost/g" $CONFIG_FILE_LOC
sed -i "s/pid_file=.*/pid_file=$PIDFILE_sed/g" $CONFIG_FILE_LOC
sed -i "s/pid-file=.*/pid-file=$PIDFILE_sed/g" $CONFIG_FILE_LOC
sed -i "s/log_error=.*/log_error=$ERRORLOG_sed/g" $CONFIG_FILE_LOC
sed -i "s/log-error=.*/log-error=$ERRORLOG_sed/g" $CONFIG_FILE_LOC
sed -i "s/basedir=.*/basedir=$BASEDIR_sed/g" $CONFIG_FILE_LOC
sed -i "s/tmpdir=.*/tmpdir=\/tmp/g" $CONFIG_FILE_LOC
sed -i "s/log_bin=.*/log_bin=mysql-bin/g" $CONFIG_FILE_LOC
sed -i "s/log-bin=.*/log-bin=mysql-bin/g" $CONFIG_FILE_LOC
sed -i "/slow[-/_]query[-/_]log/d" $CONFIG_FILE_LOC

#Add missing, required options
isMysqlD_specified=`grep -w "\[mysqld\]" $CONFIG_FILE_LOC | wc -l`
if [ $isMysqlD_specified -lt 1 ]; then
  echo "[mysqld]" >> $CONFIG_FILE_LOC
fi
isDataDir_specified=`grep "^datadir=" $CONFIG_FILE_LOC | wc -l`
if [ $isDataDir_specified -lt 1 ]; then
  DATADIR_sed=`echo $TARGET_MNT_PNT$SRC_DATADIR | sed 's./.\\\/.g' `
  sed -i "/\[mysqld\]/a datadir=$DATADIR_sed" $CONFIG_FILE_LOC
fi
isPort_specified=`grep "^port=" $CONFIG_FILE_LOC | wc -l`
if [ $isPort_specified -lt 1 ]; then
  sed -i "/\[mysqld\]/a port=$PORT" $CONFIG_FILE_LOC
fi
isServerId_specified=`grep "^server_id=" $CONFIG_FILE_LOC | wc -l`
if [ $isServerId_specified -lt 1 ]; then
  sed -i "/\[mysqld\]/a server_id=5106$PORT" $CONFIG_FILE_LOC
fi
isLogbin_specified=`grep "^log_bin=" $CONFIG_FILE_LOC | wc -l`
if [ $isLogbin_specified -lt 1 ]; then
  sed -i "/\[mysqld\]/a log_bin=mysql-bin" $CONFIG_FILE_LOC
fi
isLogError_specified=$(( `grep "^log-error=" $CONFIG_FILE_LOC | wc -l ` + `grep "^log_error=" $CONFIG_FILE_LOC | wc -l ` ))
if [ $isLogError_specified -lt 1 ]; then
  sed -i "/\[mysqld\]/a log_error=$ERRORLOG_sed" $CONFIG_FILE_LOC
fi

#Prepare actual config file
if [ -e $MYSQL_CNF ]; then
  cp $MYSQL_CNF ${MYSQL_CNF}_bkp_${DATETIME}
fi
cp $CONFIG_FILE_LOC $MYSQL_CNF
chown $OSUSER:$OSGROUP $MYSQL_CNF

# Set default DB crediantials: root@loaclhost with null password
if [ "$DBUSER" = "" ]; then 
  DBUSER=root
fi

#Set env
export PATH=$BASEDIR/bin:$PATH

#Reset credentials
RESET_CREDS=/act/touch/$ACT_JOBNAME/RESET_CRED.sql
set +x
echo "CREATE USER IF NOT EXISTS '$DBUSER'@'localhost' IDENTIFIED BY '$DBPASSWORD';" > $RESET_CREDS
echo "ALTER USER '$DBUSER'@'localhost' IDENTIFIED BY '$DBPASSWORD';" >> $RESET_CREDS
set -x
echo "GRANT ALL PRIVILEGES ON *.* TO '$DBUSER'@'localhost' WITH GRANT OPTION;" >> $RESET_CREDS
echo "flush privileges;" >> $RESET_CREDS
#Reset to master to makesure no slave configured.
echo "STOP SLAVE;" >> $RESET_CREDS
echo "RESET MASTER;" >> $RESET_CREDS
chmod 755 $RESET_CREDS

#Prepare MYSQLD_SAFE cmd
export PATH=$PATH:/sbin:$BASEDIR/sbin
MYSQLD_SAFE=mysqld_safe
which mysqld_safe
if [ $? -ne 0 ]; then
  MYSQLD_SAFE=mysqld
  which mysqld
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to find mysqld(_safe) binary"
    exit 1
  fi
fi

## Start server
echo "INFOMSG: Starting MySQL with config details: "
cat $MYSQL_CNF | grep -v "^#" | sed '/^\t/d' | sed '/^$/d'
su -m $OSUSER -c "$MYSQLD_SAFE --defaults-file=$MYSQL_CNF --init-file=$RESET_CREDS &"
retval=$?

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to start mysql server !"
  exit 1
fi

#Check MySQL started successfully or not
sleep 60
isSrvStarted=`ps -ef | grep mysqld | grep -w "\-\-defaults-file=$MYSQL_CNF" | grep -vw grep |wc -l `
if [ $isSrvStarted -lt 1 ] || [ ! -e $ERRORLOG ]; then
  echo "ERRORMSG: Failed to bringup the MySQL server !"
  exit 1
fi

waitForMySQL $ERRORLOG 10
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to bringup the MySQL server !"
  cp $ERRORLOG /act/touch/errorLog_$ACT_JOBNAME.log
  exit 1
fi

# Prepare MySQL Command
set +x
prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKFILE" "$BASEDIR"
set -x

#Check instance connectivity
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to start the MySQL instance successfully."
  cleanUpMount
  exit 1
fi
echo "MySQL server started successfully"

# Recover if needed
if [ "$UNTIL_TIME" != "" ]; then
  /act/custom_apps/mysql/clone/ACT_MYSQL_recover.sh $SOCKFILE
  if [ $? -ne 0 ]; then 
    echo "ERRORMSG: Failed to recover MySQL server!"
    /act/custom_apps/mysql/clone/ACT_MYSQL_premount.sh
    #cleanUpMount
    exit 1
  fi
fi


#----------------------------------------------------------------
if [ "$APPREMOUNT" = "TRUE" ]; then
  update_TRG_DBLIST $REMOUNT_CONFIG_file
fi
#----------------------------------------------------------------


#------------------------------------------------------------
# Rename databases
#------------------------------------------------------------
#USERDBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' | grep -v -w sys | grep -v -w mysql | grep -vw information_schema | grep -vw performance_schema | grep -vw act_logbin | grep -vw act_tmp | xargs"`
USERDBLIST=`su -m $OSUSER -c "$MYSQL_CMD -e 'show databases' | grep -v -w sys | grep -v -w mysql | grep -vw information_schema | grep -vw performance_schema | xargs"`
echo "USERDBLIST=$USERDBLIST"
DELETE_DBLIST=

for dbName in $USERDBLIST
do
  flag=0
  newName=

  for dbPair in $(echo $TARGET_DB_NAME | sed 's/,/ /g')
  do
    odb=`echo $dbPair | cut -d: -f1`
    ndb=`echo $dbPair | cut -d: -f2`
    if [ "$odb" = "$dbName" ]; then
      newName=$ndb
      flag=1; break
    fi
  done

  if [ "$flag" = 0 ]; then
    DELETE_DBLIST="$DELETE_DBLIST,$dbName"
    continue
  fi

  echo "dbName: $dbName newName: $newName"
  renameDB $dbName $newName
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to rename database $dbName to $newName"
    /act/custom_apps/mysql/clone/ACT_MYSQL_premount.sh
    #cleanUpMount
    exit 1
  fi

done
#------------------------------------------------------------

#------------------------------------------------------------
# Drop databases
#------------------------------------------------------------
echo "Deleting databases: $DELETE_DBLIST"
for dbName in $(echo $DELETE_DBLIST | sed 's/,/ /g')
do
  su -m $OSUSER -c "$MYSQL_CMD -e \"drop database $dbName;\""
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: failed to drop database $dbName"
    /act/custom_apps/mysql/clone/ACT_MYSQL_premount.sh
    #cleanUpMount
    exit 1
  fi
done
#------------------------------------------------------------

if [ "$APPREMOUNT" = "TRUE" ]; then
  TARGET_DB_NAME=$TARGET_DB_NAME_orig
fi

mkdir -p /act/touch/$ACT_JOBNAME
chown -R $OSUSER:$OSGROUP /act/touch/$ACT_JOBNAME
remount_config

# Remove temp files
removeFile $CONFIG_FILE_LOC_orig
removeFile $CONFIG_FILE_LOC
removeFile $RESET_CREDS

exit 0
