#!/bin/sh

set -x
SRC_DATADIR=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
SRC_TMPDIR=$SRC_TMPDIR
MESSAGES_DIR=$MESSAGES_DIR
TARGET_MNT_PNT=$TARGET_MNT_PNT
BASEDIR=$BASEDIR
PORT=$PORT
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

#hidden file to handel Mount and Migrate
MYSQL_MIGRATE_CONF=/act/touch/.ACT_MYSQL_migrate_"$PORT".conf

#Check unmout database required or not
if [ -f $MYSQL_MIGRATE_CONF ]; then
  PVMOVE_STATUS=`cat $MYSQL_MIGRATE_CONF | awk -F "=" '{print $2}'`
  if [ "$PVMOVE_STATUS" = "YES" ]; then
    echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
    rm -f $MYSQL_MIGRATE_CONF
    exit 0
  else
    echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
    exit 1
  fi
fi

cnfList=`ps -ef | grep mysqld | grep defaults-file= | grep -vw mysqld_safe | grep -vw grep | awk -F" --defaults-file=" '{print $2}'| awk '{print $1}' | xargs`
for cnfFile in $cnfList
do
  tmpPort=`grep -w port $cnfFile | grep -v "^#" | tail -1 | cut -d= -f2`
  if [ $tmpPort -eq $PORT ]; then
    MYSQL_CNF=$cnfFile
    break
  fi
done

if [ -z "$MYSQL_CNF" ]; then
  echo "MySQL Server not running."
  exit 0
fi

SOCKETFILE=`grep socket= $MYSQL_CNF | grep -v "^#" | tail -1 | cut -d= -f2`

# Prepare MySQL Command
set +x
prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

## Shutdown Server
su -m $OSUSER -c "$MYSQL_ADMIN shutdown"
retval=$?
sleep 30

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to shutdown the mysql server with socket $SOCKETFILE"
  exit 1
fi

#isSrvRunning=`ps -ef | grep mysqld | grep -w $PORT | wc -l `
isSrvRunning=`netstat -nalp | grep mysqld | grep -w LISTEN | awk '{print $4}' | cut -d: -f4 | grep -w $PORT | wc -l`
if [ $isSrvRunning -eq 1 ]; then
  echo "ERRORMSG: Failed to shutdown MySQL server !"
  exit 1
fi

echo "MySQL server shutdown successfull"

exit 0
