#!/bin/sh

set -x
SRC_DATADIR=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
SRC_TMPDIR=$SRC_TMPDIR
MESSAGES_DIR=$MESSAGES_DIR
TARGET_MNT_PNT=$TARGET_MNT_PNT
BASEDIR=$BASEDIR
PORT=$PORT
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x
SNATCH_PORT=$SNATCH_PORT

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

## validate values
isVarNull SRC_DATADIR $SRC_DATADIR
isVarNull SRC_VERSION $SRC_VERSION
isVarNull SRC_TMPDIR $SRC_TMPDIR
isVarNull BASEDIR $BASEDIR
isVarNull PORT $PORT
isVarNull OSUSER $OSUSER

# Verify BASEDIR
if [ ! -f $BASEDIR/bin/mysql ]; then
   echo "ERRORMSG: Invalid BASEDIR ($BASEDIR/bin/mysql not found)"
   exit 1
fi

# Verify OSUSER
isOsUser $OSUSER
if [ $? -ne 0 ]; then
   echo "ERRORMSG: Invalid OSUSER ($OSUSER)"
   exit 1
fi

# OS user should not be root
if [ "$OSUSER" = "root" ]; then
   echo "ERRORMSG: OSUSER should not be root. Please use different OSUSER"
   exit 1
fi

#--------------------------------------------------
# Function: getPortPID()
#--------------------------------------------------
getPortPID() {
  local PORT=$1

  if [ `which netstat` ]; then
    portPID=`netstat -nalp | grep -w LISTEN  | grep -w $PORT | awk '{print $NF}' | head -1 | cut -d/ -f1`
    return 0
  fi

  if [ `which lsof` ]; then
    portPID=`lsof -P | grep -w LISTEN | grep -w $PORT | awk '{print $2}' | head -1`
    return 0
  fi

  return 1
}
#--------------------------------------------------


#--------------------------------------------------
# Check port is using or not
#--------------------------------------------------
if [ "$ACT_JOBTYPE" = "mount" ]; then
  #Check given port in use are not
  portPID=null
  getPortPID $PORT

  if [ "$portPID" = "null" ]; then
    echo "WARNINGMSG: Fail to find port $PORT is already in user or not."
  elif [ ! -z $portPID ]; then

    # Exit if SNATCH_PORT option is no
    if [ "$SNATCH_PORT" != yes ]; then
      echo "ERRORMSG: Given port number $PORT is already in use."
      echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
      exit 1
    fi

    # Exit if PORT is using by non-mysql process
    isPortPID_MySQL=`ps -ef | grep -w $portPID | grep -w mysqld | wc -l`
    if [ $isPortPID_MySQL -eq 0 ]; then
      echo "ERRORMSG: Given port number $PORT is using by non-MySQL process."
      echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
      exit 1
    fi

    #Kill the MySQL instance running on given port
    killPIDs=`ps -ef | grep -w $portPID |grep -vw grep| awk '{print $2" "$3}'`
    echo "INFOMSG: Killing pids $killPIDs"
    kill -9 $killPIDs

  fi

fi
#--------------------------------------------------

exit 0
