#!/bin/sh

set -x
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
BASEDIR=$BASEDIR
SOCKETFILE=$SOCKET_FILE
SRV_TYPE=$SRV_TYPE
USER_DBLIST="$USER_DBLIST"

DUMPSCHEDULE=$DUMPSCHEDULE
DUMPBKPLOC=$DUMPBKPLOC

# Validate env
if [ "x$OSUSER" = "x" ]; then
  echo "ERRORMSG: OSUSER is not provided"
  exit 1
fi

if [ "x$DBUSER" = "x" ]; then
  echo "WARNINGMSG: DBUSER is not provided. Considering default user 'root' "
fi

if [ "x$BASEDIR" = "x" ]; then
  echo "ERRORMSG: BASEDIR is not provided"
  exit 1
fi

if [ "x$SOCKETFILE" = "x" ]; then
  echo "ERRORMSG: SOCKETFILE is not provided"
  exit 1
fi

if [ "x$SRV_TYPE" = "x" ]; then
  echo "ERRORMSG: SRV_TYPE is not provided"
  exit 1
fi

if [ "x$USER_DBLIST" = "x" ]; then
  echo "ERRORMSG: USER_DBLIST is not provided"
  exit 1
fi

# Verify 
if [ ! -e $SOCKETFILE ]; then
  echo "ERRORMSG: SOCKETFILE ($SOCKETFILE) not exists."
  exit 1
fi

if [ ! -d $DUMPBKPLOC ]; then
  echo "ERRORMSG: DUMPBKPLOC ($DUMPBKPLOC) not exists or not a directory."
  exit 1
fi

## Verification
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: No such user $OSUSER"
  exit 1
fi

if [ ! -d $BASEDIR ]; then
  echo "ERRORMSG: Not a directory - $BASEDIR "
  exit 1
fi

if [ ! -d $BASEDIR/bin ]; then
  echo "ERRORMSG: bin directory not found in - $BASEDIR "
  exit 1
fi

if [ ! -f $BASEDIR/bin/mysql ]; then
  echo "ERRORMSG: mysql not found in - $BASEDIR/bin "
  exit 1
fi

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

# Prepare MySQL Command
set +x
prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

#------------------------------------------------------------
# Check DB connection with given credentials
#------------------------------------------------------------
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to connect to mysql with given credentials"
  exit 1
fi
#------------------------------------------------------------


#------------------------------------------------------------
# Check DB user privileges
#------------------------------------------------------------
# Required privileges: RELOAD, SELECT, LOCK TABLES, SUPER (or) REPLICATION CLIENT

checkPrivilege "SELECT"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: SELECT privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "RELOAD"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: RELOAD privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "LOCK TABLES"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: LOCK TABLES privilege is missing to database user $DBUSER "
  exit 1
fi

checkPrivilege "SUPER"
isS=$?
checkPrivilege "REPLICATION CLIENT"
isR=$?

if [ $isS -ne 0 ] && [ $isR -ne 0 ]; then
  echo "ERRORMSG: SUPER and REPLICATION CLIENT privilege is missing to database user $DBUSER. Required atleast one."
  exit 1
fi
#------------------------------------------------------------

DBSIZE_Total=0
# check if databases exists or not
for dbName in $(echo $USER_DBLIST | sed 's/,/ /g')
do
  su -m $OSUSER -c "$MYSQL_CMD -D $dbName -e \"select 'yes' from dual;\" "
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Database $dbName not found."
    exit 1
  fi

  dbSize=`su $OSUSER -c "$MYSQL_CMD -e \"SELECT SUM(data_length + index_length) FROM information_schema.TABLES where table_schema='$dbName';\" "`
  if [ "$dbSize" = "NULL" ]; then
    dbSize=0;
  fi
  echo "$dbName size: $dbSize"
  DBSIZE_Total=`expr $DBSIZE_Total "+" $dbSize`
done

DUMPSCHEDULE_DB=F
echo "DBDUMPSIZE=$DBSIZE_Total"
echo "DBDUMPTYPE=$DUMPSCHEDULE_DB"

exit 0
