#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

usage() {
  echo "Usage: "
  echo "/act/custom_apps/mysql/lvm_migrate/ACT_MYSQL_lvm_migrate.sh JOBID=<Mount_JOB_ID> OSUSER=<OSUSER> DBUSER=<DBUSER> DBPASSWORD=<DBPASSWORD> BASEDIR=<BASEDIR> PORT=<PORT> SOCKET_FILE=<SOCKET_FILE> CONFIG_FILE=<CONFIG_FILE> BEGIN_TIME=\"<BEGIN_TIME>\" UNTIL_TIME=\"<UNTIL_TIME>\" LOG_BKP_MNTPT=<LOG_BKP_MNTPT> DATAVOL_DISK_MAPPING=<Actifio_Mount>:<Production_LVM_device>"
  echo ""
  echo "Example:"
  echo "/act/custom_apps/mysql/lvm_migrate/ACT_MYSQL_lvm_migrate.sh JOBID=Job_0377678 OSUSER= DBUSER=root DBPASSWORD= BASEDIR=/home/mysql/app/mysql/product/mysqlHome_10.3.9 PORT=3307 SOCKET_FILE=/home/mysql/app/mysql/admin/mysql_3307/socket/mysql_3307.socket CONFIG_FILE=/home/mysql/app/mysql/admin/mysql_3307/mysql_3307.cnf BEGIN_TIME=\"2020-02-12 21:08:27\" UNTIL_TIME=\"2020-02-12 21:10:00\" LOG_BKP_MNTPT=/mmTST_archivelog DATAVOL_DISK_MAPPING=/mmTST/mysqld_3307:/dev/mapper/vg1-m7"
  echo ""
}
usage

# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

isVarNull OSUSER $OSUSER
isVarNull BASEDIR $BASEDIR
isVarNull PORT $PORT
isVarNull JOBID $JOBID
isVarNull SOCKET_FILE $SOCKET_FILE
isVarNull CONFIG_FILE $CONFIG_FILE
isVarNull DATAVOL_DISK_MAPPING $DATAVOL_DISK_MAPPING
if [ ! -z "$UNTIL_TIME" ]; then
  isVarNull BEGIN_TIME $BEGIN_TIME
  isVarNull LOG_BKP_MNTPT $LOG_BKP_MNTPT
fi

set -x
DATAVOL_DISK_MAPPING=$DATAVOL_DISK_MAPPING
ACT_JOBNAME=$JOBID

OSUSER=$OSUSER
BASEDIR=$BASEDIR
DBUSER=$DBUSER
DBPASSWORD=$DBPASSWORD
PORT=$PORT

#Restrict UnMount if pvmove is in progress
MYSQL_MIGRATE_CONF=/act/touch/.ACT_MYSQL_migrate_"$PORT".conf

#-------------------------------------------------------------------
#Prepare tmp location for temp files
#-------------------------------------------------------------------
tmp_loc=/act/tmpdata/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
  mkdir -p $tmp_loc
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $tmp_loc

export PATH=$BASEDIR/bin:$PATH

#MYSQL_CNF=`ps -ef | grep "\-\-port=$PORT" | awk -F" --defaults-file=" '{print $2}' | awk '{print $1}'`
#if [ -z "$MYSQL_CNF" ]; then
#  echo "ERRORMSG: Fail to get MYSQL_CNF for newly mounted instance"
#  exit 1
#fi
#cp $MYSQL_CNF $tmp_loc/MYSQL_CNF
#MYSQL_CNF=$tmp_loc/MYSQL_CNF

#-------------------------------------------------------------------


#-------------------------------------------------------------------
remove_file()
{
  local fname=$1
  if [ -f $fname ]; then
    rm -f $fname
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1
  local PROD_VG_NAME=$2
  echo "INFOMSG: Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     exit 1
  fi

  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     exit 1
  fi

  #Update LV_MEARGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MEARGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "INFOMSG: Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"

  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1
  isMntd=`grep -w $mntpnt /proc/mounts | wc -l`
  if [ $isMntd -eq 1 ]; then
    umount $mntpnt
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to unmount $mntpnt"
      exit 1
    fi
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
#Prepare temp files
#-------------------------------------------------------------------
LV_MEARGE_LIST=$tmp_loc/LV_MEARGE_LIST_$ACT_JOBNAME.txt
VG_MEARGE_LIST=$tmp_loc/VG_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST=$tmp_loc/PV_MEARGE_LIST_$ACT_JOBNAME.txt
MT_MEARGE_LIST=$tmp_loc/MT_MEARGE_LIST_$ACT_JOBNAME.txt
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST
remove_file $MT_MEARGE_LIST
#-------------------------------------------------------------------


#Prepare files conntents and also check lvm size
#---------------------------------------------------------------------------------------------------------
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    exit 1
  fi

  #Get LV paths
  echo "$ACT_LV_PATH $PROD_LV_PATH"  >> $LV_MEARGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MEARGE_LIST

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  for ACT_PV_DEV in $(echo $ACT_PV_DEV_LIST|sed 's/,/ /g'); do
    echo "$ACT_PV_DEV $PROD_PV_DEV_LIST" >> $PV_MEARGE_LIST
  done

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
    PROD_MNT=$ACT_MNT
  fi
  echo "$ACT_MNT $PROD_VOL $PROD_MNT" >> $MT_MEARGE_LIST
done
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Preparing mearge files completed"


#Stop application for unmounting actifio and source mount points
#-------------------------------------------------------------------
/act/custom_apps/mysql/restore/ACT_MYSQL_prerestore.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: failed at stoping application access to source/actifio mount points"
  exit 1
fi
#-------------------------------------------------------------------


#Update hidden migrate config file to handel unmount+delete
if [ ! -f $MYSQL_MIGRATE_CONF ]; then
  echo -e "MIGRATED=NO" > $MYSQL_MIGRATE_CONF
fi

echo "INFOMSG: Actifio CBT stoping..."
call_stop_tracking
echo "INFOMSG: Actifio CBT has been stopped successfully"


#Unmount all mounts
#---------------------------------------------------------------------------------------------------------
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  unmount_mnt $umnt2
done < $MT_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Unmounted all required mount points"


#LV rename to get diff lv names
#---------------------------------------------------------------------------------------------------------
cnt=100
remove_file $LV_MEARGE_LIST.tmp
while read line
do
  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "INFOMSG: Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MEARGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MEARGE_LIST
mv $LV_MEARGE_LIST.tmp $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Renaming actifio lvm names done (temparory)"


#VG mearge
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: VG mearge completed....."


#LV rename and update
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: LV rename completed....."


#Mount devices
#---------------------------------------------------------------------------------------------------------
while read line
do
  mntP=`echo $line| awk '{print $3}'`
  mntD=`echo $line | awk '{print $2}'`
  mount -o rw,nouuid $mntD $mntP
  if [ $? -ne 0 ]; then
    #Retry without nouuid option
    mount -o rw $mntD $mntP
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to mount $mntD to $mntP"
      exit 1
    fi
  fi
done < $MT_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Mounting mount points completed....."


#PV move
#---------------------------------------------------------------------------------------------------------
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_PV_NAME=`echo $line | awk '{print $2}'`
  PROD_PV_NAME=`echo $PROD_PV_NAME | sed 's/,/ /g'`
  echo "Moving Physical Volumes: $ACT_PV_NAME $PROD_PV_NAME"

  pvmove --atomic -A y -b -q $ACT_PV_NAME $PROD_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to initiate pvmove $ACT_PV_NAME $PROD_PV_NAME"
    exit 1
  fi
done < $PV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove initiated successfully .........."


#Call AppAware mount script
#---------------------------------------------------------------------------------------------------------
/act/custom_apps/mysql/restore/ACT_MYSQL_postrestore.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to mount databases with required mount points"
  exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Application started successfully"


echo "INFOMSG: pvmove starting ......................"
#---------------------------------------------------------------------------------------------------------
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove completed successfully ....."

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "Actifio pvs removed successfully .................."

#Update hidden migrate config file to handel unmount+delete
echo -e "MIGRATED=YES" > $MYSQL_MIGRATE_CONF

echo "INFOMSG: Starting Actifio CBT..."
call_start_tracking
echo "INFOMSG: Actifion CBT started successfully"

if [ -d "$tmp_loc" ]; then
  rm -rf $tmp_loc
fi

echo "************ Migration Operation Completed Successfully ***************"

exit 0

