#!/bin/sh

set -x
OSUSER=$OSUSER
DBUSER=$DBUSER
set +x
DBPASSWORD="$DBPASSWORD"
set -x
BASEDIR=$BASEDIR
SOCKETFILE=$SOCKET_FILE
SRV_TYPE=$SRV_TYPE
DATADIR=$SRC_DATADIR
PORT=$PORT
CONFIG_FILE=$CONFIG_FILE
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

LOGBIN_LOC=$LOGBIN_LOC
PIDFILE=$PIDFILE
LOGERROR=$LOGERROR
SERVRID=$SERVRID
SRC_TMPDIR=$TMPDIR


# Get Common functions
. /act/custom_apps/mysql/act_mysql_common_functions.sh

OSGROUP=`id -gn $OSUSER`

# exit, if config file not provided
if [ "$CONFIG_FILE" = "" ]; then
  echo "ERRORMSG: CONFIG_FILE not provided"
  exit 1
fi

MYSQL_CNF=$DATADIR/mysql_$PORT.cnf
if [ -e $MYSQL_CNF ]; then
  mv $MYSQL_CNF $MYSQL_CNF.bkp.$(date +'%Y%m%d_%H%M%S')
fi
cp $CONFIG_FILE $MYSQL_CNF
chown $OSUSER:$OSGROUP $MYSQL_CNF

#Set env for mysqld_safe
export PATH=$BASEDIR/bin:$PATH
export PATH=$PATH:/sbin:$BASEDIR/sbin
MYSQLD_SAFE=mysqld_safe
which mysqld_safe
if [ $? -ne 0 ]; then
  MYSQLD_SAFE=mysqld
  which mysqld
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to find mysqld(_safe) binary"
    exit 1
  fi
fi

getErrorLog "$MYSQL_CNF" LOGERROR
if [ $? -ne 0 ]; then
   echo "WARNINGMSG: Fail to find log_error file"
else
   mv $LOGERROR ${LOGERROR}_bkp$(date +%Y%m%d%H%M%S)
fi

## Start server
su -m $OSUSER -c "$MYSQLD_SAFE --defaults-file=$MYSQL_CNF & "
retval=$?

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to start mysql server !"
  exit 1
fi

#Check MySQL started successfully or not
sleep 10
isSrvStarted=`ps -ef | grep mysqld | grep -w "\-\-defaults-file=$MYSQL_CNF" | grep -vw grep |wc -l `
if [ $isSrvStarted -lt 1 ] || [ ! -e $ERRORLOG ]; then
  echo "ERRORMSG: Failed to bringup the MySQL server !"
  exit 1
fi

if [ -z "$LOGERROR" ]; then
   sleep 30
else
   waitForMySQL "$LOGERROR" 10
   if [ $? -ne 0 ]; then
      echo "ERRORMSG: Failed to bringup the MySQL server !"
      cp $LOGERROR /act/touch/errorLog_$ACT_JOBNAME.log
      exit 1
   fi
fi

# Prepare MySQL Command
set +x
prepare_MYSQLCMD "$DBUSER" "$DBPASSWORD" "$SOCKETFILE" "$BASEDIR"
set -x

# Check server is running or not
checkMySQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Failed to bringup the MySQL server !"
  exit 1
fi

echo "MySQL server started successfully"

# Recovery if needed
if [ ! -z "$UNTIL_TIME" ]; then
  /act/custom_apps/mysql/clone/ACT_MYSQL_recover.sh $SOCKETFILE
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to recover MySQL server!"
    exit 1
  fi
fi
