#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================

set -x

function usage()
{
  echo "Please set environment var ACT_PHASE and re-run the script"
  echo "Supported phases [prepare|freeze|unfreeze|abort|cleanup|logpurge|logbackup|predump|dbdump|predumprestore|postdumprestore|prerestoretest|prerestore|postrestore|premounttest|predumprestoretest|premount|postmount|postunmount"
  return 0;
}

function cleanup_func()
{
    echo "**** Cleanup phase started ****"
    /act/custom_apps/postgresql/act_postgresql_abort.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* CLEANUP ERROR: Failed postgresql snapshot cleanup phase ************"
      return $retval
    fi
    echo "**** Cleanup phase ended ****"
    return 0;
}

function prepare_func()
{
    echo "**** Prepare phase started ****";
    /act/custom_apps/postgresql/act_postgresql_prepare.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PREPARE ERROR: Failed postgresql snapshot prepare phase ************"
      return $retval
    fi
    echo "**** Prepare phase ended ****";
    return 0;
}

function freeze_func()
{
    echo "**** Freeze phase started ****"
    /act/custom_apps/postgresql/act_postgresql_pre.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* FREEZE ERROR: Failed postgresql create snapshot phase ************"
      return $retval
    fi
    echo "**** Freeze phase ended ****"
    return 0;
}

function unfreeze_func()
{
    echo "**** UN_Freeze phase started ****"
    /act/custom_apps/postgresql/act_postgresql_post.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* UN-FREEZE ERROR: Failed postgresql unfreeze for create snapshot ************"
      return $retval
    fi
    echo "**** UN_Freeze phase ended ****"
    return 0;
}

function abort_func()
{
    echo "**** abort phase started ****"
    /act/custom_apps/postgresql/act_postgresql_abort.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* ABORT ERROR: Failed postgresql abort for create snapshot ************"
      return $retval
    fi
    echo "**** abort phase ended ****"
    return 0;
}

function premount_func()
{
    echo "**** Premount phase started ****"
    /act/custom_apps/postgresql/clone/ACT_POSTGRESQL_premount.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PREMOUNT ERROR: Failed postgresql premount before create Appaware mount ************"
      return $retval
    fi
    echo "**** Premount phase ended ****"
    return 0;
}

function predumprestoretest_func()
{
    echo "**** PreDumpRestoreTest phase started ****"
    echo "**** PreDumpRestoreTest phase ended *****"
}

function predump_func()
{
    echo "**** Predump phase started ****"
    /act/custom_apps/postgresql/dump/act_postgresql_predump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PREDUMP ERROR: Some environment variables are missing or invalied for postgresql DUMP backup"
      exit 1
    fi
    echo "**** Predump phase ended ****"
    return 0
}

function dbdump_func()
{
    echo "**** dbdump phase started ****"
    /act/custom_apps/postgresql/dump/act_postgresql_dump.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* DBDUMP ERROR: Failed postgresql dbdump phase ************"
      return $retval
    fi
    echo "**** dbdump phase ended ****"
    return 0;
}

function postmount_func()
{
    echo "**** Postmount phase started ****"
    /act/custom_apps/postgresql/clone/ACT_POSTGRESQL_postmount.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* POSTMOUNT ERROR: Failed postgresql postmount phase *************"
      return $retval
    fi
    echo "**** Postmount phase ended ****"
    return 0;
}

function premounttest_func()
{
    echo "**** Premount test phase started ****"
    /act/custom_apps/postgresql/clone/ACT_POSTGRESQL_premount_test.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "******* PREMOUNT-TEST ERROR: Failed postgresql postmount phase *************"
      return $retval
    fi
    echo "**** Premount test phase ended ****"
    return 0;
}

function postunmount_func()
{
    echo "**** PostUnMount phase started ****"
    ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$JOBID
    if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
      rm -f $ACT_PREMOUNT_FAILURE_FILE
    fi
    echo "**** PostUnMount phase ended ****"
}

function prerestoretest_func()
{
    echo "**** Pre-restore test phase started ****"
    /act/custom_apps/postgresql/restore/ACT_POSTGRESQL_prerestore_test.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRE-RESTORE ERROR: Failed postgresql predumprestore phase *****"
      return $retval
    fi
    echo "**** Pre-restore test phase ended ****";
    return 0;
}

function prerestore_func()
{
    echo "**** Prerestore phase started ****"
    /act/custom_apps/postgresql/restore/ACT_POSTGRESQL_prerestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PRERESTORE ERROR: Failed postgresql prerestore phase *****"
      return $retval
    fi
    echo "**** Prerestore phase ended ****"
    return 0;
}

function postrestore_func()
{
    echo "**** Postresotre phase started ****"
    /act/custom_apps/postgresql/restore/ACT_POSTGRESQL_postrestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** POSTRESTORE ERROR: Failed postgresql postrestore phase *****"
      return $retval
    fi
    echo "**** Postresotre phase ended ****"
    return 0;
}

function predumprestore_func()
{
    echo "**** PreDumpRestore phase started ****"
    /act/custom_apps/postgresql/dump/ACT_POSTGRESQL_predumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** PREDUMPRESTORE ERROR: Failed postgresql predumprestore phase *****"
      return $retval
    fi
    echo "**** PreDumprestore phase ended ****"
    return 0;
}

function postdumprestore_func()
{
    echo "**** PostDumpResotre phase started ****"
    /act/custom_apps/postgresql/dump/ACT_POSTGRESQL_dumprestore.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** POSTDUMPRESTORE ERROR: Failed postgresql dump restore *****"
      return $retval
    fi
    echo "**** PostDumpResotre phase ended ****"
    return 0;
}

function logbackup_func()
{
    echo "**** log backup phase started ****"
    /act/custom_apps/postgresql/act_postgresql_logbackup.sh
    retval=$?
    if [ $retval -ne 0 ]; then
        echo "******* LOGBACKUP ERROR: Failed postgresql log backup ************"
        return $retval
    fi
    echo "**** log backup phase ended ****"
    return 0;
}

function logpurge_func()
{
    echo "**** log purge phase started ****"
    /act/custom_apps/postgresql/act_postgresql_logpurge.sh
    retval=$?
    if [ $retval -ne 0 ]; then
      echo "***** LOGPURGE ERROR: failed to purge postgresql archive log files *****"
      return $retval
    fi
    echo "**** log purge phase ended ****"
    return 0;
}


if [ -n $ACT_PHASE ] 
then
  PARAM=$ACT_PHASE

  case $PARAM in 
    cleanup)
         echo "Executing cleanup phase of the script"
         cleanup_func
         retval=$?
         ;;
    prepare)
         echo "Executing prepare phase of the script"
         prepare_func
         retval=$?
         ;;
    freeze)
         echo "Executing freeze phase of the script"
         freeze_func
         retval=$?
         ;;
    unfreeze)
         echo "Executing unfreeze phase of the script"
         unfreeze_func
         retval=$?
         ;;
    predump)
         echo "Executing predump phase of the script"
         predump_func
         retval=$?
         ;;
    dbdump)
         echo "Executing dbdump phase of the script"
         dbdump_func
         retval=$?
         ;;
    prerestoretest)
         echo "Executing prerestore test phase of the script"
         prerestoretest_func
         retval=$?
         ;;
    prerestore)
         echo "Executing prerestore phase of the script"
         prerestore_func
         retval=$?
         ;;
    postrestore)
         echo "Executing postrestore phase of the script"
         postrestore_func
         retval=$?
         ;;
    predumprestore)
         echo "Executing prerestore phase of the script"
         predumprestore_func
         retval=$?
         ;;
    postdumprestore)
         echo "Executing postrestore phase of the script"
         postdumprestore_func
         retval=$?
         ;;
    premounttest)
         echo "Executing premount test phase of the script"
         premounttest_func
         retval=$?
         ;;
    predumprestoretest)
	 echo "Executing predumprestoretest phase of the script"
	 predumprestoretest_func
	 retval=$?
	 ;;
    premount)
	 if [ "$ACT_JOBTYPE" = "unmount" ]; then
           echo "Executing premount phase of the script"
           premount_func
           retval=$?
	 fi
         ;;
    postmount)
         echo "Executing postmount phase of the script"
         postmount_func
         retval=$?
         ;;
    postunmount)
	 echo "Executing postunmount phase of the script"
	 postunmount_func
	 retval=$?
	 ;;
    logbackup)
         echo "Executing logbackup phase of the script"
         logbackup_func
         retval=$?
         ;;
    logpurge)
	 echo "Executing logpurge phase of the script"
	 logpurge_func
	 retval=$?
         ;;
    abort)
         echo "Executing abort phase of the script"
         abort_func
         retval=$?
         ;;           
    *)
         usage
  esac 
else 
  usage
fi

exit $retval;
