#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#-------------------------------------------------------------------------------------------------
# function: isVarNull()
#-------------------------------------------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    exit 1
  fi
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: removeFile()
#-------------------------------------------------------------------------------------------------
removeFile() {
  local fileName=$1
  if [ -e "$fileName" ]; then
    rm -f $fileName
  fi
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: chmod755_parents()
#-------------------------------------------------------------------------------------------------
chmod755_parents() {
  local dirName=$1
  while [ -d "$dirName" ] && [ ! -z "$dirName" ] && [ "$dirName" != "/" ]; do
    chmod 755 $dirName
    dirName=`dirname $dirName`
  done
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: prepareConn()
#-------------------------------------------------------------------------------------------------
prepareConn() {
  export PGPORT=$1
  export PGUSER=$2
  export PGHOST=$5

  #Set password
  if [ -z "$3" ]; then
    export PGPASSWORD="''"
  else
    export PGPASSWORD="$3"
  fi

  #Prepre psql command
  export PSQL="psql -AXt"

  #Set database name
  if [ -z "$4" ]; then
    export PGDATABASE=postgres
  else
    export PGDATABASE=$4
  fi
}
#-------------------------------------------------------------------------------------------------


#-------------------------------------------------------------------------------------------------
# function: remount_config()
#-------------------------------------------------------------------------------------------------
remount_config() {
  config_file=/act/touch/$ACT_JOBNAME/REMOUNT_CONFIG
  echo "OLD_MOUNT=$TARGET_MNT_PNT" > $config_file
  echo "REMOUNT_DB_LIST=$TARGET_DB_NAME" >> $config_file
}
#-------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------
# functon update_TRG_DBLIST
#-------------------------------------------------------------------------------------------------
update_TRG_DBLIST() {
  local RemountConfigFile=$1
  local fmName=`grep REMOUNT_DB_LIST $RemountConfigFile | cut -d"=" -f2`
  local flName=$TARGET_DB_NAME

  TARGET_DB_NAME=
  TARGET_DB_NAME_orig=

  for ocf in $(echo $fmName | sed 's/,/ /g')
  do
    fName=`echo $ocf | cut -d: -f1`
    mName=`echo $ocf | cut -d: -f2`
    lName=

    for ncf in $(echo $flName |sed 's/,/ /g')
    do
      p_db=`echo $ncf | cut -d: -f1`
      n_db=`echo $ncf | cut -d: -f2`
      if [ "$p_db" = "$fName" ]; then
        lName=$n_db
        break
      fi
    done

    if [ -z $TARGET_DB_NAME ]; then
      TARGET_DB_NAME=$mName:$lName
      TARGET_DB_NAME_orig=$fName:$lName
    else
      TARGET_DB_NAME="$TARGET_DB_NAME,$mName:$lName"
      TARGET_DB_NAME_orig="$TARGET_DB_NAME_orig,$fName:$lName"
    fi
  done

  echo "TARGET_DB_NAME=$TARGET_DB_NAME"
  echo "TARGET_DB_NAME_orig=$TARGET_DB_NAME_orig"
}
#-------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------
# function: getLatestAlertLog()
#-------------------------------------------------------------------------------------------------
getLatestAlertLog() {
  local pgDataDir=$1
  local pgVersionNumber=$2

  local pgCNF=$pgDataDir/postgresql.conf

  is_logCollectorOn=`grep "^logging_collector" $pgCNF | tail -1 | grep -w on | wc -l`
  if [ $is_logCollectorOn -eq 0 ]; then
    # Log collector is off, consider pg_ctl start log
    LATEST_ALERTLOG=$pgDataDir/pg_log.log
    return 0
  fi

  local logDir=`grep "^log_directory" $pgCNF | tail -1 | cut -d= -f2 | cut -d'#' -f1 | xargs`

  if [ -z $logDir ]; then
    if [ $pgVersionNumber -lt 10 ]; then
      logDir=pg_log
    else
      logDir=log
    fi
  fi

  if [ "${logDir:0:1}" != "/"  ]; then
    logDir="$pgDataDir/$logDir"
  fi

  if [ ! -z "$logDir" ] && [ -d "$logDir" ]; then
    LATEST_ALERTLOG=`ls -ltr $logDir/* | tail -1 | awk '{print $NF}'`
  else
    return 1
  fi

  return 0
}
#-------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------
# function: errorHandle()
#-------------------------------------------------------------------------------------------------
errorHandle() {
  local pgData=$1
  local errorMSG=$2
  local errorLog=$3
  local pgVersionNumber=$4

  getLatestAlertLog $pgData $pgVersionNumber
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: $errorMSG"
    exit 1
  fi

  if [ $pgVersionNumber -lt 13 ]; then
    cp $LATEST_ALERTLOG $errorLog
    cat $errorLog
    echo "ERRORMSG: $errorMSG"
    exit 1
  fi

  is_IGNORABLE_FATAL_ERROR=`grep -w FATAL $LATEST_ALERTLOG | tail -1 | grep -i "recovery ended before configured recovery target was reached" | wc -l`
  if [ $is_IGNORABLE_FATAL_ERROR -ne 1 ]; then
    cp $LATEST_ALERTLOG $errorLog
    cat $errorLog
    echo "ERRORMSG: $errorMSG"
    exit 1
  fi

  # Restart the PostgreSQL as the FATAL error is ignorable
  echo "INFOMSG: Restarting PostgeSQL after ignorable fatal error."
  removeFile $pgData/recovery.signal
  su $OSUSER -c "pg_ctl start -D $pgData -w -l $pgData/pg_log.log"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to restart PostgreSQL"
    cp $pgData/pg_log.log $errorLog
    cat $errorLog
    exit 1
  fi

  return 0
}
#-------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------
# function: exit_if_no_function_privilege()
#-------------------------------------------------------------------------------------------------
exit_if_no_function_privilege() {
  local privFun=$1
  local dbUser=$2

  has_Privilege=`su $OSUSER -c "$PSQL -c 'select has_function_privilege('\''$dbUser'\'','\''$privFun'\'','\''execute'\'');'" | grep -v "Warning: your password will expire in"`
  if [ "$has_Privilege" != "t" ]; then
    echo "ERRORMSG: DBUSER ($dbUser) do not have execute privileges on $privFun"
    exit 1
  fi

  return 0
}
#-------------------------------------------------------------------------------------------------
