#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORTNO
IS_SLAVE=$ISSLAVE
VERSION=$VERSION
LOGBKPLOC=$LOGBKPLOC
PG_DATA=$PG_DATA
SOCKET_DIR=$SOCKET_DIR
ACT_JOBNAME=$JOBID

BEGINPIT=$BEGINPIT     # Previous DB beginpit

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#Preflight check for log backup
if [ "$LOGBKPLOC" = "" ] || [ ! -d "$LOGBKPLOC" ]; then
  echo "ERRORMSG: LOGBKPLOC is not provided or it is not a directory"
  exit 1
fi

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

#Check db credentials
su $OSUSER -c "$PSQL -c \"select 1;\""
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to connect to postgres on port $PORT with given credentials"
  exit 1
fi

#Check archive_mode is on or not
isArcOn=`su $OSUSER -c '$PSQL -c "show archive_mode;" ' | grep -v "Warning: your password will expire in"`
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to get archive_mode"
  exit 1
fi

if [ "$isArcOn" = "off" ]; then
  echo "ERRORMSG: Instance archive_mode is not on"
  exit 1
fi

if [ "$IS_SLAVE" = "TRUE" ] && [ "$isArcOn" != "always" ] ; then
   echo "ERRORMSG: archive_mode is not set to 'always' (standby)"
   exit 1
fi

#Check archive_command is default or not
ARCHIVE_CMD=`su $OSUSER -c '$PSQL -c "show archive_command;" ' | grep -v "Warning: your password will expire in"`
is_ARCHIVE_CMD_default=`echo $ARCHIVE_CMD | grep "cp %p " | wc -l`
if [ $is_ARCHIVE_CMD_default -eq 0 ]; then
  echo "ERRORMSG: Not using default archive_command. cannot proceed for logbackup"
  exit 1
fi

# Check if provided database user have execute privileges on function pg_switch_wal
psqlVersion=`echo $VERSION | cut -d'.' -f1`
if [ $psqlVersion -ge 10 ]; then
  exit_if_no_function_privilege "pg_switch_wal()" $DBUSER
else
  exit_if_no_function_privilege "pg_switch_xlog()" $DBUSER
fi

# Check if database OS_user have privileges to run archive_command or not
ARCHIVE_prefix=`echo $ARCHIVE_CMD | awk -F'cp ' '{print $2}' | awk '{print $2}' | awk -F'%f' '{print $1}'`
if [ ! -d "$ARCHIVE_prefix" ]; then
   # Archive command may specified in a format like /pglog/archive_3306_%f.arc
   ARCHIVE_CMD_directory=`dirname $ARCHIVE_prefix`
   ARCHIVE_prefix_basename=`basename $ARCHIVE_prefix`
else
   ARCHIVE_CMD_directory=$ARCHIVE_prefix
   ARCHIVE_prefix_basename=
fi
if [ -z "$ARCHIVE_CMD_directory" ] || [ ! -d "$ARCHIVE_CMD_directory" ] || [ "$ARCHIVE_CMD_directory" = "/" ]; then
   echo "ERRORMSG: Fail to find the archivelogs directory"
   exit 1
fi
su -m $OSUSER -c "touch $ARCHIVE_CMD_directory/ACT_____archive_command_test_file"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Database OSUSER ($OSUSER) can't execute archive_command"
  exit 1
else
  removeFile $ARCHIVE_CMD_directory/ACT_____archive_command_test_file
fi

# If Standby, WAL Switch needs to happen on primary
if [ "$IS_SLAVE" = "TRUE" ]; then

   # Find primary db hostip and port
   primHostIP=`su $OSUSER -c '$PSQL -c "SELECT sender_host FROM pg_stat_wal_receiver;" ' | grep -v "Warning: your password will expire in"`
   if [ -z "$primHostIP" ]; then
      echo "WARNINGMSG: Fail to find primany db hostip"
   fi
   primPort=`su $OSUSER -c '$PSQL -c "SELECT sender_port FROM pg_stat_wal_receiver;" ' | grep -v "Warning: your password will expire in"`
   if [ -z "$primPort" ]; then
      echo "WARNINGMSG: Fail to find primany db port"
   fi

   export PGHOST=$primHostIP
   export PGPORT=$primPort

fi

#Switch log
psqlVersion=`echo $VERSION | cut -d'.' -f1`
if [ $psqlVersion -ge 10 ]; then
  ARCH_SWITCH_QUERY="select pg_switch_wal();"
else
  ARCH_SWITCH_QUERY="select pg_switch_xlog();"
fi
su $OSUSER -c "$PSQL -c \"$ARCH_SWITCH_QUERY\" "
if [ $? -ne 0 ]; then
   if [ "$IS_SLAVE" = "TRUE" ]; then
      echo "WARNINGMSG: Fail to switch xlog/wal (standby)"
   else
      echo "ERRORMSG: Fail to switch xlog/wal"
      exit 1
   fi
fi
sleep 2 #Wait to copy xlog/wal to archive location

#Find WALs location
if [ ! -z "$ARCHIVE_prefix_basename" ]; then
   FIND_NAME_OPTION=" -name $ARCHIVE_prefix_basename* "
else
   FIND_NAME_OPTION=
fi
WALS=`find $ARCHIVE_CMD_directory $FIND_NAME_OPTION -maxdepth 1 -newermt "$BEGINPIT" -type f`

#Copy WALs one by one
for walFile in $WALS
do
  srcLogFileName=`basename $walFile`
  trgLogFile=$LOGBKPLOC/$srcLogFileName

  #Skip if file already copied
  if [ -e "$trgLogFile" ]; then
    srcSize=`ls -l $walFile | awk '{print $5}'`
    trgSize=`ls -l $trgLogFile | awk '{print $5}'`
    if [ $srcSize -eq $trgSize ]; then
      continue
    fi
  fi
  cp -p $walFile $LOGBKPLOC/
done

exit 0
