#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORTNO
IS_SLAVE=$ISSLAVE
VERSION=$VERSION
SOCKET_DIR=$SOCKET_DIR
LOGBKPLOC=$LOGBKPLOC
PG_DATA=$PG_DATA
DELLOG=$DELLOG
DELETE_LOGFILES_HOURS=$DELETE_LOGFILES_HOURS
ACT_JOBNAME=$JOBID

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

#Check db credentials
su $OSUSER -c "$PSQL -c \"select 1;\""
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to connect to postgres on port $PORT with given credentials"
  exit 1
fi

#Find WAL location
ARCHIVE_CMD=`su $OSUSER -c 'psql -AXt -c "show archive_command;" ' | grep -v "Warning: your password will expire in"`
WALS_prefix=`echo $ARCHIVE_CMD | awk -F'cp ' '{print $2}' | awk '{print $2}' | awk -F'%f' '{print $1}'`
if [ ! -d "$WALS_prefix" ]; then
   # Archive command may specified in a format like /pglog/archive_3306_%f.arc
   WALS_LOC=`dirname $WALS_prefix`
   WALS_prefix_base=`basename $WALS_prefix`
else
   WALS_LOC=$WALS_prefix
   WALS_prefix_base=
fi

if [ -z "$WALS_LOC" ] || [ ! -d "$WALS_LOC" ] || [ "$WALS_LOC" = "/" ]; then
   echo "ERRORMSG: Fail to find the archivelogs directory"
   exit 1
fi

#Delete files based on DELETE_LOGFILES_HOURS
if [ -z "$DELETE_LOGFILES_HOURS" ]; then
  DELETE_LOGFILES_HOURS=4
fi
LOG_RETENTION_MINS=`expr $DELETE_LOGFILES_HOURS "*" 60`
if [ ! -z "$WALS_prefix_base" ]; then
   FIND_NAME_OPTION=" -name $WALS_prefix_base* "
else
   FIND_NAME_OPTION=
fi
find $WALS_LOC $FIND_NAME_OPTION -type f -mmin +$LOG_RETENTION_MINS -print -exec rm -f {} \;
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to purge WAL files"
  exit 1
fi

exit 0
