#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORTNO
IS_SLAVE=$ISSLAVE
VERSION=$VERSION
SOCKET_DIR=$SOCKET_DIR
ACT_JOBNAME=$JOBID

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#Temporary file to identify if db already freezed or not
FREEZE_INFO=/act/touch/ACT_FREEZE_$PORT
if [ ! -e "$FREEZE_INFO" ]; then
  echo "ERRORMSG: Database not yet freezed by Actifio. ($FREEZE_INFO not found)"
  exit 1
fi

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

#Check backup running or not
IS_BKP_RUNNING=`su $OSUSER -c "$PSQL -c \"select pg_is_in_backup();\" " | grep -v "Warning: your password will expire in"`
if [ "$IS_BKP_RUNNING" = "f" ] && [ "$IS_SLAVE" = "FALSE" ]; then
  echo "ERRORMSG: no backup is in progress."
  exit 1
fi

#Un-Freeze
if [ "$IS_SLAVE" = "FALSE" ]; then
  su $OSUSER -c "$PSQL -c \"select pg_stop_backup();\" "
  retval=$?
else
  psqlVersion=`echo $VERSION | cut -d'.' -f1`
  if [ $psqlVersion -ge 10 ]; then
    su $OSUSER -c "$PSQL -c \"select pg_wal_replay_resume();\" "
    retval=$?
  else
    su $OSUSER -c "$PSQL -c \"select pg_xlog_replay_resume();\" "
    retval=$?
  fi
fi

#Check un-freeze is success or not
if [ $retval -ne 0 ]; then
  if [ "$IS_SLAVE" = "FALSE" ]; then
    echo "ERRORMSG: Failed to stop backup"
  else
    echo "ERRORMSG: Fail to resume the slave replay"
  fi
  exit 1
fi

#Remove temporary file created during freeze
rm -f $FREEZE_INFO

exit 0
