#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORTNO
IS_SLAVE=$ISSLAVE
VERSION=$VERSION
SOCKET_DIR=$SOCKET_DIR
ACT_JOBNAME=$JOBID

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#Temporary file to identify if db already freezed or not
FREEZE_INFO=/act/touch/ACT_FREEZE_$PORT
if [ -e "$FREEZE_INFO" ]; then
  echo "INFOMSG: DB already freezed by actifio"
  cat $FREEZE_INFO
  exit 0
fi

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

# Check if outside Actifio backup running
IS_BKP_RUNNING=`su $OSUSER -c "$PSQL -c \"select pg_is_in_backup();\" " | grep -v "Warning: your password will expire in"`
if [ "$IS_BKP_RUNNING" = "t" ]; then
  echo "ERRORMSG: Non-Actifio backup in progress"
  exit 1
fi

#Prepare freeze query
if [ "$IS_SLAVE" = "FALSE" ]; then
  FREEZE_QUERY="select pg_start_backup('$ACT_JOBNAME');"
else
  psqlVersion=`echo $VERSION | cut -d'.' -f1`
  if [ $psqlVersion -ge 10 ]; then
    FREEZE_QUERY="select pg_wal_replay_pause();"
  else
    FREEZE_QUERY="select pg_xlog_replay_pause();"
  fi
fi

#Freeze
su $OSUSER -c "$PSQL -c \"$FREEZE_QUERY\" "
retval=$?

#Check freeze success or not
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to run query: $FREEZE_QUERY"
  exit 1
fi

echo "$FREEZE_QUERY" > $FREEZE_INFO

exit 0
