#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORTNO
IS_SLAVE=$ISSLAVE
VERSION=$VERSION
SOCKET_DIR=$SOCKET_DIR

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#Check variable
isVarNull OSUSER $OSUSER
isVarNull PG_HOME $PG_HOME
isVarNull PORT $PORT
isVarNull IS_SLAVE $IS_SLAVE
isVarNull VERSION $VERSION

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

#Check db credentials
su $OSUSER -c "$PSQL -c \"select 1;\""
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to connect to postgres on port $PORT with given credentials"
  exit 1
fi

# Fail if dbuser don't have required privileges
psqlVersion=`echo $VERSION | cut -d'.' -f1`
if [ "$IS_SLAVE" = "FALSE" ]; then
  exit_if_no_function_privilege "pg_start_backup(text,boolean,boolean)" $DBUSER
  exit_if_no_function_privilege "pg_stop_backup()" $DBUSER
else
  if [ $psqlVersion -ge 10 ]; then
    exit_if_no_function_privilege "pg_wal_replay_pause()" $DBUSER
    exit_if_no_function_privilege "pg_wal_replay_resume()" $DBUSER
  else
    exit_if_no_function_privilege "pg_xlog_replay_pause()" $DBUSER
    exit_if_no_function_privilege "pg_xlog_replay_resume()" $DBUSER
  fi
fi

# for logbackup
if [ $psqlVersion -ge 10 ]; then
  exit_if_no_function_privilege "pg_switch_wal()" $DBUSER
else
  exit_if_no_function_privilege "pg_switch_xlog()" $DBUSER
fi

#Temporary file to identify if db already freezed or not
FREEZE_INFO=/act/touch/ACT_FREEZE_$PORT
