#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
PORT=$PORT
SRC_PGDATA=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
TARGET_MNT_PNT=$TARGET_MNT_PNT
DBUSER=$DBUSER
SRC_DBUSER=$SRC_DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x
TARGET_DB_NAME=$TARGET_DB_NAME

BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

ACT_MOUNT_REPLACERESTORE=$ACT_MOUNT_REPLACERESTORE	#child restore
APPREMOUNT=$APPREMOUNT					#Re-mount

REMOUNT_configfile=$CONFIG_FILE_LOC

OSGROUP=`id -gn $OSUSER`

if [ -z "$DBUSER" ] || [ "$DBUSER" = " " ]; then
  DBUSER=postgres
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh


#----------------------------------------------------------------
if [ "$APPREMOUNT" = "TRUE" ]; then
  update_TRG_DBLIST $REMOUNT_configfile
fi
#----------------------------------------------------------------

#Convert TARGET_DB_NAME to small letters as PostgreSQL databases names will be always small letters
TARGET_DB_NAME=`echo $TARGET_DB_NAME |tr '[:upper:]' '[:lower:]'`

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Update target mount-point permission
if [ "$ACT_MOUNT_REPLACERESTORE" != "TRUE" ]; then
  chown -R $OSUSER:$OSGROUP $TARGET_MNT_PNT
  chmod755_parents $TARGET_MNT_PNT
else
  TARGET_MNT_PNT=
fi

#Prepare target variables
TRG_PGDATA=$TARGET_MNT_PNT$SRC_PGDATA
PG_CONFIG_FILE=$TARGET_MNT_PNT$SRC_PGDATA/postgresql.conf
PG_HBA_FILE=$TARGET_MNT_PNT$SRC_PGDATA/pg_hba.conf

# Remove postmaster.pid if exists.
removeFile $TRG_PGDATA/postmaster.pid

# Remove if any recovery.conf is there.
removeFile $TRG_PGDATA/recovery.conf
removeFile $TRG_PGDATA/recovery.done
removeFile $TRG_PGDATA/recovery.signal # PostgreSQL version 12 and later

# Remove if tablespace_map.* files exists
tmpSpMp_Files=`ls $TRG_PGDATA/tablespace_map.*`
for tmpSpMpFile in $tmpSpMp_Files; do
  removeFile $tmpSpMpFile
done

#Check if config file exist or not
if [ ! -e "$PG_CONFIG_FILE" ]; then
  echo "ERRORMSG: postgresql.conf file is missing in target PGDATA ($TARGET_MNT_PNT$SRC_PGDATA)"
  exit 1
fi

# Get old archivelog format before updating
OLD_ARCHIVE_FORMAT=`grep "^archive_command" $PG_CONFIG_FILE | tail -1 | awk -F'cp ' '{print $2}' | awk '{print $2}' | sed 's./. .g' | sed "s.'..g" | awk '{print $NF}'`

# Edit postgresql.conf file
removeFile $PG_CONFIG_FILE.bkp
cp $PG_CONFIG_FILE $PG_CONFIG_FILE.bkp
ARCH_DIR=$TRG_PGDATA/archive_log
mkdir -p $ARCH_DIR
sed '/^$\|^#\|^\t/d' $PG_CONFIG_FILE >> $PG_CONFIG_FILE.bkp
sed -i '/^listen_addresses/d' $PG_CONFIG_FILE.bkp
sed -i '/^port/d' $PG_CONFIG_FILE.bkp
sed -i '/^hot_standby/d' $PG_CONFIG_FILE.bkp
sed -i '/^archive_mode/d' $PG_CONFIG_FILE.bkp
sed -i '/^archive_command/d' $PG_CONFIG_FILE.bkp
sed -i '/^data_directory/d' $PG_CONFIG_FILE.bkp
sed -i '/^hba_file/d' $PG_CONFIG_FILE.bkp
sed -i '/^external_pid_file/d' $PG_CONFIG_FILE.bkp
sed -i '/^logging_collector/d' $PG_CONFIG_FILE.bkp
sed -i '/^log_directory/d' $PG_CONFIG_FILE.bkp
sed -i '/^unix_socket/d' $PG_CONFIG_FILE.bkp
sed -i '/^shared_preload_libraries/d' $PG_CONFIG_FILE.bkp
sed -i '/^recovery_target/d' $PG_CONFIG_FILE.bkp
sed -i '/^cluster_name/d' $PG_CONFIG_FILE.bkp
sed -i '/^synchronous_standby_names/d' $PG_CONFIG_FILE.bkp
echo "data_directory = '$TRG_PGDATA'" >> $PG_CONFIG_FILE.bkp
echo "listen_addresses = 'localhost'" >> $PG_CONFIG_FILE.bkp
echo "port = $PORT" >> $PG_CONFIG_FILE.bkp
echo "archive_mode = on" >> $PG_CONFIG_FILE.bkp
echo "archive_command = 'test ! -f $ARCH_DIR/%f && cp %p $ARCH_DIR/%f'" >> $PG_CONFIG_FILE.bkp
echo "logging_collector = on" >> $PG_CONFIG_FILE.bkp
echo "log_directory = pg_log" >> $PG_CONFIG_FILE.bkp
echo "unix_socket_directories = '/tmp,$TRG_PGDATA'" >> $PG_CONFIG_FILE.bkp
mkdir $TRG_PGDATA/pg_log
mv $PG_CONFIG_FILE.bkp $PG_CONFIG_FILE

# Edit pg_hba.conf file
sed -i '/^host/d' $PG_HBA_FILE
sed -i '/^local/d' $PG_HBA_FILE
echo "local all all trust" >> $PG_HBA_FILE
#echo "host all all $TARGET_MASTER_NODE/32 trust" >> $PG_HBA_FILE

# relink tablespace files
#cd $TRG_PGDATA/pg_tblspc
tbList=`ls $TRG_PGDATA/pg_tblspc/`
for tbLink in $tbList; do
  oldPath=`ls -l $TRG_PGDATA/pg_tblspc/$tbLink | awk '{print $NF}'`
  newPath=$TARGET_MNT_PNT$oldPath
  echo "OldPath: $oldPath; newPath: $newPath"
  unlink $TRG_PGDATA/pg_tblspc/$tbLink
  ln -s $newPath $TRG_PGDATA/pg_tblspc/$tbLink
done

# Prepare recovery.conf if required
SRC_VERSION_NUM=`echo $SRC_VERSION | cut -d. -f1`
if [ ! -z "$UNTIL_TIME" ]; then
  if [ $SRC_VERSION_NUM -lt 12 ]; then
    RECOVERY_CONF_FILE=$TRG_PGDATA/recovery.conf
  else
    RECOVERY_CONF_FILE=$PG_CONFIG_FILE
    touch $TRG_PGDATA/recovery.signal
  fi

  echo "restore_command = 'cp $LOG_BKP_MNTPT/$OLD_ARCHIVE_FORMAT "%p"'" >> $RECOVERY_CONF_FILE
  echo "recovery_target_time='$UNTIL_TIME'" >> $RECOVERY_CONF_FILE
  chown -R $OSUSER:$OSGROUP $LOG_BKP_MNTPT
fi

# Change permission to PG_DATA directory files
chown -R $OSUSER:$OSGROUP $TRG_PGDATA
chmod -R 0700 $TRG_PGDATA

#Prepare errorLog in case failure
errorLog=/var/act/log/errorLog_$ACT_JOBNAME.err

# Prepare PGCTL_OPTIONS
PGCTL_OPTIONS="-p $PORT"

# Start the database
echo "Starting PostgreSQL server with config details:"
cat $PG_CONFIG_FILE | grep -v "^#" | sed '/^\t/d' | sed '/^$/d'
su $OSUSER -c "pg_ctl start -D $TRG_PGDATA -w -t 600 -o \" $PGCTL_OPTIONS \" -l $TRG_PGDATA/pg_log.log"
if [ $? -ne 0 ]; then
  errorHandle $TRG_PGDATA "fail to start PostgreSQL" $errorLog $SRC_VERSION_NUM
fi

# Check database status
su $OSUSER -c "pg_ctl status -D $TRG_PGDATA"
if [ $? -ne 0 ]; then
  errorHandle $TRG_PGDATA "fail to get status after starting the database" $errorLog $SRC_VERSION_NUM
fi

#Prepare db connection with default user
echo "INFOMSG: Preparing psql connection with user=$SRC_DBUSER, dbname=postgres, PORT=$PORT"
set +x
prepareConn "$PORT" "$SRC_DBUSER" "" "postgres" "/tmp"
set -x

#Check instance connectivity
su $OSUSER -c "$PSQL -c \"select 1;\" "
if [ $? -ne 0 ]; then
  errorHandle $TRG_PGDATA "Fail to connect to $TRG_PGDATA after starting the instance" $errorLog $SRC_VERSION_NUM
fi
echo "INFOMSG: Target PostgreSQL started successfully"

#Check recovery status and wait until recovery complete
if [ ! -z "$UNTIL_TIME" ]; then
  cnt=0
  waitTimes=30
  SRC_VERSION_NUM=`echo $SRC_VERSION | cut -d. -f1`

  if [ $SRC_VERSION_NUM -lt 10 ]; then
    pg9RecMsg=$(su $OSUSER -c '$PSQL -c "SELECT '\''1'\''"' 2>&1)
    while [ "$pg9RecMsg" =~ "the database system is starting up" ] && [ $cnt -lt $waitTimes ]
    do
      sleep 30
      getLatestAlertLog $TRG_PGDATA $SRC_VERSION_NUM
      isBeforRecPoint=`grep "recovery stopping before commit of transaction " $LATEST_ALERTLOG |wc -l`
      if [ $isBeforRecPoint -gt 0 ]; then
        echo "WARNINGMSG: Recovery stopping before PITR. Opening database in read/write mode..."
        su $OSUSER -c "$PSQL -c \"select pg_xlog_replay_resume();\""
        if [ $? -ne 0 ]; then
          echo "ERRORMSG: Fail to resume database after recovery."
          exit 1
        fi
        sleep 30
      fi
      pg9RecMsg=$(su $OSUSER -c '$PSQL -c "SELECT '\''1'\''"' 2>&1)
      cnt=$(( cnt + 1 ))
    done
  else
    isInRecovery=`su $OSUSER -c "echo START__RS;$PSQL -c \"SELECT pg_is_in_recovery()\"; echo END__RS"`
    isInRecovery=`echo $isInRecovery | awk -F"START__RS" '{print $2}' | awk -F"END__RS" '{print $1}' | sed 's/ //g'`
    while [ "$isInRecovery" = "t" ] && [ $cnt -lt $waitTimes ]
    do
      sleep 30
      getLatestAlertLog $TRG_PGDATA $SRC_VERSION_NUM
      isBeforRecPoint=`grep "recovery stopping before commit of transaction " $LATEST_ALERTLOG |wc -l`
      if [ $isBeforRecPoint -gt 0 ]; then
        echo "WARNINGMSG: Recovery stopping before PITR. Opening database in read/write mode..."
        su $OSUSER -c "$PSQL -c \"select pg_wal_replay_resume();\""
        if [ $? -ne 0 ]; then
          echo "ERRORMSG: Fail to resume database after recovery."
          exit 1
        fi
        sleep 30
      fi
      isInRecovery=`su $OSUSER -c "echo START__RS;$PSQL -c \"SELECT pg_is_in_recovery()\"; echo END__RS"`
      isInRecovery=`echo $isInRecovery | awk -F"START__RS" '{print $2}' | awk -F"END__RS" '{print $1}' | sed 's/ //g'`
      cnt=$(( cnt + 1 ))
    done
  fi

  if [ -e $TRG_PGDATA/recovery.done ]; then
    echo "INFOMSG: Recovery success"
  elif [ $SRC_VERSION_NUM -ge 12 ] && [ ! -e $TRG_PGDATA/recovery.signal ]; then
    echo "INFOMSG: Recovery success"
  else
    errorHandle $TRG_PGDATA "Fail to recover database" $errorLog $SRC_VERSION_NUM
  fi
fi


#--------------------------------------------------------------------------------------------------------------
#Create new database user with given details
set +x
su $OSUSER -c "$PSQL -c \"CREATE USER $DBUSER WITH LOGIN SUPERUSER PASSWORD '$DBPASSWORD';\" "
su $OSUSER -c "$PSQL -c \"ALTER USER $DBUSER WITH LOGIN SUPERUSER PASSWORD '$DBPASSWORD';\" "
if [ $? -ne 0 ];then
  echo "ERRORMSG: fail to create/update user $DBUSER"
  exit 1
fi
set -x

#Check connection with new user
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWORD" "postgres" "/tmp"
set -x

su $OSUSER -c "$PSQL -c \"select 1;\" "
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to connect to $TRG_PGDATA with user $DBUSER"
  exit 1
fi
echo "INFOMSG: Created/Update database user with given details"


#--------------------------------------------------------------------------------------------------------------
#Rename Databases if required
AVIL_DB_LIST=`su $OSUSER -c "$PSQL -c \"select datname from pg_database;\" " | grep -vw "postgres\|template1\|template0" | grep -v "Warning: your password will expire in" | xargs`
DELETE_DBLIST=
for dbName in $AVIL_DB_LIST; do
  flag=0
  newName=

  for dbPair in $(echo $TARGET_DB_NAME | sed 's/,/ /g')
  do
    odb=`echo $dbPair | cut -d: -f1`
    ndb=`echo $dbPair | cut -d: -f2`
    if [ -z "$ndb" ]; then
      ndb=$odb
    fi
    if [ "$odb" = "$dbName" ]; then
      newName=$ndb
      flag=1; break
    fi
  done

  if [ "$flag" = 0 ]; then
    DELETE_DBLIST="$DELETE_DBLIST $dbName"
    continue
  fi

  if [ "$dbName" = "$newName" ]; then
    continue	#No need to rename
  fi

  echo "INFOMSG: Renaming database $dbName to $newName ....."
  su $OSUSER -c "$PSQL -c \"ALTER DATABASE \\\"$dbName\\\" RENAME TO \\\"$newName\\\" \" "
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to rename database $dbName to $newName"
    exit 1
  fi
done
#--------------------------------------------------------------------------------------------------------------

#Drop extra databases
for dbName in $DELETE_DBLIST; do
  su $OSUSER -c "$PSQL -c \"DROP DATABASE \\\"$dbName\\\" \""
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to drop database $dbName"
    exit 1
  fi
done

if [ "$APPREMOUNT" = "TRUE" ]; then
  TARGET_DB_NAME=$TARGET_DB_NAME_orig
fi

#Prepare REMOUNT_CONFIG for ReMount
mkdir -p /act/touch/$ACT_JOBNAME
remount_config

exit 0
