#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
PORT=$PORT
SRC_PGDATA=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
TARGET_MNT_PNT=$TARGET_MNT_PNT
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x
TARGET_DB_NAME=$TARGET_DB_NAME

BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

OSGROUP=`id -gn $OSUSER`

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

#hidden file to handel Mount and Migrate
POSTGRESQL_MIGRATE_CONF=/act/touch/.ACT_POSTGRESQL_migrate_"$PORT".conf

#Check unmount database required or not
if [ -f $POSTGRESQL_MIGRATE_CONF ]; then
  PVMOVE_STATUS=`cat $POSTGRESQL_MIGRATE_CONF | awk -F "=" '{print $2}'`
  if [ "$PVMOVE_STATUS" = "YES" ]; then
    echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
    exit 0
  else
    echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
    exit 1
  fi
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Check if db running or not
TRG_PGDATA=
postgresPID_list=`ps -ef | grep " postgres: "| grep -vw grep | awk '{print $3}'| sort | uniq | xargs`
for p_pid in $postgresPID_list; do
  this_PGDATA_onCmdLine=`ps -p $p_pid -o cmd | grep " -D " | wc -l`
  if [ $this_PGDATA_onCmdLine -eq 0 ]; then
    this_PGDATA=`xargs -0 -L1 -a /proc/$p_pid/environ | grep -w "PGDATA=" | cut -d= -f2`
  else
    this_PGDATA=`ps -p $p_pid -o cmd | awk -F'-D ' 'NR==2{print $2}' | awk '{print $1}'`
  fi
  this_PORT=`grep -w port $this_PGDATA/postgresql.conf |sed 's/ //g'|sed 's/\t//g'| grep ^port=| cut -d= -f2 | cut -d'#' -f1 | sed 's/ //g'`
  if [ "$this_PORT" = "$PORT" ]; then
    TRG_PGDATA=$this_PGDATA
    break
  fi
done

if [ -z "$TRG_PGDATA" ]; then
  echo "INFOMSG: Target instance is not running"
  exit 0
fi

# Stop the instance
su $OSUSER -c "pg_ctl stop -D $TRG_PGDATA -w"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to shutdown target instance ($TRG_PGDATA)"
  exit 1
fi
