#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
PORT=$PORT
SRC_PGDATA=$SRC_DATADIR
SRC_VERSION=$SRC_VERSION
TARGET_MNT_PNT=$TARGET_MNT_PNT
DBUSER=$DBUSER
set +x
DBPASSWORD=$DBPASSWORD
set -x
TARGET_DB_NAME=$TARGET_DB_NAME
SNATCH_PORT=$SNATCH_PORT

BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

ACT_JOBNAME=$JOBID
ACT_JOBTYPE=$ACT_JOBTYPE

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#prechecks
isVarNull OSUSER $OSUSER
isVarNull PG_HOME $PG_HOME
isVarNull PORT $PORT
isVarNull SRC_PGDATA $SRC_PGDATA
isVarNull SRC_VERSION $SRC_VERSION
isVarNull TARGET_DB_NAME $TARGET_DB_NAME

#Verify PG_HOME
if [ ! -d "$PG_HOME/bin" ]; then
  echo "ERRORMSG: $PG_HOME/bin not exists"
  echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
  exit 1
elif [ ! -e "$PG_HOME/bin/pg_ctl" ]; then
  echo "ERRORMSG: $PG_HOME/bin/pg_ctl is missing. Please check the PG_HOME is correct or not."
  echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
  exit 1
fi

# Verify OS user
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: $OSUSER user does not exists"
  echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
  exit 1
fi

getPortPID() {
  local PORT=$1

  if [ `which netstat` ]; then
    portPID=`netstat -nalp | grep -w LISTEN  | grep -w $PORT | awk '{print $NF}' | head -1 | cut -d/ -f1`
    return 0
  fi

  if [ `which lsof` ]; then
    portPID=`lsof -P | grep -w LISTEN | grep -w $PORT | awk '{print $2}' | head -1`
    return 0
  fi

  return 1
}

#Prechecks for mount
if [ "$ACT_JOBTYPE" = "mount" ]; then

  #Check given port in use are not
  portPID=null
  getPortPID $PORT
  if [ "$portPID" = "null" ]; then
    echo "WARNINGMSG: Fail to find port $PORT is already in user or not."
  elif [ ! -z $portPID ]; then

    if [ "$SNATCH_PORT" != yes ]; then
      echo "ERRORMSG: Given port number $PORT is already in use."
      echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
      exit 1
    fi

    isPortPID_Postgres=`ps -ef | grep -w $portPID | grep -w postgres | wc -l`
    if [ $isPortPID_Postgres -eq 0 ]; then
      echo "ERRORMSG: Given port number $PORT is using by non-postgres process."
      echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
      exit 1
    fi

    echo "INFOMSG: Killing postgres process $portPID"
    kill -9 $portPID

  fi

  #Check source version and target version same or not
  TRG_VERSION=`$PG_HOME/bin/pg_ctl -V | awk '{print $NF}'`
  SRC_VERSION_num=`echo $SRC_VERSION | sed 's/\.//g'`
  TRG_VERSION_num=`echo $TRG_VERSION | sed 's/\.//g'`
  if [ $SRC_VERSION_num -ne $TRG_VERSION_num ]; then
    echo "ERRORMSG: Source db version ($SRC_VERSION) not equals to target version ($TRG_VERSION)"
    echo "INFOMSG: No unmount required." >  $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

fi

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

exit 0
