#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORT
IS_SLAVE=$ISSLAVE
VERSION=$SRC_VERSION
USER_DBLIST="$DB_LIST"
SOCKET_DIR=$SOCKET_DIR

ACT_JOBNAME=$JOBID

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

#Prechecks
isVarNull OSUSER $OSUSER
isVarNull PG_HOME $PG_HOME
isVarNull PORT $PORT
isVarNull USER_DBLIST $USER_DBLIST

#isVarNull DBUSER $DBUSER
#isVarNull IS_SLAVE $IS_SLAVE
#isVarNull VERSION $VERSION

if [ -z "$DBUSER" ]; then
  DBUSER=postgres
fi

OSGROUP=`id -gn $OSUSER`

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

#Prepare db connection
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "$DBNAME" "$SOCKET_DIR"
set -x

#Check Instance running or not
if [ ! -z "$SOCKET_DIR" ]; then
  sockFile=$SOCKET_DIR/.s.PGSQL.$PORT
else
  sockFile=/tmp/.s.PGSQL.$PORT
fi

if [ ! -e $sockFile ]; then
  echo "WARNINGMSG: PostgreSQL instance may not running"
  #exit 1
fi

#Verify db connection
su $OSUSER -c "$PSQL -c \"select 1;\" "
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Not able to connect to PostgreSQL. Please check username and password"
  exit 1
fi

#Drop and create empty database
for dbName in $(echo $USER_DBLIST | sed 's/,/ /g' )
do
  #Check db exists are not. Drop db if exists
  isDbExists=`su $OSUSER -c "$PSQL -c \"select datname from pg_database where datname='$dbName';\" "`
  if [ ! -z "$isDbExists" ]; then
    echo "Droping database $dbName ....."
    su $OSUSER -c "$PSQL -c \"DROP DATABASE \\\"$dbName\\\" \" "
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to drop database $dbName"
      exit 1
    fi
  fi

  echo "Creating empty database $dbName ....."
  su $OSUSER -c "createdb -T template0 \"$dbName\" "
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to create database $dbName"
    exit 1
  fi
done

exit 0
