#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

usage() {
  echo "Usage: "
  echo "/act/custom_apps/postgresql/restore/ACT_POSTGRESQL_lvm_migrate_newTarget.sh DATAVOL_DISK_MAPPING=<Actifio_Mount>:<Production_LVM_device> BASEDIR=<BASEDIR> OSUSER=<OSUSER> PORT=<PORT> [ DBUSER=<DBUSER> ] [ DBPASSWORD=<DBPASSWORD> ] [ JOBID=<JOBID> ] [ SOCKET_DIR=<SOCKET_DIR> ]"
  echo ""
  echo "Example:"
  echo "/act/custom_apps/postgresql/restore/ACT_POSTGRESQL_lvm_migrate_newTarget.sh DATAVOL_DISK_MAPPING=/chtst/pgData10.3:/dev/mapper/data-pg103,/chtst/data:/dev/mapper/data-data BASEDIR=/home/postgres/postgresql_home_10.3 OSUSER=postgres PORT=6010 DBUSER=postgres JOBID=Job_0956617 SOCKET_DIR=/tmp "
  echo ""
}
#usage

if [ $# -lt 4 ]; then
  echo "ERRORMSG: Not provided enough arguments"
  usage
  exit 1
fi

# Get Common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

isVarNull OSUSER $OSUSER
isVarNull BASEDIR $BASEDIR
isVarNull PORT $PORT
isVarNull DATAVOL_DISK_MAPPING $DATAVOL_DISK_MAPPING

set -x
DATAVOL_DISK_MAPPING=$DATAVOL_DISK_MAPPING
ACT_JOBNAME=$JOBID

OSUSER=$OSUSER
BASEDIR=$BASEDIR
DBUSER=$DBUSER
DBPASSWORD=$DBPASSWORD
PORT=$PORT

#Restrict UnMount if pvmove is in progress
POSTGRESQL_MIGRATE_CONF=/act/touch/.ACT_POSTGRESQL_migrate_"$PORT".conf


#-------------------------------------------------------------------
#Prepare tmp location for temp files
#-------------------------------------------------------------------
tmp_loc=/act/tmpdata/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
  mkdir -p $tmp_loc
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $tmp_loc

export PATH=$BASEDIR/bin:$PATH

#Prepare db connection with default user
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWORD" "postgres"
set -x

mm_PG_DATA=`su $OSUSER -c "$PSQL -c \"show data_directory\" "`
if [ -z "$mm_PG_DATA" ]; then
  echo "ERRORMSG: Fail to get PG_DATA for newly mounted instance"
  exit 1
fi
mm_PG_DATA_mntp=`df -P $mm_PG_DATA | tail -1 | awk '{print $NF}'`

PG_TBS_MNTS=$tmp_loc/PG_TBS_MNTS_$ACT_JOBNAME.txt
if [ -f "$PG_TBS_MNTS" ]; then
  rm -f $PG_TBS_MNTS
fi

PG_TBS_list=`ls -l $mm_PG_DATA/pg_tblspc | grep -v total | awk '{print $NF}' `
for tbPth in $PG_TBS_list
do
  tbMnt=`df -P $tbPth | tail -1 | awk '{print $NF}' `
  echo "$tbPth $tbMnt" >> $PG_TBS_MNTS
done
#-------------------------------------------------------------------


#-------------------------------------------------------------------
remove_file()
{
  local fname=$1
  if [ -f $fname ]; then
    rm -f $fname
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: startPostgreSQL()
#-------------------------------------------------------------------
startPostgreSQL() {
  #Update postgresql.conf file
  local old_mnt=$mm_PG_DATA_mntp
  local new_mnt=`grep "$old_mnt " $MT_MEARGE_LIST | awk '{print $3}'`
  local old_mnt_sed=`echo $old_mnt | sed 's./.\\\/.g'`
  local new_mnt_sed=`echo $new_mnt | sed 's./.\\\/.g'`
  mm_PG_DATA=`echo $mm_PG_DATA | sed "s/$old_mnt_sed/$new_mnt_sed/g"`
  sed -i "s/$old_mnt_sed/$new_mnt_sed/g" $mm_PG_DATA/postgresql.conf

  #sed -i '/^data_directory/d' $mm_PG_DATA/postgresql.conf
  #echo "data_directory = '$mm_PG_DATA'" >> $mm_PG_DATA/postgresql.conf

  # Remove pid file if exists
  remove_file $mm_PG_DATA/postmaster.pid

  # relink tablespace files
  #cd $mm_PG_DATA/pg_tblspc
  tbList=`ls $mm_PG_DATA/pg_tblspc/`
  for tbLink in $tbList; do
    oldPath=`ls -l $mm_PG_DATA/pg_tblspc/$tbLink | awk '{print $NF}'`
    oldTbMnt=`grep "^$oldPath " $PG_TBS_MNTS | awk '{print $2}'`
    newTbMnt=`grep "$oldTbMnt " $MT_MEARGE_LIST | awk '{print $3}'`
    oldTbMnt_sed=`echo $oldTbMnt | sed 's./.\\\/.g'`
    newTbMnt_sed=`echo $newTbMnt | sed 's./.\\\/.g'`
    newPath=`echo $oldPath | sed "s/$oldTbMnt_sed/$newTbMnt_sed/g"`
    echo "OldPath: $oldPath; newPath: $newPath"
    unlink $mm_PG_DATA/pg_tblspc/$tbLink
    ln -s $newPath $mm_PG_DATA/pg_tblspc/$tbLink
  done

  #Start PostgreSQL
  su $OSUSER -c "pg_ctl start -D $mm_PG_DATA"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to start PostgreSQL"
    exit 1
  fi
  sleep 30

  #Check if PostgreSQL running or not
  foundInst=0
  runningPG_list=`ps -ef | grep " postgres: "| grep -vw grep | awk '{print $3}'| sort | uniq | xargs`
  for PG_PID in $runningPG_list
  do
    tmpPGDATA=`xargs -0 -L1 -a /proc/$PG_PID/environ | grep -w "PGDATA=" | cut -d= -f2`
    if [ "$tmpPGDATA" = "$mm_PG_DATA" ]; then
      foundInst=1
      break
    fi
  done

  if [ $foundInst -eq 0 ]; then
    echo "ERRORMSG: Fail to start PostgreSQL"
    exit 1
  fi
  echo "INFOMSG: PostgreSQL started successfully"

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1
  local PROD_VG_NAME=$2
  echo "INFOMSG: Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     exit 1
  fi

  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     exit 1
  fi

  #Update LV_MEARGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MEARGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "INFOMSG: Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"

  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1
  isMntd=`grep -w $mntpnt /proc/mounts | uniq | wc -l`
  if [ $isMntd -eq 1 ]; then
    for i in {1..5}
    do
      umount $mntpnt
      grep " $mntpnt " /proc/mounts
      if [ $? -eq 0 ]; then
        if [ $i -eq 5 ]; then
          echo "ERRORMSG: Fail to unmount $mntpnt after retries"
          exit 1
        fi
        echo "ERRORMSG: Fail to unmount $mntpnt. Retrying"
        sleep 5
      else
        echo "INFOMSG: Successfully unmounted $mntpnt"
        break
      fi
    done
  fi

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
#Prepare temp files
#-------------------------------------------------------------------
LV_MEARGE_LIST=$tmp_loc/LV_MEARGE_LIST_$ACT_JOBNAME.txt
VG_MEARGE_LIST=$tmp_loc/VG_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST=$tmp_loc/PV_MEARGE_LIST_$ACT_JOBNAME.txt
MT_MEARGE_LIST=$tmp_loc/MT_MEARGE_LIST_$ACT_JOBNAME.txt
NST_MNTPNTS=$tmp_loc/NST_MNTPNTS_$ACT_JOBNAME.txt
NST_MNTPNTS_toMount=$tmp_loc/NST_MNTPNTS_toMount_$ACT_JOBNAME.txt
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST
remove_file $MT_MEARGE_LIST
remove_file $NST_MNTPNTS
remove_file $NST_MNTPNTS_toMount.tmp
#-------------------------------------------------------------------


#Prepare files conntents and also check lvm size
#---------------------------------------------------------------------------------------------------------
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    exit 1
  fi

  #Get LV paths
  echo "$ACT_LV_PATH $PROD_LV_PATH"  >> $LV_MEARGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MEARGE_LIST

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  for ACT_PV_DEV in $(echo $ACT_PV_DEV_LIST|sed 's/,/ /g'); do
    echo "$ACT_PV_DEV $PROD_PV_DEV_LIST" >> $PV_MEARGE_LIST
  done

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
    PROD_MNT=$ACT_MNT
  fi
  echo "$ACT_MNT $PROD_VOL $PROD_MNT" >> $MT_MEARGE_LIST
done
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Preparing mearge files completed"

#Find additional mounts
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt1/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt1/"`
  echo "$nstMnt" >> $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt2/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt2/"`
  echo "$nstMnt" >> $NST_MNTPNTS
done < $MT_MEARGE_LIST

#Sort, uniq
cat $NST_MNTPNTS | sort -r | uniq > $NST_MNTPNTS.tmp
mv $NST_MNTPNTS.tmp $NST_MNTPNTS
sed -i '/^$/d' $NST_MNTPNTS

#Remove migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  grep -vw "$umnt1\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  grep -vw "$umnt2\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS
done < $MT_MEARGE_LIST

echo "INFOMSG: Additional mountpoints to unmount"
cat $NST_MNTPNTS

#Update NST_MNTPNTS_toMount with mount command
cat $NST_MNTPNTS | sort > $NST_MNTPNTS_toMount
while read line
do
  if [ -z $line ]; then
    continue
  fi
  mntDev=`grep " $line " /proc/mounts | awk '{print $1}' | uniq`
  echo "mount $mntDev $line" >> $NST_MNTPNTS_toMount.tmp
done < $NST_MNTPNTS
mv $NST_MNTPNTS_toMount.tmp $NST_MNTPNTS_toMount
cat $NST_MNTPNTS_toMount


#Stop application for unmounting actifio and source mount points
#-------------------------------------------------------------------
/act/custom_apps/postgresql/clone/ACT_POSTGRESQL_premount.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: failed at stoping application access to source/actifio mount points"
  exit 1
fi
#-------------------------------------------------------------------


#Update hidden migrate config file to handel unmount+delete
if [ ! -f $POSTGRESQL_MIGRATE_CONF ]; then
  echo -e "MIGRATED=NO" > $POSTGRESQL_MIGRATE_CONF
fi

echo "INFOMSG: Actifio CBT stoping..."
call_stop_tracking
echo "INFOMSG: Actifio CBT has been stopped successfully"


#Unmount all mounts
#---------------------------------------------------------------------------------------------------------
#Unmount additional mounts first
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  unmntpnt=`echo $line | awk '{print $1}'`
  if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
    unmount_mnt $unmntpnt
  else
    echo "ERRORMSG: Found nested mount points ($unmntpnt). cannot proceed..."
    exit 1
  fi
done < $NST_MNTPNTS

#Unmount migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  unmount_mnt $umnt2
done < $MT_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Unmounted all required mount points"


#LV rename to get diff lv names
#---------------------------------------------------------------------------------------------------------
cnt=100
remove_file $LV_MEARGE_LIST.tmp
while read line
do
  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "INFOMSG: Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MEARGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MEARGE_LIST
mv $LV_MEARGE_LIST.tmp $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Renaming actifio lvm names done (temparory)"


#VG mearge
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: VG mearge completed....."


#LV rename and update
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: LV rename completed....."


#Mount devices
#---------------------------------------------------------------------------------------------------------
while read line
do
  mntP=`echo $line| awk '{print $3}'`
  mntD=`echo $line | awk '{print $2}'`
  mount -o rw,nouuid $mntD $mntP
  if [ $? -ne 0 ]; then
    #Retry without nouuid option
    mount -o rw $mntD $mntP
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to mount $mntD to $mntP"
      exit 1
    fi
  fi
done < $MT_MEARGE_LIST

#Mount additional mounts
if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
  echo "INFOMSG: Mounting the Additional Mountpoints...."
  while read line
  do
    addMntPnt=`echo $line | awk '{print $3}'`
    if [ ! -d "$addMntPnt" ]; then
      mkdir -p $addMntPnt
    fi
    $line
  done < $NST_MNTPNTS_toMount
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Mounting mount points completed....."


#PV move
#---------------------------------------------------------------------------------------------------------
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_PV_NAME=`echo $line | awk '{print $2}'`
  PROD_PV_NAME=`echo $PROD_PV_NAME | sed 's/,/ /g'`
  echo "Moving Physical Volumes: $ACT_PV_NAME $PROD_PV_NAME"

  pvmove --atomic -A y -b -q $ACT_PV_NAME $PROD_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to initiate pvmove $ACT_PV_NAME $PROD_PV_NAME"
    exit 1
  fi
done < $PV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove initiated successfully .........."


#Call AppAware mount script
#---------------------------------------------------------------------------------------------------------
startPostgreSQL
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to mount databases with required mount points"
  exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Application started successfully"


echo "INFOMSG: pvmove starting ......................"
#---------------------------------------------------------------------------------------------------------
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove completed successfully ....."

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "Actifio pvs removed successfully .................."

#Update hidden migrate config file to handel unmount+delete
echo -e "MIGRATED=YES" > $POSTGRESQL_MIGRATE_CONF

echo "INFOMSG: Starting Actifio CBT..."
call_start_tracking
echo "INFOMSG: Actifion CBT started successfully"

if [ -d "$tmp_loc" ]; then
  rm -rf $tmp_loc
fi

echo "************ Migration Operation Completed Successfully ***************"

exit 0

