#!/bin/sh

set -x
OSUSER=$OSUSER
PG_HOME=$BASEDIR
DBUSER=$DBUSER
set +x
DBPASSWD="$DBPASSWORD"
set -x
PORT=$PORT
PG_DATA=$SRC_DATADIR
SOCKET_DIR=$SOCKET_DIR
IS_SLAVE=$ISSLAVE
VERSION=$VERSION

BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

ACT_JOBNAME=$JOBID

if [ -z "$SOCKET_DIR" ]; then
  SOCKET_DIR=`grep "PORTNO=\"$PORT" /act/custom_apps/POSTGRESQL.xml | awk -F" SOCKET_DIR=" '{print $2}' | awk '{print $1}' | sed 's/"//g'`
fi

OSGROUP=`id -gn $OSUSER`

#Get common functions
. /act/custom_apps/postgresql/act_postgresql_common_functions.sh

# Set environment for PostgreSQL
export PATH=$PG_HOME/bin:$PATH
export LD_LIBRARY_PATH=$PG_HOME/lib:$LD_LIBRARY_PATH

if [ -z $VERSION ]; then
  VERSION=`pg_ctl -V | awk '{print $NF}'`
fi

# Remove previous recovery settings
removeFile $PG_DATA/recovery.conf
removeFile $PG_DATA/recovery.done
removeFile $PG_DATA/recovery.signal

# Get archivelog format to specify in recovery file
ARCHIVE_FORMAT=`grep "^archive_command" $PG_DATA/postgresql.conf | tail -1 | awk -F'cp ' '{print $2}' | awk '{print $2}' | sed 's./. .g' | sed "s.'..g" | awk '{print $NF}'`

#Recover if needed
VERSION_num=`echo $VERSION | cut -d. -f1`
if [ ! -z "$UNTIL_TIME" ]; then
  if [ $VERSION_num -lt 12 ]; then
    RECOVERY_CONF_FILE=$PG_DATA/recovery.conf
  else
    RECOVERY_CONF_FILE=$PG_DATA/postgresql.conf
    touch $PG_DATA/recovery.signal
  fi
  echo "restore_command = 'cp $LOG_BKP_MNTPT/$ARCHIVE_FORMAT "%p"'" >> $RECOVERY_CONF_FILE
  echo "recovery_target_time='$UNTIL_TIME'" >> $RECOVERY_CONF_FILE
  echo "recovery_target_action=promote" >> $RECOVERY_CONF_FILE
fi

chown -R $OSUSER:$OSGROUP $PG_DATA

#Start PostgreSQL
export PGDATA=$PG_DATA
su $OSUSER -c "pg_ctl start -w -t 600 -l $PG_DATA/pg_log.log -D $PG_DATA"
if [ $? -ne 0 ]; then
  errorHandle $PG_DATA "Fail to start PostgreSQL" $errorLog $VERSION_num
fi
sleep 30

#Prepare errorLog in case failure
errorLog=/var/act/log/errorLog_$ACT_JOBNAME.err

#Check if PostgreSQL running or not
foundInst=0
runningPG_list=`ps -ef | grep " postgres: "| grep -vw grep | awk '{print $3}'| sort | uniq | xargs`
for PG_PID in $runningPG_list
do
  tmpPGDATA=`xargs -0 -L1 -a /proc/$PG_PID/environ | grep -w "PGDATA=" | cut -d= -f2`
  if [ "$tmpPGDATA" = "$PG_DATA" ]; then
    foundInst=1
    break
  fi
done

if [ $foundInst -eq 0 ]; then
  errorHandle $PG_DATA "Fail to start PostgreSQL" $errorLog $VERSION_num
fi

#Prepare db connection with default user
set +x
prepareConn "$PORT" "$DBUSER" "$DBPASSWD" "" "$SOCKET_DIR"
set -x

#Check recovery status and wait until recovery complete
if [ ! -z "$UNTIL_TIME" ]; then
  sleep 5
  cnt=0
  waitTimes=30
  SRC_VERSION_NUM=`echo $SRC_VERSION | cut -d. -f1`

  if [ $SRC_VERSION_NUM -lt 10 ]; then
    pg9RecMsg=$(su $OSUSER -c '$PSQL -c "SELECT '\''1'\''"' 2>&1)
    while [ "$pg9RecMsg" =~ "the database system is starting up" ] && [ $cnt -lt $waitTimes ]
    do
      sleep 30
      pg9RecMsg=$(su $OSUSER -c '$PSQL -c "SELECT '\''1'\''"' 2>&1)
      cnt=$(( cnt + 1 ))
    done
  else
    isInRecovery=`su $OSUSER -c "echo START__RS;$PSQL -c \"SELECT pg_is_in_recovery()\"; echo END__RS"`
    isInRecovery=`echo $isInRecovery | awk -F"START__RS" '{print $2}' | awk -F"END__RS" '{print $1}' | sed 's/ //g'`
    while [ "$isInRecovery" = "t" ] && [ $cnt -lt $waitTimes ]
    do
      sleep 30
      isInRecovery=`su $OSUSER -c "echo START__RS;$PSQL -c \"SELECT pg_is_in_recovery()\"; echo END__RS"`
      isInRecovery=`echo $isInRecovery | awk -F"START__RS" '{print $2}' | awk -F"END__RS" '{print $1}' | sed 's/ //g'`
      cnt=$(( cnt + 1 ))
    done
  fi

  if [ -e $PG_DATA/recovery.done ]; then
    echo "INFOMSG: Recovery success"
  elif [ $SRC_VERSION_NUM -ge 12 ] && [ ! -e $PG_DATA/recovery.signal ]; then
    echo "INFOMSG: Recovery success"
  else
    errorHandle $PG_DATA "Fail to recover database" $errorLog $VERSION_num
  fi

fi

echo "INFOMSG: PostgreSQL started successfully"

exit 0
