#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

DBSID=$1
DBUSER=$2
PRETDBUSER=$3
DBPORT=$4
PORT=$5
DELLOG=$6
HANAVERSION=$7
USESYSTEMDBKEY=$8
ENDPIT="$9"
REMOVAL_HOURS=${10}
LASTBACKEDUPDBNAMES=${11}

DATA=`date`

if [ -z "$REMOVAL_HOURS" ]; then
   REMOVAL_HOURS=2
elif [ "$REMOVAL_HOURS" = "0" ]; then
     REMOVAL_HOURS=1
fi

######## Function to check if variable is empty #########
isempty()
{
   INPUT_VAR_NAME=$1
   INPUT_VAR_VALUE=$2
   if [ -z "$INPUT_VAR_VALUE" ]; then
       echo "ERRORMSG: Parameter $INPUT_VAR_NAME is empty. Please check!!"
       exit 1
   fi
}

######### END Function  ########

isempty DBUSER $DBUSER
isempty PRETDBUSER $PRETDBUSER
isempty DBPORT $DBPORT
isempty PORT $PORT
isempty DELLOG $DELLOG
isempty HANAVERSION $HANAVERSION
isempty USESYSTEMDBKEY $USESYSTEMDBKEY
isempty REMOVAL_HOURS $REMOVAL_HOURS
#isempty LASTBACKEDUPDBNAMES $LASTBACKEDUPDBNAMES


echo "************************** DATE:$DATE: start log delete post script for $DBSID database ******************"

USESYSTEMDBKEY=`echo $USESYSTEMDBKEY | tr '[A-Z]' '[a-z]'`
if [ -z "$HANAVERSION" ]; then
   echo "ERRORMSG: HANAVERSION is missing"
   exit 1
fi

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

if [ "$HANAVERSION" = "1.0" ]; then
   MULTIDB=`grep "mode=" $globalpath/SYS/global/hdb/custom/config/global.ini |awk -F"=" '{print $2}'`
   MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]' | xargs`
fi

FILE_NAME=/act/touch/hanabackupdblist.log
dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j -x "select string_agg(database_name,',')  from m_databases"`
dbnames=`echo $dbnames | sed 's/"//g'`
echo -e "backedupdbnames:$dbnames" > $FILE_NAME

if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
   SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME <= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME in ('data snapshot','complete data backup') and STATE_NAME = 'successful' "
   ID=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL | head -n1`
   ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

  if [ "$ID" = "?" ]
  then
      echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for database older then $DELLOG *********"
      #exit
      continue
  fi

  SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
  /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL

else
    dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases"`
    for i in ${dbnames}; do
       if [ $i = "DATABASE_NAME" ]; then
          :
       else
          TSID=`echo $i |awk -F '"' '{print $2}'`
          if [ $TSID = "SYSTEMDB" ]; then
             SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME <= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME in ('data snapshot','complete data backup') and STATE_NAME = 'successful' "
             ID=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL | head -n1`
             ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

             if [ "$ID" = "?" ]
             then
                echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for systemdb database for older then $DELLOG *********"
                #exit
                continue
             fi
             echo "************************** delete systemdb log ******************"
             SQL="BACKUP CATALOG DELETE FOR SYSTEMDB ALL BEFORE BACKUP_ID $ID COMPLETE "
             /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
             retval=$?
             if [ $retval -ne 0 ]; then
                echo "WARNINGMSG: Warning: Failed to delete the SYSTEMDB log for $DBSID database: check customapp-saphana.log for details"
             fi

          else
             if [ "$USESYSTEMDBKEY" = "true" ]; then
                TDBUSER=$DBUSER
                SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS_DATABASES.M_BACKUP_CATALOG where SYS_START_TIME <= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME in ('data snapshot','complete data backup') and DATABASE_NAME='$TSID' and STATE_NAME = 'successful'"
             else
                TDBUSER=$PRETDBUSER$TSID
                SQL="SELECT TOP 1 max(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME <= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME in ('data snapshot','complete data backup') and STATE_NAME = 'successful' "
             fi

             echo "************************** delete tenant log ******************"
             ID=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j $SQL | head -n1`
             ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

             if [ "$ID" = "?" ]
             then
                echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for $TSID database for older then $DELLOG *********"
                #exit
                continue
             fi

             if [ "$USESYSTEMDBKEY" = "true" ]; then
                SQL="BACKUP CATALOG DELETE FOR $TSID ALL BEFORE BACKUP_ID $ID COMPLETE "
             else
                SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
             fi

             /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j $SQL
             retval=$?
             if [ $retval -ne 0 ]; then
                echo "WARNINGMSG: Warning: Failed to delete the $TSID log for $DBSID database: check customapp-saphana.log for details"
             fi

          fi
       fi
    done
fi


#### Remove Log backups ####

#systemdate=`date +"%Y-%m-%d %H:%M:%S"`
systemdate="$ENDPIT"
globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`
globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini
LOGBACKUPPATH=`grep -iw ^basepath_logbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`

vdate=`echo $systemdate | awk '{print $1}'`
vtime=`echo $systemdate | awk '{print $2}'`

FROM_TIME=`date -d "$vtime $vdate -$REMOVAL_HOURS hours" +"%Y-%m-%d %H:%M:%S"`
    
#set +x
for lname in $(echo $LASTBACKEDUPDBNAMES | tr ',' ' ' )
do
  for cname in $(echo $dbnames | tr ',' ' ' )
  do
    NOTFOUND=
    cname=`echo $cname | sed s'/"//g'`
    if [ "$cname" = "DATABASE_NAME" ]; then
       continue;
    elif [ "$lname" = "$cname" ]; then
         break;
    else
         NOTFOUND=YES
    fi
  done
  if [ "$NOTFOUND" = "YES" ]; then
     if [ -z "$DELETED_DBLIST" ]; then
        DELETED_DBLIST=$lname
     else
        DELETED_DBLIST=$DELETED_DBLIST','$lname
     fi
  fi
done
set -x
echo "DELETE_DB:$DELETED_DBLIST"


if [ ! -z  "$LOGBACKUPPATH" ]; then
   for tsid in $(echo $dbnames | tr ',' ' '); do
      tsid=`echo $tsid | sed s'/"//g'`
      if [ "$tsid" = "DATABASE_NAME" ]; then
           :
      elif [ "$tsid" = "SYSTEMDB" ]; then
         tsid=SYSTEMDB
         if [ -d $LOGBACKUPPATH/$tsid ]; then
            find $LOGBACKUPPATH/$tsid -name "log_backup_*" -type f ! -newermt "$FROM_TIME" | xargs rm -f
         fi
      else
         if [ "$HANAVERSION" = "1.0" ]; then
            tsid=
         else
             tsid=DB_"$tsid"
         fi
         if [ -d $LOGBACKUPPATH/$tsid ]; then
            find $LOGBACKUPPATH/$tsid -name "log_backup_*" -type f ! -newermt "$FROM_TIME" | xargs rm -f
         fi
      fi
   done
   for del_tsid in $(echo $DELETED_DBLIST | tr ',' ' '); do
      tsid=DB_"$del_tsid"
      if [ -d $LOGBACKUPPATH/$tsid ]; then
         find $LOGBACKUPPATH/$tsid -name "log_backup_*" -type f | xargs rm -f
      fi
   done
fi


echo "************************** end log delete post script ******************"
exit

