#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

DBSID=$1
DBUSER=$2
DBPORT=$3
PORT=$4
HANAVERSION=$5
JOBNAME=$6


######## Function to check if variable is empty #########
isempty()
{
   INPUT_VAR_NAME=$1
   INPUT_VAR_VALUE=$2
   if [ -z "$INPUT_VAR_VALUE" ]; then
       echo "ERRORMSG: Parameter $INPUT_VAR_NAME is empty. Please check!!"
       exit 1
   fi
}
######### END Function  ########

isempty DBSID $DBSID
isempty DBUSER $DBUSER
isempty DBPORT $DBPORT
isempty PORT $PORT
isempty HANAVERSION $HANAVERSION
isempty JOBNAME $JOBNAME

CONFIG_FILEPATH=/tmp/$JOBNAME

if [ ! -d "$CONFIG_FILEPATH" ]; then
   mkdir -p $CONFIG_FILEPATH
fi
FILE_NAME=/act/touch/hanabackupdblist.log
BACKUP_XML=/act/touch/backupstatus.xml

FAIL_MSG=/act/touch/FAIL_MSG

echo "************************** DATE:$DATE: Close Snapshot $DBSID database ******************"

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j -x "select string_agg(database_name,',')  from m_databases"`
dbnames=`echo $dbnames | sed 's/"//g'`
echo -e "backedupdbnames:$dbnames" > $FILE_NAME

HCMT='ACT_'$DBSID'_SNAP'

SQL="SELECT BACKUP_ID from M_BACKUP_CATALOG WHERE COMMENT = '$HCMT' and ENTRY_TYPE_NAME = 'data snapshot' and STATE_NAME = 'prepared' and SYS_END_TIME is null "

ID=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL | head -n1`

if [ "$HANAVERSION" = "1.0" ]
then
SQL="BACKUP DATA CLOSE SNAPSHOT BACKUP_ID $ID SUCCESSFUL 'ActBackup'"
else
SQL="BACKUP DATA FOR FULL SYSTEM CLOSE SNAPSHOT BACKUP_ID $ID SUCCESSFUL 'ActBackup'"
fi

set -o pipefail
/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL 2>&1 | tee $FAIL_MSG/"$DBSID"_post_error_msg
retval=$?


if [ $retval -ne 0 ]; then
    errorstring=`cat $FAIL_MSG/"$DBSID"_post_error_msg | xargs |awk -F"rc=" '{print $2}'`
    if [ -z "$errorstring" ]; then
       errorstring=`tail -c100 $FAIL_MSG/"$DBSID"_post_error_msg`
    fi
    for i in $(echo $dbnames | tr "," " ")
    do
       echo "ERRORMSG:$i-Failed to Close Snapshot Error:$errorstring, check customapp-saphana.log for details (abort will try to close the snapshot)"
       echo -e "\t<tenant name =\"$i\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
    done
    echo "</tenants>" >> $BACKUP_XML
    exit $retval
fi
######## Write to XML ########
for i in $(echo $dbnames | tr "," " ")
do
  echo -e "\t<tenant name =\"$i\" status=\"success\"/>" >> $BACKUP_XML
done
echo "</tenants>" >> $BACKUP_XML

BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from m_backup_catalog where comment='$HCMT' and backup_id=(select max(backup_id) from m_backup_catalog where entry_type_name in ('data snapshot') and state_name='successful')"
BACKUPID_STRING=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $BACKUPID_SQL`

if [ ! -z "$BACKUPID_SQL" ]; then
   BACKUPID_STRING=`echo $BACKUPID_STRING | sed 's/"//g'`
   echo "**************** BACKUP DETAILS => DBSID:$DBSID, BACKUPTYPE:LVM Snapshot, $BACKUPID_STRING *******************"
fi

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x "select database_name from m_databases"`

if [ "$HANAVERSION" != "1.0" ]; then
  for tsid in ${dbnames}; do
    tsid=`echo $tsid | sed s'/"//g'`
    SQL="select subpath from SYS_DATABASES.M_VOLUMES where database_name = '$tsid'"
    FILEPATH=`$hdbsql -U $DBUSER -a -x -j $SQL`
    FILEPATH=`echo $FILEPATH | xargs |sed 's/"//g'`
    FILEPATH=`echo $FILEPATH|sed 's/ /:/g'`
    echo "$tsid=$FILEPATH" >> $CONFIG_FILEPATH/HANA.manifest
  done
fi

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

if [ ! -z "$globalpath" ]; then
   cp $globalpath/SYS/global/hdb/custom/config/global.ini $CONFIG_FILEPATH/
   cp $globalpath/SYS/global/hdb/custom/config/nameserver.ini $CONFIG_FILEPATH/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup Configuration files"
   fi
fi

SSF_LOCATION=`hdbuserstore list | grep "DATA FILE" | awk -F ":" '{print $2}'`
SSF_LOCATION=`dirname $SSF_LOCATION`
if [ ! -z "$SSF_LOCATION" ]; then
   cp $SSF_LOCATION/*.KEY $CONFIG_FILEPATH/
   cp $SSF_LOCATION/*.DAT $CONFIG_FILEPATH/
   cp $SSF_LOCATION/*.shm $CONFIG_FILEPATH/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup SSFS HDB Configuration files"
   fi
fi

SSFS_SECURITY_LOCATION=`echo $DIR_INSTANCE`
SSFS_SECURITY_LOCATION=`dirname $SSFS_SECURITY_LOCATION`

if [ ! -z "$SSFS_SECURITY_LOCATION" ]; then
   cp $SSFS_SECURITY_LOCATION/SYS/global/hdb/security/ssfs/*.KEY $CONFIG_FILEPATH/
   cp $SSFS_SECURITY_LOCATION/SYS/global/hdb/security/ssfs/*.DAT $CONFIG_FILEPATH/
    if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup SSFS Security Configuration files"
   fi

fi
if [ -d "$FAIL_MSG" ]; then
   rm -f $FAIL_MSG/*
fi
exit 0

