#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
DBSID=$1
DBUSER=$2
DBPORT=$3
PORT=$4
HANAVERSION=$5


######## Function to check if variable is empty #########
isempty()
{
   INPUT_VAR_NAME=$1
   INPUT_VAR_VALUE=$2
   if [ -z "$INPUT_VAR_VALUE" ]; then
       echo "ERRORMSG: Parameter $INPUT_VAR_NAME is empty. Please check!!"
       exit 1
   fi
}
######### END Function  ########

isempty DBSID $DBSID
isempty DBUSER $DBUSER
isempty DBPORT $DBPORT
isempty PORT $PORT
isempty HANAVERSION $HANAVERSION


DATE=`date +"%m%d%Y"`

DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'

echo "************************** DATE:$DATE: Pre check if catalog backup path and log backup path is same  ******************"
FILE_NAME=/act/touch/hanabackupdblist.log
BACKUP_XML=/act/touch/backupstatus.xml
FAIL_MSG=/act/touch/FAIL_MSG

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j -x "select string_agg(database_name,',')  from m_databases"`
dbnames=`echo $dbnames | sed 's/"//g'`
echo -e "backedupdbnames:$dbnames" > $FILE_NAME

globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini

LOGBACKUPPATH=`grep -i ^basepath_logbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
CATLOGBACKUPPATH=`grep -i ^basepath_catalogbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`

if [ -z "$LOGBACKUPPATH" ]; then
    echo "ERRORMSG:$dbnames-Backup Pre Check: LOGBACKUPPATH is not set in global.ini"
    exit 1
elif [ ! -z "$LOGBACKUPPATH" ]; then
    if [ -z "$CATLOGBACKUPPATH" ]; then
         CATLOGBACKUPPATH=$LOGBACKUPPATH
         cp $globalinipath $globalinipath'.'$DATE
         echo -e "basepath_catalogbackup = $CATLOGBACKUPPATH" >> $globalinipath
    elif [ ! -z "$CATLOGBACKUPPATH" ]; then
         if [ "$LOGBACKUPPATH" != "$CATLOGBACKUPPATH" ]; then
             cp $globalinipath $globalinipath'.'$DATE
             CATLOGBACKUPPATH=$LOGBACKUPPATH
             sed -i "/basepath_catalogbackup/d" $globalinipath
             sed -i "/persistence/a basepath_catalogbackup = $CATLOGBACKUPPATH" $globalinipath
         fi
    fi
fi

### backint check
bint=`cat $globalpath/SYS/global/hdb/custom/config/global.ini | grep -iw catalog_backup_using_backint | grep -i true`
if [ ! -z "$bint" ]; then
   echo "ERRORMSG:$dbnames-Backup Pre Check: The target server should not have backint configured. Please remove all the backint related entries from $globalpath/SYS/global/hdb/custom/config/global.ini file "
   exit 1
fi

HCMT='ACT_'$DBSID'_SNAP'
DATE=`date`
echo "************************** DATE:$DATE: Freezing $DBSID database ******************"

if [ "$HANAVERSION" = "1.0" ]
then
SQL="BACKUP DATA CREATE SNAPSHOT COMMENT '$HCMT'"
else
SQL="BACKUP DATA FOR FULL SYSTEM CREATE SNAPSHOT COMMENT '$HCMT'"
fi
set -o pipefail
/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -j $SQL 2>&1 | tee $FAIL_MSG/"$DBSID"_pre_error_msg
retval=$?
echo "<tenants>" >> $BACKUP_XML
if [ $retval -ne 0 ]; then
    errorstring=`cat $FAIL_MSG/"$DBSID"_pre_error_msg | xargs |awk -F"rc=" '{print $2}'`
    if [ -z "$errorstring" ]; then
       errorstring=`tail -c100 $FAIL_MSG/"$DBSID"_pre_error_msg`
    fi
    for i in $(echo $dbnames | tr "," " ")
    do
      echo "ERRORMSG:$i-Failed to Freeze,Error:$errorstring, check customapp-saphana.log for details "
      echo -e "\t<tenant name =\"$i\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
    done
    echo "</tenants>" >> $BACKUP_XML
    exit $retval
fi
if [ -d "$FAIL_MSG" ]; then
   rm -f $FAIL_MSG/*
fi

exit 0

