#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
DBSID=$1
userstorekey=$2

dbuser=$DBSID'adm'

######## Function to check Connection ########
check_connection()
{
   KEY=$1
   TDBSID=$2
   portno=$3
   echo "***** CHECKING DB CONNECTION for $TDBSID *****"
   SQL="select database_name from m_databases"
   dbnames=`su - $dbuser -c "$hdbsql -U $KEY -a -j -x $SQL"`
   if [ "$?" -gt 0 ]; then
      echo "ERRORMSG: Unable to connect to the database $TDBSID using $KEY!"
      exit 1
   else
      KEYHOST=`su - $dbuser -c "hdbuserstore list $KEY | grep -w ENV"`
      KEYPORT=`echo $KEYHOST | awk -F":" '{print $3}' |xargs`
      KEYHOST=`echo $KEYHOST | awk -F":" '{print $2}' |xargs`
      nameserverini=$globalpath/SYS/global/hdb/custom/config/nameserver.ini
      localhostname=`cat $nameserverini |grep -E "worker =|worker=" | awk -F"=" '{print $2}' |xargs`
      phostname=`hostname`
      if [ "$KEYHOST" != "$phostname" ] && [ "$KEYHOST" != "$localhostname" ]; then
         echo "ERRORMSG: USERSTORE KEY - $KEYHOST passed does not have valid hostname configured!"
         exit 1
      elif [ $KEYPORT != $portno ]; then
         echo "ERRORMSG: USERSTORE KEY - $KEYHOST passed does not have valid port# configured!"
      fi
   fi
}

######## Function to check priviliges ########
check_privs()
{
  KEY=$1
  TDBSID=$2

  echo "***** CHECKING DB PRIVILEGES for $TDBSID *****"
  DATABASE_USER=`su - $dbuser -c "hdbuserstore list $KEY | grep -w USER"`
  DATABASE_USER=`echo $DATABASE_USER | awk -F":" '{print $2}' |xargs`
  DATABASE_USER=`echo $DATABASE_USER | tr '[a-z]' '[A-Z]'`

  SQL="select count(*) from PUBLIC.EFFECTIVE_PRIVILEGES where USER_NAME = '$DATABASE_USER' and PRIVILEGE in ('CATALOG READ','BACKUP ADMIN')"
   echo $SQL
  PRIV_COUNT=`su - $dbuser -c "$hdbsql -U $KEY -a -j -x \"$SQL\""`
  if [ $PRIV_COUNT -lt 2 ]; then
     echo "ERRORMSG: Userstore key $KEY do not have required privileges (CATALOG READ, BACKUP ADMIN)  on database $TDBSID, Please check!"
     exit 1
  fi
}

######## Function to check snapshots ########
check_snapshot_exists()
{
  KEY=$1

  echo "***** CHECKING IF THERE ARE ANY SNAPSHOTS PRESENT *****"
  SQL="SELECT count(1) from M_BACKUP_CATALOG WHERE ENTRY_TYPE_NAME = 'data snapshot' and STATE_NAME = 'prepared' and SYS_END_TIME is null"
  PRIV_COUNT=`su - $dbuser -c "$hdbsql -U $KEY -a -j -x \"$SQL\""`
  if [ $PRIV_COUNT -gt 0 ]; then
     echo "ERRORMSG: There is already a snapshot present in the prepared state. Please check!"
     exit 1
  fi
}

###### END ##########

tdate=`date +%Y%m%d%H%M%S`
FILELOC="/act/touch/"
if [ ! -d "$FILELOC" ]; then
   mkdir -p $FILELOC
   chmod 755 $FILELOC
else
   chmod 755 $FILELOC
fi
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
FILE_NAME=$FILELOC/hanabackupdblist.log
BACKUP_XML=$FILELOC/backupstatus.xml
DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'
if [ ! -d /act/touch/FAIL_MSG ]; then
   mkdir -p /act/touch/FAIL_MSG
   chown "$DBADM":sapsys /act/touch/FAIL_MSG
else
   rm -f /act/touch/FAIL_MSG/*
   chown "$DBADM":sapsys /act/touch/FAIL_MSG
fi
if [ ! -f "$FILE_NAME" ]; then
   touch $FILE_NAME
   chown "$DBADM":sapsys $FILE_NAME
   chmod 755 $FILE_NAME
fi

if [ ! -f "$BACKUP_XML" ]; then
   touch $BACKUP_XML
   chown "$DBADM":sapsys $BACKUP_XML
   chmod 755 $BACKUP_XML
fi


globalpath=`su - $dbuser -c 'echo $DIR_INSTANCE'`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   SECUDIR=`su - $dbuser -c 'echo $SECUDIR'`
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

keystore=`su - $dbuser -c 'cd exe ; hdbuserstore list '`
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: can not connect to check the key store!"
  exit 1
fi
keycount=`echo $keystore | grep -w $userstorekey | wc -l`
if [ $keycount -eq 0 ]; then
  echo "ERRORMSG: keystore does not exists!"
  exit 1
fi

INSTANCENUM=`su - $dbuser -c 'env | grep TINSTANCE= | cut -d"=" -f2'`
if [ -z "$INSTANCENUM" ]; then
   INSTANCENUM=`su - $dbuser -c 'basename $DIR_INSTANCE | rev | cut -c 1-2 | rev'`
fi

HVERSIONTMP=`echo $HANAVERSION |sed 's/\.//g'`
if [ "$HVERSIONTMP" -lt "200000" ]; then
   PORTNO=3"$INSTANCENUM"15
else
   PORTNO=3"$INSTANCENUM"13
fi

check_connection $userstorekey $DBSID "$PORTNO"
check_privs $userstorekey $DBSID
check_snapshot_exists $userstorekey

exit 0
