#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#set -x

echo ""
echo "     AA          CCCCC  TTTTTTTT  IIIIII  FFFFFFF IIIIII      OOOO     /-\\"
echo "    AAAA       CC          TT       II    FF        II      OO    OO   |R|"
echo "   AA  AA     CC           TT       II    FFFF      II     OOO    OOO  \_/"
echo "  AAAAAAAA    CC           TT       II    FF        II     OOO    OOO"
echo " AAA    AAA    CC          TT       II    FF        II      OO    OO"
echo "AAAA    AAAA     CCCCC     TT     IIIIII  FF      IIIIII      OOOO "
echo ""

DBSID=$1
Function=$2
SECONDARY_NODE=$3
PRIMARY_sitename=$4
SECONDARY_sitename=$5
REPL_MODE=$6
OP_MODE=$7
VHOSTNAME=$8


if [[ X$VHOSTNAME == X ]]
 then
  LOCAL_HOST=`hostname| cut -d'.' -f-1`
 else
  LOCAL_HOST="$VHOSTNAME"
fi

TIMESTAMP=`date +%y-%m-%d`

if [[ "X$DBSID" == "X-help" ]]
then
Function=$DBSID
fi

if [[ "X$Function" == "X-help" ]]
then
echo " "
echo " ***********************************"
echo " *** SAP HANA Replication script ***"
echo " ***********************************"
echo " "
echo " Help :"
echo " "
echo " Script usage : act_saphana_replication.sh <dbsid> -function *** ***"
echo " "
echo " function:"
echo " "
echo " -register       : to register a system from Secondary to primary"
echo " "
echo " act_saphana_replication.sh <dbsid> -register <Secondary_hostname> <primary_sitename> <secondary_sitename> <syncomode> <operation_mode> <Optional : virtual_PRIMARY_Hostname>"
echo " e.g. act_saphana_replication.sh sr1 -register host2 site1 site2 sync logreplay (Vhost1)"
echo " "
echo " -unregister       : to register a system from Secondary to primary"
echo " "
echo " act_saphana_replication.sh <dbsid> -unregister <Secondary_hostname>"
echo " "
echo " -discover       : to discover current replication configuration"
echo " "
echo " act_saphana_replication.sh <dbsid> -discover <Optional : virtual_PRIMARY_Hostname>"
echo " e.g. act_saphana_replication.sh sr1 -discover Vhost1"
echo " "
echo " -failover       : to switch primary and secondary system replication - to execute from secondary"
echo " "
echo " act_saphana_replication.sh <dbsid> -failover "
echo " e.g. act_saphana_replication.sh sr1 -failover "
echo " "
echo " -takeover_force : to takeover the secondary node (in case of primary not available or to bring up a copy of primary)"
echo " "
echo " act_saphana_replication.sh <dbsid> -takeover_force"
echo " e.g. act_saphana_replication.sh sr1 -takeover_force"
exit 0
fi


if [[ "X$DBSID" != "X" ]]
then
length=`expr length "$DBSID"`
if [[ "$length" != 3 ]]
then
echo "invalid argument for SID, please use -help function for usage"
exit 0
fi
fi

if [[ "X$DBSID" == "X" ]]
then
echo "please enter the SID (instance ID 3 characters) for your Hana database : "
read -p ""  DBSID
fi

length=`expr length "$DBSID"`

if [[ "$length" != 3 ]]
then
echo "invalid length of SID"
exit 0
else
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
echo "The SID is $DBSID"
fi

DBSIDlow=`echo "$DBSID" | awk '{print tolower($0)}'`
DBSIDupper=`echo "$DBSID" | awk '{print toupper($0)}'`
HANAuser=`echo "$DBSIDlow""adm"`

if id "$HANAuser" >/dev/null 2>&1
 then echo "user $HANAuser exists"
else echo "user $HANAuser does not exist"
 exit 0
fi
SAPSYSTEMNAME=`su - "$HANAuser" bash -c 'echo $SAPSYSTEMNAME'`
SYSNUMBER=`su - "$HANAuser" bash -c 'echo $TINSTANCE'`

LOGMODE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini | grep ^log_mode | cut -d'=' -f2-`
	first_char=`echo "$LOGMODE" | cut -c1-1`
	if [[ "X$first_char" == "X " ]]
        then LOGMODE=`printf "$LOGMODE" | awk '{print substr($0,2,length($0)-1)}'`
        fi

if [[ X$LOGMODE == "X" ]]
then echo "The logmode is Default (normal)"
else
echo "The logmode is $LOGMODE"
fi

if [[ X$LOGMODE != "Xnormal" ]] && [[ X$LOGMODE != "X" ]]
then echo "ERROR - the replication cannot be used with log_mode = $LOGMODE "
exit 1
else X=X
fi

if [[ "$Function" != "-discover" ]] && [[ "$Function" != "-failover" ]] && [[ "$Function" != "-unregister" ]] && [[ "$Function" != "-register" ]] && [[ "$Function" != "-help" ]] && [[ "$Function" != "-takeover_force" ]]

then

while true
   do
read -p "Please enter the function usage for this setup execution [-discover] [-failover] [-takeover_force] [-register] [-unregister] or [-help] : " Function
  if [[ "$Function" == "-discover" ]] || [[ "$Function" == "-takeover_force" ]] || [[ "$Function" == "-failover" ]] || [[ "$Function" == "-unregister" ]] || [[ "$Function" == "-register" ]] || [[ "$Function" == "-help" ]]
  then
  echo "The selected function is $Function"
  break
  else
    echo "non-acceptable entry ... you didn't type [-discover] [-failover] [takeover_force] [-register] [-unregister] or [-help]"
  fi
    done
fi




REPL_STATE=`su - "$HANAuser" -c "hdbnsutil -sr_state"`
function discover() {
REPL_MODE=`echo "$REPL_STATE" | grep ^mode: `

#The SECONDARY_NODE is here designated to be the virtual hostname provided in argument
if [[ X$SECONDARY_NODE == X ]]
 then
  LOCAL_HOST=`hostname| cut -d'.' -f-1`
 else
  LOCAL_HOST="$SECONDARY_NODE"
fi

echo $REPL_MODE
  if [[ X$REPL_MODE == "Xmode: none" ]]
  then echo "the replication mode is disable"
  else
	if [[ X$REPL_MODE == "Xmode: sync" ]]
  	then echo "the replication mode is Synchrone"
	     SECONDARY_NODE=`echo $LOCAL_HOST`
  	else if [[ X$REPL_MODE == "Xmode: syncmem" ]]
  	     then echo "the replication mode is synchrone in memory"
	     SECONDARY_NODE=`echo $LOCAL_HOST`
  	      else if [[ X$REPL_MODE == "Xmode: async" ]]
  	         then echo "the replication mode is Asynchrone"
	         SECONDARY_NODE=`echo $LOCAL_HOST`
	         else if [[ X$REPL_MODE == "Xmode: primary" ]]
	            then echo "the replication mode is primary for $LOCAL_HOST"
		       PRIMARY_NODE=`echo $LOCAL_HOST`
		      else echo "ERROR - the replication mode is unknown"
	              exit 1
		      fi
		  fi
	     fi
	fi
  fi


#if [[ X$PRIMARY_NODE == X ]]
#then
#PRIMARY_NODE=`echo "$REPL_STATE" | grep "^primary masters: " | cut -d':' -f2-`
#	first_char=`echo "$PRIMARY_NODE" | cut -c1-1`
#        if [[ "X$first_char" == "X " ]]
#        then PRIMARY_NODE=`printf "$PRIMARY_NODE" | awk '{print substr($0,2,length($0)-1)}'`
#        fi
#fi
#if [[ $PRIMARY_NODE == $LOCAL_HOST ]]
#then
#PRIMARY_sitename=`echo "$REPL_STATE" | grep "site name: " | cut -d':' -f2-`
#	first_char=`echo "$PRIMARY_sitename" | cut -c1-1`
#        if [[ "X$first_char" == "X " ]]
#        then PRIMARY_sitename=`printf "$PRIMARY_sitename" | awk '{print substr($0,2,length($0)-1)}'`
#        fi
#else    PRIMARY_sitename=`echo "$REPL_STATE" | grep "Mapping: " | cut -d'>' -f1 | cut -c10- | cut -d' ' -f1`
#
#
#	SECONDARY_sitename=`echo "$REPL_STATE" | grep "site name: " | cut -d':' -f2-`
#        first_char=`echo "$SECONDARY_sitename" | cut -c1-1`
#        if [[ "X$first_char" == "X " ]]
#        then SECONDARY_sitename=`printf "$SECONDARY_sitename" | awk '{print substr($0,2,length($0)-1)}'`
#        fi

#fi

PRIMARY_SITE_NAME=`echo "$REPL_STATE" | grep \(primary/primary\) |  awk '{printf $1}'`
PRIMARY_sitename=$PRIMARY_SITE_NAME
PRIMARY_CLUSTER_NODES=`echo "$REPL_STATE" | grep \\\["$PRIMARY_SITE_NAME"\\\] |  awk '{printf $4 "\n"}i'`
echo ""
echo "Primary sitename : $PRIMARY_SITE_NAME"
echo ""
echo "Primary cluster node(s) (scaleout)"
echo "$PRIMARY_CLUSTER_NODES"

  for i in ${PRIMARY_CLUSTER_NODES}; do

    MASTER_value=`echo "cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep ^active_master | cut -d'=' -f2- | xargs | cut -d':' -f-1"`
    MASTER_NODE=`ssh -o "StrictHostKeyChecking=no" $i "$MASTER_value"`
  done
PRIMARY_NODE=$MASTER_NODE

echo "Primary master node is : $MASTER_NODE"
echo ""
#if [[ X$REPL_MODE != "Xmode: none" ]]
#then
#echo "primary sitename : $PRIMARY_sitename"
#fi

if [[ X$SECONDARY_sitename == "X" ]]
then
SECONDARY_sitename=`echo "$REPL_STATE" | grep "Mapping: $PRIMARY_sitename " | cut -d'>' -f2-`
        first_char=`echo "$SECONDARY_sitename" | cut -c1-1`
        if [[ "X$first_char" == "X " ]]
        then SECONDARY_sitename=`printf "$SECONDARY_sitename" | awk '{print substr($0,2,length($0)-1)}'`
        fi
PRIMARY_sitename=`echo "$REPL_STATE" | grep "Mapping: " | cut -d'>' -f1 | cut -c10- | cut -d' ' -f1`
        first_char=`echo "$PRIMARY_sitename" | cut -c1-1`
        if [[ "X$first_char" == "X " ]]
        then PRIMARY_sitename=`printf "$PRIMARY_sitename" | awk '{print substr($0,2,length($0)-1)}'`
        fi
fi

if [[ $PRIMARY_NODE == $LOCAL_HOST ]]
then
SECONDARY_NODE=`echo "$REPL_STATE" | grep "$PRIMARY_NODE" | grep "\[$SECONDARY_sitename\]"| awk '{print $4}'`
fi

#echo "secondary node : $SECONDARY_NODE"
#echo "primary sitename : $PRIMARY_sitename"
echo ""
echo "secondary sitename : $SECONDARY_sitename"

REPL_MODE=`echo "$REPL_STATE" | grep "Replication mode of $SECONDARY_sitename" | cut -d':' -f2-`
	first_char=`echo "$REPL_MODE" | cut -c1-1`
        if [[ "X$first_char" == "X " ]]
        then REPL_MODE=`printf "$REPL_MODE" | awk '{print substr($0,2,length($0)-1)}'`
        fi
echo "The replication mode is : $REPL_MODE"

OP_MODE=`echo "$REPL_STATE" | grep "Operation mode of $SECONDARY_sitename" | cut -d':' -f2-`
        first_char=`echo "$OP_MODE" | cut -c1-1`
        if [[ "X$first_char" == "X " ]]
        then OP_MODE=`printf "$OP_MODE" | awk '{print substr($0,2,length($0)-1)}'`
        fi
echo "The operation mode is : $OP_MODE"

echo ""
SECONDARY_CLUSTER_NODES=`echo "$REPL_STATE" | grep \\\["$SECONDARY_sitename"\\\] |  awk '{printf $4 "\n"}'`
echo "Secondary cluster node(s) (scaleout) "
echo "$SECONDARY_CLUSTER_NODES"

  for i in ${SECONDARY_CLUSTER_NODES}; do

    MASTER_SEC_value=`echo "cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep ^active_master | cut -d'=' -f2- | xargs | cut -d':' -f-1"`
    MASTER_SEC_NODE=`ssh -o "StrictHostKeyChecking=no" $i "$MASTER_SEC_value"`
  done

echo "Secondary master node is : $MASTER_SEC_NODE"

}

function register() {


PRIMARY_NODE=`echo $LOCAL_HOST`

if [[ "X$SECONDARY_NODE" == "X" ]]
then
echo "please enter the SECONDARY node Hostname  : "
read -p ""  SECONDARY_NODE
fi

ISEXCH_SSH_ROOT_PRI=`cat $HOME/.ssh/authorized_keys | grep "root@$SECONDARY_NODE"`
ISEXCH_SSH_CMD_ROOT_SEC=`echo "cat $HOME/.ssh/authorized_keys | grep root@$PRIMARY_NODE"`
ISEXCH_SSH_ROOT_SEC=`ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "eval $ISEXCH_SSH_CMD_ROOT_SEC"`

if [[ X$ISEXCH_SSH_ROOT_PRI == X ]]
then echo "The root  certificate is not exchanged between Secondary and primary"
exit 1
fi


if [[ X$ISEXCH_SSH_ROOT_SEC == X ]]
then echo "The root  certificate is not exchanged between Primary and Secondary"
exit 1
fi

if [[ "X$PRIMARY_sitename" == "X" ]]
then
echo "please enter the Sitename for Primary $PRIMARY_NODE  : "
read -p "" PRIMARY_sitename
fi

if [[ "X$SECONDARY_sitename" == "X" ]]
then
echo "please enter the Sitename for Secondary $SECONDARY_NODE  : "
read -p "" SECONDARY_sitename
fi

if [[ "X$REPL_MODE" == "X" ]]
then
#echo "please enter the replication mode (sync, synmem or async) : "
#read -p "" REPL_MODE
while true
   do
read -p "please enter the replication mode (sync, synmem or async) : " REPL_MODE
  if [[ "$REPL_MODE" == "sync" ]] || [[ "$REPL_MODE" == "syncmem" ]] || [[ "$REPL_MODE" == "async" ]]
  then
  echo "The selected replication mode is $REPL_MODE"
  break
  else
    echo "non-acceptable entry ... you didn't type [sync] [async] or [syncmem]"
  fi
    done
fi



if [[ "X$OP_MODE" == "X" ]]
then
#echo "please enter the operation mode (logreplay, logreplay_readaccess or delta_datashipping) : "
while true
   do
read -p "please enter the operation mode (logreplay, logreplay_readaccess or delta_datashipping) : " OP_MODE
  if [[ "$OP_MODE" == "logreplay" ]] || [[ "$OP_MODE" == "logreplay_readaccess" ]] || [[ "$OP_MODE" == "delta_datashipping" ]]
  then
  echo "The selected operation mode is $OP_MODE"
  break
  else
    echo "non-acceptable entry ... you didn't type [logreplay] [logreplay_readaccess] or [delta_datashipping]"
  fi
    done
fi


PRIMARY_NODE=`echo $LOCAL_HOST`

ISPRIACTIVE=`echo "$REPL_STATE" | grep "active primary site: " | cut -d':' -f2-`
  first_char=`echo "$ISPRIACTIVE" | cut -c1-1`
  if [[ "X$first_char" == "X " ]]
  then ISPRIACTIVE=`printf "$ISPRIACTIVE" | awk '{print substr($0,2,length($0)-1)}'`
  fi

if [[ X$ISPRIACTIVE == X1 ]]
then
echo "INFO - you cannot register an already PRIMARY instance"
exit 0
else


ENABLE_CMD=`echo "hdbnsutil -sr_enable --name=$PRIMARY_sitename"`
su - "$HANAuser" bash -c "eval $ENABLE_CMD"
fi

ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function StopSystem HDB\""

HANAISUP_CMD=`echo "ssh -o \"StrictHostKeyChecking=no\" $SECONDARY_NODE \"su - \"$HANAuser\" bash -c \\\\\"HDB info\\\\\" | grep nameserver | grep -v \\\\\"grep\\\\\" | wc \""`
echo $HANAISUP_CMD
HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`

if [[ X$HANAISUP != X0 ]]
then

until [[ X$HANAISUP == X0 ]]
do
sleep 5

HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
echo "HANA is still up on $SECONDARY_NODE please wait"
done
fi

ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"mv /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/data/SSFS_\\\"$DBSIDupper\\\".DAT /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/data/SSFS_\\\"$DBSIDupper\\\".DAT_$TIMESTAMP\""
ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"mv /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/key/SSFS_\\\"$DBSIDupper\\\".KEY /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/key/SSFS_\\\"$DBSIDupper\\\".KEY_$TIMESTAMP\""


scp -pr /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/key/SSFS_"$DBSIDupper".KEY $SECONDARY_NODE:/usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/key/SSFS_"$DBSIDupper".KEY
scp -pr /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/data/SSFS_"$DBSIDupper".DAT $SECONDARY_NODE:/usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/data/SSFS_"$DBSIDupper".DAT

ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "chown \"$HANAuser\":sapsys /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/key/SSFS_\"$DBSIDupper\".KEY /usr/sap/$DBSIDupper/SYS/global/security/rsecssfs/data/SSFS_\"$DBSIDupper\".DAT"

HANAISUP_CMD=`echo "ssh -o \"StrictHostKeyChecking=no\" $SECONDARY_NODE \"su - \"$HANAuser\" bash -c \\\\\"HDB info\\\\\" | grep nameserver | grep -v \\\\\"grep\\\\\" | wc \""`
echo $HANAISUP_CMD
HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`

if [[ X$HANAISUP != X0 ]]
then

until [[ X$HANAISUP == X0 ]]
do
sleep 5

HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
echo "HANA is still up on $SECONDARY_NODE please wait"
done
fi

CMD_LINE_REGISTR=`echo "hdbnsutil -sr_register --name=$SECONDARY_sitename --remoteHost=$PRIMARY_NODE --remoteInstance=$SYSNUMBER --ReplicationMode=$REPL_MODE --operationMode=$OP_MODE"`
ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"eval $CMD_LINE_REGISTR\""
ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function StartSystem HDB &\""

}

function unregister() {

PRIMARY_NODE=`echo $LOCAL_HOST`

if [[ "X$SECONDARY_NODE" == "X" ]]
then
echo "please enter the SECONDARY node Hostname  : "
read -p ""  SECONDARY_NODE
fi


ssh -o "StrictHostKeyChecking=no" $SECONDARY_NODE "su - \"$HANAuser\" bash -c \"hdbnsutil -sr_unregister\""

}

function register_old() {
ISONLINE=`echo "$REPL_STATE" | grep "online: " | cut -d':' -f2-`
  first_char=`echo "$ISONLINE" | cut -c1-1`
  if [[ "X$first_char" == "X " ]]
  then ISONLINE=`printf "$ISONLINE" | awk '{print substr($0,2,length($0)-1)}'`
  fi

if [[ X$ISONLINE == Xtrue ]]
then
echo "INFO - please stop the database before executing registration"
exit 0
else if [[ X$ISONLINE == X ]]
     then
     echo "INFO - please execute initial registration"
     exit 0
     fi
fi

PRIMARY_NODE=`echo "HDBSettings.sh systemReplicationStatus.py --sapcontrol=1 | grep SECONDARY_HOST | head -1 | cut -d'=' -f2-"`
PRIMARY_NODE=`su - "$HANAuser" bash -c "eval $PRIMARY_NODE"`

if [[ X$PRIMARY_NODE == X$LOCAL_HOST ]] || [[ X$PRIMARY_NODE == X ]]
then
PRIMARY_NODE=`echo "HDBSettings.sh systemReplicationStatus.py --sapcontrol=1 | grep PRIMARY_MASTERS | head -1 | cut -d'=' -f2-"`
PRIMARY_NODE=`su - "$HANAuser" bash -c "eval $PRIMARY_NODE"`
fi

echo "The primary node for registration is $PRIMARY_NODE"
SECONDARY_sitename=`echo "$REPL_STATE" | grep "site name: " | cut -d':' -f2-`
  first_char=`echo "$SECONDARY_sitename" | cut -c1-1`
  if [[ "X$first_char" == "X " ]]
  then SECONDARY_sitename=`printf "$SECONDARY_sitename" | awk '{print substr($0,2,length($0)-1)}'`
  fi

if [[ X$FO_REPL_MODE == "X" ]] || [[ X$FO_OP_MODE == "X" ]]
then
echo "please specify the replication and/or operation mode in argument e.g. ./act_saphana_replication.sh <DBSID> -register sync logreplay"
else
  if [[ X$FO_REPL_MODE == "Xsync" ]] || [[ X$FO_REPL_MODE == "Xasync" ]]
  then X=X
  else
  echo "invalid argument $FO_REPL_MODE"
  exit 1
  fi

  if [[ X$FO_OP_MODE == "Xlogreplay" ]] || [[ X$FO_OP_MODE == "Xdelta_datashipping" ]] || [[ X$FO_OP_MODE == "Xlogreplay_readaccess" ]]
  then X=X
  else
  echo "invalid argument $FO_OP_MODE"
  exit 1
  fi

CMD_LINE_REGISTR=`echo "hdbnsutil -sr_register --name=$SECONDARY_sitename --remoteHost=$PRIMARY_NODE --remoteInstance=$SYSNUMBER --ReplicationMode=$FO_REPL_MODE --operationMode=$FO_OP_MODE"`


su - "$HANAuser" bash -c "eval $CMD_LINE_REGISTR"
su - "$HANAuser" bash -c "sapcontrol -nr $SYSNUMBER -function StartSystem HDB"
#su - "$HANAuser" bash -c "HDB start"
fi
}

function takeover_force() {
if [[ $PRIMARY_NODE == $LOCAL_HOST ]]
then echo "The takeover_force have to be executed from Secondary system"

else
CMD_LINE_FO=`echo "hdbnsutil -sr_takeover"`

su - "$HANAuser" bash -c "eval $CMD_LINE_FO"
su - "$HANAuser" bash -c "sapcontrol -nr $SYSNUMBER -function StartSystem HDB &"
#su - "$HANAuser" bash -c "HDB start &"
fi
}

function failover() {
if [[ $PRIMARY_NODE == $LOCAL_HOST ]]
then echo "The failover have to be executed from Secondary system"

else

ISREP_SYNC_CMD=`echo "ssh -o \"StrictHostKeyChecking=no\" $PRIMARY_NODE \"su - \"$HANAuser\" bash -c \\\\\"HDBSettings.sh systemReplicationStatus.py --sapcontrol=1 | grep REPLICATION_STATUS= | grep -v "ACTIVE"\\\\\"\""`
echo $ISREP_SYNC
ISREP_SYNC=`eval $ISREP_SYNC_CMD`

if [[ X$ISREP_SYNC != X ]]
then

until [[ X$ISREP_SYNC == X ]]
do
sleep 5
ISREP_SYNC=`eval $ISREP_SYNC_CMD`
echo "HANA is not yet synchronized - please wait"
done
fi

CMD_LINE_FO=`echo "hdbnsutil -sr_takeover"`

su - "$HANAuser" bash -c "eval $CMD_LINE_FO"
su - "$HANAuser" bash -c "sapcontrol -nr $SYSNUMBER -function StartSystem HDB &"
echo "please stop the DB on Primary node : $PRIMARY_NODE"
ssh -o "StrictHostKeyChecking=no" $PRIMARY_NODE "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function StopSystem HDB &\""


HANAISUP_CMD=`echo "ssh -o \"StrictHostKeyChecking=no\" $PRIMARY_NODE \"su - \"$HANAuser\" bash -c \\\\\"HDB info\\\\\" | grep nameserver | grep -v \\\\\"grep\\\\\" | wc \""`
echo $HANAISUP_CMD
HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`

if [[ X$HANAISUP != X0 ]]
then

until [[ X$HANAISUP == X0 ]]
do
sleep 5

HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
echo "HANA is still up on $PRIMARY_NODE please wait"
done
fi


CMD_LINE_REGISTR=`echo "hdbnsutil -sr_register --name=$PRIMARY_sitename --remoteHost=$SECONDARY_NODE --remoteInstance=$SYSNUMBER --ReplicationMode=$REPL_MODE --operationMode=$OP_MODE"`
ssh -o "StrictHostKeyChecking=no" $PRIMARY_NODE "su - \"$HANAuser\" bash -c \"eval $CMD_LINE_REGISTR\""
ssh -o "StrictHostKeyChecking=no" $PRIMARY_NODE "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function StartSystem HDB &\""
#ssh -o "StrictHostKeyChecking=no" $PRIMARY_NODE "su - \"$HANAuser\" bash -c \"HDB start\""
fi
}


if [[ "X$Function" == "X-discover" ]]
then discover
fi

if [[ "X$Function" == "X-failover" ]]
then
discover
failover
fi

if [[ "X$Function" == "X-register" ]]
then
register
fi

if [[ "X$Function" == "X-takeover_force" ]]
then
discover
takeover_force
fi
