#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

DATE=`date`
echo "$DATE"

DBSID=$1
OLDDBSID=$2
DBUSER=$3
HANAVERSION=$4
APPREMOUNT=$5
DATAPATH="$6"
LOGPATH="$7"
RECOVERYTIME="$8"

dbsid=$DBSID
echo "tstamp:$RECOVERYTIME"

x=`grep python_support $HOME/.sapenv.sh | awk -F"=" '{print $NF}' | awk '{print $NF}' | awk -F"'" '{print $1}'`
PYTHONPATH=${x/\$SAPSYSTEMNAME/$SAPSYSTEMNAME}
echo "pythonpath is: ***** $PYTHONPATH *************"

if [ -z $PYTHONPATH ]; then
echo "ERRORMSG: Recovery:  PYTHONPATH-- path for recoverSys.py is not set "
exit 1
fi

DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'

DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
OLDDBSID=`echo $OLDDBSID | tr '[a-z]' '[A-Z]'`

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

LOGBACKUPPATH=`grep -i basepath_logbackup $globalpath/SYS/global/hdb/custom/config/global.ini | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`

if [ ! -d "$LOGBACKUPPATH" ]; then
   mkdir -p $LOGBACKUPPATH
fi

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

waitforhdbnameserver() {
  #Set a timeout of 8 minutes (8*6*10)
  waitFlag=true
  timeout=0
  count=30
  if [ "$HANAVERSION" = "1.0" ]; then
     count=90
  fi
  while $waitFlag; do
    timeout=$((timeout+1))
    if [ $timeout -gt $count ]; then
      echo "ERRORMSG: hdbnameserver did not come up after 10 minutes!. Please check!"
      exit 1
    fi
    nameserverProcess=`ps -ef | grep -i hdbnameserver | grep -v grep | grep -i $DBADM`
    if [ -z "$nameserverProcess" ]; then
       echo "Waiting for hdbnameserver process to be up..."
       sleep 30
       continue
    else
      hdbpreprocessorProcess=`ps -ef | grep -i hdbpreprocessor | grep -v grep | grep -i $DBADM`
      if [ -z "$hdbpreprocessorProcess" ]; then
        echo "Waiting for hdbnameserver process to be up..."
        sleep 30
        continue
      else
        sleep 30
        dbnames=`$hdbsql -U $DBUSER -x "select database_name from m_databases"`
        if [ $? -ne 0 ]; then
         echo "Not yet able to execute db select query on m_databases..Retry"
         waitFlag=true
        else
         waitFlag=false
        fi
      fi
      sleep 30
      #waitFlag=false
    fi
  done
  return 0
}

if [ "$APPREMOUNT" = "TRUE" ]; then
   echo "************ Remount Flag: $APPREMOUNT ***************"
   INSTANCENUM=`env | grep TINSTANCE= | cut -d"=" -f2`
   sapcontrol -nr $INSTANCENUM -function StartSystem HDB

   if [ "$?" -gt 0 ]; then
      echo "ERRORMSG: Failed to start database during Remount Process!"
      exit 1;
   fi

   waitforhdbnameserver
   exit 0
fi
echo "************************** running HDB info  ******************"
HDB info

echo "************************** running recoverSys.py  ******************"

if [ ! -z "$RECOVERYTIME" ]; then
  LOG_BKP_PATH=""
  for path in $( echo $LOGPATH | tr ',' ' ')
     do
       if [ -z "$LOG_BKP_PATH" ]; then
          LOG_BKP_PATH=\'$path/SYSTEMDB\'
       else
          LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/SYSTEMDB\'
       fi
    done
if [ "$HANAVERSION" = "1.0" ]; then
HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING DATA PATH ('$DATAPATH') USING LOG PATH ($LOG_BKP_PATH) USING SNAPSHOT CHECK ACCESS USING FILE"

else
  CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
  CATALOGPATH=$CATALOGPATH/SYSTEMDB/
  LOG_BKP_PATH=""
  for path in $( echo $LOGPATH | tr ',' ' ')
     do
       if [ -z "$LOG_BKP_PATH" ]; then
          LOG_BKP_PATH=\'$path/SYSTEMDB\'
       else
          LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/SYSTEMDB\'
       fi
    done
HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DATAPATH') USING LOG PATH ($LOG_BKP_PATH) USING SNAPSHOT CHECK ACCESS USING FILE"

fi
else
HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATA USING SNAPSHOT CLEAR LOG"
fi

retval=$?

if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Recovery:  Failed to recover database $dbsid: check customapp-saphana.log for details."
    exit $retval
fi

waitforhdbnameserver

echo "************************** running HDB info  ******************"
HDB info

if [ "$HANAVERSION" = "2.0" ]; then
#if [ ! -z "$RECOVERYTIME" ]; then
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
for i in ${dbnames}; do
     TSID=`echo $i |awk -F '"' '{print $2}'`
     if [ $i = "DATABASE_NAME" ]; then
                :
     elif [ $TSID = "SYSTEMDB" ]; then
                :
     else
        if [ ! -z "$RECOVERYTIME" ]; then
           TSID=`echo $i |awk -F '"' '{print $2}'`
           CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
           CATALOGPATH=$CATALOGPATH"/DB_"$TSID
           LOG_BKP_PATH=""
           for path in $( echo $LOGPATH | tr ',' ' ')
           do
             if [ -z "$LOG_BKP_PATH" ]; then
                LOG_BKP_PATH=\'$path/DB_$TSID\'
             else
                LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/DB_$TSID\'
             fi
           done
           if [ -d "$CATALOGPATH" ]; then
              CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
              CATALOGPATH=$CATALOGPATH"/DB_"$TSID"/"
              LOG_BKP_PATH=""
           for path in $( echo $LOGPATH | tr ',' ' ')
           do
              if [ -z "$LOG_BKP_PATH" ]; then
                 LOG_BKP_PATH=\'$path/DB_$TSID\'
              else
                 LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/DB_$TSID\'
              fi
           done
           else
              CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
              CATALOGPATH=$CATALOGPATH"/DB_"$OLDDBSID"/"
              LOG_BKP_PATH=""
              for path in $( echo $LOGPATH | tr ',' ' ')
              do
                if [ -z "$LOG_BKP_PATH" ]; then
                   LOG_BKP_PATH=\'$path/DB_$OLDDBSID\'
               else
                  LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/DB_$OLDDBSID\'
               fi
              done
              if [ ! -d "$CATALOGPATH" ]; then
                 echo "$CATALOGPATH for $TSID could not found. $TSID recovery will not be recovered."
                 HDB stop
                 exit 1
              fi
           fi

           SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DATAPATH') USING LOG PATH ($LOG_BKP_PATH) USING SNAPSHOT CHECK ACCESS USING FILE"
           echo "********** recovering HANA TENANT DB recover:DBUSER=$DBUSER and dbsid=$TSID ************"

           $hdbsql -U $DBUSER -a -j $SQL
           retval=$?

           if [ $retval -ne 0 ]; then
              echo "ERRORMSG: Recovery: Failed to recover Tenant database $TSID: check customapp-saphana.log for details"
              HDB stop
              exit $retval
           fi
           if [ $retval -ne 0 ]; then
              echo "ERRORMSG: Recovery: Failed to recover Tenant database $TSID: check customapp-saphana.log for details"
              HDB stop
             exit $retval
           fi
        else
           STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
           SQL="RECOVER DATA FOR $TSID USING SNAPSHOT CLEAR LOG"
           echo "********** recovering HANA TENANT DB recover:DBUSER=$DBUSER and dbsid=$dbsid ************"
           $hdbsql -U $DBUSER -a -j $STOPSQL
           retval=$?
           if [ $retval -ne 0 ]; then
              echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
              exit $retval
           fi

           $hdbsql -U $DBUSER -a -j $SQL
           retval=$?
           if [ $retval -ne 0 ]; then
              echo "ERRORMSG: Recovery: Failed to recover Tenant database $TSID: check customapp-saphana.log for details"
              HDB stop
              exit $retval
           fi
        fi
     fi
done
echo "************************** running HDB info  ******************"
HDB info
for i in ${dbnames}; do
     TSID=`echo $i |awk -F '"' '{print $2}'`
     if [ $i = "DATABASE_NAME" ]; then
                :
     elif [ $TSID = "SYSTEMDB" ]; then
                :
     else
        TSID=`echo $i |awk -F '"' '{print $2}'`

       echo "********** starting HANA TENANT DB recover:DBUSER=$DBUSER and dbsid=$TSID ************"
       SQL="alter system start database $TSID"
       $hdbsql -U $DBUSER -a -j $SQL
       retval=$?

       if [ $retval -ne 0 ]; then
          echo "WARNINGMSG: Recovery: Failed saphana  tenant start for $TSID"
       fi
     fi
done
echo "********** end HANA TENANT DB recover ************"

echo "************************** running HDB info  ******************"
HDB info
fi

echo "************************** Database recovery complete ******************"

exit 0

