#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x

dbsid=$1
hanaversion=$2
userstorekey=$3
datavolpath=$4
logvolpath=$5

dbsid=`echo $dbsid | tr '[A-Z]' '[a-z]'`

dbuser=$dbsid'adm'
echo $dbuser

usercount=`su - $dbuser -c 'ls | wc -l'`
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: $dbsid does not exists, please check!"
  exit 1
fi

if [ -f /act/touch/.hana_migrate_"$dbsid".conf ]; then
   PVMOVE_STATUS=`cat /act/touch/.hana_migrate_"$dbsid".conf | awk -F "=" '{print $2}'`
   if [ "$PVMOVE_STATUS" = "YES" ]; then
      echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
      exit 0
   else
      echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
      exit 1
   fi
fi


HVERSION=`su - $dbuser -c "HDB version | grep -i version: | cut -d':' -f2 | sed -e 's/^[ \t]*//'"`
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: can not get HANA vserion!"
  exit 2
fi

VERSION=`echo $HVERSION | cut -c1-3`

echo "installed version: $VERSION"
echo "Source version: $hanaversion"

if [ "$hanaversion" != "$VERSION" ]; then
  echo "ERRORMSG: Source $hanaversion is different than target version :$VERSION"
  exit 3
fi

keystore=`su - $dbuser -c 'cd exe ; hdbuserstore list '`
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: can not connect to check the key store!"
  exit 4
fi
keycount=`echo $keystore | grep -i $userstorekey | wc -l`
if [ $keycount -eq 0 ]; then
  echo "ERRORMSG: keystore does not exists!"
  exit 5
fi

tdate=`date +"%m%d%Y"`
TARGET_MNT_PT=$ACT_MOUNT_POINTS
globalpath=`su - $dbuser -c 'echo $DIR_INSTANCE'`
globalpath=`dirname $globalpath`
globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini

if [ ! -f "$globalinipath" ]; then
   echo "ERRORMSG: The configuration file $globalinipath does not exist. Please check!"
   exit 1
fi

DATAVOL=`grep -i basepath_datavolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
LOGVOL=`grep -i basepath_logvolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
LOGBKPPATH=`grep -i basepath_logbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`

if [ -f /act/touch/"$dbsid"_mount_params ]; then
   echo -e "LOG_BACKUP_PATH=$LOGBKPPATH" >> /act/touch/"$dbsid"_mount_params
fi

if [ "$datavolpath" != "$DATAVOL" ]; then
  echo "WARNINGMSG: $datavolpath is different then global.ini basepath_datavolumes:$DATAVOL, updating global.ini file"
  cp $globalinipath $globalinipath'.'$tdate

  if [ ! -f "$globalinipath".orig ]; then
     cp $globalinipath $globalinipath'.'orig
  fi
  
  sed -i '/basepath_datavolumes/d' $globalinipath
  sed -i "/\bpersistence\b/a basepath_datavolumes = $datavolpath" $globalinipath
  sed -i '/basepath_logvolumes/d' $globalinipath
  sed -i "/\bpersistence\b/a basepath_logvolumes = $logvolpath" $globalinipath


    LOGBACKUPPATH=$logvolpath/logbackup
    sed -i "/basepath_logbackup/d" $globalinipath
    sed -i "/basepath_catalogbackup/d" $globalinipath
    CATLOGBACKUPPATH=$LOGBACKUPPATH


    sed -i "/\bpersistence\b/a basepath_catalogbackup = $CATLOGBACKUPPATH" $globalinipath
    sed -i "/\bpersistence\b/a basepath_logbackup = $LOGBACKUPPATH" $globalinipath
fi
exit 0
