#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

DATE=`date`
echo "$DATE"

DBSID=$1
DBPORT=$2
DBUSER=$3
EXCLUDE_DB_LIST="$4"
INCLUDE_DB_LIST="$5"
HANAVERSION="$6"
HANABACKUPPATH=$7
RECOVERYTIME="$8"
LOGPATH=$9

echo "tstamp:$RECOVERYTIME"
x=`grep python_support $HOME/.sapenv.sh | awk -F"=" '{print $NF}' | awk '{print $NF}' | awk -F"'" '{print $1}'`
PYTHONPATH=${x/\$SAPSYSTEMNAME/$SAPSYSTEMNAME}

echo "pythonpath is: ***** $PYTHONPATH *************"

if [ -z $PYTHONPATH ]; then
echo "ERRORMSG: Recovery: PYTHONPATH-- path for recoverSys.py is not set "
exit 1
fi

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

MULTIDB=`grep -w "mode" $globalpath/SYS/global/hdb/custom/config/global.ini |grep -iw multidb |awk -F"=" '{print $2}'`
MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]' |xargs`

recoverSystemDBEX=`echo $EXCLUDE_DB_LIST | grep -i SYSTEMDB |wc -l`
recoverSystemDBIN=`echo $INCLUDE_DB_LIST | grep -i SYSTEMDB |wc -l`
if [ $recoverSystemDBEX -gt 0 ]; then
   recoverSystemDbFlag="NO"
elif [ $recoverSystemDBIN -gt 0 ]; then
   recoverSystemDbFlag="YES"
elif [ "$EXCLUDE_DB_LIST" = "null" ] && [ "$INCLUDE_DB_LIST" = "null" ]; then
   recoverSystemDbFlag="YES"
elif [ "$EXCLUDE_DB_LIST" != "null" ] && [ "$INCLUDE_DB_LIST" = "null" ]; then
     if [ $recoverSystemDBEX -eq 0 ]; then
        recoverSystemDbFlag="YES"
     fi
fi


if [ ! -z "$EXCLUDE_DB_LIST" ] && [ "$EXCLUDE_DB_LIST" != "null" ]; then
        DB_LIST=`echo $EXCLUDE_DB_LIST | tr '[a-z]' '[A-Z]'`
        DB_LIST=`echo $DB_LIST | sed -e "s/^/'/" -e "s/\$/'/" -e "s/,/',\'/g"`
        DB_LIST='not in ('"$DB_LIST"')'
elif [ ! -z "$INCLUDE_DB_LIST" ] && [ "$INCLUDE_DB_LIST" != "null" ]; then
       DB_LIST=`echo $INCLUDE_DB_LIST | tr '[a-z]' '[A-Z]'`
       DB_LIST=`echo $DB_LIST | sed -e "s/^/'/" -e "s/\$/'/" -e "s/,/',\'/g"`
       DB_LIST=' in ('"$DB_LIST"')'
else
       DB_LIST="null"
fi


DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'

DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
waitforhdbnameserver() {
  #Set a timeout of 8 minutes (8*6*10)
  waitFlag=true
  timeout=0
  if [ "$HANAVERSION" = "1.0" ]; then
     exittimeout=90
  else
     exittimeout=30
  fi
  while $waitFlag; do
    timeout=$((timeout+1))
    if [ $timeout -gt $exittimeout ]; then
      echo "ERRORMSG: hdbnameserver did not come up after 10 minutes!. Please check!"
      exit 1
    fi
    nameserverProcess=`ps -ef | grep -i hdbnameserver | grep -v grep | grep -i $DBADM`
    if [ -z "$nameserverProcess" ]; then
       echo "Waiting for hdbnameserver process to be up..."
       sleep 30
       continue
    else
      sleep 30
      if [ "$DB_LIST" = "null" ]; then
          dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases"`
      else
          dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases where database_name $DB_LIST"`
      fi
      if [ $? -ne 0 ]; then
         echo "Not yet able to execute db select query on m_databases..Retry"
         waitFlag=true
      else
         waitFlag=false
      fi
    fi
  done
  return 0
}

echo "************************** running HDB info  ******************"
HDB info


if [ ! -z $LOGPATH ]; then
  LOGPATHCHECK=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev` 
  checklogs=`find $LOGPATHCHECK/ -name "log_backup*" | wc -l`
 if [ "$checklogs" -eq 0 ]; then
   RECOVERYTIME=''
 fi
fi
if [ "$recoverSystemDbFlag" = "YES" ]; then
echo "************************** running recoverSys.py  ******************"

  ##### ADDED to recover from FULL+INCR BACKUP when there is no log and recoverytime
   if [ "$HANAVERSION" = "1.0" ]; then
      if [ "$MULTIDB" = "multidb" ]; then
         INCR_BKP_COUNT=`ls -l $HANABACKUPPATH/SYSTEMDB/ | grep INCR | wc -l` 
         catalogbackup_exists=`ls -l $HANABACKUPPATH/SYSTEMDB/ | grep log_backup_0_0_0_0 | wc -l`
      else
          INCR_BKP_COUNT=`ls -l $HANABACKUPPATH/$DBSID/ | grep INCR | wc -l` 
          catalogbackup_exists=`ls -l $HANABACKUPPATH/$DBSID/ | grep log_backup_0_0_0_0 | wc -l`
      fi
   else
      INCR_BKP_COUNT=`ls -l $HANABACKUPPATH/SYSTEMDB/ | grep INCR | wc -l` 
      catalogbackup_exists=`ls -l $HANABACKUPPATH/SYSTEMDB/ | grep log_backup_0_0_0_0 | wc -l`
   fi

   if [ -z "$LOGPATH" ] && [ -z "$RECOVERYTIME" ] && [ "$INCR_BKP_COUNT" -gt 0 ]; then  
         if [ "$catalogbackup_exists" -eq 0 ]; then
            if [ "$HANAVERSION" = "1.0" ]; then
               if [ "$MULTIDB" = "multidb" ]; then
                   hdbbackupdiag --generate --databaseName SYSTEMDB -d $HANABACKUPPATH/SYSTEMDB --dataDir $HANABACKUPPATH/SYSTEMDB --logDirs $HANABACKUPPATH/SYSTEMDB
                   retval=$?
               else
                   hdbbackupdiag --generate --databaseName $DBSID -d $HANABACKUPPATH/$DBSID --dataDir $HANABACKUPPATH/$DBSID --logDirs $HANABACKUPPATH/$DBSID
                   retval=$?
               fi
            else
                hdbbackupdiag --generate --databaseName SYSTEMDB -d $HANABACKUPPATH/SYSTEMDB --dataDir $HANABACKUPPATH/SYSTEMDB --logDirs $HANABACKUPPATH/SYSTEMDB
                retval=$?
            fi
            if [ "$retval" -gt 0 ]; then
               echo "ERRORMSG: Failed to generate catalog file: check customapp-saphana.log for details."
               exit 1
            fi
         fi
         if [ "$HANAVERSION" = "1.0" ]; then
             if [ "$MULTIDB" = "multidb" ]; then
                DPATH=$HANABACKUPPATH"/SYSTEMDB"
             else 
                DPATH=$HANABACKUPPATH"/$DBSID" 
             fi
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '2099-12-01 00:00:00' CLEAR LOG USING DATA PATH ('$DPATH') USING LOG PATH ('$DPATH')"
            retval=$?
         else
            CATALOGPATH=$HANABACKUPPATH"/SYSTEMDB/"
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '2099-12-01 00:00:00' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$CATALOGPATH') USING LOG PATH ('$CATALOGPATH')"
            retval=$?
         fi  
   elif [ -z "$LOGPATH" ]; then
      if [ -z "$RECOVERYTIME" ]; then
         if [ "$HANAVERSION" = "1.0" ]; then
            if [ "$MULTIDB" = "multidb" ]; then
               DPATH=$HANABACKUPPATH"/SYSTEMDB/COMPLETE_DATA_BACKUP" 
            else
                DPATH=$HANABACKUPPATH"/$DBSID/COMPLETE_DATA_BACKUP"
            fi
         else
             DPATH=$HANABACKUPPATH"/SYSTEMDB/COMPLETE_DATA_BACKUP"
         fi
         CATALOGPATH=$HANABACKUPPATH"/SYSTEMDB/"
         HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATA USING FILE ('$DPATH') CLEAR LOG"
         retval=$?
      else
         if [ "$HANAVERSION" = "1.0" ]; then
            if [ "$MULTIDB" = "multidb" ]; then
               DPATH=$HANABACKUPPATH"/SYSTEMDB"
               LOG_BKP_PATH=""
               for path in $( echo $LOGPATH | tr ',' ' ') 
               do
                 if [ -z "$LOG_BKP_PATH" ]; then
                    LOG_BKP_PATH=\'$path/SYSTEMDB\'
                 else
                    LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/SYSTEMDB\'
                 fi
               done
            else
               DPATH=$HANABACKUPPATH"/$DBSID"
               LOG_BKP_PATH=""
               for path in $( echo $LOGPATH | tr ',' ' ') 
               do
                 if [ -z "$LOG_BKP_PATH" ]; then
                    LOG_BKP_PATH=\'$path\'
                 else
                     LOG_BKP_PATH=$LOG_BKP_PATH','\'$path\'
                 fi
               done
            fi
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING DATA PATH ('$DPATH') USING LOG PATH ($LOG_BKP_PATH)"
            retval=$?
         else
           CATALOGPATH=$HANABACKUPPATH"/SYSTEMDB/"
           HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$CATALOGPATH')"
           retval=$?
         fi
      fi
   else
      if [ -z "$RECOVERYTIME" ]; then
         if [ "$HANAVERSION" = "1.0" ]; then
            if [ "$MULTIDB" = "multidb" ]; then
               DPATH=$HANABACKUPPATH"/SYSTEMDB/COMPLETE_DATA_BACKUP"
            else
               DPATH=$HANABACKUPPATH"/$DBSID/COMPLETE_DATA_BACKUP"
            fi
         else
             DPATH=$HANABACKUPPATH"/SYSTEMDB/COMPLETE_DATA_BACKUP"
         fi
          CATALOGPATH=$HANABACKUPPATH"/SYSTEMDB/"
         HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATA USING FILE ('$DPATH') CLEAR LOG"
   	retval=$?
      else
         if [  "$HANAVERSION" = "1.0" ]; then
            if [ "$MULTIDB" = "multidb" ]; then
               DPATH=$HANABACKUPPATH"/SYSTEMDB"
               LOG_BKP_PATH=""
               for path in $( echo $LOGPATH | tr ',' ' ')
               do
                 if [ -z "$LOG_BKP_PATH" ]; then
                    LOG_BKP_PATH=\'$path/SYSTEMDB\'
                 else
                    LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/SYSTEMDB\'
                 fi
               done
            else
               DPATH=$HANABACKUPPATH"/$DBSID"
               LOG_BKP_PATH=""
               for path in $( echo $LOGPATH | tr ',' ' ')
               do
                 if [ -z "$LOG_BKP_PATH" ]; then
                    LOG_BKP_PATH=\'$path\'
                 else
                    LOG_BKP_PATH=$LOG_BKP_PATH','\'$path\'
                 fi
               done
            fi
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING DATA PATH ('$DPATH') USING LOG PATH ($LOG_BKP_PATH)"
         else
            CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
            CATALOGPATH=$CATALOGPATH/SYSTEMDB/
            LOG_BKP_PATH=""
            for path in $( echo $LOGPATH | tr ',' ' ') 
            do
              if [ -z "$LOG_BKP_PATH" ]; then
                 LOG_BKP_PATH=\'$path/SYSTEMDB\'
              else
                 LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/SYSTEMDB\'
              fi
           done

            DPATH=$HANABACKUPPATH/"SYSTEMDB/"
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DPATH') USING LOG PATH ($LOG_BKP_PATH) "
         fi
         retval=$?
      fi
   fi
#   retval=$?
   JOB_FLAG=$retval
   if [ "$retval" -eq 0 ]; then
      RECOVERSYS_TRACE_LOC=$SAP_RETRIEVAL_PATH/trace
      if [ -f $RECOVERSYS_TRACE_LOC/recoverSys.trc ]; then
         JOB_FLAG=`tail -1 $RECOVERSYS_TRACE_LOC/recoverSys.trc | grep -i "failed" |wc -l`
      else
          JOB_FLAG=$retval
      fi
   fi
   if [ "$JOB_FLAG" -ne 0 ]; then
      echo "ERRORMSG: Failed to recover database $DBSID: check customapp-saphana.log for details."
      if [ "$HANAVERSION" = "1.0" ]; then
         echo "************* RESTORE STATUS=> DB: $DBSID, STATUS: FAILED **************"
      else
        echo "************* RESTORE STATUS=> DB: SYSTEMDB, STATUS: FAILED **************" 
      fi
      exit 1
   else
       if [ "$HANAVERSION" = "1.0" ]; then
         echo "************* RESTORE STATUS=> DB: $DBSID, STATUS: SUCCESS **************"
      else
        echo "************* RESTORE STATUS=> DB: SYSTEMDB, STATUS: SUCCESS **************"
      fi
   fi

fi
# get the tenant database name


SSF_LOCATION=`hdbuserstore list | grep "DATA FILE" | awk -F ":" '{print $2}'`
SSF_LOCATION=`dirname $SSF_LOCATION`
tdate=`date +%Y%m%d%H%M%S`
mkdir -p $SSF_LOCATION/$tdate
mv $SSF_LOCATION/*.* $SSF_LOCATION/$tdate
cp $HANABACKUPPATH/SSFS/*.* $SSF_LOCATION/
waitforhdbnameserver

if [ "$HANAVERSION" = "2.0" ] || [ "$MULTIDB" = "multidb" ]; then
for i in ${dbnames}; do
     TSID=`echo $i |awk -F '"' '{print $2}'`
     if [ $i = "DATABASE_NAME" ]; then
		:
     elif [ $TSID = "SYSTEMDB" ]; then
		:
     else
  	TSID=`echo $i |awk -F '"' '{print $2}'`
       
        INCR_BKP_COUNT=`ls -l $HANABACKUPPATH/$TSID/ | grep INCR | wc -l`
        catalogbackup_exists=`ls -l $HANABACKUPPATH/$TSID/ | grep log_backup_0_0_0_0 | wc -l`
        if [ -z "$LOGPATH" ] && [ -z "$RECOVERYTIME" ] && [ "$INCR_BKP_COUNT" -gt 0 ]; then
           if [ "$catalogbackup_exists" -eq 0 ]; then
                  hdbbackupdiag --generate --databaseName $TSID -d $HANABACKUPPATH/$TSID --dataDir $HANABACKUPPATH/$TSID --logDirs $HANABACKUPPATH/SYSTEMDB
                  retval=$?
           fi
           if [ "$retval" -gt 0 ]; then
                  echo "ERRORMSG: Failed to generate catalog file: check customapp-saphana.log for details."
                  exit 1
           fi 
           CATALOGPATH=$HANABACKUPPATH"/"$TSID"/"
           STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
           if [ "$MULTIDB" = "multidb" ] && [ "$HANAVERSION" = "1.0" ]; then
              DATAPATH=$HANABACKUPPATH"/"$TSID
              SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '2099-12-01 00:00:00' CLEAR LOG USING DATA PATH ('$DATAPATH') USING LOG PATH ('$DATAPATH')"
           else
              SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '2099-12-01 00:00:00' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$CATALOGPATH')"
           fi
           /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $STOPSQL
           retval=$?
           if [ $retval -ne 0 ]; then
              echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
              exit $retval
           fi
           /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
	   retval=$?
	elif [ -z "$LOGPATH" ]; then
		if [ -z "$RECOVERYTIME" ]; then
			DPATH=$HANABACKUPPATH/$TSID"/COMPLETE_DATA_BACKUP"
                        STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
			SQL="RECOVER DATA FOR $TSID USING FILE ('$DPATH') CLEAR LOG"
                        /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $STOPSQL
                           retval=$?
                           if [ $retval -ne 0 ]; then
                              echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                              exit $retval
                           fi 
			/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
	                 retval=$?
        	else
			CATALOGPATH=$HANABACKUPPATH"/"$TSID"/"
                        STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
                        if [ "$MULTIDB" = "multidb" ] && [ "$HANAVERSION" = "1.0" ]; then
                           DATAPATH=$HANABACKUPPATH"/"$TSID"/"
			   SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING DATA PATH ('$DATAPATH') USING LOG PATH ('$DATAPATH')"
                        else
			    SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$CATALOGPATH')"
                        fi
                        /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $STOPSQL
                        retval=$?
                        if [ $retval -ne 0 ]; then
                           echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                           exit $retval
                        fi
			/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
	                 retval=$?
		fi
	else
                if [ -z "$RECOVERYTIME" ]; then
                        DPATH=$HANABACKUPPATH/$TSID"/COMPLETE_DATA_BACKUP"
                        STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
                        SQL="RECOVER DATA FOR $TSID USING FILE ('$DPATH') CLEAR LOG"
                        /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $STOPSQL
                        retval=$?
                        if [ $retval -ne 0 ]; then
                           echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                           exit $retval
                        fi
                        /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
	                 retval=$?
                else
		###################################	CATALOGPATH=$LOGPATH"/DB_"$TSID"/"
                        CATALOGPATH=`echo $LOGPATH | rev |awk -F"," '{ print $1}' |rev`
                        CATALOGPATH=$CATALOGPATH"/DB_"$TSID"/"
                        LOG_BKP_PATH=""
                        for path in $( echo $LOGPATH | tr ',' ' ') 
                        do
                          if [ -z "$LOG_BKP_PATH" ]; then
                             LOG_BKP_PATH=\'$path/DB_$TSID\'
                          else
                             LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/DB_$TSID\'
                          fi
                        done

			DPATH=$HANABACKUPPATH"/"$TSID"/"
                        STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
			if [ "$MULTIDB" = "multidb" ] && [ "$HANAVERSION" = "1.0" ]; then
                           DPATH=$HANABACKUPPATH"/"$TSID"/"
                          ####################################### LOGBKPATH=$LOGPATH"/DB_"$TSID"/"
                           LOG_BKP_PATH=""
                           for path in $( echo $LOGPATH | tr ',' ' ')
                           do
                             if [ -z "$LOG_BKP_PATH" ]; then
                                LOG_BKP_PATH=\'$path/DB_$TSID\'
                             else
                                LOG_BKP_PATH=$LOG_BKP_PATH','\'$path/DB_$TSID\'
                             fi
                           done
			   SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING DATA PATH ('$DPATH') USING LOG PATH ($LOG_BKP_PATH) "
                        else
			    SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DPATH') USING LOG PATH ($LOG_BKP_PATH) "
                        fi
                        /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $STOPSQL
                        retval=$?
                        if [ $retval -ne 0 ]; then
                           echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                           exit $retval
                        fi
			/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL
	                 retval=$?
               fi
	fi
	if [ $retval -ne 0 ]; then
    		echo "ERRORMSG: Failed to recover tenant database $TSID: check customapp-saphana.log for details!"
                echo "************* RESTORE STATUS=> TENANT DB: $TSID, STATUS: FAILED **************"
    		exit $retval
        else
           echo "************* RESTORE STATUS=> TENANT DB: $TSID, STATUS: SUCCESS **************"
	fi
     fi
done
fi

exit 0

