#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

DBSID=$1
DBUSER=$2
PRETDBUSER=$3
DBPORT=$4
ACT_JOBNAME="$5"
PORT=$6
HANAVERSION=$7
HANABACKUPPATH=$8
DUMPSCHEDULE=$9
USESYSTEMDBKEY=${10}
RUNTENANTPARALLEL=${11}
PREVDBFULLDUMPSAMEDAY=${12}
IGNOREPARTIALFAILURE=${13}

######## Function to check if variable is empty #########
isempty()
{
   INPUT_VAR_NAME=$1
   INPUT_VAR_VALUE=$2
   if [ -z "$INPUT_VAR_VALUE" ]; then
       echo "ERRORMSG: Parameter $INPUT_VAR_NAME is empty. Please check!!"
       exit 1
   fi
}
######### END Function  ########

isempty DBSID $DBSID
isempty DBUSER $DBUSER
isempty PRETDBUSER $PRETDBUSER
isempty DBPORT $DBPORT
isempty ACT_JOBNAME $ACT_JOBNAME
isempty PORT $PORT
isempty HANAVERSION $HANAVERSION
isempty HANABACKUPPATH $HANABACKUPPATH
isempty DUMPSCHEDULE $DUMPSCHEDULE
isempty USESYSTEMDBKEY $USESYSTEMDBKEY
isempty RUNTENANTPARALLEL $RUNTENANTPARALLEL

DATE=`date`
tdate=`date +%Y%m%d%H%M%S`
FILE_NAME=/act/touch/hanabackupdblist.log
BACKUP_XML=/act/touch/backupstatus.xml
FAIL_MSG=/act/touch/FAIL_MSG

RUNTENANTPARALLEL=`echo $RUNTENANTPARALLEL |tr '[A-Z]' '[a-z]'`
IGNOREPARTIALFAILURE=`echo $IGNOREPARTIALFAILURE | tr '[A-Z]' '[a-z]'`
echo "************************** DATE:$DATE: start $DBSID database backup ******************"
echo "************************** Copying *.ini files ***************************"

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

USESYSTEMDBKEY=`echo $USESYSTEMDBKEY | tr '[A-Z]' '[a-z]'`


if [ ! -d $HANABACKUPPATH/INI ]; then
   mkdir -p $HANABACKUPPATH/INI
fi

cp $globalpath/SYS/global/hdb/custom/config/*.ini $HANABACKUPPATH/INI/

if [ "$?" -gt 0 ]; then
   echo "WARNINGMSG: Failed to backup Configuration files"
fi

if [ -z "$HANAVERSION" ]; then
   echo "ERRORMSG: HANAVERSION is missing. Please Check!"
   exit 1
fi

SSF_LOCATION=`hdbuserstore list | grep "DATA FILE" | awk -F ":" '{print $2}'`
SSF_LOCATION=`dirname $SSF_LOCATION`
if [ ! -z "$SSF_LOCATION" ]; then
   if [ ! -d $HANABACKUPPATH/SSFS ]; then
      mkdir -p $HANABACKUPPATH/SSFS
   fi
   cp $SSF_LOCATION/*.* $HANABACKUPPATH/SSFS/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup Configuration files"
   fi
fi

SSFS_SECURITY_LOCATION=`echo $DIR_INSTANCE`
SSFS_SECURITY_LOCATION=`dirname $SSFS_SECURITY_LOCATION`

if [ ! -z "$SSFS_SECURITY_LOCATION" ]; then
   if [ ! -d $HANABACKUPPATH/SSFS_SECURITY ]; then
      mkdir -p $HANABACKUPPATH/SSFS_SECURITY
   fi
   cp $SSFS_SECURITY_LOCATION/SYS/global/hdb/security/ssfs/*.* $HANABACKUPPATH/SSFS_SECURITY/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup SSFS Security Configuration files"
   fi

fi

if [ "$DUMPSCHEDULE" = "F" ]; then
    TYPE="F"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
    TYPE="I"
else
    TYPE=${DUMPSCHEDULE:`date +%w`:1}
fi

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j -x "select string_agg(database_name,',')  from m_databases"`
dbnames=`echo $dbnames | sed 's/"//g'`
echo -e "dbnames: $dbnames" > $FILE_NAME

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases"`

retval=$?

if [ -z "$dbnames" ]; then
   echo "ERRORMSG: Failed to get database names. Please check the Userstorekey!"
   exit 1
elif [ "$retval" -ne "0" ]; then
     echo "ERRORMSG: Failed to get database names. Please check the Userstorekey!"
     exit 1
fi

echo "<tenants>" >> $BACKUP_XML

ORIG_TYPE=$TYPE
for i in ${dbnames}; do
TYPE=$ORIG_TYPE

if [ "$TYPE" = "F" ]; then
   BACKUPTYPE="FULL"
elif [ "$TYPE" = "I" ]; then
   BACKUPTYPE="INCREMENTAL"
else
   BACKUPTYPE="DIFFERENTIAL"
fi

 if [ $i = "DATABASE_NAME" ]; then
   :
  else
    TSID=`echo $i |awk -F '"' '{print $2}'`
          if [ -d "$HANABACKUPPATH/$TSID" ]; then
              CHECK_FULL_BKP=`ls $HANABACKUPPATH/$TSID |grep -i "COMPLETE" | wc -l`
                if [ "$CHECK_FULL_BKP" -eq "0" ]; then
                   TYPE="F"
                   if [ "$ORIG_TYPE" != "$TYPE" ]; then
		      echo "WARNINGMSG: Performing Full backup as previous Full backup is not found for $TSID!"	
                      BACKUPTYPE="FULL"
		   fi
                fi 
          else
              TYPE="F"
              BACKUPTYPE="FULL"
          fi

   ######## GENERATE COMMENT BASED HANA VERSION ###########
    if [ "$HANAVERSION" = "1.0" ]; then
       COMMENT=
       MULTIDB=`grep "mode" $globalpath/SYS/global/hdb/custom/config/global.ini | grep -iw multidb |awk -F"=" '{print $2}'`
       MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]'|xargs`
    else
       COMMENT="COMMENT 'ACTIFIO_DUMP_BACKUP'"
    fi 

    case $TYPE in
    I)
        HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/INCR_DATA_BACKUP
        if [ $TSID = "SYSTEMDB" ]; then
           BACKUPSQL="BACKUP DATA INCREMENTAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
        else
            if [ "$USESYSTEMDBKEY" = "true" ]; then
               if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
                  BACKUPSQL="BACKUP DATA INCREMENTAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
               else
                  BACKUPSQL="BACKUP DATA INCREMENTAL FOR $TSID USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
               fi
            else
               BACKUPSQL="BACKUP DATA INCREMENTAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
            fi
        fi
      ;;
    D)
        HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/DIFF_DATA_BACKUP
        if [ $TSID = "SYSTEMDB" ]; then
           BACKUPSQL="BACKUP DATA DIFFERENTIAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
        else
           if [ "$USESYSTEMDBKEY" = "true" ]; then
              if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
                 BACKUPSQL="BACKUP DATA DIFFERENTIAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
              else
                 BACKUPSQL="BACKUP DATA DIFFERENTIAL FOR $TSID USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
              fi
           else
               BACKUPSQL="BACKUP DATA DIFFERENTIAL USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
           fi
        fi
      ;;
    F)
       if [ ! -z $HANABACKUPPATH/$TSID ];  then
           echo "************ $HANABACKUPPATH/$TSID"
           mkdir -p $HANABACKUPPATH/$TSID
       fi
       if [ $TSID = "SYSTEMDB" ]; then
          HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/COMPLETE_DATA_BACKUP
          BACKUPSQL="BACKUP DATA USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
       else 
           HANABACKUPPATHTDB=$HANABACKUPPATH/$TSID/COMPLETE_DATA_BACKUP
           if [ "$USESYSTEMDBKEY" = "true" ]; then
               if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
                  BACKUPSQL="BACKUP DATA USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
               else
                  BACKUPSQL="BACKUP DATA FOR $TSID USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
               fi
           else
               BACKUPSQL="BACKUP DATA USING FILE ('$HANABACKUPPATHTDB') $COMMENT"
           fi
      fi
      ;;
    esac

    echo "************************** backingup $TSID database backup ******************"

    # Perform the backup
  echo "************************** $BACKUPSQL ******************"

   # SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"

     if [ "$HANAVERSION" = "1.0" ]; then
        if [ "$MULTIDB" = "multidb" ]; then
           SQL="SELECT TOP 1 DESTINATION_PATH FROM SYS_DATABASES.M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
        else
            SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
        fi
     else
        SQL="SELECT TOP 1 DESTINATION_PATH FROM SYS_DATABASES.M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
     fi

    if [ $TSID = "SYSTEMDB" ]; then
           set -o pipefail
         /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg
         if [ $? -ne 0 ]; then
            errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
	    if [ -z "$errorstring" ]; then
               errorstring=`tail -c1000 $FAIL_MSG/"$TSID"_error_msg`
            fi
            for i in ${dbnames}; do
              i=`echo $i | sed 's/"//g'`
              if [ "$i" = "DATABASE_NAME" ]; then
                 :
               elif [ "$i" = "SYSTEMDB" ]; then  
                  echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
               else
                  echo "ERRORMSG:$i Failed to backup $i database,Error:SYSTEMDB Backup Failed. Please check customapp-saphana.log for details!"
               fi
            done
            if [ -z "$DBRPTF" ]; then
               DBRPTF=$TSID
            else
               DBRPTF="$DBRPTF"','"$TSID"
            fi
            echo -e "faileddbnames:$TSID" >> $FILE_NAME
            echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
            echo "</tenants> " >> $BACKUP_XML
            echo -e "************************* $ACT_JOBNAME END *************************" >> $FILE_NAME
            exit 1
         else
            if [ -z "$DBRPTSUCCESS" ]; then
               DBRPTSUCCESS=$TSID
            else
               DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
            fi
            echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML
         fi
         BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL | head -n1 | sed s/\"//g`
         cp $BCATALOG $HANABACKUPPATH/$TSID
         if [ "$HANAVERSION" = "1.0" ]; then
            BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from m_backup_catalog where backup_id=(select max(backup_id) from m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup') and state_name='successful')"
        else
           BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from SYS_DATABASES.m_backup_progress where entry_type_name in ('incremental data backup','complete data backup') and backup_id=(select max(backup_id) from SYS_DATABASES.m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup') and DATABASE_NAME='$TSID' and comment='ACTIFIO_DUMP_BACKUP' and state_name='successful') and state_name='successful' and DATABASE_NAME='$TSID'"
        fi
         BACKUPID_STRING=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $BACKUPID_SQL`
         BACKUPID_STRING=`echo $BACKUPID_STRING | sed 's/"//g'`
         echo "**************** BACKUP DETAILS:DBSID:SYSTEMDB, BACKUPTYPE:$BACKUPTYPE, $BACKUPID_STRING *******************"
         if [ -z "$BACKUPID_STRING" ]; then
            if [ -z "$DBRPTF" ]; then
               DBRPTF=$TSID
            else
               DBRPTF="$DBRPTF"','"$TSID"
            fi
            echo -e "faileddbnames:$TSID" >> $FILE_NAME
            echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
            echo "</tenants> " >> $BACKUP_XML
            echo -e "************************* $ACT_JOBNAME END *************************" >> $FILE_NAME
            exit 1
         fi
    else
       TDBSTATUS=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x -a "select active_status from m_databases where database_name='$TSID'"`
       if [ $TDBSTATUS = '"YES"' ]; then
          if [ "$USESYSTEMDBKEY" = "true" ]; then
              TDBUSER=$DBUSER
          else
              if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
                 TDBUSER=$PRETDBUSER
              else
                  TDBUSER=$PRETDBUSER$TSID
              fi
              SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
          fi
           set -o pipefail
            if [ "$RUNTENANTPARALLEL" = "true" ]; then
                /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg &
               processid=$!
               if [ -z "$processidlist" ]; then
                  processidlist=$TSID':'$processid 
               else
                  processidlist=$processidlist','$TSID':'$processid
               fi
            else
                /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg
                if [ $? -ne 0 ]; then
                   errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
                   if [ -z "$errorstring" ]; then
                      errorstring=`tail -c1000 $FAIL_MSG/"$TSID"_error_msg`
                   fi
                   echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
                   echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
                   if [ -z $DBRPTF ]; then
                      DBRPTF=$TSID
                   else
                       DBRPTF="$DBRPTF"','"$TSID"
                   fi             
                   if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
                      echo "</tenants>" >> $BACKUP_XML
                      exit 1
                   fi
                else
                    BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j $SQL | head -n1 | sed s/\"//g`
                    cp $BCATALOG $HANABACKUPPATH/$TSID
                    if [ "$HANAVERSION" = "1.0" ]; then
                       BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from m_backup_catalog where backup_id=(select max(backup_id) from m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup'))"
                    else
                        BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from SYS_DATABASES.m_backup_progress where entry_type_name in ('incremental data backup','complete data backup') and DATABASE_NAME='$TSID' and state_name='successful' and backup_id > (select max(BACKUP_ID) from SYS_DATABASES.m_backup_progress where entry_type_name in ('incremental data backup','complete data backup') and backup_id=(select max(backup_id) from SYS_DATABASES.m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup') and DATABASE_NAME='SYSTEMDB' and comment='ACTIFIO_DUMP_BACKUP' and state_name='successful') and state_name='successful' and DATABASE_NAME='SYSTEMDB') and service_name='indexserver'"
                    fi
                    BACKUPID_STRING=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $BACKUPID_SQL`
                    BACKUPID_STRING=`echo $BACKUPID_STRING | sed 's/"//g'`
                    echo "**************** BACKUP DETAILS:DBSID:$TSID, BACKUPTYPE:$BACKUPTYPE, $BACKUPID_STRING *******************"
                    if [ -z "$BACKUPID_STRING" ]; then
                      if [ -z $DBRPTF ]; then
                      DBRPTF=$TSID
                      else
                       DBRPTF="$DBRPTF"','"$TSID"
                      fi
                      if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
                      echo "</tenants>" >> $BACKUP_XML
                      exit 1
                      fi
                   else 
                    echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML
                    if [ -z "$DBRPTSUCCESS" ]; then
                       DBRPTSUCCESS=$TSID
                    else
                       DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
                    fi
                  fi 
                fi
            fi
       else 
         echo "WARNINGMSG: $TSID is not active. Current status is: $TDBSTATUS,  Please Check!"
         echo -e "\t<tenant name =\"$TSID\" status=\"skipped\" error=\"Tenant is inactive\"/>" >> $BACKUP_XML
               if [ -z $DBRPTS ]; then
                  DBRPTS=$TSID
              else
                  DBRPTS="$DBRPTS"','"$TSID"
              fi
       fi
    fi
  fi
  echo "************************** end $TSID database backup ******************"
done

######## Check TSID Backup status when started Parallely ############
for x in $(echo $processidlist | tr ',' ' ')
do
   pid=`echo $x | awk -F":" '{print $2}'`
   TSID=`echo $x | awk -F":" '{print $1}'`
   wait $pid
   retval=$?
   if [ $retval -ne 0 ]; then
      errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
      if [ -z "$errorstring" ]; then
         errorstring=`tail -c100 $FAIL_MSG/"$TSID"_error_msg`
      fi
      echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
      echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
      if [ -z $DBRPTF ]; then
         DBRPTF=$TSID
      else
         DBRPTF="$DBRPTF"','"$TSID"
      fi
      if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
         echo "</tenants>" >> $BACKUP_XML
         exit 1
      fi
   else
      ###### If Success copy the catalog 
      if [ "$USESYSTEMDBKEY" = "true" ]; then
         TDBUSER=$DBUSER
         SQL="SELECT TOP 1 DESTINATION_PATH FROM SYS_DATABASES.M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
      else
         TDBUSER=$PRETDBUSER$TSID
         SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
      fi
      BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j $SQL | head -n1 | sed s/\"//g`
      cp $BCATALOG $HANABACKUPPATH/$TSID
      if [ "$HANAVERSION" = "1.0" ]; then
         BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from m_backup_catalog where backup_id=(select max(backup_id) from m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup'))"
      else
          BACKUPID_SQL="select ',BACKUPID: '||BACKUP_ID||', END TIME: '||SYS_END_TIME||', STATUS: '||STATE_NAME from SYS_DATABASES.m_backup_progress where entry_type_name in ('incremental data backup','complete data backup') and DATABASE_NAME='$TSID' and state_name='successful' and backup_id > (select max(BACKUP_ID) from SYS_DATABASES.m_backup_progress where entry_type_name in ('incremental data backup','complete data backup') and backup_id=(select max(backup_id) from SYS_DATABASES.m_backup_catalog where entry_type_name in ('incremental data backup','complete data backup') and DATABASE_NAME='SYSTEMDB' and comment='ACTIFIO_DUMP_BACKUP' and state_name='successful') and state_name='successful' and DATABASE_NAME='SYSTEMDB') and service_name='indexserver'"
      fi
      BACKUPID_STRING=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $BACKUPID_SQL`
      BACKUPID_STRING=`echo $BACKUPID_STRING | sed 's/"//g'`
      echo "**************** BACKUP DETAILS:DBSID:$TSID, BACKUPTYPE:$BACKUPTYPE, $BACKUPID_STRING *******************"
      if [ -z "$BACKUPID_STRING" ]; then
          if [ -z $DBRPTF ]; then
            DBRPTF=$TSID
          else
            DBRPTF="$DBRPTF"','"$TSID"
          fi
          if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
            echo "</tenants>" >> $BACKUP_XML
            exit 1
          fi
      else 
          echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML
          if [ -z "$DBRPTSUCCESS" ]; then
             DBRPTSUCCESS=$TSID
           else
              DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
           fi
     fi
   fi

done ##### Process Check

   echo -e "backedupdbnames:$DBRPTSUCCESS" >> $FILE_NAME
   echo -e "faileddbnames:$DBRPTF" >> $FILE_NAME
   echo -e "skippeddbnames:$DBRPTS" >> $FILE_NAME
cat $FILE_NAME
if [ ! -z "$DBRPTF" ] && [ "$HANAVERSION" = "1.0" ]; then
   echo "ERRORMSG:$TSID Failed to backup $TSID,Error:$errorstring. Please check customapp-saphana.log for details!"
   echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
   echo "</tenants>" >> $BACKUP_XML
   exit 1
fi
if [ -d "$FAIL_MSG" ]; then
   rm -f $FAIL_MSG/*
fi
echo "************************** end database backup ******************"

echo "</tenants>" >> $BACKUP_XML
exit 0

