#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

DBSID=$1
DBUSER=$2
DBPORT=$3
HANAVERSION=$4
PRETDBUSER=$5
USESYSTEMDBKEY=$6
TYPE=$7

FAIL_MSG=/act/touch/FAIL_MSG
DATA=`date`
tdate=`date "+%Y-%m-%d %H:%M:%S"`
USESYSTEMDBKEY=`echo $USESYSTEMDBKEY | tr '[A-Z]' '[a-z]'`

errormessage() {
   retval=$1
   dsid=$2
   if [ "$retval" -gt 0 ]; then
       errorstring=`cat $FAIL_MSG/"$DBSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
       if [ -z "$errorstring" ]; then
          errorstring=`tail -c100 $FAIL_MSG/"$DBSID"_error_msg`
       fi
       echo "ERRORMSG: Failed to get the DB Size for $dsid,Error:$errorstring,Please check customapp-saphana.log for details!"
       exit 1
   fi  
}

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

if [ "$HANAVERSION" = "1.0" ]; then
   MULTIDB=`grep -w "mode" $globalpath/SYS/global/hdb/custom/config/global.ini| grep -iw multidb |awk -F"=" '{print $2}'`
   MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]' |xargs`
fi

######### FULL AND INCR CHANGES for 2.0 #############
if [ "$TYPE" = "F" ]; then  
   if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
      SQL="SELECT sum(USED_SIZE) as usedSpace FROM "M_VOLUME_FILES" WHERE FILE_TYPE='DATA'"
   else
      SQL="SELECT sum(ESTIMATED_SIZE) as usedSpace FROM SYS_DATABASES.m_backup_size_estimations where ENTRY_TYPE_NAME='complete data backup'"
   fi
else
   if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
      SQL_FULL="SELECT sum(USED_SIZE) as usedSpace FROM "M_VOLUME_FILES" WHERE FILE_TYPE='DATA'"
      SQL_INCR="SELECT sum(USED_SIZE) as usedSpace FROM "M_VOLUME_FILES" WHERE FILE_TYPE='DATA'"
   else
      SQL_FULL="SELECT sum(ESTIMATED_SIZE) as usedSpace FROM SYS_DATABASES.m_backup_size_estimations where ENTRY_TYPE_NAME='complete data backup'"
      SQL_INCR="SELECT sum(ESTIMATED_SIZE) as usedSpace FROM SYS_DATABASES.m_backup_size_estimations where ENTRY_TYPE_NAME='incremental data backup'"
   fi
fi

set -o pipefail
if [ "$TYPE" = "F" ]; then
   dbsizeFULL=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $SQL 2>&1 | tee $FAIL_MSG/"$DBSID"_error_msg`
   retval=$?
else
   dbsizeFULL=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $SQL_FULL 2>&1 | tee $FAIL_MSG/"$DBSID"_error_msg`
   dbsizeINCR=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -x -j $SQL_INCR 2>&1 | tee $FAIL_MSG/"$DBSID"_error_msg`
   retval=$?
fi

errormessage $retval $DBSID

if [ "$TYPE" = "I" ] || [ "$TYPE" = "D" ]; then
   dbsizeFULL=`echo $dbsizeFULL | rev | awk '{print $1}' |rev`
   dbsizeINCR=`echo $dbsizeINCR | rev | awk '{print $1}' |rev`
else
    dbsizeFULL=`echo $dbsizeFULL | rev | awk '{print $1}' |rev`
fi

echo "$dbsizeFULL $dbsizeINCR"

#DBSIZE=`echo ${DBSIZE} | sed -e 's/^[ \t]*//'`

exit

