#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
DBSID=$1
userstorekey=$2
USESYSTEMDBKEY=$3
PRETDBUSER=$4

dbuser=$DBSID'adm'

######## Function to validate variable values ########
isempty()
{
   INPUT_VAR_NAME=$1
   INPUT_VAR_VALUE=$2
   if [ -z "$INPUT_VAR_VALUE" ]; then
       echo "ERRORMSG: Parameter $INPUT_VAR_NAME is empty. Please check!!"
       exit 1
   fi
}

######## Function to check Connection ########
check_connection()
{
   KEY=$1
   TDBSID=$2
   echo "***** CHECKING DB CONNECTION for $TDBSID *****"
   SQL="select database_name from m_databases"
   dbnames=`su - $dbuser -c "$hdbsql -U $KEY -a -j -x $SQL"`
   if [ "$?" -gt 0 ]; then
      echo "ERRORMSG: Unable to connect to the database $TDBSID using $KEY!"
      exit 1
    else
      if [ "$HANAVERSION" != "1.0" ]; then
       if [ "$TDBSID" != "SYSTEMDB" ]; then
        echo "checking the database name from m_databases"
        dbcheck=`echo $dbnames | grep $TDBSID | grep -v SYSTEMDB`
        if [ -z $dbcheck ]; then
           echo "ERRORMSG: Unable to connect to the database $TDBSID using $KEY!"
           exit 1
        fi
       fi
      fi
   fi
}
######## Function to check priviliges ########
check_privs()
{
  KEY=$1
  TDBSID=$2
  
  echo "***** CHECKING DB PRIVILEGES for $TDBSID *****"
  DATABASE_USER=`su - $dbuser -c "hdbuserstore list $KEY | grep -w USER"`
  DATABASE_USER=`echo $DATABASE_USER | awk -F":" '{print $2}' |xargs`
  DATABASE_USER=`echo $DATABASE_USER | tr '[a-z]' '[A-Z]'`

  SQL="select count(*) from PUBLIC.EFFECTIVE_PRIVILEGES where USER_NAME = '$DATABASE_USER' and PRIVILEGE in ('CATALOG READ','BACKUP ADMIN')"
   echo $SQL
  PRIV_COUNT=`su - $dbuser -c "$hdbsql -U $KEY -a -j -x \"$SQL\""`
  if [ $PRIV_COUNT -lt 2 ]; then
     echo "ERRORMSG: Userstore key $KEY do not have required privileges (CATALOG READ, BACKUP ADMIN)  on database $TDBSID, Please check!"
     exit 1
  fi
}

###### END ##########

isempty DBSID $DBSID
isempty USERSTOREKEY $userstorekey
isempty USESYSTEMDBKEY $USESYSTEMDBKEY
isempty PRETDBUSER $PRETDBUSER


USESYSTEMDBKEY=`echo $USESYSTEMDBKEY | tr '[A-Z]' '[a-z]'`
tdate=`date +%Y%m%d%H%M%S`
FILELOC="/act/touch/"
if [ ! -d "$FILELOC" ]; then
   mkdir -p $FILELOC
   chmod 755 $FILELOC
else
   chmod 755 $FILELOC
fi
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
FILE_NAME=$FILELOC/hanabackupdblist.log
BACKUP_XML=$FILELOC/backupstatus.xml
DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'
if [ ! -d /act/touch/FAIL_MSG ]; then
   mkdir -p /act/touch/FAIL_MSG
   chown "$DBADM":sapsys /act/touch/FAIL_MSG
else
   rm -f /act/touch/FAIL_MSG/*
   chown "$DBADM":sapsys /act/touch/FAIL_MSG
fi
if [ ! -f "$FILE_NAME" ]; then
   touch $FILE_NAME
   chown "$DBADM":sapsys $FILE_NAME
   chmod 755 $FILE_NAME
fi

if [ ! -f "$BACKUP_XML" ]; then
   touch $BACKUP_XML
   chown "$DBADM":sapsys $BACKUP_XML
   chmod 755 $BACKUP_XML
fi



keystore=`su - $dbuser -c 'cd exe ; hdbuserstore list '`
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: can not connect to check the key store!"
  exit 1
fi
keycount=`echo $keystore | grep -w $userstorekey | wc -l`
if [ $keycount -eq 0 ]; then
  echo "ERRORMSG: keystore does not exists!"
  exit 1
fi


globalpath=`su - $dbuser -c 'echo $DIR_INSTANCE'`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   SECUDIR=`su - $dbuser -c 'echo $SECUDIR'`
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi

if [ "$USESYSTEMDBKEY" = "false" ]; then
   SQL="select database_name from m_databases"
   dbnames=`su - $dbuser -c "$hdbsql -U $userstorekey -a -j -x $SQL"`
   if [ -z "$dbnames" ]; then
      echo "ERRORMSG:Unable to connect to database using $userstorekey, Please Check!"
      exit 1
   fi
   for tsid in ${dbnames}; do
     tsid=`echo $tsid | sed s'/"//g'`
     if [ "$tsid" = "SYSTEMDB" ]; then
        check_connection $userstorekey SYSTEMDB
        check_privs $userstorekey SYSTEMDB 
     else
         if [ "$HANAVERSION" = "1.0" ]; then
            tenantkey=$PRETDBUSER
         else
            tenantkey=$PRETDBUSER$tsid
         fi
         tenantkeycount=`echo $keystore | grep -w $tenantkey | wc -l`
         if [ "$tenantkeycount" -eq 0 ]; then
            echo "ERRORMSG: keystore $tenantkey does not exists!"
            exit 1
         else
            check_connection $tenantkey $tsid
            check_privs $tenantkey $tsid 
         fi
     fi
   done
else
   if [ "$HANAVERSION" = "1.0" ]; then
      check_connection $userstorekey $DBSID
      check_privs $userstorekey $DBSID
   else
      check_connection $userstorekey SYSTEMDB
      check_privs $userstorekey SYSTEMDB
   fi   
fi
