#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

DATE=`date`
echo "$DATE"

DBSID=$1
DBPORT=$2
DBUSER=$3
EXCLUDE_DB_LIST="$4"
INCLUDE_DB_LIST="$5"
HANAVERSION="$6"
HANABACKUPPATH=$7
RECOVERYTIME="$8"
LOGPATH=$9


if [ -z "$RECOVERYTIME" ]; then
   LOGPATH=
fi

scp_files()
{
  BACKINT_LOG_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint_log.par
  BACKINT_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint.par
  for nodename in $(echo $OTHERNODES); do
    scp -pr $BACKINT_PAR_FILE $nodename:$BACKINT_PAR_FILE
    scp -pr $BACKINT_LOG_PAR_FILE $nodename:$BACKINT_LOG_PAR_FILE
    if [ "$?" -gt "0" ]; then
       echo "ERRORMSG: Failed to SCP backint related parameter files to other nodes during backup."
       exit 1
    fi 
  done
}

echo "tstamp:$RECOVERYTIME"
x=`grep python_support $HOME/.sapenv.sh | awk -F"=" '{print $NF}' | awk '{print $NF}' | awk -F"'" '{print $1}'`
PYTHONPATH=${x/\$SAPSYSTEMNAME/$SAPSYSTEMNAME}

echo "pythonpath is: ***** $PYTHONPATH *************"

if [ -z $PYTHONPATH ]; then
echo "ERRORMSG: Recovery: PYTHONPATH-- path for recoverSys.py is not set "
exit 1
fi

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi
NODECOUNT=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | wc -l`
MASTERNODE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep -E "active_master =|active_master=" |awk -F"=" '{print $2}' |awk -F":" '{print $1}' |xargs`
OTHERNODES=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | cut -d'=' -f-1 | cut -c7- |grep -v "$MASTERNODE" |xargs`

MULTIDB=`grep -w "mode" $globalpath/SYS/global/hdb/custom/config/global.ini |grep -iw multidb |awk -F"=" '{print $2}'`
MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]' |xargs`

recoverSystemDBEX=`echo $EXCLUDE_DB_LIST | grep -i SYSTEMDB |wc -l`
recoverSystemDBIN=`echo $INCLUDE_DB_LIST | grep -i SYSTEMDB |wc -l`
if [ $recoverSystemDBEX -gt 0 ]; then
   recoverSystemDbFlag="NO"
elif [ $recoverSystemDBIN -gt 0 ]; then
   recoverSystemDbFlag="YES"
elif [ "$EXCLUDE_DB_LIST" = "null" ] && [ "$INCLUDE_DB_LIST" = "null" ]; then
   recoverSystemDbFlag="YES"
elif [ "$EXCLUDE_DB_LIST" != "null" ] && [ "$INCLUDE_DB_LIST" = "null" ]; then
     if [ $recoverSystemDBEX -eq 0 ]; then
        recoverSystemDbFlag="YES"
     fi
fi


if [ ! -z "$EXCLUDE_DB_LIST" ] && [ "$EXCLUDE_DB_LIST" != "null" ]; then
        DB_LIST=`echo $EXCLUDE_DB_LIST | tr '[a-z]' '[A-Z]'`
        DB_LIST=`echo $DB_LIST | sed -e "s/^/'/" -e "s/\$/'/" -e "s/,/',\'/g"`
        DB_LIST='not in ('"$DB_LIST"')'
elif [ ! -z "$INCLUDE_DB_LIST" ] && [ "$INCLUDE_DB_LIST" != "null" ]; then
       DB_LIST=`echo $INCLUDE_DB_LIST | tr '[a-z]' '[A-Z]'`
       DB_LIST=`echo $DB_LIST | sed -e "s/^/'/" -e "s/\$/'/" -e "s/,/',\'/g"`
       DB_LIST=' in ('"$DB_LIST"')'
else
       DB_LIST="null"
fi


DBADM=`echo $DBSID | tr '[A-Z]' '[a-z]'`
DBADM=$DBADM'adm'

DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
waitforhdbnameserver() {
  #Set a timeout of 8 minutes (8*6*10)
  waitFlag=true
  timeout=0
  while $waitFlag; do
    timeout=$((timeout+1))
    if [ $timeout -gt 30 ]; then
      echo "ERRORMSG: hdbnameserver did not come up after 10 minutes!. Please check!"
      exit 1
    fi
    nameserverProcess=`ps -ef | grep -i hdbnameserver | grep -v grep | grep -i $DBADM`
    if [ -z "$nameserverProcess" ]; then
       echo "Waiting for hdbnameserver process to be up..."
       sleep 30
       continue
    else
      sleep 30
      if [ "$DB_LIST" = "null" ]; then
          dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases"`
      else
          dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases where database_name $DB_LIST"`
      fi
      if [ $? -ne 0 ]; then
         echo "Not yet able to execute db select query on m_databases..Retry"
         waitFlag=true
      else
         waitFlag=false
      fi
    fi
  done
  return 0
}

echo "************************** running HDB info  ******************"
HDB info

BACKINT_LOC=/usr/sap/$DBSID/SYS/global/hdb/backint
BACKINT_CATALOG=$HANABACKUPPATH/catalog/ACTcatalog.log
BACKINT_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint.par
BACKINT_LOG_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint_log.par
if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
   echo "export DBTYPE=$DBSID" > $BACKINT_PAR_FILE
   echo "export DB_BACKUP_PATH=$HANABACKUPPATH" >> $BACKINT_PAR_FILE
   echo "export BS=10M" >> $BACKINT_PAR_FILE
   if [ "$NODECOUNT" -gt 1 ]; then
      scp_files
   fi
   if [ ! -z $LOGPATH ]; then
      checklogs=`find $LOGPATH/ -name "log_backup*" | wc -l`
      if [ "$checklogs" -eq 0 ]; then
         RECOVERYTIME=''
      fi
   fi
   BACKUPID=`grep ^backup $BACKINT_CATALOG | grep "$DBSID" |grep COMPLETE | awk '{print $5}' |uniq | head -1`
      INCR_COUNT=`ls -l $HANABACKUPPATH/DB_"$DBSID" | grep INCR_DATA_BACKUP| grep -v grep |wc -l`
      DPATH=$BACKINT_LOC/DB_"$DBSID"/

        if [ -z "$LOGPATH" ]; then
         if [ -z "$RECOVERYTIME" ] && [ "$INCR_COUNT" -gt "0" ]; then
            echo "************** Recovering from FULL + INCR Backup only ***************"
            echo "export DB_BACKUP_PATH=$HANABACKUPPATH" > $BACKINT_LOG_PAR_FILE
            echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
            if [ "$NODECOUNT" -gt 1 ]; then
               scp_files
            fi
            ln -s $HANABACKUPPATH/DB_$DBSID $BACKINT_LOC/DB_$DBSID
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '9999-01-01 00:00:00' CLEAR LOG USING LOG PATH ('$DPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
            retval=$?
         elif [ -z "$RECOVERYTIME" ]; then
            echo "************** Recovering from FULL Backup only ***************"
            ln -s $HANABACKUPPATH/DB_$DBSID $BACKINT_LOC/DB_$DBSID
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATA USING BACKUP_ID $BACKUPID USING DATA PATH ('$DPATH') CLEAR LOG"
            retval=$?
         fi
      else
         if [ -z "$RECOVERYTIME" ]; then
            LGPATH=$LOGPATH/backint
            echo "export DBTYPE=$DBSID" > $BACKINT_LOG_PAR_FILE
            echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
            echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
            if [ "$NODECOUNT" -gt 1 ]; then
               scp_files
            fi
            ln -s $HANABACKUPPATH/DB_$DBSID $BACKINT_LOC/DB_$DBSID
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '9999-01-01 00:00:00' CLEAR LOG USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
            retval=$?
         else
            LGPATH=$LOGPATH/backint
            echo "export DBTYPE=$DBSID" > $BACKINT_LOG_PAR_FILE
            echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
            echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
            if [ "$NODECOUNT" -gt 1 ]; then
               scp_files
            fi
            ln -s $HANABACKUPPATH/DB_$DBSID $BACKINT_LOC/DB_$DBSID
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
            retval=$?
         fi
      fi
   waitforhdbnameserver
   rm $BACKINT_LOC/DB_$DBSID
    ################################################# END 1.0 ############

elif [ "$HANAVERSION" = "2.0" ] || [ "$MULTIDB" = "multidb" ]; then
   echo "export DBTYPE=SYSTEMDB" > $BACKINT_PAR_FILE
   echo "export DB_BACKUP_PATH=$HANABACKUPPATH" >> $BACKINT_PAR_FILE
   echo "export BS=10M" >> $BACKINT_PAR_FILE
   if [ "$NODECOUNT" -gt 1 ]; then
      scp_files
   fi 
   if [ ! -z $LOGPATH ]; then
    checklogs=`find $LOGPATH/ -name "log_backup*" | wc -l`
    if [ "$checklogs" -eq 0 ]; then
      RECOVERYTIME=''
    fi
   fi
   if [ "$recoverSystemDbFlag" = "YES" ]; then
   echo "************************** running recoverSys.py  ******************"
   #   BACKUPID=`grep "SYSTEMDB_COMPLETE_DATA_BACKUP" $BACKINT_CATALOG | awk '{print $5}' | uniq`
      BACKUPID=`grep ^backup $BACKINT_CATALOG | grep "SYSTEMDB" |grep COMPLETE | awk '{print $5}' |uniq | head -1`
      INCR_COUNT=`ls -l $HANABACKUPPATH/SYSTEMDB | grep INCR_DATA_BACKUP| grep -v grep |wc -l`
      DPATH=$BACKINT_LOC/SYSTEMDB/
      CATALOGPATH=$HANABACKUPPATH/SYSTEMDB
   
      if [ "$HANAVERSION" = "1.0" ]; then
         DPATH=$BACKINT_LOC/$DBSID
         CATALOGPATH=$HANABACKUPPATH/$DBSID
      else
         DPATH=$BACKINT_LOC/SYSTEMDB/
         CATALOGPATH=$HANABACKUPPATH/SYSTEMDB
      fi
      
      if [ -z "$LOGPATH" ]; then
         if [ -z "$RECOVERYTIME" ]; then
            echo "************** Recovering from FULL Backup only ***************"
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATA USING BACKUP_ID $BACKUPID USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DPATH') CLEAR LOG"
            retval=$?
         fi
      else
         if [ -z "$RECOVERYTIME" ]; then
            if [ "$HANAVERSION" = "1.0" ]; then
               LGPATH=$LOGPATH/$DBSID
               CATALOGPATH=$LGPATH
            else
               LGPATH=$LOGPATH/SYSTEMDB
               CATALOGPATH=$LGPATH
            fi
            echo "export DBTYPE=SYSTEMDB" > $BACKINT_LOG_PAR_FILE
            echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
            echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
            if [ "$NODECOUNT" -gt 1 ]; then
               scp_files
            fi
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '9999-01-01 00:00:00' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
            retval=$?
         else
            LGPATH=$LOGPATH/SYSTEMDB
            CATALOGPATH=$LGPATH
            echo "export DBTYPE=SYSTEMDB" > $BACKINT_LOG_PAR_FILE
            echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
            echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
            if [ "$NODECOUNT" -gt 1 ]; then
               scp_files
            fi
            HDBSettings.sh $PYTHONPATH/recoverSys.py --command="RECOVER DATABASE UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
            retval=$?
         fi
      fi 
   
      if [ "$retval" -eq 0 ]; then
         RECOVERSYS_TRACE_LOC=$SAP_RETRIEVAL_PATH/trace
         JOB_FLAG=`tail -1 $RECOVERSYS_TRACE_LOC/recoverSys.trc | grep -i "failed" |wc -l`
      fi
      if [ "$JOB_FLAG" -ne 0 ]; then
         echo "ERRORMSG: Failed to recover database $DBSID: check customapp-saphana.log for details."
                exit 1
      fi
   
   fi
   
   # get the tenant database name
   
   waitforhdbnameserver
fi 

   SSF_LOCATION=`hdbuserstore list | grep "DATA FILE" | awk -F ":" '{print $2}'`
   SSF_LOCATION=`dirname $SSF_LOCATION`
   tdate=`date +%Y%m%d%H%M%S`
   mkdir -p $SSF_LOCATION/$tdate
   mv $SSF_LOCATION/*.* $SSF_LOCATION/$tdate
   cp $HANABACKUPPATH/SSFS/*.* $SSF_LOCATION/
   
if [ "$HANAVERSION" = "2.0" ]; then
for i in ${dbnames}; do
     TSID=`echo $i |awk -F '"' '{print $2}'`
     if [ $i = "DATABASE_NAME" ]; then
		:
     elif [ $TSID = "SYSTEMDB" ]; then
		:
     else
        echo "export DBTYPE=$TSID" > $BACKINT_PAR_FILE
        echo "export DB_BACKUP_PATH=$HANABACKUPPATH" >> $BACKINT_PAR_FILE
        echo "export BS=10M" >> $BACKINT_PAR_FILE
        if [ "$NODECOUNT" -gt 1 ]; then
           scp_files
        fi
  	TSID=`echo $i |awk -F '"' '{print $2}'`

#        BACKUPID=`grep DB_"$TSID"_COMPLETE_DATA_BACKUP $BACKINT_CATALOG | awk '{print $5}' | uniq`
	BACKUPID=`grep ^backup $BACKINT_CATALOG | grep DB_"$TSID"  |grep COMPLETE | awk '{print $5}' |uniq | head -1`
        INCR_COUNT=`ls -l $HANABACKUPPATH/DB_"$TSID" | grep INCR_DATA_BACKUP| grep -v grep |wc -l`
        DPATH=$BACKINT_LOC/DB_"$TSID"/
        CATALOGPATH=$HANABACKUPPATH/DB_"$TSID"

        if [ -z "$LOGPATH" ]; then
           if [ -z "$RECOVERYTIME" ]; then
              STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
              SQL="RECOVER DATA FOR $TSID USING BACKUP_ID $BACKUPID USING CATALOG PATH ('$CATALOGPATH') USING DATA PATH ('$DPATH') CLEAR LOG"
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $STOPSQL
              retval=$?
              if [ $retval -ne 0 ]; then
                 echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                 exit $retval
              fi
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $SQL
              retval=$?
            fi
        else
           if [ -z "$RECOVERYTIME" ]; then
              LGPATH=$LOGPATH/DB_"$TSID"
              CATALOGPATH=$LGPATH
              echo "export DBTYPE=$TSID" > $BACKINT_LOG_PAR_FILE
              echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
              echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
              if [ "$NODECOUNT" -gt 1 ]; then
                 scp_files
              fi
              STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
              SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '9999-01-01 00:00:00' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $STOPSQL
              retval=$?
              if [ $retval -ne 0 ]; then
                 echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                 exit $retval
              fi                   
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $SQL
              retval=$?
           else
              LGPATH=$LOGPATH/DB_"$TSID"
              CATALOGPATH=$LGPATH
              echo "export DBTYPE=$TSID" > $BACKINT_LOG_PAR_FILE
              echo "export DB_BACKUP_PATH=$LOGPATH" >> $BACKINT_LOG_PAR_FILE
              echo "export BS=10M" >> $BACKINT_LOG_PAR_FILE
              if [ "$NODECOUNT" -gt 1 ]; then
                 scp_files
              fi
              STOPSQL="ALTER SYSTEM STOP DATABASE $TSID"
              SQL="RECOVER DATABASE FOR $TSID UNTIL TIMESTAMP '$RECOVERYTIME' CLEAR LOG USING CATALOG PATH ('$CATALOGPATH') USING LOG PATH ('$LGPATH') USING DATA PATH ('$DPATH') USING BACKUP_ID $BACKUPID"
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $STOPSQL
              retval=$?
              if [ $retval -ne 0 ]; then
                 echo "ERRORMSG: Failed to stop $TSID: check customapp-saphana.log for details."
                 exit $retval
              fi
              /usr/sap/$DBSID/$DBPORT/exe/hdbsql -U $DBUSER -a -j $SQL 
              retval=$?
           fi
        fi
	if [ $retval -ne 0 ]; then
    		echo "ERRORMSG: Failed to recover tenant database $TSID: check customapp-saphana.log for details!"
    		exit $retval
	fi
     fi
done
fi

exit 0

