#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

DBSID=$1
DBUSER=$2
PRETDBUSER=$3
DBPORT=$4
ACT_JOBNAME="$5"
PORT=$6
HANAVERSION=$7
HANABACKUPPATH=$8
DUMPSCHEDULE=$9
USESYSTEMDBKEY=${10}
RUNTENANTPARALLEL=${11}
PREVDBFULLDUMPSAMEDAY=${12}
IGNOREPARTIALFAILURE=${13}


DATE=`date`
tdate=`date +%Y%m%d%H%M%S`
FILE_NAME=/act/touch/hanabackupdblist.log
BACKUP_XML=/act/touch/backupstatus.xml
FAIL_MSG=/act/touch/FAIL_MSG

RUNTENANTPARALLEL=`echo $RUNTENANTPARALLEL |tr '[A-Z]' '[a-z]'`
IGNOREPARTIALFAILURE=`echo $IGNOREPARTIALFAILURE | tr '[A-Z]' '[a-z]'`

echo "************************** DATE:$DATE: start $DBSID database backup ******************"
echo "************************** Copying *.ini files ***************************"

TMP_DBUSER=$DBUSER
#---------------------
function last_catalog_file_extraction() 
{
 DBUSER=$1
 CURR_DB=$2

if [ $CURR_DB = "SYSTEMDB" ]; then
          declare DBTYPE_$CURR_DB='SYSTEMDB'
        else
          declare DBTYPE_$CURR_DB=`echo DB_"$CURR_DB"`
        fi

 length=`expr length "$DBSID"`

 if [[ "$length" != 3 ]]
     then
     echo "ERRORMSG: invalid length of SID - catalog_file_extraction"
     exit 1
  else
     DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
     echo "INFO: The SID is $DBSID - catalog_file_extraction"
  fi

  DBSIDlow=`echo "$DBSID" | awk '{print tolower($0)}'`
  DBSIDupper=`echo "$DBSID" | awk '{print toupper($0)}'`
  HANAuser=`echo "$DBSIDlow""adm"`

  if id "$HANAuser" >/dev/null 2>&1
  then echo "INFO: User $HANAuser exists - catalog_file_extraction"
  else echo "ERRORMSG: User $HANAuser does not exist - catalog_file_extraction"
  exit 1
  fi

  #SAPSYSTEMNAME=`echo $SAPSYSTEMNAME`
  SYSNUMBER=`echo $TINSTANCE`

  globalpath=`env | grep DIR_INSTANCE | cut -d"=" -f2`
  globalpath=`dirname $globalpath`
  globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini

  if [ "$HANAVERSION" = "1.0" ]; then
     COMMENT=
     MULTIDB=`grep "mode" $globalpath/SYS/global/hdb/custom/config/global.ini | grep -iw multidb |awk -F"=" '{print $2}'`
     MULTIDB=`echo $MULTIDB | tr '[A-Z]' '[a-z]'|xargs`
  fi

  PAR_FILE=`cat $globalinipath | grep ^catalog_backup_parameter_file | cut -d'=' -f2-`
  echo "the paramter file is $PAR_FILE"
  if [[ "X$PAR_FILE" == "X" ]]
      then echo "ERRORMSG: The catalog backup parameter file doesn't exist - catalog_file_extraction"
      exit 1
  else echo "INFO: The parameter file is $PAR_FILE - catalog_file_extraction"
  fi

#  if [[ $DBTYPE != "SYSTEMDB" ]]
#  then if [[ $HANAVERSION == "1.0" ]] 
#  then DBUSER=$DBUSER 
#  else DBUSER=$TDBUSER
#  fi
#  fi


#  SQL_CURR_DB="select CURRENT_DATABASE() from dummy"
#  CURR_DB=`hdbsql -U $DBUSER -a -j $SQL_CURR_DB | head -n1`

  if [[ "X$CURR_DB" == "X" ]]
  then echo "ERRORMSG: Cannot identify the current database name - catalog_file_extraction"
  exit 1
  else echo "INFO: The current database is $CURR_DB - catalog_file_extraction"
  fi

  if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
      SQL_BACKUP_ID="select top 1 T1.BACKUP_ID  from M_BACKUP_CATALOG AS T1 INNER JOIN M_BACKUP_CATALOG_FILES AS T2 ON T1.ENTRY_ID = T2.ENTRY_ID where T1.entry_type_name='log backup' and T1.state_name='successful' and T2.source_type_name='catalog' and T1.UTC_START_TIME >= (select top 1 UTC_END_TIME from M_BACKUP_CATALOG where entry_type_name in ('complete data backup', 'differential data backup','incremental data backup') order by UTC_END_TIME desc) order by UTC_END_TIME desc;"
  elif [ "$USESYSTEMDBKEY" = "true" ]; then
  SQL_BACKUP_ID="select top 1 T1.BACKUP_ID  from SYS_DATABASES.M_BACKUP_CATALOG AS T1 INNER JOIN SYS_DATABASES.M_BACKUP_CATALOG_FILES AS T2 ON T1.ENTRY_ID = T2.ENTRY_ID where T1.entry_type_name='log backup' and T1.database_name='$TSID' and T1.state_name='successful' and T2.source_type_name='catalog' and T1.UTC_START_TIME >= (select top 1 UTC_END_TIME from SYS_DATABASES.M_BACKUP_CATALOG where database_name='$TSID' and entry_type_name in ('complete data backup', 'differential data backup','incremental data backup') order by UTC_END_TIME desc) order by UTC_END_TIME desc;"
  else
  #SQL_BACKUP_ID="select top 1 CATALOG_BACKUP_ID from SYS.M_DEV_BACKUP_CATALOG_LOG_ WHERE DATABASE_NAME = CURRENT_DATABASE() and catalog_state_name='successful' order by UTC_END_TIME desc"
  #SQL_BACKUP_ID="select BACKUP_ID from SYS.M_DEV_BACKUP_CATALOG_LOG_ where UTC_START_TIME >= (select top 1 UTC_END_TIME from SYS.M_DEV_BACKUP_CATALOG_DATA_ where  DATABASE_NAME = CURRENT_DATABASE() and catalog_state_name='successful' order by UTC_END_TIME desc) and state_name='successful' and DATABASE_NAME=CURRENT_DATABASE();"
  SQL_BACKUP_ID="select top 1 T1.BACKUP_ID from M_BACKUP_CATALOG AS T1 INNER JOIN M_BACKUP_CATALOG_FILES AS T2 ON T1.ENTRY_ID = T2.ENTRY_ID where T1.entry_type_name='log backup' and T1.state_name='successful' and T2.source_type_name='catalog' and T1.UTC_START_TIME >= (select top 1 UTC_END_TIME from M_BACKUP_CATALOG where entry_type_name in ('complete data backup', 'differential data backup','incremental data backup') order by UTC_END_TIME desc) order by UTC_END_TIME desc;"
  fi

  BACKUP_ID=`hdbsql -U $DBUSER -a -j $SQL_BACKUP_ID | grep -v row | head -n1`

  COUNTER=0
  until [[ X$BACKUP_ID != X ]]
  do
    sleep 5

    BACKUP_ID=`hdbsql -U $DBUSER -a -j $SQL_BACKUP_ID | grep -v row | head -n1`
    echo "INFO -- CATALOG_BACKUP is not yet successfuly backed up please wait"
    COUNTER=$((COUNTER+1))
    if [[ $COUNTER == 60 ]]
    then echo "ERRORMSG: A Successful catalog backup after last data backup cannot be found after 3 minutes - catalog_file_extraction"
    exit 1
    fi
  done


  declare DESTINATION_FILE_$BACKUP_ID=`eval echo $HANABACKUPPATH"/"$"$""DBTYPE_$CURR_DB"/log_backup_0_0_0_0."$BACKUP_ID"`

  if [ -f `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` ]
     then echo "ERRORMSG: The file `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` already exists - catalog_file_extraction"
     exit 1
  fi

  if [ -d `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` ]
     then echo "ERRORMSG: The destination file `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` is a directory - catalog_file_extraction"
     exit 1
  fi

  #DIRECTORY=${DESTINATION_FILE%/*}
  declare DIRECTORY_$BACKUP_ID=$(dirname `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"`)

  #if [ ! -d  $DIRECTORY ]
  if [ ! -d  `eval echo $"$""DIRECTORY_$BACKUP_ID"` ]
     then echo "ERRORMSG: The Directory `eval echo $"$""DIRECTORY_$BACKUP_ID"` doesn't exist - catalog_file_extraction"
     exit 1
  fi

  CHECK_USER_DIR=`eval ls -lrt -d $"$""DIRECTORY_$BACKUP_ID" | awk '{print substr(\$3,0,50)}'`
  CHECK_GRP_DIR=`eval ls -lrt -d $"$""DIRECTORY_$BACKUP_ID" | awk '{print substr(\$4,0,50)}'`
  CHECK_AUTH_DIR=`eval ls -lrt -d $"$""DIRECTORY_$BACKUP_ID" | awk '{print substr(\$1,0,10)}'`

  if [[ $CHECK_USER_DIR == $HANAuser ]]
  then
      EFFECTIVE_RIGHTS=`echo $CHECK_AUTH_DIR | awk '{print substr(\$1,3,1)}'`
  else
      GRP_DIR=`id -ng $HANAuser`
        if [[ $CHECK_GRP_DIR == $GRP_DIR ]]
          then EFFECTIVE_RIGHTS=`echo $CHECK_AUTH_DIR | awk '{print substr(\$1,6,1)}'`
          else
             EFFECTIVE_RIGHTS=`echo $CHECK_AUTH_DIR | awk '{print substr(\$1,9,1)}'`
        fi
  fi

  if [[ $EFFECTIVE_RIGHTS == w ]]
  then echo "INFO: There is writing authorization for  `eval echo $"$""DIRECTORY_$BACKUP_ID"` - catalog_file_extraction"
     else echo "ERRORMSG: There is writing authorizations for `eval echo $"$""DIRECTORY_$BACKUP_ID"` - catalog_file_extraction"
     exit 1
  fi

  if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
     SQL_DEST_EBID="SELECT DESTINATION_PATH, EXTERNAL_BACKUP_ID FROM \"M_BACKUP_CATALOG_FILES\" where backup_ID='$BACKUP_ID'"
  elif [ "$USESYSTEMDBKEY" = "true" ]; then
  SQL_DEST_EBID="SELECT DESTINATION_PATH, EXTERNAL_BACKUP_ID FROM \"SYS_DATABASES\".\"M_BACKUP_CATALOG_FILES\" where backup_ID='$BACKUP_ID'"
  else
  SQL_DEST_EBID="SELECT DESTINATION_PATH, EXTERNAL_BACKUP_ID FROM \"SYS\".\"M_BACKUP_CATALOG_FILES\" where backup_ID='$BACKUP_ID'"
  fi

  DEST_EBID=`hdbsql -U $DBUSER -a -j $SQL_DEST_EBID | head -n1`
  DESTINATION_PATH=`echo $DEST_EBID | cut -d',' -f-1`
  DESTINATION_PATH=`echo "$DESTINATION_PATH" | cut -d "\"" -f 2`

  EXT_BACKUPID=`echo $DEST_EBID | cut -d',' -f2-`


  if [[ "X$DESTINATION_PATH" == "X" ]]
     then echo "ERRORMSG: Cannot find destination path file for catalog backup - catalog_file_extraction"
     exit 1
  else echo "INFO: The destination path file is $DESTINATION_PATH - catalog_file_extraction"
  fi

  if [[ "X$EXT_BACKUPID" == "X" ]]
     then echo "ERRORMSG: Cannot find external backup ID #EBID - catalog_file_extraction"
     exit 1
  else echo "INFO: External backup ID #EBID=$EXT_BACKUPID - catalog_file_extraction"
  fi

  echo "#SOFTWAREID \"backint 1.04\" \"Actifio Backint catalog backup extraction\"" > /tmp/.input_file_cat_extract.$BACKUP_ID
  echo "#NULL \"$DESTINATION_PATH\" \"/tmp/.catalog_backup_$BACKUP_ID\"" >> /tmp/.input_file_cat_extract.$BACKUP_ID

  if [ ! -f "/tmp/.input_file_cat_extract.$BACKUP_ID" ]
     then
     echo "ERRORMSG: The input file /tmp/.input_file_cat_extract.$BACKUP_ID was not generated - catalog_file_extraction"
     exit 1
  fi
  CHECK_INPUT_FILE=`cat /tmp/.input_file_cat_extract.$BACKUP_ID | grep NULL`

  if [[ "X$CHECK_INPUT_FILE" == "X" ]]
     then
     echo "ERRORMSG: Generated input file /tmp/.input_file_cat_extract.$BACKUP_ID is incorrect - catalog_file_extraction"
     exit 1
  else echo "INFO: Input file generated /tmp/.input_file_cat_extract.$BACKUP_ID correctly - catalog_file_extraction"
       echo "INFO: Input file contains :"
       echo ""
       cat /tmp/.input_file_cat_extract.$BACKUP_ID
       echo ""
  fi

  mkfifo /tmp/.catalog_backup_$BACKUP_ID
  touch /tmp/.catalog_backup_$BACKUP_ID.out
  chown $HANAuser:sapsys /tmp/.catalog_backup_$BACKUP_ID
  chown $HANAuser:sapsys /tmp/.catalog_backup_$BACKUP_ID.out

  sleep 1

  dd bs=10M if="/tmp/.catalog_backup_$BACKUP_ID" of=`eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` iflag=fullblock &

  /usr/sap/$DBSIDupper/SYS/global/hdb/opt/hdbbackint -p $PAR_FILE -f restore -i /tmp/.input_file_cat_extract.$BACKUP_ID -o /tmp/.catalog_backup_$BACKUP_ID.out -u $DBSIDupper

  echo "INFO: PIPE FILE is /tmp/.catalog_backup_$BACKUP_ID - catalog_file_extraction"
  echo "INFO: Output FILE is /tmp/.catalog_backup_$BACKUP_ID.out - catalog_file_extraction"
  echo "INFO: Output FILE contains:"
  echo ""
  cat /tmp/.catalog_backup_$BACKUP_ID.out
  echo ""
  echo "INFO: Destination catalog backup file is `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"` - catalog_file_extraction"

  chown $HANAuser:sapsys `eval echo $"$""DESTINATION_FILE_$BACKUP_ID"`
  rm -f /tmp/.catalog_backup_$BACKUP_ID
  rm -f /tmp/.catalog_backup_$BACKUP_ID.out
  rm -f /tmp/.input_file_cat_extract.$BACKUP_ID
}

scp_files()
{
  BACKINT_LOG_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint_log.par
  BACKINT_PAR_FILE=/act/custom_apps/saphana/sapbackint/actifio_backint.par
  for nodename in $(echo $OTHERNODES); do
    echo "Copying backint files to :$nodename"
    scp -pr $BACKINT_PAR_FILE $nodename:$BACKINT_PAR_FILE
    scp -pr $BACKINT_LOG_PAR_FILE $nodename:$BACKINT_LOG_PAR_FILE
    if [ "$?" -gt "0" ]; then
       echo "ERRORMSG: Failed to SCP backint related parameter files to other nodes during backup."
       exit 1
    fi

  done
}

#---------------------

globalpath=`echo $DIR_INSTANCE`
globalpath=`dirname $globalpath`
globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini

CTPATH=`grep -iw ^basepath_logbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`

SSLENFORCE=`grep "sslenforce" $globalpath/SYS/global/hdb/custom/config/global.ini`
SSLENFORCE=`echo $SSLENFORCE | awk -F "=" '{print $2}'|xargs`

if [ "$SSLENFORCE" = "true" ]; then
   hdbsql="hdbsql -e -sslprovider commoncrypto -sslkeystore $SECUDIR/sapsrv.pse -ssltruststore $SECUDIR/sapsrv.pse"
else
   hdbsql="hdbsql"
fi
NODECOUNT=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | wc -l`
MASTERNODE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep -E "active_master =|active_master=" |awk -F"=" '{print $2}' |awk -F":" '{print $1}' |xargs`
OTHERNODES=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | cut -d'=' -f-1 | cut -c7- |grep -v "$MASTERNODE" |xargs`

USESYSTEMDBKEY=`echo $USESYSTEMDBKEY | tr '[A-Z]' '[a-z]'`

if [ ! -d $HANABACKUPPATH/catalog ]; then
   mkdir -p $HANABACKUPPATH/catalog
fi
if [ ! -d $HANABACKUPPATH/INI ]; then
   mkdir -p $HANABACKUPPATH/INI
fi

cp $globalpath/SYS/global/hdb/custom/config/*.ini $HANABACKUPPATH/INI/

BACKINTPATH="$globalpath"/SYS/global/hdb/backint
LOGBACKUPPATH=`grep -iw ^basepath_logbackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
LGBACKUPPATH="DB_BACKUP_PATH="$LOGBACKUPPATH
echo "export $LGBACKUPPATH" > /act/custom_apps/saphana/sapbackint/actifio_backint_log.par
echo "export BS=10M" >> /act/custom_apps/saphana/sapbackint/actifio_backint_log.par

if [ "$NODECOUNT" -gt 1 ]; then
   scp_files
fi

if [ "$?" -gt 0 ]; then
   echo "WARNINGMSG: Failed to backup Configuration files"
fi

if [ -z "$HANAVERSION" ]; then
   echo "ERRORMSG: HANAVERSION is missing. Please Check!"
   exit 1
fi

SSF_LOCATION=`hdbuserstore list | grep "DATA FILE" | awk -F ":" '{print $2}'`
SSF_LOCATION=`dirname $SSF_LOCATION`
if [ ! -z "$SSF_LOCATION" ]; then
   if [ ! -d $HANABACKUPPATH/SSFS ]; then
      mkdir -p $HANABACKUPPATH/SSFS
   fi
   cp $SSF_LOCATION/*.* $HANABACKUPPATH/SSFS/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup Configuration files"
   fi
fi

SSFS_SECURITY_LOCATION=`echo $DIR_INSTANCE`
SSFS_SECURITY_LOCATION=`dirname $SSFS_SECURITY_LOCATION`

if [ ! -z "$SSFS_SECURITY_LOCATION" ]; then
   if [ ! -d $HANABACKUPPATH/SSFS_SECURITY ]; then
      mkdir -p $HANABACKUPPATH/SSFS_SECURITY
   fi
   cp $SSFS_SECURITY_LOCATION/SYS/global/hdb/security/ssfs/*.* $HANABACKUPPATH/SSFS_SECURITY/
   if [ "$?" -gt 0 ]; then
      echo "WARNINGMSG: Failed to backup SSFS Security Configuration files"
   fi

fi

if [ "$DUMPSCHEDULE" = "F" ]; then
    TYPE="F"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
    TYPE="I"
else
    TYPE=${DUMPSCHEDULE:`date +%w`:1}
fi

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j -x "select string_agg(database_name,',')  from m_databases"`
dbnames=`echo $dbnames | sed 's/"//g'`
echo -e "************************* $ACT_JOBNAME DATE:$tdate *************************" >> $FILE_NAME
echo -e "dbnames: $dbnames" > $FILE_NAME

dbnames=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x "select database_name from m_databases"`

retval=$?

if [ -z "$dbnames" ]; then
   echo "ERRORMSG: Failed to get database names. Please check the Userstorekey!"
   exit 1
elif [ "$retval" -gt 0 ]; then
     echo "ERRORMSG: Failed to get database names. Please check the Userstorekey!"
     exit 1
fi

echo "<tenants>" >> $BACKUP_XML

ORIG_TYPE=$TYPE
for i in ${dbnames}; do
TYPE=$ORIG_TYPE
 if [ $i = "DATABASE_NAME" ]; then
   :
  else
    TSID=`echo $i |awk -F '"' '{print $2}'`

	if [ $TSID = "SYSTEMDB" ]; then
          DBTYPE='SYSTEMDB'
          echo "export DBTYPE=$DBTYPE" > /act/custom_apps/saphana/sapbackint/actifio_backint.par
          HBACKUPPATH="DB_BACKUP_PATH="$HANABACKUPPATH
          echo "export $HBACKUPPATH" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
          echo "export BS=10M" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
          if [ "$NODECOUNT" -gt 1 ]; then
             scp_files
          fi
       	else
          DBTYPE=DB_"$TSID"
          echo "export DBTYPE=$DBTYPE" > /act/custom_apps/saphana/sapbackint/actifio_backint.par
      	  HBACKUPPATH="DB_BACKUP_PATH="$HANABACKUPPATH
      	  echo "export $HBACKUPPATH" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
      	  echo "export BS=10M" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
          if [ "$NODECOUNT" -gt 1 ]; then
             scp_files
          fi
    	fi

          #if [ -d "$HANABACKUPPATH/$TSID" ]; then
          if [ -d "$HANABACKUPPATH/$DBTYPE" ]; then
              CHECK_FULL_BKP=`ls $HANABACKUPPATH/$DBTYPE |grep -i "COMPLETE" | wc -l`
                if [ "$CHECK_FULL_BKP" -eq "0" ]; then
                   TYPE="F"
                   if [ "$ORIG_TYPE" != "$TYPE" ]; then
		      echo "WARNINGMSG: Performing Full backup as previous Full backup is not found for $TSID!"	
		   fi
                fi 
          else
              TYPE="F"
          fi
    case $TYPE in
    I)
        HANABACKUPPATHTDB=$BACKINTPATH/$DBTYPE/INCR_DATA_BACKUP
	echo "export BACKUPTYPE=INCREMENTAL" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
        if [ "$NODECOUNT" -gt 1 ]; then
           scp_files
        fi
        if [ $TSID = "SYSTEMDB" ]; then
           BACKUPSQL="BACKUP DATA INCREMENTAL USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
        elif [ "$HANAVERSION" = "1.0" ]; then
             BACKUPSQL="BACKUP DATA INCREMENTAL USING BACKINT ('$HANABACKUPPATHTDB')" 
        else
            if [ "$USESYSTEMDBKEY" = "true" ]; then
               BACKUPSQL="BACKUP DATA INCREMENTAL FOR $TSID USING BACKINT ('$HANABACKUPPATHTD') COMMENT 'ACTIFIO_DUMP_BACKUP'"
            else
               BACKUPSQL="BACKUP DATA INCREMENTAL USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
            fi
        fi
      ;;
    D)
        HANABACKUPPATHTDB=$BACKINTPATH/$DBTYPE/DIFF_DATA_BACKUP
	  echo "export BACKUPTYPE=DIFFERENTIAL" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
          if [ "$NODECOUNT" -gt 1 ]; then
             scp_files
          fi
        if [ $TSID = "SYSTEMDB" ]; then
           BACKUPSQL="BACKUP DATA DIFFERENTIAL USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
        elif [ "$HANAVERSION" = "1.0" ]; then
             BACKUPSQL="BACKUP DATA DIFFERENTIAL USING BACKINT ('$HANABACKUPPATHTDB')"
        else
           if [ "$USESYSTEMDBKEY" = "true" ]; then
              BACKUPSQL="BACKUP DATA FOR $TSID DIFFERENTIAL USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
           else
               BACKUPSQL="BACKUP DATA DIFFERENTIAL USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
           fi
        fi
      ;;
    F)
       if [ ! -z $HANABACKUPPATH/$DBTYPE ]; then
           echo "************ $HANABACKUPPATH/$DBTYPE"
           mkdir -p $HANABACKUPPATH/$DBTYPE
       fi
       if [ $TSID = "SYSTEMDB" ]; then
          HANABACKUPPATHTDB=$BACKINTPATH/$DBTYPE/COMPLETE_DATA_BACKUP
	echo "export BACKUPTYPE=COMPLETE" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
        if [ "$NODECOUNT" -gt 1 ]; then
           scp_files
        fi

          BACKUPSQL="BACKUP DATA USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
       elif [ "$HANAVERSION" = "1.0" ]; then
            HANABACKUPPATHTDB=$BACKINTPATH/$DBTYPE/COMPLETE_DATA_BACKUP
	echo "export BACKUPTYPE=COMPLETE" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
        if [ "$NODECOUNT" -gt 1 ]; then
           scp_files
        fi

            BACKUPSQL="BACKUP DATA USING BACKINT ('$HANABACKUPPATHTDB')"
       else 
           HANABACKUPPATHTDB=$BACKINTPATH/$DBTYPE/COMPLETE_DATA_BACKUP
	echo "export BACKUPTYPE=COMPLETE" >> /act/custom_apps/saphana/sapbackint/actifio_backint.par
        if [ "$NODECOUNT" -gt 1 ]; then
           scp_files
        fi
           if [ "$USESYSTEMDBKEY" = "true" ]; then
               BACKUPSQL="BACKUP DATA FOR $TSID USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
           else
               BACKUPSQL="BACKUP DATA USING BACKINT ('$HANABACKUPPATHTDB') COMMENT 'ACTIFIO_DUMP_BACKUP'"
           fi
      fi
      ;;
    esac

    echo "************************** backingup $TSID database backup ******************"

    # Perform the backup
  echo "************************** $BACKUPSQL ******************"

   # SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"

     if [ "$HANAVERSION" = "1.0" ]; then
        SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
     else
        SQL="SELECT TOP 1 DESTINATION_PATH FROM SYS_DATABASES.M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
     fi

    if [ $TSID = "SYSTEMDB" ]; then
           set -o pipefail
         /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg
         if [ $? -ne 0 ]; then
            errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
	    if [ -z "$errorstring" ]; then
               errorstring=`tail -c100 $FAIL_MSG/"$TSID"_error_msg`
            fi
            for i in ${dbnames}; do
              i=`echo $i | sed 's/"//g'`
              if [ "$i" = "DATABASE_NAME" ]; then
                 :
               elif [ "$i" = "SYSTEMDB" ]; then  
                  echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
               else
                  echo "ERRORMSG:$i Failed to backup $i database,Error:SYSTEMDB Backup Failed. Please check customapp-saphana.log for details!"
               fi
            done
            if [ -z "$DBRPTF" ]; then
               DBRPTF=$TSID
            else
               DBRPTF="$DBRPTF"','"$TSID"
            fi
            echo -e "faileddbnames:$TSID" >> $FILE_NAME
            echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
            echo "</tenants> " >> $BACKUP_XML
            echo -e "************************* $ACT_JOBNAME END *************************" >> $FILE_NAME
            exit 1
         else
            if [ -z "$DBRPTSUCCESS" ]; then
               DBRPTSUCCESS=$TSID
            else
               DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
            fi
            echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML
         fi
         #BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -a -j $SQL | head -n1 | sed s/\"//g`
	last_catalog_file_extraction $DBUSER $TSID 

    else
       if [ ! -z $TMP_DBUSER ]; then
         DBUSER=$TMP_DBUSER
       fi
       TDBSTATUS=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $DBUSER -x -a "select active_status from m_databases where database_name='$TSID'"`
       if [ $TDBSTATUS = '"YES"' ]; then
          if [ "$USESYSTEMDBKEY" = "true" ]; then
              TDBUSER=$DBUSER
          else
              TDBUSER=$PRETDBUSER$TSID
              SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
          fi
           set -o pipefail
            if [ "$RUNTENANTPARALLEL" = "true" ]; then
                /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg &
               processid=$!
               if [ -z "$processidlist" ]; then
                  processidlist=$TSID':'$processid 
               else
                  processidlist=$processidlist','$TSID':'$processid
               fi
            else
                /usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j "$BACKUPSQL" 2>&1 | tee $FAIL_MSG/"$TSID"_error_msg
                if [ $? -ne 0 ]; then
                   errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
                   if [ -z "$errorstring" ]; then
                      errorstring=`tail -c100 $FAIL_MSG/"$TSID"_error_msg`
                   fi
                   echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
                   echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML
                   if [ -z $DBRPTF ]; then
                      DBRPTF=$TSID
                   else
                       DBRPTF="$DBRPTF"','"$TSID"
                   fi             
                   if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
                      echo "</tenants>" >> $BACKUP_XML
                      exit 1
                   fi
                else
            last_catalog_file_extraction $TDBUSER $TSID
                    echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML
                    if [ -z "$DBRPTSUCCESS" ]; then
                       DBRPTSUCCESS=$TSID
                    else
                       DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
                    fi
                fi
            fi
       else 
         echo "WARNINGMSG: $TSID is not active. Current status is: $TDBSTATUS,  Please Check!"
         echo -e "\t<tenant name =\"$TSID\" status=\"skipped\" error=\"Tenant is inactive\"/>" >> $BACKUP_XML
               if [ -z $DBRPTS ]; then
                  DBRPTS=$TSID
              else
                  DBRPTS="$DBRPTS"','"$TSID"
              fi
       fi
    fi
  fi
  echo "************************** end $TSID database backup ******************"
done

######## Check TSID Backup status when started Parallely ############

for x in $(echo $processidlist | tr ',' ' ')
do
   pid=`echo $x | awk -F":" '{print $2}'`
   TSID=`echo $x | awk -F":" '{print $1}'`
   wait $pid
   retval=$?
   if [ $retval -ne 0 ]; then
      errorstring=`cat $FAIL_MSG/"$TSID"_error_msg | xargs |awk -F"rc=" '{print $2}'`
      if [ -z "$errorstring" ]; then
         errorstring=`tail -c100 $FAIL_MSG/"$TSID"_error_msg`
      fi
      echo "ERRORMSG:$TSID Failed to backup $TSID database,Error:$errorstring. Please check customapp-saphana.log for details!"
      echo -e "\t<tenant name =\"$TSID\" status=\"failed\" error=\"$errorstring\"/>" >> $BACKUP_XML

      if [ -z $DBRPTF ]; then
         DBRPTF=$TSID
      else
         DBRPTF="$DBRPTF"','"$TSID"
      fi
      if [ "$IGNOREPARTIALFAILURE" = "false" ]; then
         echo "</tenants>" >> $BACKUP_XML
         exit 1
      fi

   else
      ###### If Success copy the catalog 
      if [ "$HANAVERSION" = "1.0" ] && [ "$MULTIDB" != "multidb" ]; then
          SQL="SELECT TOP 1 DESTINATION_PATH FROM M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
      else
          SQL="SELECT TOP 1 DESTINATION_PATH FROM SYS_DATABASES.M_BACKUP_CATALOG_FILES WHERE DATABASE_NAME='$TSID' and SOURCE_TYPE_NAME = 'catalog' ORDER BY BACKUP_ID desc"
      fi
      #BCATALOG=`/usr/sap/$DBSID/$DBPORT/exe/$hdbsql -U $TDBUSER -a -j $SQL | head -n1 | sed s/\"//g`
      #cp $BCATALOG $HANABACKUPPATH/$TSID
      if [ "$USESYSTEMDBKEY" = "true" ]; then
         TDBUSER=$DBUSER
      else
         TDBUSER=$PRETDBUSER$TSID
     fi

last_catalog_file_extraction $TDBUSER $TSID
      echo -e "\t<tenant name =\"$TSID\" status=\"success\"/>" >> $BACKUP_XML

      if [ -z "$DBRPTSUCCESS" ]; then
         DBRPTSUCCESS=$TSID
      else
         DBRPTSUCCESS="$DBRPTSUCCESS"','"$TSID"
      fi
   fi

done ##### Process Check

   echo -e "backedupdbnames:$DBRPTSUCCESS" >> $FILE_NAME
   echo -e "faileddbnames:$DBRPTF" >> $FILE_NAME
   echo -e "skippeddbnames:$DBRPTS" >> $FILE_NAME
cat $FILE_NAME
if [ ! -z "$DBRPTF" ] && [ "$HANAVERSION" = "1.0" ]; then
   echo "ERRORMSG:$TSID Failed to backup $TSID,Error:$errorstring. Please check customapp-saphana.log for details!"
   exit 1
fi
if [ -d "$FAIL_MSG" ]; then
   rm -f $FAIL_MSG/*
fi
echo "************************** end database backup ******************"
echo "</tenants>" >> $BACKUP_XML
exit 0

