#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#


set -x

DBSID=$1
ACT_MOUNT_POINT=$2
DEVICE_NAMES="$3"


conffile=/act/custom_apps/saphana/lvm_migrate/ACT_DB_pvmove.conf

if [ "$#" -lt 3 ]; then
   if [ -f $conffile ]; then
      source $conffile
      if [ -z "$DBSID" ] || [ -z "$ACT_MOUNT_POINT" ] || [ -z "$DEVICE_NAMES" ]; then
         echo "ERRORMSG: Required input parameters are Missing, Please Check the usage for details!"
         echo "./ACT_DB_pvmove.sh <DBSID> <TARGET MOUNT POINT> <SOURCE DEVICE NAMES separated by ,> "
         exit 1
      fi
   else
      echo "ERRORMSG: Required input parameters are Missing, Please Check the usage for details!"
      echo "./ACT_DB_pvmove.sh <DBSID> <TARGET MOUNT POINT> <SOURCE DEVICE NAMES separated by ,> "
      exit 1
   fi
fi
#---------------------------
# Function to Cleanup Files
#---------------------------

cleanup_file() {
  file_name=$1
  if [ -f $file_name ]; then
    rm -f $file_name
  fi
}


FILE_LOC=/act/touch

if [ ! -d $FILE_LOC ]; then
  mkdir -p $FILE_LOC
  chmod 755 $FILE_LOC
fi

if [ ! -f $FILE_LOC/.hana_migrate_"$DBSID".conf ]; then
   echo -e "MIGRATED=NO" > $FILE_LOC/.hana_migrate_"$DBSID".conf
fi

ACTFIO_DEVICE_DETAILS=$FILE_LOC/act_hanadevice_details_"$DBSID".txt
ACTFIO_DEVICE_DETAILS_TMP=$FILE_LOC/act_hanadevice_details_tmp_"$DBSID".txt
cleanup_file $ACTFIO_DEVICE_DETAILS

#### Generate file with PV, LV and Size
ACT_MOUNT_POINT=`echo $ACT_MOUNT_POINT | sed 's/\///g'`
for x in $(vgs | grep "$ACT_MOUNT_POINT"|grep -iv TransactionLog| awk '{print $1}'); do
    echo $x
    DEVICE_PV_NAME=`vgdisplay -v $x  | grep -w "PV Name" | awk '{print $3}'`
    DEVICE_LV_NAME=`vgdisplay -v $x  | grep -w "LV Name" | awk '{print $3}'`
    LVSIZE=`lsblk -n -b -i -o SIZE /dev/mapper/"$x"-act_staging_vol`
    echo -e "$x:$DEVICE_PV_NAME:$DEVICE_LV_NAME:$LVSIZE" >> $ACTFIO_DEVICE_DETAILS
done

#### Check the device counts and compare
 
LOCAL_DEVICE_COUNT=`echo "$DEVICE_NAMES" | tr ',' ' ' | wc -w`
ACTIFIO_DEVICE_COUNT=`cat $ACTFIO_DEVICE_DETAILS|wc -l`

if [ "$LOCAL_DEVICE_COUNT" -ne "$ACTIFIO_DEVICE_COUNT" ]; then
   echo "ERRORMSG: Number of devices provided does not match with number of Actifio devices mounted!"
   exit 1
fi

#### Sort the ACTIFIO devices based on Size

cat $ACTFIO_DEVICE_DETAILS | sort -n -t: -k 4 > $ACTFIO_DEVICE_DETAILS_TMP
cleanup_file $ACTFIO_DEVICE_DETAILS
LOCAL_DEVICE_DETAILS=$FILE_LOC/local_devices_"$DBSID".txt


#### Get local device details with size

DEVICE_NAMES_TMP=`echo $DEVICE_NAMES | tr ',' ' '`
lsblk -b -n -i -o NAME,SIZE $DEVICE_NAMES_TMP | sort -n -k 2 > $LOCAL_DEVICE_DETAILS
retval=$?
 if [ "$retval" -gt 0 ]; then
     echo "ERRORMSG: Failed to Execute LSBLK Command!. Please check for errors!"
     exit 1
 fi

  IFS=","
for k in ${DEVICE_NAMES}
do
  name=`echo $k | rev | awk -F "/" '{print $1}' | rev`
  sed -i "s|$name|$k|g" $LOCAL_DEVICE_DETAILS
done

#### Update the file with Local device name and Size details

while read device
do
  line=$(head -n 1 $ACTFIO_DEVICE_DETAILS_TMP)
  sed -i '1d' $ACTFIO_DEVICE_DETAILS_TMP
  local_device_name=`echo $device |awk '{print $1}'`
  local_device_name_size=`echo $device |awk '{print $2}'`
  echo "$line:$local_device_name:$local_device_name_size" >> $ACTFIO_DEVICE_DETAILS  
  pvcreate $local_device_name
  retval=$?

  if [ "$retval" -gt 0 ]; then
     echo "ERRORMSG: Failed to Execute PVCREATE Command!. Please check for errors!"
     exit 1
  fi
done < $LOCAL_DEVICE_DETAILS

  cleanup_file $LOCAL_DEVICE_DETAILS
   cleanup_file $ACTFIO_DEVICE_DETAILS_TMP 
   
   echo " **********   Starting VGEXTEND  **********"
   
   while read line
   do
      ACT_VG=`echo $line |awk -F ":" '{print $1}'`
      LOCAL_DEVICE=`echo $line |awk -F ":" '{print $5}'`
      ACT_DEVICE_SIZE=`echo $line |awk -F ":" '{print $4}'`
      LOCAL_DEVICE_SIZE=`echo $line |awk -F ":" '{print $6}'`
      if [ "$LOCAL_DEVICE_SIZE" -lt "$ACT_DEVICE_SIZE" ]; then
         echo "ERRORMSG: The device $LOCAL_DEVICE is less than the required device size, Please check!"
         exit 1
      fi
      vgextend $ACT_VG $LOCAL_DEVICE
      retval=$?

      if [ "$retval" -gt 0 ]; then
         echo "ERRORMSG: Failed to Execute VGEXTEND Command!. Please check for errors!"
         exit 1
      fi
   done < $ACTFIO_DEVICE_DETAILS


   echo "**********   Starting Storage Migration  ********** "
   
   while read line
   do 
      ACT_STG_VOLUME=`echo $line |awk -F ":" '{print $3}'`
      ACT_DEVICE=`echo $line |awk -F ":" '{print $2}'`
      LOCAL_DEVICE=`echo $line |awk -F ":" '{print $5}'`
      pvmove -n $ACT_STG_VOLUME --atomic -A y -b -q $ACT_DEVICE $LOCAL_DEVICE
      retval=$?

      if [ "$retval" -gt 0 ]; then
         echo "ERRORMSG: Failed to Execute PVCREATE Command!. Please check for errors!"
         exit 1
      fi

   done < $ACTFIO_DEVICE_DETAILS 
   pvmove   

   echo " ************* Running VGREDUCE Command ************"
  
   while read line
   do
      ACT_VG=`echo $line |awk -F ":" '{print $1}'`
      ACT_DEVICE=`echo $line |awk -F ":" '{print $2}'`
      vgreduce $ACT_VG $ACT_DEVICE
      retval=$?

      if [ "$retval" -gt 0 ]; then
         echo "ERRORMSG: Failed to Execute VGREDUCE Command!. Please check for errors!"
         exit 1
      fi
   done < $ACTFIO_DEVICE_DETAILS  
  
   echo "************ Running PVREMOVE Command ***********"

   while read line
   do
     ACT_DEVICE=`echo $line |awk -F ":" '{print $2}'`
     pvremove $ACT_DEVICE
     retval=$?
   done < $ACTFIO_DEVICE_DETAILS
   
   if [ "$retval" -eq 0 ]; then
      echo -e "MIGRATED=YES" > $FILE_LOC/.hana_migrate_"$DBSID".conf
   else
      echo "ERRORMSG: Failed to remove the devices. Please check for errors!"
      exit 1
   fi
   
   cleanup_file $ACTFIO_DEVICE_DETAILS



echo "********* LVM Migration completed successfully *************"

exit 0


