#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
dbsid=$1
PROD_DATAVOL=$2
PROD_LOGVOL=$3

dbsid=`echo $dbsid | tr '[A-Z]' '[a-z]'`
CONTROL_FILE=/act/act_hana_recovery.lock

if [ -f "$CONTROL_FILE" ]; then
   rm -f $CONTROL_FILE
fi

if [ "$#" -lt 3 ]; then
   echo "Usage:/act/custom_apps/saphana/lvm_migrate/ACT_HANADB_lvm_migrate.sh <DBSID> <PROD DATA VOLUME> <PROD LOG VOLUME>"
   exit 1
fi

remove_file()
{
   fname=$1
   if [ -f $fname ]; then
      rm -f $fname
   fi
}

if [ -f /act/touch/"$dbsid"_mount_params ]; then
   source /act/touch/"$dbsid"_mount_params
   cp /act/touch/"$dbsid"_mount_params /act/touch/"$dbsid"_mount_params_inprogress
   remove_file /act/touch/"$dbsid"_mount_params
elif [ -f /act/touch/"$dbsid"_mount_params_inprogress ]; then
   source /act/touch/"$dbsid"_mount_params_inprogress
else
   NDBSID=`echo $dbsid | tr '[a-z]' '[A-Z]'`
   if [ -f /act/touch/"$NDBSID"_mount_params ]; then
      source /act/touch/"$NDBSID"_mount_params
      cp /act/touch/"$NDBSID"_mount_params /act/touch/"$NDBSID"_mount_params_inprogress
      remove_file /act/touch/"$NDBSID"_mount_params
   elif [ -f /act/touch/"$NDBSID"_mount_params_inprogress ]; then
      source /act/touch/"$NDBSID"_mount_params_inprogress
   else
      echo "ERRORMSG: Parameter file does not Exist. Please check!"
      call_updatecontrolfile "ERRORMSG: Parameter file does not Exist. Please check!"
      exit 1
   fi
fi

parameter_file=/act/touch/"$dbsid"_mount_params
ACT_DATAVOL=`df -P $DATAVOLPATH | awk 'NR==2{print $NF}'`
ACT_LOGVOL=`df -P $LOGVOLPATH | awk 'NR==2{print $NF}'`
devices_processed=
MERGE_LIST=
SKIPPED_MERGE_LIST=

create_pfile()
{
  echo -e "DATAVOLPATH=$DATAVOLPATH" > "$parameter_file"
  echo -e "LOGVOLPATH=$LOGVOLPATH" >> "$parameter_file"
  echo -e "OLDDBSID=$OLDDBSID" >> "$parameter_file"
}

if [ -z $dbsid ]; then
   echo "ERRORMSG: Pre Mount: Database SID is not set!"
   call_updatecontrolfile "ERRORMSG: Pre Mount: Database SID is not set!"
   create_pfile
   exit 1
fi

dbsidadm=$dbsid"adm"
export dbsidadm


stop_application() 
{

   ####### Stopping Application ########

   echo "****** Stopping database: $dbsid ******"

   su - $dbsidadm -c "/act/custom_apps/saphana/clone/ACT_HANADB_saphana_stop.sh $dbsid"
   retval=$?

   if [ $retval -ne 0 ]; then
       echo "ERRORMSG: Pre Mount: Failed to stop HANA DB services. Please check!"
       call_updatecontrolfile "ERRORMSG: Pre Mount: Failed to stop HANA DB services. Please check!"
       create_pfile
       remove_file /act/touch/globaliniupdate
       exit $retval
   fi

}

check_device_size()
{
  ACT_DEVICE=$1
  PROD_DEVICE=$2
  
   ### Validate Size
  ACT_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_DEVICE`
  PROD_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_DEVICE`

  DIFF_IN_SIZE=$(($PROD_DEVICE_SIZE-$ACT_DEVICE_SIZE))

  if [ "$PROD_DEVICE_SIZE" -lt "$ACT_DEVICE_SIZE" ]; then
     echo "ERRORMSG: The device $PROD_DEVICE does not have enough free space, Please check!"
     call_updatecontrolfile "ERRORMSG: The device $PROD_DEVICE does not have enough free space, Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi
}


vgmerge_fun()
{
  ACT_DEVICE=$1
  PROD_DEVICE=$2

  PROD_VG_NAME=`lvdisplay $PROD_DEVICE | grep -w "VG Name" | awk '{print $3}'`
  PROD_LV_PATH=`lvdisplay $PROD_DEVICE | grep -w "LV Path" | awk '{print $3}'`
  PROD_PV_NAME=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $3}'`
  #PROD_LV_NAME=`vgdisplay -v $PROD_VG_NAME | grep -w "LV Name" | awk '{print $3}'`
  PROD_LV_NAME=`lvs --noheadings -o lv_name $PROD_DEVICE|xargs `
  PROD_MOUNT_PT=`df -kh | grep -w "$PROD_DEVICE" | rev |awk '{print $1}' | rev`  
  
  ACT_VG_NAME=`lvdisplay -v $ACT_DEVICE | grep -w "VG Name" | awk '{print $3}'`
  ACT_PV_NAME=`vgdisplay -v $ACT_VG_NAME  | grep -w "PV Name" | awk '{print $3}'`
  ACT_LV_NAME=`vgdisplay -v $ACT_VG_NAME  | grep -w "LV Name" | awk '{print $3}'`
  ACTMNT=`df -kh | grep -w "$ACT_DEVICE" | rev | awk '{print $1}'|rev`

  if [ -z "$PROD_VG_NAME" ]; then
     echo "ERRORMSG: Failed to get LVNAME for Prod Device $PROD_DEVICE!"
     call_updatecontrolfile "ERRORMSG: Failed to get LVNAME for Prod Device $PROD_DEVICE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi
  
  if [ -z "$ACT_VG_NAME" ]; then
     echo "ERRORMSG: Failed to get LVNAME for ACT Device $ACT_DEVICE!"
     call_updatecontrolfile "ERRORMSG: Failed to get LVNAME for ACT Device $ACT_DEVICE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi

  if [ ! -z "$PROD_LV_NAME" ]; then
     completepath_prodflag=`echo $PROD_LV_NAME | grep "/dev/" | wc -l `
     if [ "$completepath_prodflag" -eq 0 ]; then
        PROD_LV_PATH="/dev/$PROD_VG_NAME/$PROD_LV_NAME"
     else
        PROD_LV_PATH="$PROD_LV_NAME"
     fi
  else
     echo "ERRORMSG: Failed to get LV Path for Prod Device $PROD_DEVICE!"
     call_updatecontrolfile "ERRORMSG: Failed to get LV Path for Prod Device $PROD_DEVICE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi


  if [ ! -z "$PROD_MOUNT_PT" ]; then
     umount $PROD_MOUNT_PT
  fi
  umount $ACTMNT
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO UNMOUNT ACT DEVICES. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO UNMOUNT ACT DEVICES. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi 
  call_updatecontrolfile "Starting VGCHANGE...!"
  vgchange -an $ACT_VG_NAME
  
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGCHANGE. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO PERFORM VGCHANGE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi

  call_updatecontrolfile "Starting VGMERGE...!" 
  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGMERGE. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO PERFORM VGMERGE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi
  
   lvname=
   lvpath=
  #lvpath=`vgdisplay -v $PROD_VG_NAME | grep "LV Path" | grep "act_staging_vol" |awk '{print $3}'`
   lvname=`vgdisplay -v $PROD_VG_NAME | grep "LV Name" | grep "act_staging_vol" |awk '{print $3}'`

  if [ ! -z "$lvname" ]; then
     completepath=`echo $lvname | grep "/dev/" | wc -l`
     if [ "$completepath" -eq 0 ]; then
        lvpath="/dev/$PROD_VG_NAME/$lvname"
     else
        lvpath=$lvname
     fi
  else
     echo "ERRORMSG: Failed to get LVPATH after VGMERGE."
     call_updatecontrolfile "ERRORMSG: Failed to get LVPATH after VGMERGE Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi

  lvchange -ay $lvpath
  if [ ! -z "$PROD_MOUNT_PT" ]; then
     umount $PROD_MOUNT_PT
  fi
  call_updatecontrolfile "Starting LVREMOVE...!"
  lvremove -f $PROD_LV_PATH
    if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED to PERFORM LVREMOVE. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED to PERFORM LVREMOVE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
    fi
  call_updatecontrolfile "Starting LVRENAME...!"
  lvrename $lvpath $PROD_LV_PATH
    if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM LVRENAME. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO PERFORM LVRENAME. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
    fi
  if [ ! -z "$PROD_MOUNT_PT" ]; then
     mount $PROD_DEVICE $PROD_MOUNT_PT
     PROD_MNT=TRUE
     sleep 15
     MOUNTED=`df -kh | grep "$PROD_MOUNT_PT" | wc -l`
     if [ "$MOUNTED" -eq 0 ]; then
        systemctl daemon-reload
        sleep 30
        mount $PROD_DEVICE $PROD_MOUNT_PT
     fi
  else
     mount $PROD_DEVICE $ACTMNT
     PROD_MNT=FALSE
  fi
}

generate_pvmove_list_fun() 
{
  PROD_DEVICE=$1
  ACT_VG_NAME=$2

  PROD_PV_NAME=`lvs -o devices $PROD_DEVICE |grep -v Devices | awk -F"(" '{print $1}'|sort|uniq |xargs`
  ACT_PV_NAME=`vgdisplay -v $ACT_VG_NAME  | grep -w "PV Name" | awk '{print $3}' | xargs`

  echo "PROD PV'S: $PROD_PV_NAME"
  echo "ACT PV'S: $ACT_PV_NAME"
  proddevice_count=`echo $PROD_PV_NAME|wc -w`
  actdevice_count=`echo $ACT_PV_NAME|wc -w`

}

pvmovedata_fun()
{
      PROD_PV_NAME_DATA=$1
      ACT_PV_NAME_DATA=$2
      if [ ! -z "$PROD_PV_NAME_DATA" ] && [ ! -z "$ACT_PV_NAME_DATA" ]; then
         for actdevicedata in $(echo $ACT_PV_NAME_DATA)
           do
             echo "Initiating PVMOVE for $actdevicedata...!"
             pvmove --atomic -A y -b -q $actdevicedata $PROD_PV_NAME_DATA
             retval=$?
             if [ "$retval" -gt 0 ] && [ "$retval" -ne 5 ]; then
                echo "ERRORMSG:Failed to initiate PVMOVE. Please check the error message"
                exit 1
             fi
             pvmove
             while true
             do
                    proddevicelistdata=`echo $PROD_PV_NAME_DATA | sed 's/ /|/g'`
                    isdonedata=`lvs -a -o +devices |grep pvmove| grep -E "$proddevicelistdata" |wc -l`
                    if [ "$isdonedata" -gt 0 ]; then
                       echo "PVMOVE is still running for $proddevice, retrying!"
                       sleep 300
                    else
                       echo "PVMOVE completed for $actdevicedata"
                       break;
                    fi
             done
         done  
      fi
}

pvmovelog_fun()
{
      PROD_PV_NAME_LOG=$1
      ACT_PV_NAME_LOG=$2
      if [ ! -z "$PROD_PV_NAME_LOG" ] && [ ! -z "$ACT_PV_NAME_LOG" ]; then
         for actdevicelog in $(echo $ACT_PV_NAME_LOG)
           do
             echo "Initiating PVMOVE for $actdevicelog...!"
             pvmove --atomic -A y -b -q $actdevicelog $PROD_PV_NAME_LOG
             retvallog=$?
             if [ "$retvallog" -gt 0 ] && [ "$retvallog" -ne 5 ]; then
                echo "ERRORMSG:Failed to initiate PVMOVE. Please check the error message"
                exit 1
             fi
             pvmove
             while true
             do
                    proddevicelistlog=`echo $PROD_PV_NAME_LOG | sed 's/ /|/g'`
                    isdonelog=`lvs -a -o +devices |grep pvmove| grep -E "$proddevicelistlog" |wc -l`
                    if [ "$isdonelog" -gt 0 ]; then
                       echo "PVMOVE is still running for $actdevicelog, retrying!"
                       sleep 300
                    else
                       echo "PVMOVE completed for $actdevicelog"
                       break;
                    fi
             done
         done
      fi
}

vgreduce_fun() 
{
    ACT_PV_NAME=$1
    PROD_DEVICE=$2
    PROD_VG_NAME=`lvdisplay $PROD_DEVICE | grep -w "VG Name" | awk '{print $3}'`
   
   vgreduce $PROD_VG_NAME $ACT_PV_NAME
   if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGREDUCE. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO PERFORM VGREDUCE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
  fi
   pvremove $ACT_PV_NAME
   if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. Please check!"
     call_updatecontrolfile "ERRORMSG: FAILED TO PERFORM PVREMOVE. Please check!"
     create_pfile
     remove_file /act/touch/globaliniupdate
     exit 1
   fi
}

update_globalini()
{
   DATAPATH=$1
   LOGPATH=$2
   tdate=`date +"%m%d%Y"`
   dbuser="$dbsid"adm
   globalpath=`su - $dbuser -c 'echo $DIR_INSTANCE'`
   globalpath=`dirname $globalpath`
   globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini
   DATAVOL=`grep -i basepath_datavolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
   LOGVOL=`grep -i basepath_logvolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
   DATABACKUPPATH=`grep -i basepath_databackup $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'| head -1`
   if [ ! -z "$DATABACKUPPATH" ]; then
      DATABACKUPPATH=`dirname $DATABACKUPPATH`
   fi

   if [ "$DATAPATH" != "$DATAVOL" ]; then
       if [ ! -f "$globalinipath".orig ]; then
           cp $globalinipath $globalinipath'.'orig
       fi
       cp $globalinipath $globalinipath'.'$tdate
       chown $dbuser:sapsys $globalinipath'.'$tdate
       sed -i '/basepath_datavolumes/d' $globalinipath
       sed -i '/basepath_logvolumes/d' $globalinipath
       sed -i "/\bpersistence\b/a basepath_datavolumes = $DATAPATH" $globalinipath
       sed -i "/\bpersistence\b/a basepath_logvolumes = $LOGPATH" $globalinipath
       BASE_DATAPATH=$(echo "$DATAPATH" |cut -d "/" -f2)
       BASE_LOGPATH=$(echo "$LOGPATH" |cut -d "/" -f2)
       if [ ! -z "$BASE_DATAPATH" ]; then
          chown -R $dbuser:sapsys /$BASE_DATAPATH
       fi
       if [ "$BASE_DATAPATH" != "$BASE_LOGPATH" ]; then
          if [ ! -z "$BASE_LOGPATH" ]; then
             chown -R $dbuser:sapsys /$BASE_LOGPATH    
          fi
       fi       
       if [ ! -z "$LOG_BACKUP_PATH" ]; then
          if [ ! -d "$LOG_BACKUP_PATH" ]; then
             mkdir -p $LOG_BACKUP_PATH
          fi
          BASE_LOGBKPPATH=$(echo "$LOG_BACKUP_PATH" |cut -d "/" -f2)
          if [ ! -z "$BASE_LOGBKPPATH" ]; then
             if [ "$MOUNTPOINTROOT" = "/$BASE_LOGBKPPATH" ]; then
                chown -R $dbuser:sapsys /$BASE_LOGBKPPATH 
             fi
          fi
          sed -i "/basepath_logbackup/d" $globalinipath
          sed -i "/basepath_catalogbackup/d" $globalinipath
          sed -i "/\bpersistence\b/a basepath_catalogbackup = $LOG_BACKUP_PATH" $globalinipath
          sed -i "/\bpersistence\b/a basepath_logbackup = $LOG_BACKUP_PATH" $globalinipath
       else
          if [ ! -z "$DATABACKUPPATH" ]; then
             LOGBACKUPPATH=$DATABACKUPPATH/log
             CATLOGBACKUPPATH=$LOGBACKUPPATH
             if [ ! -d "$LOGBACKUPPATH" ]; then
                 mkdir -p $LOGBACKUPPATH
                 chown $dbuser:sapsys $LOGBACKUPPATH
             fi          
             BASE_LOGBACKUPPATH=$(echo "$LOGBACKUPPATH" |cut -d "/" -f2)
             if [ ! -z "$BASE_LOGBACKUPPATH" ]; then
                if [ "$MOUNTPOINTROOT" = "/$BASE_LOGBACKUPPATH" ]; then
                   chown -R $dbuser:sapsys /$BASE_LOGBACKUPPATH
                fi
             fi
             sed -i "/basepath_logbackup/d" $globalinipath
             sed -i "/basepath_catalogbackup/d" $globalinipath
             sed -i "/\bpersistence\b/a basepath_catalogbackup = $CATLOGBACKUPPATH" $globalinipath
             sed -i "/\bpersistence\b/a basepath_logbackup = $LOGBACKUPPATH" $globalinipath
          else
             LOG_BKP_PATH_FLAG=`echo $LOGVOL | grep $LOGPATH`
             if [ -z "$LOG_BKP_PATH_FLAG" ]; then
  
                 LOGBACKUPPATH=$LOGPATH/logbackup
                 sed -i "/basepath_logbackup/d" $globalinipath
                 sed -i "/basepath_catalogbackup/d" $globalinipath
                 CATLOGBACKUPPATH=$LOGBACKUPPATH
                 if [ ! -d "$LOGBACKUPPATH" ]; then
                    mkdir -p $LOGBACKUPPATH
                    chown $dbuser:sapsys $LOGBACKUPPATH
                 fi
                 BASE_LOGBACKUPPATH=$(echo "$LOGBACKUPPATH" |cut -d "/" -f2)
                 if [ ! -z "$BASE_LOGBACKUPPATH" ]; then
                    if [ "$MOUNTPOINTROOT" = "/$BASE_LOGBACKUPPATH" ]; then
                       chown -R $dbuser:sapsys /$BASE_LOGBACKUPPATH
                    fi
                 fi
                 sed -i "/\bpersistence\b/a basepath_catalogbackup = $CATLOGBACKUPPATH" $globalinipath
                 sed -i "/\bpersistence\b/a basepath_logbackup = $LOGBACKUPPATH" $globalinipath
             else 
                 LOGBACKUPPATH=$LOGVOL/logbackup
                 sed -i "/basepath_logbackup/d" $globalinipath
                 sed -i "/basepath_catalogbackup/d" $globalinipath
                 CATLOGBACKUPPATH=$LOGBACKUPPATH
                 if [ ! -d "$LOGBACKUPPATH" ]; then
                    mkdir -p $LOGBACKUPPATH
                    chown $dbuser:sapsys $LOGBACKUPPATH
                 fi
                 BASE_LOGBACKUPPATH=$(echo "$LOGBACKUPPATH" |cut -d "/" -f2)
                 if [ ! -z "$BASE_LOGBACKUPPATH" ]; then
                    if [ "$MOUNTPOINTROOT" = "/$BASE_LOGBACKUPPATH" ]; then
                       chown -R $dbuser:sapsys /$BASE_LOGBACKUPPATH
                    fi
                 fi
                 sed -i "/\bpersistence\b/a basepath_catalogbackup = $CATLOGBACKUPPATH" $globalinipath
                 sed -i "/\bpersistence\b/a basepath_logbackup = $LOGBACKUPPATH" $globalinipath
             fi 
          
          fi
       fi
   fi
}

call_stop_tracking()
{
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}

call_start_tracking()
{
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}

call_commentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -e "/$devicename/ s/^#*/#/" -i /etc/fstab
}

call_uncommentfstab()
{
  devicename=$1
  devicename=`echo $devicename |sed 's|\/|\\\/|g'`
  sed -i "/$devicename/s/^#//g" /etc/fstab
  
}

cleanup_logbackup()
{
   dbuser="$dbsid"adm
   globalpath=`su - $dbuser -c 'echo $DIR_INSTANCE'`
   globalpath=`dirname $globalpath`
   globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini
   LOGVOL=`grep -i basepath_logvolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
   LOGBACKUP_PATH=`grep -i basepath_logvolumes $globalinipath | cut -d"=" -f2 | sed -e 's/^[ \t]*//'`
   if [ "$LOGVOL/logbackup" != "$LOGBACKUP_PATH" ]; then
      if [ -d "$LOGVOL/logbackup" ]; then
         echo "Cleaning up $LOGVOL/logbackup..."
         rm -rf "$LOGVOL/logbackup"
         echo "Cleanup is done"
      fi
   fi
}

call_updatecontrolfile()
{
  inputstring=$1
  echo -e "$inputstring" >> $CONTROL_FILE
  echo -e " " >> $CONTROL_FILE
}
#### Main

if [ ! -f /act/touch/.hana_migrate_"$dbsid".conf ]; then
   echo -e "MIGRATED=NO" > /act/touch/.hana_migrate_"$dbsid".conf
fi

call_updatecontrolfile "Starting Migration!"

ACT_DATA_DEVICE=`df -kh | grep $ACT_DATAVOL | awk '{print $1}'`
ACT_LOG_DEVICE=`df -kh | grep $ACT_LOGVOL | awk '{print $1}'`


if [ -z "$ACT_DATA_DEVICE" ] || [ -z "$ACT_LOG_DEVICE" ]; then
   echo "ERRORMSG: Unable to find Actifio Devices, Please check!"
   call_updatecontrolfile "ERRORMSG: Unable to find Actifio Devices, Please check!"
   create_pfile
   exit 1
fi

ACTDATA_VG_NAME=`lvdisplay -v $ACT_DATA_DEVICE | grep -w "VG Name" | awk '{print $3}'`
#ACTDATA_PV_NAME=`vgdisplay -v $ACTDATA_VG_NAME  | grep -w "PV Name" | awk '{print $3}'`
#ACTDATA_PV_NAME=`lvs -o devices $ACT_DATA_DEVICE |grep -v Devices | awk -F"(" '{print $1}'|sort|uniq |xargs`
ACTDATA_PV_NAME=`vgdisplay -v $ACTDATA_VG_NAME  | grep -w "PV Name" | awk '{print $3}'|sort|uniq|xargs`

ACTLOG_VG_NAME=`lvdisplay -v $ACT_LOG_DEVICE | grep -w "VG Name" | awk '{print $3}'`
#ACTLOG_PV_NAME=`lvs -o devices $ACT_LOG_DEVICE |grep -v Devices | awk -F"(" '{print $1}'|sort|uniq |xargs`
ACTLOG_PV_NAME=`vgdisplay -v $ACTLOG_VG_NAME  | grep -w "PV Name" | awk '{print $3}'|sort|uniq|xargs`



check_device_size $ACT_DATA_DEVICE $PROD_DATAVOL
check_device_size $ACT_LOG_DEVICE $PROD_LOGVOL

call_updatecontrolfile "Disabling CBT...!"
call_stop_tracking

PRODDATAVGNAME=`lvdisplay -v $PROD_DATAVOL | grep -w "VG Name" | awk '{print $3}'`
PRODLOGVGNAME=`lvdisplay -v $PROD_LOGVOL | grep -w "VG Name" | awk '{print $3}'`

if [ "$PRODDATAVGNAME" = "$PRODLOGVGNAME" ]; then
   PROD_DATA_PVS=`lvs -o devices $PROD_DATAVOL |grep -v Devices | awk -F"(" '{print $1}'|sort|uniq |xargs`
   PROD_LOG_PVS=`lvs -o devices $PROD_LOGVOL |grep -v Devices | awk -F"(" '{print $1}'|sort|uniq |xargs`
else
  PROD_DATA_PVS=`vgdisplay -v $PRODDATAVGNAME | grep -w "PV Name" | awk '{print $3}'|sort|uniq|xargs`
  PROD_LOG_PVS=`vgdisplay -v $PRODLOGVGNAME | grep -w "PV Name" | awk '{print $3}'|sort|uniq|xargs`
fi
#echo "MERGE LIST:$MERGE_LIST"

call_updatecontrolfile "Stopping the application...!"
echo "updating global.ini" > /act/touch/globaliniupdate
stop_application

tdate=`date +"%m%d%Y"`

call_updatecontrolfile "Commenting fstab if not commented...!"
call_commentfstab $PROD_DATAVOL
call_commentfstab $PROD_LOGVOL

call_updatecontrolfile "Starting Device migration steps for DATA Volume...!"


vgmerge_fun $ACT_DATA_DEVICE $PROD_DATAVOL
call_updatecontrolfile "Starting Device migration steps for LOG volume...!"
vgmerge_fun $ACT_LOG_DEVICE $PROD_LOGVOL



############# Start the Application #######

### Update Global.in
call_updatecontrolfile "Updating global.ini file...!"
if [ "$PROD_MNT" = "TRUE" ]; then
   OLDDBSID=`echo $OLDDBSID |tr '[a-z]' '[A-Z]'`
   PROD_DATA_PT=`df -kh | grep "$PROD_DATAVOL" | rev |awk '{print $1}' | rev`
   CHECK_DATA_MNTPT=`ls -l $PROD_DATA_PT/ | grep mnt00001 | wc -l`

   if [ "$CHECK_DATA_MNTPT" -gt "0" ]; then
      DATAPATH=$PROD_DATA_PT
   else
       DATAPATH=$PROD_DATA_PT/$OLDDBSID
       CHECK_DATA_FLAG=`ls -l $DATAPATH/ | grep mnt00001 | wc -l`
       if [ "$CHECK_DATA_FLAG" -eq 0 ]; then
          echo "ERRORMSG: Failed to find the datafiles, Please check the DATAVOLPATH.!"
          call_updatecontrolfile "ERRORMSG: Failed to find the datafiles, Please check the DATAVOLPATH.!"
          remove_file /act/touch/globaliniupdate
          exit 1
       fi
   fi

   PROD_LOG_PT=`df -kh | grep "$PROD_LOGVOL" | rev |awk '{print $1}' | rev`
   CHECK_LOG_MNTPT=`ls -l $PROD_LOG_PT/ | grep mnt00001 | wc -l`

   if [ "$CHECK_LOG_MNTPT" -gt "0" ]; then
      LOGPATH=$PROD_LOG_PT
   else
      LOGPATH=$PROD_LOG_PT/$OLDDBSID
      CHECK_LOG_FLAG=`ls -l $LOGPATH/ | grep mnt00001 | wc -l`
       if [ "$CHECK_LOG_FLAG" -eq 0 ]; then
          echo "ERRORMSG: Failed to find the logfiles, Please check the LOGVOLPATH.!"
          call_updatecontrolfile "ERRORMSG: Failed to find the logfiles, Please check the LOGVOLPATH.!"
          remove_file /act/touch/globaliniupdate
          exit 1
       fi
   fi
   
   update_globalini $DATAPATH $LOGPATH
else
   update_globalini $DATAVOLPATH $LOGVOLPATH 
fi

call_updatecontrolfile "Starting the application...!"
su - $dbsidadm -c 'HDB start'
remove_file /act/touch/globaliniupdate
if [ "$?" -gt 0 ]; then
   echo "ERRORMSG: Failed to Start the application, please check the logs for details!"
   call_updatecontrolfile "ERRORMSG: Failed to Start the application, please check the logs for details!"
   remove_file /act/touch/globaliniupdate
   #exit 1
fi

call_updatecontrolfile "PVMOVE is in progress...!"

pvmovedata_fun "$PROD_DATA_PVS" "$ACTDATA_PV_NAME" &
dataprocespid=$!
pvmovelog_fun "$PROD_LOG_PVS" "$ACTLOG_PV_NAME" &
logprocesspid=$!

wait $logprocesspid
wait $dataprocespid
retval=$?

if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   call_updatecontrolfile "ERRORMSG: PVMOVE Failed, Please check the error message!"
   create_pfile
   remove_file /act/touch/globaliniupdate
   exit 1
fi

call_updatecontrolfile "Starting VGREDUCE...!"
for i in ${ACTDATA_PV_NAME}
do
   vgreduce_fun $i $PROD_DATAVOL
done

for i in ${ACTLOG_PV_NAME}
do
   vgreduce_fun $i $PROD_LOGVOL
done

retval=$?
if [ "$retval" -eq 0 ]; then
   echo -e "MIGRATED=YES" > /act/touch/.hana_migrate_"$dbsid".conf
else
   echo "ERRORMSG: Failed to remove the devices. Please check for errors!"
   call_updatecontrolfile "ERRORMSG: Failed to remove the devices. Please check for errors!"
   create_pfile
   remove_file /act/touch/globaliniupdate
   exit 1
fi

call_updatecontrolfile "Starting CBT...!"
call_start_tracking
call_updatecontrolfile "Uncommenting fstab...!"
call_uncommentfstab $PROD_DATAVOL
call_uncommentfstab $PROD_LOGVOL
remove_file /act/touch/"$dbsid"_mount_params_inprogress
remove_file $CONTROL_FILE
remove_file /act/touch/globaliniupdate
cleanup_logbackup

call_updatecontrolfile "Migration Completed...!"
echo "************ Migration Operation Completed Successfully ***************"

exit 0
