#!/bin/bash
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means byN
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
set -x

dbsid=$1
JOBNAME=$2

INPUTFILE=`echo /act/touch/${JOBNAME}_lvmapping`
DBSIDlow=`echo "$dbsid" | awk '{print tolower($0)}'`
DBSIDupper=`echo "$dbsid" | awk '{print toupper($0)}'`
HANAuser=`echo "$DBSIDlow""adm"`
SYSNUMBER=`su - "$HANAuser" bash -c 'echo $TINSTANCE'`
if [ -z "$SYSNUMBER" ]; then
   SYSNUMBER=`su - $HANAuser -c 'basename $DIR_INSTANCE | rev | cut -c 1-2 | rev'`
fi
globalpath=`su - $HANAuser -c 'echo $DIR_INSTANCE'`
globalpath=`dirname $globalpath`
globalinipath=$globalpath/SYS/global/hdb/custom/config/global.ini
TIMESTAMP=`date +%y-%m-%d`
exepath=$globalpath/SYS/exe/hdb
pythonpath=$exepath/python_support

if [[ X$dbsid == X ]]
then echo "ERRORMSG: Missing argument Database SID"
  exit 1
else
  length=`expr length "$dbsid"`
  if [[ "$length" != 3 ]]
  then echo "ERRORMSG: Invalid argument for Database SID $dbsid - length is different than 3"
    exit 1
  else
    echo "INFO : Database SID is $DBSIDupper"
  fi
fi


if id "$HANAuser" >/dev/null 2>&1
then echo "INFO: user $HANAuser exists"
else echo "ERRORMSG: user $HANAuser does not exist"
  exit 1
fi

if [[ X$SYSNUMBER == X ]]
then "ERRORMSG: System number of instance $DBSIDupper was not detected"
  exit 1
fi

if [[ X$INPUTFILE == X ]]
then echo "ERRORMSG: Missing argument Input file for pvmove back to source"
  exit 1
else
  if [[ ! -f $INPUTFILE ]]
  then echo "ERRORMSG: Input file for pvmove back to source $INPUTFILE does not exist"
    exit 1
  else
    echo "INFO: Input file for pvmove back to source is $INPUTFILE"
    echo "INFO: Input file contain :"
    cat $INPUTFILE
  fi
fi



HANAISUP_CMD=`echo "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function GetSystemInstanceList | grep -Ei \\\\\"GREEN|YELLOW\\\\\" | wc -l\" "`
echo "$HANAISUP_CMD"
HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`


if [[ X$HANAISUP != X0 ]]
then
    su - $HANAuser bash -c "sapcontrol -nr $SYSNUMBER -function StopSystem"
    until [[ X$HANAISUP == X0 ]]
    do
      sleep 15
      HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
      echo "HANA is still up please wait"
    done
fi

ISNONSHARED=`cat $globalinipath | grep ^basepath_shared | cut -d'=' -f2-`
ISNONSHARED="$(echo -e "${ISNONSHARED}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

if [[ $ISNONSHARED != no ]]
   then echo "ERRORMSG: The instance $DBSIDupper is not a non-shared cluster"
   exit 1
fi

ISFCCLIENT=`cat $globalinipath | grep ^ha_provider | cut -d'=' -f2-`
ISFCCLIENT="$(echo -e "${ISFCCLIENT}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

if [[ X$ISFCCLIENT == X ]]
   then SCLOUT_CONFIG=`echo NONSHARED_MOUNTED`
   else SCLOUT_CONFIG=`echo NONSHARED_NOT_MOUNTED`
fi

unmount_volumes() {
  mountpt=$1

  systemctl stop $mountpt
  checkmount=`df -kh | grep -w "$mountpt" |wc -l`
  if [ "$checkmount" -gt 0 ]; then
     umount $mountpt
     if [ "$?" -gt 0 ]; then
        lsof | grep "^$mountpt"
        echo "ERRORMSG: Failed unmount $mountpt, Please check"
        exit 1
     fi
  fi
}

function mountcheck() {
  set -x
  globalinipath=$1

  DATAVOLUME_FS_CHECK=`cat $globalinipath | grep ^basepath_datavolumes | cut -d'=' -f2-`
  DATAVOLUME_FS_CHECK="$(echo -e "${DATAVOLUME_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

  LOGVOLUME_FS_CHECK=`cat $globalinipath | grep ^basepath_logvolumes | cut -d'=' -f2-`
  LOGVOLUME_FS_CHECK="$(echo -e "${LOGVOLUME_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

  MOUNTPOINT_CMD=`echo "df -l --output=target | grep -E \"^${DATAVOLUME_FS_CHECK}|^${LOGVOLUME_FS_CHECK}\""`
  MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`


  if [[ X$MNTPOINT_RESULT != X ]]
  then
     COUNT=0
     until [[ X$MNTPOINT_RESULT == X ]]
     do
       sleep 15
       COUNT=$((COUNT+1))
       MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`
       echo "INFO: there is still at least one FS mounted please wait"
       if [[ $COUNT  == 6 ]]
          then break
       fi
     done
  fi


  MOUNT_CHECK=`df -l | grep -E "${DATAVOLUME_FS_CHECK}|${LOGVOLUME_FS_CHECK}"  | wc -l`

  if [[ X$MOUNT_CHECK == X0 ]]
     then echo "INFO: File systems are unmounted, the procedure can continue"
     else echo "WARNING: File systems are still mounted procedure will unmount it force"
  fi

  if [[ X$MOUNT_CHECK != X0 ]]
     then MOUNTPOINT_DATA=`df -l --output=target | grep ^${DATAVOLUME_FS_CHECK}`
	if [[ X$MOUNTPOINT_DATA != X ]]
       	   then UNMOUNT_CMD_DATA=`echo "umount $MOUNTPOINT_DATA"`
           eval $UNMOUNT_CMD_DATA
	fi

        MOUNTPOINT_LOG=`df -l --output=target | grep ^${LOGVOLUME_FS_CHECK}`
        if [[ X$MOUNTPOINT_LOG != X ]]
	then UNMOUNT_CMD_LOG=`echo "umount $MOUNTPOINT_LOG"`
          eval $UNMOUNT_CMD_LOG
	fi
  fi

  MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`

  if [[ X$MNTPOINT_RESULT != X ]]
     then
     COUNT=0
     until [[ X$MNTPOINT_RESULT == X ]]
     do
       sleep 15
       COUNT=$((COUNT+1))
       MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`
       echo "INFO: there is still at least one FS mounted please wait"
       if [[ $COUNT  == 4 ]]
          then break
       fi
     done
  fi

  MOUNT_CHECK_FORCE=`df -l | grep -E "^${DATAVOLUME_FS_CHECK}|^${LOGVOLUME_FS_CHECK}"  | wc -l`

  if [[ X$MOUNT_CHECK_FORCE == X0 ]]
     then echo "INFO: File systems are force unmounted, the procedure can continue"
  else echo "ERRORMSG: File systems are still mounted procedure abort"
       exit 1
  fi
}

function mountcheck_staging() {
  set -x
  globalinipath=$1
  ACT_STAGINGDATA_FS_CHECK=`cat $globalinipath | grep ^basepath_datavolumes | cut -d'=' -f2-`
  ACT_STAGINGDATA_FS_CHECK="$(echo -e "${ACT_STAGINGDATA_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

  ACT_STAGINGLOG_FS_CHECK=`cat $globalinipath | grep ^basepath_logvolumes | cut -d'=' -f2-`
  ACT_STAGINGLOG_FS_CHECK="$(echo -e "${ACT_STAGINGLOG_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

  MOUNTPOINT_CMD=`echo "df -l --output=target | grep -E \"^${ACT_STAGINGDATA_FS_CHECK}|^${ACT_STAGINGLOG_FS_CHECK}\""`
  MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`

  if [[ X$MNTPOINT_RESULT != X ]]
  then
     COUNT=0
     until [[ X$MNTPOINT_RESULT == X ]]
     do
       sleep 15
       COUNT=$((COUNT+1))
       MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`
       echo "INFO: there is still at least one FS mounted please wait"
       if [[ $COUNT  == 4 ]]
          then break
       fi
     done
  fi

  ACT_STAGINGMOUNT_CHECK=`df -l | grep -E "^${ACT_STAGINGDATA_FS_CHECK}|^${ACT_STAGINGLOG_FS_CHECK}"  | wc -l`

  if [[ X$ACT_STAGINGMOUNT_CHECK == X0 ]]
     then echo "INFO: File systems on staging disks are not mounted, the procedure can continue"
  else echo "WARNING: File systems on staging disks are mounted procedure will unmount it force"
  fi

  if [[ X$ACT_STAGINGMOUNT_CHECK != X0 ]]
     then ACT_STAGINGMOUNTPOINT_DATA=`df -l --output=target | grep ^${ACT_STAGINGDATA_FS_CHECK}`
     if [[ X$ACT_STAGINGMOUNTPOINT_DATA != X ]]
        then UNMOUNT_CMD_DATA=`echo "umount $ACT_STAGINGMOUNTPOINT_DATA"`
         eval $UNMOUNT_CMD_DATA
     fi

     ACT_STAGINGMOUNTPOINT_LOG=`df -l --output=target | grep ^${ACT_STAGINGLOG_FS_CHECK}`
     if [[ X$ACT_STAGINGMOUNTPOINT_LOG != X ]]
        then UNMOUNT_CMD_LOG=`echo "umount $ACT_STAGINGMOUNTPOINT_LOG"`
        eval $UNMOUNT_CMD_LOG
     fi
  fi

  MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`


  if [[ X$MNTPOINT_RESULT != X ]]
  then
      COUNT=0
      until [[ X$MNTPOINT_RESULT == X ]]
      do
        sleep 15
        COUNT=$((COUNT+1))
        MNTPOINT_RESULT=`eval $MOUNTPOINT_CMD`
        echo "INFO: there is still at least one FS mounted please wait"
        if [[ $COUNT  == 4 ]]
           then break
        fi
      done
  fi


  ACT_STAGING_MOUNT_CHECK_FORCE=`df -l | grep -E "^${ACT_STAGINGDATA_FS_CHECK}|^${ACT_STAGINGLOG_FS_CHECK}"  | wc -l`

  if [[ X$ACT_STAGING_MOUNT_CHECK_FORCE == X0 ]]
     then echo "INFO: File systems on staging disks are force unmounted, the procedure can continue"
  else echo "ERRORMSG: File systems on staging disks are still mounted - procedure abort"
     exit 1
  fi
}

check_device_size()
{
  ACT_DEVICE=$1
  PROD_DEVICE=$2

   ### Validate Size
  ACT_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_DEVICE |xargs`
  PROD_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_DEVICE|xargs`

  DIFF_IN_SIZE=$(($PROD_DEVICE_SIZE-$ACT_DEVICE_SIZE))

  if [ "$PROD_DEVICE_SIZE" -ge "$ACT_DEVICE_SIZE" ] && [ "$DIFF_IN_SIZE" -ge "0" ]; then
     echo "PROD_DEVICE_SIZE: $PROD_DEVICE_SIZE, ACTFIO_DEVICE_SIZE:$ACT_DEVICE_SIZE, Continuing with PV Move process..!"
  else
     echo "ERRORMSG: The device $PROD_DEVICE does not have enough free space, Please check!"
     "ERRORMSG: The device $PROD_DEVICE does not have enough free space, Please check!"
     exit 1
  fi
}

masternode=`cat ${globalpath}/SYS/global/hdb/custom/config/nameserver.ini | grep ^master |cut -d'=' -f2- |cut -d':' -f-1`
masternode="$(echo -e "${masternode}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

servername=`cat ${globalpath}/SYS/global/hdb/custom/config/nameserver.ini | grep ^roles_ |cut -d'_' -f2- |cut -d'=' -f-1`


if [ -z "$masternode" ]; then
   echo "ERRORMSG: Failed to find MASTER Node, Please check nameserver.ini"
   exit 1
fi

if [ -z "$servername" ]; then
   echo "ERRORMSG: Failed to find node information. Please check nameserver.ini"
   exit 1
fi

if [[ $SCLOUT_CONFIG == NONSHARED_NOT_MOUNTED ]]
then
  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
     if [[ $NODE != $masternode ]]
     then
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f mountcheck);mountcheck $globalinipath"
     else mountcheck $globalinipath
     fi
   done
else
   #NONSHARED_MOUNTED

   DATAVOLUME_FS_CHECK=`cat $globalinipath | grep ^basepath_datavolumes | cut -d'=' -f2-`
   DATAVOLUME_FS_CHECK="$(echo -e "${DATAVOLUME_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
   DATAVOLUME_FS_CHECK=`df -P $DATAVOLUME_FS_CHECK |awk 'NR==2{print $NF}'`

   LOGVOLUME_FS_CHECK=`cat $globalinipath | grep ^basepath_logvolumes | cut -d'=' -f2-`
   LOGVOLUME_FS_CHECK="$(echo -e "${LOGVOLUME_FS_CHECK}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
   LOGVOLUME_FS_CHECK=`df -P $LOGVOLUME_FS_CHECK | awk 'NR==2{print $NF}'`


function MNT_detection_fun(){
       set -x
       i=$1
       MNT_DETECTION=$2

       MNT_DETECTION=`dirname $MNT_DETECTION/mnt*/hdb* | head -1`
       MNT_DETECTION=`basename $MNT_DETECTION`
       echo $MNT_DETECTION > /act/touch/.MNT_DETECTION_$i
}

function VGRENAME_fun()
{
   name=$1
   flag=$2
   INPUTFILE=$3
   dbsid=$4

   ########### Check unquiness of VGNAMES ##########
   numofnodes=`cat $INPUTFILE | awk -F":" '{print $1}' | sort|uniq |xargs`

   for nodename in $(echo $numofnodes)
   do
      ismapper=`cat $INPUTFILE| grep $nodename | awk -F":" '{print $2}' | grep "mapper" | wc -l`
      if [ "$ismapper" -gt 0 ]; then
         vgname=`cat $INPUTFILE| grep $nodename | awk -F":" '{print $2}' | awk -F"/" '{print $4}' | awk -F"-" '{print $1}' | sort |uniq|xargs`
      else
          vgname=`cat $INPUTFILE| grep $nodename | awk -F":" '{print $2}' | awk -F"/" '{print $3}' | sort|uniq | xargs`
      fi

      for i in $(echo $vgname)
      do
        if [ "$ismapper" -gt 0 ]; then
           checkuniq=`cat $INPUTFILE | grep -v $nodename | awk -F":" '{print $2}' | awk -F"/" '{print $4}' | awk -F"-" '{print $1}' | grep -w $i | wc -l`
        else
           checkuniq=`cat $INPUTFILE | grep -v $nodename | awk -F"/" '{print $3}' | grep -w $i | wc -l`
        fi

        if [ "$checkuniq" -gt 0 ]; then
           UNIQUE=false
           break;
        fi
      done
      if [ "$UNIQUE" = "false" ]; then
         break;
      fi
   done
   if [ "$UNIQUE" = "false" ]; then
      vgrenamefile=/act/touch/.vgrename_$name
      if [ -f /act/touch/.MNT_DETECTION_$name ]; then
         mntdir=`cat /act/touch/.MNT_DETECTION_$name`
         mntdirrename=`cat /act/touch/.MNT_DETECTION_$name | rev | cut -c1`
         lvlist=`cat $INPUTFILE | grep $mntdir`
         if [ "$flag" = "beforevgmerge" ]; then
            for list in $(echo $lvlist)
            do
              lname=`echo $list | awk -F":" '{print $2}'`
              vgname=`lvs --noheadings -o vg_name $lname | xargs`
                 if [ "$vgname" != "$vgname$mntdir" ] & [ ! -z "$vgname" ]; then
                    vgrename $vgname $vgname$mntdirrename
                    newlist=`echo $list | awk -F":" '{print $1":"$2}'`
                    newlist=`echo $newlist | sed "s|$vgname|$vgname$mntdirrename|g"`
                    echo "$vgname:$vgname$mntdirrename" >> $vgrenamefile
                    list=`echo $list | awk -F":" '{print $1":"$2}'`
                    sed -i "s|$list|$newlist|g" $INPUTFILE
                    sed -i "s|$vgname|$vgname$mntdirrename|g" /etc/fstab
                    sed -i "s|$vgname|$vgname$mntdirrename|g" ${INPUTFILE}_${dbsid}_fstab
                 elif [ "$vgname$mntdirrename" = "$vgname$mntdirrename" ];  then
                      sed -i "s|$list|$newlist|g" $INPUTFILE
                 fi
            done
         elif [ "$flag" = "aftervgreduce" ]; then
              vglist=`cat $vgrenamefile | xargs`
              for vname in $(echo $vglist)
              do
                oldvgname=`echo $vname | awk -F":" '{print $1}'`
                tempvgname=`echo $vname | awk -F":" '{print $2}'`
                if [ "$oldvgname" != "$newvgname" ]; then
                   vgrename  $tempvgname $oldvgname
                   sed -i "s|$tempvgname|$oldvgname|g" /etc/fstab
                fi
             done
         fi
      fi
   fi
}

  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

     MNT_DETECTION=`cat $globalinipath | grep ^basepath_datavolumes | cut -d'=' -f2-`
     MNT_DETECTION="$(echo -e "${MNT_DETECTION}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

     if [[ X$DATAVOLUME_FS_CHECK != X ]]
     then
       if [[ $NODE != $masternode ]]
       then

         ssh -o "StrictHostKeyChecking=no" $NODE "eval /opt/act/cbt/bin/cbt_deactivate.sh"
        # ssh -o "StrictHostKeyChecking=no" $NODE "MNT_DETECTION=`dirname $MNT_DETECTION/mnt*/hdb* | head -1` && MNT_DETECTION=`basename $MNT_DETECTION` && echo $MNT_DETECTION > /act/touch/.MNT_DETECTION_$i"
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f MNT_detection_fun);MNT_detection_fun $i $MNT_DETECTION"
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f unmount_volumes);unmount_volumes $LOGVOLUME_FS_CHECK"
         retval=$?
         if [ "$retval" -gt 0 ]; then
            echo "ERRORMSG: Failed to unmount devices on $NODE, Please check the processes!"
            exit 1
         fi
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f unmount_volumes);unmount_volumes $DATAVOLUME_FS_CHECK"
         retval=$?
         if [ "$retval" -gt 0 ]; then
            echo "ERRORMSG: Failed to unmount devices on $NODE, Please check the processes!"
            exit 1
         fi

         ssh -o "StrictHostKeyChecking=no" $NODE "rm /act/touch/.vgrename_$NODE"
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f VGRENAME_fun);VGRENAME_fun $i beforevgmerge $INPUTFILE $dbsid"
         retval=$?
         if [ "$retval" -gt 0 ]; then
            echo "ERRORMSG: Failed to perform VGRENAME, please check the error message for details!!"
            exit 1
         fi
       else
           eval /opt/act/cbt/bin/cbt_deactivate.sh
          # MNT_DETECTION=`dirname $MNT_DETECTION/mnt*/hdb* | head -1`
          # MNT_DETECTION=`basename $MNT_DETECTION`
          # echo $MNT_DETECTION > /act/touch/.MNT_DETECTION_$i
          MNT_detection_fun $i $MNT_DETECTION
          if [ -f /act/touch/.vgrename_$NODE ]; then
             rm /act/touch/.vgrename_$NODE
          fi
          VGRENAME_fun $i beforevgmerge $INPUTFILE $dbsid
          unmount_volumes $LOGVOLUME_FS_CHECK
          unmount_volumes $DATAVOLUME_FS_CHECK
          inputvar=`cat $INPUTFILE`
             for Y in ${inputvar}
             do
             VGSTAGING=`printf "$Y" | awk -F ":" '{ printf $3 }'`

             FS_TO_UNMOUNT=`df -l  | grep  ^${VGSTAGING} | awk '{ printf $6}'`
               if [[ X$FS_TO_UNMOUNT != X ]]
               then
               umount $FS_TO_UNMOUNT
               fi
             done
       fi
     else
      echo "ERRORMSG: The datavolume / logvolume cannot be identified for unmount"
      exit 1
     fi
   done
fi

function VGMERGE_fun(){
set -x
INPUTFILE=$1
echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"
echo ""
echo "VGMERGE from staging disk back to source    "
echo ""
echo "$INPUTFILE"
echo ""
echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"

var=`cat $INPUTFILE`

for v in $(tr ',' '\n' <<< "$var")
do
        Initial_VGN=`printf "$v" | awk -F ":" '{ printf $1 }'`
        Initial_VGN=`printf "$Initial_VGN" | awk -F "-" '{ printf $1 }'`
            if [ -z "$Initial_VGN" ]; then
            echo "ERRORMSG: Failed to get VGNAME for Initial!"
            exit 1
            fi

        Staging_VGN=`printf "$v" | awk -F ":" '{ printf $2 }'`
        Staging_VGN=`printf "$Staging_VGN" | awk -F "-" '{ printf $1 }'`
            if [ -z "$Staging_VGN" ]; then
            echo "ERRORMSG: Failed to get VGNAME for Staging!"
            exit 1
            fi

        VGMERGE_CMD=`echo "vgmerge $Initial_VGN $Staging_VGN"`

        eval $VGMERGE_CMD
         if [ "$?" -gt 0 ]; then
         echo "ERRORMSG: FAILED TO PERFORM VGMERGE. Please check!"
         exit 1
         fi

	PVNAME_REDUCE_STAGING=`pvdisplay -C -o pv_name -S vgname=$Staging_VGN | tail -1`
#VGREDUCE IS MISSING
        VGREDUCE_CMD=`echo "vgreduce $Initial_VGN $PVNAME_REDUCE_STAGING" `
echo "INFO the vgmerge back to source  will be changed from $Staging_VGN to $Initial_VGN"
done
echo $VGREDUCE_CMD
}

function Vgmere_mounted()
{
set -x
globalinipath=$1
INPUTFILE=$2
NODE=$3

MNT_DETECTION=`cat /act/touch/.MNT_DETECTION_$NODE`
var=`cat $INPUTFILE | grep $MNT_DETECTION`
for v in ${var}
  do
   Staging_VGN=`printf "$v" | awk -F ":" '{ printf $3 }'`
   Staging_VGN=`basename $Staging_VGN`
   Staging_VGN=`echo $Staging_VGN | cut -d'-' -f-1`
   Source_VGN=`printf "$v" | awk -F ":" '{ printf $2 }'`
   Source_VGN=`lvdisplay -C -o "lv_path,lv_dm_path" $Source_VGN --noheadings | awk '{print $2}'`
   Source_VGN=`basename $Source_VGN`
   Source_VGN=`echo $Source_VGN | cut -d'-' -f-1`

   Staging_VGN="$(echo -e "${Staging_VGN}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
   Source_VGN="$(echo -e "${Source_VGN}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

    NODE_PROTECTED=`echo "${NODE}" | sed 's:-:_:g'`
   declare LIST_Source_PVN_${NODE_PROTECTED}_${Source_VGN}=`pvdisplay -C -o pv_name -S vgname="$Source_VGN" --noheadings |xargs`
   echo "LIST_Source_PVN_${NODE_PROTECTED}_${Source_VGN}=\"`pvdisplay -C -o pv_name -S vgname="$Source_VGN" --noheadings |xargs`\"" >> ${INPUTFILE}_variable
   declare LIST_Staging_PVN_${NODE_PROTECTED}_${Staging_VGN}=`pvdisplay -C -o pv_name -S vgname="$Staging_VGN" --noheadings|xargs`
   echo "LIST_Staging_PVN_${NODE_PROTECTED}_${Staging_VGN}=\"`pvdisplay -C -o pv_name -S vgname="$Staging_VGN" --noheadings|xargs`\"" >> ${INPUTFILE}_variable
#deactivation of source_VGN
   vgchange -an $Staging_VGN
     if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGCHANGE. Please check!"
     exit 1
     fi

    vgmerge_CMD="vgmerge $Source_VGN $Staging_VGN"

    eval $vgmerge_CMD

     if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGMERGE. Please check!"
     exit 1
     fi

    Source_LVN=`printf "$v" | awk -F ":" '{ printf $2 }'`
    Source_LVN_rename=`echo $Source_LVN`
    Prefix_LVN=`dirname $Source_LVN`
    Source_LVN=`lvdisplay -C -o "lv_path,lv_dm_path" $Source_LVN --noheadings | awk '{print $2}'`

    lvremove -f  $Source_LVN

     if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM LVREMOVE $Source_LVN. Please check!"
     exit 1
     fi

    Staging_LVN=`printf "$v" | awk -F ":" '{ printf $3 }'`
    Staging_LVN=`basename $Staging_LVN`
    Staging_LVN=`echo $Staging_LVN | cut -d'-' -f2-`
    Staging_LVN=`echo ${Prefix_LVN}/${Staging_LVN}`

    lvrename $Staging_LVN $Source_LVN_rename

     if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM LVRENAME $Staging_LVN $Source_LVN. Please check!"
     exit 1
     fi

    vgchange -ay $Source_VGN
     if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: FAILED TO PERFORM VGCHANGE Activation. Please check!"
     exit 1
     fi

   echo "INFO: Vgmerge executed from $Staging_VGN to $Source_VGN"

  done

     #vgreduce source vg name pvname_reduce_staging
}

if [[ $SCLOUT_CONFIG == NONSHARED_NOT_MOUNTED ]]
then
  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
    ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f VGMERGE_fun);VGMERGE_fun $INPUTFILE"
    retval=$?
    if [ "$retval" -gt 0 ]; then
       echo "ERRORMSG: Failed to perform VMERGE, please check the error message for details!!"
       exit 1
    fi
    break
    fi
   done

  echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"
  echo ""
  echo "Update global.ini file with original VGs mounted"
  echo ""
  echo "cat $INPUTFILE"
  echo ""
  echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"

  var=`cat $INPUTFILE`

  for v in $(tr ',' '\n' <<< "$var")
  do
        Source_value=`printf "$v" | awk -F ":" '{ printf $1 }'`
        Update_value=`printf "$v" | awk -F ":" '{ printf $2 }'`
        CHANGE_CMD="sed -i 's/"$Update_value"/"$Source_value"/g' $globalinipath"

        eval $CHANGE_CMD

  echo "INFO the source value will be changed $Update_value to $Source_value"
  done

  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
    ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f mountcheck_staging);mountcheck_staging $globalinipath"
    else mountcheck_staging $globalinipath
    fi
   done
else

#check Size
 for i in ${servername}
   do
       NODE=`echo $i`
       NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
     if [[ $NODE != $masternode ]]
     then
       MNT_DETECTION=`cat /act/touch/.MNT_DETECTION_$NODE`
       devices=`cat $INPUTFILE | grep $MNT_DETECTION`
        for D in ${devices}
        do
         ACT_DEVICE=`printf "$D" | awk -F ":" '{ printf $3 }'`
         PROD_DEVICE=`printf "$D" | awk -F ":" '{ printf $2 }'`
         ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f check_device_size);check_device_size $ACT_DEVICE $PROD_DEVICE"
        done
     else
      MNT_DETECTION=`cat /act/touch/.MNT_DETECTION_$NODE`
       devices=`cat $INPUTFILE | grep $MNT_DETECTION`
        for D in ${devices}
        do
         ACT_DEVICE=`printf "$D" | awk -F ":" '{ printf $3 }'`
         PROD_DEVICE=`printf "$D" | awk -F ":" '{ printf $2 }'`
         check_device_size $ACT_DEVICE $PROD_DEVICE
        done
    fi
   done

 for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"

if [[ $NODE != $masternode ]]
    then
      ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f Vgmere_mounted);Vgmere_mounted $globalinipath $INPUTFILE $NODE"
      ssh -o "StrictHostKeyChecking=no" $NODE "mv /etc/fstab /etc/fstab.actifio"
      ssh -o "StrictHostKeyChecking=no" $NODE "cp -pr ${INPUTFILE}_${dbsid}_fstab /etc/fstab"

      ssh -o "StrictHostKeyChecking=no" $NODE "systemctl daemon-reload"
      ssh -o "StrictHostKeyChecking=no" $NODE "mount -a"
    else Vgmere_mounted $globalinipath $INPUTFILE $NODE
         mv /etc/fstab /etc/fstab.actifio
         cp -pr ${INPUTFILE}_${dbsid}_fstab /etc/fstab
         systemctl daemon-reload
         mount -a
    fi

   done
fi

su - $HANAuser bash -c "sapcontrol -nr $SYSNUMBER -function StartSystem"

HANAISUP_CMD=`echo "su - \"$HANAuser\" bash -c \"sapcontrol -nr $SYSNUMBER -function GetSystemInstanceList | grep -Ei \\\\\"GRAY|YELLOW\\\\\" | wc -l\" "`
echo "$HANAISUP_CMD"
HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`

count=1
if [[ X$HANAISUP != X0 ]]
then

until [[ X$HANAISUP == X0 ]]
do
  if [ "$count" -eq 30 ]; then
     echo "ERRORMSG: Unable to bring database after VGMERGE"
     exit 1
  fi
sleep 30
count=$(($count+1))

HANAISUP=`eval $HANAISUP_CMD | awk '{printf $1}'`
echo "HANA is still down please wait"
done
fi

function PVMOVE_fun(){
set -x
INPUTFILE=$1
echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"
echo ""
echo "PVMOVE from staging disk back to source    "
echo ""
echo "cat $INPUTFILE"
echo ""
echo "-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"

var=`cat $INPUTFILE`

for v in $(tr ',' '\n' <<< "$var")
do
        Initial_PVN=`printf "$v" | awk -F ":" '{ printf $1 }'`
        Initial_PVN=`printf "$Initial_PVN" | awk -F "-" '{ printf $1 }'`
	Initial_PVN=`pvdisplay -C -o pv_name -S vgname="$Initial_PVN" | tail -1`
	    if [ -z "$Initial_PVN" ]; then
            echo "ERRORMSG: Failed to get PVNAME for Initial!"
     	    exit 1
   	    fi

        Staging_PVN=`printf "$v" | awk -F ":" '{ printf $2 }'`
        Staging_PVN=`printf "$Staging_PVN" | awk -F "-" '{ printf $1 }'`
	Staging_PVN=`pvdisplay -C -o pv_name -S vgname="$Staging_PVN" | tail -1`
            if [ -z "$Staging_PVN" ]; then
            echo "ERRORMSG: Failed to get PVNAME for Staging!"
            exit 1
            fi

        PVMOVE_CMD=`echo "pvmove --atomic -A y -q $Staging_PVN $Initial_PVN"`
        PVMOVE_CMD=`eval $PVMOVE_CMD && echo OK || echo "ERRORMSG: failed pvmove for $Staging_PVN to $Initial_PVN"`
	eval $PVMOVE_CMD &

echo "INFO the PVMOVE back to source  will be changed from $Staging_PVN to $Initial_PVN"
done
wait
}

function PVMOVE_mounted_parallel_fun(){
set -x
for S in $(tr ' ' '\n' <<< "$Staging_PVN")
  do
      CHECK_VALUE=`cat ${INPUTFILE}_variable | grep LIST_Staging_PVN_ | awk -F "=" '{ printf $2 }' | sed 's:^"\(.*\)"$:\1:'`

       for C in $(tr '""' '\n' <<< "${CHECK_VALUE}")
       do
       Source_PVN=`echo $Source_PVN | sed -e s:${C}::`
       #Source_PVN=`echo $Source_PVN | sed -e s:${S}::`
       done

      pvmove --atomic -A y -q $S $Source_PVN
    if [ "$?" -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVMOVE Action from $S $Source_PVN. Please check!"
    exit 1
    fi
  done

}

function PVMOVE_mounted_fun1(){

set -x
globalinipath=$1
INPUTFILE=$2
NODE=$3

source ${INPUTFILE}_variable
#rm -f ${INPUTFILE}_variable

MNT_DETECTION=`cat /act/touch/.MNT_DETECTION_$NODE`
var=`cat $INPUTFILE | grep $MNT_DETECTION`
for v in ${var}
  do
   Staging_VGN=`printf "$v" | awk -F ":" '{ printf $3 }'`
   Staging_VGN=`basename $Staging_VGN`
   Staging_VGN=`echo $Staging_VGN | cut -d'-' -f-1`
   Source_VGN=`printf "$v" | awk -F ":" '{ printf $2 }'`
   Source_VGN=`lvdisplay -C -o "lv_path,lv_dm_path" $Source_VGN --noheadings | awk '{print $2}'`
   Source_VGN=`basename $Source_VGN`
   Source_VGN=`echo $Source_VGN | cut -d'-' -f-1`
   Source_VGN_orig=$Source_VGN

     NODE_PROTECTED=`echo "${NODE}" | sed 's:-:_:g'`
   eval "Source_PVN=\$LIST_Source_PVN_${NODE_PROTECTED}_${Source_VGN}"
   echo $Source_PVN
   eval "Staging_PVN=\$LIST_Staging_PVN_${NODE_PROTECTED}_${Staging_VGN}"
   echo $Staging_PVN

###### NEED TO FORMAT LIST

#   Source_PVN=`pvdisplay -C -o pv_name -S vgname="$Source_VGN" | tail -1`
#   Staging_PVN=`pvdisplay -C -o pv_name -S vgname="$Staging_VGN" | tail -1`

#for S in $(tr ' ' '\n' <<< "$Staging_PVN")
#  do
#      pvmove --atomic -A y -b -q $S $Source_PVN
#    if [ "$?" -gt 0 ]; then
#    echo "ERRORMSG: FAILED TO PERFORM PVMOVE Action from $S $Source_PVN. Please check!"
#    exit 1
#    fi
#  done

PVMOVE_mounted_parallel_fun &
done
wait
}

function PVMOVE_mounted_fun2(){

set -x
globalinipath=$1
INPUTFILE=$2
NODE=$3

source ${INPUTFILE}_variable
#rm -f ${INPUTFILE}_variable

MNT_DETECTION=`cat /act/touch/.MNT_DETECTION_$NODE`
var=`cat $INPUTFILE | grep $MNT_DETECTION`
for v in ${var}
  do
   Staging_VGN=`printf "$v" | awk -F ":" '{ printf $3 }'`
   Staging_VGN=`basename $Staging_VGN`
   Staging_VGN=`echo $Staging_VGN | cut -d'-' -f-1`
   Source_VGN=`printf "$v" | awk -F ":" '{ printf $2 }'`
   Source_VGN=`lvdisplay -C -o "lv_path,lv_dm_path" $Source_VGN --noheadings | awk '{print $2}'`
   Source_VGN=`basename $Source_VGN`
   Source_VGN=`echo $Source_VGN | cut -d'-' -f-1`

    NODE_PROTECTED=`echo "${NODE}" | sed 's:-:_:g'`
   eval "Source_PVN=\$LIST_Source_PVN_${NODE_PROTECTED}_${Source_VGN}"
   echo $Source_PVN
   eval "Staging_PVN=\$LIST_Staging_PVN_${NODE_PROTECTED}_${Staging_VGN}"
   echo $Staging_PVN

for S in $(tr ' ' '\n' <<< "$Staging_PVN")
  do
      vgreduce $Source_VGN $Staging_PVN
    if [ "$?" -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM VGREDUCE Action for $Source_VGN. Please check!"
    exit 1
    fi

      pvremove $Staging_PVN
    if [ "$?" -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE Action for $Staging_PVN. Please check!"
    exit 1
    fi
  done

done
}

if [[ $SCLOUT_CONFIG == NONSHARED_NOT_MOUNTED ]]
then
  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
    ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f PVMOVE_fun);PVMOVE_fun $INPUTFILE"
    retval=$?
    if [ "$retval" -gt 0 ]; then
       echo "ERRORMSG: Failed to perform PVMOVE, please check the error message for details!!"
       exit 1
    fi
    fi
   break
   done
else

  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
      ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f PVMOVE_mounted_fun1 PVMOVE_mounted_parallel_fun);PVMOVE_mounted_fun1 $globalinipath $INPUTFILE $NODE" &
    else
      PVMOVE_mounted_fun1 $globalinipath $INPUTFILE $NODE &
    fi
   done

wait
  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
      ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f PVMOVE_mounted_fun2);PVMOVE_mounted_fun2 $globalinipath $INPUTFILE $NODE"
    else
      PVMOVE_mounted_fun2 $globalinipath $INPUTFILE $NODE
    fi
   done


  for i in ${servername}
   do
     NODE=`echo $i`
     NODE="$(echo -e "${NODE}" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')"
    if [[ $NODE != $masternode ]]
    then
#      ssh -o "StrictHostKeyChecking=no" $NODE "$(declare -f VGRENAME_fun);VGRENAME_fun $i aftervgreduce $INPUTFILE $dbsid"
      ssh -o "StrictHostKeyChecking=no" $NODE "/opt/act/cbt/bin/cbt_activate.sh"
      ssh -o "StrictHostKeyChecking=no" $NODE "rm /act/touch/.MNT_DETECTION_$NODE"
      ssh -o "StrictHostKeyChecking=no" $NODE "rm /act/touch/.vgrename_$NODE"
    else
     # VGRENAME_fun $i aftervgreduce $INPUTFILE $dbsid
      eval /opt/act/cbt/bin/cbt_activate.sh
      rm /act/touch/.MNT_DETECTION_$NODE
      rm /act/touch/.vgrename_$NODE
    fi
   done

fi

echo "INFO the procedure pvmove back to source is done correctly for $DBSIDupper"

exit 0
