#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

WPWD=/act/custom_apps/saphana/acthanadbm/
cd /act/custom_apps/saphana/acthanadbm/
APP=$1
SOURCE_HOST=$2
TARGET_HOST=$3
TARGET_MNT_PT=$4
DBUSER=$5
HANAVERSION=$6
DATAPATH=$7
LOGPATH=$8
DEVICE_NAMES="$9"
AGM="${10}"
TARGETDBSID="${11}"
MIGRATE="${12}"
LOGMOUNTPATH="${13}"
RECOVERYTIME="${14}"


conffile=/act/custom_apps/saphana/lvm_migrate/MountAndMigrate/ACT_HANADB_mountANDmigrate_LVM.conf

if [ "$SOURCE_HOST" = "$TARGET_HOST" ]; then
   TARGETDBSID=$APP
fi

OLDDBSID=$APP

if [ "$#" -lt 12 ]; then
   if [ -f $conffile ]; then
      source $conffile
      if [ -z "$APP" ] || [ -z "$SOURCE_HOST" ] || [ -z "$TARGET_HOST" ] || [ -z "$TARGET_MNT_PT" ] || [ -z "$DBUSER" ] || [ -z "$HANAVERSION" ] || [ -z "$DATAPATH" ] || [ -z "$LOGPATH" ] || [ -z "$DEVICE_NAMES" ] || [ -z "$AGM" ] || [ -z "$TARGETDBSID" ] || [ -z "$MIGRATE" ]; then
         echo "Please check the input parameters!"
	 echo "./ACT_HANADB_mountANDmigrate_LVM.sh <APP NAME> <SOURCE HOST> <TARGET HOST> <TARGET MOUNT POINT> <DBUSER> <HANAVERSION> <DATAPATH> <LOGPATH> <DEVICE NAMES> <AGM IP> <TARGET DBSID> <MIGRATE YES/NO> [LOGMOUNT PATH] [RECOVERY TIME]"
         exit 1
      fi   
   else
      echo "No input parameters/configuration file exist!. Please check the usage"
      echo "./ACT_HANADB_mountANDmigrate_LVM.sh <APP NAME> <SOURCE HOST> <TARGET HOST> <TARGET MOUNT POINT> <DBUSER> <HANAVERSION> <DATAPATH> <LOGPATH> <DEVICE NAMES> <AGM IP> <TARGET DBSID> <<MIGRATE YES/NO> [LOGMOUNT PATH] [RECOVERY TIME]"
      exit 1
   fi
fi

APPREMOUNT=FALSE

echo "*************** Starting MOUNT Operation ****************"
/act/custom_apps/saphana/acthanadbm/actHANADBM.pl --type mount --appName $APP --sourceHost $SOURCE_HOST --targetHost $TARGET_HOST --mountpoint "$TARGET_MNT_PT" --AGM $AGM --wait yes
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: Mount operation failed. Please check logs for details!"
   exit 1
fi
fileloc=/act/touch
if [ ! -d "$file_loc" ]; then
   mkdir -p /act/touch
   chmod 775 /act/touch
fi

ORIGINAL_VGNAME=$fileloc/originalvgname.lst

lsblk -i -o NAME,MOUNTPOINT| grep $TARGET_MNT_PT |grep -v TransactionLog |sed 's/^`-//g' > $ORIGINAL_VGNAME

count=1
while read line
do
  line=`echo $line |awk '{print $1}' | awk -F"-" '{print $1}'`
  newname=`echo $TARGET_MNT_PT | sed 's/\///g'`
  newname=$newname'actvg'$count
  vgrename $line $newname
  count=$(($count+1))
  sed -i "s/$line/$newname/g" $ORIGINAL_VGNAME
done < $ORIGINAL_VGNAME

while read line
do
  mntpt=`echo $line |awk '{print $2}'`
  vgname=`echo $line |awk '{print $1}'`
  umount $mntpt
  mount  /dev/mapper/"$vgname" $mntpt
done < $ORIGINAL_VGNAME

if [ -f $ORIGINAL_VGNAME ]; then
   rm -f $ORIGINAL_VGNAME
fi

echo "*************** Starting RECOVERY Operation ****************"
/act/custom_apps/saphana/clone/ACT_HANADB_mountrecover.sh $TARGETDBSID $TARGET_MNT_PT $DBUSER $HANAVERSION $DATAPATH $LOGPATH $OLDDBSID $LOGMOUNTPATH "$RECOVERYTIME"
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: Appaware Mount operation failed. Please check logs for details!"
   exit 1
fi
echo "******************** HANA DB Recovery Compeleted ******************"

sleep 60

if [ "$MIGRATE" = "YES" ]; then
   echo "************ Starting Storage Migration Process..! ************"
else
   echo -e "MIGRATED=NO" > /act/touch/.hana_migrate_"$TARGETDBSID".conf
   echo "***********!!! For Migration please run migratePostRecovery.sh when ready. !!!************"
   exit 0 
fi


/act/custom_apps/saphana/lvm_migrate/ACT_DB_pvmove.sh $TARGETDBSID $TARGET_MNT_PT "$DEVICE_NAMES" 
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: Storage Migration failed. Please check logs for details!"
   exit 1
fi

echo "*************** Starting UNMOUNT + DELETE Operation ****************"
/act/custom_apps/saphana/acthanadbm/actHANADBM.pl --type unmountdelete --appName $APP --sourceHost $SOURCE_HOST --targetHost $TARGET_HOST --targetAppName $TARGET_MNT_PT --AGM $AGM --wait yes
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: Unmount Delete  operation failed. Please check logs for details!"
   exit 1
fi

echo "*************** All the operations completed successfully. Please verify the application! ****************"

exit 0
