#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x

WPWD=/act/custom_apps/saphana/restore

DBSID=$1
DBUSER=$2
HANAVERSION="$3"
RECOVERYTIME="$4"
LOGMOUNTPATH="$5"
DATAPATH="$6"

#source /act/custom_apps/saphana/restore/hana_restore.conf

#export ACT_JOBTYPE=mount
#export ACT_PHASE=post

if [ -z $DBSID ]; then
echo "ERRORMSG: Database SID is not set "
exit 1
fi

dbsidadm=$DBSID"adm"
export dbsidadm

echo "############ Restoring USERSTORE Keys #############"

SSF_LOCATION=`su - $dbsidadm -c "hdbuserstore list | grep \"DATA FILE\" | awk -F \":\" '{print \\$2}'"`
SSF_LOCATION=`dirname $SSF_LOCATION`

COPYHDBSTOREKEY=`echo $COPYHDBSTOREKEY | tr '[A-Z]' '[a-z]'`
if [ "$COPYHDBSTOREKEY" = "true" ]; then
   if [ -d $SSF_LOCATION ] && [ -f /act/tmpdata/$JOBID/SSFS_HDB.DAT ]; then
      echo "############ Restoring USERSTORE Keys #############"
      tdate=`date +%Y%m%d%H%M%S`
      mkdir -p $SSF_LOCATION/$tdate
      chown $dbsidadm:sapsys $SSF_LOCATION/$tdate
      mv $SSF_LOCATION/*.* $SSF_LOCATION/$tdate
      chown $dbsidadm:sapsys $SSF_LOCATION/$tdate/*
      cp /act/tmpdata/$JOBID/SSFS_HDB* $SSF_LOCATION/
      cp /act/tmpdata/$JOBID/SQLDBC.shm $SSF_LOCATION/
      chown $dbsidadm:sapsys $SSF_LOCATION/SSFS_HDB*
   fi
   echo "########### USERSTOREKEY is copied ###########"
fi

###### Added for replication cluster recovery ######
if [ -z "$CLUSTERTYPE" ]; then
   CLUSTERTYPE=`grep CLUSTERTYPE /act/touch/"$DBSID"_mount_params | awk -F"=" '{print $2}'`
fi
if [ "$CLUSTERTYPE" = "replication" ]; then
   if [ -z "$CATALOGBACKUPPATH" ]; then
      CATALOGBACKUPPATH=`grep CATALOGBACKUPPATH /act/touch/"$DBSID"_mount_params | awk -F"=" '{print $2}'`
   fi
   logdirpath=`dirname $CATALOGBACKUPPATH`
   logdir=`basename $CATALOGBACKUPPATH`
   if [ -d "$CATALOGBACKUPPATH" ] && [ ! -z "$LOGMOUNTPATH" ]; then
      tdate=`date +%Y%m%d%H%M%S`
      mv $CATALOGBACKUPPATH $logdirpath/$logdir'.'$tdate
      createdir=false
   fi
   if [ ! -d "$logdirpath" ]; then
      createdir=true
      mkdir -p $logdirpath
   fi
   if [ ! -z "$LOGMOUNTPATH" ]; then
      ln -s $LOGMOUNTPATH $logdirpath/$logdir
      chown -R $dbsidadm:sapsys $logdirpath
   fi
fi

echo "********** call HANA DB recovery ************"

su - $dbsidadm -c "$WPWD/ACT_HANADB_RESTORE_saphana_recover.sh $DBSID $DBUSER $HANAVERSION \"$RECOVERYTIME\" $LOGMOUNTPATH $DATAPATH"

retval=$?

if [ "$createdir" = "true" ]; then
   if [ ! -z "$logdirpath" ]; then
      rm -rf $logdirpath
   fi
elif [ "$createdir" = "false" ]; then
     rm -f $logdirpath/$logdir
     mv $logdirpath/$logdir'.'$tdate $logdirpath/$logdir
fi

if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Post restore: Failed to recover database $DBSID: check customapp-saphana.log for details"
    exit $retval
fi

echo "********** end HANA DB recover with exit code: $retval ************"
exit 0


