#!/bin/bash

set -x

Function=$1
DBSID=$2
USERTYPE=$3
DBUSER=$4
PASSWD=$5

if [[ "$Function" != "-install" ]] && [[ "$Function" != "-uninstall" ]] && [[ "$Function" != "-help" ]]
then

while true
   do
read -p "Please enter the function usage for this setup execution [-install] [-help] or [-uninstall] : " Function
  if [[ "$Function" == "-install" ]] || [[ "$Function" == "-uninstall" ]] || [[ "$Function" == "-help" ]]
  then
  echo "The selected function is $Function"
  break
  else
    echo "non-acceptable entry ... you didn't type [-install] [-help] or [-uninstall]"
  fi
    done
fi

if [[ $Function == "-help" ]]
then 
echo ""
echo "******************************"
echo "ACTIFIO BACKINT SETUP - HELP"
echo "******************************"
echo ""
echo "Usage of Setup backint configuration, setup.sh [-Function] [DBSID] [KEY/DBUSER] [DBusername/KeyEntry] [DBpasswd]" 
echo ""
echo "example for KEY installation : setup.sh -install SR1 KEY BACKUPKEY"
echo "example for DBUSER installation : setup.sh -install SR1 DBUSER SYSTEM PASSw0rd"
echo "example for KEY uninstallation : setup.sh -uninstall SR1 KEY BACKUPKEY"
echo "example for DBUSER uninstallation : setup.sh -uninstall SR1 DBUSER SYSTEM PASSw0rd"
echo ""
echo "WARNING"
echo ""
echo "A prerequisite of SSH key is needed to be able to use the installation in silent mode, with all servers nodes (local node also i.e. ssh-keygen && cat \$HOME/.ssh/id_rsa.pub >> \$HOME/.ssh/authorized_keys)"
echo ""
exit 0
fi

if [[ "X$DBSID" != "X" ]]
then 
length=`expr length "$DBSID"`
if [[ "$length" != 3 ]]
then
echo "invalid argument for SID, please type setup.sh -help for setup usage"
exit 0
fi
fi

echo ""
echo "**************************"
echo "ACTIFIO BACKINT SETUP"
echo "**************************"
echo ""
echo "This setup will configure BACKINT for SAP Hana"
echo ""

if [[ "X$DBSID" == "X" ]]
then
echo "please enter the SID (instance ID 3 characters) for your Hana database : "
read -p ""  DBSID
fi

length=`expr length "$DBSID"`

if [[ "$length" != 3 ]]
then 
echo "invalid length of SID"
exit 0
else 
DBSID=`echo $DBSID | tr '[a-z]' '[A-Z]'`
echo "The SID is $DBSID"
fi

#if [[ "X$BACKUPLOGDIR" == "X" ]]
#then 
#echo ""
#read -p "Where do you want to backup log backup files : " BACKUPLOGDIR
#fi

if [[ "X$USERTYPE" == "X" ]]
then
echo ""

while true
   do
read -p "Please enter a Type of user connection : userstore Key [KEY] or Database USER [DBUSER] : " USERTYPE
  if [[ "$USERTYPE" == "KEY" ]] || [[ "$USERTYPE" == "DBUSER" ]]
  then
  echo "$USERTYPE"
  break
  else
    echo "non-acceptable entry ... you didn't type [DBUSER] or [KEY]"
  fi
    done
fi

if [[ "X$USERTYPE" == "XDBUSER" ]]
then
  if [[ "X$DBUSER" == "X" ]]
   then
   echo ""
   read -p "Please enter a Database user with INIFILE admin authorizations : " DBUSER
  fi

  if [[ "X$PASSWD" == "X" ]]
   then
   echo ""
   echo "Please enter $DBUSER password : "
   stty -echo
   read PASSWD
   stty echo
   echo ""
  fi

else
  if [[ "X$USERTYPE" == "XKEY" ]]
  then 
    if [[ "X$DBUSER" == "X" ]]
    then
    echo ""
    read -p "Please enter a Userstore KEY : " BACKINTCONFUSER
    else 
    BACKINTCONFUSER=$DBUSER
    fi
  fi
fi


DBSIDlow=`echo "$DBSID" | awk '{print tolower($0)}'`
DBSIDupper=`echo "$DBSID" | awk '{print toupper($0)}'`
HANAuser=`echo "$DBSIDlow""adm"`


if id "$HANAuser" >/dev/null 2>&1
 then echo "user $HANAuser exists"
else echo "user $HANAuser does not exist" 
 exit 0
fi

SAPSYSTEMNAME=`su - "$HANAuser" bash -c 'echo $SAPSYSTEMNAME'`

BACKUPLOGDIR=`grep -wi "^basepath_logbackup" /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |awk -F"=" '{print $2}'`

#SERVERS_NODE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | cut -d'=' -f-1 | cut -c7- | rev | cut -c2- | rev`
SERVERS_NODE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini | grep roles_ | cut -d'=' -f-1 | cut -c7- | xargs`

echo "Backint will be installed on this server(s) node(s) : "
echo "$SERVERS_NODE"

NUMBER_OF_NODE=`echo "$SERVERS_NODE" | wc -l`

if [[ $NUMBER_OF_NODE == 0 ]]
then echo "ERRORMSG -- The server node is not detected - abort"
exit 1
else 
  if [[ $NUMBER_OF_NODE == 1 ]]
  then 

    if [[ ! -d /act/custom_apps/saphana/backintlog ]]
    then mkdir /act/custom_apps/saphana/backintlog
    else echo "directory /act/custom_apps/saphana/backintlog already exist"
    fi
 
  else

    while IFS= read -r line
    do
      TARGET=`echo "$line"`
      echo "$TARGET"


    TARGET_CHECK=`cat ~/.ssh/authorized_keys | grep $TARGET`
      if [[ X$TARGET_CHECK == X ]]
      then echo "ERRORMSG -- The ssh key for $TARGET is not set as requested in prerequsites"
      exit 1
      fi

    ssh -o "StrictHostKeyChecking=no" $TARGET "
    if [[ ! -d /act/custom_apps/saphana/backintlog ]]
    then mkdir /act/custom_apps/saphana/backintlog
    else echo \"directory /act/custom_apps/saphana/backintlog already exist\"
    fi"

    done < <(printf '%s\n' "$SERVERS_NODE")
  fi
fi

if [[ "$DBSIDupper" != "$SAPSYSTEMNAME" ]]
then echo "the Database SID $DBSIDupper doesn't exist for user $HANAuser"
  exit 0
fi

group=`id -g -n $HANAuser`

if [[ $NUMBER_OF_NODE == 1 ]]
then

chown "$HANAuser":"$group" /act/custom_apps/saphana/backintlog

chown "$HANAuser":"$group" /act/custom_apps/saphana/sapbackint/Bfunction
chown "$HANAuser":"$group" /act/custom_apps/saphana/sapbackint/Rfunction
chown "$HANAuser":"$group" /act/custom_apps/saphana/sapbackint/backint

chmod +x /act/custom_apps/saphana/sapbackint/Bfunction
chmod +x /act/custom_apps/saphana/sapbackint/Rfunction
chmod +x /act/custom_apps/saphana/sapbackint/backint


else
  while IFS= read -r line
  do
      TARGET=`echo "$line"`
      echo "$TARGET"
  ssh -o "StrictHostKeyChecking=no" $TARGET "
  chown \"$HANAuser\":\"$group\" /act/custom_apps/saphana/backintlog

  chown \"$HANAuser\":\"$group\" /act/custom_apps/saphana/sapbackint/Bfunction
  chown \"$HANAuser\":\"$group\" /act/custom_apps/saphana/sapbackint/Rfunction
  chown \"$HANAuser\":\"$group\" /act/custom_apps/saphana/sapbackint/backint

  chmod +x /act/custom_apps/saphana/sapbackint/Bfunction
  chmod +x /act/custom_apps/saphana/sapbackint/Rfunction
  chmod +x /act/custom_apps/saphana/sapbackint/backint

  first_char=`echo "$BACKUPLOGDIR" | cut -c1-1`
  if [[ "X$first_char" == "X " ]]
  then BACKUPLOGDIR=`echo $BACKUPLOGDIR | awk '{print substr($0,1,length($0))}'`
  fi


  if [[ ! -d \"$BACKUPLOGDIR\" ]]
  then 
  mkdir -p \"$BACKUPLOGDIR\"

    if [[ ! -d \"$BACKUPLOGDIR\" ]]
    then 
    echo \"\"$BACKUPLOGDIR\" IS NOT created\"
    exit 0
    else echo \"\"$BACKUPLOGDIR\" created successfully\"
    fi
  else echo \"Backup log destination directory \"$BACKUPLOGDIR\" already exist\"
  fi
  "
  done < <(printf '%s\n' "$SERVERS_NODE")
fi

echo "export DB_BACKUP_PATH="$BACKUPLOGDIR"" > /act/custom_apps/saphana/sapbackint/actifio_backint.par
echo "export DB_BACKUP_PATH="$BACKUPLOGDIR"" > /act/custom_apps/saphana/sapbackint/actifio_backint_log.par
echo "export BS=10M" >> /act/custom_apps/saphana/sapbackint/actifio_backint_log.par
chown $HANAuser:sapsys /act/custom_apps/saphana/sapbackint/actifio_backint_log.par
chown $HANAuser:sapsys /act/custom_apps/saphana/sapbackint/actifio_backint.par

if [ ! -d /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt ]; then
   su - "$HANAuser" -c "mkdir -p /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt"
fi

if [[ ! -f /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/hdbbackint ]]
then
su - "$HANAuser" -c "ln -s /act/custom_apps/saphana/sapbackint/backint /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/hdbbackint"
else echo "link /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/hdbbackint already exist"
fi

if [[ ! -f /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint.par ]]
then
su - "$HANAuser" -c "ln -s /act/custom_apps/saphana/sapbackint/actifio_backint.par /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint.par"
else echo "link /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint.par already exist"
fi

if [[ ! -f /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint_log.par ]]
then
su - "$HANAuser" -c "ln -s /act/custom_apps/saphana/sapbackint/actifio_backint_log.par /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint_log.par"
else echo "link /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/opt/actifio_backint_log.par already exist"
fi

HOSTNAME=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini |grep active_master | cut -c17- | awk -v FS=':' '{print $1}'`
echo $HOSTNAME
SYSNR=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/nameserver.ini |grep active_master | cut -c17- | awk -v FS=':' '{print $2}'| cut -c-3 | cut -c2-`
HOSTPORT="$HOSTNAME"":"3"$SYSNR"13

if [[ "X$USERTYPE" == "XDBUSER" ]]
then
stty -echo
USEREXIST=`su - "$HANAuser" bash -c "hdbsql -u ${DBUSER} -p ${PASSWD} \"select * from dummy\"" | head -n1`
stty echo
#USEREXIST=`cat /tmp/.USEREXIST_$$.txt`
#rm -f /tmp/.USEREXIST_$$.txt

if [[ "X$USEREXIST" == "XDUMMY" ]]
then
 echo ""
 echo "user "$DBUSER" exist and is able to connect to database"
 echo ""
else
 echo ""
 echo "user "$DBUSER" doesn't exist or password is wrong"
 echo "the setup will exit"
 echo ""
 exit 0
fi

stty -echo
su - "$HANAuser" bash -c "hdbuserstore set BACKINTCONFUSER ${HOSTPORT} ${DBUSER} ${PASSWD}"
stty echo
echo "hdbuserstore set BACKINTCONFUSER "$HOSTPORT" "$DBUSER" ****** "

BACKINTCONFUSER=BACKINTCONFUSER
fi

if [[ "X$USERTYPE" == "XKEY" ]]
then 
KEYEXIST=`su - "$HANAuser" bash -c "hdbsql -U ${BACKINTCONFUSER} \"select * from dummy\"" | head -n1`

#KEYEXIST=`cat /tmp/.KEYEXIST_$$.txt`
#rm -f /tmp/.KEYEXIST_$$.txt

  if [[ "X$KEYEXIST" == "XDUMMY" ]]
  then
   echo ""
   echo "KEY "$BACKINTCONFUSER" exist and is able to connect to database"
   echo ""
  else
   echo ""
   echo "KEY "$BACKINTCONFUSER" doesn't exist or password combinaison is wrong"
   echo "the setup will exit"
   echo ""
   exit 0
  fi
fi
function_param()
{

CT_USING_BACKINT=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |grep ^catalog_backup_using_backint | cut -c32- | awk -v FS=':' '{print $1}'`
echo "catalog_backup_using_backint = $CT_USING_BACKINT"

CT_LOG_PAR_FILE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |grep ^catalog_backup_parameter_file | cut -c33- | awk -v FS=':' '{print $1}'`
echo "catalog_backup_parameter_file = $CT_LOG_PAR_FILE"


DATA_PAR_FILE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |grep ^data_backup_parameter_file | cut -c30- | awk -v FS=':' '{print $1}'`
echo "data_backup_parameter_file = $DATA_PAR_FILE"

LOG_PAR_FILE=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |grep ^log_backup_parameter_file | cut -c29- | awk -v FS=':' '{print $1}'`
echo "log_backup_parameter_file = $LOG_PAR_FILE"


LOG_BK_USING_BINT=`cat /usr/sap/$SAPSYSTEMNAME/SYS/global/hdb/custom/config/global.ini |grep ^log_backup_using_backint | cut -c28- | awk -v FS=':' '{print $1}'`
echo "log_backup_using_backint = $LOG_BK_USING_BINT"
}

function_recover_config()
{
echo "#!/bin/bash"
echo "set -x"
echo "su - \"$HANAuser\" bash -c \"hdbsql -U $BACKINTCONFUSER \\\"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_using_backint') = '$CT_USING_BACKINT' WITH RECONFIGURE;\\\"\""

echo "su - \"$HANAuser\" bash -c \"hdbsql -U $BACKINTCONFUSER \\\"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_parameter_file') = '$CT_LOG_PAR_FILE' WITH RECONFIGURE;\\\"\""

echo "su - \"$HANAuser\" bash -c \"hdbsql -U $BACKINTCONFUSER \\\"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'data_backup_parameter_file') = '$DATA_PAR_FILE' WITH RECONFIGURE;\\\"\""

echo "su - \"$HANAuser\" bash -c \"hdbsql -U $BACKINTCONFUSER \\\"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_parameter_file') = '$LOG_PAR_FILE' WITH RECONFIGURE;\\\"\""

echo "su - \"$HANAuser\" bash -c \"hdbsql -U $BACKINTCONFUSER \\\"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_using_backint') = 'true' WITH RECONFIGURE;\\\"\""
}


function_install()
{

echo ""
echo "Adapt database parameter configuration"

function_param 
function_recover_config > /act/custom_apps/saphana/backintlog/${DBSIDupper}_config_recover
chmod +x /act/custom_apps/saphana/backintlog/${DBSIDupper}_config_recover

	if [[ "X$CT_USING_BACKINT" != "Xtrue"  ]]
	then 
echo ""
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_using_backint') = 'true' WITH RECONFIGURE;\""
	fi


	if [[ "X$CT_LOG_PAR_FILE" != "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par"  ]]
	then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_parameter_file') = '/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par' WITH RECONFIGURE;\""
	fi

	if [[ "X$DATA_PAR_FILE" != "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint.par"  ]]
	then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'data_backup_parameter_file') = '/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint.par' WITH RECONFIGURE;\""
	fi

	if [[ "X$LOG_PAR_FILE" != "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par" ]]
	then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_parameter_file') = '/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par' WITH RECONFIGURE;\""
	fi

	if [[ "$LOG_BK_USING_BINT" != "true" ]]
	then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_using_backint') = 'true' WITH RECONFIGURE;\""
	fi
}

function_uninstall()
{

echo ""
echo "Reset database parameter configuration"

function_param

        if [[ "X$CT_USING_BACKINT" != "Xfalse"  ]]
        then
echo ""
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_using_backint') = 'false' WITH RECONFIGURE;\""
	fi

        if [[ "X$CT_LOG_PAR_FILE" == "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par"  ]]
        then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'catalog_backup_parameter_file') = '' WITH RECONFIGURE;\""
	fi

        if [[ "X$DATA_PAR_FILE" == "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint.par"  ]]
        then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'data_backup_parameter_file') = '' WITH RECONFIGURE;\""
	fi

        if [[ "X$LOG_PAR_FILE" == "X/usr/sap/${SAPSYSTEMNAME}/SYS/global/hdb/opt/actifio_backint_log.par" ]]
        then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_parameter_file') = '' WITH RECONFIGURE;\""
	fi

        if [[ "$LOG_BK_USING_BINT" != "false" ]]
        then
su - "$HANAuser" bash -c "hdbsql -U $BACKINTCONFUSER \"ALTER SYSTEM ALTER CONFIGURATION ('global.ini', 'SYSTEM') SET ('backup', 'log_backup_using_backint') = 'false' WITH RECONFIGURE;\""
	fi
}

if [[ "$Function" == "-install" ]]
then function_install
elif [[ "$Function" == "-uninstall" ]]
then function_uninstall
fi

if [[ "X$USERTYPE" == "XDBUSER" ]]
then
su - "$HANAuser" bash -c "hdbuserstore delete BACKINTCONFUSER"
fi
