#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
#                  !!!Actifio Private Script. DO NOT MODIFY!!!
#
# ==============================================================================#

export COLUMNS=4000
unset LANG
unset LC_ALL

isProvided() {
  local varName=$1
  local varVal=$2
  if [ -z "$varVal" ]; then
    echo "ERRORMSG: $varName is empty."
    usage
    exit 1
  fi
}

usage() {
  set +x
  echo "Usage: "
  echo "/act/scripts/CALL_ACT_SYBASE_newtargetdbrestore.sh \"\" <OSUSER> <TARGET_SERVER_NAME> <TARGET_SYBASE_SQLD> <TARGET_DB_USER> <TARGET_DBUSER_PASSWD/null> <TARGET_DBUSER_OTHERCRED/null> <SRC_DBNAME> [ <UNTIL_TIME> ] "
  set -x
  exit 1 
}

set -x

echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ "$ACT_JOBTYPE" = "unmount" ] || [ "$ACT_MULTI_OPNAME" = "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ "$ACT_JOBTYPE" = "mount" ] || [ "$ACT_MULTI_OPNAME" = "mount" ] ; then
   if [ "$ACT_PHASE" = "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ "$ACT_PHASE" = "post" ] ; then
        echo "************************* start post during mount ************************"


OSUSER=$2			#User will provide
TARGET_SERVER_NAME=$3		#User will provide
TARGET_SYBASE_SQLD=$4		#User will provide
TARGET_DB_USER=$5		#User will provide
set +x
TARGET_DBUSER_PASSWD=$6		#User will provide
set -x
TARGET_DBUSER_OTHERCRED=$7	#User will provide
SRC_DBNAME=$8			#User will provide
UNTIL_TIME="$9"			#User will provide

if [ ! -z "$ACT_JOBNAME" ] && [ -z "$JOBID" ]; then
  JOBID=$ACT_JOBNAME
fi

if [ -z $JOBID ]; then
  echo "ERRORMSG: JOBID not provided"
  exit 1
fi

#Read the parameters passed by connector
PARAMETER_FILE=`ls /act/touch/*_"$JOBID"_params | head -1`
if [ ! -f "$PARAMETER_FILE" ]; then
  echo "ERRORMSG: $PARAMETER_FILE not exists."
  exit 1
fi

TARGET_MNT_PNT=`grep MOUNTPOINTROOT $PARAMETER_FILE | cut -d= -f2`	#Connector can provide
LOG_BKP_MNTPT=`grep ARCHIVELOGMOUNTPATH $PARAMETER_FILE | cut -d= -f2`	#Connector can provide
BEGIN_TIME=`grep BEGIN_TIME $PARAMETER_FILE | cut -d= -f2`	#Connector can provide


set +x
isProvided OSUSER $OSUSER
isProvided TARGET_SERVER_NAME $TARGET_SERVER_NAME
isProvided TARGET_SYBASE_SQLD $TARGET_SYBASE_SQLD
isProvided TARGET_DB_USER $TARGET_DB_USER
isProvided SRC_DBNAME $SRC_DBNAME
isProvided TARGET_MNT_PNT $TARGET_MNT_PNT

if [ ! -z "$UNTIL_TIME" ]; then
  isProvided LOG_BKP_MNTPT $LOG_BKP_MNTPT
  isProvided BEGIN_TIME "$BEGIN_TIME"
fi

if [ -z "$TARGET_DBUSER_PASSWD" ] && [ -z "$TARGET_DBUSER_OTHERCRED" ]; then
  echo "ERRORMSG: Both TARGET_DBUSER_PASSWD and TARGET_DBUSER_OTHERCRED are not provided. Need at least one of these two."
  exit 1
fi

set -x


echo "********** call SybaseASE recovery ************"
set +x
/act/custom_apps/sybase/restore/act_sybase_lvm_customdb_recovery.sh OSUSER=$OSUSER TARGET_SYBASE_SQLD=$TARGET_SYBASE_SQLD TARGET_MNT_PNT=$TARGET_MNT_PNT TARGET_SERVER_NAME=$TARGET_SERVER_NAME TARGET_DB_USER=$TARGET_DB_USER STRIPEON=$STRIPEON TARGET_DBUSER_PASSWD="$TARGET_DBUSER_PASSWD" SRC_DBNAME=$SRC_DBNAME LOG_BKP_MNTPT=$LOG_BKP_MNTPT UNTIL_TIME="$UNTIL_TIME" BEGIN_TIME="$BEGIN_TIME" JOBID=$JOBID TARGET_DBUSER_OTHERCRED="$TARGET_DBUSER_OTHERCRED"
retval=$?
set -x

if [ $retval -ne 0 ]; then
    echo "ERRORMSG: fail to perform single db restore."
    exit $retval
fi

echo "********** end SybaseASE recover with exit code: $retval ************"
exit 0
fi
fi

