#!/bin/sh
set -x

OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
set -x
SRC_USER_OTHERCRED=$SRC_USER_OTHERCRED
SRC_SERVER_NAME=$SRC_SERVER_NAME
LOGBKPLOC=$LOGBKPLOC
DATAVOLUMEOWNER=$DATAVOLUMEOWNER
USER_DBLIST="$USER_DBLIST"
STRIPEON=$STRIPEON

#----------------------------------------------------------------------
#Get dump history
#----------------------------------------------------------------------
getLogDumpHist() {
  local tStmp=$1
  local outFile=$2

  set +x
  dumpHistory=`su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 10000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
sp_dump_history 'list', @dump_type = 'TRAN'
go
retval=$?
EOF`
  set -x

  set +x
  retval=`echo -e "$dumpHistory" | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  set -x
  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Fail to get dump history"
    return 1
  fi

  set +x
  echo -e "Dump history: \n $dumpHistory" | grep $tStmp >> $outFile
  set -x

  return 0
}
#----------------------------------------------------------------------

## Checking env variables

if [ "$SRC_BACKUP_USER" = "" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "$SRC_USER_PASSWD" = "''" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ -z $SRC_SERVER_NAME ]; then
  echo "ERRORMSG: SRC_SERVER_NAME is not provided"
  exit 1
fi

if [ -z $LOGBKPLOC ]; then
  echo "ERRORMSG: LOGBKPLOC is not provided"
  exit 1
fi

if [ -z $USER_DBLIST ]; then
  echo "ERRORMSG: USER_DBLIST is empty"
  exit 1
fi

if [ -z $OSUSER ]; then
  OSUSER=`ps -ef | grep -i "bin/dataserver"| grep $SRC_SERVER_NAME | grep -v -w grep | awk '{print $1}'|uniq|xargs`
fi

if [ -z $DATAVOLUMEOWNER ]; then
  OSGROUP=`id $OSUSER | awk '{print $NF}' |cut -d'(' -f2 | cut -d')' -f1`
  DATAVOLUMEOWNER="$OSUSER:$OSGROUP"
fi

if [ -z $SRC_SYBASE_SQLD ]; then
  SRC_SYBASE_ASE=`ps -ef | grep "bin/dataserver"|grep $SRC_SERVER_NAME | grep -v -w grep | awk -F'bin/dataserver' '{print $1}' | grep -v -w awk | awk '{print $NF}' | head -1`
  SRC_SYBASE_SQLD="$SRC_SYBASE_ASE/.."/`cat $SRC_SYBASE_ASE/../SYBASE.sh | grep SYBASE_OCS | grep -v export | sed 's/"//g' | awk -F "=" '{print $2}'`
fi

#Validate LOGBKPLOC location
if [ ! -d "$LOGBKPLOC" ]; then
  echo "ERRORMSG: Log backup location (LOGBKPLOC=$LOGBKPLOC) is not a directory."
  exit 1
fi

# Validate OSuser existence
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: User $OSUSER not found"
  exit 1
fi

# Validate SRC_SYBASE_SQLD
if [ ! -f $SRC_SYBASE_SQLD/../SYBASE.sh ]; then
  echo "ERRORMSG: SRC_SYBASE_SQLD ($SRC_SYBASE_SQLD) is not valied location"
  exit 1
fi

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#---------------------------------------------
# validate user credintials
#---------------------------------------------
set +x
. $SRC_SYBASE_SQLD/../SYBASE.sh

su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" << EOF
use master
go
EOF
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Invalide SRC_BACKUP_USER or SRC_USER_PASSWD"
  exit 1
fi
#---------------------------------------------


#---------------------------------------------
#Get the available privileges
set +x
allPriviles=`su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select v.name from sysprotects p, master.dbo.spt_values v, (Select sr.lrid from sysroles sr where sr.id in (select srid from sysloginroles where suid in (select suid from syslogins where name in (select suser_name())))) s where p.action=v.number and p.protecttype=1 and v.type = 'GP' and s.lrid=p.uid
go
EOF`
retval=$?
set -x

retval=`echo $allPriviles | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to find dbuser ($SRC_BACKUP_USER) priviletes"
  exit 1
fi

#Check for required privileges
set +x
hasDmAD=`echo -e "$allPriviles" | grep " Dump Any Database " | wc -l`
if [ $hasDmAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Dump Any Database' privileges"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMSC=`echo -e "$allPriviles" | grep " Manage Server Configuration " | wc -l`
if [ $hasMSC -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Manage Server Configuration' privileges. This is required to 'enable dump history'"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMDC=`echo -e "$allPriviles" | grep " Manage Dump Configuration " | wc -l`
if [ $hasMDC -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Manage Dump Configuration' privileges. This is required to read sp_dump_history results"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

#hasDAT=`echo -e "$allPriviles" | grep " Drop Any Table " | wc -l`
#if [ $hasDAT -lt 1 ]; then
#  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Drop Any Table' privileges. This is required to read sp_dump_history results"
#  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
#  exit 1
#fi
set -x
#---------------------------------------------

#Check dump backup histrory status
set +x
isDBH_On=`su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
Select value from master.dbo.sysconfigures where name='enable dump history'
go
EOF`
retval=$?
set -x

retval=`echo $isDBH_On | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to find config value for 'enable dump history'"
  exit 1
fi

if [ $isDBH_On -ne 1 ]; then
  echo "INFOMSG: 'enable dump history' is OFF, making on..."

  set +x
  su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF
set nocount on
go
use master
go
sp_configure 'enable dump history', 1
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to change config 'enable dump history'"
    exit 1
  fi
fi


## Change owner to mounted log backup location
chown -R $DATAVOLUMEOWNER $LOGBKPLOC
set +x
. $SRC_SYBASE_SQLD/../SYBASE.sh
set -x

echo "USER_DBLIST: $USER_DBLIST"

logbkptmp=$LOGBKPLOC
dumpHistFile=/act/touch/dumpHist_$JOBID.txt

#For loop for two log backups
for x in {1..2}
do
  echo "Logbackup set $x ..."
  if [ $x -eq 2 ]; then
     sleep 1
  fi

echo "" > $dumpHistFile
for dbname in $(echo $USER_DBLIST | sed "s/,/ /g")
do
  LOGBKPLOC=$logbkptmp
  LOGBKPLOC=$LOGBKPLOC/$dbname
  if [ ! -d $LOGBKPLOC ]; then
     mkdir -p $LOGBKPLOC
     chown -R $DATAVOLUMEOWNER $LOGBKPLOC
  fi
  tsname=`date +%Y%m%d`
  tsname1=`date +%H%M%S`
  a="dump transaction $dbname to"
  lf="$dbname"_trans_"$tsname"_"$tsname1"
  if [ -z "$STRIPEON" ] || [ $STRIPEON -eq 0 ]; then
    STRIPEON=1
  fi
  b=""
  c="STRIPE ON"
  #Iterating on number of STRIPE ON
  for (( j=1;  j<= $STRIPEON; ++j ))
  do
      if [ "$STRIPEON" -le "9" ]; then
        if [ "$j" -eq "$STRIPEON" ];then
          b=`echo "$b \"$LOGBKPLOC"/"$lf"_S0"$j"-0"$STRIPEON".cdump"\""`
        else
          b=`echo "$b \"$LOGBKPLOC"/"$lf"_S0"$j"-0"$STRIPEON".cdump"\" $c"`
        fi
      else
          if [ "$j" -le "9" ];then
           if [ "$j" -eq "$STRIPEON" ];then
             b=`echo "$b \"$LOGBKPLOC"/"$lf"_S0"$j"-"$STRIPEON".cdump"\""`
           else
             b=`echo "$b \"$LOGBKPLOC"/"$lf"_S0"$j"-"$STRIPEON".cdump"\" $c"`
           fi
          else
           if [ "$j" -eq "$STRIPEON" ];then
            b=`echo "$b \"$LOGBKPLOC"/"$lf"_S"$j"-"$STRIPEON".cdump"\" "`
           else
             b=`echo "$b \"$LOGBKPLOC"/"$lf"_S"$j"-"$STRIPEON".cdump"\" $c"`
           fi
          fi
      fi
   done

  if [ -f $LOGBKPLOC/syb_backup_dump ]; then
     rm -f $LOGBKPLOC/syb_backup_dump
  fi
  echo "$a" "$b" >> $LOGBKPLOC/syb_backup_dump
  echo go >> $LOGBKPLOC/syb_backup_dump
  chown $DATAVOLUMEOWNER $LOGBKPLOC/syb_backup_dump

  echo "Running logbackup for database $dbname from script ...... "
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror -i $LOGBKPLOC/syb_backup_dump "
  retval=$?
  set -x
  if [ "$retval" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform Transaction log backup for the database $dbname. Please check the logs!"
     exit 1
  fi

  if [ -f $LOGBKPLOC/syb_backup_dump ]; then
     rm -f $LOGBKPLOC/syb_backup_dump
  fi

  if [ $x -eq 2 ]; then
    getLogDumpHist $lf $dumpHistFile
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Log backup not registered in sp_dump_history."
      exit 1
    fi
  fi

done


  # Display history only in second iteration
  if [ $x -eq 2 ]; then
    echo "Dump history:"
    cat $dumpHistFile

    # Calculate minimum dump date from dump history
    MIN_DUMP_TIME=`grep TRAN $dumpHistFile | tr -s ' ' | awk '{print $6"-"$5"-"$7" "$8}' | sed 's/:/ /3' | awk '{$3 = substr($3,4,2)} 1' | sed 's/ //2' | while read line; do date -d "$line" +%Y%m%d%H%M%S ; done | sort | uniq | head -1`
    MIN_DUMP_TIME=`echo ${MIN_DUMP_TIME:0:4}"-"${MIN_DUMP_TIME:4:2}"-"${MIN_DUMP_TIME:6:2}" "${MIN_DUMP_TIME:8:2}":"${MIN_DUMP_TIME:10:2}":"${MIN_DUMP_TIME:12:2}`
    MIN_DUMP_TIME=`date -d "-1 seconds $MIN_DUMP_TIME" "+%Y-%m-%d %H:%M:%S"`
    echo "MIN_DUMP_TIME_final=$MIN_DUMP_TIME"

    #End of loop no. of log backups
  fi
done

rm $dumpHistFile
echo "INFOMSG: Log backup successfully completed."

exit 0
