#!/bin/sh

set -x

ACT_JOBNAME=$JOBID
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
set -x
SRC_USER_OTHERCRED=$SRC_USER_OTHERCRED
SRC_SERVER_NAME=$SRC_SERVER_NAME
SRC_DBNAME=$USER_DBLIST
SYS_DBLIST="$sysdblist"
INDIVIDUAL=$INDIVIDUAL

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#---------------------------------------------
# function: is_DB_Quiesced()
#---------------------------------------------
is_DB_Quiesced() {
  local dbList=$1
  local dbListF=

  for dbn in $(echo $dbList | sed 's/,/ /g')
  do
    if [ "$dbListF" = "" ]; then
      dbListF="'$dbn'"
    else
      dbListF="$dbListF,'$dbn'"
    fi
  done

  set +x
  local dbStat=`su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select distinct status3 from sysdatabases where name in ($dbListF)
go
EOF`
  local retval=$?
  set -x

  retval=`echo $dbStat | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    return 2
  fi

  dbStat=`echo $dbStat | xargs`
  if [ "$dbStat" != "131200" ]; then
    return 1
  fi

  return 0
}
#---------------------------------------------

#### function - releaseDB
releaseDB() {

  tagName=$1
  echo "COMMAND: quiesce database $tagName release"
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" <<EOF
use master
go
quiesce database $tagName release
go
exit
EOF
  retval=$?
  set -x

  return $retval

}
#####################


DATE=`date`
echo "************************** DATE:$DATE: Prearing for UnFreeze sybase databases in server $SRC_SERVER_NAME  ******************"

#Prepare a tag name for database quiesce
tagname="tag_$ACT_JOBNAME"                             #User database unquiesce
tagname_mst="mstDump_$ACT_JOBNAME"
echo tag: $tagname
FREEZE_INFO=/act/touch/ACT_FREEZE_${SRC_SERVER_NAME}
if [ ! -e "$FREEZE_INFO" ]; then
  echo "ERRORMSG: Not freezed by Actifio. ($FREEZE_INFO not exists)"
  exit 1
fi
tagname_mst=`grep "mstDump_" $FREEZE_INFO | awk '{print $3}'`


#set sybase env variables
set +x
source $SRC_SYBASE_SQLD/../SYBASE.sh
set -x


is_DB_Quiesced $SYS_DBLIST
if [ $? -ne 0 ]; then
  echo "ERRORMSG: System DBs ($SYS_DBLIST) not in quiesce state."
  exit 1
fi

echo "************************** DATE:$DATE: unFreezing system databases in server $SRC_SERVER_NAME ******************"
releaseDB $tagname_mst
if [ $? -gt 0 ]; then
  echo "ERRORMSG: Failed to release master database! "
  exit 1
fi


echo "************************** DATE:$DATE: unFreezing user database(s) in server $SRC_SERVER_NAME ******************"
if [ "$INDIVIDUAL" != "NO" ]; then

  for dbName in $(echo $SRC_DBNAME | sed 's/,/ /g')
  do
    is_DB_Quiesced $dbName
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: DB $dbName not in quiesce state."
      exit 1
    fi

    tagnameDB=$dbName
    releaseDB $tagnameDB
    if [ $? -gt 0 ]; then
      echo "ERRORMSG: Failed to Release database!"
      exit 1
    fi
  done

else

  releaseDB $tagname
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: Failed to Release database!"
    exit 1
  fi

fi


#Create backupstatus.xml to tracke individual db backup status (success or fail)
if [ ! -d /act/tmpdata/$JOBID ]; then
  mkdir -p /act/tmpdata/$JOBID
fi
BACKUP_STATUS_XML=/act/tmpdata/$JOBID/backupstatus.xml
echo "<tenants>" > $BACKUP_STATUS_XML
for dbName in $(echo $SRC_DBNAME | sed 's/,/ /g')
do
  echo "<tenant name=\"$dbName\" status=\"success\" />" >> $BACKUP_STATUS_XML
done
echo "</tenants>" >> $BACKUP_STATUS_XML


#Remove freeze_info file
if [ -e "$FREEZE_INFO" ]; then
  rm -f $FREEZE_INFO
fi

exit $retval
