#!/bin/sh

set -x
ACT_JOBNAME=$JOBID
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
set -x
SRC_USER_OTHERCRED=$SRC_USER_OTHERCRED
SRC_SERVER_NAME=$SRC_SERVER_NAME
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC
SRC_DBNAME="$USER_DBLIST"
SYS_DBLIST="$sysdblist"
INDIVIDUAL=$INDIVIDUAL

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#---------------------------------------------------------------------------------------------------------
# function: isFreezed_sysdb()
#---------------------------------------------------------------------------------------------------------
isFreezed_sysdb() {
  set +x
  isFreezed_sysdb=`su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 20000 --retserverror" <<EOF | grep -w "quiesce database"|wc -l
use master
go
sp_helpdb master
go
EOF`
  retval=$?
  set -x

  retval=`echo $isFreezed_sysdb | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get master database quiesce state"
    exit 1
  fi

  if [ $isFreezed_sysdb -gt 0 ]; then
    echo "FYIMSG: System dbs are freezed."
    return 1
  else
    echo "FYIMSG: System dbs are not freezed."
    return 0
  fi
}
#---------------------------------------------------------------------------------------------------------

#---------------------------------------------------------------------------------------------------------
#### function - quiesceDB
#---------------------------------------------------------------------------------------------------------
quiesceDB() {

  tagName=$1
  dbList=$2
  MF_NAME=$3

  echo "COMMAND: quiesce database $tagName hold $dbList for external dump to '$datadev_dir/$MF_NAME' with override"
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" <<EOF
use master
go
quiesce database $tagName hold $dbList for external dump to '$datadev_dir/$MF_NAME' with override
go
exit
EOF
  retval=$?
  set -x

  return $retval
}
#---------------------------------------------------------------------------------------------------------


if [ ! -z "$EXCLUDE_DB_LIST" ]; then
   SRC_DBNAME_TMP=`echo $SRC_DBNAME | sed 's/,/ /g'`
   for x in $(echo $EXCLUDE_DB_LIST|sed 's/,/ /g')
   do
     SRC_DBNAME_TMP=`echo $SRC_DBNAME_TMP | sed "s/$x//g"`
   done
   SRC_DBNAME=`echo $SRC_DBNAME_TMP | sed 's/ /,/g'`
fi

DATE=`date`
echo "************************** DATE:$DATE: Prearing for Freeze sybase databases in server $SRC_SERVER_NAME  ******************"

#Prepare a tag name for database quiesce
tagname="tag_$ACT_JOBNAME"                             #User database quiesce
tagname_mst="mstDump_$ACT_JOBNAME"
MANIFEST_FILE="Manifest_File_$SRC_SERVER_NAME"
FREEZE_INFO=/act/touch/ACT_FREEZE_${SRC_SERVER_NAME}

#set sybase env variables
set +x
source $SRC_SYBASE_SQLD/../SYBASE.sh
set -x

echo "SYS_DBLIST: $SYS_DBLIST"

if [ -z "$SRC_DBNAME" ]; then
   echo "ERRORMSG: No User databases are available. Please check!"
   exit 1
fi

datadev_dir=$MANIFEST_FILE_LOC
echo loc: $datadev_dir
echo tag: $tagname



# Check if System databases already freez. Wait for 3mins and then fail if still freeze
isFreezed_sysdb
if [ $? -eq 0 ]; then
  FLAG=0
else
  FLAG=1
fi
cnt=0
while [ $cnt -le 2 ] && [ $FLAG -ne 0 ]
do
  cnt=$(( cnt + 1 ))
  echo "Retry after 60 seconds...(retry "$cnt")"
  sleep 60
  isFreezed_sysdb
  if [ $? -eq 0 ]; then
    FLAG=0
    break
  fi
done

if [ $FLAG -eq 1 ]; then
  echo "ERRORMSG: master database is in QUIESCE state. Cannot proceed to backup"
  exit 1
else
  echo "Proceed to backup"
fi

#Create backupstatus.xml to tracke individual db backup status (success or fail)
if [ ! -d /act/tmpdata/$JOBID ]; then
  mkdir -p /act/tmpdata/$JOBID
fi
BACKUP_STATUS_XML=/act/tmpdata/$JOBID/backupstatus.xml
echo "<tenants>" > $BACKUP_STATUS_XML
for dbName in $(echo $SRC_DBNAME | sed 's/,/ /g')
do
  # Consider all backup failure until all success
  echo "<tenant name=\"$dbName\" status=\"failed\" error=\"Fail to quiesce (or) release the db\" />" >> $BACKUP_STATUS_XML
done
echo "</tenants>" >> $BACKUP_STATUS_XML


echo "************************** DATE:$DATE: Freezing user database(s) $SRC_DBNAME ******************"
if [ "$INDIVIDUAL" != "NO" ]; then

  ### individual manifest files creations
  for dbName in $(echo $SRC_DBNAME | sed 's/,/ /g')
  do
    tagnameDB=$dbName
    MANIFEST_FILE_DB=$MANIFEST_FILE'_'$dbName

    quiesceDB $tagnameDB $dbName $MANIFEST_FILE_DB
    if [ "$retval" -gt 0 ]; then
       echo "ERRORMSG: Unable to quiesce the database $dbName."
       exit $retval
    else
      echo "quiesce database $tagnameDB release" >> $FREEZE_INFO
    fi
  done

else

  #### Single manifest file creation
  quiesceDB $tagname $SRC_DBNAME $MANIFEST_FILE
  if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: Unable to quiesce the database(s)!."
   exit $retval
  else
    echo "quiesce database $tagname release" >> $FREEZE_INFO
  fi

fi


echo "************************** DATE:$DATE: Freezing system databases $SYS_DBLIST ******************"
echo " COMMAND: quiesce database $tagname_mst hold $SYS_DBLIST"
set +x
su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" <<EOF
use master
go
quiesce database $tagname_mst hold $SYS_DBLIST
go
exit
EOF
retval=$?
set -x
if [ "$retval" -gt 0 ]; then
  echo "ERRORMSG: Unable to quiesce system databases! "
  exit $retval
else
  echo "quiesce database $tagname_mst release">> $FREEZE_INFO
fi


exit $retval
