#!/bin/sh

set -x
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$SRC_BACKUP_USER
set +x
SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
set -x
SRC_USER_OTHERCRED=$SRC_USER_OTHERCRED
SRC_SERVER_NAME=$SRC_SERVER_NAME
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC
SRC_DBNAME="$USER_DBLIST"
SYS_DBLIST="$sysdblist"
INDIVIDUAL=$INDIVIDUAL


##Validated env
if [ "$OSUSER" = "" ]; then
  echo "ERRORMSG: Sybase OSUSER is not provided."
  exit 1
fi

if [ "$SRC_SYBASE_SQLD" = "" ]; then
  echo "ERRORMSG: Sybase isql location (SRC_SYBASE_SQLD) is not provided"
  exit 1
fi

if [ "$SRC_BACKUP_USER" = "" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "$SRC_USER_PASSWD" = "''" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ "$SRC_SERVER_NAME" = "" ];then
  echo "ERRORMSG: Sybase server name (SRC_SERVER_NAME)  is not provided"
  exit 1
fi

if [ "$MANIFEST_FILE_LOC" = "" ]; then
  echo "ERRORMSG: Manifest file location (MANIFEST_FILE_LOC) is empty. "
  exit 1
fi

if [ "$SRC_DBNAME" = "" ]; then
  echo "ERRORMSG: Sybase database list (SRC_DBNAME) is empty "
  exit 1
fi

if [ "$SYS_DBLIST" = "" ]; then
  echo "ERRORMSG: Sybase system database list (SYS_DBLIST) is empty "
  exit 1
fi
###############

# Validate OSuser existence
id $OSUSER
if [ $? -ne 0 ]; then
  echo "ERRORMSG: User $OSUSER not found"
  exit 1
fi

# Validate SRC_SYBASE_SQLD
if [ ! -f $SRC_SYBASE_SQLD/../SYBASE.sh ]; then
  echo "ERRORMSG: SRC_SYBASE_SQLD ($SRC_SYBASE_SQLD) is not valied location"
  exit 1
fi


#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------


#---------------------------------------------
# Check server running or not
#---------------------------------------------
SRC_SERVER_STATUS=`ps -ef | grep "bin/dataserver" |awk -F "-s" '{print $2}'| awk '{print $1}' | uniq | grep -w $SRC_SERVER_NAME |wc -l`
echo "SRC_SERVER_NAME status: $SRC_SERVER_STATUS"
if [ $SRC_SERVER_STATUS -ne 1 ]; then
  echo "ERRORMSG: SRC_SERVER_NAME ($SRC_SERVER_NAME) is not running"
  exit 1
fi
#---------------------------------------------


#---------------------------------------------
# validate user credintials
#---------------------------------------------
set +x
. $SRC_SYBASE_SQLD/../SYBASE.sh

su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" << EOF
use master
go
EOF
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Invalide SRC_BACKUP_USER or SRC_USER_PASSWD"
  exit 1
fi
#---------------------------------------------


#---------------------------------------------
# Validate dbuser privileges
#---------------------------------------------
set +x
displayRolesOutput=`su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF
set nocount on
go
use master
go
sp_displayroles $SRC_BACKUP_USER
go
EOF`
retval=$?
set -x

set +x
retval=`echo $displayRolesOutput | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
set -x
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to find dbuser ($SRC_BACKUP_USER) roles"
  exit 1
fi

hasSAROLE=`echo $displayRolesOutput | grep -w sa_role | wc -l`
if [ $hasSAROLE -lt 1 ]; then
  echo "WARNINGMSG: DBuser ($SRC_BACKUP_USER) do not has sa_role"
  echo -e "sp_displayroles: \n$displayRolesOutput"
else
  echo "INFOMSG: DBuser ($SRC_BACKUP_USER) has sa_role"
  #exit 0
fi

#Get the available privileges
set +x
allPriviles=`su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF
set nocount on
go
use master
go
select v.name from sysprotects p, master.dbo.spt_values v, (Select sr.lrid from sysroles sr where sr.id in (select srid from sysloginroles where suid in (select suid from syslogins where name in (select suser_name())))) s where p.action=v.number and p.protecttype=1 and v.type = 'GP' and s.lrid=p.uid
go
EOF`
retval=$?
set -x

set +x
retval=`echo $allPriviles | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
set -x
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to find dbuser ($SRC_BACKUP_USER) priviletes"
  exit 1
fi

#Check for required privileges
set +x
hasQAD=`echo -e "$allPriviles" | grep " Quiesce Any Database " | wc -l`
if [ $hasQAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Quiesce Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasOAD=`echo -e "$allPriviles" | grep " Online Any Database " | wc -l`
if [ $hasOAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Online Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasUAD=`echo -e "$allPriviles" | grep " Unmount Any Database " | wc -l`
if [ $hasUAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Unmount Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasSd=`echo -e "$allPriviles" | grep " Shutdown " | wc -l`
if [ $hasSd -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Shutdown' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasDmAD=`echo -e "$allPriviles" | grep " Dump Any Database " | wc -l`
if [ $hasDmAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Dump Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasLdAD=`echo -e "$allPriviles" | grep " Load Any Database " | wc -l`
if [ $hasLdAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Load Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMAD=`echo -e "$allPriviles" | grep " Mount Any Database " | wc -l`
if [ $hasMAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Mount Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi
set -x

#---------------------------------------------

exit 0
