#!/bin/sh

set -x
TARGET_SERVER_NAME=$TARGET_SERVER_NAME
TARGET_DB_USER=$TARGET_DB_USER
set +x
TARGET_DBUSER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
set -x
TARGET_DBUSER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
SRC_DBNAME=$SRC_DBNAME
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST
ACT_JOBNAME=$JOBID

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

tmp_loc=/act/touch/$ACT_JOBNAME
if [ ! -d $tmp_loc ]; then
  mkdir -p $tmp_loc
  chmod 755 $tmp_loc
fi

## Validate env
if [ "$TARGET_SERVER_NAME" = "" ]; then
  echo "ERRORMSG: Sybase target server name (TARGET_SERVER_NAME) is not provided."
  exit 1
fi

if [ "$TARGET_DB_USER" = "" ]; then
  echo "ERRORMSG: Sybase target server database username (TARGET_DB_USER) is not provided."
  exit 1
fi

set +x
if [ "$TARGET_DBUSER_PASSWD" = "" ]; then
  echo "ERRORMSG: Sybase target server user password (TARGET_DBUSER_PASSWD) is not provided."
  exit 1
fi
set -x

if [ "$SRC_DBNAME" = "" ]; then
  echo "ERRORMSG: SRC_DBNAME is not provided."
  exit 1
fi

###############
#hidden file to handel Mount and Migrate
ASE_MIGRATE_CONF_prepare=`echo $TARGET_SERVER_NAME,$TARGET_DBNAME_LIST | sed "s/:/ /g" | sed "s/,/ /g" | tr " " "\n" | sort | tr "\n" " " | sed "s/ /_/g"`
ASE_MIGRATE_CONF=/act/touch/.ASE_migrate_"$ASE_MIGRATE_CONF_prepare".conf

#Check unmout database required or not
if [ -f $ASE_MIGRATE_CONF ]; then
  PVMOVE_STATUS=`cat $ASE_MIGRATE_CONF | awk -F "=" '{print $2}'`
  if [ "$PVMOVE_STATUS" = "YES" ]; then
    echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
    rm -f $ASE_MIGRATE_CONF
    exit 0
  else
    echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
    exit 1
  fi
fi

#Find OS user for target server
OSUSER=`ps -ef | grep -i "bin/dataserver"| grep $TARGET_SERVER_NAME | grep -v -w grep | awk '{print $1}'|uniq|xargs`
OSGROUP=`id -gn $OSUSER`

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$TARGET_DBUSER_OTHERCRED" ]; then
  TARGET_DBUSER_PASSWD=`su - $OSUSER -c "$TARGET_DBUSER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $TARGET_DBUSER_OTHERCRED "
    exit 1
  else
    TARGET_DBUSER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
  fi
fi
set -x
#---------------------------------------------

#change owner for $tmp_loc
chown -R $OSUSER:$OSGROUP $tmp_loc
#tmp_loc_base=`echo $tmp_loc| cut -d/ -f2`
#if [ "$tmp_loc_base" != "" ]; then
#  find /$tmp_loc_base -type d -exec chmod 755 {} +
#fi

#Check if ASE is running or not
isASE_Running=`ps -ef | grep "bin/dataserver" | awk -F"bin/dataserver" '{print $2}' | awk -F"-s" '{print $2}' | awk '{print $1}' | grep -w $TARGET_SERVER_NAME | wc -l`
if [ $isASE_Running -lt 1 ]; then
  echo "WARNING: ASE $TARGET_SERVER_NAME is not running. Proceding to unmount (Stale entries may exists) "
  exit 0
fi

#Find SQLD location for target server
syb_home=`su -m $OSUSER -c 'echo \$SYBASE'`
if [ "$syb_home" = "" ]; then
     SYBASE_ASE_location=`ps -ef | grep "bin/dataserver"|grep $TARGET_SERVER_NAME | grep -v -w grep | awk -F'bin/dataserver' '{print $1}' | grep -v -w awk | awk '{print $NF}' | head -1`
     syb_home=$SYBASE_ASE_location/..
fi
syb_ocs=`cat $syb_home/SYBASE.sh | grep SYBASE_OCS | grep -v export `
syb_ocs=`echo $syb_ocs | awk -F "=" '{print $2}'`
syb_ocs=`echo $syb_ocs |sed 's/"//g'`
TARGET_SYBASE_SQLD=$syb_home/$syb_ocs

#set env
set +x
. $TARGET_SYBASE_SQLD/../SYBASE.sh
set -x

UNMNT_MANIFEST_FILE="$tmp_loc/REMOUNT_MANIFEST_FILE"

######### Find the list of database to be unmount
dbnamelist=
for i in $(echo $TARGET_DBNAME_LIST | sed "s/,/ /g")
do
  db_newname=`echo $i |awk -F":" '{print $2}'`

  if [ "$db_newname" != "" ]; then
    if [ "$dbnamelist" = "" ];then
      dbnamelist="'$db_newname'"
    else
      dbnamelist="$dbnamelist,'$db_newname'"
    fi
  fi
done
######### END: Find the list of database to be unmount 

######## Find list of database currently mounted #####
set +x
MTD_DBLIST=`su -m $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select name from master..sysdatabases where name in ($dbnamelist) and name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','sybsecurity','saptools','saptempdb')
go
EOF`
retval=$?
set -x

retval=`echo $MTD_DBLIST | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to find mounted databases list"
  echo $MTD_DBLIST
  exit 1
fi
######### END: Find list of database currently mounted #####

#List of database to be unmount and currently mounted
UM_DBLIST=""
for dbName in $MTD_DBLIST
do
  if [ "$UM_DBLIST" = "" ]; then
    UM_DBLIST="$dbName"
  else
    UM_DBLIST="$UM_DBLIST,$dbName"
  fi
done

### Unmounting the databases
echo "Unmounting database: $UM_DBLIST"
if [ ! -z "$UM_DBLIST" ]; then
set +x
su -m $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
unmount database $UM_DBLIST to '$UNMNT_MANIFEST_FILE'
go
EOF
retval=$?
set -x
fi

if [ $retval -gt "0" ]; then
 echo "ERRORMSG: Unmount database command Failed!"
 exit 1
fi

exit 0
