#!/bin/sh

set -x
TARGET_DBUSER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
OVERWRITE_DATABASE=$OVERWRITE_DATABASE
ACT_JOBNAME=$JOBID

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

#Validate env for Pre and Post mounts

if [ "$MANIFEST_FILE_LOC" = "" ]; then
  echo "ERRORMSG: Manifest file location (MANIFEST_FILE_LOC) is not provided."
  exit 1
fi

if [ "$TARGET_SERVER_NAME" = "" ]; then
  echo "ERRORMSG: Sybase target server name (TARGET_SERVER_NAME) is not provided."
  exit 1
fi

if [ "$TARGET_DB_USER" = "" ]; then
  echo "ERRORMSG: Sybase target server database username (TARGET_DB_USER) is not provided."
  exit 1
fi

set +x
if [ "$TARGET_DBUSER_PASSWD" = "" ]; then
  echo "ERRORMSG: Sybase target server user password (TARGET_DBUSER_PASSWD) is not provided."
  exit 1
fi
set -x

if [ "$SRC_DBNAME" = "" ]; then
  echo "ERRORMSG: SRC_DBNAME is not provided."
  exit 1
fi

if [ "$UNTIL_TIME" != "" ] && [ "$BEGIN_TIME" = "" ]; then
  echo "ERRORMSG: BEGIN_TIME is not provided."
  exit 1
fi

if [ "$UNTIL_TIME" != "" ] && { [ "$LOG_BKP_MNTPT" = "" ] || [ ! -d "$LOG_BKP_MNTPT" ]; }; then
  echo "ERRORMSG: LOG_BKP_MNTPT is not provided while LOG_RECOVERY is YES."
  exit 1
fi

if [ "$APPREMOUNT" = "TRUE" ] && [ "$CONFIG_FILE_LOC" = "" ]; then
  echo "ERRORMSG: CONFIG_FILE_LOC is not provided"
  exit 1
fi

if [ "$ACT_JOBTYPE" = "mount" ] && [ "$SRC_PAGE_SIZE" = "" ]; then
  echo "ERRORMSG: SRC_PAGE_SIZE is not provided"
  exit 1
fi

###Verify values
IS_ASE_EXISTS=$(ps -ef | grep "bin/dataserver" | awk -F"bin/dataserver" '{print $2}' | awk -F"-s" '{print $2}' | awk '{print $1}' | uniq | xargs | grep -w $TARGET_SERVER_NAME | wc -l)
if [ $IS_ASE_EXISTS -ne 1 ]; then
  echo "ERRORMSG: TARGET_SERVER_NAME $TARGET_SERVER_NAME is not found in this host"
  exit 1
fi

#Find OSUSER or TARGET_SERVER_NAME
OSUSER=`ps -ef | grep "bin/dataserver" | awk -F"-s" '{print $1" "$2}' | grep -w $TARGET_SERVER_NAME | awk '{print $1}' | head -1`
OSGROUP=`id -gn $OSUSER`

tmp_loc=/act/touch/$ACT_JOBNAME/premount
if [ ! -d $tmp_loc ]; then
  mkdir -p $tmp_loc
  chmod 755 $tmp_loc
fi
chmod 755 /act/touch
chown -R $OSUSER:$OSGROUP $tmp_loc/..


#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
TARGET_DBUSER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
if [ ! -z "$TARGET_DBUSER_OTHERCRED" ]; then
  TARGET_DBUSER_PASSWD=`su - $OSUSER -c "$TARGET_DBUSER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $TARGET_DBUSER_OTHERCRED "
    exit 1
  else
    TARGET_DBUSER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
  fi
fi
set -x
#---------------------------------------------

#Verify username and password
SYBASE_ASE_location=`ps -ef | grep "bin/dataserver"|grep $TARGET_SERVER_NAME | grep -v -w grep | awk -F'bin/dataserver' '{print $1}' | grep -v -w awk | awk '{print $NF}' | head -1`
set +x
. $SYBASE_ASE_location/../SYBASE.sh
su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
EOF
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Sybase TARGET_DB_USER or TARGET_DBUSER_PASSWD not correct"
  echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
  exit 1
fi

#------------------------------------------------------------
# function: getTotalDevInAllMF()
#------------------------------------------------------------
getTotalDevInAllMF() {
  local totalDev=

  for mfF in $(echo $MANIFEST_FILE_LOC | sed 's/,/ /g')
  do
    chown $OSUSER $mfF
    set +x
    tmpDevNo=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF | grep -v "Your password will expire in" | sed '1,/device/d' | wc -l
set nocount on
go
use master
go
mount database all from '$mfF' with listonly
go
EOF`
    retval=$?
    set -x

    retval=`echo $tmpDevNo | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
    if [ $retval -ne 0 ]; then
      echo "ERRORMSG: Failed to find no. of devices in manifestfile: $mfF"
      exit 1
    fi

    echo "No. of devices in $mfF = $tmpDevNo"
    totalDev=$((totalDev + tmpDevNo))
  done

  echo "Totla no. of devices in all ManifestFiles = $totalDev"
  MOUNTING_DEVICES=$totalDev

  return 0
}
#------------------------------------------------------------

#------------------------------------------------------------
######function: is_device_enough() ( Find max number of devices and current number of devices before mounting )
#------------------------------------------------------------
is_device_enough() {
  CFG_FILE=`ps -ef | grep "bin/dataserver"| grep -v -w grep | awk -F'-s' '{print $1 $2}'| grep -w $TARGET_SERVER_NAME | awk -F'-c' '{print $2}' | awk '{print $1}' | grep .cfg`
  MAX_DEVICES=`grep "number of devices" $CFG_FILE | cut -d= -f2 | awk '{print $1}'`

  if [ "$MAX_DEVICES" = "DEFAULT" ]; then
     MAX_DEVICES=10
  fi

  re_number='^[0-9]+$'
  if ! [[ $MAX_DEVICES =~ $re_number ]] ; then
     echo "ERRORMSG: Not able to find max number of devices in $TARGET_SERVER_NAME"
     exit 1
  fi

  set +x
  CURRENT_DEVICES=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select count(vdevno) from master..sysdevices where status & 2 = 2
go
EOF`
  set -x

  CURRENT_DEVICES=`echo $CURRENT_DEVICES | awk '{print $1}'`
  if ! [[ $CURRENT_DEVICES =~ $re_number ]] ; then
     echo "ERRORMSG: Not able to find current number of devices in $TARGET_SERVER_NAME"
     exit 1
  fi

  MOUNTING_DEVICES=
  getTotalDevInAllMF
  if ! [[ $MOUNTING_DEVICES =~ $re_number ]] ; then
     echo "ERRORMSG: Not able to find number of devices in MANIFEST_FILE"
     exit 1
  fi

  echo "MAX_DEVICES:$MAX_DEVICES, CURRENT_DEVICES:$CURRENT_DEVICES, MOUNTING_DEVICES:$MOUNTING_DEVICES"
  if [[ $MAX_DEVICES -lt $((CURRENT_DEVICES+MOUNTING_DEVICES)) ]]; then
    echo "ERRORMSG: Cannot add $MOUNTING_DEVICES more devices to target server $TARGET_SERVER_NAME"
    echo "Please increase 'number of device' limit to $((CURRENT_DEVICES+MOUNTING_DEVICES)) atleast"
    echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

  return 0
}
#------------------------------------------------------------

#------------------------------------------------------------
# function: validateDBNAMEs()
#------------------------------------------------------------
validateDBNAMEs() {
  dbName=$1
  if [[ $dbName == *['!'@#\$%^\&*'()'+-]* ]]; then
    echo "ERRORMSG: TARGET_DBNAME_LIST with special charecters not allowed."
    exit 1
  fi
}
#------------------------------------------------------------

#------------------------------------------------------------
# function: deleteDB()
#------------------------------------------------------------
deleteDB() {
  local dbName=$1
  echo "INFOMSG: Deleting database $dbName"

  #Unmount database dbName
  set +x
  su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF
set nocount on
go
use master
go
unmount database $dbName to '$tmp_loc/$dbName.mf'
go
EOF
  retval=$?
  set -x

  if [ $retval -eq 0 ]; then
    echo "INFOMSG: Successfully unmounted database $dbName to $tmp_loc/$dbName.mf "
    return 0
  fi

  echo "INFOMSG: Fail to unmount database $dbName. Dropping the database"
  set +x
  spidList=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 1000 --retserverror " <<EOF | grep -v "Your password will expire in" | grep $dbName | awk '{print $2}'
set nocount on
go
use master
go
sp_who
go
EOF`
  retval=$?
  set -x

  if [ $? -ne 0 ]; then
    # Kill the connections
    echo "FYI: Killing Connections $spidList in database $dbName"
    for conn_id in $spidList
    do
      set +x
      su -m $SYBOSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 1000 --retserverror " <<EOF
      use master
      go
      kill $conn_id
      go
EOF
      retval=$?
      set -x
    done
  fi

  set +x
  echo "INFOMSG: getting devices list for db $dbName"
  local dbDevicesList=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 2000 --retserverror" <<EOF | grep -v "Your password will expire in"
use master
go
select phyname from sysdevices where vdevno in (select vdevno from sysusages where dbid in (select dbid from sysdatabases where name='$dbName'))
EOF`
  retval=$?
  set -x

  set +x
  echo "INFOMSG: Dropping database $dbName"
  su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF
use master
go
dbcc traceon (7436)
go
drop database $dbName
go
dbcc traceoff (7436)
go
EOF
  retval=$?
  set -x

  for dbDevice in $dbDevicesList
  do
    set +x
    echo "INFOMSG: Dropping device $dbDevice"
    su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF
    use master
    go
    sp_dropdevice $dbDevice
    go
EOF
    set -x
  done

  return 0
}
#------------------------------------------------------------

#------------------------------------------------------------
# function: deleteDBifStale()
#------------------------------------------------------------
deleteDBifStale() {
  local dbName=$1
  echo "INFOMSG: Checking database $dbName is stale or not"

  local dbDeviceList=
  set +x
  dbDeviceList=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 20000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select phyname from sysdevices where vdevno in (select vdevno from sysusages where dbid in (select dbid from sysdatabases where name='$dbName'))
go
EOF`
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    deleteDB $dbName
    return 0
  fi

  echo "INFOMSG: Checkin below files exists or not."
  echo $dbDeviceList
  for devPhyFile in $dbDeviceList
  do
    if [ ! -e $devPhyFile ]; then
      deleteDB $dbName
      return 0
    fi
  done

  echo "ERRORMSG: Database $dbName exists and not stale."
  echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
  exit 1
}
#------------------------------------------------------------



# Check if databases already exists or not
if [ "$ACT_JOBTYPE" = "mount" ]; then

  #get new database names list
  newDBList=
  for ndbset in $(echo $TARGET_DBNAME_LIST | sed 's/,/ /g')
  do
    ndb=`echo $ndbset | cut -d: -f2`
    if [ -z "$ndb" ]; then
      ndb=`echo $ndbset | cut -d: -f1`
    fi
    if [ -z "$newDBList" ]; then
      newDBList=$ndb
    else
      newDBList="$newDBList $ndb"
    fi
  done
  validateDBNAMEs $ndb
  echo "New db names = $newDBList"

  #get existing dbs on server
  set +x
  existDBList=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF | grep -v "Your password will expire in" | xargs
set nocount on
go
use master
go
select name from master..sysdatabases where name not in ('master','model','pubs2','pubs3','sybmgmtdb','sybpcidb','sybsystemdb','sybsystemprocs','tempdb','sybsecurity','saptools','saptempdb')
go
EOF`
  retval=$?
  set -x

  retval=`echo $existDBList | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to get current database list on server $TARGET_SERVER_NAME "
    exit 1
  fi
  echo "Existing db names = $existDBList"

  #Crosscheck current dblist with new dblist
  for dbnew in $newDBList
  do
    flag=0
    for dbex in $existDBList
    do
      if [ "$dbex" = "$dbnew" ]; then
        flag=1
        break
      fi
    done
    if [ $flag -ne 0 ]; then
      case $OVERWRITE_DATABASE in
	yes)
	  deleteDB $dbnew
	  ;;
	stale)
	  deleteDBifStale $dbnew
	  ;;
	*)
          echo "ERRORMSG: Database $dbnew already exists on server $TARGET_SERVER_NAME. Cannot mount with dbname $dbnew"
          echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
          exit 1
	  ;;
      esac
    fi
  done

fi

# Check pagesize for target server
if [ "$ACT_JOBTYPE" = "mount" ]; then

  #get target server page size
  set +x
  TARGET_PAGE_SIZE=`su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF | grep -v "Your password will expire in" | xargs
set nocount on
go
use master
go
SELECT @@maxpagesize
go
EOF`
  retval=$?
  set -x

  retval=`echo $TARGET_PAGE_SIZE | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to find the current page_size of target server ($TARGET_SERVER_NAME)"
    exit 1
  fi

  #get source server page size
  SRC_PAGE_SIZE=$SRC_PAGE_SIZE
  if [ -z "$SRC_PAGE_SIZE" ]; then
    SRC_PAGE_SIZE=4096
  fi

  if [ $TARGET_PAGE_SIZE -ne $SRC_PAGE_SIZE ]; then
    echo "ERRORMSG: Cannot proceed to mount as target server page_size($TARGET_PAGE_SIZE) is not equals to source server page_size($SRC_PAGE_SIZE)."
    echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

fi

# Validate required no. of devices
if [ "$ACT_JOBTYPE" = "mount" ]; then
  mfBase=`echo $MANIFEST_FILE_LOC | cut -d/ -f2`
  if [ ! -z "$mfBase" ]; then
    find /$mfBase -type d -exec chmod 755 {} +
  fi
  is_device_enough
fi

# Chech if mount and migrate is in progress
if [ "$ACT_JOBTYPE" = "unmount" ]; then
  #hidden file to handel Mount and Migrate
  ASE_MIGRATE_CONF=/act/touch/.ase_migrate_"$TARGET_DBNAME_LIST".conf

  #Check unmout database required or not
  if [ -f $ASE_MIGRATE_CONF ]; then
    PVMOVE_STATUS=`cat $ASE_MIGRATE_CONF | awk -F "=" '{print $2}'`
    if [ "$PVMOVE_STATUS" = "YES" ]; then
      echo "INFOMSG: PVMOVE is completed. Unmount delete will proceed!"
      exit 0
    else
      echo "ERRORMSG: PVMOVE is in progress. Unmount delete cannot be done!"
      exit 1
    fi
  fi
fi

exit 0
