#!/bin/sh

set -x
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
set -x
SRC_USER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
SRC_SERVER_NAME=$ACT_NAME
DUMPBKPLOC=$TARGET_MNT_PNT
USER_DBLIST="$SRC_DBNAME"
UNTIL_TIME=$UNTIL_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT

ACT_JOBNAME=$JOBID


######################## function - Restore database
restoreDB() {
  echo "Restoring database $1 ..."
  local dbName=$1
  local DF=$2
  local BKP_TYPE=
  if [ "$DF" = "I" ]; then
    BKP_TYPE=cumulative
  else
    BKP_TYPE=
  fi
 
  ## Prepare SQL format disk list
  bkpSetSQL=

  bkpFiles=`ls -ltr $DUMPBKPLOC/$dbName/"$dbName"_dumpbkp"$DF"* | sed 's/\// /g' | awk '{print $NF}' | cut -d_ -f1,2,3,4 |sort| uniq | tail -1`
  if [ -z "$bkpFiles" ]; then
     return 0
  fi
  bkpFiles=`ls -ltr $DUMPBKPLOC/$dbName/$bkpFiles* | awk '{print $NF}' `

  for bkpFl in $bkpFiles
  do
    if [ "$bkpSetSQL" = "" ]; then
	bkpSetSQL=" '$bkpFl' "
    else
	bkpSetSQL="$bkpSetSQL stripe on '$bkpFl' "
    fi
  done
  
  echo "COMMAND: load database $dbName $BKP_TYPE from $bkpSetSQL "
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror " <<EOF
use master
go
load database $dbName $BKP_TYPE from $bkpSetSQL
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    return 1
  else
    return 0
  fi
}
#############################################



######################## function - Online database
onlineDB() {
  echo "Online database $1 ..."
  local dbName=$1

  echo "COMMAND: online database $dbName "
  set +x 
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror " <<EOF
use master
go
online database $dbName
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    return 1
  else
    return 0
  fi
}
#############################################


#---------------------------------------------------------------------------
# Get current database present
#---------------------------------------------------------------------------
isDB_exists() {
  local dbName=$1

  set +x
  dbExists=`su -m $SYBOSUSER -c "isql -S$SRC_SERVER_NAME -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -X -n -b -w 1000 --retserverror " <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select distinct name from master..sysdatabases where name in ("$dbName")
go
EOF`
  retval=$?
  set -x

  retval=`echo $dbExists | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to find DB exists or not."
    exit 1
  fi

  if [ -z "$dbExists" ]; then
    return 1
  else
    return 0
  fi
}
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Get disk exits state
#---------------------------------------------------------------------------
isDisk_exists() {
  local diskName=$1

  set +x
  dbExists=`su -m $SYBOSUSER -c "isql -S$SRC_SERVER_NAME -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -X -n -b -w 1000 --retserverror " <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select distinct name from master..sysdevices where name in ("$diskName")
go
EOF`
  retval=$?
  set -x

  retval=`echo $dbExists | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to find disk exists or not."
    exit 1
  fi

  if [ -z "$dbExists" ]; then
    return 1
  else
    return 0
  fi
}
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# function: createDBfromDump()
#---------------------------------------------------------------------------
createDBfromDump() {
  local dbName=$1
  local DF=F
  local dbCreateSQL=$tmp_loc/dbCreate_$dbName.sql

  bkpFiles=`ls -ltr $DUMPBKPLOC/$dbName/"$dbName"_dumpbkp"$DF"* | sed 's/\// /g' | awk '{print $NF}' | cut -d_ -f1,2,3,4 |sort| uniq | tail -1`
  if [ -z "$bkpFiles" ]; then
    echo "ERRORMSG: Fail to find full backup"
    exit 1
  fi
  bkpFiles=`ls -ltr $DUMPBKPLOC/$dbName/$bkpFiles* | awk '{print $NF}' | head -1`

  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror -o $dbCreateSQL " <<EOF
use master
go
load database $dbName from "$bkpFiles" with listonly=create_sql
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to get create database statement from backup"
    if [ -e $dbCreateSQL ]; then
      sed -i "/Your password will expire in/d" $dbCreateSQL
      cat $dbCreateSQL
    fi
    exit 1
  fi

  #Create required SQL files
  cat $dbCreateSQL | awk '/CREATE  DATABASE /{flag=1} flag; /go/{flag=0}' > $dbCreateSQL.dbOnly
  cat $dbCreateSQL | awk '/DISK INIT/{flag=1} flag; /CREATE  DATABASE /{flag=0}' | grep -v "CREATE  DATABASE " > $dbCreateSQL.disksOnly
  cat $dbCreateSQL.disksOnly | tr '\r\n' ' '| sed 's/ go /\ngo\n/g' > $dbCreateSQL.disksOnly.tmp
  chown $OSUSER:$OSGROUP $dbCreateSQL*

  dbDisks=`grep " name = " $dbCreateSQL.disksOnly.tmp | awk -F"name = " '{print $2}' | awk '{print $1}' | sed "s.'..g"`
  for diskNameStr in $dbDisks
  do
    echo "Check if disk '$diskNameStr' exists or not"
    isDisk_exists $diskNameStr
    if [ $? -ne 0 ]; then
      diskCrStmt=`grep -w $diskNameStr $dbCreateSQL.disksOnly.tmp`
      echo "Disk create statement:"
      echo "$diskCrStmt"
      #Create disks
      set +x
      su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" <<EOF
$diskCrStmt
go
EOF
      retval=$?
      set -x
      if [ $retval -ne 0 ];then
	echo "ERRORMSG: Fail to create disk"
        exit 1
      fi
    else
      echo "Disk '$diskNameStr' already exists"
    fi
  done

  echo "DB $dbName recreating with following statements: "
  cat $dbCreateSQL.dbOnly
  #Create database
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror -i $dbCreateSQL.dbOnly"
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to re-create database $dbName"
    exit 1
  fi
}
#---------------------------------------------------------------------------


# Change permissions to DUMPBKPLOC 

if [ "$DUMPBKPLOC" = "" ]; then
  echo "ERRORMSG: DUMPBKPLOC is not provided"
  exit 1
fi

if [ "$UNTIL_TIME" != "" ] && [ "$LOG_BKP_MNTPT" = "" ]; then
  echo "ERRORMSG: LOG_BKP_MNTPT is not provided"
  exit 1
fi

OSGROUP=`id -gn $OSUSER`
chown -R $OSUSER:$OSGROUP $DUMPBKPLOC
DUMPBKPLOC_BASE=`echo $DUMPBKPLOC | cut -d/ -f2`
if [ "$DUMPBKPLOC_BASE" != "" ]; then
  find /$DUMPBKPLOC_BASE -type d -exec chmod 755 {} +
fi

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#Prepare tmp space
tmp_loc=/act/touch/$ACT_JOBNAME/tempDir
mkdir -p $tmp_loc
chmod 755 $tmp_loc
chmod 755 /act/touch/$ACT_JOBNAME
#find /act -type d -exec chmod 755 {} +
chown -R $OSUSER:$OSGROUP $tmp_loc

#set env
set +x
. $SRC_SYBASE_SQLD/../SYBASE.sh
set -x

# Restore databases one by one
for db in $(echo $USER_DBLIST | sed 's/,/ /g')
do
  #Check db exists or not
  isDB_exists $db
  if [ $? -ne 0 ]; then
    #Create db if not exists
    echo "DB $db not exists"
    createDBfromDump $db
  fi

  echo "********* Restore_Recovery_Online: $db **********"

  # Restore database [FULL]
  restoreDB $db "F"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to restore database $db [full]"
    exit 1
  fi

  # Restore database [INC]
  restoreDB $db "I"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to restore database $db [incremental]"
    exit 1
  fi

  # Recover database if needed
  if [ "x$UNTIL_TIME" != "x" ]; then
    /act/custom_apps/sybase/dump/ACT_SYBASE_RESTORE_recovery.sh $db
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Failed to recover database $db"
      exit 1
    fi
  fi

  # Bring database online
  onlineDB $db
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to bring online database $db"
    exit 1
  fi
done
