#!/bin/sh

set -x

## Checking env variables

if [ "x$SRC_BACKUP_USER" = "x" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "x$SRC_USER_PASSWD" = "x" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ "x$SRC_SERVER_NAME" = "x" ]; then
  echo "ERRORMSG: SRC_SERVER_NAME is not provided"
  exit 1
fi

if [ "x$USER_DBLIST" = "x" ]; then
  echo "ERRORMSG: USER_DBLIST is empty"
  exit 1
fi

if [ "x$SYBOSUSER" = "x" ]; then
  echo "ERRORMSG: OSUSER is empty"
  exit 1
fi

if [ "x$DUMPSCHEDULE" = "x" ]; then
  echo "WARNING: DUMPSCHEDULE is empty. Considering incremental backup"
fi

if [ "x$SRC_SYBASE_SQLD" = "x" ] || [ ! -d $SRC_SYBASE_SQLD ]; then
  echo "ERRORMSG: SRC_SYBASE_SQLD is empty"
  exit 1
fi

## Find sybase Home
sybHome=`grep "name=\"$SRC_SERVER_NAME\"" /act/custom_apps/SYBASE.xml | grep "SRC_SYBASE_SQLD=" | awk -F"SRC_SYBASE_SQLD=" '{print $2}' | awk '{print $1}'`
sybHome=`echo $sybHome| sed s.\"..g`

set +x
. $sybHome/../SYBASE.sh
set -x

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD=`printf '%q' $SRC_USER_PASSWD`
  fi
fi
set -x
#---------------------------------------------


#---------------------------------------------
# Get the available privileges
set +x
allPriviles=`su -m $OSUSER -c "isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X -n -b -w 1000 --retserverror" <<EOF | grep -v "Your password will expire in"
set nocount on
go
use master
go
select v.name from sysprotects p, master.dbo.spt_values v, (Select sr.lrid from sysroles sr where sr.id in (select srid from sysloginroles where suid in (select suid from syslogins where name in (select suser_name())))) s where p.action=v.number and p.protecttype=1 and v.type = 'GP' and s.lrid=p.uid
go
EOF`
retval=$?
set -x

set +x
retval=`echo $allPriviles | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
set -x
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to find dbuser ($SRC_BACKUP_USER) priviletes"
  exit 1
fi

# Check for required privileges
set +x
hasDmAD=`echo -e "$allPriviles" | grep " Dump Any Database " | wc -l`
if [ $hasDmAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Dump Any Database' privileges"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasLdAD=`echo -e "$allPriviles" | grep " Load Any Database " | wc -l`
if [ $hasLdAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Load Any Database' privileges"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMAD=`echo -e "$allPriviles" | grep " Mount Any Database " | wc -l`
if [ $hasMAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Mount Any Database' privileges"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasUAD=`echo -e "$allPriviles" | grep " Unmount Any Database " | wc -l`
if [ $hasUAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Unmount Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasOAD=`echo -e "$allPriviles" | grep " Online Any Database " | wc -l`
if [ $hasOAD -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Online Any Database' privilege"
  echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMSC=`echo -e "$allPriviles" | grep " Manage Server Configuration " | wc -l`
if [ $hasMSC -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Manage Server Configuration' privileges. This is required to 'enable dump history'"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi

hasMDC=`echo -e "$allPriviles" | grep " Manage Dump Configuration " | wc -l`
if [ $hasMDC -lt 1 ]; then
  echo "ERRORMSG: DBuser ($SRC_BACKUP_USER) do not have 'Manage Dump Configuration' privileges. This is required to read sp_dump_history results"
  set +x; echo -e "Available privileges: \n $allPriviles"; set -x
  exit 1
fi
set -x

#---------------------------------------------

#Find dump type
if [ "$DUMPSCHEDULE" = "F" ]; then
  DUMPSCHEDULE_DB="F"
elif [ -z "$DUMPSCHEDULE" ]; then
  DUMPSCHEDULE_DB="I"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
  DUMPSCHEDULE_DB="I"
else
  DUMPSCHEDULE_DB=${DUMPSCHEDULE:`date +%w`:1}
fi

#Caliculate total dump size
DBSIZE_Total=0

## Check USER_DBLIST exists or not
for dbName in $(echo $USER_DBLIST | sed "s/,/ /g")
do
  set +x
  DBSIZE_dts=`su -m $SYBOSUSER -c "isql -S$SRC_SERVER_NAME -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -D$dbName -X -w 2000 --retserverror " <<EOF | grep -v "Your password will expire in" | awk 'NR==3{print $(NF-1)" "$NF}'
sp_spaceused
go
EOF`
  retval=$?
  set -x

  retval=`echo $DBSIZE_dts | grep -w "Msg" | grep -w "Level" | grep -w "State" | wc -l`
  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Database $dbName not present !"
    exit 1
  fi

  sizeType=`echo $DBSIZE_dts| awk '{print $2}'`
  dbSize=`echo $DBSIZE_dts | awk '{print $1}'`
  dbSize=`echo $dbSize | awk -F"." '{print $1}'`
  if [ "$sizeType" = "GB" ]; then
    dbSize=`expr $dbSize "*" 1024 "*" 1024 "*" 1024`
  elif [ "$sizeType" = "MB" ]; then
    dbSize=`expr $dbSize "*" 1024 "*" 1024`
  elif [ "$sizeType" = "KB" ]; then
    dbSize=`expr $dbSize "*" 1024`
  fi
  echo "dbSize of $dbName db: $dbSize"
  DBSIZE_Total=`expr $DBSIZE_Total "+" $dbSize`
done

echo "DBDUMPSIZE=$DBSIZE_Total"
echo "DBDUMPTYPE=$DUMPSCHEDULE_DB"

exit 0
