#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

set -x
PROD_DATAVOL=$PROD_DATAVOL
#DATAVOLPATH=$DATAVOLPATH
MOUNTPOINTROOT=$MOUNTPOINTROOT

#Env provided during restore
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD="'$TARGET_DBUSER_PASSWD'"
set -x
SRC_SERVER_NAME=$ACT_NAME
SRC_SERVER_CFG=$SYBASE_CONFIG_PATH
MASTER_DAT=$MASTER_DAT
SERVER_ERROR_LOG=$SERVER_ERROR_LOG
SRC_DBNAME="$SRC_DBNAME"
BACKUP_SERVER_NAME=$BKP_SERVER_NAME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
UNTIL_TIME=$UNTIL_TIME
ACT_JOBNAME=$JOBID

export SYBOSUSER
export SRC_SYBASE_SQLD
export TARGET_DB_USER=$TARGET_DB_USER
set +x
export TARGET_DBUSER_PASSWD=$TARGET_DBUSER_PASSWD
set -x
export ACT_NAME=$ACT_NAME
export SYBASE_CONFIG_PATH=$SYBASE_CONFIG_PATH
export MASTER_DAT
export SERVER_ERROR_LOG
export SRC_DBNAME
export BKP_SERVER_NAME=$BKP_SERVER_NAME
export LOG_BKP_MNTPT
export UNTIL_TIME

#-------------------------------------------------------------------
# function: isVarNull()
#-------------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    exit 1
  fi
}
#-------------------------------------------------------------------

#Pre-checks
isVarNull PROD_DATAVOL $PROD_DATAVOL
isVarNull MOUNTPOINTROOT $MOUNTPOINTROOT

isVarNull SYBOSUSER $SYBOSUSER
isVarNull SRC_SYBASE_SQLD $SRC_SYBASE_SQLD
isVarNull TARGET_DB_USER $TARGET_DB_USER
isVarNull ACT_NAME $ACT_NAME
isVarNull SYBASE_CONFIG_PATH $SYBASE_CONFIG_PATH
isVarNull MASTER_DAT $MASTER_DAT
isVarNull SERVER_ERROR_LOG $SERVER_ERROR_LOG
isVarNull SRC_DBNAME $SRC_DBNAME
isVarNull BKP_SERVER_NAME $BKP_SERVER_NAME

if [ ! -z "$UNTIL_TIME" ]; then
  isVarNull LOG_BKP_MNTPT
fi

#Prepare tmp location for temp files
tmp_loc=/act/touch/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
   mkdir -p $tmp_loc
fi
find /act -type d -exec chmod 755 {} +

#Prepare DATAVOLPATH
DATAVOLPATH=
for dVol in $(echo $PROD_DATAVOL | sed 's/,/ /g')
do
  ActDVol=$MOUNTPOINTROOT$dVol
  ActDVol_mntpnt=`df -P "$MOUNTPOINTROOT$dVol" | awk 'NR==2{print $NF}'`

  #Check weather it is mountpoint or not
  if [ "$ActDVol_mntpnt" != "$ActDVol" ]; then
    echo "ERRORMSG: Mountpoint not found ($ActDVol)"
    exit 1
  fi

  #Prepare datavolpath variable
  if [ -z "$DATAVOLPATH" ]; then
    DATAVOLPATH="$ActDVol"
  else
    DATAVOLPATH="$DATAVOLPATH,$ActDVol"
  fi
done

#-------------------------------------------------------------------
remove_file()
{
   local fname=$1
   if [ -f $fname ]; then
      rm -f $fname
   fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1

  isMntd=`grep -w $mntpnt /proc/mounts | wc -l`
  if [ $isMntd -eq 1 ]; then

    umount $mntpnt
    if [ $? -ne 0 ]; then
      echo "ERRORMSG: Fail to unmount $mntpnt"
      exit 1
    fi

  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
check_device_size()
{
  echo "Checking device size diff ....."
  local ACT_DEVICE=$1
  local PROD_DEVICE=$2
  
  ### Validate Size
  local ACT_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_DEVICE`
  local PROD_DEVICE_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_DEVICE`

  if [ -z $ACT_DEVICE_SIZE ] || [ -z $PROD_DEVICE_SIZE ]; then
    echo "ERRORMSG: Fail to find the size of device $ACT_DEVICE or $PROD_DEVICE"
    exit 1
  fi

  DIFF_IN_SIZE=$(($ACT_DEVICE_SIZE-$PROD_DEVICE_SIZE))

  if [ "$DIFF_IN_SIZE" -gt "10737418240" ]; then
     echo "ERRORMSG: The device $PROD_DEVICE does not have enough free space, Please check!"
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
get_equl_pv()
{
  local ACT_PV_DEV=$1
  local PROD_PV_DEV_LIST=$2
  local PROD_PV_DEV_size=
  PROD_PV_DEV=

  #ACT_PV_SIZE=`pvdisplay $ACT_PV_DEV | grep -w "Total PE" | awk '{print $NF}'`
  ACT_PV_SIZE=`fdisk -l $ACT_PV_DEV | grep "Disk /dev" | awk '{print $(NF-1)}'`

  for ppv in $PROD_PV_DEV_LIST
  do
    #check if already assigned
    isAssigned=`cat $PV_MEARGE_LIST| awk '{pritn $2}' | grep -w $ppv | wc -l`
    if [ $isAssigned -eq 1 ]; then
      continue
    fi

    ppv_size=`fdisk -l $ppv | grep "Disk /dev" | awk '{print $(NF-1)}'`
    if [ $ACT_PV_SIZE -le $ppv_size ]; then

      #Initial finding
      if [ -z $PROD_PV_DEV ]; then
        PROD_PV_DEV=$ppv
	PROD_PV_DEV_size=$ppv_size
	continue
      fi

      #Better finding
      if [ $PROD_PV_DEV_size -gt $ppv_size ]; then
	PROD_PV_DEV=$ppv
	PROD_PV_DEV_size=$ppv_size
      fi

    fi
  done
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1 
  local PROD_VG_NAME=$2
  echo "Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     exit 1
  fi
 
  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     exit 1
  fi

  #Update LV_MEARGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MEARGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"
  
  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgreduce_fun() 
{
    ACT_PV_NAME=$1
    PROD_DEVICE=$2
    PROD_VG_NAME=`lvdisplay $PROD_DEVICE | grep -w "VG Name" | awk '{print $3}'`
   
   vgreduce $PROD_VG_NAME $ACT_PV_NAME
   if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGREDUCE. Please check!"
     exit 1
  fi
   pvremove $ACT_PV_NAME
   if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform PVREMOVE. Please check!"
     exit 1
   fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#### Main
echo "Starting here ..............."

#Prepare temp files
LV_MEARGE_LIST=$tmp_loc/LV_MEARGE_LIST_$ACT_JOBNAME.txt
VG_MEARGE_LIST=$tmp_loc/VG_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST=$tmp_loc/PV_MEARGE_LIST_$ACT_JOBNAME.txt
MNT_PD_AD_LIST=$tmp_loc/MNT_PD_AD_LIST_$ACT_JOBNAME.txt
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST
remove_file $MNT_PD_AD_LIST

#prepare required files
for dVol in $(echo $PROD_DATAVOL | sed 's/,/ /g')
do
  #Get prod data device
  dVol_dev=`df -P $dVol | awk 'NR==2{print $1}'`
  if [ -z "$dVol_dev" ]; then
    echo "ERRORMSG: Unable to find production device for $dVol"
    exit 1
  fi

  #Get act data device
  ACT_DATAVOL=$MOUNTPOINTROOT$dVol
  ACT_DATA_DEVICE=`df -P $ACT_DATAVOL | awk 'NR==2{print $1}'`
  if [ -z "$ACT_DATA_DEVICE" ]; then
    echo "ERRORMSG: Unable to find Actifio Devices, Please check!"
    exit 1
  fi

  #Check device size 
  check_device_size $ACT_DATA_DEVICE $dVol_dev

  #Get LV paths
  PROD_LV_NAME=`lvdisplay -v $dVol_dev | grep -w "LV Path" | awk '{print $NF}' `
  ACT_LV_NAME=`lvdisplay -v $ACT_DATA_DEVICE | grep -w "LV Path" | awk '{print $NF}'`
  echo "$ACT_LV_NAME $PROD_LV_NAME"  >> $LV_MEARGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $dVol_dev | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_DATA_DEVICE | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MEARGE_LIST

  #Get PV paths
  #ACT_PV_DEV=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}'`
  #PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs`
  #PROD_PV_DEV=
  #get_equl_pv "$ACT_PV_DEV" "$PROD_PV_DEV_LIST"
  #if [ -z $PROD_PV_DEV ]; then
  #  echo "ERRORMSG: Fail to find equl pv for $ACT_PV_DEV"
  #  exit 1
  #fi
  #echo "$ACT_PV_DEV $PROD_PV_DEV" >> $PV_MEARGE_LIST

  #Get Mount point details
  echo "$dVol $dVol_dev $ACT_DATA_DEVICE $ACT_DATAVOL" >> $MNT_PD_AD_LIST
done

#Check device size enough or not
PROD_VGS=`cat $VG_MEARGE_LIST| awk '{print $NF}'|sort|uniq|xargs`
echo "Production VolumeGroups: $PROD_VGS"
for PVG in $PROD_VGS
do
  PVG_SIZE=`vgs --units B --nosuffix --noheadings -o size $PVG`
  PROV_PVS=`vgdisplay -v $PVG | grep -w "PV Name" | awk '{print $NF}' | xargs`
  echo "Production VolumeGroup: $PVG, TotalSize: $PVG_SIZE"

  ACT_VGS=`grep -w $PVG $VG_MEARGE_LIST| awk '{print $1}'|xargs`
  ACT_PVS=
  ACT_VGS_SIZE=0
  for AVG in $ACT_VGS
  do
    AVG_Size=`vgs --units B --nosuffix --noheadings -o size $AVG`
    ACT_VGS_SIZE=$((ACT_VGS_SIZE+AVG_Size))
    AVG_PVS=`vgdisplay -v $AVG | grep -w "PV Name" | awk '{print $NF}' | xargs`
    ACT_PVS="$ACT_PVS $AVG_PVS"
  done
  echo "Actifio VolumeGroups ($ACT_VGS) total size: $ACT_VGS_SIZE"

  if [ $ACT_VGS_SIZE -gt $PVG_SIZE ]; then
    echo "ERRORMSG: Provided Actifio volumegroup size ($ACT_VGS: $ACT_VGS_SIZE) is not suffeciant to mearge into production volumeGroup ($PVG: $PVG_SIZE)"
    exit 1
  fi

  #Prepare pv moves
  for A_PV in $ACT_PVS
  do
    echo "$A_PV $PROV_PVS">> $PV_MEARGE_LIST
  done
done

call_stop_tracking

############ Stop the Application #######
/act/custom_apps/sybase/restore/ACT_SYBASE_prerestore.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to stop the application"
  exit 1
fi

#Unmount devices
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  umnt1=`echo $line| awk '{print $1}'`
  umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  unmount_mnt $umnt2
done < $MNT_PD_AD_LIST
echo "Unmounting mount points completed....."


#LV rename to get diff lv names
cnt=100
echo "" > $LV_MEARGE_LIST.tmp
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MEARGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MEARGE_LIST
mv $LV_MEARGE_LIST.tmp $LV_MEARGE_LIST


#VG mearge
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MEARGE_LIST
echo "VG mearge completed....."


#LV rename and update
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MEARGE_LIST
echo "LV rename completed....."


#Mount devices
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  mntP=`echo $line| awk '{print $1}'`
  mntD=`echo $line | awk '{print $2}'`
  mount $mntD $mntP
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to mount $mntD to $mntP"
    exit 1
  fi
done < $MNT_PD_AD_LIST
echo "Mounting mount points completed....."


#PV move
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  echo "Moving Physical Volumes: $line"

  pvmove --atomic -A y -b -q $line
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to initiate pvmove $line"
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "pvmove initiated successfully .........."


############ Start the Application #######
/act/custom_apps/sybase/restore/ACT_SYBASE_postrestore.sh
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to start the application"
  exit 1
fi

echo "pvmove starting ......................"
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   exit 1
fi

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "Actifio pvs removed successfully .................."

retval=$?
if [ "$retval" -ne 0 ]; then
   echo "ERRORMSG: Failed to remove the devices. Please check for errors!"
   exit 1
fi

call_start_tracking
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST

echo "************ Migration Operation Completed Successfully ***************"

exit 0
