#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#

#Prepare env
for ARGUMENT in "$@"
do
   ARGUMENT="${ARGUMENT%\"}"
   ARGUMENT="${ARGUMENT#\"}"
   argLft=`echo $ARGUMENT|cut -d= -f1`
   argRit=`echo $ARGUMENT|cut -d= -f2`
   export "$argLft=$argRit"
done

usage() {
  echo "Usage: "
  echo "/act/custom_apps/sybase/lvm_migrate/ACT_SYBASE_lvm_migrate_newTarget.sh DATAVOL_DISK_MAPPING=<DATAVOL_DISK_MAPPING> TARGET_SERVER_NAME=<TARGET_SERVER_NAME> TARGET_DB_USER=<TARGET_DB_USER> [ TARGET_DBUSER_PASSWD=<TARGET_DBUSER_PASSWD> | TARGET_DBUSER_OTHERCRED=<TARGET_DBUSER_OTHERCRED> ] TARGET_DBNAME_LIST=<TARGET_DBNAME_LIST> [ JOBID=<AppawareMount_JobId> ]"
  echo ""
  echo "Example: "
  echo "/act/custom_apps/sybase/lvm_migrate/ACT_SYBASE_lvm_migrate_newTarget.sh DATAVOL_DISK_MAPPING=/mntNT/log/CU1:/dev/mapper/vg2-lv2,/mntNT/data/CU1:/dev/mapper/vg2-lv1 TARGET_SERVER_NAME=ASE1 TARGET_DB_USER=sa TARGET_DBUSER_PASSWD=sybase TARGET_DBNAME_LIST=CU1:SU2 JOBID=JOB_034213 TARGET_DBUSER_OTHERCRED="
  echo ""
}

isProvided() {
  local varName=$1
  local varVal=$2
  if [ -z "$varVal" ]; then
    echo "ERRORMSG: $varName is empty."
    usage
    exit 1
  fi
}

DATAVOL_DISK_MAPPING=$DATAVOL_DISK_MAPPING
ACT_JOBNAME=$JOBID

TARGET_SERVER_NAME=$TARGET_SERVER_NAME
TARGET_DB_USER=$TARGET_DB_USER
TARGET_DBUSER_PASSWD=$TARGET_DBUSER_PASSWD
TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST

ASE_MIGRATE_CONF_prepare=`echo $TARGET_SERVER_NAME,$TARGET_DBNAME_LIST | sed "s/:/ /g" | sed "s/,/ /g" | tr " " "\n" | sort | tr "\n" " " | sed "s/ /_/g"`
ASE_MIGRATE_CONF=/act/touch/.ASE_migrate_"$ASE_MIGRATE_CONF_prepare".conf

if [ $# -lt 6 ]; then
  echo "ERRORMSG: Some of the parameter are missing"
  usage
  exit 1
fi

isProvided DATAVOL_DISK_MAPPING $DATAVOL_DISK_MAPPING
isProvided TARGET_SERVER_NAME $TARGET_SERVER_NAME
isProvided TARGET_DB_USER $TARGET_DB_USER
isProvided TARGET_DBNAME_LIST $TARGET_DBNAME_LIST


set -x
#-------------------------------------------------------------------
#Prepare tmp location for temp files
#-------------------------------------------------------------------
tmp_loc=/act/tmpdata/$ACT_JOBNAME/tempDir
if [ ! -d "$tmp_loc" ]; then
  mkdir -p $tmp_loc
fi
#find /act -type d -exec chmod 755 {} +

#Prepare env to connect to sybaseASE
SYBASE_ASE_dir=`ps -ef | grep -w "\-s$TARGET_SERVER_NAME" | grep -vw grep | awk -F"bin/dataserver" '{print $1}'| awk '{print $NF}'`
OSUSER=`ps -ef | grep -w "\-s$TARGET_SERVER_NAME" | grep -vw grep | awk '{print $1}'`
OSGROUP=`id -gn $OSUSER`

set +x
. $SYBASE_ASE_dir/../SYBASE.sh
set -x
chown -R $OSUSER:$OSGROUP $tmp_loc
UNMOUNTDB_FILE=$tmp_loc/UNMOUNTDB_FILE.mf
#-------------------------------------------------------------------

#Get password
set +x
if [ ! -z "$TARGET_DBUSER_OTHERCRED" ]; then
  TARGET_DBUSER_PASSWD=`su - $OSUSER -c "$TARGET_DBUSER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $TARGET_DBUSER_OTHERCRED "
    exit 1
  else
    TARGET_DBUSER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
  fi
fi
set -x

#-------------------------------------------------------------------
# function: UnmountTargetDBs()
#-------------------------------------------------------------------
UnmountTargetDBs() {
  local DBLIST=

  #Call unmount for each database
  for dbNameSet in $(echo $TARGET_DBNAME_LIST|sed 's/,/ /g')
  do
    dbName=`echo $dbNameSet|cut -d: -f2`
    if [ -z "$DBLIST" ]; then
      DBLIST=$dbName
    else
      DBLIST="$DBLIST,$dbName"
    fi
  done

  #unmountDBs
  set +x
  su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b --retserverror" <<EOF
use master
go
unmount database $DBLIST to '$UNMOUNTDB_FILE'
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to unmount databases $DBLIST"
    return 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# functon: mountTargetDBs()
#-------------------------------------------------------------------
mountTargetDBs() {
  MOUNTCMD_FILE=$tmp_loc/MOUNTCMD_FILE.mf
  MT_MEARGE_LIST=$tmp_loc/MT_MEARGE_LIST_$ACT_JOBNAME.txt

  set +x
  su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 20000 -o $MOUNTCMD_FILE --retserverror" <<EOF
use master
go
mount database all from '$UNMOUNTDB_FILE' with listonly
go
EOF
  retval=$?
  set -x

  #get devicelist
  sed '1,/device/d' $MOUNTCMD_FILE | sed 's/=//g' | sed "s.'..g" > $MOUNTCMD_FILE.deviceOnly

  #update devicelist with required mount points
  while read line
  do
    SRC_LOC=`echo $line | awk '{print $1}'`
    TRG_LOC=`echo $line | awk '{print $NF}'`
    SRC_LOC_sed=`echo $SRC_LOC | sed 's./.\\\/.g' `
    TRG_LOC_sed=`echo $TRG_LOC | sed 's./.\\\/.g' `
    sed "s/^$SRC_LOC_sed/$TRG_LOC_sed/g" $MOUNTCMD_FILE.deviceOnly > $MOUNTCMD_FILE.deviceOnly.tmp
    mv $MOUNTCMD_FILE.deviceOnly.tmp $MOUNTCMD_FILE.deviceOnly
  done <$MT_MEARGE_LIST

  while read line
  do
    phyDev=`echo $line | awk '{print $1}'`
    lgcDev=`echo $line | awk '{print $2}'`
    if [ -z "$USING_DEV_CMD" ]; then
      USING_DEV_CMD="$lgcDev as \"$phyDev\""
    else
      USING_DEV_CMD="$USING_DEV_CMD,$lgcDev as \"$phyDev\" "
    fi
  done < $MOUNTCMD_FILE.deviceOnly

  #Prepare mount command and mount
  local MOUNTCMD="mount database all from '$UNMOUNTDB_FILE' using $USING_DEV_CMD"
  echo "MOUNTCMD: $MOUNTCMD"
  set +x
  su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 20000 --retserverror" <<EOF
use master
go
$MOUNTCMD
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: fail to mount databases"
    return 1
  fi

  #Online databases
  for dbNameSet in $(echo $TARGET_DBNAME_LIST|sed 's/,/ /g')
  do
    dbName=`echo $dbNameSet|cut -d: -f2`
    set +x
    su -m $OSUSER -c "isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -w 20000 --retserverror" <<EOF
use master
go
online database $dbName
go
EOF
    retval=$?
    set -x

    if [ $retval -ne 0 ]; then
      echo "ERRORMSG: Fail to bring database online ($dbName)"
      retval 1
    fi
  done
}
#-------------------------------------------------------------------

#-------------------------------------------------------------------
remove_file()
{
  local fname=$1
  if [ -f $fname ]; then
    rm -f $fname
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
vgmerge_fun()
{
  local ACT_VG_NAME=$1
  local PROD_VG_NAME=$2
  echo "INFOMSG: Merging VolumeGroups: $ACT_VG_NAME to $PROD_VG_NAME"

  vgchange -an $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGCHANGE. Please check!"
     exit 1
  fi

  vgmerge $PROD_VG_NAME $ACT_VG_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform VGMERGE. Please check!"
     exit 1
  fi

  #Update LV_MEARGE_LIST
  sed -i "s/$ACT_VG_NAME/$PROD_VG_NAME/g" $LV_MEARGE_LIST
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
lvmerge_fun()
{
  local ACT_LV_NAME=$1
  local PROD_LV_NAME=$2
  echo "INFOMSG: Merging Logical Volumes: $ACT_LV_NAME to $PROD_LV_NAME"

  lvchange -ay $ACT_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvchange (activate) $ACT_LV_NAME"
    exit 1
  fi

  lvchange -an $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to lvchange (deactivate) $PROD_LV_NAME"
    exit 1
  fi

  lvremove -f $PROD_LV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to lvremove $PROD_LV_NAME"
    exit 1
  fi

  lvrename $ACT_LV_NAME $PROD_LV_NAME
  if [ "$?" -gt 0 ]; then
     echo "ERRORMSG: Failed to perform LVRENAME. Please check!"
     exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
unmount_mnt()
{
  local mntpnt=$1
  isMntd=`grep -w $mntpnt /proc/mounts | uniq | wc -l`
  if [ $isMntd -eq 1 ]; then
    for i in {1..5}
    do
      umount $mntpnt
      grep " $mntpnt " /proc/mounts
      if [ $? -eq 0 ]; then
        if [ $i -eq 5 ]; then
          echo "ERRORMSG: Fail to unmount $mntpnt after retries"
          exit 1
        fi
        echo "ERRORMSG: Fail to unmount $mntpnt. Retrying"
        sleep 5
      else
        echo "INFOMSG: Successfully unmounted $mntpnt"
        break
      fi
    done
  fi

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_stop_tracking()
{
  echo "cbt_deactivate.sh 2>/dev/null"
  /opt/act/cbt/bin/cbt_deactivate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
call_start_tracking()
{
   echo "cbt_activate.sh 2>/dev/null"
   /opt/act/cbt/bin/cbt_activate.sh 2>/dev/null
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
#Prepare temp files
#-------------------------------------------------------------------
LV_MEARGE_LIST=$tmp_loc/LV_MEARGE_LIST_$ACT_JOBNAME.txt
VG_MEARGE_LIST=$tmp_loc/VG_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST=$tmp_loc/PV_MEARGE_LIST_$ACT_JOBNAME.txt
PV_MEARGE_LIST_finel=$tmp_loc/PV_MEARGE_LIST_finel_$ACT_JOBNAME.txt
MT_MEARGE_LIST=$tmp_loc/MT_MEARGE_LIST_$ACT_JOBNAME.txt
NST_MNTPNTS=$tmp_loc/NST_MNTPNTS_$ACT_JOBNAME.txt
NST_MNTPNTS_toMount=$tmp_loc/NST_MNTPNTS_toMount_$ACT_JOBNAME.txt
remove_file $LV_MEARGE_LIST
remove_file $VG_MEARGE_LIST
remove_file $PV_MEARGE_LIST
remove_file $PV_MEARGE_LIST_finel
remove_file $MT_MEARGE_LIST
remove_file $NST_MNTPNTS
remove_file $NST_MNTPNTS_toMount.tmp
#-------------------------------------------------------------------


#Prepare files conntents and also check lvm size
#---------------------------------------------------------------------------------------------------------
for datavolMap in $(echo $DATAVOL_DISK_MAPPING|sed 's/,/ /g')
do
  PROD_VOL=`echo $datavolMap|cut -d: -f2`
  PROD_LV_PATH=`lvdisplay $PROD_VOL|grep "LV Path"| awk '{print $NF}'`

  ACT_MNT=`echo $datavolMap| cut -d: -f1`
  ACT_VOL=`df -P $ACT_MNT|awk 'NR==2{print $1}'`
  ACT_LV_PATH=`lvdisplay $ACT_VOL|grep "LV Path"| awk '{print $NF}'`

  PROD_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $PROD_LV_PATH`
  ACT_LV_SIZE=`lvs --units B --nosuffix --noheadings -o size $ACT_LV_PATH`

  if [ $PROD_LV_SIZE -lt $ACT_LV_SIZE ]; then
    echo "ERRORMSG: Provided device $PROD_VOL do not have enough space to mearge with $ACT_MNT"
    exit 1
  fi

  #Get LV paths
  echo "$ACT_LV_PATH $PROD_LV_PATH"  >> $LV_MEARGE_LIST

  #Get VG names
  PROD_VG_NAME=`lvdisplay -v $PROD_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  ACT_VG_NAME=`lvdisplay -v $ACT_LV_PATH | grep -w "VG Name" | awk '{print $3}'`
  echo "$ACT_VG_NAME $PROD_VG_NAME " >> $VG_MEARGE_LIST

  #Get PV paths
  ACT_PV_DEV_LIST=`vgdisplay -v $ACT_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  PROD_PV_DEV_LIST=`vgdisplay -v $PROD_VG_NAME | grep -w "PV Name" | awk '{print $NF}' | xargs | sed 's/ /,/g'`
  for ACT_PV_DEV in $(echo $ACT_PV_DEV_LIST|sed 's/,/ /g'); do
    echo "$ACT_PV_DEV $PROD_PV_DEV_LIST" >> $PV_MEARGE_LIST
  done

  #Get MNT paths
  PROD_MNT=`lsblk -n --output MOUNTPOINT $PROD_VOL`
  if [ -z "$PROD_MNT" ]; then
    PROD_MNT=$ACT_MNT
  fi
  echo "$ACT_MNT $PROD_VOL $PROD_MNT" >> $MT_MEARGE_LIST
done
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Preparing mearge files completed"

#Find additional mounts
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt1/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt1/"`
  echo "$nstMnt" >> $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  nstMnt=`df  | awk '{print $(NF)}' | grep "^$umnt2/"`
  #nstMnt=`cat /proc/mounts | awk '{print $2}' | grep "^$umnt2/"`
  echo "$nstMnt" >> $NST_MNTPNTS
done < $MT_MEARGE_LIST

#Sort, uniq
cat $NST_MNTPNTS | sort -r | uniq > $NST_MNTPNTS.tmp
mv $NST_MNTPNTS.tmp $NST_MNTPNTS
sed -i '/^$/d' $NST_MNTPNTS

#Remove migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  grep -vw "$umnt1\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS

  umnt2=`echo $line | awk '{print $NF}'`
  grep -vw "$umnt2\$" $NST_MNTPNTS > $NST_MNTPNTS.tmp
  mv $NST_MNTPNTS.tmp $NST_MNTPNTS
done < $MT_MEARGE_LIST

echo "INFOMSG: Additional mountpoints to unmount"
cat $NST_MNTPNTS

#Update NST_MNTPNTS_toMount with mount command
cat $NST_MNTPNTS | sort > $NST_MNTPNTS_toMount
while read line
do
  if [ -z $line ]; then
    continue
  fi
  mntDev=`grep " $line " /proc/mounts | awk '{print $1}' | uniq`
  echo "mount $mntDev $line" >> $NST_MNTPNTS_toMount.tmp
done < $NST_MNTPNTS
mv $NST_MNTPNTS_toMount.tmp $NST_MNTPNTS_toMount
cat $NST_MNTPNTS_toMount


#Update hidden migrate config file to handel unmount+delete
if [ ! -f $ASE_MIGRATE_CONF ]; then
  echo -e "MIGRATED=NO" > $ASE_MIGRATE_CONF
fi

echo "INFOMSG: Actifio CBT stoping..."
call_stop_tracking
echo "INFOMSG: Actifio CBT has been stopped successfully"

#Stop application for unmounting actifio and source mount points
#-------------------------------------------------------------------
UnmountTargetDBs
if [ $? -ne 0 ]; then
  echo "ERRORMSG: failed at stoping application access to source/actifio mount points"
  exit 1
fi
#-------------------------------------------------------------------


#Unmount all mounts
#---------------------------------------------------------------------------------------------------------
#Unmount additional mounts first
while read line
do
  if [ -z "$line" ]; then
    continue
  fi
  unmntpnt=`echo $line | awk '{print $1}'`
  if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
    unmount_mnt $unmntpnt
  else
    echo "ERRORMSG: Found nested mount points ($unmntpnt). cannot proceed..."
    exit 1
  fi
done < $NST_MNTPNTS

#Unmount migrate mount points
while read line
do
  umnt1=`echo $line| awk '{print $1}'`
  umnt2=`echo $line | awk '{print $NF}'`
  unmount_mnt $umnt1
  unmount_mnt $umnt2
done < $MT_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Unmounted all required mount points"


#LV rename to get diff lv names
#---------------------------------------------------------------------------------------------------------
cnt=100
remove_file $LV_MEARGE_LIST.tmp
while read line
do
  actLV=`echo $line | awk '{print $1}'`
  prodLV=`echo $line | awk '{print $2}'`

  echo "INFOMSG: Renaming $actLV to $actLV$cnt"
  lvchange -ay $actLV
  lvrename $actLV $actLV$cnt

  echo "$actLV$cnt $prodLV" >> $LV_MEARGE_LIST.tmp
  cnt=$(($cnt + 1))
done < $LV_MEARGE_LIST
mv $LV_MEARGE_LIST.tmp $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Renaming actifio lvm names done (temparory)"


#VG mearge
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_VG_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`echo $line | awk '{print $2}'`
  vgmerge_fun $ACT_VG_NAME $PROD_VG_NAME
done < $VG_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: VG mearge completed....."


#LV rename and update
#---------------------------------------------------------------------------------------------------------
while read line
do
  ACT_LV_NAME=`echo $line | awk '{print $1}'`
  PROD_LV_NAME=`echo $line | awk '{print $2}'`
  lvmerge_fun $ACT_LV_NAME $PROD_LV_NAME
done < $LV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: LV rename completed....."


#Mount devices
#---------------------------------------------------------------------------------------------------------
while read line
do
  mntP=`echo $line| awk '{print $3}'`
  mntD=`echo $line | awk '{print $2}'`
  mount -o rw,nouuid $mntD $mntP
  if [ $? -ne 0 ]; then
    mount -o rw $mntD $mntP
    if [ $? -ne 0 ]; then
       isAlreadyMounted=`df | grep "$mntD" | grep -w $mntP$ | wc -l`
       if [ $isAlreadyMounted -eq 0 ]; then
          echo "ERRORMSG: Fail to mount $mntD to $mntP"
          exit 1
       fi
    fi
  fi
done < $MT_MEARGE_LIST

#Mount additional mounts
if [ "$NSTMNT_CHECK" = "TRUE" ] ;then
  echo "INFOMSG: Mounting the Additional Mountpoints...."
  while read line
  do
    addMntPnt=`echo $line | awk '{print $3}'`
    if [ ! -d "$addMntPnt" ]; then
      mkdir -p $addMntPnt
    fi
    $line
  done < $NST_MNTPNTS_toMount
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Mounting mount points completed....."


#Call AppAware mount script
#---------------------------------------------------------------------------------------------------------
mountTargetDBs
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to mount databases with required mount points"
  exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: Application started successfully"


#Prepare pvmove finel list
#---------------------------------------------------------------------------------------------------------
while read line
do
  #Skip empty lines
  if [ -z "$line" ]; then
    continue
  fi

  #Read Actifio PVs
  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  ACT_PV_NAME=`echo $ACT_PV_NAME | sed 's/,/ /g'`

  #Read Production PVs
  PROD_PV_NAME=`echo $line | awk '{print $2}'`
  PROD_PV_NAME=`echo $PROD_PV_NAME | sed 's/,/ /g'`

  #Prepare lines
  for ACT_PVNAME in $ACT_PV_NAME
  do
    echo "$ACT_PVNAME $PROD_PV_NAME" >> $PV_MEARGE_LIST_finel
  done

done < $PV_MEARGE_LIST
#---------------------------------------------------------------------------------------------------------


#PV move
#---------------------------------------------------------------------------------------------------------
# Calculate pending pvmoves
NoOfPvmovesRequired=`grep "^/dev" $PV_MEARGE_LIST_finel | wc -l`
echo "INFOMSG: NoOfPvmovesRequired=$NoOfPvmovesRequired"

# Perfrom pvmove
while [ $NoOfPvmovesRequired -gt 0 ]
do
  # Get current line (Set of pv devices to be participate in pvmove)
  getline=`grep "^/dev" $PV_MEARGE_LIST_finel | head -1`

  # Check if any of the pvs are already in use
  PV_BUSY=FALSE
  for pvNameCurr in $getline
  do
    echo $pvNameCurr
    isPVinUse=`lvs -a -o+devices | grep pvmove | awk '{print $5}' | grep "/dev/sd" | cut -d "(" -f1 | sort | uniq | grep -w $pvNameCurr | wc -l`
    if [ $isPVinUse -gt 0 ]; then
      PV_BUSY=TRUE
      break
    fi
  done
  if [ "$PV_BUSY" = "TRUE" ]; then
    echo "INFOMSG: $pvNameCurr is in use ($getline)"

    # Remove this line
    tmpgetline=`echo $getline | sed 's./.\\\/.g'`
    sed -i "/$tmpgetline/d" $PV_MEARGE_LIST_finel

    # Write this line at the end for next time
    echo $getline >> $PV_MEARGE_LIST_finel

    # Display pending
    echo "INFOMSG: Pending pvmoves"
    cat $PV_MEARGE_LIST_finel

    sleep 30
    continue
  fi

  # Perform pvmove
  echo "INFOMSG: Moving Physical Volumes: $getline"
  echo "Command: pvmove --atomic -A y -b -q $getline"
  pvmove --atomic -A y -b -q $getline
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: pvmove command failes (for $getline)"
    exit 1
  fi
  echo "INFOMSG: pvmove initiated successfully (for $getline) .........."

  # remove current line on successful pvmove initiation
  tmpgetline=`echo $getline | sed 's./.\\\/.g'`
  sed -i "/$tmpgetline/d" $PV_MEARGE_LIST_finel

  # Re-calculate pending pvmoves
  NoOfPvmovesRequired=`grep "^/dev" $PV_MEARGE_LIST_finel | wc -l`
  echo "INFOMSG: NoOfPvmovesRequired=$NoOfPvmovesRequired"
done
#---------------------------------------------------------------------------------------------------------


echo "INFOMSG: pvmove status ......................"
#---------------------------------------------------------------------------------------------------------
pvmove
retval=$?
if [ "$retval" -gt 0 ]; then
   echo "ERRORMSG: PVMOVE Failed, Please check the error message!"
   exit 1
fi
#---------------------------------------------------------------------------------------------------------
echo "INFOMSG: pvmove completed successfully ....."

#Remove Actifio PVs
while read line
do
  if [ -z "$line" ]; then
    continue
  fi

  ACT_PV_NAME=`echo $line | awk '{print $1}'`
  PROD_VG_NAME=`pvs | grep -w $ACT_PV_NAME | awk '{print $2}'`

  vgreduce $PROD_VG_NAME $ACT_PV_NAME
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to vgreduce $PROD_VG_NAME $ACT_PV_NAME"
    exit 1
  fi

  pvremove $ACT_PV_NAME
  if [ $? -gt 0 ]; then
    echo "ERRORMSG: FAILED TO PERFORM PVREMOVE. ($ACT_PV_NAME) Please check!"
    exit 1
  fi
done < $PV_MEARGE_LIST
echo "Actifio pvs removed successfully .................."

#Update hidden migrate config file to handel unmount+delete
echo -e "MIGRATED=YES" > $ASE_MIGRATE_CONF

echo "INFOMSG: Starting Actifio CBT..."
call_start_tracking
echo "INFOMSG: Actifion CBT started successfully"

if [ -d "$tmp_loc" ]; then
  rm -rf $tmp_loc
fi

echo "************ Migration Operation Completed Successfully ***************"

exit 0

