#!/bin/sh

set -x
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
set -x
SRC_USER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
SRC_SERVER_NAME=$ACT_NAME
SRC_DBNAME="$SRC_DBNAME"
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
BEGIN_TIME=$BEGIN_TIME
UNTIL_TIME=$UNTIL_TIME

## Validate env
if [ "$OSUSER" = "" ]; then
  echo "ERRORMSG: Sybase OSUSER is not provided."
  exit 1
fi

if [ "$SRC_SYBASE_SQLD" = "" ]; then
  echo "ERRORMSG: Sybase isql location (SRC_SYBASE_SQLD) is not provided"
  exit 1
fi

if [ "$SRC_BACKUP_USER" = "" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "$SRC_USER_PASSWD" = "''" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ "$SRC_SERVER_NAME" = "" ];then
  echo "ERRORMSG: Sybase server name (SRC_SERVER_NAME)  is not provided"
  exit 1
fi

if [ "$LOG_BKP_MNTPT" = "" ] || [ ! -d "$LOG_BKP_MNTPT" ]; then
  echo "ERRORMSG: LOG_BKP_MNTPT is not provided or its not a directory "
  exit 1
fi

if [ "$SRC_DBNAME" = "" ]; then
  echo "ERRORMSG: Sybase database list (SRC_DBNAME) is empty "
  exit 1
fi
###############

#Change permissions to LOG_BKP_MNTPT
chown -R $OSUSER:$OSGROUP $LOG_BKP_MNTPT
LOG_BKP_MNTPT_BASE=`echo $LOG_BKP_MNTPT | cut -d/ -f2`
if [ "$LOG_BKP_MNTPT" != "" ]; then
  find /$LOG_BKP_MNTPT_BASE -type d -exec chmod 755 {} +
fi

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#Set sybase env
set +x
. $SRC_SYBASE_SQLD/../SYBASE.sh
set -x
rollforwardtime=$UNTIL_TIME
UNTIL_TIME=`echo $UNTIL_TIME| sed 's/:\| \|-//g' `


for db in $(echo $SRC_DBNAME | sed "s/,/ /g")
do
  echo "Recovery start for database $db_act ....."

  #Find logBackupSets for database 
  if [ ! -e $LOG_BKP_MNTPT/$db ] || [ ! -d $LOG_BKP_MNTPT/$db ]; then
    continue
  fi
  #logBkpSets=`ls -ltr $LOG_BKP_MNTPT/$db/ | grep -vw total |awk '{print $NF}' | cut -d_ -f1,2,3,4 | uniq`
  logBkpSets=`find $LOG_BKP_MNTPT/$db/ -newermt "$BEGIN_TIME" | sort | sed 's/\// /g' | awk '{print $NF}' | cut -d_ -f1,2,3,4 | uniq | grep -vw $db`

  for logBkpSet in $logBkpSets
  do

    #logBkpSetEndTime=`echo $logBkpSet | cut -d_ -f3,4 | sed "s/_//g"`
    logBkpSetEndTime=`stat $LOG_BKP_MNTPT/$db/$logBkpSet* | grep "Modify: "| awk '{print $2$3}' | sed 's/-\|://g' |cut -d. -f1 | sort | uniq | tail -1`
    logBkpSet="$LOG_BKP_MNTPT/$db/$logBkpSet"
    break_Flag="false"
    loadTranStmt=""

    #For each log backup set run 'load transaction'
    for fileName in $(ls $logBkpSet*)
    do
      if [ "$loadTranStmt" = "" ]; then
        loadTranStmt="load transaction $db from \"$fileName\" "
      else
        loadTranStmt="$loadTranStmt stripe on \"$fileName\" "
      fi
    done
    
    # If until time is specified
    if [ $UNTIL_TIME -le $logBkpSetEndTime ]; then
      #rollforwardtime=`date -d "${UNTIL_TIME:0:8} ${UNTIL_TIME:8:2}:${UNTIL_TIME:10:2}:${UNTIL_TIME:12:2}" +"%Y-%m-%d %H:%M:%S"`
      loadTranStmt="$loadTranStmt with until_time=\"$rollforwardtime\" "
      break_Flag="true"
    fi
    
    #Run the recovery
    set +x
    su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X --retserverror" <<EOF
use master
go
$loadTranStmt
go
EOF
    retval=$?
    set -x

    #Break if break_Flag is true
    if [ "$break_Flag" = "true" ]; then
      break
    fi
  done
  
  if [ "$retval" -gt 0 ]; then
    echo "ERRORMSG: Failed to recover database $db"
    exitCode=1
  fi

done

exit $exitCode

