#!/bin/sh

set -x
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
set -x
SRC_USER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
SRC_SERVER_NAME=$ACT_NAME
SRC_SERVER_CFG=$SYBASE_CONFIG_PATH
MASTER_DAT=$MASTER_DAT
SERVER_ERROR_LOG=$SERVER_ERROR_LOG
SRC_DBNAME="$SRC_DBNAME"
BACKUP_SERVER_NAME=$BKP_SERVER_NAME
#RRECOVERY=$RRECOVERY
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
UNTIL_TIME=$UNTIL_TIME

## Validate env
if [ "$OSUSER" = "" ]; then
  echo "ERRORMSG: Sybase OSUSER is not provided."
  exit 1
fi

if [ "$SRC_SYBASE_SQLD" = "" ]; then
  echo "ERRORMSG: Sybase isql location (SRC_SYBASE_SQLD) is not provided"
  exit 1
fi

if [ "$SRC_BACKUP_USER" = "" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "$SRC_USER_PASSWD" = "''" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ "$SRC_SERVER_NAME" = "" ];then
  echo "ERRORMSG: Sybase server name (SRC_SERVER_NAME)  is not provided"
  exit 1
fi

if [ "$SRC_DBNAME" = "" ]; then
  echo "ERRORMSG: Sybase database list (SRC_DBNAME) is empty "
  exit 1
fi

if [ "$SRC_SERVER_CFG" = "" ] || [ ! -f "$SRC_SERVER_CFG" ]; then
  echo "ERRORMSG: Sybase config file (SRC_SERVER_CFG) not provided or it is not a file."
  exit 1
fi

if [ "$MASTER_DAT" = "" ] || [ ! -f "$MASTER_DAT" ]; then
  echo "ERRORMSG: $SRC_SERVER_NAME server master.dat file not provided or it is not valide file"
  exit 1
fi
 
if [ "$SERVER_ERROR_LOG" = "" ]; then
  echo "ERRORMSG: Sybase server error log file name not provided "
  exit 1
fi

if [ "$BACKUP_SERVER_NAME" = "" ]; then
  echo "ERRORMSG: BACKUP_SERVER_NAME is not provided"
  exit 1
fi

#if [ "$RRECOVERY" = "true" ] && { [ "$LOG_BKP_MNTPT" = "" ] || [ ! -d "$LOG_BKP_MNTPT" ]; }; then
#  echo "ERRORMSG: Recovery needed but LOG_BKP_MNTPT is not provided or not a valide directory"
#  exit 1
#fi
###############

OSGROUP=`id -Gn $OSUSER`

#set sybase env variables
set +x
source $SRC_SYBASE_SQLD/../SYBASE.sh
set -x

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#---------------------------------------------------------------------------------------------------------
# function: createBackupServerRunfile
#---------------------------------------------------------------------------------------------------------
createBackupServerRunfile() {
   RUN_FILE_BKP=$SYBASE/$SYBASE_ASE/install/ACT_RUN_$BACKUP_SERVER_NAME

   #Backup RUN_FILE_BKP (ACT) file if exists
   if [ -f $RUN_FILE_BKP ]; then
      echo "Backing up $RUN_FILE_BKP to $RUN_FILE_BKP.actbkp"
      mv $RUN_FILE_BKP $RUN_FILE_BKP.actbkp
   fi

   #Create RUN_FILE_BKP
   echo "$SYBASE/$SYBASE_ASE/bin/backupserver \\" > $RUN_FILE_BKP
   echo "-e$SYBASE/$SYBASE_ASE/install/$BACKUP_SERVER_NAME.log \\" >> $RUN_FILE_BKP
   echo "-I$SYBASE/interfaces \\" >> $RUN_FILE_BKP
   echo "-M$SYBASE/$SYBASE_ASE/bin/sybmultbuf \\" >> $RUN_FILE_BKP
   echo "-S$BACKUP_SERVER_NAME \\" >> $RUN_FILE_BKP

   #Update file privileges
   chown $OSUSER:$OSGROUP $RUN_FILE_BKP
   chmod 755 $RUN_FILE_BKP
}
#---------------------------------------------------------------------------------------------------------

#Start backup server
RUN_FILE_BKP="$SYBASE/$SYBASE_ASE/install/RUN_$BACKUP_SERVER_NAME"
if [ ! -e $RUN_FILE_BKP ]; then
   createBackupServerRunfile
fi
set +x
su -m $OSUSER -c "nohup startserver -f $RUN_FILE_BKP >& $SYBASE/$SYBASE_ASE/install/logfilebackupserver.out &"
retval=$?
set -x
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to start backup server $BACKUP_SERVER_NAME"
  exit 1
fi

sleep 60
sybaseBkpSrvStatus=$(ps -ef | grep "bin/backupserver" |awk -F"bin/backupserver" '{print $2}' |awk -F "-S" '{print $2}'| awk '{print $1}' | grep -w $BACKUP_SERVER_NAME)
if [ -z $sybaseBkpSrvStatus ]; then
  echo "ERRORMSG: Failed to bring up backup server $SRC_SERVER_NAME"
  exit 1
fi

#Backup ACT_RUN_servername file if exists
RUN_FILE=$SYBASE/$SYBASE_ASE/install/ACT_RUN_$SRC_SERVER_NAME
if [ -f $RUN_FILE ]; then
  echo "Backing up $RUN_FILE to $RUN_FILE.actbkp"
  mv $RUN_FILE $RUN_FILE.actbkp
fi

#Copy config file from backup
CFG_FILE=$SYBASE/$SYBASE_ASE/$SRC_SERVER_NAME.cfg
DATE_TIME=`date +%Y%m%d`_`date +%H%M%S`
if [ -e $CFG_FILE ]; then
  cp $CFG_FILE ${CFG_FILE}_ACT_${DATE_TIME}
fi
cp $SRC_SERVER_CFG $CFG_FILE
chown $OSUSER:$OSGROUP $CFG_FILE
chmod 640 $CFG_FILE

#Create RUN_FILE based on our requirements
echo "$SYBASE/$SYBASE_ASE/bin/dataserver \\" > $RUN_FILE
echo "-s$SRC_SERVER_NAME \\" >> $RUN_FILE
echo "-c$CFG_FILE \\" >> $RUN_FILE
echo "-d$MASTER_DAT \\" >> $RUN_FILE
echo "-e$SERVER_ERROR_LOG \\" >> $RUN_FILE
echo "-M$SYBASE/$SYBASE_ASE \\" >> $RUN_FILE
echo "-N$SYBASE/$SYBASE_ASE/sysam/$SRC_SERVER_NAME.properties \\" >> $RUN_FILE
echo "-i$SYBASE \\" >> $RUN_FILE
echo "-q \\" >> $RUN_FILE
chown $OSUSER:$OSGROUP $RUN_FILE
chmod 755 $RUN_FILE

echo "Starting server with following options: "
cat $RUN_FILE

# Start dataserver
set +x
su -m $OSUSER -c "nohup startserver -f $RUN_FILE >& $SYBASE/$SYBASE_ASE/install/logfiledataserver.out &"
retval=$?
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to start server $SRC_SERVER_NAME"
  exit 1
fi
set -x

sleep 60
sybaseStatus=$(ps -ef | grep "bin/dataserver" |awk -F"bin/dataserver" '{print $2}' | awk -F "-s" '{print $2}'| awk '{print $1}' | grep -w $SRC_SERVER_NAME)
if [ -z "$sybaseStatus" ]; then
  echo "ERRORMSG: Failed to bring up server $SRC_SERVER_NAME"
  exit 1
fi
echo "Server started successfully"

if [ "$UNTIL_TIME" != "" ]; then
  echo "Recover starting..."
  /act/custom_apps/sybase/restore/ACT_SYBASE_RESTORE_recover.sh
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Failed to recover the databases. Leaving the databases in mount state and exit grasefully."
    exit 0
  fi
fi

#Bring user databases online
echo "Bringing databases online...."
for dbname in $(echo $SRC_DBNAME | sed "s/,/ /g")
do
set +x
su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X  --retserverror" <<EOF
use master
go
online database $dbname
go
EOF
retval=$?
set -x
if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Failed to bring database $dbname online."
fi
done
