#!/bin/sh

set -x
OSUSER=$SYBOSUSER
SRC_SYBASE_SQLD=$SRC_SYBASE_SQLD
SRC_BACKUP_USER=$TARGET_DB_USER
set +x
SRC_USER_PASSWD=`printf '%q' $TARGET_DBUSER_PASSWD`
set -x
SRC_USER_OTHERCRED=$TARGET_DBUSER_OTHERCRED
SRC_SERVER_NAME=$ACT_NAME
BACKUP_SERVER_NAME=$BKP_SERVER_NAME

## Validate env
if [ "$OSUSER" = "" ]; then
  echo "ERRORMSG: Sybase OSUSER is not provided."
  exit 1
fi

if [ "$SRC_SYBASE_SQLD" = "" ]; then
  echo "ERRORMSG: Sybase isql location (SRC_SYBASE_SQLD) is not provided"
  exit 1
fi

if [ "$SRC_BACKUP_USER" = "" ]; then
  echo "ERRORMSG: Sybase database username (SRC_BACKUP_USER) is not provided "
  exit 1
fi

set +x
if [ "$SRC_USER_PASSWD" = "''" ]; then
  echo "WARNING: Sybase database password (SRC_USER_PASSWD) is empty"
fi
set -x

if [ "$SRC_SERVER_NAME" = "" ];then
  echo "ERRORMSG: Sybase server name (SRC_SERVER_NAME)  is not provided"
  exit 1
fi

if [ "$BACKUP_SERVER_NAME" = "" ]; then
  echo "ERRORMSG: Sybase backup server name (BACKUP_SERVER_NAME) is not provided"
  exit 1
fi
###############

#set sybase env variables
set +x
source $SRC_SYBASE_SQLD/../SYBASE.sh
set -x

#---------------------------------------------
#Get the password
#---------------------------------------------
set +x
if [ ! -z "$SRC_USER_OTHERCRED" ]; then
  SRC_USER_PASSWD=`su - $OSUSER -c "$SRC_USER_OTHERCRED"`
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to get password from the password function: $SRC_USER_OTHERCRED "
    exit 1
  else
    SRC_USER_PASSWD="'$SRC_USER_PASSWD'"
  fi
fi
set -x
#---------------------------------------------

#Check if SRC_SERVER running or not
RUNNING_SERVERS=`ps -ef | grep "bin/dataserver" |awk -F "-s" '{print $2}'| awk '{print $1}' | uniq |xargs`
echo "Currently running servers: $RUNNING_SERVERS"
SRC_SERVER_STATUS=`ps -ef | grep "bin/dataserver" |awk -F "-s" '{print $2}'| awk '{print $1}' | uniq | grep -w $SRC_SERVER_NAME |wc -l`
echo "SRC_SERVER_NAME status: $SRC_SERVER_STATUS"

#Check backup server running or not
RUNNING_BKP_SERVERS=`ps -ef | grep "bin/backupserver" | awk -F "-S" '{print $2}' | awk '{print $1}' | uniq | xargs`
echo "Currently running backup servers: $RUNNING_BKP_SERVERS"
BKP_SRV_STATUS=`ps -ef | grep "bin/backupserver" | awk -F "-S" '{print $2}' | awk '{print $1}' | uniq | grep -w $BACKUP_SERVER_NAME | wc -l`
echo "BACKUP_SERVER_NAME status: $BKP_SRV_STATUS"

#Shutdown backup server
if [ $BKP_SRV_STATUS -ne 1 ]; then
  echo "Target backup server ($BACKUP_SERVER_NAME) is not running, Continuing to shutdown ASE Server"
else
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X  --retserverror" <<EOF
use master
go
shutdown SYB_BACKUP
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to shutdown backup server $BACKUP_SERVER_NAME"
    exit 1
  fi
fi

#Shutdown target server
if [ $SRC_SERVER_STATUS -ne 1 ]; then
  echo "Target server ($SRC_SERVER_NAME) is not running. Continuing to restore..."
else
  set +x
  su -m $OSUSER -c "$SRC_SYBASE_SQLD/bin/isql -U$SRC_BACKUP_USER -P$SRC_USER_PASSWD -S$SRC_SERVER_NAME -X  --retserverror" <<EOF
use master
go
shutdown
go
EOF
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Failed to shutdown server $SRC_SERVER_NAME"
    exit 1
  fi
fi

echo "Target server shutdowned. Continuing to restore..."
exit 0
