#!/bin/sh


#-------------------------------------------------------------------
# function: isVarNull()
#-------------------------------------------------------------------
isVarNull() {
  local varName=$1
  local varValue=$2

  if [ "x$varValue" = "x" ]; then
    echo "ERRORMSG: $varName is not provided"
    exit 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: removeFile()
#-------------------------------------------------------------------
removeFile() {
  local fileName=$1
  if [ -e "$fileName" ]; then
    rm -f $fileName
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: prepareTmpLoc()
#-------------------------------------------------------------------
prepareTmpLoc() {
  tmp_loc=/act/touch/$ACT_JOBNAME

  if [ ! -d "$tmp_loc" ]; then
    mkdir -p $tmp_loc
    chmod 755 $tmp_loc
  fi
  chmod 755 /act/touch
  chmod -R 755 /act/tmpdata
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: prepareConnString()
#-------------------------------------------------------------------
prepareConnString() {
  local dbUser=$1
  set +x
  local dbPwd=$2
  set -x
  local dbName=$3
  local dbEng=$4
  local dbPort=$5
  CONN_STR=

  if [ "$TLS_ENABLED" = "TRUE" ]; then
    TLS_ENC="Encryption=TLS(trusted_certificates=NONE)"
  else
    TLS_ENC=
  fi

  if [ -z "$dbEng" ]; then
    dbEng=`dblocate -d |  awk 'NR > 3 {print}' | sed '$d' |grep -w $dbName | xargs|awk '{print $1}'`
    if [ -z "$dbEng" ]; then
      echo "ERRORMSG: DB is not running"
      exit 1
    fi
  fi

  local HOSTNAME=`dblocate -d | awk 'NR > 3 {print}' | sed '$d' | grep -w $(hostname) | grep -w $dbName | xargs | awk '{print $2}' | cut -d: -f1`
  if [ -z "$HOSTNAME" ]; then
    HOSTNAME=`dblocate -d | awk 'NR > 3 {print}' | sed '$d' | grep -w $(hostname -i) | grep -w $dbName | xargs | awk '{print $2}' | cut -d: -f1`
    if [ -z "$HOSTNAME" ]; then
      #echo "ERRORMSG: Fail to find hostname for database $dbName"
      #exit 1
      HOSTNAME=localhost	# Must be running on localhost. Otherwise job will fail anyway
    fi
  fi

  if [ "$dbName" = "utility_db" ] || [ -z "$dbPort" ]; then
    local dbPort=`dblocate -d -n | awk 'NR > 3 {print}' | sed '$d' | grep -w $dbName | xargs | awk '{print $2}' | cut -d: -f2`
    if [ -z "$dbPort" ]; then
      echo "ERRORMSG: Fail to find port-number for database $dbName"
      exit 1
    fi
  fi

  set +x
  CONN_STR="uid=$dbUser;pwd=$dbPwd;dbn=$dbName;eng=$dbEng;$TLS_ENC"
  export SQLCONNECT="uid=$dbUser;pwd=$dbPwd;dbn=$dbName;eng=$dbEng;$TLS_ENC"
  set -x

  HOST_PORT_OPT=" -host $HOSTNAME -port $dbPort "

  return 0
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: isDB_running()
#-------------------------------------------------------------------
isDB_running() {
  local dbName=$1
  local isRunning=`ps -ef | grep iqsrv | grep -w $dbName.db | grep -vw grep | wc -l`
  if [ $isRunning -gt 0 ]; then
    return 0
  else 
    return 1
  fi
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: Create the config file for remount
#-------------------------------------------------------------------
remount_config() {
  config_file=/act/touch/$ACT_JOBNAME/REMOUNT_CONFIG
  echo "OLD_MOUNT=$TARGET_MNT" > $config_file
  echo "REMOUNT_DB_LIST=$SRC_DB_NAME:$TGT_DB_NAME" >> $config_file
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: rmEMTdirs()
#-------------------------------------------------------------------
rmEMTdirs() {
  local dirNAME=$1
  while [ "$dirNAME" != "/" ]; do
    rmdir $dirNAME
    if [ $? -ne 0 ]; then
      break
    fi
    dirNAME=`dirname $dirNAME`
  done
}
#-------------------------------------------------------------------


#-------------------------------------------------------------------
# function: switchLog()
#-------------------------------------------------------------------
switchLog() {
   local targetDir=$1
   local dateTime=`date +%Y%m%d_%H%M%S`
   local tmpSqlFile=/act/touch/${JOBID}_logSwitch.sql

   if [ -z $targetDir ]; then
      echo "WARNINGMSG: targetDir not provided for switchLog()"
      return 1
   fi

   if [ ! -d "$targetDir" ]; then
      mkdir -p $targetDir
   fi
   chown $OSUSER:$OSGROUP $targetDir

   echo "BACKUP DATABASE POINT IN TIME RECOVERY LOGS ONLY TO '$targetDir/logBkp_$dateTime'; " > $tmpSqlFile
   echo "exit;" >> $tmpSqlFile
   chown $OSUSER:$OSGROUP $tmpSqlFile

   su -m $OSUSER -c "timeout 60s dbisql -nogui -onerror exit $HOST_PORT_OPT $tmpSqlFile"
   if [ $? -ne 0 ] ; then
      removeFile $tmpSqlFile
      return 1
   fi

   removeFile $tmpSqlFile
   return 0
}
#-------------------------------------------------------------------
