#!/bin/sh

set -x
OSUSER=$OSUSER
SRC_DB_NAME=$SRC_DB_NAME
TGT_DB_NAME=$TGT_DB_NAME
CATALOG_DB_FILE=$CATALOG_DB_FILE
ACT_JOBNAME=$JOBID
TARGET_MNT=$TARGET_MNT
utilitydbuser=$SRC_DB_DBA_USER
utilitydbpwd=$SRC_DB_DBA_PWD
RECOVERY_TIME=$RECOVERY_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
INCR_BKP_LOC=$INCR_BKP_LOC
SYBIQ_HOME=$SYBIQ_HOME

#Get envdetails
OSGROUP=`id -gn $OSUSER`
IQSRV_CMD=`ps -A -o command | grep /iqsrv | awk '{print $1}' | sort|uniq |grep -v grep`
if [ -z "$SYBIQ_HOME" ]; then
  SYBIQ_HOME=`dirname $IQSRV_CMD`
  SYBIQ_HOME=$SYBIQ_HOME/../..
fi
version=`echo $IQSRV_CMD| rev| cut -d/ -f3| rev`
DATA_LOC=`dirname $CATALOG_DB_FILE`
FULL_VIR_BKP_LOC=$DATA_LOC/FULL_VIR_DEC

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

###CONFIG FILE LOCATION
SRC_DBDIR=`dirname $CATALOG_DB_FILE`
config_file=$SRC_DBDIR/config.txt

#####creating sql file for restore
prepareTmpLoc
chown -R $OSUSER:$OSGROUP $tmp_loc
chown -R $OSUSER:$OSGROUP $TARGET_MNT
RESTORE_FULL_SQL=$tmp_loc/restore_full.sql
RESTORE_INC_SQL=$tmp_loc/restore_inc.sql

##Full Decoupled Backup restore
echo "restore database '$SRC_DBDIR/$TGT_DB_NAME.db' from '$FULL_VIR_BKP_LOC' " > $RESTORE_FULL_SQL
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_FULL_SQL

#Prepare connection string
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "utility_db"
set -x

chown -R $OSUSER:$OSGROUP $RESTORE_FULL_SQL

###MAIN 
set +x
echo "Executing restore full backup using file $FULL_VIR_BKP_LOC ..."
cat $RESTORE_FULL_SQL
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_FULL_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to clone database $TGT_DB_NAME"
  sleep 5000
  exit 1
fi

######Incremental Backup Restore
INCR_BKP_LOC=`dirname $INCR_BKP_LOC`
chown -R $OSUSER:$OSGROUP $INCR_BKP_LOC*
INCR_BKP_LOC=$INCR_BKP_LOC/INC_BKP
echo "restore database '$SRC_DBDIR/$TGT_DB_NAME.db' from '$INCR_BKP_LOC' " > $RESTORE_INC_SQL

#if [ ! -z "$RECOVERY_TIME" ]; then
#  IQ_SYSTEM_DATA_file_dir=$TARGET_MNT$SRC_DBDIR
#  echo "RECOVER UNTIL TIMESTAMP '$RECOVERY_TIME' USING LOG PATH  '$LOG_BKP_MNTPT/$SRC_DB_NAME/PITR','$IQ_SYSTEM_DATA_file_dir','$IQ_SYSTEM_LOG_file_dir' " >> $RESTORE_INC_SQL
#fi

echo "OVERWRITE EXISTING clear log;" >> $RESTORE_INC_SQL

chown -R $OSUSER:$OSGROUP $RESTORE_INC_SQL

set +x
echo "Executing restore incremental backup using file $INCR_BKP_LOC ..."
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_INC_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
    echo "ERRORMSG: Failed to clone database $TGT_DB_NAME"
    exit 1
fi

#######starting IQ server
su -m $OSUSER -c "start_iq $SRC_DBDIR/$TGT_DB_NAME.db"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to start the database"
  exit 1
fi

#Prepare connection string for new database
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "$TGT_DB_NAME" "$TGT_DB_NAME"
set -x

#Trunoff PITR to target
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to turnoff PITR to target"
  exit 1
fi

#Restart PITR
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to on PITR to new database"
  exit 1
fi

exit 0
