#!/bin/sh

set -x
OSUSER=$OSUSER
SRC_DB_NAME=$SRC_DB_NAME
TGT_DB_NAME=$TGT_DB_NAME
CATALOG_DB_FILE=$CATALOG_DB_FILE
ACT_JOBNAME=$JOBID
TARGET_MNT=$TARGET_MNT
utilitydbuser=$SRC_DB_DBA_USER
set +x
utilitydbpwd=$SRC_DB_DBA_PWD
set -x
RECOVERY_TIME=$RECOVERY_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
INCR_BKP_LOC=$INCR_BKP_LOC
SYBIQ_HOME=$SYBIQ_HOME
TLS_ENABLED=$TLS_ENABLED
DB_ENG=$DB_ENG
PORTNO=$PORTNO

#Get envdetails
OSGROUP=`id -gn $OSUSER`
IQSRV_CMD=`ps -A -o command | grep /iqsrv | awk '{print $1}' | sort|uniq |grep -v grep`
if [ -z "$SYBIQ_HOME" ]; then
  SYBIQ_HOME=`dirname $IQSRV_CMD`
  SYBIQ_HOME=$SYBIQ_HOME/../..
fi
version=`echo $IQSRV_CMD| rev| cut -d/ -f3| rev`
DATA_LOC=`dirname $CATALOG_DB_FILE`
FULL_VIR_BKP_LOC=$TARGET_MNT$DATA_LOC/FULL_VIR_DEC

#No PITR, as it is not working properly
#RECOVERY_TIME=

#Child restore
if [ "$ACT_MOUNT_REPLACERESTORE" = "TRUE" ]; then
  /act/custom_apps/sybaseiq/clone/ACT_SYBASEIQ_childRestore.sh
  exit $?
fi

if [ "$APPREMOUNT" = "TRUE" ]; then
  /act/custom_apps/sybaseiq/clone/ACT_SYBASEIQ_remount.sh
  exit $?
fi

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

###CONFIG FILE LOCATION
SRC_DBDIR=`dirname $CATALOG_DB_FILE`
config_file="$TARGET_MNT"$SRC_DBDIR/config.txt

#####creating sql file for restore
prepareTmpLoc
chown -R $OSUSER:$OSGROUP $tmp_loc
chown -R $OSUSER:$OSGROUP $TARGET_MNT
RESTORE_FULL_SQL=$tmp_loc/restore_full.sql
RESTORE_INC_SQL=$tmp_loc/restore_inc.sql

#Create symbolic links if required
IQ_SYSTEM_LOG_file=`grep -w IQ_SYSTEM_LOG $config_file| awk '{print $NF}'`
IQ_SYSTEM_LOG_file_dir=`dirname $IQ_SYSTEM_LOG_file| sed 's."..g'`

if [ -z "$IQ_SYSTEM_LOG_file_dir" ]; then
  echo "ERRORMSG: Fail to get source transaction log directory "
  exit 1
fi

#Check if it is exists or not
if [ -d "$IQ_SYSTEM_LOG_file_dir" ]; then
  chmod 777 $IQ_SYSTEM_LOG_file_dir
  if [ -e $IQ_SYSTEM_LOG_file ]; then
    #if already transaction log exists, fail the job
    echo "ERRORMSG: $IQ_SYSTEM_LOG_file exists. Cannot proceed to appaware mount as it will clear this file. "
    exit 1
  else
    touch $IQ_SYSTEM_LOG_file
    chmod 644 $IQ_SYSTEM_LOG_file
  fi
else
  #Create softlink and required directories
  reqSoftlink=$IQ_SYSTEM_LOG_file_dir
  reqSoftlink_base=`dirname $reqSoftlink`
  while [ ! -e $reqSoftlink_base ]
  do
    reqSoftlink=$reqSoftlink_base
    reqSoftlink_base=`dirname $reqSoftlink_base`
  done
  echo "Creating softlink from $TARGET_MNT$reqSoftlink to $reqSoftlink"
  DEL_SOFTLINK_AT_END=$reqSoftlink
  ln -s $TARGET_MNT$reqSoftlink $reqSoftlink
fi

#Check softlink properly created or not
if [ ! -e "$IQ_SYSTEM_LOG_file" ]; then
  echo "ERRORMSG: Fail to create softlink to IQ_SYSTEM_LOG. $IQ_SYSTEM_LOG_file not exists "
  rm -f $DEL_SOFTLINK_AT_END
  exit 1
fi

#Create source PITR logs location if not exists
IQ_PITR_LOC___ACT_dir=`grep -w IQ_PITR_LOC___ACT $config_file| awk '{print $NF}'`
if [ ! -z "$IQ_PITR_LOC___ACT_dir" ] && [ ! -d "$IQ_PITR_LOC___ACT_dir" ]; then
  mkdir -p $IQ_PITR_LOC___ACT_dir
  chown $OSUSER:$OSGROUP $IQ_PITR_LOC___ACT_dir
  DEL_DIR_AT_END=$IQ_PITR_LOC___ACT_dir
fi

#Remove old PITR logs. They are of no use. Recovery is not supported
rm -f $TARGET_MNT$IQ_PITR_LOC___ACT_dir/*.iqlog

##Full Decoupled Backup restore
echo "restore database '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db' from '$FULL_VIR_BKP_LOC' " > $RESTORE_FULL_SQL
while read line
do
  fLabel=`echo $line| awk '{print $1}'`
  fPath=`echo $line| awk '{print $2}'`
  dbSpaceName=`echo $line| awk '{print $3}'`

  #Skip renaming transaction log or PITR location
  if [ "$fLabel" = "IQ_SYSTEM_LOG" ] || [ "$fLabel" = "IQ_PITR_LOC___ACT" ]; then
    continue
  fi

  #Check and correct file obsolete path
  if [ "${fPath:0:1}" != "/" ]; then
    fPath=$SRC_DBDIR/$fPath
  fi

  if [ ! -e "$TARGET_MNT$fPath" ]; then
    echo "ERRORMSG: One of the datafile is missing ($TARGET_MNT$fPath) "
    if [ -e "$DEL_DIR_AT_END" ]; then
      rm -f $DEL_DIR_AT_END/*
      rmEMTdirs $DEL_DIR_AT_END
    fi
    removeFile $IQ_SYSTEM_LOG_file
    exit 1
  fi

  echo "rename $fLabel TO '$TARGET_MNT$fPath'" >> $RESTORE_FULL_SQL
done<"$config_file"
#echo "rename IQ_SYSTEM_TEMP TO '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.tmp'" >> $RESTORE_FULL_SQL
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_FULL_SQL

#Prepare connection string
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "utility_db"
set -x

# Get utility_db logfile
UTILITY_LOG=`ps -eo cmd:2000 | grep iqsrv | grep -w utility_db | awk -F' -o ' '{print $2}' | awk '{print $1}'`
if [ ! -f "$UTILITY_LOG" ]; then
   echo "WARNINGMSG: Fail to find UTILITY_LOG"
fi

chown -R $OSUSER:$OSGROUP $RESTORE_FULL_SQL

###MAIN 
set +x
echo "Executing restore full backup using file $FULL_VIR_BKP_LOC ..."
cat $RESTORE_FULL_SQL
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_FULL_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to clone database $TGT_DB_NAME "
  if [ -e "$DEL_DIR_AT_END" ]; then
    rm -f $DEL_DIR_AT_END/*
    rmEMTdirs $DEL_DIR_AT_END
  fi
  removeFile $IQ_SYSTEM_LOG_file

  # Get Utility_db error message
  if [ -f "$UTILITY_LOG" ]; then
     UTILITY_THIS_RESTORE_NO=`grep -n "Starting database \"a_restore\" ($TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db) at " $UTILITY_LOG | tail -1 | awk '{print $1}' | cut -d: -f1`
     UTILITY_ERR=`sed "1,${UTILITY_THIS_RESTORE_NO}d" $UTILITY_LOG | grep "Error:" | awk -F"Error: " '{print $2}'`
     if [ ! -z "$UTILITY_ERR" ]; then
        echo "ERRORMSG: $UTILITY_ERR "
     fi
  fi
  exit 1
fi

######Incremental Backup Restore
INCR_BKP_LOC=`dirname $INCR_BKP_LOC`
chown -R $OSUSER:$OSGROUP $INCR_BKP_LOC*
INCR_BKP_LOC=$INCR_BKP_LOC/INC_BKP
echo "restore database '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db' from '$INCR_BKP_LOC' " > $RESTORE_INC_SQL

while read line
do
  fLabel=`echo $line| awk '{print $1}'`
  fPath=`echo $line| awk '{print $2}'`
  dbSpaceName=`echo $line| awk '{print $3}'`

  #Skip renaming transaction log or PITR location
  if [ "$fLabel" = "IQ_SYSTEM_LOG" ] || [ "$fLabel" = "IQ_PITR_LOC___ACT" ]; then
    continue
  fi

  #Check and correct file obsolete path
  if [ "${fPath:0:1}" != "/" ]; then
    fPath=$SRC_DBDIR/$fPath
  fi
  echo "rename $fLabel TO '$TARGET_MNT$fPath'" >> $RESTORE_INC_SQL
done<"$config_file"
#echo "rename IQ_SYSTEM_TEMP TO '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.tmp'" >> $RESTORE_INC_SQL
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_INC_SQL

chown -R $OSUSER:$OSGROUP $RESTORE_INC_SQL

set +x
echo "Executing restore incremental backup using file $INCR_BKP_LOC ..."
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_INC_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to clone database $TGT_DB_NAME "
  if [ -e "$DEL_DIR_AT_END" ]; then
    rm -f $DEL_DIR_AT_END/*
    rmEMTdirs $DEL_DIR_AT_END
  fi
  removeFile $IQ_SYSTEM_LOG_file

  # Get Utility_db error message
  if [ -f "$UTILITY_LOG" ]; then
     UTILITY_THIS_RESTORE_NO=`grep -n "Starting database \"a_restore\" ($TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db) at " $UTILITY_LOG | tail -1 | awk '{print $1}' | cut -d: -f1`
     UTILITY_ERR=`sed "1,${UTILITY_THIS_RESTORE_NO}d" $UTILITY_LOG | grep "Error:" | awk -F"Error: " '{print $2}'`
     if [ ! -z "$UTILITY_ERR" ]; then
        echo "ERRORMSG: $UTILITY_ERR "
     fi
  fi
  exit 1
fi

#Rename transaction log for new database
su -m $OSUSER -c "dblog -t $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.log $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db"
if [ $? -ne 0 ];then
  echo "ERRORMSG: Fail to rename IQ_SYSTEM_LOG path "
  if [ -e "$DEL_DIR_AT_END" ]; then
    rm -f $DEL_DIR_AT_END/*
    rmEMTdirs $DEL_DIR_AT_END
  fi
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#######starting IQ server
su -m $OSUSER -c "start_iq $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to start the database "
  if [ -e "$DEL_DIR_AT_END" ]; then
    rm -f $DEL_DIR_AT_END/*
    rmEMTdirs $DEL_DIR_AT_END
  fi
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

## Remove the created softlink
removeFile $DEL_SOFTLINK_AT_END
removeFile $IQ_SYSTEM_LOG_file

#Prepare connection string for new database
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "$TGT_DB_NAME" "$TGT_DB_NAME"
set -x

#Trunoff PITR to target
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to turnoff PITR to target "
  if [ -e "$DEL_DIR_AT_END" ]; then
    rm -f $DEL_DIR_AT_END/*
    rmEMTdirs $DEL_DIR_AT_END
  fi
  exit 1
fi

#Rename PITR logs location
set +x
echo "Running SQL: ALTER DBSPACE IQ_SYSTEM_LOG RENAME '$TARGET_MNT$SRC_DBDIR'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"ALTER DBSPACE IQ_SYSTEM_LOG RENAME '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME';\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to rename IQ_SYSTEM_LOG location "
  exit 1
fi

#Restart PITR
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to on PITR to new database "
  exit 1
fi

#Remove source PITR log directory if exists
if [ -e "$DEL_DIR_AT_END" ]; then
  rm -f $DEL_DIR_AT_END/*
  rmEMTdirs $DEL_DIR_AT_END
fi

#Prepare config file for remount
mkdir -p /act/touch/$ACT_JOBNAME
chown -R $OSUSER:$OSGROUP /act/touch/$ACT_JOBNAME
remount_config

if [ -e $IQ_SYSTEM_LOG_file_dir ]; then
  chmod 755 $IQ_SYSTEM_LOG_file_dir
fi

exit 0
