#!/bin/sh

set -x
OSUSER=$OSUSER
SYBIQ_HOME=$SYBIQ_HOME
version=$VERSION
SRC_DB_NAME=$SRC_DB_NAME
TGT_DB_NAME=$TGT_DB_NAME
CATALOG_DB_FILE=$CATALOG_DB_FILE
ACT_JOBNAME=$JOBID
TARGET_MNT=$TARGET_MNT
utilitydbuser=$SRC_DB_DBA_USER
set +x
utilitydbpwd=$SRC_DB_DBA_PWD
set -x
FULL_VIR_BKP_LOC=$FULL_VIR_BKP_LOC
INCR_BKP_LOC=$INCR_BKP_LOC
SYBIQ_HOME=$SYBIQ_HOME
TLS_ENABLED=$TLS_ENABLED
DB_ENG=$DB_ENG
PORTNO=$PORTNO

# If it's a premount_test failure, don't run unmount.
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME
if [ -e $ACT_PREMOUNT_FAILURE_FILE ]; then
  echo "INFOMSG: It's a premount-test failure. No unmount required."
  exit 0
fi

OSGROUP=`id -gn $OSUSER`
IQSRV_CMD=`ps -A -o command | grep /iqsrv | awk '{print $1}' | sort|uniq |grep -v grep`
if [ -z "$SYBIQ_HOME" ]; then
  SYBIQ_HOME=`dirname $IQSRV_CMD`
  SYBIQ_HOME=$SYBIQ_HOME/../..
fi
version=`echo $IQSRV_CMD| rev| cut -d/ -f3| rev`

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

PWDS=/act/custom_apps/sybaseiq

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh


#Check db running or not
isDB_running $TGT_DB_NAME
if [ $? -ne 0 ]; then
  echo "Database is not running"
  exit 0
fi

#Prepare connection string
set +x
prepareConnString "$utilitydbuser" "$utilitydbpwd" "$TGT_DB_NAME" "$TGT_DB_NAME"
set -x

#-----------------------------------------------------------------------------------------------------
#Take full virtual backup for remount, if db is running
CATALOG_DB_FILE=`ps -A -o command | grep iqsrv | grep -w $TGT_DB_NAME.db | awk '{print $2}'`
if [ ! -z "$CATALOG_DB_FILE" ]; then

  #Kill db connections
  set +x
  CONN_LIST=`su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT sp_iqcontext" | grep -vw "IQconnID\|sp_iqcontext" |awk '{print $2}'| grep -vw "rows\|^$" | xargs`
  retval=$?
  set -x
  echo "Killing connections: $CONN_LIST"
  for connHeader in $CONN_LIST
  do
    if [ "$connHeader" = "1000000004" ]; then continue; fi
    set +x
    su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT drop connection $connHeader"
    retval=$?
    set -x
    if [ $retval -ne 0 ]; then
      echo "ERRORMSG: Fail to kill the connection $connHeader"
      exit 1
    fi
  done

  DATA_LOC=`dirname $CATALOG_DB_FILE`
  FULL_BKP_FILE=$DATA_LOC/FULL_VIR_DEC
  IQFILES_DETAILS=$DATA_LOC/ACTIFIO_config.txt
  #REMOUNT_CONFIG=$DATA_LOC/ACTIFIO_REMOUNT_CONFIG
  removeFile $IQFILES_DETAILS
  #removeFile $REMOUNT_CONFIG
  #cp /act/tmpdata/$ACT_JOBNAME/REMOUNT_CONFIG $REMOUNT_CONFIG

  set +x
  echo "ForRemount: Taking full virtual decoupled to $FULL_BKP_FILE ..."
  su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT " <<EOF
BACKUP DATABASE FULL VIRTUAL DECOUPLED TO '$FULL_BKP_FILE'
EOF
  retval=$?
  set -x
  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Fail to take full virtual backup. Remount will not work."
  fi

  #finding iqfiles
  set +x
  su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"select dbf.dbfile_name,dbf.file_name,dbs.dbspace_name from SYS.SYSDBSPACE dbs, SYS.SYSDBFILE dbf where dbf.dbspace_id=dbs.dbspace_id and dbs.dbspace_name not in ('system','temporary');\"" | awk '{print $1,$2,$3}' |tail -n+3 |head -n-3 > $IQFILES_DETAILS
  retval=$?
  set -x
  if [ $retval -ne 0 ]; then
    echo "WARNINGMSG: Fail to get iqfiles. Remount will not work."
  fi
  cat $IQFILES_DETAILS | grep -v IQ_SYSTEM_LOG > $IQFILES_DETAILS.tmp
  mv $IQFILES_DETAILS.tmp $IQFILES_DETAILS

  #finding iqlogfile
  IQ_SYSTEM_LOG=`su -m $OSUSER -c "dblog $CATALOG_DB_FILE" | grep "is using log file"|awk '{print $NF}' | sed 's."..g'`
  if [ $? -ne 0 ]; then
    echo "WARNINGMSG: Fail to get iq log file. Remount will not work."
  fi
  echo "IQ_SYSTEM_LOG $IQ_SYSTEM_LOG" >> $IQFILES_DETAILS

  #find PITR loc
  set +x
  IQ_PITR_LOC___ACT=`su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"select db_property('IQPointInTimeRecoveryLogFilename')\" " | awk 'NR==3{print $1}'`
  retval=$?
  set -x
  IQ_PITR_LOC___ACT=`dirname $IQ_PITR_LOC___ACT`
  echo "IQ_PITR_LOC___ACT $IQ_PITR_LOC___ACT" >> $IQFILES_DETAILS
fi
#-----------------------------------------------------------------------------------------------------


#Stop database
set +x
echo "Stoping the database $TGT_DB_NAME ..."
su -m $OSUSER -c "dbstop -c \"$CONN_STR\" "
retval=$?
set -x

if [ $retval -ne 0 ];then
  echo "ERRORMSG: Fail to shutdown the database $TGT_DB_NAME"
  exit 1
else
  echo "Successfully stoped database $TGT_DB_NAME"
fi
