#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x

set -x
OSUSER=$OSUSER
SYBIQ_HOME=$SYBIQ_HOME
version=$VERSION
SRC_DB_NAME=$SRC_DB_NAME
TGT_DB_NAME=$TGT_DB_NAME
DBCATALOGFILE=$DBCATALOGFILE
ACT_JOBNAME=$JOBID
TARGET_MNT=$TARGET_MNT
utilitydbuser=$SRC_DB_DBA_USER
set +x
utilitydbpwd=$SRC_DB_DBA_PWD
set -x
FULL_VIR_BKP_LOC=$FULL_VIR_BKP_LOC
INCR_BKP_LOC=$INCR_BKP_LOC
TLS_ENABLED=$TLS_ENABLED
DB_ENG=$DB_ENG
PORTNO=$PORTNO
OVERWRITE_DATABASE=$OVERWRITE_DATABASE

#Prepare act-touch file to identify premount failure
ACT_PREMOUNT_FAILURE_FILE=/act/touch/premount_test_failure_$ACT_JOBNAME

#------------------------------------------------------------
# function: deleteDB()
#------------------------------------------------------------
deleteDB() {
  local dbName=$1
  echo "INFOMSG: Deleting database $dbName"
  iqDB_PID=`ps -ef | grep "/iqsrv" | grep -i "/$dbName.db" | grep -vw grep | awk '{print $2}'`
  kill -9 $iqDB_PID
  return 0
}
#------------------------------------------------------------

#------------------------------------------------------------
# function: deleteDBifStale()
#------------------------------------------------------------
deleteDBifStale() {
  local dbName=$1

  local DbFile=`ps -e -o cmd | grep -i "/$dbName.db" | grep -vw grep | awk '{print $2}'`
  if [ ! -z "$DbFile" ] && [ -e "$DbFile" ]; then
    echo "ERRORMSG: Database $dbName exists and not stale."
    echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
    exit 1
  fi

  deleteDB $dbName
  return 0
}
#------------------------------------------------------------

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

OSGROUP=`id -ngr $OSUSER`

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

#Check db running or not
isDB_running $TGT_DB_NAME
if [ $? -eq 0 ]; then
  DB_EXISTS=YES
else
  DB_EXISTS=NO
fi

#checking utility server status
util_status=`ps -ef | grep iqsrv | grep -w utility_db | wc -l`
if [ "$util_status" -eq 0 ] ; then
     echo "ERRORMSG : utility server is not running ! please check logs"
     exit 1
fi

#Prepare connection string
set +x
prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "utility_db"
set -x

#Check utility server credientials
set +x
su -m $OSUSER  -c "dbisql -nogui -onerror exit -c \"$CONN_STR\" $HOST_PORT_OPT \"exit;\" "
retval=$?
set -x

if [ $retval -ne 0 ];then
  echo "ERRORMSG: Failed to connect utility database ! please check logs for more Details"
  exit 1
fi

if [ "$ACT_JOBTYPE" = "mount" ]; then

  #Check db exist 
  if [ "$DB_EXISTS" = "YES" ]; then
    case $OVERWRITE_DATABASE in
      yes)
        deleteDB $TGT_DB_NAME
        ;;
      stale)
        deleteDBifStale $TGT_DB_NAME
        ;;
      *)
        echo "ERRORMSG: Specified target db alread exists ($TGT_DB_NAME)"
        echo "INFOMSG: No unmount required." > $ACT_PREMOUNT_FAILURE_FILE
        exit 1
        ;;
    esac

  fi

fi
exit 0
