#!/bin/sh

set -x
OSUSER=$OSUSER
SRC_DB_NAME=$SRC_DB_NAME
TGT_DB_NAME=$TGT_DB_NAME
CATALOG_DB_FILE=$CATALOG_DB_FILE
ACT_JOBNAME=$JOBID
TARGET_MNT=$TARGET_MNT
utilitydbuser=$SRC_DB_DBA_USER
set +x
utilitydbpwd=$SRC_DB_DBA_PWD
set -x
SYBIQ_HOME=$SYBIQ_HOME

#----------------------------------------------------------------------
# function: remountSameMnt
#----------------------------------------------------------------------
remountSameMnt() {

  #Prepare Restore commands
  echo "restore database '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db' from '$FULL_VIR_BKP_LOC' " > $RESTORE_FULL_SQL
  echo "OVERWRITE EXISTING clear log;" >> $RESTORE_FULL_SQL

  #Prepare connection string
  set +x
  prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "$utility_db"
  set -x

  chown -R $OSUSER:$OSGROUP $RESTORE_FULL_SQL

  #Restore database
  set +x
  output=`su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_FULL_SQL "`
  retval=$?
  set -x

  if [ "$retval" -ne 0 ] ;then
    echo "ERRORMSG: Failed to restore database $TGT_DB_NAME"
    return 1
  fi

  # Start database
  su $OSUSER -c "start_iq $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db"
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: fail to start the database $TGT_DB_NAME"
    return 1
  fi

  #Prepare connection string
  set +x
  echo "Preparing connection string..."
  prepareConnString "$utilitydbuser" "$utilitydbpwd" "$TGT_DB_NAME" "$TGT_DB_NAME"
  set -x

  # Enable PITR logs
  set +x
  echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'"
  su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'\" "
  retval=$?
  set -x

  if [ $retval -ne 0 ]; then
    echo "ERRORMSG: Fail to start PITR logging for database $TGT_DB_NAME"
    return 1
  fi

  return 0
}
#----------------------------------------------------------------------

#Get envdetails
OSGROUP=`id -gn $OSUSER`
IQSRV_CMD=`ps -A -o command | grep /iqsrv | awk '{print $1}' | sort|uniq |grep -v grep`
if [ -z "$SYBIQ_HOME" ]; then
  SYBIQ_HOME=`dirname $IQSRV_CMD`
  SYBIQ_HOME=$SYBIQ_HOME/../..
fi
version=`echo $IQSRV_CMD| rev| cut -d/ -f3| rev`

REMOUNT_CONFIG_FILE=/act/tmpdata/$ACT_JOBNAME/REMOUNT_CONFIG
if [ ! -e "$REMOUNT_CONFIG_FILE" ]; then
  echo "ERRORMSG: REMOUNT_CONFIG file is missing"
  exit 1
fi
OLD_DBNAME=`grep -w REMOUNT_DB_LIST $REMOUNT_CONFIG_FILE | cut -d: -f2`
OLD_MNTPNT=`grep -w OLD_MOUNT $REMOUNT_CONFIG_FILE | cut -d= -f2`
DATA_LOC=`dirname $CATALOG_DB_FILE`
FULL_VIR_BKP_LOC=$TARGET_MNT$DATA_LOC/FULL_VIR_DEC


########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

###CONFIG FILE LOCATION
SRC_DBDIR=`dirname $CATALOG_DB_FILE`
config_file="$TARGET_MNT"$SRC_DBDIR/ACTIFIO_config.txt

#####creating sql file for restore
prepareTmpLoc
chown -R $OSUSER:$OSGROUP $tmp_loc
chown -R $OSUSER:$OSGROUP $TARGET_MNT
RESTORE_FULL_SQL=$tmp_loc/restore_full.sql
#RESTORE_INC_SQL=$tmp_loc/restore_inc.sql

if [ "$OLD_MNTPNT" = "$TARGET_MNT" ]; then

  if [ "$OLD_DBNAME" = "$TGT_DB_NAME" ]; then
    echo "Re-Mount with Same mountpoint and Same dbname"
  else
    echo "Re-Mount with Same mountpoint and different dbname"
  fi

  remountSameMnt
  if [ $? -ne 0 ]; then
    echo "ERRORMSG: Fail to Re-Mount on same mount-point"
    exit 1
  else
    echo "INFOMSG: Remount success."
    exit 0
  fi

fi

#Create symbolic links if required
IQ_SYSTEM_LOG_file=`grep -w IQ_SYSTEM_LOG $config_file| awk '{print $NF}'`
IQ_SYSTEM_LOG_file_dir=`dirname $IQ_SYSTEM_LOG_file| sed 's."..g'`

#Check if it is exists or not
if [ -d "$IQ_SYSTEM_LOG_file_dir" ]; then
  chmod 777 $IQ_SYSTEM_LOG_file_dir
  if [ -e $IQ_SYSTEM_LOG_file ]; then
    #if already transaction log exists, fail the job
    echo "ERRORMSG: $IQ_SYSTEM_LOG_file exists. Cannot proceed to appaware mount as it will clear this file."
    exit 1
  else
    touch $IQ_SYSTEM_LOG_file
    chmod 644 $IQ_SYSTEM_LOG_file
  fi
else
  #Create softlink and required directories
  OLD_MNTPNT_dir=`dirname $OLD_MNTPNT`
  if [ ! -d "$OLD_MNTPNT_dir" ]; then
    mkdir -p $OLD_MNTPNT_dir
  fi
  ln -s $TARGET_MNT $OLD_MNTPNT
  DEL_SOFTLINK_AT_END=$OLD_MNTPNT
fi

#Check softlink properly created or not
if [ ! -e "$IQ_SYSTEM_LOG_file" ]; then
  echo "ERRORMSG: Fail to create softlink to IQ_SYSTEM_LOG. $IQ_SYSTEM_LOG_file not exists"
  rm -f $DEL_SOFTLINK_AT_END
  exit 1
fi

#prepare config file with latest mountpoints
OLD_MNTPNT_sed=`echo " $OLD_MNTPNT"|sed 's./.\\\/.g'`
TARGET_MNT_SED=`echo " $TARGET_MNT"|sed 's./.\\\/.g'`
sed -i "s/$OLD_MNTPNT_sed/$TARGET_MNT_SED/g" $config_file

#Remove old PITR logs
IQ_PITR_LOC___ACT_loc=`grep -w IQ_PITR_LOC___ACT $config_file | awk '{print $NF}'`
rm -f $IQ_PITR_LOC___ACT_loc/*.iqlog

##Full Decoupled Backup restore
echo "restore database '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db' from '$FULL_VIR_BKP_LOC' " > $RESTORE_FULL_SQL
while read line
do
  fLabel=`echo $line| awk '{print $1}'`
  fPath=`echo $line| awk '{print $2}'`
  if [ "$fLabel" = "IQ_SYSTEM_LOG" ] || [ "$fLabel" = "IQ_PITR_LOC___ACT" ]; then
    continue
  fi
  #Check and correct file obsolete path
  if [ "${fPath:0:1}" != "/" ]; then
    fPath=$SRC_DBDIR/$fPath
  fi

  if [ ! -e "$fPath" ]; then
    echo "ERRORMSG: One of the datafile missing ($fPath)"
    removeFile $IQ_SYSTEM_LOG_file
    exit 1
  fi
  echo "rename $fLabel TO '$fPath'" >> $RESTORE_FULL_SQL
done<"$config_file"
#echo "rename IQ_SYSTEM_TEMP TO '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.tmp'" >> $RESTORE_FULL_SQL
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_FULL_SQL
chown -R $OSUSER:$OSGROUP $RESTORE_FULL_SQL

#Prepare connection string
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "utility_db"
set -x

###MAIN 
set +x
echo "Executing restore full backup using file $FULL_VIR_BKP_LOC ..."
echo "Command:"
cat $RESTORE_FULL_SQL
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_FULL_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to clone database $TGT_DB_NAME"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#Copy back original transaction log if exists
#if [ -e "$IQ_SYSTEM_LOG_file.ACT.bkp" ]; then
#  cp -p $IQ_SYSTEM_LOG_file.ACT.bkp $IQ_SYSTEM_LOG_file
#  rm -f $IQ_SYSTEM_LOG_file.ACT.bkp
#fi

#Rename transaction log for new database
su -m $OSUSER -c "dblog -t $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.log $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db"
if [ $? -ne 0 ];then
  echo "ERRORMSG: Fail to rename IQ_SYSTEM_LOG path"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#######starting IQ server
su -m $OSUSER -c "start_iq $TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME.db"
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to start the database"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#Prepare connection string
set +x
echo "Preparing connection string..."
prepareConnString "$utilitydbuser" "$utilitydbpwd" "$TGT_DB_NAME" "$TGT_DB_NAME"
set -x

#Trunoff PITR to target
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'OFF'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to turnoff PITR to target"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#Rename PITR logs location
set +x
echo "Running SQL: ALTER DBSPACE IQ_SYSTEM_LOG RENAME '$TARGET_MNT$SRC_DBDIR'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"ALTER DBSPACE IQ_SYSTEM_LOG RENAME '$TARGET_MNT$SRC_DBDIR/$TGT_DB_NAME';\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to rename IQ_SYSTEM_LOG location"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#Restart PITR
set +x
echo "Running SQL: SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'"
su $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"SET OPTION PUBLIC.IQ_POINT_IN_TIME_RECOVERY_LOGGING = 'ON'\" "
retval=$?
set -x

if [ $retval -ne 0 ]; then
  echo "ERRORMSG: Fail to on PITR to new database"
  removeFile $IQ_SYSTEM_LOG_file
  exit 1
fi

#Prepare config file for remount
mkdir -p /act/touch/$ACT_JOBNAME
chown -R $OSUSER:$OSGROUP /act/touch/$ACT_JOBNAME
remount_config

## Remove the created softlink
if [ -e $DEL_SOFTLINK_AT_END ]; then
  rm -f $DEL_SOFTLINK_AT_END
  if [ $? -ne 0 ]; then
    echo "WARNING: Fail to remove softlink $DEL_SOFTLINK_AT_END"
  fi
fi
removeFile $IQ_SYSTEM_LOG_file

# Remove if stail directories present
while [ -d "$OLD_MNTPNT_dir" ]; do
  rmdir $OLD_MNTPNT_dir
  if [ $? -ne 0 ]; then
    break
  fi
  OLD_MNTPNT_dir=`dirname $OLD_MNTPNT_dir`
done

if [ ! -z $IQ_SYSTEM_LOG_file_dir ] && [ -d "$IQ_SYSTEM_LOG_file_dir" ]; then
  chmod 755 $IQ_SYSTEM_LOG_file_dir
fi

exit 0
