#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x
OSUSER=$OSUSER
DB_NAME=$ACT_NAME
DBA_USER=$SRC_DB_DBA_USER
set +x
DBA_PWD=$SRC_DB_DBA_PWD
set -x
TARGET_MNT=$LOGBKPLOC
SYBIQ_HOME=$SYBIQ_HOME
version=$VERSION
DB_ENG=$DB_ENG
PORTNO=$PORTNO

BEGINPIT=$BEGINPIT     # Previous DB beginpit

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

#Get vals from SYBASEIQ.xml for now
if [ -z "$DB_ENG" ]; then
  DB_ENG=`grep "name=\"$DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"DB_ENG=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi
if [ -z "$PORTNO" ]; then
  PORTNO=`grep "name=\"$DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"PORTNO=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi

isVarNull OSUSER $OSUSER
isVarNull SYBIQ_HOME $SYBIQ_HOME
isVarNull version $version
isVarNull SRC_DB_DBA_USER $SRC_DB_DBA_USER
set +x
isVarNull SRC_DB_DBA_PWD $SRC_DB_DBA_PWD
set -x
isVarNull DB_NAME $DB_NAME
isVarNull DB_ENG $DB_ENG
isVarNull PORTNO $PORTNO

#OSGROUP
OSGROUP=`id -ngr $OSUSER`

#SET  env
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

if [ ! -d /act/touch ];then
  mkdir -p /act/touch
fi


#Deleting tmp files
function Deltmpfiles {
  rm -f /act/touch/$DB_NAME.pitrstatus.txt
  rm -f /act/touch/$DB_NAME.pitrlogfile.txt
  rm -f /act/touch/backup.sql
}

#check DB status
DB_STATUS=`ps -ef | grep iqsrv | grep -w $DB_NAME | wc -l`
if [ "$DB_STATUS" -eq 0 ];then
  echo "ERRORMSG: Database $DB_NAME is not running"
  exit 1
fi
     
#PITR logging status
touch /act/touch/$DB_NAME.pitrstatus.txt
touch /act/touch/$DB_NAME.pitrlogfile.txt
chown -R $OSUSER:$OSGROUP /act/touch

#DB_ENG=`dblocate -d | grep -vw utility_db | grep -vw iqdemo |  awk 'NR > 3 {print}' | sed '$d' | grep -w $DB_NAME | xargs|awk '{print $1}'`
set +x
prepareConnString "$SRC_DB_DBA_USER" "$SRC_DB_DBA_PWD" "$DB_NAME" "$DB_ENG" "$PORTNO"
set -x

su -m $OSUSER -c  "dbisql -nogui -onerror exit $HOST_PORT_OPT \"select db_property('IQPointInTimeRecoveryLogStatus');OUTPUT TO \"/act/touch/$DB_NAME.pitrstatus.txt\" format ascii;exit;\""
su -m $OSUSER -c  "dbisql -nogui -onerror exit $HOST_PORT_OPT \"select db_property('IQPointInTimeRecoveryLogFilename');OUTPUT TO \"/act/touch/$DB_NAME.pitrlogfile.txt\" format ascii;exit;\""
su -m $OSUSER -c  "dbisql -nogui -onerror exit $HOST_PORT_OPT 'select * from sa_db_info ();OUTPUT TO '/act/touch/$dbname.txt' format ascii;exit;'"

PITR_STATUS=`cat /act/touch/$DB_NAME.pitrstatus.txt | grep -w On |wc -l`
PITR_LOGFILE_TMP=`cat /act/touch/$DB_NAME.pitrlogfile.txt | sed "s/'//g"`
PITR_LOC=`dirname $PITR_LOGFILE_TMP`
log=`cat /act/touch/$dbname.txt | awk -F"," '{print $6}' | sed  "s/'//g"`
if [ $PITR_STATUS -ne 1 ]; then
  echo "ERRORMSG:  Point In Time Recovery is not enabled for Database $DB_NAME ! Please check logs for more details"
  exit 1
else
  echo "continue"
  echo "$PITR_LOGFILE_TMP"
fi

#creating log backup directories on $TARGET_MNT
if [ ! -d $TARGET_MNT/$DB_NAME/PITR ]; then
  mkdir -p $TARGET_MNT/$DB_NAME/PITR
fi

TARGET_MNT_TMP=$TARGET_MNT/$DB_NAME
DT=`date +%Y%m%d_%H%M%S`

#Change ownership of new loaction
chown -R $OSUSER:$OSGROUP $TARGET_MNT

#LOG BACKUP
#Recovery PITR logs backup
switchLog $TARGET_MNT/$DB_NAME/PITR
if [ $? -ne 0 ] ; then
  echo "ERRORMSG: Failed to take PITR  logs backup for Database $DBNAME check logs for more details !"
  exit 1
fi

#find files newer thatn BEGINPIT to copy
logFilesToCopy=`find $PITR_LOC/*iqlog -newermt "$BEGINPIT" | xargs`
for logFile in $logFilesToCopy; do
  srcLogFileName=`basename $logFile`
  trgLogFile=$TARGET_MNT/$DB_NAME/PITR/$srcLogFileName

  #Skip if file already copied
  if [ -e "$trgLogFile" ]; then
    srcSize=`ls -l $logFile | awk '{print $5}'`
    trgSize=`ls -l $trgLogFile | awk '{print $5}'`
    if [ $srcSize -eq $trgSize ]; then
        continue
    fi
  fi

  cp $logFile $trgLogFile
done

  cp $log $TARGET_MNT/$DB_NAME/PITR/"$DB_NAME"_savelog
  echo " Database $DB_NAME PITR logs backup is success"
 
#Calling Deletetmpfiles function
Deltmpfiles


exit 0
