#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
set -x
OSUSER=$OSUSER
SRC_DB_DBA_USER=$SRC_DB_DBA_USER
set +x
SRC_DB_DBA_PWD=$SRC_DB_DBA_PWD
set -x
SRC_DB_ENG=$SRC_DB_NAME
DUMPSCHEDULE=$DUMPSCHEDULE
DB_NAME=$ACT_NAME
DB_ENG=$DB_ENG
PORTNO=$PORTNO
SYBIQ_HOME=$SYBIQ_HOME
version=$VERSION

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

#Get vals from SYBASEIQ.xml for now
if [ -z "$DB_ENG" ]; then
  DB_ENG=`grep "name=\"$DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"DB_ENG=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi
if [ -z "$PORTNO" ]; then
  PORTNO=`grep "name=\"$DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"PORTNO=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

isVarNull OSUSER $OSUSER
isVarNull SRC_DB_DBA_USER $SRC_DB_DBA_USER
set +x
isVarNull SRC_DB_DBA_PWD $SRC_DB_DBA_PWD
set -x
isVarNull DUMPSCHEDULE $DUMPSCHEDULE
isVarNull DB_NAME $ACT_NAME
isVarNull SYBIQ_HOME $SYBIQ_HOME
isVarNull version $version

#check DB status
DB_STATUS=`ps -ef | grep iqsrv | grep -w $DB_NAME | wc -l`
if [ "$DB_STATUS" -eq  0 ];then
  echo "ERRORMSG: Database $DB_NAME is not running"
  exit 1
else
  echo "Database $DB_NAME is running"
fi

# Prepare DB connection
set +x
prepareConnString "$SRC_DB_DBA_USER" "$SRC_DB_DBA_PWD" "$DB_NAME" "$DB_ENG" "$PORTNO"
set -x

# checking DB connection
tmpSqlFile=/act/touch/${JOBID}_checkConn.sql
echo " exit;" > $tmpSqlFile
chown -R $OSUSER:$OSGROUP $tmpSqlFile
set +x
su -m $OSUSER  -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"$tmpSqlFile\" "
retval=$?
set -x
if [ $retval -eq 0 ];then
  echo " Database $DB_NAME connection is success"
else
  echo "ERRORMSG: Failed to connect  database  $DB_NAME ! please check logs"
  exit 1
fi 
removeFile $tmpSqlFile


#Get backup type
if [ "$DUMPSCHEDULE" = "F" ]; then
  DUMPSCHEDULE="F"
elif [ "$PREVDBFULLDUMPSAMEDAY" = "TRUE" ]; then
  DUMPSCHEDULE="I"
else
  weekday=`date +%w`
  DUMPSCHEDULE=`expr substr  "$DUMPSCHEDULE" $weekday 1`
fi

#Get DB size
set +x
blockSize=`su -m $OSUSER  -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT \"select block_size from SYSIQINFO\" " |awk 'NR==3{print $1}'`
dbTotalBlocks=`su -m $OSUSER  -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT sp_iqdbsize" | awk 'NR==3{print $2}'`
set -x
echo "blockSize: $blockSize, dbTotalBlocks: $dbTotalBlocks"
dbSize=`expr $blockSize "*" $dbTotalBlocks`

echo "DBDUMPSIZE=$dbSize"
echo "DBDUMPTYPE=$DUMPSCHEDULE"

exit 0
