#!/bin/sh

set -x
OSUSER=$OSUSER
SYBIQ_HOME=$SYBIQ_HOME
version=$VERSION
SRC_DB_NAME=$SRC_DB_NAME
CATALOG_DB_FILE=$CATALOG_DB_FILE
ACT_JOBNAME=$JOBID
utilitydbuser=$SRC_DB_DBA_USER
set +x
utilitydbpwd=$SRC_DB_DBA_PWD
set -x
INCR_BKP_LOC=$INCR_BKP_LOC

RECOVERY_TIME=$RECOVERY_TIME
LOG_BKP_MNTPT=$LOG_BKP_MNTPT
PITR_LOG=$PITR_LOG

#Get envdetails
OSGROUP=`id -gn $OSUSER`
IQSRV_CMD=`ps -A -o command | grep iqsrv | awk '{print $1}' | sort|uniq |grep -v grep`
SYBIQ_HOME=`dirname $IQSRV_CMD`
SYBIQ_HOME=$SYBIQ_HOME/../..
version=`echo $IQSRV_CMD| rev| cut -d/ -f3| rev`
DATA_LOC=`dirname $CATALOG_DB_FILE`
FULL_VIR_BKP_LOC=$DATA_LOC/FULL_VIR_DEC

########SET  env########
set +x
. $SYBIQ_HOME/$version/$version.sh
set -x

PWDS=/act/custom_apps/sybaseiq

. /act/custom_apps/sybaseiq/act_sybaseiq_common_functions.sh

#Get vals from SYBASEIQ.xml for now
if [ -z "$DB_ENG" ]; then
  DB_ENG=`grep "name=\"$SRC_DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"DB_ENG=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi
if [ -z "$PORTNO" ]; then
  PORTNO=`grep "name=\"$SRC_DB_NAME\" " /act/custom_apps/SYBASEIQ.xml | awk -F"PORTNO=" '{print $2}' | awk '{print $1}' |sed 's."..g'`
fi

###CONFIG FILE LOCATION
config_file_tmp=`dirname $CATALOG_DB_FILE`
config_file=$config_file_tmp/config.txt
cp $config_file "$config_file"_orig

#####creating sql file for restore

prepareTmpLoc
chown -R $OSUSER:$OSGROUP $tmp_loc
RESTORE_FULL_SQL=$tmp_loc/restore_full.sql
RESTORE_INC_SQL=$tmp_loc/restore_inc.sql


##Full Decoupled Backup restore

echo "restore database '$config_file_tmp/$SRC_DB_NAME.db' from '$FULL_VIR_BKP_LOC' " > $RESTORE_FULL_SQL
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_FULL_SQL

#Prepare connection string
set +x
prepareConnString "$utilitydbuser" "$utilitydbpwd" "utility_db" "utility_db"
set -x

chown -R $OSUSER:$OSGROUP $RESTORE_FULL_SQL

###MAIN 
set +x
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_FULL_SQL "`
retval=$?
set -x

if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to restore database $SRC_DB_NAME"
  exit 1
fi


######Incremental Backup Restore
INCR_BKP_LOC=`dirname $INCR_BKP_LOC`
chown -R $OSUSER:$OSGROUP $INCR_BKP_LOC/INC_BKP*
INCR_BKP_LOC=$INCR_BKP_LOC/INC_BKP

echo "restore database '$config_file_tmp/$SRC_DB_NAME.db' from '$INCR_BKP_LOC' " > $RESTORE_INC_SQL
if [ ! -z "$RECOVERY_TIME" ]; then
  chown -R $OSUSER:$OSGROUP $LOG_BKP_MNTPT
  LOG_BKP_MNTPT_base=`echo $LOG_BKP_MNTPT| cut -d/ -f2`
  if [ ! -z "$LOG_BKP_MNTPT_base" ]; then
    find /$LOG_BKP_MNTPT_base -type d -exec chmod 755 {} +
  fi
  echo "RECOVER UNTIL TIMESTAMP '$RECOVERY_TIME' USING LOG PATH '$LOG_BKP_MNTPT/$SRC_DB_NAME/PITR','$PITR_LOG' " >> $RESTORE_INC_SQL
fi
echo "OVERWRITE EXISTING clear log;" >> $RESTORE_INC_SQL

chown -R $OSUSER:$OSGROUP $RESTORE_INC_SQL

set +x
output=`su -m $OSUSER -c "dbisql -nogui -onerror exit -c '$CONN_STR' $HOST_PORT_OPT $RESTORE_INC_SQL "`
retval=$?
set -x
if [ "$retval" -ne 0 ] ;then
  echo "ERRORMSG: Failed to Recover database $SRC_DB_NAME"
  exit 1
fi


#######starting IQ server
su -m $OSUSER -c "start_iq -n $DB_ENG $config_file_tmp/$SRC_DB_NAME.db -x tcpip{port=$PORTNO} "
if [ $? -ne 0 ]; then
  echo "ERRORMSG: fail to start the database"
  exit 1
fi

#Check server started or not
isDB_running $SRC_DB_NAME
if [ $? -ne 0 ]; then
  echo "ERRORMSG: Fail to start the database"
  exit 1
fi

#Prepare connection string
set +x
prepareConnString "$utilitydbuser" "$utilitydbpwd" "$SRC_DB_NAME" "$DB_ENG" "$PORTNO"
set -x

#Cleanup temp files
removeFile $RESTORE_FULL_SQL
removeFile $RESTORE_INC_SQL
rmdir $tmp_loc
exit 0
