#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
#                  !!!Actifio Private Script. DO NOT MODIFY!!!
#
# ==============================================================================#

export COLUMNS=4000
unset LANG
unset LC_ALL

usage() {
  echo "Usage: "
  echo "/act/scripts/CALL_ACT_SYBASE_mountrecover.sh \"\" <TARGET_SERVER_NAME> <TARGET_DBNAME_LIST> <TARGET_DB_USER> <TARGET_DBUSER_PASSWD/null> <TARGET_DBUSER_OTHERCRED/null> [ <UNTIL_TIME> ]"
}

isProvided() {
  local varName=$1
  local varVal=$2
  if [ -z "$varVal" ]; then
    echo "ERRORMSG: $varName is empty."
    usage
    exit 1
  fi
}

set -x

#Read the arguments
TARGET_SERVER_NAME=$2		# (User need to give)
TARGET_DBNAME_LIST=$3		# Comma(,) or double colon (::) seperated list of <SRC_DBNAME>:<new_dbname> (User need to give)
TARGET_DB_USER=$4			# (User need to give)
set +x
TARGET_DBUSER_PASSWD="$5"		# (User need to give)
set -x
TARGET_DBUSER_OTHERCRED=$6	# (User need to give)
UNTIL_TIME="$7"			# Recover time (User need to give)

if [ ! -z "$ACT_JOBNAME" ] && [ -z "$JOBID" ]; then
  JOBID=$ACT_JOBNAME
fi

if [ -z $JOBID ]; then
  echo "ERRORMSG: JOBID not provided"
  exit 1
fi

#Read the parameters passed by connector
PARAMETER_FILE=`ls /act/touch/*_"$JOBID"_params | head -1`
if [ ! -f "$PARAMETER_FILE" ]; then
  echo "ERRORMSG: $PARAMETER_FILE not exists."
  exit 1
fi

TARGET_MNT_PNT=`grep MOUNTPOINTROOT $PARAMETER_FILE | cut -d= -f2`
LOG_BKP_MNTPT=`grep ARCHIVELOGMOUNTPATH $PARAMETER_FILE | cut -d= -f2`
BEGIN_TIME=`grep BEGIN_TIME $PARAMETER_FILE | cut -d= -f2`
#SRC_DBNAME=`grep SRC_DBNAME $PARAMETER_FILE | cut -d= -f2`
MANIFEST_FILE_LOC=`grep MANIFEST_FILE_LOC $PARAMETER_FILE | cut -d= -f2`
SRC_PAGE_SIZE=`grep SRC_PAGE_SIZE $PARAMETER_FILE | cut -d= -f2`

isProvided MANIFEST_FILE_LOC $MANIFEST_FILE_LOC
isProvided TARGET_MNT_PNT $TARGET_MNT_PNT
isProvided TARGET_SERVER_NAME $TARGET_SERVER_NAME
isProvided TARGET_DB_USER $TARGET_DB_USER
#isProvided SRC_DBNAME $SRC_DBNAME
isProvided SRC_PAGE_SIZE $SRC_PAGE_SIZE
isProvided TARGET_DBNAME_LIST $TARGET_DBNAME_LIST
if [ ! -z "$UNTIL_TIME" ]; then
  isProvided LOG_BKP_MNTPT $LOG_BKP_MNTPT
  isProvided BEGIN_TIME "$BEGIN_TIME"
fi

if [ -z "$TARGET_DBUSER_PASSWD" ] && [ -z "$TARGET_DBUSER_OTHERCRED" ]; then
  echo "ERRORMSG: Both TARGET_DBUSER_PASSWD and TARGET_DBUSER_OTHERCRED are not provided. Need at least one of these two."
  exit 1
fi

#Update parameters for ACT_SYBASE_mountrecover.sh
TARGET_DBNAME_LIST=`echo $TARGET_DBNAME_LIST |sed 's/::/,/g'`
SRC_DBNAME=
MANIFEST_FILE_LOC_update=
for dbNameSet in $(echo $TARGET_DBNAME_LIST |sed 's/,/ /g')
do
  dbSrcName=`echo $dbNameSet | cut -d: -f1`
  dbManifestFile=`echo $MANIFEST_FILE_LOC | tr "," "\n" | grep "_$dbSrcName$"`

  if [ -z "$SRC_DBNAME" ]; then
    SRC_DBNAME=$dbSrcName
  else
    SRC_DBNAME="$SRC_DBNAME,$dbSrcName"
  fi

  if [ -z "$MANIFEST_FILE_LOC_update" ]; then
    MANIFEST_FILE_LOC_update=$dbManifestFile
  else
    MANIFEST_FILE_LOC_update="$MANIFEST_FILE_LOC_update,$dbManifestFile"
  fi

done
MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC_update

echo "Updated MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC"
echo "Updated SRC_DBNAME=$SRC_DBNAME"


echo "***** Calling /act/custom_apps/sybase/clone/ACT_SYBASE_mountrecover.sh for MountRecovery *****"
/act/custom_apps/sybase/clone/ACT_SYBASE_mountrecover.sh MANIFEST_FILE_LOC=$MANIFEST_FILE_LOC TARGET_MNT_PNT=$TARGET_MNT_PNT TARGET_SERVER_NAME=$TARGET_SERVER_NAME SRC_DBNAME=$SRC_DBNAME TARGET_DBNAME_LIST=$TARGET_DBNAME_LIST TARGET_DB_USER=$TARGET_DB_USER TARGET_DBUSER_PASSWD="$TARGET_DBUSER_PASSWD" TARGET_DBUSER_OTHERCRED=$TARGET_DBUSER_OTHERCRED LOG_BKP_MNTPT=$LOG_BKP_MNTPT UNTIL_TIME="$UNTIL_TIME" BEGIN_TIME="$BEGIN_TIME" JOBID=$JOBID SRC_PAGE_SIZE=$SRC_PAGE_SIZE
if [ $? -ne 0 ]; then
  echo "ERRORMSG: MountRecover operation failed"
  exit 1
fi
