#!/bin/sh

SkyIP=$1
if [ -z $SkyIP ]; then
  echo "Error: SKYIP is mandatory"
  exit 1
fi

sessionExists=$(iscsiadm -m session | grep -w $SkyIP:3260)
if [ -z "$sessionExists" ]; then
  iscsiadm -m iface -I actifiosap
  retval=$?
  if [ $retval -ne 0 ]; then
    targetIQN=$(iscsiadm -m discovery -t st -p $SkyIP:3260 2>/dev/null | awk '{print $NF}')
    iscsiadm -m node -T $targetIQN -p $SkyIP:3260 -l
    retval=$?
  else
    targetIQN=$(iscsiadm --mode discoverydb --type sendtargets -I actifiosap --portal $SkyIP:3260 --discover 2>/dev/null | tail -n 1 | awk '{print $NF}')
    if [ -z "$targetIQN" ]; then
      echo "********** Error discovering Sky Portal $SkyIP: $targetIQN *************"
      exit 1
    fi
    iscsiadm -m node -T $targetIQN --login --portal=$SkyIP:3260 -I actifiosap
    retval=$?
  fi

  if [ $retval -ne 0 ]; then
    if [ $retval -eq 15 ]; then
      echo "*** Session Already Exists****"
      exit
    fi
    echo "*** ERROR: Iscsi Login to $SkyIP has Failed************"
    exit $retval
  fi
fi
iscsiadm -m session --rescan
dmsetup remove_all
vgscan --mknodes
sleep 30
vgimport -a

vgnames=$(vgs --noheadings -o vg_name 2>/dev/null | grep "dumpvg")
vgnames=$(echo $vgnames | tr " " "\n")
if [ ! -z "$vgnames" ]; then
  for vgname in $vgnames; do
    vgname=$(echo $vgname | grep "dumpvg$")
    if [ ! -z "$vgname" ]; then
      lvname=$(echo ${vgname%dumpvg})'dumplv'
      umount /dev/$vgname/$lvname
      mount | grep -w /dev/$vgname/$lvname
      if [ "$?" -eq 0 ]; then
        echo "ERROR:Exiting /dev/$vgname/$lvname is in use. Probably DB Dump Backup Job is using this mountpoint!!"
        echo "Cancel the DB Job using this mountpoint and after completion try cleanup again!!!"
        exit 1
      fi
      vgchange -a y $vgname
      appname=$(echo ${vgname%dumpvg})
      mkdir -p /$appname
      mount /dev/$vgname/$lvname /$appname
    fi
  done
fi

exit 0
